/*
	BtCaptureStream.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Log:
	- 10/21/2001	ejtagle		Reduced number of videoformats to 5 per channel due to limits of WDM wrapper

*/

#include "Debug.hpp"
#include "BtCaptureStream.hpp"
#include "BtDigitizer.hpp"
#include "VideoFormats.hpp"

/* 
 Notes: 

	1] Some programs get the first stream format available for a given colorformat
	   and try to open the stream in that format (the default resolution). VideoForWindows wrappers
	   do this.
	2] The WDM wrapper (kswdmcap.ax) does not offer the maximum resolution of the stream
	   as one of the resolutions available, unless one of those resolutions matches the 
	   one of the internal listing. This means that the listing of available resolutions
	   listed by kswdmcap.ax (default resolution&colorspace picker under DirectShow) simply
	   are internally fixed, and they are shown if the stream allows it. At least the default
	   resolution is taken from the stream.
	3] NetMeeting is looking for a digital camera (KS_AnalogVideoNone), so we must provide 
	   and entry for it if we want to use it.
	4] Overlay Mixer normally gives negative resolutions!!!! - Be careful

  So , to cope with this we will have to:
	-We must provide as the first on the capture stream an entry with KS_AnalogVideoNone
	 , colorspace YUY2 and default resolution of 320x240 for netmeeting and VFW2WDM wrappers
	-The first entry on the preview stream has to have 320x240 as the default resolution.

  */


// RGB 32
KS_DATARANGE_VIDEO BtVCaptureStream::FormatRGB32[] = {
	DECL_CAPTURESTRFMT( 32, 32, 
		0xe436eb7e, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 , 
		1, 1, KS_BI_RGB)
};

// RGB24
KS_DATARANGE_VIDEO BtVCaptureStream::FormatRGB24[] = { 
	DECL_CAPTURESTRFMT( 24, 24, 
		0xe436eb7d, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 ,
		1, 1, KS_BI_RGB)
};

// RGB555
KS_DATARANGE_VIDEO BtVCaptureStream::FormatRGB555[] = {
	DECL_CAPTURESTRFMT( 16, 16, 
		0xe436eb7c, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70,
		1, 1, KS_BI_RGB) /* Per ActiveMovie Spec, default to RGB555 if RGB16 detected */
};

// RGB565
KS_DATARANGE_VIDEO_WITHMASK BtVCaptureStream::FormatRGB565[] = {
	DECL_CAPTURESTRFMT_WITHMASK( 16, 16, 
		0xe436eb7b, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70, 
		1, 1, KS_BI_BITFIELDS, 0xF800 , 0x07E0 , 0x001F)
};

// YVU9 = YUV9 = Planar : Cada 16Y, hay 1 U, 1 V 
KS_DATARANGE_VIDEO BtVCaptureStream::FormatYVU9[] = {
	DECL_CAPTURESTRFMT( 9, 9,/*16, */
		KS_BI_YVU9, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		16, 4, KS_BI_YVU9)
};

// YUY2 = YUY2 Mixed Y Cb Y Cr 
KS_DATARANGE_VIDEO BtVCaptureStream::FormatYUY2[] = {
	DECL_CAPTURESTRFMT( 16, 16, 
		KS_BI_YUY2, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		2, 1, KS_BI_YUY2)
};

// UYVY = YUY2 Mixed Y Cb Y Cr 
KS_DATARANGE_VIDEO BtVCaptureStream::FormatUYVY[] = {
	DECL_CAPTURESTRFMT( 16, 16, 
		KS_BI_UYVY, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		2, 1, KS_BI_UYVY)
};

// Y41P = BTYUV = 8Y ,2U, 2V (12bpp)
KS_DATARANGE_VIDEO BtVCaptureStream::FormatY41P[] = {
	DECL_CAPTURESTRFMT( 12, 12, 
		KS_BI_Y41P, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 1, KS_BI_Y41P)
};
		
// YV12 = Y422 Planar 
KS_DATARANGE_VIDEO BtVCaptureStream::FormatYV12[] = {
	DECL_CAPTURESTRFMT( 12, 12, 
		KS_BI_YV12, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 2, KS_BI_YV12)
};

// I420 = Y422 Planar 
KS_DATARANGE_VIDEO BtVCaptureStream::FormatI420[] = {
	DECL_CAPTURESTRFMT( 12, 12, 
		KS_BI_I420, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 2, KS_BI_I420)
};

// For the advanced users, the Videoinfo2 struct!

// RGB 32
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatRGB32_2[] = {
	DECL_CAPTURESTRFMT2( 32, 32, 
		0xe436eb7e, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 , 
		1, 1, KS_BI_RGB)
};

// RGB24
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatRGB24_2[] = { 
	DECL_CAPTURESTRFMT2( 24, 24, 
		0xe436eb7d, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 ,
		1, 1, KS_BI_RGB)
};

// RGB555
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatRGB555_2[] = {
	DECL_CAPTURESTRFMT2( 16, 16, 
		0xe436eb7c, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70,
		1, 1, KS_BI_RGB)
};

// RGB565
KS_DATARANGE_VIDEO2_WITHMASK BtVCaptureStream::FormatRGB565_2[] = {
	DECL_CAPTURESTRFMT2_WITHMASK( 16, 16, 
		0xe436eb7b, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70, 
		1, 1, KS_BI_BITFIELDS, 0xF800 , 0x07E0 , 0x001F)
};

// YVU9 = YUV9 = Planar : Cada 16Y, hay 1 U, 1 V 
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatYVU9_2[] = {
	DECL_CAPTURESTRFMT2( 9, 9,/*16, */
		KS_BI_YVU9, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		16, 4, KS_BI_YVU9)
};

// YUY2 = YUY2 Mixed Y Cb Y Cr 
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatYUY2_2[] = {
	DECL_CAPTURESTRFMT2( 16, 16, 
		KS_BI_YUY2, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		2, 1, KS_BI_YUY2)
};

// UYVY = YUY2 Mixed Y Cb Y Cr 
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatUYVY_2[] = {
	DECL_CAPTURESTRFMT2( 16, 16, 
		KS_BI_UYVY, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		2, 1, KS_BI_UYVY)
};

// Y41P = BTYUV = 8Y ,2U, 2V (12bpp)
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatY41P_2[] = {
	DECL_CAPTURESTRFMT2( 12, 12, 
		KS_BI_Y41P, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 1, KS_BI_Y41P)
};
		
// YV12 = Y422 Planar 
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatYV12_2[] = {
	DECL_CAPTURESTRFMT2( 12, 12, 
		KS_BI_YV12, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 2, KS_BI_YV12)
};

// I420 = Y422 Planar 
KS_DATARANGE_VIDEO2 BtVCaptureStream::FormatI420_2[] = {
	DECL_CAPTURESTRFMT2( 12, 12, 
		KS_BI_I420, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 2, KS_BI_I420)
};

/* --------------------------------------------- */

//---------------------------------------------------------------------------
//  Stream 0 (Capture) Formats
//---------------------------------------------------------------------------

PKSDATAFORMAT BtVCaptureStream::Formats[] =
{
	// Declare this entry for NetMeeting!
	GET_LPDATAFORMATS(BtVCaptureStream::FormatRGB24), // Need a VideoInfoHeader RGB24 for NetMeeting!

	// Now the VIDEOINFOHEADER2 so they won't be used by the selection box ...
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatRGB24_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatRGB32_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatYUY2_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatYVU9_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatUYVY_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatY41P_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatYV12_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatI420_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatRGB555_2),
	GET_LPDATAFORMATS2(BtVCaptureStream::FormatRGB565_2),

	// And now the VIDEOINFOHEADER
	GET_LPDATAFORMATS(BtVCaptureStream::FormatRGB24), // Repeated to workaround Dshow flaws
	GET_LPDATAFORMATS(BtVCaptureStream::FormatRGB32),
	GET_LPDATAFORMATS(BtVCaptureStream::FormatYUY2),
	GET_LPDATAFORMATS(BtVCaptureStream::FormatYVU9),
	GET_LPDATAFORMATS(BtVCaptureStream::FormatUYVY),
	GET_LPDATAFORMATS(BtVCaptureStream::FormatY41P),
	GET_LPDATAFORMATS(BtVCaptureStream::FormatYV12),
	GET_LPDATAFORMATS(BtVCaptureStream::FormatI420),
	GET_LPDATAFORMATS(BtVCaptureStream::FormatRGB555),
	GET_LPDATAFORMATS(BtVCaptureStream::FormatRGB565)
};



// ------------------------------------------------------------------------
// Property set for all video capture/preview streams
// ------------------------------------------------------------------------

const KSPROPERTY_ITEM BtVCaptureStream::Connection[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_CONNECTION_ALLOCATORFRAMING,
      TRUE,                                   // GetSupported or Handler
      sizeof( KSPROPERTY ),                   // MinProperty
      sizeof( KSALLOCATOR_FRAMING ),          // MinData
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      sizeof( ULONG )                         // SerializedSize
   )
};

const KSPROPERTY_ITEM BtVCaptureStream::DroppedFrames[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_DROPPEDFRAMES_CURRENT,
      TRUE,                                   // GetSupported or Handler
      sizeof(KSPROPERTY_DROPPEDFRAMES_CURRENT_S),// MinProperty
      sizeof(KSPROPERTY_DROPPEDFRAMES_CURRENT_S),// MinData
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      0                                       // SerializedSize
   )
};

const KSPROPERTY_ITEM BtVCaptureStream::Allocator[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_ALLOCATOR_CONTROL_HONOR_COUNT,
      TRUE,                                   // GetSupported or Handler
      sizeof(KSPROPERTY),                     // MinProperty
      sizeof(BOOL),		                      // MinData
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      0                                       // SerializedSize
   ),
};

// ------------------------------------------------------------------------
// Array of all of the property sets supported by video capture/preview streams
// ------------------------------------------------------------------------

KSPROPERTY_SET BtVCaptureStream::Properties[] =
{
    DEFINE_KSPROPERTY_SET
    (
        &KSPROPSETID_Connection,                          // Set
        SIZEOF_ARRAY( BtVCaptureStream::Connection ),      // PropertiesCount
        BtVCaptureStream::Connection,					  // PropertyItem
        0,                                                // FastIoCount
        NULL                                              // FastIoTable
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_VIDCAP_DROPPEDFRAMES,				    // Set
        SIZEOF_ARRAY( BtVCaptureStream::DroppedFrames ),		// PropertiesCount
        BtVCaptureStream::DroppedFrames,                     // PropertyItem
        0,                                                  // FastIoCount
        NULL                                                // FastIoTable
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_ALLOCATOR_CONTROL,						// Set
        SIZEOF_ARRAY( BtVCaptureStream::Allocator ),			// PropertiesCount
        BtVCaptureStream::Allocator,							// PropertyItem
        0,													// FastIoCount
        NULL												// FastIoTable
    )
};

// -----------------------------------------------------------------
// Stream 0 (Capture)
// -----------------------------------------------------------------

// HW_STREAM_OBJECT ------------------------------------------------
HW_STREAM_OBJECT BtVCaptureStream::StreamObject = 
{
	sizeof (HW_STREAM_OBJECT),              // SizeOfThisPacket
	STREAM_IDX_CAPTURE,                     // StreamNumber
	0,                                      // HwStreamExtension
	BtDigitizer::cbReceiveDataPacket,		// HwReceiveDataPacket
	BtDigitizer::cbReceiveCtrlPacket,		// HwReceiveControlPacket
	{ NULL, 0 },                            // HW_CLOCK_OBJECT
	TRUE,									// Dma
	TRUE,                                   // Pio
	0,										// HwDeviceExtension
	sizeof (KS_FRAME_INFO),                 // StreamHeaderMediaSpecific
	0,                                      // StreamHeaderWorkspace 
	TRUE,                                   // Allocator 
	NULL,                                   // HwEventRoutine
	{ 0, 0 },                               // Reserved[2]
};

// HW_STREAM_INFORMATION for each videonorm -------------------------------------------
HW_STREAM_INFORMATION BtVCaptureStream::StreamInfo = 
{
	1,                                              // NumberOfPossibleInstances
	KSPIN_DATAFLOW_OUT,                             // DataFlow
	TRUE,                                           // DataAccessible
	sizeof (BtVCaptureStream::Formats) 
		/ sizeof (PKSDATAFORMAT),                   // NumberOfFormatArrayEntries
	BtVCaptureStream::Formats,                      // StreamFormatsArray
	{
		0,                                          // ClassReserved[0]
		0,                                          // ClassReserved[1]
		0,                                          // ClassReserved[2]
		0                                           // ClassReserved[3]
	},
	SIZEOF_ARRAY(BtVCaptureStream::Properties),     // NumStreamPropArrayEntries
	(PKSPROPERTY_SET)BtVCaptureStream::Properties,  // StreamPropertiesArray
	0,                                              // NumStreamEventArrayEntries;
	0,                                              // StreamEventsArray;
	(GUID *) &PINNAME_VIDEO_CAPTURE,                // Category
	(GUID *) &PINNAME_VIDEO_CAPTURE,                // Name
	1,                                              // MediumsCount
	&BtDigitizer::StandardMedium,                   // Mediums
	FALSE
};


// Functions
BtVCaptureStream::BtVCaptureStream(BtDigitizer *dev,BtVPreviewStream *Prev) :
	BtVideoStream(dev,StreamObject) , Preview(Prev)
{
	TRACE(("BtVCaptureStream::BtVCaptureStream"));
}

BtVCaptureStream::~BtVCaptureStream()
{
	TRACE(("BtVCaptureStream::~BtVCaptureStream"));
}


/*
 * Method: BtVCaptureStream::TimeStamp
 * Purpose: To Timestamp packets
 */
void BtVCaptureStream::TimeStamp( PKSSTREAM_HEADER pDataPacket , PHW_STREAM_REQUEST_BLOCK pSrb)
{
	TRACE(("BtVCaptureStream::TimeStamp"));

	// If we must timestamp ...
	if ( hMasterClock ) {

		// timestamp the buffer
		pDataPacket->OptionsFlags |= KSSTREAM_HEADER_OPTIONSF_DURATIONVALID;
		pDataPacket->OptionsFlags |= KSSTREAM_HEADER_OPTIONSF_TIMEVALID;

		HW_TIME_CONTEXT	TimeContext;
		TimeContext.HwDeviceExtension = (struct _HW_DEVICE_EXTENSION *)pSrb->HwDeviceExtension;
		TimeContext.HwStreamObject    = pSrb->StreamObject;
		TimeContext.Function          = TIME_GET_STREAM_TIME;
		StreamClassQueryMasterClockSync ( hMasterClock, &TimeContext );
		pDataPacket->PresentationTime.Time = TimeContext.Time;
		// Duration is already filled... 

#ifdef ENABLE_TIMESTAMP_FIX

		// DirectShow will fail to capture video if the Sampling time of this picture is before the 
		// sampling time of the previous picture + the dureation of the previous picture. This can happen
		// due to clock jitter and/or unstable video sources (such as VCR signals). So, we will modify
		// timing if required to avoid this...
		if (LastSamplingTime) {

			// Estimate the minimum expected time...
			LONGLONG ExpectedTime = LastSamplingTime + pDataPacket->Duration;

#if DBG
			LARGE_INTEGER expectedtm, realtm, expecteddur, realdur;
			expectedtm.QuadPart = ExpectedTime;
			realtm.QuadPart = TimeContext.Time;
			expecteddur.QuadPart = pDataPacket->Duration;
			realdur.QuadPart = pDataPacket->Duration;
#endif

			// If we time-stamped the current frame with a previous to this time, fix time!
			if (pDataPacket->PresentationTime.Time < ExpectedTime) {

				// We will adjust Duration to let the next frame arrive at the expected time...
				if (pDataPacket->Duration) {
					pDataPacket->Duration -= (ExpectedTime - pDataPacket->PresentationTime.Time);
					// Duration must be at least 100 ns!
					if (!pDataPacket->Duration) {
						pDataPacket->Duration = 1;
					}
				}

#if DBG
				realdur.QuadPart = pDataPacket->Duration;
#endif

				// We will also adjust arribal time to avoid DirectShow cancelling this frame..
				pDataPacket->PresentationTime.Time = ExpectedTime;

				ERROR(("Frame arrived sooner than expected:\n  Arrival Time: 0x%08x%08x , Expected Time: 0x%08x%08x\n Set Duration: 0x%08x%08x , Expected Time: 0x%08x%08x\n",realtm.HighPart,realtm.LowPart,expectedtm.HighPart,expectedtm.LowPart,realdur.HighPart,realdur.LowPart,expecteddur.HighPart,expecteddur.LowPart));
			}
		} 
		
		// Remember the last sampling time...
		LastSamplingTime = pDataPacket->PresentationTime.Time;

#endif

	} else {

		// don't timestamp the buffer
		pDataPacket->OptionsFlags &= ~KSSTREAM_HEADER_OPTIONSF_DURATIONVALID;
		pDataPacket->OptionsFlags &= ~KSSTREAM_HEADER_OPTIONSF_TIMEVALID;
		pDataPacket->PresentationTime.Time = 0;
		pDataPacket->Duration = 0;

#ifdef ENABLE_TIMESTAMP_FIX
		LastSamplingTime = 0;
#endif

	}

}

NTSTATUS BtVCaptureStream::Create()
{
	TRACE(("BtVCaptureStream::Create"));

	// Create VideoChannel if not created.
	if (!vc) {

		// Create video Channel if possible!
		if (!bIsVideoInfo2) {

			// (-Re)Normalize frame to be able to get Stride and ColorFormat
			ColFmt col;
			dwStride = NormalizeFrame(VidHeader,col);

			bFieldBased = FALSE;				// Frame based.
			bFieldTimeBetweenSamples = FALSE;	// Frame timing

			// We will capture the odd field unless the opposite was specified...
			VCMode vcmode = digitizer->UseOddFieldForPreview() ? VCM_AutoEven : VCM_AutoOdd;

			// Open video Channel if possible!
			vc = digitizer->GetBtCard().OpenVideoChannel(
					vcmode ,
					this,
					MRect(VidHeader.rcSource ),MRect(VidHeader.rcTarget),col
					);

		} else {

			// (-re)Normalize frame to be able to get Stride and ColorFormat
			ColFmt col;
			dwStride = NormalizeFrame(VidHeader2,col);

/* Non-Interleaved Bob (field 1 only)
KS_INTERLACE_IsInterlaced | 
KS_INTERLACE_1FieldPerSample | 
KS_INTERLACE_FieldPatField1Only |
KS_INTERLACE_DisplayModeBobOnly */

/* Non-Interleaved Bob (field 2 only)
KS_INTERLACE_IsInterlaced | 
KS_INTERLACE_1FieldPerSample | 
KS_INTERLACE_FieldPatField2Only |
KS_INTERLACE_DisplayModeBobOnly */

/* Alternating fields
KS_INTERLACE_IsInterlaced |
KS_INTERLACE_1FieldPerSample |
KS_INTERLACE_FieldPatBothRegular |
KS_INTERLACE_DisplayModeBobOrWeave  */

/* Interlaced fields
KS_INTERLACE_IsInterlaced |
KS_INTERLACE_FieldPatBothRegular |
KS_INTERLACE_DisplayModeBobOrWeave  */

			// Note: KS_INTERLACE_DisplayModeBobOnly is "set" for the following, however
			//       it is defined as 0
			// KS_INTERLACE_FieldPatField1Only is defined as 0!
			//
			if	( VidHeader2.dwInterlaceFlags & KS_INTERLACE_IsInterlaced ) { 

				// Check if 2 fields per sample. Else, we need both fields
				if ( VidHeader2.dwInterlaceFlags & KS_INTERLACE_1FieldPerSample ) { 

					// 1 Field per sample ... Or alternating fields, or one field ... Check!
					UINT Fields = VidHeader2.dwInterlaceFlags & KS_INTERLACE_FieldPatternMask;
					switch (Fields) { 
						case KS_INTERLACE_FieldPatField1Only:
							// Only one field, and it is the Odd field!
							bFieldBased = TRUE;
							bFieldTimeBetweenSamples = FALSE; // But only one!
							// Open video Channel if possible!
							vc = digitizer->GetBtCard().OpenVideoChannel(
									VCM_Odd,
									this,
									MRect( VidHeader2.rcSource ),
									MRect( VidHeader2.rcTarget ),
									col
									);
							break;
						case KS_INTERLACE_FieldPatField2Only:
							// Only one field, and it is the Even field!
							bFieldBased = TRUE;
							bFieldTimeBetweenSamples = FALSE; // But only one!
							// Open video Channel if possible!
							vc = digitizer->GetBtCard().OpenVideoChannel(
									VCM_Even,
									this,
									MRect( VidHeader2.rcSource ),
									MRect( VidHeader2.rcTarget ),
									col
									);
							break;
						case KS_INTERLACE_FieldPatBothRegular:
						case KS_INTERLACE_FieldPatBothIrregular:
							// Both fields, alternating.
							bFieldBased = TRUE;
							bFieldTimeBetweenSamples = TRUE; // Both fields
							// Open video Channel if possible!
							vc = digitizer->GetBtCard().OpenVideoChannel(
									VCM_Alternating,
									this,
									MRect( VidHeader2.rcSource ),
									MRect( VidHeader2.rcTarget ),
									col
									);
							break;
					}
				} else {
					// two fields per sample. Neccesarily interlaced video.
					// Both fields, interlaced
					bFieldBased = FALSE;
					bFieldTimeBetweenSamples = FALSE; // Frame timing
					// Open video Channel if possible!
					vc = digitizer->GetBtCard().OpenVideoChannel(
							VCM_Interlaced,
							this,
							MRect( VidHeader2.rcSource ),
							MRect( VidHeader2.rcTarget ),
							col
							);
				}

			} else {
				// Nothing specified. Simply try to open channel!
				// Open video Channel if possible!
				bFieldBased = FALSE;
				bFieldTimeBetweenSamples = FALSE; // Frame timing

				// We will capture the odd field unless the opposite was specified...
				VCMode vcmode = digitizer->UseOddFieldForPreview() ? VCM_AutoEven : VCM_AutoOdd;

				vc = digitizer->GetBtCard().OpenVideoChannel(
						vcmode ,
						this,
						MRect(VidHeader2.rcSource ),
						MRect(VidHeader2.rcTarget ),
						col
						);

			}	
		}

		if (!vc) {
			// Tell Preview we are capturing!
			Preview->DeallocChannel();

			// Create video Channel if possible!
			if (!bIsVideoInfo2) {

				// Normalize frame to be able to get Stride and ColorFormat
				ColFmt col;
				dwStride = NormalizeFrame(VidHeader,col);

				bFieldBased = FALSE;				// Frame based.
				bFieldTimeBetweenSamples = FALSE;	// Frame timing

				// We will capture the odd field unless the opposite was specified...
				VCMode vcmode = digitizer->UseOddFieldForPreview() ? VCM_AutoEven : VCM_AutoOdd;

				// Open video Channel if possible!
				vc = digitizer->GetBtCard().OpenVideoChannel(
						vcmode ,
						this,
						MRect(VidHeader.rcSource ),MRect(VidHeader.rcTarget),col
						);

			} else {

				// Normalize frame to be able to get Stride and ColorFormat
				ColFmt col;
				dwStride = NormalizeFrame(VidHeader2,col);

				// Note: KS_INTERLACE_DisplayModeBobOnly is "set" for the following, however
				//       it is defined as 0
				// KS_INTERLACE_FieldPatField1Only is defined as 0!
				//
				if	( VidHeader2.dwInterlaceFlags & KS_INTERLACE_IsInterlaced ) { 

					// Check if 2 fields per sample. Else, we need both fields
					if ( VidHeader2.dwInterlaceFlags & KS_INTERLACE_1FieldPerSample ) { 

						// 1 Field per sample ... Or alternating fields, or one field ... Check!
						UINT Fields = VidHeader2.dwInterlaceFlags & KS_INTERLACE_FieldPatternMask;
						switch (Fields) { 
							case KS_INTERLACE_FieldPatField1Only:
								// Only one field, and it is the Odd field!
								bFieldBased = TRUE;
								bFieldTimeBetweenSamples = FALSE; // But only one!
								// Open video Channel if possible!
								vc = digitizer->GetBtCard().OpenVideoChannel(
										VCM_Odd,
										this,
										MRect( VidHeader2.rcSource ),
										MRect( VidHeader2.rcTarget ),
										col
										);
								break;
							case KS_INTERLACE_FieldPatField2Only:
								// Only one field, and it is the Even field!
								bFieldBased = TRUE;
								bFieldTimeBetweenSamples = FALSE; // But only one!
								// Open video Channel if possible!
								vc = digitizer->GetBtCard().OpenVideoChannel(
										VCM_Even,
										this,
										MRect( VidHeader2.rcSource ),
										MRect( VidHeader2.rcTarget ),
										col
										);
								break;
							case KS_INTERLACE_FieldPatBothRegular:
							case KS_INTERLACE_FieldPatBothIrregular:
								// Both fields, alternating.
								bFieldBased = TRUE;
								bFieldTimeBetweenSamples = TRUE; // Both fields
								// Open video Channel if possible!
								vc = digitizer->GetBtCard().OpenVideoChannel(
										VCM_Alternating,
										this,
										MRect( VidHeader2.rcSource ),
										MRect( VidHeader2.rcTarget ),
										col
										);
								break;
						}
					} else {
						// two fields per sample. Neccesarily interlaced video.
						// Both fields, interlaced
						bFieldBased = FALSE;
						bFieldTimeBetweenSamples = FALSE; // Frame timing
						// Open video Channel if possible!
						vc = digitizer->GetBtCard().OpenVideoChannel(
								VCM_Interlaced,
								this,
								MRect( VidHeader2.rcSource ),
								MRect( VidHeader2.rcTarget ),
								col
								);
					}

				} else {
					// Nothing specified. Simply try to open channel!
					// Open video Channel if possible!
					bFieldBased = FALSE;
					bFieldTimeBetweenSamples = FALSE; // Frame timing

					// We will capture the odd field unless the opposite was specified...
					VCMode vcmode = digitizer->UseOddFieldForPreview() ? VCM_AutoEven : VCM_AutoOdd;

					vc = digitizer->GetBtCard().OpenVideoChannel(
							vcmode ,
							this,
							MRect(VidHeader2.rcSource ),
							MRect(VidHeader2.rcTarget ),
							col
							);
	
				}	
			}

			if (!vc) {

				// Realloc preview Channel
				Preview->ReallocChannel();
				return STATUS_IO_DEVICE_ERROR;
			}

		}
	}

	// Realloc preview Channel if needed!
	Preview->ReallocChannel();

	return STATUS_SUCCESS;
}

NTSTATUS BtVCaptureStream::Destroy()
{
	TRACE(("BtVCaptureStream::Destroy"));

	// Stop videoChannel and Close It!
	if (vc) { // If videoChannel already created ....
		digitizer->GetBtCard().CloseVideoChannel(vc); 
		vc = NULL;
	}

	// Realloc preview Channel if needed!
	Preview->ReallocChannel();

	return STATUS_SUCCESS;
}

/* Method: BtVCaptureStream::GetConnectionProperty
 * Purpose: Obtains allocator and state properties
 */
NTSTATUS BtVCaptureStream::GetConnectionProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVCaptureStream::GetConnectionProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;

	switch ( pSPD->Property->Id ) {
		case KSPROPERTY_CONNECTION_ALLOCATORFRAMING: {

			DUMP(("KSPROPERTY_CONNECTION_ALLOCATORFRAMING\n"));

			PKSALLOCATOR_FRAMING Framing = 
				(PKSALLOCATOR_FRAMING) pSPD->PropertyInfo;
			Framing->RequirementsFlags   =
				/*KSALLOCATOR_REQUIREMENTF_MUST_ALLOCATE |*/
	            KSALLOCATOR_REQUIREMENTF_PREFERENCES_ONLY | 
				KSALLOCATOR_REQUIREMENTF_INPLACE_MODIFIER |
				KSALLOCATOR_REQUIREMENTF_SYSTEM_MEMORY;

			Framing->PoolType = NonPagedPool;

			Framing->Frames = 8;
			if( bIsVideoInfo2 ) {
				Framing->FrameSize = VidHeader2.bmiHeader.biSizeImage;
			} else {
				Framing->FrameSize = VidHeader.bmiHeader.biSizeImage;
			}

			Framing->FileAlignment = 0; // To make Overlay mixer work
			Framing->Reserved = 0;

			DUMP(("RequirementsFlags = 0x%x , PoolType = 0x%x , Frames = %d\n",
					Framing->RequirementsFlags, Framing->PoolType , Framing->Frames ));
			DUMP(("FrameSize = %d , FileAlignment = 0x%x , Reserved = %d\n",
					Framing->FrameSize,Framing->FileAlignment, Framing->Reserved ));

			pSrb->ActualBytesTransferred = sizeof( KSALLOCATOR_FRAMING );
			return STATUS_SUCCESS;
		}
	}
	ERROR(("Unsupported property (%d)\n",pSPD->Property->Id ));
	return STATUS_INVALID_PARAMETER;
}

NTSTATUS BtVCaptureStream::GetAllocatorProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVCaptureStream::GetAllocatorProperty"));

	// This whole stupid function/property is to get the overlay mixer to work again
	// with Windows 98 SP1

    PSTREAM_PROPERTY_DESCRIPTOR pSpd = pSrb->CommandData.PropertyInfo;
    ULONG nS  = pSpd->PropertyOutputSize;        // size of data supplied

    ENSURE( nS >= sizeof( ULONG));

	switch (pSpd->Property->Id) {

		case KSPROPERTY_ALLOCATOR_CONTROL_HONOR_COUNT :

			DUMP(("KSPROPERTY_ALLOCATOR_CONTROL_HONOR_COUNT\n"));

			// [TMZ] - Setting this to TRUE screws up ring 3 overlay page flipping.
			//         But, it helps ring 0 page flipping.

			*( PBOOL ) ( pSpd->PropertyInfo) = 0;
			pSrb->ActualBytesTransferred = sizeof( BOOL ); //BOOL

			return STATUS_SUCCESS;
    }
	ERROR(("Unsupported property"));
	return STATUS_INVALID_PARAMETER;
}


/*	Used to try to let preview run while we are capturing at a 
	lower framerate than available.
*/
const VideoDataBuf BtVCaptureStream::OnSampling(bool GetSample)
{
	TRACE(("BtVCaptureStream::OnSampling"));

	// Let the preview Stream provide us a buffer!
	return Preview->OnCaptureSample(vc,GetSample);
}

HW_STREAM_INFORMATION& BtVCaptureStream::Info( void )
{
	TRACE(("BtVCaptureStream::Info"));

	// Return the correct info.
	return StreamInfo;

}