/*
	BtDigitizerStream.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "Debug.hpp"
#include "BtDigitizerStream.hpp"
#include "Adapter.hpp"
#include "BtDigitizer.hpp"

// BtDigitizerStream Functions ...

BtDigitizerStream::BtDigitizerStream(BtDigitizer *dev) :
	KSState(KSSTATE_STOP) ,
	digitizer(dev) ,
	vc(NULL) ,
	hMasterClock(NULL) ,
	bDataDiscontinuity(FALSE) ,
	dwStride(0) ,
	bStreamOpened(FALSE)
{
	TRACE(("BtDigitizerStream::BtDigitizerStream"));
}

BtDigitizerStream::~BtDigitizerStream()
{
	TRACE(("BtDigitizerStream::~BtDigitizerStream"));
}


// To cancel all pending data SRBs
void BtDigitizerStream::EmptyIncomingDataBufferQueue()
{
#ifdef ENABLE_DDRAW_STUFF
	// Release all DirectDraw surfaces
	DDraw.ReleaseAllSurfaces();
#endif

	// Now Complete with failure all pending Data SRBs.
	BufferLock.Acquire();

	// While buffers present ...
	PHW_STREAM_REQUEST_BLOCK pSrb = NULL;
	while (BufferQueue.Remove(pSrb)) {

		// Release lock.
		BufferLock.Release();

		// Signal completion with failure. Failure is indicated by setting 
		// DataUsed to 0. NEVER cancel a packet failing it with STATUS_CANCELLED 
		// (unless asked by stream class)
		pSrb->CommandData.DataBufferArray->DataUsed = 0;
		pSrb->Status = STATUS_SUCCESS;

		// Complete Data Packet
		CompleteDataPacket(pSrb);

		// Adquire lock
		BufferLock.Acquire();
	}
	// Release Lock
	BufferLock.Release();

}

/*
** BtDigitizerStream::Close()
**
**   Close the requested data stream
**
** Arguments:
**
**   pSrb the request block requesting to close the stream
**
** Returns:
**
** Side Effects:  none
*/
// - Called at PASSIVE_LEVEL
NTSTATUS BtDigitizerStream::Close( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::Close"));

#if DBG
	if (KSState != KSSTATE_STOP) {
		ERROR(("Warning: Cleanup without being in the stopped state"));
    }
#endif


	// Set to Stop State
	KSState = KSSTATE_STOP;

	// Release the VideoChannel
	NTSTATUS ret = Destroy(); /* (this modifies videoChannel) */
	if (ret != STATUS_SUCCESS) {
		return ret;
	}

	// No more references to Clock
	hMasterClock = NULL;

	// Stream is now closed.
	bStreamOpened = FALSE;

	// Now Complete all pending Data SRBs.
	EmptyIncomingDataBufferQueue();

	return STATUS_SUCCESS;
}

// This is NOT syncronized, so buffer access mut be serialized!
// -Called at PASSIVE_LEVEL. 
void BtDigitizerStream::TimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::TimeoutDataPacket"));

	// Unless we are in the Stopped State ...
	if (KSState == KSSTATE_STOP /* note: Atomic read, does not hurt */
		) { 

		// We are currently Stopped ... Try to complete buffers!
		TRACE(("Attempting to complete Srbs.\n"));

		// Now Try to Complete all pending Data SRBs.
		EmptyIncomingDataBufferQueue();

	} else {

		//
		// Unlike most devices, we need to hold onto data SRBs indefinitely,
		// since the graph could be in a pause state indefinitely
		//
		pSrb->TimeoutCounter = pSrb->TimeoutOriginal;
	}
}

// This is NOT syncronized, so buffer access mut be serialized!
// -Called at PASSIVE_LEVEL. 
void BtDigitizerStream::CancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::CancelDataPacket"));

	// Remove the request from the queue
	BufferLock.Acquire();
	bool Found = BufferQueue.RemoveThis( pSrb );
	BufferLock.Release();

	// If found, invalidate it!
	if (Found) {
	
		// Only in this case, we can really cancel a Packet!
		DUMP(("FOUND Srb=%x\n", pSrb));

#ifdef ENABLE_DDRAW_STUFF
		// Close the DirectDraw Surface...
		DDraw.RemoveSurfaceFromPool( pSrb );
#endif

		pSrb->Status = STATUS_CANCELLED;
		CompleteDataPacket(pSrb);

	} else {

		ERROR(("SRB NOT FOUND!!! - Probably in process of being filled- Marked to be cancelled!\n"));
		pSrb->Status = STATUS_CANCELLED;

	}
}

// When we get a new buffer to Fill -> Request MUST be completed by ourserves

/* Notes: -This is NOT synchronized to the processing of the stream
		  and can be called concurrently. 
		  -Called at PASSIVE_LEVEL. 
*/
void BtDigitizerStream::OnReadData( PHW_STREAM_REQUEST_BLOCK pSrb) 
{
	TRACE(("BtDigitizerStream::OnReadData"));

	// Rule: 
    // Only accept read requests when in either the Pause or Run
	// States.  If Stopped, immediately return the SRB.

	// By default, packets are performed successfully.
	pSrb->Status = STATUS_SUCCESS;

	if (KSState == KSSTATE_STOP /* atomic READ, does not hurt */
		) { 

		DUMP(("stream is stopped, so we'll just complete the srb(%x)\n", pSrb));

		pSrb->CommandData.DataBufferArray->DataUsed = 0;
		CompleteDataPacket( pSrb );

        return;
	} 

#ifdef ENABLE_DDRAW_STUFF
	// Add this surface to the DirectDraw pool
	DDraw.AddSurfaceToPool( pSrb );
#endif
	
	// Add buffer to the Queue
	BufferLock.Acquire();
	
	// Insert the request in the queue
	bool Done = BufferQueue.Insert( pSrb );

	// Buffer queued, release mutex
	BufferLock.Release();

	// If failed...
	if (!Done) {

#ifdef ENABLE_DDRAW_STUFF
		// Remove surface from DirectDraw pool
		DDraw.RemoveSurfaceFromPool( pSrb );
#endif

		// If failed, return buffer...
		pSrb->CommandData.DataBufferArray->DataUsed = 0;
		CompleteDataPacket( pSrb );
	}

	// Since another thread COULD HAVE MODIFIED THE STREAM STATE
	// in the midst of adding it to the queue, check the stream
	// state again, and cancel the SRB if necessary.  Note that
	// this race condition was NOT handled in the original DDK
	// release of testcap!

	if (KSState == KSSTATE_STOP) {

		// Get lock
		BufferLock.Acquire();
	
		// Remove the request from the queue
		BufferQueue.RemoveThis( pSrb );

		// Release lock.
		BufferLock.Release();

#ifdef ENABLE_DDRAW_STUFF
		// Remove surface from DirectDraw pool
		DDraw.RemoveSurfaceFromPool( pSrb );
#endif

		DUMP(("stream is stopped, so we'll just complete the srb(%x)\n", pSrb));

		pSrb->CommandData.DataBufferArray->DataUsed = 0;
		CompleteDataPacket( pSrb );
	}
}

NTSTATUS BtDigitizerStream::GetState( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::GetState"));
	
	pSrb->CommandData.StreamState = KSState;
	pSrb->ActualBytesTransferred = sizeof (KSSTATE);

	// A very odd rule:
	// When transitioning from stop to pause, DShow tries to preroll
	// the graph.  Capture sources can't preroll, and indicate this
	// by returning VFW_S_CANT_CUE in user mode.  To indicate this
	// condition from drivers, they must return ERROR_NO_DATA_DETECTED
	//
	// JayBo says KSSTATE_ACQUIRE should return success 

	if (pSrb->CommandData.StreamState == KSSTATE_PAUSE) {
		return STATUS_NO_DATA_DETECTED;
	}

	return STATUS_SUCCESS;
}

//
// For each stream, the following states are used:
//
// Stop:    Absolute minimum resources are used.  No outstanding IRPs.
//			Must cancel all SRBs
// Acquire: KS only state that has no DirectShow correpondence
//          Acquire needed resources.
// Pause:   Getting ready to run.  Allocate needed resources so that
//          the eventual transition to Run is as fast as possible.
//          Read SRBs will be queued at either the Stream class
//          or in your driver (depending on when you send "ReadyForNext")
//          and whether you're using the Stream class for synchronization
// Run:     Streaming.
//
// Moving to Stop to Run always transitions through Pause.
//
// But since a client app could crash unexpectedly, drivers should handle
// the situation of having outstanding IRPs cancelled and open streams
// being closed WHILE THEY ARE STREAMING!
//
// Note that it is quite possible to transition repeatedly between states:
// Stop -> Pause -> Stop -> Pause -> Run -> Pause -> Run -> Pause -> Stop
//

// - Called at PASSIVE_LEVEL
NTSTATUS BtDigitizerStream::SetState( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::SetState"));

	// [TMZ]  The dropped frame reporting was broken.  One reason was that we were
	// too eager to clear the counters.  They should only be cleared at stream
	// creation and when we get a command to go to KSSTATE_PAUSE while we are
	// in either KSSTATE_ACQUIRE or KSSTATE_STOP (see testcap).

	// RUN->PAUSE \
	// ->STOP     / Both must flush buffers

	// determine which new state is requested
	const KSSTATE newKSState = pSrb->CommandData.StreamState;

	// Error status to return by default
	NTSTATUS ret = STATUS_IO_DEVICE_ERROR;

   	DUMP(("change stream state from (%d) to (%d)\n", KSState, newKSState));

	switch ( newKSState ) {

		case KSSTATE_STOP:
			{
				DUMP(("KSSTATE_STOP\n"));
				// Using absolute minimum of resources. No outstanding data SRBs at the driver.

				// Remember actual KSState
				const KSSTATE oldKSState = KSState;

				// Set State to Stopped. This will force every buffer received to be returned to 
				// the Stream Class completed with DataUsed = 0
				KSState = newKSState;
			 
				// Destroy channel (first, stop it if needed): Note that this waits until all pending (in RISC engine) buffers are processed
				ret = Destroy(); /* (modifes videoChannel) */

				// Restore status of Channel
				KSState = oldKSState;

				// If successfull ...
				if (ret == STATUS_SUCCESS ) {
	
					ENSURE(!vc);

					// Set new state
					KSState = newKSState;

					// Now Complete all pending Data SRBs.
					EmptyIncomingDataBufferQueue();

				} 
			}
			break;

		case KSSTATE_ACQUIRE:

			DUMP(("KSSTATE_ACQUIRE ... falling through to PAUSE\n"));
			// Acquiring resources. Allocating all needed resources such as bandwidth on USB and 1394

		case KSSTATE_PAUSE:

			DUMP(("KSSTATE_PAUSE\n"));
			// Preparing to instantly make transition to the Run state. 

			// Create channel if not created. 
			ret = Create(); /* (modifies videoChannel) */

			// If successfull ...
			if (ret == STATUS_SUCCESS ) {
				ENSURE(vc);

				// Ensure channel is stopped!
				if (vc->Stop() == Success) {

					// [TMZ] see comment above regarding dropped frame reporting.
					if (KSState == KSSTATE_ACQUIRE || KSState == KSSTATE_STOP) {  
						ResetCounters();
					}

					// Set new state
					KSState = newKSState;

				} else { 

					ret = STATUS_IO_DEVICE_ERROR;
				}

			}

			break;

		case KSSTATE_RUN: 

         	DUMP(("KSSTATE_RUN\n"));
			// Streaming. Fill buffers and complete SRBs through CompleteStreamSRB

			// Make sure channel is created first!
			ret = Create(); /* (modifies videoChannel) */

			// If successfull ...
			if (ret == STATUS_SUCCESS ) {
				ENSURE(vc);

				// Start streaming!
				if (vc->Start() == Success) {

					// Set new state
					KSState = newKSState;

				} else { 

					ret = STATUS_IO_DEVICE_ERROR;
				}

			}
			break;

		default:

			ERROR(("*** KSSTATE_??? (%x)\n", pSrb->CommandData.StreamState));
			return STATUS_INVALID_PARAMETER;

	}

	return ret;
}

// Check if video format is now available
NTSTATUS BtDigitizerStream::ProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::ProposeFormat"));
	if (! VerifyFormat ( pSrb->CommandData.OpenFormat ) ) {
		ERROR(("SRB_PROPOSE_DATA_FORMAT failed\n" ));
		return STATUS_NOT_SUPPORTED;
	}
	return STATUS_SUCCESS;
}

// Get actual Video Format
NTSTATUS BtDigitizerStream::GetFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::GetFormat"));

	ERROR(("Not implemented yet")); // BUGBUG

	return STATUS_NOT_IMPLEMENTED;
}

/*
** BtDigitizerStream::GetProperty()
**
**    Routine to process video property requests
**
** Arguments:
**
**    pSrb - pointer to the stream request block for properties
**
** Returns:
**
** Side Effects:  none
*/
NTSTATUS BtDigitizerStream::GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::GetProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;

	if ( KSPROPSETID_Connection == pSPD->Property->Set ) {
		return GetConnectionProperty( pSrb );
	} else if ( PROPSETID_VIDCAP_DROPPEDFRAMES == pSPD->Property->Set ) {
		return GetDroppedFramesProperty( pSrb );
	} else if ( PROPSETID_ALLOCATOR_CONTROL == pSPD->Property->Set ) {
		return GetAllocatorProperty (pSrb);
	}
	ERROR(("Property not implemented\n"));
	return STATUS_NOT_IMPLEMENTED;
}

// To SET stream properties
NTSTATUS BtDigitizerStream::SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::SetProperty"));

	ERROR(("Not implemented yet")); // BUGBUG

	return STATUS_NOT_IMPLEMENTED;

}

// Indicate the stream that will we master
NTSTATUS BtDigitizerStream::IndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::IndicateMasterClock"));

	hMasterClock = pSrb->CommandData.MasterClockHandle;

	return STATUS_SUCCESS;
}

// Routine called to Change notifications of VideoChannel

/* Note: This is NOT synchronized to the processing of the other streams
 *	and can be called concurrently.   */

void BtDigitizerStream::ChangeNotification( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtDigitizerStream::ChangeNotification")); // NOOP for video classes
}

BOOL BtDigitizerStream::IsOpen( void )
{
	TRACE(("BtDigitizerStream::IsOpen"));
	return bStreamOpened;
}