/*
	BtPreviewStream.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Log:
	- 10/21/2001	ejtagle		Reduced number of videoformats to 5 per channel due to limits of WDM wrapper

*/


#include "Debug.hpp"
#include "WDMHeaders.hpp"
#include "BtPreviewStream.hpp"
#include "BtDigitizer.hpp"
#include "VideoFormats.hpp"


/* 
 Notes: 

	1] Some programs get the first stream format available for a given colorformat
	   and try to open the stream in that format (the default resolution). VideoForWindows wrappers
	   do this.
	2] The WDM wrapper (kswdmcap.ax) does not offer the maximum resolution of the stream
	   as one of the resolutions available, unless one of those resolutions matches the 
	   one of the internal listing. This means that the listing of available resolutions
	   listed by kswdmcap.ax (default resolution&colorspace picker under DirectShow) simply
	   are internally fixed, and they are shown if the stream allows it. At least the default
	   resolution is taken from the stream.
	3] NetMeeting is looking for a digital camera (KS_AnalogVideoNone), so we must provide 
	   and entry for it if we want to use it.
	4] Overlay Mixer normally gives negative resolutions!!!! - Be careful

  So , to cope with this we will have to:
	-We must provide as the first on the capture stream an entry with KS_AnalogVideoNone
	 , colorspace YUY2 and default resolution of 320x240 for netmeeting and VFW2WDM wrappers
	-The first entry on the preview stream has to have 320x240 as the default resolution and be VIDEOINFOHEADER
	
  */


// RGB 32
KS_DATARANGE_VIDEO BtVPreviewStream::FormatRGB32[] = {
	DECL_PREVIEWSTRFMT( 32, 32, 
		0xe436eb7e, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 , 
		1, 1, KS_BI_RGB)
};

// RGB24
KS_DATARANGE_VIDEO BtVPreviewStream::FormatRGB24[] = { 
	DECL_PREVIEWSTRFMT( 24, 24, 
		0xe436eb7d, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 ,
		1, 1, KS_BI_RGB)
};

// RGB555
KS_DATARANGE_VIDEO BtVPreviewStream::FormatRGB555[] = {
	DECL_PREVIEWSTRFMT( 16, 16, 
		0xe436eb7c, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70,
		1, 1, KS_BI_RGB)
};

// RGB565
KS_DATARANGE_VIDEO_WITHMASK BtVPreviewStream::FormatRGB565[] = {
	DECL_PREVIEWSTRFMT_WITHMASK( 16, 16, 
		0xe436eb7b, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70, 
		1, 1, KS_BI_BITFIELDS, 0xF800 , 0x07E0 , 0x001F)
};

// YVU9 = YUV9 = Planar : Cada 16Y, hay 1 U, 1 V 
KS_DATARANGE_VIDEO BtVPreviewStream::FormatYVU9[] = {
	DECL_PREVIEWSTRFMT( 9, 9,/*16, */
		KS_BI_YVU9, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		16, 4, KS_BI_YVU9)
};

// YUY2 = YUY2 Mixed Y Cb Y Cr 
KS_DATARANGE_VIDEO BtVPreviewStream::FormatYUY2[] = {
	DECL_PREVIEWSTRFMT( 16, 16, 
		KS_BI_YUY2, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		2, 1, KS_BI_YUY2)
};

// UYVY = YUY2 Mixed Y Cb Y Cr 
KS_DATARANGE_VIDEO BtVPreviewStream::FormatUYVY[] = {
	DECL_PREVIEWSTRFMT( 16, 16, 
		KS_BI_UYVY, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		2, 1, KS_BI_UYVY)
};

// Y41P = BTYUV = 8Y ,2U, 2V (12bpp)
KS_DATARANGE_VIDEO BtVPreviewStream::FormatY41P[] = {
	DECL_PREVIEWSTRFMT( 12, 12, 
		KS_BI_Y41P, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 1, KS_BI_Y41P)
};
		
// YV12 = Y422 Planar 
KS_DATARANGE_VIDEO BtVPreviewStream::FormatYV12[] = {
	DECL_PREVIEWSTRFMT( 12, 12, 
		KS_BI_YV12, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 2, KS_BI_YV12)
};

// I420 = Y422 Planar 
KS_DATARANGE_VIDEO BtVPreviewStream::FormatI420[] = {
	DECL_CAPTURESTRFMT( 12, 12, 
		KS_BI_I420, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 2, KS_BI_I420)
};

// For the advanced users, the Videoinfo2 struct!

// RGB 32
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatRGB32_2[] = {
	DECL_PREVIEWSTRFMT2( 32, 32, 
		0xe436eb7e, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 , 
		1, 1, KS_BI_RGB)
};

// RGB24
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatRGB24_2[] = { 
	DECL_PREVIEWSTRFMT2( 24, 24, 
		0xe436eb7d, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70 ,
		1, 1, KS_BI_RGB)
};

// RGB555
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatRGB555_2[] = {
	DECL_PREVIEWSTRFMT2( 16, 16, 
		0xe436eb7c, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70,
		1, 1, KS_BI_RGB)
};

// RGB565
KS_DATARANGE_VIDEO2_WITHMASK BtVPreviewStream::FormatRGB565_2[] = {
	DECL_PREVIEWSTRFMT2_WITHMASK( 16, 16, 
		0xe436eb7b, 0x524f, 0x11ce, 0x9f, 0x53, 0x00, 0x20, 0xaf, 0x0b, 0xa7, 0x70, 
		1, 1, KS_BI_BITFIELDS, 0xF800 , 0x07E0 , 0x001F)
};

// YVU9 = YUV9 = Planar : Cada 16Y, hay 1 U, 1 V 
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatYVU9_2[] = {
	DECL_PREVIEWSTRFMT2( 9, 9,/*16, */
		KS_BI_YVU9, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		16, 4, KS_BI_YVU9)
};

// YUY2 = YUY2 Mixed Y Cb Y Cr 
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatYUY2_2[] = {
	DECL_PREVIEWSTRFMT2( 16, 16, 
		KS_BI_YUY2, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		2, 1, KS_BI_YUY2)
};

// UYVY = YUY2 Mixed Y Cb Y Cr 
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatUYVY_2[] = {
	DECL_PREVIEWSTRFMT2( 16, 16, 
		KS_BI_UYVY, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		2, 1, KS_BI_UYVY)
};

// Y41P = BTYUV = 8Y ,2U, 2V (12bpp)
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatY41P_2[] = {
	DECL_PREVIEWSTRFMT2( 12, 12, 
		KS_BI_Y41P, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 1, KS_BI_Y41P)
};
		
// YV12 = Y422 Planar 
// Since we couldn't find the codec for YV12, this stream of format uses I420 for compression
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatYV12_2[] = {
	DECL_PREVIEWSTRFMT2( 12, 12, 
		KS_BI_YV12, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 2, KS_BI_YV12)
};

// I420 = Y422 Planar 
KS_DATARANGE_VIDEO2 BtVPreviewStream::FormatI420_2[] = {
	DECL_CAPTURESTRFMT2( 12, 12, 
		KS_BI_I420, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71, 
		8, 2, KS_BI_I420)
};

/* --------------------------------------------- */


//---------------------------------------------------------------------------
//  Stream 1 (Preview) Formats: Use RGB or YUV
//---------------------------------------------------------------------------

PKSDATAFORMAT BtVPreviewStream::Formats[] =
{
	GET_LPDATAFORMATS(BtVPreviewStream::FormatYUY2), // Need this first so the overlay Mixer works

	// VIDEOINFOHEADER2 structs
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatYUY2_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatRGB24_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatRGB32_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatYVU9_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatUYVY_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatY41P_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatYV12_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatI420_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatRGB555_2),
	GET_LPDATAFORMATS2(BtVPreviewStream::FormatRGB565_2),

	// VIDEOINFOHEADER structs
	GET_LPDATAFORMATS(BtVPreviewStream::FormatYUY2), // To workaround DShow bugs...
	GET_LPDATAFORMATS(BtVPreviewStream::FormatRGB24),
	GET_LPDATAFORMATS(BtVPreviewStream::FormatRGB32),
	GET_LPDATAFORMATS(BtVPreviewStream::FormatYVU9),
	GET_LPDATAFORMATS(BtVPreviewStream::FormatUYVY),
	GET_LPDATAFORMATS(BtVPreviewStream::FormatY41P),
	GET_LPDATAFORMATS(BtVPreviewStream::FormatYV12),
	GET_LPDATAFORMATS(BtVPreviewStream::FormatI420),
	GET_LPDATAFORMATS(BtVPreviewStream::FormatRGB555),
	GET_LPDATAFORMATS(BtVPreviewStream::FormatRGB565)
};


// ------------------------------------------------------------------------
// Property set for all video capture/preview streams
// ------------------------------------------------------------------------

const KSPROPERTY_ITEM BtVPreviewStream::Connection[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_CONNECTION_ALLOCATORFRAMING,
      TRUE,                                   // GetSupported or Handler
      sizeof( KSPROPERTY ),                   // MinProperty
      sizeof( KSALLOCATOR_FRAMING ),          // MinData
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      sizeof( ULONG )                         // SerializedSize
   )
};

const KSPROPERTY_ITEM BtVPreviewStream::DroppedFrames[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_DROPPEDFRAMES_CURRENT,
      TRUE,                                   // GetSupported or Handler
      sizeof(KSPROPERTY_DROPPEDFRAMES_CURRENT_S),// MinProperty
      sizeof(KSPROPERTY_DROPPEDFRAMES_CURRENT_S),// MinData
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      0                                       // SerializedSize
   )
};

const KSPROPERTY_ITEM BtVPreviewStream::Allocator[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_ALLOCATOR_CONTROL_HONOR_COUNT,
      TRUE,                                   // GetSupported or Handler
      sizeof(KSPROPERTY),                     // MinProperty
      sizeof(BOOL),			                  // MinData
      FALSE,                                  // SetSupported or Handler
      NULL,                                   // Values
      0,                                      // RelationsCount
      NULL,                                   // Relations
      NULL,                                   // SupportHandler
      0                                       // SerializedSize
   ),
};

// ------------------------------------------------------------------------
// Array of all of the property sets supported by video capture/preview streams
// ------------------------------------------------------------------------

KSPROPERTY_SET BtVPreviewStream::Properties[] =
{
    DEFINE_KSPROPERTY_SET
    (
        &KSPROPSETID_Connection,                          // Set
        SIZEOF_ARRAY( BtVPreviewStream::Connection ),      // PropertiesCount
        BtVPreviewStream::Connection,					  // PropertyItem
        0,                                                // FastIoCount
        NULL                                              // FastIoTable
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_VIDCAP_DROPPEDFRAMES,				    // Set
        SIZEOF_ARRAY( BtVPreviewStream::DroppedFrames ),		// PropertiesCount
        BtVPreviewStream::DroppedFrames,                     // PropertyItem
        0,                                                  // FastIoCount
        NULL                                                // FastIoTable
    ),
    DEFINE_KSPROPERTY_SET
    ( 
        &PROPSETID_ALLOCATOR_CONTROL,						// Set
        SIZEOF_ARRAY( BtVPreviewStream::Allocator ),			// PropertiesCount
        BtVPreviewStream::Allocator,							// PropertyItem
        0,													// FastIoCount
        NULL												// FastIoTable
    )
};

// -----------------------------------------------------------------
// Stream 1 (Preview)
// -----------------------------------------------------------------


// HW_STREAM_OBJECT ------------------------------------------------
HW_STREAM_OBJECT BtVPreviewStream::StreamObject =
{
	sizeof (HW_STREAM_OBJECT),						// SizeOfThisPacket
	STREAM_IDX_PREVIEW,								// StreamNumber
	0,												// HwStreamExtension
	BtDigitizer::cbReceiveDataPacket,				// HwReceiveDataPacket
	BtDigitizer::cbReceiveCtrlPacket,				// HwReceiveControlPacket
	{ NULL, 0 },									// HW_CLOCK_OBJECT
	TRUE,											// Dma
	TRUE,				                            // Pio
	0,												// HwDeviceExtension
	sizeof (KS_FRAME_INFO),							// StreamHeaderMediaSpecific
	0,												// StreamHeaderWorkspace 
	TRUE,											// Allocator 
	NULL,											// HwEventRoutine
	{ 0, 0 },										// Reserved[2]
};

// HW_STREAM_INFORMATION for each videonorm -------------------------------------------
HW_STREAM_INFORMATION BtVPreviewStream::StreamInfo =
{
	1,                                              // NumberOfPossibleInstances
	KSPIN_DATAFLOW_OUT,                             // DataFlow
	TRUE,                                           // DataAccessible
	sizeof (BtVPreviewStream::Formats) 
			/ sizeof (PKSDATAFORMAT),                   // NumberOfFormatArrayEntries
		BtVPreviewStream::Formats,                      // StreamFormatsArray
	{
	    0,                                          // ClassReserved[0]
		0,                                          // ClassReserved[1]
		0,                                          // ClassReserved[2]
		0                                           // ClassReserved[3]
	},
	SIZEOF_ARRAY(BtVPreviewStream::Properties),     // NumStreamPropArrayEntries
	(PKSPROPERTY_SET)BtVPreviewStream::Properties,  // StreamPropertiesArray
	0,                                              // NumStreamEventArrayEntries;
	0,                                              // StreamEventsArray;
	(GUID *) &PINNAME_VIDEO_PREVIEW,                // Category
	(GUID *) &PINNAME_VIDEO_PREVIEW,                // Name
	1,                                              // MediumsCount
	&BtDigitizer::StandardMedium,                   // Mediums
	FALSE
};


// Functions
BtVPreviewStream::BtVPreviewStream(BtDigitizer *dev) :
	BtVideoStream(dev,StreamObject) , bWasAlloc(false) 
{
	TRACE(("BtVPreviewStream::BtVPreviewStream"));
}

BtVPreviewStream::~BtVPreviewStream()
{
	TRACE(("BtVPreviewStream::~BtVPreviewStream"));
}


/*
 * Method: BtVPreviewStream::TimeStamp
 * Purpose: To Timestamp packets
 */
void BtVPreviewStream::TimeStamp( PKSSTREAM_HEADER pDataPacket , PHW_STREAM_REQUEST_BLOCK pSrb)
{
	TRACE(("BtVPreviewStream::TimeStamp"));

	// For preview. don't timestamp the buffer
	pDataPacket->OptionsFlags &= ~KSSTREAM_HEADER_OPTIONSF_DURATIONVALID;
	pDataPacket->OptionsFlags &= ~KSSTREAM_HEADER_OPTIONSF_TIMEVALID;
	pDataPacket->PresentationTime.Time = 0;
	pDataPacket->Duration = 0; 

}

NTSTATUS BtVPreviewStream::Create()
{
	TRACE(("BtVPreviewStream::Create"));

	// Create VideoChannel if not created.
	if (!vc) {

		// Create video Channel if possible!
		if (!bIsVideoInfo2) {

			// Normalize frame to be able to get Stride and ColorFormat
			ColFmt col;
			dwStride = NormalizeFrame(VidHeader,col);

			bFieldBased = FALSE;				// Frame based.
			bFieldTimeBetweenSamples = FALSE;	// Frame timing

			// We will capture the even field unless the opposite was specified...
			VCMode vcmode = digitizer->UseOddFieldForPreview() ? VCM_AutoOdd : VCM_AutoEven;

			// Open video Channel if possible!
			vc = digitizer->GetBtCard().OpenVideoChannel(
					vcmode ,
					this,
					MRect(VidHeader.rcSource ),MRect(VidHeader.rcTarget),col
					);

		} else {

			// Normalize frame to be able to get Stride and ColorFormat
			ColFmt col;
			dwStride = NormalizeFrame(VidHeader2,col);

/* Non-Interleaved Bob (field 1 only)
KS_INTERLACE_IsInterlaced | 
KS_INTERLACE_1FieldPerSample | 
KS_INTERLACE_FieldPatField1Only |
KS_INTERLACE_DisplayModeBobOnly */

/* Non-Interleaved Bob (field 2 only)
KS_INTERLACE_IsInterlaced | 
KS_INTERLACE_1FieldPerSample | 
KS_INTERLACE_FieldPatField2Only |
KS_INTERLACE_DisplayModeBobOnly */

/* Alternating fields
KS_INTERLACE_IsInterlaced |
KS_INTERLACE_1FieldPerSample |
KS_INTERLACE_FieldPatBothRegular |
KS_INTERLACE_DisplayModeBobOrWeave  */

/* Interlaced fields
KS_INTERLACE_IsInterlaced |
KS_INTERLACE_FieldPatBothRegular |
KS_INTERLACE_DisplayModeBobOrWeave  */

			// Note: KS_INTERLACE_DisplayModeBobOnly is "set" for the following, however
			//       it is defined as 0
			// KS_INTERLACE_FieldPatField1Only is defined as 0!
			//
			if	( VidHeader2.dwInterlaceFlags & KS_INTERLACE_IsInterlaced ) { 

				// Check if 2 fields per sample. Else, we need both fields
				if ( VidHeader2.dwInterlaceFlags & KS_INTERLACE_1FieldPerSample ) { 

					// 1 Field per sample ... Or alternating fields, or one field ... Check!
					UINT Fields = VidHeader2.dwInterlaceFlags & KS_INTERLACE_FieldPatternMask;
					switch (Fields) { 
						case KS_INTERLACE_FieldPatField1Only:
							// Only one field, and it is the Odd field!
							bFieldBased = TRUE;
							bFieldTimeBetweenSamples = FALSE; // But only one!
							// Open video Channel if possible!
							vc = digitizer->GetBtCard().OpenVideoChannel(
									VCM_Odd,
									this,
									MRect( VidHeader2.rcSource ),
									MRect( VidHeader2.rcTarget ),
									col
									);
							break;
						case KS_INTERLACE_FieldPatField2Only:
							// Only one field, and it is the Even field!
							bFieldBased = TRUE;
							bFieldTimeBetweenSamples = FALSE; // But only one!
							// Open video Channel if possible!
							vc = digitizer->GetBtCard().OpenVideoChannel(
									VCM_Even,
									this,
									MRect( VidHeader2.rcSource ),
									MRect( VidHeader2.rcTarget ),
									col
									);
							break;
						case KS_INTERLACE_FieldPatBothRegular:
						case KS_INTERLACE_FieldPatBothIrregular:
							// Both fields, alternating.
							bFieldBased = TRUE;
							bFieldTimeBetweenSamples = TRUE; // Both fields
							// Open video Channel if possible!
							vc = digitizer->GetBtCard().OpenVideoChannel(
									VCM_Alternating,
									this,
									MRect( VidHeader2.rcSource ),
									MRect( VidHeader2.rcTarget ),
									col
									);
							break;
					}
				} else {
					// two fields per sample. Neccesarily interlaced video.
					// Both fields, interlaced
					bFieldBased = FALSE;
					bFieldTimeBetweenSamples = FALSE; // Frame timing
					// Open video Channel if possible!
					vc = digitizer->GetBtCard().OpenVideoChannel(
							VCM_Interlaced,
							this,
							MRect( VidHeader2.rcSource ),
							MRect( VidHeader2.rcTarget ),
							col
							);
				}

			} else {
				// Nothing specified. Simply try to open channel!
				// Open video Channel if possible!
				bFieldBased = FALSE;
				bFieldTimeBetweenSamples = FALSE; // Frame timing

				// We will capture the even field unless the opposite was specified...
				VCMode vcmode = digitizer->UseOddFieldForPreview() ? VCM_AutoOdd : VCM_AutoEven;

				vc = digitizer->GetBtCard().OpenVideoChannel(
						vcmode ,
						this,
						MRect(VidHeader2.rcSource ),
						MRect(VidHeader2.rcTarget ),
						col
						);

			}	
		}

		// If failed, simply virtualize it!
		if (!vc) {
			CreateVirtualized();
		}

		if (!vc) {
			return STATUS_IO_DEVICE_ERROR;
		}
	}
	return STATUS_SUCCESS;
}

NTSTATUS BtVPreviewStream::Destroy()
{
	TRACE(("BtVPreviewStream::Destroy"));

	// Stop videoChannel and Close It!
	if (vc) { // If videoChannel already created ....
		digitizer->GetBtCard().CloseVideoChannel(vc); 
		vc = NULL;
	}

	return STATUS_SUCCESS;
}

/* Method: BtVPreviewStream::GetConnectionProperty
 * Purpose: Obtains allocator and state properties
 */
NTSTATUS BtVPreviewStream::GetConnectionProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVPreviewStream::GetConnectionProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
	ULONG Id = pSPD->Property->Id;              // index of the property

	switch ( Id ) {
		case KSPROPERTY_CONNECTION_ALLOCATORFRAMING: {

			DUMP(("KSPROPERTY_CONNECTION_ALLOCATORFRAMING\n"));

			PKSALLOCATOR_FRAMING Framing = 
				(PKSALLOCATOR_FRAMING) pSPD->PropertyInfo;
			Framing->RequirementsFlags   =
				/*KSALLOCATOR_REQUIREMENTF_MUST_ALLOCATE |*/
	            KSALLOCATOR_REQUIREMENTF_PREFERENCES_ONLY | 
				KSALLOCATOR_REQUIREMENTF_INPLACE_MODIFIER |
				KSALLOCATOR_REQUIREMENTF_SYSTEM_MEMORY;

			Framing->PoolType = NonPagedPool;

			Framing->Frames = 6;
			if( bIsVideoInfo2 ) {
				Framing->FrameSize = VidHeader2.bmiHeader.biSizeImage;
			} else {
				Framing->FrameSize = VidHeader.bmiHeader.biSizeImage;
			}

			Framing->FileAlignment = 0; // To make Overlay mixer work
			Framing->Reserved = 0;

			DUMP(("RequirementsFlags = 0x%x , PoolType = 0x%x , Frames = %d\n",
					Framing->RequirementsFlags, Framing->PoolType , Framing->Frames ));
			DUMP(("FrameSize = %d , FileAlignment = 0x%x , Reserved = %d\n",
					Framing->FrameSize,Framing->FileAlignment, Framing->Reserved ));

			pSrb->ActualBytesTransferred = sizeof( KSALLOCATOR_FRAMING );
			return STATUS_SUCCESS;
		}
	}
	ERROR(("Unsupported property (%d)\n",Id ));
	return STATUS_INVALID_PARAMETER;
}

NTSTATUS BtVPreviewStream::GetAllocatorProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVPreviewStream::GetAllocatorProperty"));

	// This whole stupid function/property is to get the overlay mixer to work again
	// with Windows 98 SP1

    PSTREAM_PROPERTY_DESCRIPTOR pSpd = pSrb->CommandData.PropertyInfo;
    ULONG Id  = pSpd->Property->Id;              // index of the property
    ULONG nS  = pSpd->PropertyOutputSize;        // size of data supplied

    ENSURE( nS >= sizeof( ULONG));

	switch (Id) {
		
		case KSPROPERTY_ALLOCATOR_CONTROL_HONOR_COUNT :

			DUMP(("KSPROPERTY_ALLOCATOR_CONTROL_HONOR_COUNT\n"));

			// [TMZ] - Setting this to TRUE screws up ring 3 overlay page flipping.
			//         But, it helps ring 0 page flipping.

			*( PBOOL ) ( pSpd->PropertyInfo) = 0;
			pSrb->ActualBytesTransferred = sizeof( BOOL); //BOOL

			return STATUS_SUCCESS;
    }
	ERROR(("Unsupported property"));
	return STATUS_INVALID_PARAMETER;
}

void BtVPreviewStream::CreateVirtualized()
{
	if (vc) { // Real channel, dont virtualize!
		return;
	}

	// Open a virtual Channel
	if (!bIsVideoInfo2) {

		// Remember it wasn't open as alternating fields!
		bFieldBased = FALSE;

		// Normalize frame to be able to get Stride and ColorFormat
		ColFmt col;
		dwStride = NormalizeFrame(VidHeader,col);

		// Open video Channel if possible!
		vc = digitizer->GetBtCard().OpenVideoChannel(
				VCM_Virtual,
				this,
				MRect(VidHeader.rcSource ),
				MRect(VidHeader.rcTarget ),
				col
				);

	} else {

		// Assume that it wasn't open as alternating fields!
		bFieldBased = FALSE;

		// Normalize frame to be able to get Stride and ColorFormat
		ColFmt col;
		dwStride = NormalizeFrame(VidHeader2,col);

		// Note: KS_INTERLACE_DisplayModeBobOnly is "set" for the following, however
		//       it is defined as 0
		if	(( VidHeader2.dwInterlaceFlags & KS_INTERLACE_IsInterlaced ) &&
			(  VidHeader2.dwInterlaceFlags & KS_INTERLACE_1FieldPerSample )) {

			// We were asked to open an alternating channel!
			bFieldBased = TRUE;
		}

		// Open video Channel if possible!
		vc = digitizer->GetBtCard().OpenVideoChannel(
				VCM_Virtual,
				this,
				MRect( VidHeader2.rcSource ),
				MRect( VidHeader2.rcTarget ),
				col
			);
	}

	// There wont be errors with a virtual channel!
	ENSURE(vc);
}

// Some problems arise here ... 
// - If the channel IS NOT a virtual channel, go on to a virtual one!

/* Note: This will be called from the Capture Stream, so this entry point is
   asyncronous to the rest of the functions of this stream */
bool BtVPreviewStream::DeallocChannel()
{
	bWasAlloc = false;
	if (vc && vc->GetMode() != VCM_Virtual) {
		bWasAlloc = true;
		Destroy();

		return true;
	}

	return false;
}

bool BtVPreviewStream::ReallocChannel()
{
	if (bWasAlloc) {
		ENSURE(vc == NULL);

		// Recreate channel! - Can be a virtual one!
		Create();
		ENSURE(vc);

		// If we were running, restart stream!
		if (KSState == KSSTATE_RUN) {
			vc->Start();
		}

		bWasAlloc = false; // No more allocations!

		return true;
	}

	return false;
}

/*
 * Function to let the capture Stream get a buffer of the preview Stream for previewving purposes!
 */
/*
 * Note: This is an asyncronous entry point respect the other fns of this stream:
		 But, we can count on all VideChannel parameters to remain constant, as to
		 modify them the respective channel must be stopped first, but the stop will
		 wait until this fn has returned, because this fn is executing from the RISC
		 interrupt handler, and the Stop cannot proceed if thr RISC int is executing.
 */
const VideoDataBuf BtVPreviewStream::OnCaptureSample(VideoChannel* Capture, bool GetBuffer)
{
	TRACE(("BtVPreviewStream::OnCaptureSample"));

	// If we were called without a capture channel, dont bother doing this!
	if (!Capture) {
		return VideoDataBuf();
	}

	// if we dont have a VideoChannel, dont bother doing this!
	if (!vc) {
		return VideoDataBuf();
	}

	// If we have a real VideoChannel for preview, dont provide a buffer to be filled by the capture channel
	if (vc->GetMode() != VCM_Virtual) {
		return VideoDataBuf();
	}

	// If we are NOT previewing, return a NULL Buffer
	if (KSState != KSSTATE_RUN) {
		return VideoDataBuf();
	}

	// -------------------------------------------------------------------------
	// We will reach here if we have a Virtual Channel for preview, and capture
	// is using a real channel (Capture does always use one of them) and we are
	// previewing!
	// -------------------------------------------------------------------------

	// If color format and/or size does not match, or we are not required to return a buffer,
	// we wont provide a buffer. Just use this to Actualize Time between buffers
	// 
	if (!GetBuffer ||
		(Capture->GetColorFormat() != vc->GetColorFormat()) ||	// Or colorformat is not the same ...
		(Capture->GetAnalogWindow() != vc->GetAnalogWindow()) ||// Or AnalogWindow is not the same ...
		(Capture->GetDigitalWindow() != vc->GetDigitalWindow()) || // Or digitalWindow is not the same ...
	    (KSState != KSSTATE_RUN)								// Only preview in the RUN state
		) 
	{
		// Get requested time between samples
		REFERENCE_TIME TimeBetweenSamples;
		if (bIsVideoInfo2) {
			TimeBetweenSamples = VidHeader2.AvgTimePerFrame;
		} else {
			TimeBetweenSamples = VidHeader.AvgTimePerFrame;
		}

		// Get Time between Hardware taken samples ...
		REFERENCE_TIME TimeBetweenHwSampling = 
			(digitizer->GetBtCard().TimeBetweenFields() << ((bFieldTimeBetweenSamples) ? 0 : 1) );

		// If Time between samples requested is less than possible (faster FPS than available), 
		// set it to the maximum possible!
		if (TimeBetweenSamples < TimeBetweenHwSampling) {
			ERROR(("Requested more FPS than available"));
			TimeBetweenSamples = TimeBetweenHwSampling;
		}

		// Calculate time to next Hardware sampling
		TimeToNextFrame += TimeBetweenHwSampling;

		// Unfortunately, maybe we wont be able to keep up to the preview rate. If TimeToNextFrame
		// seems to grow too large, it can Wrap Around. Keep it reasonable.
		if (TimeToNextFrame > TimeBetweenSamples * 100) {

			// Leave only the remaining time to the next sample! - This avoids the WrapAround!
			TimeToNextFrame -= TimeBetweenSamples * 100;
		}

		// Return a NULL buffer!
		return VideoDataBuf();
	}

	// If we reach here, we are supposed to get a Buffer to be filled for preview by the capture channel
	// We will get a new buffer ... Get it and return it!
	return StreamGetNextBuffer();

}
		
HW_STREAM_INFORMATION& BtVPreviewStream::Info( void )
{
	TRACE(("BtVPreviewStream::Info"));
	
	// Return the correct info.
	return StreamInfo;
}
