/*
	BtVideoProcAmp.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "BtVideoProcAmp.hpp"

// ------------------------------------------------------------------------
// Property set for VideoProcAmp
// ------------------------------------------------------------------------

#define DEF_RANGEANDSTEP( min , max , steps )						\
   {																\
      ((max)-(min))/(steps),	/* SteppingDelta (range / steps) */ \
      0,				/* Reserved */								\
      {																\
         {															\
            (min),		/* Minimum inclusive */						\
            (max)		/* Maximum inclusive */						\
         }															\
      }																\
   }

// ------------------------------------------------------------------------

/*
Gamma:
	The value of the gamma setting is expressed in gamma multiplied by 100.
	The value type must be VT_I4. The required range must be a subset of 1 
	to 500, and the default values are typically 100 (gamma = 1) or 220 
	(gamma = 2.2).
 */
KSPROPERTY_STEPPING_LONG BtVideoProcAmp::GammaRangeAndStep [] =
{
	DEF_RANGEANDSTEP( 100 , 280 , 9)
};
const ULONG BtVideoProcAmp::GammaDefault = 140;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::GammaMembersList [] =
{
   {
      {
          KSPROPERTY_MEMBER_RANGES,
          sizeof (BtVideoProcAmp::GammaRangeAndStep),
          SIZEOF_ARRAY (BtVideoProcAmp::GammaRangeAndStep),
          0
      },
      (PVOID) BtVideoProcAmp::GammaRangeAndStep
   },
   {
     {
         KSPROPERTY_MEMBER_VALUES,
         sizeof( BtVideoProcAmp::GammaDefault ),
         sizeof( BtVideoProcAmp::GammaDefault ),
         KSPROPERTY_MEMBER_FLAG_DEFAULT
     },
     (PVOID) &BtVideoProcAmp::GammaDefault
   }
};

KSPROPERTY_VALUES BtVideoProcAmp::GammaValues =
{
   {
      {
         STATICGUIDOF( KSPROPTYPESETID_General ),
         VT_I4,
         0
      }
   },
   SIZEOF_ARRAY( BtVideoProcAmp::GammaMembersList ),
   BtVideoProcAmp::GammaMembersList
};

//-------------------------------------------------------------------------
/*
Sharpness:
	Specifies the sharpness value expressed in arbitrary units. Value 
	ranges from zero to 100, and the default of 50. The value type must be 
	VT_I4.	
 */
KSPROPERTY_STEPPING_LONG BtVideoProcAmp::SharpnessRangeAndStep [] =
{
	DEF_RANGEANDSTEP( 18 , 81 , 63)
};
const ULONG BtVideoProcAmp::SharpnessDefault = 50;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::SharpnessMembersList [] =
{
   {
      {
          KSPROPERTY_MEMBER_RANGES,
          sizeof (BtVideoProcAmp::SharpnessRangeAndStep),
          SIZEOF_ARRAY (BtVideoProcAmp::SharpnessRangeAndStep),
          0
      },
      (PVOID) BtVideoProcAmp::SharpnessRangeAndStep
   },
   {
     {
         KSPROPERTY_MEMBER_VALUES,
         sizeof( BtVideoProcAmp::SharpnessDefault ),
         sizeof( BtVideoProcAmp::SharpnessDefault ),
         KSPROPERTY_MEMBER_FLAG_DEFAULT
     },
     (PVOID) &BtVideoProcAmp::SharpnessDefault
   }
};

KSPROPERTY_VALUES BtVideoProcAmp::SharpnessValues =
{
   {
      {
         STATICGUIDOF( KSPROPTYPESETID_General ),
         VT_I4,
         0
      }
   },
   SIZEOF_ARRAY( BtVideoProcAmp::SharpnessMembersList ),
   BtVideoProcAmp::SharpnessMembersList
};

//-------------------------------------------------------------------------
/*
WhiteBalance:
	Specifies the white balance setting expressed as a color temperature in
	degrees Kelvin. Every video capture driver must define a range and 
	default value for this property. The value type must be VT_I4. 
 */
KSPROPERTY_STEPPING_LONG BtVideoProcAmp::WhiteBalanceRangeAndStep [] =
{
	DEF_RANGEANDSTEP( -100 , 100 , 200)
};
const ULONG BtVideoProcAmp::WhiteBalanceDefault = 0;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::WhiteBalanceMembersList [] =
{
   {
      {
          KSPROPERTY_MEMBER_RANGES,
          sizeof (BtVideoProcAmp::WhiteBalanceRangeAndStep),
          SIZEOF_ARRAY (BtVideoProcAmp::WhiteBalanceRangeAndStep),
          0
      },
      (PVOID) BtVideoProcAmp::WhiteBalanceRangeAndStep
   },
   {
     {
         KSPROPERTY_MEMBER_VALUES,
         sizeof( BtVideoProcAmp::WhiteBalanceDefault ),
         sizeof( BtVideoProcAmp::WhiteBalanceDefault ),
         KSPROPERTY_MEMBER_FLAG_DEFAULT
     },
     (PVOID) &BtVideoProcAmp::WhiteBalanceDefault
   }
};

KSPROPERTY_VALUES BtVideoProcAmp::WhiteBalanceValues =
{
   {
      {
         STATICGUIDOF( KSPROPTYPESETID_General ),
         VT_I4,
         0
      }
   },
   SIZEOF_ARRAY( BtVideoProcAmp::WhiteBalanceMembersList ),
   BtVideoProcAmp::WhiteBalanceMembersList
};

//-------------------------------------------------------------------------
/* 
Backlight Compensation
	A value of zero indicates that the backlight compensation is disabled.
	A value of 1 indicates that the backlight compensation is enabled.
	The range for this value is 0 to 1. The default value for this property
	is 1. The value type must be VT_I4.
	Notes: The actual values used in the property sets are multiplied by 
	100 to give improved granularity. (What does this mean? )
 */
KSPROPERTY_STEPPING_LONG BtVideoProcAmp::BacklightCompensationRangeAndStep [] =
{
	DEF_RANGEANDSTEP( 0 , 1 , 1 )
};
const ULONG BtVideoProcAmp::BacklightCompensationDefault = 1;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::BacklightCompensationMembersList [] =
{
   {
      {
          KSPROPERTY_MEMBER_RANGES,
          sizeof (BtVideoProcAmp::BacklightCompensationRangeAndStep),
          SIZEOF_ARRAY (BtVideoProcAmp::BacklightCompensationRangeAndStep),
          0
      },
      (PVOID) BtVideoProcAmp::BacklightCompensationRangeAndStep
   },
   {
     {
         KSPROPERTY_MEMBER_VALUES,
         sizeof( BtVideoProcAmp::BacklightCompensationDefault ),
         sizeof( BtVideoProcAmp::BacklightCompensationDefault ),
         KSPROPERTY_MEMBER_FLAG_DEFAULT
     },
     (PVOID) &BtVideoProcAmp::BacklightCompensationDefault
   }
};

KSPROPERTY_VALUES BtVideoProcAmp::BacklightCompensationValues =
{
   {
      {
         STATICGUIDOF( KSPROPTYPESETID_General ),
         VT_I4,
         0
      }
   },
   SIZEOF_ARRAY( BtVideoProcAmp::BacklightCompensationMembersList ),
   BtVideoProcAmp::BacklightCompensationMembersList
};

//-------------------------------------------------------------------------
/*
ColorEnable:
	Specifies the color enable setting which is a Boolean value. Value 
	ranges from zero to 1, and the default is 1. A value of zero indicates
	that color is disabled. A value of 1 indicates that color is enabled. 
	The value type must be VT_I4. 
*/
KSPROPERTY_STEPPING_LONG BtVideoProcAmp::ColorEnableRangeAndStep [] = 
{
	DEF_RANGEANDSTEP( 0 , 1 , 1)
};
const ULONG BtVideoProcAmp::ColorEnableDefault = 1;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::ColorEnableMembersList [] = 
{
    {
        {
            KSPROPERTY_MEMBER_RANGES,
            sizeof (BtVideoProcAmp::ColorEnableRangeAndStep),
            SIZEOF_ARRAY (BtVideoProcAmp::ColorEnableRangeAndStep),
            0
        },
        (PVOID) BtVideoProcAmp::ColorEnableRangeAndStep
     },
     {
        {
            KSPROPERTY_MEMBER_VALUES,
            sizeof (BtVideoProcAmp::ColorEnableDefault),
            sizeof (BtVideoProcAmp::ColorEnableDefault), 
            KSPROPERTY_MEMBER_FLAG_DEFAULT
        },
        (PVOID) &BtVideoProcAmp::ColorEnableDefault,
    }    
};

KSPROPERTY_VALUES BtVideoProcAmp::ColorEnableValues =
{
    {
        STATICGUIDOF (KSPROPTYPESETID_General),
        VT_I4,
        0
    },
    SIZEOF_ARRAY (BtVideoProcAmp::ColorEnableMembersList),
    BtVideoProcAmp::ColorEnableMembersList
};

// ------------------------------------------------------------------------
/* 
Brightness:
	Brightness is also known as black level. Modifying the brightness
	setting shifts the entire video signal equally for all luminance
	values. The value is expressed in IRE units * 100 for NTSC. For
	non-NTSC sources, the units are arbitrary, with zero representing 
	blanking and 10000 representing pure white. The range for Value is
	-10000 to 10000; the default value is 750 (7.5 IRE). The value type 
	must be VT_I4.

  Notes :
	The active video portion of a video signal goes from 7.5 IRE to
	100 IRE. This maps to 0 to 255 (luma digitized value). This gives
	the bt848 a range adjustment of 45.888671 IRE units up or 46.25 down 
	always (we can warranty this range)

*/
KSPROPERTY_STEPPING_LONG BtVideoProcAmp::BrightnessRangeAndStep [] =
{
	DEF_RANGEANDSTEP(-4625,4589,256)
};
const ULONG BtVideoProcAmp::BrightnessDefault = 750;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::BrightnessMembersList [] =
{
   {
      {
          KSPROPERTY_MEMBER_RANGES,
          sizeof (BtVideoProcAmp::BrightnessRangeAndStep),
          SIZEOF_ARRAY (BtVideoProcAmp::BrightnessRangeAndStep),
          0
      },
      (PVOID) BtVideoProcAmp::BrightnessRangeAndStep
   },
   {
     {
         KSPROPERTY_MEMBER_VALUES,
         sizeof( BtVideoProcAmp::BrightnessDefault ),
         sizeof( BtVideoProcAmp::BrightnessDefault ),
         KSPROPERTY_MEMBER_FLAG_DEFAULT
     },
     (PVOID) &BtVideoProcAmp::BrightnessDefault
   }
};

KSPROPERTY_VALUES BtVideoProcAmp::BrightnessValues =
{
   {
      {
         STATICGUIDOF( KSPROPTYPESETID_General ),
         VT_I4,
         0
      }
   },
   SIZEOF_ARRAY( BtVideoProcAmp::BrightnessMembersList ),
   BtVideoProcAmp::BrightnessMembersList
};

// ------------------------------------------------------------------------
/*
Contrast:
	The contrast or luma gain setting is expressed as a gain factor
	multiplied by 100. The Value range is from zero to 10000, and the 
	default is 100 (1x). The value type must be VT_I4.
  Notes: The analog video gain setting can range from 0 to 236.57% on
	the Bt chip. But, for digital video, can range from 0 to 399.22%
	We limit range to the analog video case. (Seems to be more than
	enough for me)
 */
KSPROPERTY_STEPPING_LONG BtVideoProcAmp::ContrastRangeAndStep [] =
{
	DEF_RANGEANDSTEP( 0 , 237 , 237 )
};
const ULONG BtVideoProcAmp::ContrastDefault = 100;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::ContrastMembersList [] =
{
   {
      {
         KSPROPERTY_MEMBER_RANGES,
         sizeof( BtVideoProcAmp::ContrastRangeAndStep ),
         SIZEOF_ARRAY( BtVideoProcAmp::ContrastRangeAndStep ),
         0
      },
      (PVOID) BtVideoProcAmp::ContrastRangeAndStep
   },
   {
     {
         KSPROPERTY_MEMBER_VALUES,
         sizeof( BtVideoProcAmp::ContrastDefault ),
         sizeof( BtVideoProcAmp::ContrastDefault ),
         KSPROPERTY_MEMBER_FLAG_DEFAULT
     },
     (PVOID) &BtVideoProcAmp::ContrastDefault
   }
};

KSPROPERTY_VALUES BtVideoProcAmp::ContrastValues =
{
   {
      {
         STATICGUIDOF( KSPROPTYPESETID_General ),
         VT_I4,
         0
      }
   },
   SIZEOF_ARRAY( BtVideoProcAmp::ContrastMembersList ),
   BtVideoProcAmp::ContrastMembersList
};

// ------------------------------------------------------------------------
/*
Hue:
	The value of the hue setting is expressed in degrees multiplied by 100. 
	Value range is from -18000 to 18000 ( -180 to +180 degrees), and the
	default is zero. The value type must be VT_I4. 
*/
KSPROPERTY_STEPPING_LONG BtVideoProcAmp::HueRangeAndStep [] =
{
	DEF_RANGEANDSTEP( -9000, 8930 , 256 )
};
const ULONG BtVideoProcAmp::HueDefault = 0;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::HueMembersList [] =
{
   {
      {
         KSPROPERTY_MEMBER_RANGES,
         sizeof( BtVideoProcAmp::HueRangeAndStep ),
         SIZEOF_ARRAY( BtVideoProcAmp::HueRangeAndStep ),
         0
      },
      (PVOID) BtVideoProcAmp::HueRangeAndStep
   },
   {
     {
         KSPROPERTY_MEMBER_VALUES,
         sizeof( BtVideoProcAmp::HueDefault ),
         sizeof( BtVideoProcAmp::HueDefault ),
         KSPROPERTY_MEMBER_FLAG_DEFAULT
     },
     (PVOID) &BtVideoProcAmp::HueDefault
   }
};

KSPROPERTY_VALUES BtVideoProcAmp::HueValues =
{
   {
      {
         STATICGUIDOF( KSPROPTYPESETID_General ),
         VT_I4,
         0
      }
   },
   SIZEOF_ARRAY( BtVideoProcAmp::HueMembersList ),
   BtVideoProcAmp::HueMembersList
};

// ------------------------------------------------------------------------
/*
Saturation:
	The saturation or chroma gain setting is expressed as gain multiplied 
	by 100. Value ranges from zero to 10000, and the default is 100 (1x).
	The value type must be VT_I4. 
  Notes:
	We have to adjust U and V gains. The worst case is with NTSC_U that uses
	0xFE to signal Gain = 1x. Then, maximum allowable gain without distorsion
	is 201.18%. Digital video can reach 399%, but we limit it to the minimum
	we can get under any condition. (200% seems to be a good number!)
*/

KSPROPERTY_STEPPING_LONG BtVideoProcAmp::SaturationRangeAndStep [] =
{
	DEF_RANGEANDSTEP( 0 , 202 , 202 )
};
const ULONG BtVideoProcAmp::SaturationDefault = 100;

KSPROPERTY_MEMBERSLIST BtVideoProcAmp::SaturationMembersList [] =
{
   {
      {
         KSPROPERTY_MEMBER_RANGES,
         sizeof( BtVideoProcAmp::SaturationRangeAndStep ),
         SIZEOF_ARRAY( BtVideoProcAmp::SaturationRangeAndStep ),
         0
      },
      (PVOID) BtVideoProcAmp::SaturationRangeAndStep
   },
   {
     {
         KSPROPERTY_MEMBER_VALUES,
         sizeof( BtVideoProcAmp::SaturationDefault ),
         sizeof( BtVideoProcAmp::SaturationDefault ),
         KSPROPERTY_MEMBER_FLAG_DEFAULT
     },
     (PVOID) &BtVideoProcAmp::SaturationDefault
   }
};

KSPROPERTY_VALUES BtVideoProcAmp::SaturationValues =
{
   {
      {
         STATICGUIDOF( KSPROPTYPESETID_General ),
         VT_I4,
         0
      }
   },
   SIZEOF_ARRAY( BtVideoProcAmp::SaturationMembersList ),
   BtVideoProcAmp::SaturationMembersList
};


// ------------------------------------------------------------------------
const KSPROPERTY_ITEM BtVideoProcAmp::Properties[] = 
{
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_CONTRAST,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::ContrastValues,			// Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof(ULONG)								// SerializedSize
   ),
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_GAMMA,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::GammaValues,				// Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof(ULONG)								// SerializedSize
   ),
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_BACKLIGHT_COMPENSATION,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::BacklightCompensationValues,  // Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof(ULONG)								// SerializedSize
   ),
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_COLORENABLE,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::ColorEnableValues,		// Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof(ULONG)								// SerializedSize
   ),

   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_WHITEBALANCE,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::WhiteBalanceValues,		// Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof(ULONG)								// SerializedSize
   ),
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_SHARPNESS,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::SharpnessValues,			// Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof(ULONG)								// SerializedSize
   ),
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_BRIGHTNESS,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::BrightnessValues,		// Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof(ULONG)								// SerializedSize
   ),
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_HUE,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::HueValues,				// Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof( ULONG )							// SerializedSize
   ),
   DEFINE_KSPROPERTY_ITEM
   (
      KSPROPERTY_VIDEOPROCAMP_SATURATION,
      TRUE,										// GetSupported or Handler
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinProperty
      sizeof(KSPROPERTY_VIDEOPROCAMP_S),		// MinData
      TRUE,										// SetSupported or Handler
      &BtVideoProcAmp::SaturationValues,		// Values
      0,										// RelationsCount
      NULL,										// Relations
      NULL,										// SupportHandler
      sizeof( ULONG )							// SerializedSize
   )
};
