/*
	BtVideoStream.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Debug.hpp"
#include "BtVideoStream.hpp"
#include "BtDigitizer.hpp"

// The mechanism for describing a bitmap format is with the KS_BITMAPINFOHEADER
// This is really messy to deal with because it invariably has fields that
// follow it holding bit fields, palettes and the rest. This function gives
// the number of bytes required to hold a VIDEOINFO that represents it. This
// count includes the prefix information (like the rcSource rectangle) the
// BITMAPINFOHEADER field, and any other colour information on the end.
//
DWORD BtVideoStream::GetBitmapFormatSizeWithoutPalette(const KS_BITMAPINFOHEADER& pHeader)
{
    // Everyone has this to start with this  
    DWORD Size = pHeader.biSize;

    // Truecolour formats may have a BI_BITFIELDS specifier for compression
    // type which means that room for three DWORDs should be allocated that
    // specify where in each pixel the RGB colour components may be found

    if (pHeader.biCompression == KS_BI_BITFIELDS) {
        Size += KS_SIZE_MASKS;
    }

    return Size;
}

DWORD BtVideoStream::GetBitmapFormatSize(const KS_BITMAPINFOHEADER& pHeader)
{
    // Everyone has this to start with this  
    DWORD Size = GetBitmapFormatSizeWithoutPalette(pHeader);

    // Does this format use a palette, if the number of colours actually used
    // is zero then it is set to the maximum that are allowed for that colour
    // depth (an example is 256 for eight bits). Truecolour formats may also
    // pass a palette with them in which case the used count is non zero

    if (pHeader.biBitCount <= 8 || pHeader.biClrUsed) {
        DWORD Entries = (DWORD) 1 << pHeader.biBitCount;
        if (pHeader.biClrUsed) {
            Entries = pHeader.biClrUsed;
        }
        Size += Entries * sizeof(KS_RGBQUAD);
    }

    return Size;
}

BtVideoStream::BtVideoStream(BtDigitizer *dev,HW_STREAM_OBJECT& so) :
	BtDigitizerStream(dev) ,
	bIsVideoInfo2(FALSE) ,
	bFieldBased(FALSE) ,
	bFieldTimeBetweenSamples(FALSE) ,
	TimeToNextFrame(0) ,
#ifdef ENABLE_TIMESTAMP_FIX
	LastSamplingTime(0) ,
#endif
	VStreamObject(so)
{
	TRACE(("BtVideoStream::BtVideoStream"));

	// Clean all structs
	memset(&VidHeader,0,sizeof(VidHeader));
	memset(&VidHeader2,0,sizeof(VidHeader2));

	// No Cropping area by default
	rcCropping.Set(0,0,0,0);

	// Reset Counters
	BtVideoStream::ResetCounters();

}

BtVideoStream::~BtVideoStream()
{
	TRACE(("BtVideoStream::~BtVideoStream"));
}

// Reset Statistic counters
void BtVideoStream::ResetCounters()
{
	TRACE(("BtVideoStream::ResetCounters"));
	FrameInfo.PictureNumber = 0;
	FrameInfo.DropCount = 0;
	bDataDiscontinuity = FALSE; // Reset Flag
	TimeToNextFrame = 0;		// Time to get next frame
#ifdef ENABLE_TIMESTAMP_FIX
	LastSamplingTime = 0;		// Last sampling time
#endif
}

void BtVideoStream::IncPictureNumber()
{
	TRACE(("BtVideoStream::IncPictureNumber"));
	FrameInfo.PictureNumber++;
}

void BtVideoStream::IncDropCount()
{
	TRACE(("BtVideoStream::IncDropCount"));
	FrameInfo.DropCount++;
	bDataDiscontinuity = TRUE; 
}

/*
** BtVideoStream::GetDroppedFramesProperty
**
**    Gets dropped frame information
**
** Arguments:
**
**    pSrb - pointer to the stream request block for properties
**
** Returns:
**
** Side Effects:  none
*/

NTSTATUS BtVideoStream::GetDroppedFramesProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVideoStream::GetDroppedFramesProperty"));

	PSTREAM_PROPERTY_DESCRIPTOR pSPD = pSrb->CommandData.PropertyInfo;
	ULONG Id = pSPD->Property->Id;              // index of the property

	switch (Id) {

		case KSPROPERTY_DROPPEDFRAMES_CURRENT: {

				DUMP(("KSPROPERTY_DROPPEDFRAMES_CURRENT\n"));
				PKSPROPERTY_DROPPEDFRAMES_CURRENT_S pDroppedFrames = (PKSPROPERTY_DROPPEDFRAMES_CURRENT_S) pSPD->PropertyInfo;
	
				// Estimate divisor...
				ULONG FieldDivider = ((bFieldTimeBetweenSamples) ? 1 : 0);

				pDroppedFrames->PictureNumber	 = FrameInfo.PictureNumber >> FieldDivider;
				pDroppedFrames->DropCount		 = FrameInfo.DropCount >> FieldDivider;

				// Get the size of the image in bytes
				if (bIsVideoInfo2) {
					pDroppedFrames->AverageFrameSize = VidHeader2.bmiHeader.biSizeImage;
				} else {
					pDroppedFrames->AverageFrameSize = VidHeader.bmiHeader.biSizeImage;
				}
	
				DUMP(("KSPROPERTY_DROPPEDFRAMES_CURRENT:\n"));
				DUMP(("   PicNum(%d)\n", (ULONG)pDroppedFrames->PictureNumber));
				DUMP(("   DropCnt(%d)\n", (ULONG)pDroppedFrames->DropCount));
				DUMP(("   FrameSize(%d)\n", pDroppedFrames->AverageFrameSize));
	
				pSrb->ActualBytesTransferred = sizeof (KSPROPERTY_DROPPEDFRAMES_CURRENT_S);
				return STATUS_SUCCESS;
		   }
		default:

			ERROR(("KSPROPERTY_DROPPEDFRAMES_???(%d)\n", Id));

	}
	return STATUS_INVALID_PARAMETER;
}

/* Function: BtVideoStream::NormalizeFrame
 * Purpose: Makes the infoheader structure contain an Stride Always (As used in DDraw stuff)
 *   , all RECTs filled explicitly, and positive Widths and heights
 * Input: VidHdr: KS_VIDEOINFOHEADER &
 * Output: Stride in bytes, ColorSpace to use
 */
DWORD BtVideoStream::NormalizeFrame( KS_VIDEOINFOHEADER_WITHMASK &VidHdr , ColFmt &col)
{
	TRACE(("BtVideoStream::NormalizeFrame"));

	// ENSURE width and Height are NOT negative (Overlay mixer seems to do this!)
	VidHdr.bmiHeader.biHeight = abs( VidHdr.bmiHeader.biHeight );
	VidHdr.bmiHeader.biWidth = abs( VidHdr.bmiHeader.biWidth );

	// If stride not provided ...
	if (RectEmpty(&VidHdr.rcTarget)) { // dst empty, width is the real width

		// To be able to use Stride != width, force destination rect!
		VidHdr.rcTarget.top = 0;
		VidHdr.rcTarget.bottom = VidHdr.bmiHeader.biHeight;
		VidHdr.rcTarget.left = 0;
		VidHdr.rcTarget.right = VidHdr.bmiHeader.biWidth;

	}

	// Get maximum dimensions for Analog and Digital Windows
	RECT Max;
	digitizer->GetBtCard().VideoGetMaxFrame(Max);

	// If source not provided ... Fill it!
	if (RectEmpty(&VidHdr.rcSource)) { // source empty, use full frame
		VidHdr.rcSource = Max;
	}

	// Obvious Checks: rcSource can't be bigger than Max frame for videonorm.
	MRect Source(VidHdr.rcSource); 
	MSize sSource = Source.Size();
	if ( sSource.cx > Max.right) {
		VidHdr.rcSource.right = VidHdr.rcSource.left + Max.right;
	}
	if ( sSource.cy > Max.bottom) {
		VidHdr.rcSource.bottom = VidHdr.rcSource.top + Max.bottom;
	}

	// Target can't be bigger than Max frame for videonorm.
	MRect Target(VidHdr.rcTarget);
	MSize sTarget = Target.Size();
	if ( sTarget.cx > Max.right) {
		VidHdr.rcTarget.right = VidHdr.rcTarget.left + Max.right;
	}
	if ( sTarget.cy > Max.bottom) {
		VidHdr.rcTarget.bottom = VidHdr.rcTarget.top + Max.bottom;
	}

	// And Target CANT be bigger than biWidth,biHeight (to understand this,
	// bi.. refers to all the bitmap, while target gives the region filled by
	// the image.
	Target = VidHdr.rcTarget;
	sTarget = Target.Size();
	if ( sTarget.cx > VidHdr.bmiHeader.biWidth) {
		VidHdr.rcTarget.right = VidHdr.rcTarget.left + VidHdr.bmiHeader.biWidth;
	}
	if ( sTarget.cy > VidHdr.bmiHeader.biHeight) {
		VidHdr.rcTarget.bottom = VidHdr.rcTarget.top + VidHdr.bmiHeader.biHeight;
	}

	// Last, but VERY IMPORTANT: We can't expand, so Target MUST be equal or smaller
	// than source.
	Target = VidHdr.rcTarget;
	sTarget = Target.Size();
	Source = VidHdr.rcSource;
	sSource = Source.Size();
	if (sTarget.cx > sSource.cx )
		VidHdr.rcTarget.bottom = VidHdr.rcTarget.top + sSource.cx;
	if (sTarget.cy > sSource.cy )
		VidHdr.rcTarget.right = VidHdr.rcTarget.left + sSource.cy;

	// Now, calculate Stride
	DWORD biCompression = VidHdr.bmiHeader.biCompression;
	WORD biBitCount = VidHdr.bmiHeader.biBitCount;   
	LONG biWidthBytes = VidHdr.bmiHeader.biWidth; // Always stride in pixels!

	// Calculate real Stride in bytes
	ColorSpace tmp( 
		biCompression, 
		biBitCount, 
		(biCompression == KS_BI_BITFIELDS) ? VidHdr.dwBitMasks[0] : 0, // Dont access the fields if they are not provided!
		(biCompression == KS_BI_BITFIELDS) ? VidHdr.dwBitMasks[1] : 0, // Dont access the fields if they are not provided!
		(biCompression == KS_BI_BITFIELDS) ? VidHdr.dwBitMasks[2] : 0 // Dont access the fields if they are not provided!
		);

	biWidthBytes = tmp.GetPitchBpp() * biWidthBytes / 8;

	// If size of Bitmap not provided, calculate it!
	if (!VidHdr.bmiHeader.biSizeImage) {
		VidHdr.bmiHeader.biSizeImage = KS_DIBSIZE(VidHdr.bmiHeader);
	}

	// Fill in the produced bitrate...
	VidHdr.dwBitRate = (DWORD)(
		(VidHdr.bmiHeader.biSizeImage * 80000000i64) / 
		((VidHdr.AvgTimePerFrame) ? VidHdr.AvgTimePerFrame : 10000000i64));
	VidHdr.dwBitErrorRate = 0;

	// can it be non-aligned to DWORD ?- Don't think so!
	biWidthBytes += 3;
	biWidthBytes &= ~3;

	// Return Colorformat
	col=tmp.GetColorFormat();

	// Return Stride in bytes
	return biWidthBytes;
}

/* Function: BtVideoStream::NormalizeFrame
 * Purpose: Makes the infoheader structure contain an Stride Always (As used in DDraw stuff)
 * Purpose: Makes the infoheader structure contain an Stride Always (As used in DDraw stuff)
 *   , all RECTs filled explicitly, and positive Widths and heights
 * Input: VidHdr: KS_VIDEOINFOHEADER2 &
 * Output: Stride in bytes, ColorSpace to use
 */
DWORD BtVideoStream::NormalizeFrame( KS_VIDEOINFOHEADER2_WITHMASK &VidHdr , ColFmt &col )
{
	TRACE(("BtVideoStream::NormalizeFrame(2)"));

	// ENSURE width and Height are NOT negative (Overlay mixer seems to do this!)
	VidHdr.bmiHeader.biHeight = abs( VidHdr.bmiHeader.biHeight );
	VidHdr.bmiHeader.biWidth = abs( VidHdr.bmiHeader.biWidth );

	// If stride not provided ...
	if (RectEmpty(&VidHdr.rcTarget)) { // dst empty, width is the real width

		// To be able to use Stride != width, force destination rect!
		VidHdr.rcTarget.top = 0;
		VidHdr.rcTarget.bottom = VidHdr.bmiHeader.biHeight;
		VidHdr.rcTarget.left = 0;
		VidHdr.rcTarget.right = VidHdr.bmiHeader.biWidth;

	}

	// Get maximum dimensions for Analog and Digital Windows
	RECT Max;
	digitizer->GetBtCard().VideoGetMaxFrame(Max);

	// Based on the frame/field request, go!
	if (
		(VidHdr.dwInterlaceFlags & KS_INTERLACE_IsInterlaced) &&
		(VidHdr.dwInterlaceFlags & KS_INTERLACE_1FieldPerSample)
		) {
		// Field output is requested... Adjust...
		Max.bottom >>= 1;
	}

	// If source not provided ... Fill it!
	if (RectEmpty(&VidHdr.rcSource)) { // source empty, use full frame
		VidHdr.rcSource = Max;
	}

	// Obvious Checks: rcSource can't be bigger than Max frame for videonorm.
	MRect Source(VidHdr.rcSource);
	MSize sSource = Source.Size();
	if ( sSource.cx > Max.right) {
		VidHdr.rcSource.right = VidHdr.rcSource.left + Max.right;
	}
	if ( sSource.cy > Max.bottom) {
		VidHdr.rcSource.bottom = VidHdr.rcSource.top + Max.bottom;
	}

	// Target can't be bigger than Max frame for videonorm.
	MRect Target(VidHdr.rcTarget);
	MSize sTarget = Target.Size();
	if ( sTarget.cx > Max.right) {
		VidHdr.rcTarget.right = VidHdr.rcTarget.left + Max.right;
	}
	if ( sTarget.cy > Max.bottom) {
		VidHdr.rcTarget.bottom = VidHdr.rcTarget.top + Max.bottom;
	}

	// And Target CANT be bigger than biWitdth,biHeight (to understand this,
	// bi.. refers to all the bitmap, while target gives the region filled by
	// the image.
	Target = VidHdr.rcTarget;
	sTarget = Target.Size();
	if ( sTarget.cx > VidHdr.bmiHeader.biWidth) {
		VidHdr.rcTarget.right = VidHdr.rcTarget.left + VidHdr.bmiHeader.biWidth;
	}
	if ( sTarget.cy > VidHdr.bmiHeader.biHeight) {
		VidHdr.rcTarget.bottom = VidHdr.rcTarget.top + VidHdr.bmiHeader.biHeight;
	}

	// Last, but VERY IMPORTANT: We can't expand, so Target MUST be equal or smaller
	// than source.
	Target = VidHdr.rcTarget;
	sTarget = Target.Size();
	Source = VidHdr.rcSource;
	sSource = Source.Size();
	if (sTarget.cx > sSource.cx )
		VidHdr.rcTarget.bottom = VidHdr.rcTarget.top + sSource.cx;
	if (sTarget.cy > sSource.cy )
		VidHdr.rcTarget.right = VidHdr.rcTarget.left + sSource.cy;

	// Now, calculate Stride
	DWORD biCompression = VidHdr.bmiHeader.biCompression;
	WORD biBitCount = VidHdr.bmiHeader.biBitCount;   
	LONG biWidthBytes = VidHdr.bmiHeader.biWidth; // Always stride!

	// Calculate real Stride in bytes
	ColorSpace tmp( 
		biCompression, 
		biBitCount, 
		(biCompression == KS_BI_BITFIELDS) ? VidHdr.dwBitMasks[0] : 0, // Dont access the fields if they are not provided!
		(biCompression == KS_BI_BITFIELDS) ? VidHdr.dwBitMasks[1] : 0, // Dont access the fields if they are not provided!
		(biCompression == KS_BI_BITFIELDS) ? VidHdr.dwBitMasks[2] : 0 // Dont access the fields if they are not provided!
	);

	biWidthBytes = tmp.GetPitchBpp() * biWidthBytes / 8;

	// If size of Bitmap not provided, calculate it!
	if (!VidHdr.bmiHeader.biSizeImage) {
		VidHdr.bmiHeader.biSizeImage = KS_DIBSIZE(VidHdr.bmiHeader);
	}

	// Fill in the produced bitrate...
	VidHdr.dwBitRate = (DWORD)(
		(VidHdr.bmiHeader.biSizeImage * 80000000i64) / 
		((VidHdr.AvgTimePerFrame) ? VidHdr.AvgTimePerFrame : 10000000i64));
	VidHdr.dwBitErrorRate = 0;

	// can it be non-aligned to DWORD ?- Don't think so!
	biWidthBytes += 3;
	biWidthBytes &= ~3;

	// Return Colorformat
	col=tmp.GetColorFormat();

	// Return Stride in bytes
	return biWidthBytes;
}

/*
 * Method: BtVideoStream::StreamGetNextBuffer
 * Purpose: Get the next databuffer to use (called back from the videochannel
 *		    class.
 * Notes: Xface to VideoChannel!
 */

/* Note: This can be called async respect the others entry points of this stream class
		 It wont be called when channel is stopping or stopped.
  */

/* Note: this is called at IRQL = DISPATCH_LEVEL */
const VideoDataBuf BtVideoStream::StreamGetNextBuffer()
{
	TRACE(("BtVideoStream::StreamGetNextBuffer"));

    // Only capture in the RUN state
    if (KSState == KSSTATE_RUN /* atomic READ, does not hurt */
		/* In the Stop State, its supposed that we wont be called */
		) {

		// We are running ... We must calculate if it is time to sample next frame or not.
		
		// Get requested time between samples
		REFERENCE_TIME TimeBetweenSamples;
		if (bIsVideoInfo2) {
			TimeBetweenSamples = VidHeader2.AvgTimePerFrame;
		} else {
			TimeBetweenSamples = VidHeader.AvgTimePerFrame;
		}

		// But the Time is time between frames... We must convert it to Time Between 
		// fields if we are working with fields... 
		TimeBetweenSamples >>= ((bFieldTimeBetweenSamples) ? 1 : 0);

		// Get Time between Hardware taken samples ...
		REFERENCE_TIME TimeBetweenHwSampling = 
			(digitizer->GetBtCard().TimeBetweenFields() << ((bFieldTimeBetweenSamples) ? 0 : 1) );

		// If Time between samples requested is less than possible (faster FPS than available), 
		// set it to the maximum possible!
		if (TimeBetweenSamples < TimeBetweenHwSampling) {
			ERROR(("Requested more FPS than available"));
			TimeBetweenSamples = TimeBetweenHwSampling;
		}

		// Calculate time to next Hardware sampling
		TimeToNextFrame += TimeBetweenHwSampling;

		// If Time to next Hardware Sampling later than needed, sample now!
		if (TimeToNextFrame > TimeBetweenSamples) {

			// Leave only the remaining time to the next sample!
			TimeToNextFrame -= TimeBetweenSamples;

			// And sample now!

			// maintain statistics
			IncPictureNumber();

			// Try to get a new Data Buffer
			PHW_STREAM_REQUEST_BLOCK pSrb = NULL;
			BufferLock.Acquire();
			if (!BufferQueue.Remove(pSrb)) {
				pSrb = NULL;
			}
			BufferLock.Release();

			if (pSrb) {

				// Create a new VideoBuffer (we are its owner, so will signal us on it's completion)
				VideoDataBuf pBuf(this,pSrb,dwStride);

				// Call our Sampling callback without getting a Buffer!
				OnSampling(false);

				// Return VideoBuffer
				return pBuf;

			} else {

				// If buffer unavailable, mark it as dropped and remember this data discontinuity
				IncDropCount();
			}

		}

		// If we reach here, the Capture Channel has a free slot (captured frame) unneeded by the capture
		// Stream, Make good use of it, and try to use it for preview.

		// Call our Sampling callback trying to get a Buffer!
		return OnSampling(true);

	}	

	// Return a NULL buffer (either error, or NOT time to sample!)
	return VideoDataBuf();
}

/* 
 * Method: BtVideoStream::StreamBufferDone
 * Purpose: Called by the videoChannel Class when the adquisition of a Videoframe
 *			has finished. Notifies the Stream Class this frame is done.
 * Input:  pSrb, Vidfield
 * Output: None
 * Notes:  
 */

/* Note: This can be called async respect the other entry points of this stream class
 */

/* Note: this is called at IRQL = DISPATCH_LEVEL */
void BtVideoStream::StreamBufferDone( const VideoDataBuf& pBuf , const VidField vf )
{
	TRACE(("BtVideoStream::StreamBufferDone"));

	// Get the pointer to the Srb
	PHW_STREAM_REQUEST_BLOCK pSrb = pBuf.GetSrb();
	
	// Get pointer to the current Packet.
	PKSSTREAM_HEADER pDataPacket = pSrb->CommandData.DataBufferArray;

	// Avoid returning data in the Stopped or cancelled State
	if (KSState == KSSTATE_STOP ||  /* atomic READ, does not hurt */
		pSrb->Status == STATUS_CANCELLED ) {

		pDataPacket->DataUsed = 0;

	} else {

		// Estimate divisor...
		ULONG FieldDivider = ((bFieldTimeBetweenSamples) ? 1 : 0);

		// Complete buffers if required.
		if( bIsVideoInfo2 ) {
			pDataPacket->DataUsed = VidHeader2.bmiHeader.biSizeImage;
			pDataPacket->Duration = VidHeader2.AvgTimePerFrame >> FieldDivider;
		} else {
			pDataPacket->DataUsed = VidHeader.bmiHeader.biSizeImage;
			pDataPacket->Duration = VidHeader.AvgTimePerFrame >> FieldDivider;
		}

		DUMP(("DataUsed = %d\n", pDataPacket->DataUsed));

		pDataPacket->PresentationTime.Numerator = 1;
		pDataPacket->PresentationTime.Denominator = 1;

		// Time-Stamp Sample!
		TimeStamp(pDataPacket,pSrb);

		// Copy statistics to output buffer
		PKS_FRAME_INFO pFrameInfo = (PKS_FRAME_INFO) ( pSrb->CommandData.DataBufferArray + 1 );

		// copy the information to the outbound buffer
		pFrameInfo->ExtendedHeaderSize = sizeof( KS_FRAME_INFO );
		pFrameInfo->PictureNumber =      FrameInfo.PictureNumber >> FieldDivider;
		pFrameInfo->DropCount =          FrameInfo.DropCount >> FieldDivider;

		DUMP(("Frame: %d, Dropped: %d\n", pFrameInfo->PictureNumber, pFrameInfo->DropCount));

		// If data discontinuity...
		if ( bDataDiscontinuity ) {
			bDataDiscontinuity = FALSE; // Reset Flag
			pSrb->CommandData.DataBufferArray->OptionsFlags |= KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY;
		} else {
			pSrb->CommandData.DataBufferArray->OptionsFlags &= ~KSSTREAM_HEADER_OPTIONSF_DATADISCONTINUITY;
		}

		// Every frame we generate is a key frame (aka SplicePoint)
		// Delta frames (B or P) should not set this flag
		pDataPacket->OptionsFlags |= KSSTREAM_HEADER_OPTIONSF_SPLICEPOINT;
		
		// This is a I frame!
		pFrameInfo->dwFrameFlags |= KS_VIDEO_FLAG_I_FRAME;

		// don't forget to report our field type !
		pFrameInfo->dwFrameFlags &= ~3;
		pFrameInfo->dwFrameFlags |= 
			(bFieldBased) 
				? ( (vf == VF_Odd) ? KS_VIDEO_FLAG_FIELD1 : KS_VIDEO_FLAG_FIELD2 ) 
				: KS_VIDEO_FLAG_FRAME ;
	}

	// Completing the request!
	pSrb->Status = STATUS_SUCCESS;

#ifdef ENABLE_DDRAW_STUFF

	// If we are dealing with a DirectDraw surface handle... , display that surface...
	if ( DDraw.ShowSurface( pBuf.GetSrb() ) ) {

		// We must post the Srb back to the Queue... Note that if stream is stopped, all surfaces will be freed
		// by the SetState(KSSTATE_STOP) ...

		// Add buffer to the Queue
		BufferLock.Acquire();
	
		// Insert the request in the queue
		bool Done = BufferQueue.Insert( pBuf.GetSrb() );
	
		// Buffer queued, release mutex
		BufferLock.Release();

		// If failed... Hope it does not, as the Surface will be lost...
		if (!Done) {

			ERROR(("DirectDraw Surface lost!\n"));

			// If failed, return buffer...
			pBuf.GetSrb()->CommandData.DataBufferArray->DataUsed = 0;
			CompleteDataPacket( pBuf.GetSrb() );
		}

	} else {
		// We were not able to show it... Assume it's not a DDraw surface...

#endif

		// Inform the class requests are complete
		CompleteDataPacket( pSrb );

#ifdef ENABLE_DDRAW_STUFF
	}
#endif

}

// Called when next sampling period
const VideoDataBuf BtVideoStream::OnSampling(bool GetBuffer)
{
	TRACE(("BtVideoStream::OnSampling"));
	// No buffer by default
	return VideoDataBuf();
}


/*
** BtVideoStream::Open()
**
**   This routine is called when an OpenStream SRB request is received
**
** Arguments:
**
**   pSrb - pointer to stream request block for the Open command
**
** Returns:
**
** Side Effects:  none
*/
NTSTATUS BtVideoStream::Open( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVideoStream::Open"));

	// If already open, stop and close channel first!
	NTSTATUS ret = Destroy();	/* (modifies videoChannel) */
	if (ret != STATUS_SUCCESS) {
		return ret;
	}

	//
	// Check the validity of the format being requested
	//
	if (!VerifyFormat (pSrb->CommandData.OpenFormat)) {

		ERROR(("Exit (failed)\n"));
		return STATUS_INVALID_PARAMETER;
	}

	// Get pointers to very used data!
	const KS_DATAFORMAT_VIDEOINFOHEADER &rKSDataFormat = *(PKS_DATAFORMAT_VIDEOINFOHEADER) pSrb->CommandData.OpenFormat;
	const KS_VIDEOINFOHEADER &rVideoInfoHdrRequested = rKSDataFormat.VideoInfoHeader;
	const KS_DATAFORMAT_VIDEOINFOHEADER2 &rKSDataFormat2 = *(PKS_DATAFORMAT_VIDEOINFOHEADER2) pSrb->CommandData.OpenFormat;
	const KS_VIDEOINFOHEADER2 &rVideoInfoHdrRequested2 = rKSDataFormat2.VideoInfoHeader2;

	// at this point have to see what type of channel is to be opened:
	// single field, alternating or interleaved
	// algorithm is like this:
	// 1. look at the video format Specifier guid. If it's a infoheader2, it
	//    will tell us type of stream to open.
	// 2. else look at the vertical size to decide single field vs. interleaved

	// VIDEOINFOHEADER
	if ( rKSDataFormat.DataFormat.Specifier == KSDATAFORMAT_SPECIFIER_VIDEOINFO ) {

		DUMP(("VIDEOINFOHEADER\n"));

		DUMP(("Source (%d,%d,%d,%d)  Dest (%d,%d,%d,%d)\n",
				rVideoInfoHdrRequested.rcSource.left,
				rVideoInfoHdrRequested.rcSource.top,
				rVideoInfoHdrRequested.rcSource.right,
				rVideoInfoHdrRequested.rcSource.bottom,
				rVideoInfoHdrRequested.rcTarget.left,
				rVideoInfoHdrRequested.rcTarget.top,
				rVideoInfoHdrRequested.rcTarget.right,
				rVideoInfoHdrRequested.rcTarget.bottom ));


		DUMP(("Width=%d  Height=%d  BitCount=%d  Format=%x  biSizeImage=%d\n",
               rVideoInfoHdrRequested.bmiHeader.biWidth,
               rVideoInfoHdrRequested.bmiHeader.biHeight,
               rVideoInfoHdrRequested.bmiHeader.biBitCount,
               rVideoInfoHdrRequested.bmiHeader.biCompression,
			   rVideoInfoHdrRequested.bmiHeader.biSizeImage
            ));


		// Remember the format of this Stream... Only access provided fields!
		RtlCopyMemory(
			&VidHeader,
			&rVideoInfoHdrRequested,
			GetVideoInfoFormatSizeWithoutPalette(rVideoInfoHdrRequested) );
		bIsVideoInfo2 = FALSE;

		// Force Cropping rect if we were asked to...
		if (!rcCropping.IsNull()) {

			// Force Cropping region!
			VidHeader.rcSource = rcCropping;
		}

	} else 
	// VIDEOINFOHEADER2
	if ( rKSDataFormat2.DataFormat.Specifier == KSDATAFORMAT_SPECIFIER_VIDEOINFO2 ) {

		DUMP(("VIDEOINFOHEADER2\n"));

		DUMP(("Source (%d,%d,%d,%d)  Dest (%d,%d,%d,%d)\n",
				rVideoInfoHdrRequested2.rcSource.left,
				rVideoInfoHdrRequested2.rcSource.top,
				rVideoInfoHdrRequested2.rcSource.right,
				rVideoInfoHdrRequested2.rcSource.bottom,
				rVideoInfoHdrRequested2.rcTarget.left,
				rVideoInfoHdrRequested2.rcTarget.top,
				rVideoInfoHdrRequested2.rcTarget.right,
				rVideoInfoHdrRequested2.rcTarget.bottom ));

		DUMP(("Width=%d  Height=%d  BitCount=%d  Format=%x  biSizeImage=%d\n",
               rVideoInfoHdrRequested2.bmiHeader.biWidth,
               rVideoInfoHdrRequested2.bmiHeader.biHeight,
               rVideoInfoHdrRequested2.bmiHeader.biBitCount,
               rVideoInfoHdrRequested2.bmiHeader.biCompression,
			   rVideoInfoHdrRequested2.bmiHeader.biSizeImage
            ));

		DUMP(("dwInterlaceFlags=%d (0x%08x)\n",rVideoInfoHdrRequested2.dwInterlaceFlags,rVideoInfoHdrRequested2.dwInterlaceFlags));

		// Remember the format of this Stream... Only access provided fields!
		RtlCopyMemory(
			&VidHeader2,
			&rVideoInfoHdrRequested2,
			GetVideoInfoFormatSizeWithoutPalette(rVideoInfoHdrRequested2) );
		bIsVideoInfo2 = TRUE;

		// Force Cropping rect if we were asked to...
		if (!rcCropping.IsNull()) {
			// Force Cropping region!
			VidHeader2.rcSource = rcCropping;
		}

	} else {

		ERROR(("Invalid video format header\n"));
		return STATUS_INVALID_PARAMETER;

	}

	// Set to Stop State
	KSState = KSSTATE_STOP;

	// the structure of the driver is such that a single callback could be used
	// for all stream requests. But the code below could be used to supply
	// different entry points for different streams

	pSrb->StreamObject->ReceiveDataPacket			= VStreamObject.ReceiveDataPacket;
	pSrb->StreamObject->ReceiveControlPacket		= VStreamObject.ReceiveControlPacket;
	pSrb->StreamObject->Dma							= VStreamObject.Dma;
	pSrb->StreamObject->Allocator					= VStreamObject.Allocator;

	// The PIO flag must be set when the mini driver will be accessing the data
	// buffers passed in using logical addressing

	pSrb->StreamObject->Pio							= VStreamObject.Pio;
	pSrb->StreamObject->StreamHeaderMediaSpecific	= VStreamObject.StreamHeaderMediaSpecific;
	pSrb->StreamObject->StreamHeaderWorkspace		= VStreamObject.StreamHeaderWorkspace;
	pSrb->StreamObject->HwClockObject				= VStreamObject.HwClockObject;

	// Now Stream is open!
	bStreamOpened = TRUE;

	DUMP(("Created!\n"));

	return STATUS_SUCCESS;
}

// Set video format
NTSTATUS BtVideoStream::SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) 
{
	TRACE(("BtVideoStream::SetFormat"));

	if (!VerifyFormat (pSrb->CommandData.OpenFormat ) ) {

		ERROR(("SRB_SET_DATA_FORMAT FAILED\n"));
		return STATUS_NO_MATCH;

	} 

	// Get pointers to very used data!
	const KS_DATAFORMAT_VIDEOINFOHEADER &rKSDataFormat = *(PKS_DATAFORMAT_VIDEOINFOHEADER) pSrb->CommandData.OpenFormat;
	const KS_VIDEOINFOHEADER &rVideoInfoHdrRequested = rKSDataFormat.VideoInfoHeader;
	const KS_DATAFORMAT_VIDEOINFOHEADER2 &rKSDataFormat2 = *(PKS_DATAFORMAT_VIDEOINFOHEADER2) pSrb->CommandData.OpenFormat;
	const KS_VIDEOINFOHEADER2 &rVideoInfoHdrRequested2 = rKSDataFormat2.VideoInfoHeader2;

	// at this point have to see what type of channel is to be opened:
	// single field, alternating or interleaved
	// algorithm is like this:
	// 1. look at the video format Specifier guid. If it's a infoheader2, it
	//    will tell us type of stream to open.
	// 2. else look at the vertical size to decide single field vs. interleaved

	// VIDEOINFOHEADER
	if ( rKSDataFormat.DataFormat.Specifier == KSDATAFORMAT_SPECIFIER_VIDEOINFO ) {

		DUMP(("VIDEOINFOHEADER\n"));

		DUMP(("Source (%d,%d,%d,%d)  Dest (%d,%d,%d,%d)\n",
				rVideoInfoHdrRequested.rcSource.left,
				rVideoInfoHdrRequested.rcSource.top,
				rVideoInfoHdrRequested.rcSource.right,
				rVideoInfoHdrRequested.rcSource.bottom,
				rVideoInfoHdrRequested.rcTarget.left,
				rVideoInfoHdrRequested.rcTarget.top,
				rVideoInfoHdrRequested.rcTarget.right,
				rVideoInfoHdrRequested.rcTarget.bottom ));

		DUMP(("Width=%d  Height=%d  BitCount=%d  Format=%x  biSizeImage=%d\n",
               rVideoInfoHdrRequested.bmiHeader.biWidth,
               rVideoInfoHdrRequested.bmiHeader.biHeight,
               rVideoInfoHdrRequested.bmiHeader.biBitCount,
               rVideoInfoHdrRequested.bmiHeader.biCompression,
			   rVideoInfoHdrRequested.bmiHeader.biSizeImage
            ));

		// Remember the last Time between frames and the cropping region
		REFERENCE_TIME AvgTimePerFrame;
		if (bIsVideoInfo2) {
			AvgTimePerFrame = VidHeader2.AvgTimePerFrame;
		} else {
			AvgTimePerFrame = VidHeader.AvgTimePerFrame;
		}

		// If no videochannel, only remember new format... 
		if (!vc) {

			DUMP(("No VideoChannel created - Only remember new format\n"));

			// Remember the format of this Stream... Only access provided fields!
			RtlCopyMemory(
				&VidHeader,
				&rVideoInfoHdrRequested,
				GetVideoInfoFormatSizeWithoutPalette(rVideoInfoHdrRequested) );
			bIsVideoInfo2 = FALSE;

			// A renderer may be switching formats, and in this case, the AvgTimePerFrame
			// will be zero.  Don't overwrite a previously set framerate.
			if (!VidHeader.AvgTimePerFrame) {
				VidHeader.AvgTimePerFrame = AvgTimePerFrame;
			}

			// Force Cropping rect if we were asked to...
			if (!rcCropping.IsNull()) {
				// Force Cropping region!
				VidHeader.rcSource = rcCropping;
			}

		} else {

			// There are 2 problems here: If we want to switch between alternating 
			// and normal mode, we MUST destroy videochannel. But, if only changing size,
			// perhaps we can do it without destroying it. The only problem is that
			// the creation of a VideoChannel can fail, and SetFormat shouldn't destroy
			// the videoChannel. So, get first the original parameters of VideoChannel.
			// destroy VideoChannel, try to create with the new parameters, if it fails,
			// Recreate with the old Parameters!

			// Destroy VideoChannel! - Hope to get success !
			NTSTATUS ret = Destroy();
			if (ret != STATUS_SUCCESS) {
				return ret;
			}

			// Remember the previous format - Always all fields present!
			KS_VIDEOINFOHEADER_WITHMASK oldVidHeader = VidHeader;
			KS_VIDEOINFOHEADER2_WITHMASK oldVidHeader2 = VidHeader2;
			BOOL oldbIsVideoInfo2 = bIsVideoInfo2;

			// Set the new format!

			// Remember the format of this Stream... Only access provided fields!
			RtlCopyMemory(
				&VidHeader,
				&rVideoInfoHdrRequested,
				GetVideoInfoFormatSizeWithoutPalette(rVideoInfoHdrRequested) );
			bIsVideoInfo2 = FALSE;

			// A renderer may be switching formats, and in this case, the AvgTimePerFrame
			// will be zero.  Don't overwrite a previously set framerate.
			if (!VidHeader.AvgTimePerFrame) {
				VidHeader.AvgTimePerFrame = AvgTimePerFrame;
			}

			// Force Cropping rect if we were asked to...
			if (!rcCropping.IsNull()) {
				// Force Cropping region!
				VidHeader.rcSource = rcCropping;
			}

			ret = Create();
			if (ret != STATUS_SUCCESS) { // Failure ... Recretate channel with old parameters!

				// Restore original format - Always all fields present!
				VidHeader = oldVidHeader;
				VidHeader2 = oldVidHeader2;
				bIsVideoInfo2 = oldbIsVideoInfo2;

				// Recreate channel!
				ret = Create();
				if (ret != STATUS_SUCCESS) { // Failure ... Why ?
					return ret;
				}

				// If we were running, restart stream!
				if (KSState == KSSTATE_RUN) {
					vc->Start();
				}
				return STATUS_NO_MATCH;

			} else {

				// If we were running, restart stream!
				if (KSState == KSSTATE_RUN) {
					vc->Start();
				}

			}
		}

	} else 
	// VIDEOINFOHEADER2
	if ( rKSDataFormat2.DataFormat.Specifier == KSDATAFORMAT_SPECIFIER_VIDEOINFO2 ) {

		DUMP(("VIDEOINFOHEADER2\n"));

		DUMP(("Source (%d,%d,%d,%d)  Dest (%d,%d,%d,%d)\n",
				rVideoInfoHdrRequested2.rcSource.left,
				rVideoInfoHdrRequested2.rcSource.top,
				rVideoInfoHdrRequested2.rcSource.right,
				rVideoInfoHdrRequested2.rcSource.bottom,
				rVideoInfoHdrRequested2.rcTarget.left,
				rVideoInfoHdrRequested2.rcTarget.top,
				rVideoInfoHdrRequested2.rcTarget.right,
				rVideoInfoHdrRequested2.rcTarget.bottom ));

		DUMP(("Width=%d  Height=%d  BitCount=%d  Format=%x  biSizeImage=%d\n",
               rVideoInfoHdrRequested2.bmiHeader.biWidth,
               rVideoInfoHdrRequested2.bmiHeader.biHeight,
               rVideoInfoHdrRequested2.bmiHeader.biBitCount,
               rVideoInfoHdrRequested2.bmiHeader.biCompression,
			   rVideoInfoHdrRequested2.bmiHeader.biSizeImage
            ));

		DUMP(("dwInterlaceFlags=%d (0x%08x)\n",rVideoInfoHdrRequested2.dwInterlaceFlags,rVideoInfoHdrRequested2.dwInterlaceFlags));

		// Remember the last Time between frames and the cropping region
		REFERENCE_TIME AvgTimePerFrame;
		if (bIsVideoInfo2) {
			AvgTimePerFrame = VidHeader2.AvgTimePerFrame;
		} else {
			AvgTimePerFrame = VidHeader.AvgTimePerFrame;
		}

		// If no videochannel, only remember new format
		if (!vc) {

			DUMP(("No VideoChannel created - Only remember new format\n"));

			// Remember the format of this Stream... Only access provided fields!
			RtlCopyMemory(
				&VidHeader2,
				&rVideoInfoHdrRequested2,
				GetVideoInfoFormatSizeWithoutPalette(rVideoInfoHdrRequested2) );
			bIsVideoInfo2 = TRUE;

			// A renderer may be switching formats, and in this case, the AvgTimePerFrame
			// will be zero.  Don't overwrite a previously set framerate.
			if (!VidHeader2.AvgTimePerFrame) {
				VidHeader2.AvgTimePerFrame = AvgTimePerFrame;
			}

			// Force Cropping rect if we were asked to...
			if (!rcCropping.IsNull()) {
				// Force Cropping region!
				VidHeader2.rcSource = rcCropping;
			}

		} else {

			// There are 2 problems here: If we want to switch between alternating 
			// and normal mode, we MUST destroy videochannel. But, if only changing size,
			// perhaps we can do it without destroying it. The only problem is that
			// the creation of a VideoChannel can fail, and SetFormat shouldn't destroy
			// the videoChannel. So, get first the original parameters of VideoChannel.
			// destroy VideoChannel, try to create with the new parameters, if it fails,
			// Recreate with the old Parameters!

			// Destroy VideoChannel! - Hope to get success !
			NTSTATUS ret = Destroy();
			if (ret != STATUS_SUCCESS) {
				return ret;
			}

			// Remember the previous format - All fields always available!
			KS_VIDEOINFOHEADER_WITHMASK oldVidHeader = VidHeader;
			KS_VIDEOINFOHEADER2_WITHMASK oldVidHeader2 = VidHeader2;
			BOOL oldbIsVideoInfo2 = bIsVideoInfo2;

			// Set the new format!
			// Remember the format of this Stream... Only access provided fields!
			RtlCopyMemory(
				&VidHeader2,
				&rVideoInfoHdrRequested2,
				GetVideoInfoFormatSizeWithoutPalette(rVideoInfoHdrRequested2) );
			bIsVideoInfo2 = TRUE;

			// A renderer may be switching formats, and in this case, the AvgTimePerFrame
			// will be zero.  Don't overwrite a previously set framerate.
			if (!VidHeader2.AvgTimePerFrame) {
				VidHeader2.AvgTimePerFrame = AvgTimePerFrame;
			}

			// Force Cropping rect if we were asked to...
			if (!rcCropping.IsNull()) {
				// Force Cropping region!
				VidHeader2.rcSource = rcCropping;
			}

			ret = Create();
			if (ret != STATUS_SUCCESS) { // Failure ... Recretate channel with old parameters!

				// Restore original format - Always all fields available!
				VidHeader = oldVidHeader;
				VidHeader2 = oldVidHeader2;
				bIsVideoInfo2 = oldbIsVideoInfo2;

				// Recreate channel!
				ret = Create();
				if (ret != STATUS_SUCCESS) { // Failure ... Why ?
					return ret;
				}

				// If we were running, restart stream!
				if (KSState == KSSTATE_RUN) {
					vc->Start();
				}
				return STATUS_NO_MATCH;

			} else {

				// If we were running, restart stream!
				if (KSState == KSSTATE_RUN) {
					vc->Start();
				}

			}
		}

	} else {

		ERROR(("Invalid video header\n"));
		return STATUS_INVALID_PARAMETER;
	}

	// Everything OK ...
	return STATUS_SUCCESS;
}

/*
** BtVideoStream::VerifyFormat()
**
**   Checks the validity of a format request by walking through the
**       array of supported KSDATA_RANGEs for a given stream.
**
** Arguments:
**
**   pKSDataFormat - pointer of a KSDATAFORMAT structure.
**
** Returns:
** 
**   TRUE if the format is supported
**   FALSE if the format cannot be suppored
**
** Side Effects:  none
*/

BOOL BtVideoStream::VerifyFormat( PKSDATAFORMAT pKSDataFormatToVerify )
{
    TRACE(("BtVideoStream::VerifyFormat"));

	BOOL                        fOK = FALSE;
    ULONG                       j;

    // Get the pointer to the array of available formats
	PKSDATAFORMAT *pAvailableFormats = Info().StreamFormatsArray;
	ULONG NumFormats = Info().NumberOfFormatArrayEntries;

    DUMP(("FormatSize=%d\n",  pKSDataFormatToVerify->FormatSize));
    DUMP(("MajorFormat=%x\n", pKSDataFormatToVerify->MajorFormat));

    //
    // Walk the formats supported by the stream
    //

	// Get the VideoStandard of the requested format and the actual VideoStandard used in the chip.
	ULONG currVideoStandard = (ULONG) digitizer->GetBtCard().VideoGetCurrentStandard();

    for (j = 0; j < NumFormats; j++, pAvailableFormats++) {

        // Check for a match on the three GUIDs and format size
        if (!CompareGUIDsAndFormatSize(
                        pKSDataFormatToVerify, 
                        *pAvailableFormats,
                        FALSE)) {
            continue;
        }

        // Now that the three GUIDs match, switch on the Specifier
        // to do a further type-specific check

        // -------------------------------------------------------------------
        // Specifier FORMAT_VideoInfo for VIDEOINFOHEADER
        // -------------------------------------------------------------------
#if 0
        DUMP(("Try to match specifier\n"));
        DumpGUID(pKSDataFormatToVerify->Specifier);
        DUMP(("to KSDATAFORMAT_SPECIFIER_VIDEOINFO\n"));
        DumpGUID(KSDATAFORMAT_SPECIFIER_VIDEOINFO);
        DUMP(( "or KSDATAFORMAT_SPECIFIER_VIDEOINFO2\n"));
        DumpGUID(KSDATAFORMAT_SPECIFIER_VIDEOINFO2);
#endif
        if ( pKSDataFormatToVerify->Specifier == KSDATAFORMAT_SPECIFIER_VIDEOINFO ) {
                
            PKS_DATAFORMAT_VIDEOINFOHEADER  pDataFormatVideoInfoHeader = (PKS_DATAFORMAT_VIDEOINFOHEADER) pKSDataFormatToVerify;
            PKS_VIDEOINFOHEADER				pVideoInfoHdrToVerify = (PKS_VIDEOINFOHEADER) &pDataFormatVideoInfoHeader->VideoInfoHeader;

            PKS_DATARANGE_VIDEO             pKSDataRangeVideo = (PKS_DATARANGE_VIDEO) *pAvailableFormats;
            KS_VIDEO_STREAM_CONFIG_CAPS    *pConfigCaps = &pKSDataRangeVideo->ConfigCaps;

            RECT                            rcImage;

            DUMP(("VIDEOINFOHEADER\n"));
            DUMP(("pVideoInfoHdrToVerify=%x\n", pVideoInfoHdrToVerify));
            DUMP(("KS_VIDEOINFOHEADER size=%d\n", KS_SIZE_VIDEOHEADER (pVideoInfoHdrToVerify)));

            DUMP(("Width=%d  Height=%d  BitCount=%d  Format=%x\n", 
               pVideoInfoHdrToVerify->bmiHeader.biWidth,
               pVideoInfoHdrToVerify->bmiHeader.biHeight,
               pVideoInfoHdrToVerify->bmiHeader.biBitCount,
               pVideoInfoHdrToVerify->bmiHeader.biCompression
            ));

            DUMP(("biSizeImage=%d\n", 
                pVideoInfoHdrToVerify->bmiHeader.biSizeImage));

            /************************************************************** 
            *  HOW BIG IS THE IMAGE REQUESTED (pseudocode follows)
            *
            *  if (IsRectEmpty (&rcTarget) {
            *      SetRect (&rcImage, 0, 0, 
            *              BITMAPINFOHEADER.biWidth,
            *              BITMAPINFOHEADER.biHeight);
            *  }
            *  else {
            *      // Probably rendering to a DirectDraw surface,
            *      // where biWidth is used to expressed the "stride" 
            *      // in units of pixels (not bytes) of the destination surface.
            *      // Therefore, use rcTarget to get the actual image size 
            *      
            *      rcImage = rcTarget;
            *  }
            **************************************************************/
        
            if ((pVideoInfoHdrToVerify->rcTarget.right - 
                 pVideoInfoHdrToVerify->rcTarget.left <= 0) ||
                (pVideoInfoHdrToVerify->rcTarget.bottom - 
                 pVideoInfoHdrToVerify->rcTarget.top <= 0)) {
            
                 rcImage.left = rcImage.top = 0;
                 rcImage.right = abs(pVideoInfoHdrToVerify->bmiHeader.biWidth);
                 rcImage.bottom = abs(pVideoInfoHdrToVerify->bmiHeader.biHeight);
            }
            else {
                 rcImage = pVideoInfoHdrToVerify->rcTarget;
            }

			// Check that we support the requested format... (validate only the most important ones)
			if (
				// Invalid standard...
				(currVideoStandard & pConfigCaps->VideoStandard) == 0 ||

				// Invalid granularity...
				(pConfigCaps->OutputGranularityX != 0 && ((rcImage.right - rcImage.left) & (pConfigCaps->OutputGranularityX-1)) != 0) ||
				(pConfigCaps->OutputGranularityY != 0 && ((rcImage.bottom - rcImage.top) & (pConfigCaps->OutputGranularityY-1)) != 0) ||

				// Invalid size...
				(rcImage.right - rcImage.left) < pConfigCaps->MinOutputSize.cx ||
				(rcImage.right - rcImage.left) > pConfigCaps->MaxOutputSize.cx ||
				(rcImage.bottom - rcImage.top) < pConfigCaps->MinOutputSize.cy ||
				(rcImage.bottom - rcImage.top) > pConfigCaps->MaxOutputSize.cy ||

				// Invalid compression...
				pVideoInfoHdrToVerify->bmiHeader.biCompression == KS_BI_RLE8 ||
				pVideoInfoHdrToVerify->bmiHeader.biCompression == KS_BI_RLE4

				) 
			{
				// Can't use it...
				continue;
			} 

			// HOORAY, the format passed all of the tests, so we support it

            fOK = TRUE;
            break;

        } // End of VIDEOINFOHEADER specifier

        // -------------------------------------------------------------------
        // Specifier FORMAT_VideoInfo for VIDEOINFOHEADER2
        // -------------------------------------------------------------------

        else 
		if ( pKSDataFormatToVerify->Specifier == KSDATAFORMAT_SPECIFIER_VIDEOINFO2 ) {
                
            KS_DATAFORMAT_VIDEOINFOHEADER2*  pDataFormatVideoInfoHeader = (KS_DATAFORMAT_VIDEOINFOHEADER2*) pKSDataFormatToVerify;
            KS_VIDEOINFOHEADER2*  pVideoInfoHdrToVerify = (KS_VIDEOINFOHEADER2*) &pDataFormatVideoInfoHeader->VideoInfoHeader2;
            PKS_DATARANGE_VIDEO2             pKSDataRangeVideo = (KS_DATARANGE_VIDEO2*) *pAvailableFormats;
            KS_VIDEO_STREAM_CONFIG_CAPS    *pConfigCaps = &pKSDataRangeVideo->ConfigCaps;
            RECT                            rcImage;

            DUMP(("VIDEOINFOHEADER2\n"));
            DUMP(("pVideoInfoHdrToVerify=%x\n", pVideoInfoHdrToVerify));
            DUMP(("KS_VIDEOINFOHEADER size=%d\n", KS_SIZE_VIDEOHEADER2 (pVideoInfoHdrToVerify)));
            
            DUMP(("Width=%d  Height=%d  BitCount=%d  Format=%x\n", 
               pVideoInfoHdrToVerify->bmiHeader.biWidth,
               pVideoInfoHdrToVerify->bmiHeader.biHeight,
               pVideoInfoHdrToVerify->bmiHeader.biBitCount,
               pVideoInfoHdrToVerify->bmiHeader.biCompression
            ));

            DUMP(("biSizeImage=%d\n", pVideoInfoHdrToVerify->bmiHeader.biSizeImage));
			DUMP(("dwInterlaceFlags=%d (0x%08x)\n",pVideoInfoHdrToVerify->dwInterlaceFlags,pVideoInfoHdrToVerify->dwInterlaceFlags));

            /************************************************************** 
            *  HOW BIG IS THE IMAGE REQUESTED (pseudocode follows)
            *
            *  if (IsRectEmpty (&rcTarget) {
            *      SetRect (&rcImage, 0, 0, 
            *              BITMAPINFOHEADER.biWidth,
            *              BITMAPINFOHEADER.biHeight);
            *  }
            *  else {
            *      // Probably rendering to a DirectDraw surface,
            *      // where biWidth is used to expressed the "stride" 
            *      // in units of pixels (not bytes) of the destination surface.
            *      // Therefore, use rcTarget to get the actual image size 
            *      
            *      rcImage = rcTarget;
            *  }
            **************************************************************/
        
            if ((pVideoInfoHdrToVerify->rcTarget.right - 
                 pVideoInfoHdrToVerify->rcTarget.left <= 0) ||
                (pVideoInfoHdrToVerify->rcTarget.bottom - 
                 pVideoInfoHdrToVerify->rcTarget.top <= 0)) {
            
                 rcImage.left = rcImage.top = 0;
                 rcImage.right = abs(pVideoInfoHdrToVerify->bmiHeader.biWidth);
                 rcImage.bottom = abs(pVideoInfoHdrToVerify->bmiHeader.biHeight);
            }
            else {
                 rcImage = pVideoInfoHdrToVerify->rcTarget;
            }

			// Check that we support the requested format... (validate only the most important ones)
			if (
				// Invalid standard...
				(currVideoStandard & pConfigCaps->VideoStandard) == 0 ||

				// Invalid granularity...
				(pConfigCaps->OutputGranularityX != 0 && ((rcImage.right - rcImage.left) & (pConfigCaps->OutputGranularityX-1)) != 0) ||
				(pConfigCaps->OutputGranularityY != 0 && ((rcImage.bottom - rcImage.top) & (pConfigCaps->OutputGranularityY-1)) != 0) ||

				// Invalid size...
				(rcImage.right - rcImage.left) < pConfigCaps->MinOutputSize.cx ||
				(rcImage.right - rcImage.left) > pConfigCaps->MaxOutputSize.cx ||
				(rcImage.bottom - rcImage.top) < pConfigCaps->MinOutputSize.cy ||
				(rcImage.bottom - rcImage.top) > pConfigCaps->MaxOutputSize.cy ||

				// Invalid compression...
				pVideoInfoHdrToVerify->bmiHeader.biCompression == KS_BI_RLE8 ||
				pVideoInfoHdrToVerify->bmiHeader.biCompression == KS_BI_RLE4 ||

				// Invalid interlacing flags... (so we apply the right entry) , without paying attention to the Display order...
				(pVideoInfoHdrToVerify->dwInterlaceFlags & (~KS_INTERLACE_DisplayModeMask)) != 
					(pKSDataRangeVideo->VideoInfoHeader.dwInterlaceFlags & (~KS_INTERLACE_DisplayModeMask))
				) 
			{
				// Can't use it...
				continue;
			} 

			// HOORAY, the format passed all of the tests, so we support it

            fOK = TRUE;
            break;


        } // End of VIDEOINFOHEADER2 specifier

    } // End of loop on all formats for this stream
    
    return fOK;
}

/*
** BtVideoStream::FormatFromRange()
**
**   Returns a DATAFORMAT from a DATARANGE
**
** Arguments:
**
**         IN PHW_STREAM_REQUEST_BLOCK pSrb
**
** Returns:
**
**   TRUE if the format is supported
**   FALSE if the format cannot be supported
**
** Side Effects:  none
*/
NTSTATUS BtVideoStream::FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BtVideoStream::FormatFromRange"));

	/*
		Note: Here, I will assume that this Fn maps to the DirectShow function 
			CheckMediaType. This way I can force, if needed, DShow GraphBuilder
			to use Same format for Preview and Capture at full size. This should
			avoid those Preview problems... Unfortunately, there is no way to query
			the downstream filter connected to the preview pin if it will support
			a new format from kernel Space ... If someone knows how, tell!
	
		*/

	PSTREAM_DATA_INTERSECT_INFO IntersectInfo;
	PKSDATARANGE                DataRange; // Note that DATARANGE and DATARANGE2 are the same up to the VIDEOINFOHEADER struct
	ULONG                       FormatSize = 0;
	ULONG                       j;
   
	IntersectInfo = pSrb->CommandData.IntersectInfo;
	DataRange = IntersectInfo->DataRange;

	//
	// Get the pointer to the array of available formats and the format count.
	//

	PKSDATAFORMAT *pAvailableFormats = Info().StreamFormatsArray;
	ULONG NumFormats = Info().NumberOfFormatArrayEntries;

	//
	// Is the caller trying to get the format, or the size of the format?
	//

	const bool OnlyWantsSize = (IntersectInfo->SizeOfDataFormatBuffer == sizeof( ULONG ) ) ||
							   (IntersectInfo->SizeOfDataFormatBuffer == 0 )
								;

	//
	// Walk the formats supported by the stream searching for a match
	// of the three GUIDs which together define a DATARANGE
	//

	// Get the VideoStandard of the requested format and the actual VideoStandard used in the chip.
	/*ULONG currVideoStandard = (ULONG) digitizer->GetBtCard().VideoGetCurrentStandard();*/

	for ( j = 0; j < NumFormats ; j++, pAvailableFormats++ ) {

		if ( !CompareGUIDsAndFormatSize( DataRange, *pAvailableFormats, TRUE )) {
			continue;
		}

		//
		// Now that the three GUIDs match, switch on the Specifier
		// to do a further type specific check
		//

		// -------------------------------------------------------------------
		// Specifier FORMAT_VideoInfo for VIDEOINFOHEADER
		// -------------------------------------------------------------------

		// VIDEOINFOHEADER
		if ( DataRange->Specifier == KSDATAFORMAT_SPECIFIER_VIDEOINFO ) 
		{

			PKS_DATARANGE_VIDEO DataRangeVideoToVerify = (PKS_DATARANGE_VIDEO)DataRange;
			PKS_DATARANGE_VIDEO DataRangeVideo = (PKS_DATARANGE_VIDEO)*pAvailableFormats;

            DUMP(("VIDEOINFOHEADER: To verify\n"));

            DUMP(("  FixedSzSmp=%x  TempComp=%x  StramDescrFlgs=%x  MemAllocFlgs=%x\n", 
					DataRangeVideoToVerify->bFixedSizeSamples,
					DataRangeVideoToVerify->bTemporalCompression,
					DataRangeVideoToVerify->StreamDescriptionFlags,
					DataRangeVideoToVerify->MemoryAllocationFlags
            ));

			DUMP(("  VideoStd=%x  InSz=(%d,%d)  MinCropSz=(%d,%d)  MaxCropSz=(%d,%d)\n",
					DataRangeVideoToVerify->ConfigCaps.VideoStandard,
					DataRangeVideoToVerify->ConfigCaps.InputSize.cx,DataRangeVideoToVerify->ConfigCaps.InputSize.cy,
					DataRangeVideoToVerify->ConfigCaps.MinCroppingSize.cx,DataRangeVideoToVerify->ConfigCaps.MinCroppingSize.cy,
					DataRangeVideoToVerify->ConfigCaps.MaxCroppingSize.cx,DataRangeVideoToVerify->ConfigCaps.MaxCroppingSize.cy ));

			DUMP(("  CropGran=(%d,%d)  CropAlign=(%d,%d)  MinOutSz=(%d,%d)  MaxOutSz=(%d,%d)\n",
					DataRangeVideoToVerify->ConfigCaps.CropGranularityX,DataRangeVideoToVerify->ConfigCaps.CropGranularityY,
					DataRangeVideoToVerify->ConfigCaps.CropAlignX,DataRangeVideoToVerify->ConfigCaps.CropAlignY,
					DataRangeVideoToVerify->ConfigCaps.MinOutputSize.cx,DataRangeVideoToVerify->ConfigCaps.MinOutputSize.cy,
					DataRangeVideoToVerify->ConfigCaps.MaxOutputSize.cx,DataRangeVideoToVerify->ConfigCaps.MaxOutputSize.cy ));

			DUMP(("  OutGran=(%d,%d)  StretchTaps=(%d,%d)  ShrinkTaps=(%d,%d)\n",
					DataRangeVideoToVerify->ConfigCaps.OutputGranularityX,DataRangeVideoToVerify->ConfigCaps.OutputGranularityY,
					DataRangeVideoToVerify->ConfigCaps.StretchTapsX,DataRangeVideoToVerify->ConfigCaps.StretchTapsY,
					DataRangeVideoToVerify->ConfigCaps.ShrinkTapsX,DataRangeVideoToVerify->ConfigCaps.ShrinkTapsY ));
			
			DUMP(("  MinFrmInt=%d  MaxFrmInt=%d MinBps=%d  MaxBps=%d\n",
					DataRangeVideoToVerify->ConfigCaps.MinFrameInterval,DataRangeVideoToVerify->ConfigCaps.MaxFrameInterval,
					DataRangeVideoToVerify->ConfigCaps.MinBitsPerSecond,DataRangeVideoToVerify->ConfigCaps.MaxBitsPerSecond ));

			DUMP(("Src(%d,%d)-(%d,%d) Dst(%d,%d)-(%d,%d) BitRt=%d,BitER=%d,AvgTPF=%d\n",
					DataRangeVideoToVerify->VideoInfoHeader.rcSource.left,
					DataRangeVideoToVerify->VideoInfoHeader.rcSource.top,
					DataRangeVideoToVerify->VideoInfoHeader.rcSource.right,
					DataRangeVideoToVerify->VideoInfoHeader.rcSource.bottom,
					DataRangeVideoToVerify->VideoInfoHeader.rcTarget.left,
					DataRangeVideoToVerify->VideoInfoHeader.rcTarget.top,
					DataRangeVideoToVerify->VideoInfoHeader.rcTarget.right,
					DataRangeVideoToVerify->VideoInfoHeader.rcTarget.bottom,
					DataRangeVideoToVerify->VideoInfoHeader.dwBitRate,
					DataRangeVideoToVerify->VideoInfoHeader.dwBitErrorRate,
					DataRangeVideoToVerify->VideoInfoHeader.AvgTimePerFrame
			));
			DUMP(("biSz=%d,WxH=(%d,%d),Pln=%d,Bpp=%d,Cmp=%04x,ImgSz=%d,PPM(%d,%d),ClrUsd=%d,ClrImp=%d\n",
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biSize,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biWidth,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biHeight,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biPlanes,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biBitCount,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biCompression,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biSizeImage,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biXPelsPerMeter,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biYPelsPerMeter,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biClrUsed,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biClrImportant
			));

			DUMP(("With:\n"));
            DUMP(("  FixedSzSmp=%x  TempComp=%x  StramDescrFlgs=%x  MemAllocFlgs=%x\n", 
					DataRangeVideo->bFixedSizeSamples,
					DataRangeVideo->bTemporalCompression,
					DataRangeVideo->StreamDescriptionFlags,
					DataRangeVideo->MemoryAllocationFlags
            ));

			DUMP(("  VideoStd=%x  InSz=(%d,%d)  MinCropSz=(%d,%d)  MaxCropSz=(%d,%d)\n",
					DataRangeVideo->ConfigCaps.VideoStandard,
					DataRangeVideo->ConfigCaps.InputSize.cx,DataRangeVideo->ConfigCaps.InputSize.cy,
					DataRangeVideo->ConfigCaps.MinCroppingSize.cx,DataRangeVideo->ConfigCaps.MinCroppingSize.cy,
					DataRangeVideo->ConfigCaps.MaxCroppingSize.cx,DataRangeVideo->ConfigCaps.MaxCroppingSize.cy ));

			DUMP(("  CropGran=(%d,%d)  CropAlign=(%d,%d)  MinOutSz=(%d,%d)  MaxOutSz=(%d,%d)\n",
					DataRangeVideo->ConfigCaps.CropGranularityX,DataRangeVideo->ConfigCaps.CropGranularityY,
					DataRangeVideo->ConfigCaps.CropAlignX,DataRangeVideo->ConfigCaps.CropAlignY,
					DataRangeVideo->ConfigCaps.MinOutputSize.cx,DataRangeVideo->ConfigCaps.MinOutputSize.cy,
					DataRangeVideo->ConfigCaps.MaxOutputSize.cx,DataRangeVideo->ConfigCaps.MaxOutputSize.cy ));

			DUMP(("  OutGran=(%d,%d)  StretchTaps=(%d,%d)  ShrinkTaps=(%d,%d)\n",
					DataRangeVideo->ConfigCaps.OutputGranularityX,DataRangeVideo->ConfigCaps.OutputGranularityY,
					DataRangeVideo->ConfigCaps.StretchTapsX,DataRangeVideo->ConfigCaps.StretchTapsY,
					DataRangeVideo->ConfigCaps.ShrinkTapsX,DataRangeVideo->ConfigCaps.ShrinkTapsY ));
			
			DUMP(("  MinFrmInt=%d  MaxFrmInt=%d MinBps=%d  MaxBps=%d\n",
					DataRangeVideo->ConfigCaps.MinFrameInterval,DataRangeVideo->ConfigCaps.MaxFrameInterval,
					DataRangeVideo->ConfigCaps.MinBitsPerSecond,DataRangeVideo->ConfigCaps.MaxBitsPerSecond ));


			DUMP(("Src(%d,%d)-(%d,%d) Dst(%d,%d)-(%d,%d) BitRt=%d,BitER=%d,AvgTPF=%d\n",
					DataRangeVideo->VideoInfoHeader.rcSource.left,
					DataRangeVideo->VideoInfoHeader.rcSource.top,
					DataRangeVideo->VideoInfoHeader.rcSource.right,
					DataRangeVideo->VideoInfoHeader.rcSource.bottom,
					DataRangeVideo->VideoInfoHeader.rcTarget.left,
					DataRangeVideo->VideoInfoHeader.rcTarget.top,
					DataRangeVideo->VideoInfoHeader.rcTarget.right,
					DataRangeVideo->VideoInfoHeader.rcTarget.bottom,
					DataRangeVideo->VideoInfoHeader.dwBitRate,
					DataRangeVideo->VideoInfoHeader.dwBitErrorRate,
					DataRangeVideo->VideoInfoHeader.AvgTimePerFrame
			));
			DUMP(("biSz=%d,WxH=(%d,%d),Pln=%d,Bpp=%d,Cmp=%04x,ImgSz=%d,PPM(%d,%d),ClrUsd=%d,ClrImp=%d\n",
					DataRangeVideo->VideoInfoHeader.bmiHeader.biSize,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biWidth,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biHeight,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biPlanes,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biBitCount,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biCompression,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biSizeImage,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biXPelsPerMeter,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biYPelsPerMeter,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biClrUsed,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biClrImportant
			));

	    //
        // KS_DATAFORMAT_VIDEOINFOHEADER:
        //    fields marked with 'm' must match; 
        //           marked with 'r' must within range;
        //           marked with 'f' is filled by us
        //
        //     KSDATAFORMAT == KSDATARANGE
        //       m ULONG   FormatSize;
        //         ULONG   Flags;
        //         ULONG   SampleSize;
        //         ULONG   Reserved;
        //       m GUID    MajorFormat;
        //       m GUID    SubFormat;
        //       m GUID    Specifier;.
        //  m  BOOL                         bFixedSizeSamples;      // all samples same size?
        //  m  BOOL                         bTemporalCompression;   // all I frames?
        //  m  DWORD                        StreamDescriptionFlags; // KS_VIDEO_DESC_*
        //  m  DWORD                        MemoryAllocationFlags;  // KS_VIDEO_ALLOC_*
        //  m  KS_VIDEO_STREAM_CONFIG_CAPS  ConfigCaps;
        //     KS_VIDEOINFOHEADER 
        //         RECT                rcSource;          // The bit we really want to use
        //         RECT                rcTarget;          // Where the video should go
        //     f   DWORD               dwBitRate;         // Approximate bit data rate
        //     f   DWORD               dwBitErrorRate;    // Bit error rate for this stream
        //     r/f REFERENCE_TIME      AvgTimePerFrame;   // Average time per frame (100ns units)
        //         KS_BITMAPINFOHEADER bmiHeader;
        //       f     DWORD      biSize;
        //       r/f   LONG       biWidth;
        //       r/f   LONG       biHeight;
        //       f     WORD       biPlanes;
        //       f     WORD       biBitCount;
        //       f     DWORD      biCompression;
        //       f     DWORD      biSizeImage;
        //       f     LONG       biXPelsPerMeter;
        //       f     LONG       biYPelsPerMeter;
        //       f     DWORD      biClrUsed;
        //       f     DWORD      biClrImportant;
        //     

			// We have already matched the colorspace ... Match the others non-dependent on VideoStandard..

			// Only accept formats we can handle right now..
			/* CAN'T BE DONE, OR DSHOW runtime will bomb!
			if (DataRangeVideo->ConfigCaps.VideoStandard && 
				(DataRangeVideo->ConfigCaps.VideoStandard & currVideoStandard) == 0) {
				DUMP(("Invalid format for the current video std\n"));
				continue;
			}*/

			//
			// Check that the other fields match
			//
			if ( (DataRangeVideoToVerify->bFixedSizeSamples     != DataRangeVideo->bFixedSizeSamples)      ||
				(DataRangeVideoToVerify->bTemporalCompression   != DataRangeVideo->bTemporalCompression)   ||
				(DataRangeVideoToVerify->StreamDescriptionFlags != DataRangeVideo->StreamDescriptionFlags) ||
				(DataRangeVideoToVerify->MemoryAllocationFlags  != DataRangeVideo->MemoryAllocationFlags)  ||
	            (!RtlEqualMemory( &DataRangeVideoToVerify->ConfigCaps,	&DataRangeVideo->ConfigCaps, sizeof( KS_VIDEO_STREAM_CONFIG_CAPS ) ) ) 
			   ) 
			{
				DUMP(( "At least one field does not match\n" ));
				continue;
			}

			DUMP(( "Match Found!\n" ));

			// MATCH FOUND!
			FormatSize = sizeof( KSDATAFORMAT ) + GetVideoInfoFormatSize(DataRangeVideo->VideoInfoHeader);

			if ( OnlyWantsSize ) {
				break;
			}

			// Caller wants the full data format
			if ( IntersectInfo->SizeOfDataFormatBuffer < FormatSize ) {
				// Format was found, but provided buffer is too small...
				DUMP(( "STATUS_BUFFER_TOO_SMALL\n" ));
				return STATUS_BUFFER_TOO_SMALL;
			}

			// Copy over the KSDATAFORMAT, followed by the
			// actual VideoInfoHeader
			PKS_DATAFORMAT_VIDEOINFOHEADER InterVidHdr = 
				(PKS_DATAFORMAT_VIDEOINFOHEADER)IntersectInfo->DataFormatBuffer;

			// Copy the GUID of ColorFormat
			RtlCopyMemory( 
				&InterVidHdr->DataFormat,
				&DataRangeVideo->DataRange,
				sizeof( KSDATARANGE ) );

			// Set the format size
			InterVidHdr->DataFormat.FormatSize = FormatSize;

			// AvgTimePerFrame (FrameRate) need to be within (ConfigCaps.MinFrameInterval, ConfigCaps.MaxFrameInterval)
			LONGLONG AvgTimePerFrame = DataRangeVideoToVerify->VideoInfoHeader.AvgTimePerFrame;
			if (AvgTimePerFrame > DataRangeVideoToVerify->ConfigCaps.MaxFrameInterval ||      
				AvgTimePerFrame < DataRangeVideoToVerify->ConfigCaps.MinFrameInterval) {

				AvgTimePerFrame = DataRangeVideo->VideoInfoHeader.AvgTimePerFrame;
				DUMP(("AvgTimePerFrame out of range; so set it to default (%ld)\n",AvgTimePerFrame));

			}

			// Copy to destination to original VIDEOINFOHEADER (will we accepted)!
			// Unfortunately, those stupid people at M$ didnt pass all the required
			// info in the DataRangeVideoToVerify->VideoInfoHeader, so we must use our
			// template to do this!
			RtlCopyMemory(
				&InterVidHdr->VideoInfoHeader,
				&DataRangeVideoToVerify->VideoInfoHeader,
				GetVideoInfoFormatSize(DataRangeVideo->VideoInfoHeader));
        
			// report back the image size as we know it

			// Get a copy of the VideoInfoHeader ... Only access provided fields!
			KS_VIDEOINFOHEADER_WITHMASK vidHDR;
			RtlCopyMemory(
				&vidHDR,
				&DataRangeVideoToVerify->VideoInfoHeader,
				GetVideoInfoFormatSizeWithoutPalette(DataRangeVideo->VideoInfoHeader) );

			// Pass the requested framerate...
			InterVidHdr->VideoInfoHeader.AvgTimePerFrame = AvgTimePerFrame;

			// Now we must fill-in all the missing fields (Microsoft Opus) : Simply keep width and Height, 
			// copy all the members and restore them
			DWORD biWidth = InterVidHdr->VideoInfoHeader.bmiHeader.biWidth;
			DWORD biHeight = InterVidHdr->VideoInfoHeader.bmiHeader.biHeight;
			RtlCopyMemory(
				&InterVidHdr->VideoInfoHeader.bmiHeader,
				&DataRangeVideo->VideoInfoHeader.bmiHeader,
				GetBitmapFormatSize(DataRangeVideo->VideoInfoHeader.bmiHeader) );
			InterVidHdr->VideoInfoHeader.bmiHeader.biWidth = biWidth;
			InterVidHdr->VideoInfoHeader.bmiHeader.biHeight = biHeight;

			/************************************************************** 
            *  HOW BIG IS THE IMAGE REQUESTED (pseudocode follows)
            *
            *  if (IsRectEmpty (&rcTarget) {
            *      SetRect (&rcImage, 0, 0, 
            *              BITMAPINFOHEADER.biWidth,
            *              BITMAPINFOHEADER.biHeight);
            *  }
            *  else {
            *      // Probably rendering to a DirectDraw surface,
            *      // where biWidth is used to expressed the "stride" 
            *      // in units of pixels (not bytes) of the destination surface.
            *      // Therefore, use rcTarget to get the actual image size 
            *      
            *      rcImage = rcTarget;
            *  }
            **************************************************************/
        
            if ((DataRangeVideoToVerify->VideoInfoHeader.rcTarget.right - 
                 DataRangeVideoToVerify->VideoInfoHeader.rcTarget.left <= 0) ||
                (DataRangeVideoToVerify->VideoInfoHeader.rcTarget.bottom - 
                 DataRangeVideoToVerify->VideoInfoHeader.rcTarget.top <= 0)) {
            
				// Image size can be found on the BITMAPINFOHEADER... bound and fix it.
				if (DataRangeVideo->ConfigCaps.OutputGranularityX) {
					InterVidHdr->VideoInfoHeader.bmiHeader.biWidth &= (-DataRangeVideo->ConfigCaps.OutputGranularityX);
				}
				if (DataRangeVideo->ConfigCaps.OutputGranularityY) {
					InterVidHdr->VideoInfoHeader.bmiHeader.biHeight &= (-DataRangeVideo->ConfigCaps.OutputGranularityY);
				}
				if (abs(InterVidHdr->VideoInfoHeader.bmiHeader.biWidth) < DataRangeVideo->ConfigCaps.MinOutputSize.cx) {
					InterVidHdr->VideoInfoHeader.bmiHeader.biWidth =
						((InterVidHdr->VideoInfoHeader.bmiHeader.biWidth < 0) ? -1 : 1) *
						DataRangeVideo->ConfigCaps.MinOutputSize.cx;
				}
				if (abs(InterVidHdr->VideoInfoHeader.bmiHeader.biWidth) > DataRangeVideo->ConfigCaps.MaxOutputSize.cx) {
					InterVidHdr->VideoInfoHeader.bmiHeader.biWidth = 
						((InterVidHdr->VideoInfoHeader.bmiHeader.biWidth < 0) ? -1 : 1) *
						DataRangeVideo->ConfigCaps.MaxOutputSize.cx;
				}
				if (abs(InterVidHdr->VideoInfoHeader.bmiHeader.biHeight) < DataRangeVideo->ConfigCaps.MinOutputSize.cy) {
					InterVidHdr->VideoInfoHeader.bmiHeader.biHeight = 
						((InterVidHdr->VideoInfoHeader.bmiHeader.biHeight < 0) ? -1 : 1) *
						DataRangeVideo->ConfigCaps.MinOutputSize.cy;
				}
				if (abs(InterVidHdr->VideoInfoHeader.bmiHeader.biHeight) > DataRangeVideo->ConfigCaps.MaxOutputSize.cy) {
					InterVidHdr->VideoInfoHeader.bmiHeader.biHeight = 
						((InterVidHdr->VideoInfoHeader.bmiHeader.biHeight < 0) ? -1 : 1) *
						DataRangeVideo->ConfigCaps.MaxOutputSize.cy;
				}
				vidHDR.bmiHeader.biWidth = InterVidHdr->VideoInfoHeader.bmiHeader.biWidth;
				vidHDR.bmiHeader.biHeight = InterVidHdr->VideoInfoHeader.bmiHeader.biHeight;
			} else {
				// We are dealing with a DirectDraw Buffer... Fix what we need...
				// The BITMAPINFOHEADER width must be a multiple of 4....
				// The RECT must comply with all restrictions.... Left & right must be a multiple of OutpoutGranularity. (bt8x8 limitation)


				vidHDR.bmiHeader.biWidth &= (-4);
				InterVidHdr->VideoInfoHeader.bmiHeader.biWidth &= (-4);

				if (DataRangeVideo->ConfigCaps.OutputGranularityX) {
					InterVidHdr->VideoInfoHeader.rcTarget.left &= (-DataRangeVideo->ConfigCaps.OutputGranularityX);
					InterVidHdr->VideoInfoHeader.rcTarget.right &= (-DataRangeVideo->ConfigCaps.OutputGranularityX);
				}
				if (DataRangeVideo->ConfigCaps.OutputGranularityY) {
					InterVidHdr->VideoInfoHeader.rcTarget.top &= (-DataRangeVideo->ConfigCaps.OutputGranularityY);
					InterVidHdr->VideoInfoHeader.rcTarget.bottom &= (-DataRangeVideo->ConfigCaps.OutputGranularityY);
				}

				if ((InterVidHdr->VideoInfoHeader.rcTarget.right - InterVidHdr->VideoInfoHeader.rcTarget.left) < 
						DataRangeVideo->ConfigCaps.MinOutputSize.cx) {
					InterVidHdr->VideoInfoHeader.rcTarget.right = InterVidHdr->VideoInfoHeader.rcTarget.left + DataRangeVideo->ConfigCaps.MinOutputSize.cx;
				}
				if ((InterVidHdr->VideoInfoHeader.rcTarget.right - InterVidHdr->VideoInfoHeader.rcTarget.left) > 
						DataRangeVideo->ConfigCaps.MaxOutputSize.cx) {
					InterVidHdr->VideoInfoHeader.rcTarget.right = InterVidHdr->VideoInfoHeader.rcTarget.left + DataRangeVideo->ConfigCaps.MaxOutputSize.cx;
				}

				if ((InterVidHdr->VideoInfoHeader.rcTarget.bottom - InterVidHdr->VideoInfoHeader.rcTarget.top) < 
						DataRangeVideo->ConfigCaps.MinOutputSize.cy) {
					InterVidHdr->VideoInfoHeader.rcTarget.bottom = InterVidHdr->VideoInfoHeader.rcTarget.top + DataRangeVideo->ConfigCaps.MinOutputSize.cy;
				}
				if ((InterVidHdr->VideoInfoHeader.rcTarget.bottom - InterVidHdr->VideoInfoHeader.rcTarget.top) > 
						DataRangeVideo->ConfigCaps.MaxOutputSize.cy) {
					InterVidHdr->VideoInfoHeader.rcTarget.bottom = InterVidHdr->VideoInfoHeader.rcTarget.top + DataRangeVideo->ConfigCaps.MaxOutputSize.cy;
				}

				vidHDR.rcTarget = InterVidHdr->VideoInfoHeader.rcTarget;
            }

			// Normalize Frame
			ColFmt col;
			NormalizeFrame(vidHDR,col);

			// make sure the dimensions are acceptable
			ColorSpace  tmpCol( DataRange->SubFormat );

			// Get the destination Rect
			MRect dst( vidHDR.rcTarget );
			MSize size = dst.Size();

			// If color is correct ...
			if ( tmpCol.IsValid() && tmpCol.CheckDimentions( size ) &&
				tmpCol.CheckLeftTop( dst.TopLeft() ) ) 
			{

				// Calculate biSizeImage for this request, and put the result in both
				// the biSizeImage field of the bmiHeader AND in the SampleSize field
				// of the DataFormat.
				//
				// Note that for compressed sizes, this calculation will probably not
				// be just width * height * bitdepth

				// Use the Bitmap size to estimate size in bytes
				InterVidHdr->VideoInfoHeader.bmiHeader.biSizeImage =
				InterVidHdr->DataFormat.SampleSize = (DWORD) KS_DIBSIZE(vidHDR.bmiHeader);

				// Fill in the produced bitrate...
				InterVidHdr->VideoInfoHeader.dwBitRate = (DWORD)(
					(InterVidHdr->DataFormat.SampleSize * 80000000i64) / 
					((AvgTimePerFrame) ? AvgTimePerFrame : 10000000i64));
				InterVidHdr->VideoInfoHeader.dwBitErrorRate = 0;


				DUMP(("InterVidHdr->VideoInfoHeader.bmiHeader.biSizeImage = %d\n", InterVidHdr->VideoInfoHeader.bmiHeader.biSizeImage));
				break;
			} 
			else 
			{
				// The requested cropping/alignment is not supported, continue search!
	            ERROR(("Invalid parameters (cropping/slignment)\n"));
				continue;
			}
		} // End of VIDEOINFOHEADER specifier

		// -------------------------------------------------------------------
		// Specifier FORMAT_VideoInfo2 for VIDEOINFOHEADER2
		// -------------------------------------------------------------------
		else if ( DataRange->Specifier == KSDATAFORMAT_SPECIFIER_VIDEOINFO2 ) 
		{

			PKS_DATARANGE_VIDEO2 DataRangeVideoToVerify = (PKS_DATARANGE_VIDEO2) DataRange;
			PKS_DATARANGE_VIDEO2 DataRangeVideo = (PKS_DATARANGE_VIDEO2) *pAvailableFormats;


			DUMP(("VIDEOINFOHEADER2 To verify\n"));
            
            DUMP(("  FixedSzSmp=%x  TempComp=%x  StramDescrFlgs=%x  MemAllocFlgs=%x\n", 
					DataRangeVideoToVerify->bFixedSizeSamples,
					DataRangeVideoToVerify->bTemporalCompression,
					DataRangeVideoToVerify->StreamDescriptionFlags,
					DataRangeVideoToVerify->MemoryAllocationFlags
            ));

			DUMP(("  VideoStd=%x  InSz=(%d,%d)  MinCropSz=(%d,%d)  MaxCropSz=(%d,%d)\n",
					DataRangeVideoToVerify->ConfigCaps.VideoStandard,
					DataRangeVideoToVerify->ConfigCaps.InputSize.cx,DataRangeVideoToVerify->ConfigCaps.InputSize.cy,
					DataRangeVideoToVerify->ConfigCaps.MinCroppingSize.cx,DataRangeVideoToVerify->ConfigCaps.MinCroppingSize.cy,
					DataRangeVideoToVerify->ConfigCaps.MaxCroppingSize.cx,DataRangeVideoToVerify->ConfigCaps.MaxCroppingSize.cy ));

			DUMP(("  CropGran=(%d,%d)  CropAlign=(%d,%d)  MinOutSz=(%d,%d)  MaxOutSz=(%d,%d)\n",
					DataRangeVideoToVerify->ConfigCaps.CropGranularityX,DataRangeVideoToVerify->ConfigCaps.CropGranularityY,
					DataRangeVideoToVerify->ConfigCaps.CropAlignX,DataRangeVideoToVerify->ConfigCaps.CropAlignY,
					DataRangeVideoToVerify->ConfigCaps.MinOutputSize.cx,DataRangeVideoToVerify->ConfigCaps.MinOutputSize.cy,
					DataRangeVideoToVerify->ConfigCaps.MaxOutputSize.cx,DataRangeVideoToVerify->ConfigCaps.MaxOutputSize.cy ));

			DUMP(("  OutGran=(%d,%d)  StretchTaps=(%d,%d)  ShrinkTaps=(%d,%d)\n",
					DataRangeVideoToVerify->ConfigCaps.OutputGranularityX,DataRangeVideoToVerify->ConfigCaps.OutputGranularityY,
					DataRangeVideoToVerify->ConfigCaps.StretchTapsX,DataRangeVideoToVerify->ConfigCaps.StretchTapsY,
					DataRangeVideoToVerify->ConfigCaps.ShrinkTapsX,DataRangeVideoToVerify->ConfigCaps.ShrinkTapsY ));
			
			DUMP(("  MinFrmInt=%d  MaxFrmInt=%d MinBps=%d  MaxBps=%d\n",
					DataRangeVideoToVerify->ConfigCaps.MinFrameInterval,DataRangeVideoToVerify->ConfigCaps.MaxFrameInterval,
					DataRangeVideoToVerify->ConfigCaps.MinBitsPerSecond,DataRangeVideoToVerify->ConfigCaps.MaxBitsPerSecond ));

			DUMP(("Src(%d,%d)-(%d,%d) Dst(%d,%d)-(%d,%d) BitRt=%d,BitER=%d,AvgTPF=%d\n",
					DataRangeVideoToVerify->VideoInfoHeader.rcSource.left,
					DataRangeVideoToVerify->VideoInfoHeader.rcSource.top,
					DataRangeVideoToVerify->VideoInfoHeader.rcSource.right,
					DataRangeVideoToVerify->VideoInfoHeader.rcSource.bottom,
					DataRangeVideoToVerify->VideoInfoHeader.rcTarget.left,
					DataRangeVideoToVerify->VideoInfoHeader.rcTarget.top,
					DataRangeVideoToVerify->VideoInfoHeader.rcTarget.right,
					DataRangeVideoToVerify->VideoInfoHeader.rcTarget.bottom,
					DataRangeVideoToVerify->VideoInfoHeader.dwBitRate,
					DataRangeVideoToVerify->VideoInfoHeader.dwBitErrorRate,
					DataRangeVideoToVerify->VideoInfoHeader.AvgTimePerFrame
			));
			DUMP(("biSz=%d,WxH=(%d,%d),Pln=%d,Bpp=%d,Cmp=%04x,ImgSz=%d,PPM(%d,%d),ClrUsd=%d,ClrImp=%d\n",
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biSize,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biWidth,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biHeight,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biPlanes,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biBitCount,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biCompression,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biSizeImage,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biXPelsPerMeter,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biYPelsPerMeter,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biClrUsed,
					DataRangeVideoToVerify->VideoInfoHeader.bmiHeader.biClrImportant
			));
			DUMP(("dwInterlaceFlags=%d (0x%08x)\n",DataRangeVideoToVerify->VideoInfoHeader.dwInterlaceFlags,DataRangeVideoToVerify->VideoInfoHeader.dwInterlaceFlags));

			DUMP(("With:\n"));
            DUMP(("  FixedSzSmp=%x  TempComp=%x  StramDescrFlgs=%x  MemAllocFlgs=%x\n", 
					DataRangeVideo->bFixedSizeSamples,
					DataRangeVideo->bTemporalCompression,
					DataRangeVideo->StreamDescriptionFlags,
					DataRangeVideo->MemoryAllocationFlags
            ));

			DUMP(("  VideoStd=%x  InSz=(%d,%d)  MinCropSz=(%d,%d)  MaxCropSz=(%d,%d)\n",
					DataRangeVideo->ConfigCaps.VideoStandard,
					DataRangeVideo->ConfigCaps.InputSize.cx,DataRangeVideo->ConfigCaps.InputSize.cy,
					DataRangeVideo->ConfigCaps.MinCroppingSize.cx,DataRangeVideo->ConfigCaps.MinCroppingSize.cy,
					DataRangeVideo->ConfigCaps.MaxCroppingSize.cx,DataRangeVideo->ConfigCaps.MaxCroppingSize.cy ));

			DUMP(("  CropGran=(%d,%d)  CropAlign=(%d,%d)  MinOutSz=(%d,%d)  MaxOutSz=(%d,%d)\n",
					DataRangeVideo->ConfigCaps.CropGranularityX,DataRangeVideo->ConfigCaps.CropGranularityY,
					DataRangeVideo->ConfigCaps.CropAlignX,DataRangeVideo->ConfigCaps.CropAlignY,
					DataRangeVideo->ConfigCaps.MinOutputSize.cx,DataRangeVideo->ConfigCaps.MinOutputSize.cy,
					DataRangeVideo->ConfigCaps.MaxOutputSize.cx,DataRangeVideo->ConfigCaps.MaxOutputSize.cy ));

			DUMP(("  OutGran=(%d,%d)  StretchTaps=(%d,%d)  ShrinkTaps=(%d,%d)\n",
					DataRangeVideo->ConfigCaps.OutputGranularityX,DataRangeVideo->ConfigCaps.OutputGranularityY,
					DataRangeVideo->ConfigCaps.StretchTapsX,DataRangeVideo->ConfigCaps.StretchTapsY,
					DataRangeVideo->ConfigCaps.ShrinkTapsX,DataRangeVideo->ConfigCaps.ShrinkTapsY ));
			
			DUMP(("  MinFrmInt=%d  MaxFrmInt=%d MinBps=%d  MaxBps=%d\n",
					DataRangeVideo->ConfigCaps.MinFrameInterval,DataRangeVideo->ConfigCaps.MaxFrameInterval,
					DataRangeVideo->ConfigCaps.MinBitsPerSecond,DataRangeVideo->ConfigCaps.MaxBitsPerSecond ));


			DUMP(("Src(%d,%d)-(%d,%d) Dst(%d,%d)-(%d,%d) BitRt=%d,BitER=%d,AvgTPF=%d\n",
					DataRangeVideo->VideoInfoHeader.rcSource.left,
					DataRangeVideo->VideoInfoHeader.rcSource.top,
					DataRangeVideo->VideoInfoHeader.rcSource.right,
					DataRangeVideo->VideoInfoHeader.rcSource.bottom,
					DataRangeVideo->VideoInfoHeader.rcTarget.left,
					DataRangeVideo->VideoInfoHeader.rcTarget.top,
					DataRangeVideo->VideoInfoHeader.rcTarget.right,
					DataRangeVideo->VideoInfoHeader.rcTarget.bottom,
					DataRangeVideo->VideoInfoHeader.dwBitRate,
					DataRangeVideo->VideoInfoHeader.dwBitErrorRate,
					DataRangeVideo->VideoInfoHeader.AvgTimePerFrame
			));
			DUMP(("biSz=%d,WxH=(%d,%d),Pln=%d,Bpp=%d,Cmp=%04x,ImgSz=%d,PPM(%d,%d),ClrUsd=%d,ClrImp=%d\n",
					DataRangeVideo->VideoInfoHeader.bmiHeader.biSize,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biWidth,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biHeight,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biPlanes,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biBitCount,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biCompression,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biSizeImage,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biXPelsPerMeter,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biYPelsPerMeter,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biClrUsed,
					DataRangeVideo->VideoInfoHeader.bmiHeader.biClrImportant
			));

			DUMP(("dwInterlaceFlags=%d (0x%08x)\n",DataRangeVideo->VideoInfoHeader.dwInterlaceFlags,DataRangeVideo->VideoInfoHeader.dwInterlaceFlags));
	    //
        // KS_DATAFORMAT_VIDEOINFOHEADER:
        //    fields marked with 'm' must match; 
        //           marked with 'r' must within range;
        //           marked with 'f' is filled by us
        //
        //     KSDATAFORMAT == KSDATARANGE
        //       m ULONG   FormatSize;
        //         ULONG   Flags;
        //         ULONG   SampleSize;
        //         ULONG   Reserved;
        //       m GUID    MajorFormat;
        //       m GUID    SubFormat;
        //       m GUID    Specifier;.
        //  m  BOOL                         bFixedSizeSamples;      // all samples same size?
        //  m  BOOL                         bTemporalCompression;   // all I frames?
        //  m  DWORD                        StreamDescriptionFlags; // KS_VIDEO_DESC_*
        //  m  DWORD                        MemoryAllocationFlags;  // KS_VIDEO_ALLOC_*
        //  m  KS_VIDEO_STREAM_CONFIG_CAPS  ConfigCaps;
        //     KS_VIDEOINFOHEADER 
        //         RECT                rcSource;          // The bit we really want to use
        //         RECT                rcTarget;          // Where the video should go
        //     f   DWORD               dwBitRate;         // Approximate bit data rate
        //     f   DWORD               dwBitErrorRate;    // Bit error rate for this stream
        //     r/f REFERENCE_TIME      AvgTimePerFrame;   // Average time per frame (100ns units)
		//     m   DWORD               dwInterlaceFlags;   // use AMINTERLACE_* defines. Reject connection if undefined bits are not 0
		//         DWORD               dwCopyProtectFlags; // use AMCOPYPROTECT_* defines. Reject connection if undefined bits are not 0
		//         DWORD               dwPictAspectRatioX; // X dimension of picture aspect ratio, e.g. 16 for 16x9 display
		//         DWORD               dwPictAspectRatioY; // Y dimension of picture aspect ratio, e.g.  9 for 16x9 display
		//         DWORD               dwReserved1;        // must be 0; reject connection otherwise
		//         DWORD               dwReserved2;        // must be 0; reject connection otherwise
        //         KS_BITMAPINFOHEADER bmiHeader;
        //       f     DWORD      biSize;
        //       r/f   LONG       biWidth;
        //       r/f   LONG       biHeight;
        //       f     WORD       biPlanes;
        //       f     WORD       biBitCount;
        //       f     DWORD      biCompression;
        //       f     DWORD      biSizeImage;
        //       f     LONG       biXPelsPerMeter;
        //       f     LONG       biYPelsPerMeter;
        //       f     DWORD      biClrUsed;
        //       f     DWORD      biClrImportant;
        //     

			// Only accept formats we can handle right now....

			/* CAN'T BE DONE, OR DSHOW runtime will bomb!
			if (DataRangeVideo->ConfigCaps.VideoStandard && 
				(DataRangeVideo->ConfigCaps.VideoStandard & currVideoStandard) == 0) {
				DUMP(("Invalid format for the current video std\n"));
				continue;
			}*/

			//
			// Check that the other fields match
			//
			if ( (DataRangeVideoToVerify->bFixedSizeSamples      != DataRangeVideo->bFixedSizeSamples)      ||
				(DataRangeVideoToVerify->bTemporalCompression   != DataRangeVideo->bTemporalCompression)   ||
				(DataRangeVideoToVerify->StreamDescriptionFlags != DataRangeVideo->StreamDescriptionFlags) ||
				(DataRangeVideoToVerify->MemoryAllocationFlags  != DataRangeVideo->MemoryAllocationFlags)  ||
				// Make sure we take into account the field order required, without paying attention to the Display order...
				((DataRangeVideoToVerify->VideoInfoHeader.dwInterlaceFlags & (~KS_INTERLACE_DisplayModeMask)) 
					!= (DataRangeVideo->VideoInfoHeader.dwInterlaceFlags & (~KS_INTERLACE_DisplayModeMask)) ) || 
				(!RtlEqualMemory( &DataRangeVideoToVerify->ConfigCaps, &DataRangeVideo->ConfigCaps, sizeof( KS_VIDEO_STREAM_CONFIG_CAPS ) )) 
			   ) 
			{
				DUMP(( "At least one field does not match\n" ));
				continue;
			}


			DUMP(( "Match Found!\n" ));

			// MATCH FOUND!
			FormatSize = sizeof( KSDATAFORMAT ) + GetVideoInfoFormatSize(DataRangeVideo->VideoInfoHeader);

			if ( OnlyWantsSize ) {
	            break;
			}

			// Caller wants the full data format
			if ( IntersectInfo->SizeOfDataFormatBuffer < FormatSize ) 
			{
	            DUMP(("STATUS_BUFFER_TOO_SMALL\n" ));
				return STATUS_BUFFER_TOO_SMALL;
			}

			// Copy over the KSDATAFORMAT, followed by the
			// actual VideoInfoHeader
			PKS_DATAFORMAT_VIDEOINFOHEADER2 InterVidHdr =
	            (PKS_DATAFORMAT_VIDEOINFOHEADER2)IntersectInfo->DataFormatBuffer;

			// Copy the GUID of ColorFormat
	        RtlCopyMemory( 
				&InterVidHdr->DataFormat,
				&DataRangeVideo->DataRange, 
				sizeof( KSDATARANGE ) );

			// Set the format size
			InterVidHdr->DataFormat.FormatSize = FormatSize;

			// AvgTimePerFrame (FrameRate) need to be within (ConfigCaps.MinFrameInterval, ConfigCaps.MaxFrameInterval)
			LONGLONG AvgTimePerFrame = DataRangeVideoToVerify->VideoInfoHeader.AvgTimePerFrame;
			if (AvgTimePerFrame > DataRangeVideoToVerify->ConfigCaps.MaxFrameInterval ||      
				AvgTimePerFrame < DataRangeVideoToVerify->ConfigCaps.MinFrameInterval) {

				AvgTimePerFrame = DataRangeVideo->VideoInfoHeader.AvgTimePerFrame;
				DUMP(("AvgTimePerFrame out of range; so set it to default (%ld)\n",AvgTimePerFrame));

			}

			// Copy to destination to original VIDEOINFOHEADER2 (will we accepted)!
			// Unfortunately, those stupid people at M$ didnt pass all the required
			// info in the DataRangeVideoToVerify->VideoInfoHeader, so we must use our
			// template to do this!
			RtlCopyMemory(
				&InterVidHdr->VideoInfoHeader2,
				&DataRangeVideoToVerify->VideoInfoHeader,
				GetVideoInfoFormatSize(DataRangeVideo->VideoInfoHeader));
        
			// report back the image size as we know it

			// Get a copy of the VideoInfoHeader ... Only access provided fields!
			KS_VIDEOINFOHEADER2_WITHMASK vidHDR;
			RtlCopyMemory(
				&vidHDR,
				&DataRangeVideoToVerify->VideoInfoHeader,
				GetVideoInfoFormatSizeWithoutPalette(DataRangeVideo->VideoInfoHeader) );

			// Pass the requested framerate...
			InterVidHdr->VideoInfoHeader2.AvgTimePerFrame = AvgTimePerFrame;

			// Now we must fill-in all the missing fields (Microsoft Opus) : Simply keep width and Height, 
			// copy all the members and restore them
			DWORD biWidth = InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth;
			DWORD biHeight = InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight;
			RtlCopyMemory(
				&InterVidHdr->VideoInfoHeader2.bmiHeader,
				&DataRangeVideo->VideoInfoHeader.bmiHeader,
				GetBitmapFormatSize(DataRangeVideo->VideoInfoHeader.bmiHeader) );
			InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth = biWidth;
			InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight = biHeight;

			/************************************************************** 
            *  HOW BIG IS THE IMAGE REQUESTED (pseudocode follows)
            *
            *  if (IsRectEmpty (&rcTarget) {
            *      SetRect (&rcImage, 0, 0, 
            *              BITMAPINFOHEADER.biWidth,
            *              BITMAPINFOHEADER.biHeight);
            *  }
            *  else {
            *      // Probably rendering to a DirectDraw surface,
            *      // where biWidth is used to expressed the "stride" 
            *      // in units of pixels (not bytes) of the destination surface.
            *      // Therefore, use rcTarget to get the actual image size 
            *      
            *      rcImage = rcTarget;
            *  }
            **************************************************************/
        
            if ((DataRangeVideoToVerify->VideoInfoHeader.rcTarget.right - 
                 DataRangeVideoToVerify->VideoInfoHeader.rcTarget.left <= 0) ||
                (DataRangeVideoToVerify->VideoInfoHeader.rcTarget.bottom - 
                 DataRangeVideoToVerify->VideoInfoHeader.rcTarget.top <= 0)) {
            
				// Image size can be found on the BITMAPINFOHEADER... bound and fix it.
				if (DataRangeVideo->ConfigCaps.OutputGranularityX) {
					InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth &= (-DataRangeVideo->ConfigCaps.OutputGranularityX);
				}
				if (DataRangeVideo->ConfigCaps.OutputGranularityY) {
					InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight &= (-DataRangeVideo->ConfigCaps.OutputGranularityY);
				}
				if (abs(InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth) < DataRangeVideo->ConfigCaps.MinOutputSize.cx) {
					InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth =
						((InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth < 0) ? -1 : 1) *
						DataRangeVideo->ConfigCaps.MinOutputSize.cx;
				}
				if (abs(InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth) > DataRangeVideo->ConfigCaps.MaxOutputSize.cx) {
					InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth = 
						((InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth < 0) ? -1 : 1) *
						DataRangeVideo->ConfigCaps.MaxOutputSize.cx;
				}
				if (abs(InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight) < DataRangeVideo->ConfigCaps.MinOutputSize.cy) {
					InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight = 
						((InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight < 0) ? -1 : 1) *
						DataRangeVideo->ConfigCaps.MinOutputSize.cy;
				}
				if (abs(InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight) > DataRangeVideo->ConfigCaps.MaxOutputSize.cy) {
					InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight = 
						((InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight < 0) ? -1 : 1) *
						DataRangeVideo->ConfigCaps.MaxOutputSize.cy;
				}
				vidHDR.bmiHeader.biWidth = InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth;
				vidHDR.bmiHeader.biHeight = InterVidHdr->VideoInfoHeader2.bmiHeader.biHeight;
			} else {
				// We are dealing with a DirectDraw Buffer... Fix what we need...
				// The BITMAPINFOHEADER width must be a multiple of 4....
				// The RECT must comply with all restrictions.... Left & right must be a multiple of OutpoutGranularity. (bt8x8 limitation)


				vidHDR.bmiHeader.biWidth &= (-4);
				InterVidHdr->VideoInfoHeader2.bmiHeader.biWidth &= (-4);

				if (DataRangeVideo->ConfigCaps.OutputGranularityX) {
					InterVidHdr->VideoInfoHeader2.rcTarget.left &= (-DataRangeVideo->ConfigCaps.OutputGranularityX);
					InterVidHdr->VideoInfoHeader2.rcTarget.right &= (-DataRangeVideo->ConfigCaps.OutputGranularityX);
				}
				if (DataRangeVideo->ConfigCaps.OutputGranularityY) {
					InterVidHdr->VideoInfoHeader2.rcTarget.top &= (-DataRangeVideo->ConfigCaps.OutputGranularityY);
					InterVidHdr->VideoInfoHeader2.rcTarget.bottom &= (-DataRangeVideo->ConfigCaps.OutputGranularityY);
				}

				if ((InterVidHdr->VideoInfoHeader2.rcTarget.right - InterVidHdr->VideoInfoHeader2.rcTarget.left) < 
						DataRangeVideo->ConfigCaps.MinOutputSize.cx) {
					InterVidHdr->VideoInfoHeader2.rcTarget.right = InterVidHdr->VideoInfoHeader2.rcTarget.left + DataRangeVideo->ConfigCaps.MinOutputSize.cx;
				}
				if ((InterVidHdr->VideoInfoHeader2.rcTarget.right - InterVidHdr->VideoInfoHeader2.rcTarget.left) > 
						DataRangeVideo->ConfigCaps.MaxOutputSize.cx) {
					InterVidHdr->VideoInfoHeader2.rcTarget.right = InterVidHdr->VideoInfoHeader2.rcTarget.left + DataRangeVideo->ConfigCaps.MaxOutputSize.cx;
				}

				if ((InterVidHdr->VideoInfoHeader2.rcTarget.bottom - InterVidHdr->VideoInfoHeader2.rcTarget.top) < 
						DataRangeVideo->ConfigCaps.MinOutputSize.cy) {
					InterVidHdr->VideoInfoHeader2.rcTarget.bottom = InterVidHdr->VideoInfoHeader2.rcTarget.top + DataRangeVideo->ConfigCaps.MinOutputSize.cy;
				}
				if ((InterVidHdr->VideoInfoHeader2.rcTarget.bottom - InterVidHdr->VideoInfoHeader2.rcTarget.top) > 
						DataRangeVideo->ConfigCaps.MaxOutputSize.cy) {
					InterVidHdr->VideoInfoHeader2.rcTarget.bottom = InterVidHdr->VideoInfoHeader2.rcTarget.top + DataRangeVideo->ConfigCaps.MaxOutputSize.cy;
				}

				vidHDR.rcTarget = InterVidHdr->VideoInfoHeader2.rcTarget;
            }

			// Normalize Frame
			ColFmt col;
			NormalizeFrame(vidHDR,col);

			// make sure the dimensions are acceptable
			ColorSpace  tmpCol( DataRange->SubFormat );

			// Get the destination Rect
			MRect dst( vidHDR.rcTarget );
			MSize size = dst.Size();

			// If color is correct ...
			if ( tmpCol.IsValid() && tmpCol.CheckDimentions( size ) &&
				tmpCol.CheckLeftTop( dst.TopLeft() ) ) 
			{

			 
				// Calculate biSizeImage for this request, and put the result in both
				// the biSizeImage field of the bmiHeader AND in the SampleSize field
				// of the DataFormat.
				//
				// Note that for compressed sizes, this calculation will probably not
				// be just width * height * bitdepth

				// if width is different, use it ( in bytes ) to calculate the size
				InterVidHdr->VideoInfoHeader2.bmiHeader.biSizeImage =
				InterVidHdr->DataFormat.SampleSize = (DWORD) KS_DIBSIZE(vidHDR.bmiHeader);

				// Fill in the produced bitrate...
				InterVidHdr->VideoInfoHeader2.dwBitRate = (DWORD)(
					(InterVidHdr->DataFormat.SampleSize * 80000000i64) / 
					((AvgTimePerFrame) ? AvgTimePerFrame : 10000000i64));
				InterVidHdr->VideoInfoHeader2.dwBitErrorRate = 0;

				DUMP(("InterVidHdr->VideoInfoHeader2.bmiHeader.biSizeImage = %d\n", InterVidHdr->VideoInfoHeader2.bmiHeader.biSizeImage));
				break;
			} 
			else 
			{
				// The requested cropping/alignment is not supported, continue search!
	            ERROR(("Invalid parameters (cropping/slignment)\n"));
				continue;
			}

		} // End of VIDEOINFOHEADER2 specifier

		// -------------------------------------------------------------------
		// No MATCH ! - Means that the requested format is NOT supported
		// -------------------------------------------------------------------

		else 
		{
			ERROR(("STATUS_NO_MATCH\n" ));
			return STATUS_NO_MATCH;
		}

	} // End of loop on all formats for this stream

	if (FormatSize == 0) {
		ERROR(("STATUS_NO_MATCH (no format matched)\n" ));
		return STATUS_NO_MATCH;
	}

	if ( OnlyWantsSize ) {

        // Is the return buffer size = 0 ?
        if(IntersectInfo->SizeOfDataFormatBuffer == 0) {
			DUMP(("Proxy wants to know the required buffer size\n"));
            // the proxy wants to know the actuall buffer size to allocate.
            pSrb->ActualBytesTransferred = FormatSize;
            return STATUS_BUFFER_OVERFLOW;
        }

	    DUMP(("only wants size\n" ));
		*(PULONG) IntersectInfo->DataFormatBuffer = FormatSize;
		pSrb->ActualBytesTransferred = sizeof( ULONG );
		return STATUS_SUCCESS;
	}

	pSrb->ActualBytesTransferred = FormatSize;
	DUMP(("done\n" ));

	return STATUS_SUCCESS;
}

ULONG BtVideoStream::GetVideoControlCaps( void )
{
	TRACE(("BtVideoStream::GetVideoControlCaps"));

	// All videoStreams could support vertical inversion
	return 0/*KS_VideoControlFlag_FlipVertical*/;
}

ULONG BtVideoStream::GetVideoControlMode( void )
{
	TRACE(("BtVideoStream::GetVideoControlMode"));
	return 0;
}

NTSTATUS BtVideoStream::SetVideoControlMode( ULONG mode )
{
	TRACE(("BtVideoStream::SetVideoControlMode"));
	return STATUS_NOT_SUPPORTED;
}

LONGLONG BtVideoStream::GetActualFramerateInNs( void )
{
	TRACE(("BtVBIStream::GetActualFramerateInNs"));

	// Get requested time between samples
	REFERENCE_TIME TimeBetweenSamples;
	if (bIsVideoInfo2) {
		TimeBetweenSamples = VidHeader2.AvgTimePerFrame;
	} else {
		TimeBetweenSamples = VidHeader.AvgTimePerFrame;
	}
	return TimeBetweenSamples;
}

LONGLONG BtVideoStream::GetMaxAvailableFramerateInNs( void )
{
	TRACE(("BtVBIStream::GetMaxAvailableFramerateInNs"));

	// Get Time between Hardware taken samples ...
	REFERENCE_TIME TimeBetweenHwSampling = 
		(digitizer->GetBtCard().TimeBetweenFields() << ((bFieldTimeBetweenSamples) ? 0 : 1) );

	return TimeBetweenHwSampling;
}

NTSTATUS BtVideoStream::GetAnalogWindow( RECT& rc )
{
	TRACE(( "BtVideoStream::GetAnalogWindow" ));

	// Return the forced analog window if ther is one...
	rc = rcCropping;

	return STATUS_SUCCESS;
}

NTSTATUS BtVideoStream::SetAnalogWindow( const RECT& rc )
{
	TRACE(( "BtVideoStream::SetAnalogWindow" ));

	// Remember the AnalogWindow to use...
	rcCropping = rc;

	// Set the analog window if channel is already created
	if (vc) {

		// If source cropping area not provided, use the maximum!
		MRect rcSrc = rcCropping;
		if (rcCropping.IsNull()) { 
			// Get maximum dimensions for Analog Window
			digitizer->GetBtCard().VideoGetMaxFrame(rcSrc);
		}
		
		// Set the analog window
		vc->SetAnalogWindow(rcSrc);
	}
	return STATUS_SUCCESS;
}

NTSTATUS BtVideoStream::GetDigitalWindow( RECT& rc )
{
	TRACE(( "BtVideoStream::GetDigitalWindow" ));
	
	// If channel already created, get data from channel; else get data from user provided data
	if (vc) {
		rc = vc->GetDigitalWindow();
	} else {

		if (!bIsVideoInfo2) {

			ColFmt col;
			NormalizeFrame(VidHeader,col);
			rc = VidHeader.rcTarget;

		} else {

			ColFmt col;
			NormalizeFrame(VidHeader2,col);
			rc = VidHeader2.rcTarget;
		}
	}
	return STATUS_SUCCESS;
}

void BtVideoStream::RescaleCropping() // Invalidates the Cropping region
{
	TRACE(("BtVideoStream::RescaleCropping"));

	// Just prepare things - Make sure no cropping area is used!
	RECT empty = {0,0,0,0};
	SetAnalogWindow(empty);
}
