/*
	BtVideoStream.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef BTVIDEOSTREAM_HP_
#define BTVIDEOSTREAM_HP_

// #define ENABLE_TIMESTAMP_FIX /* Can fix timestamp jitter... But seems to cause some audio sync problems... DISABLED BY NOW */

#include "BtDigitizerStream.hpp"
#include "VideoFormats.hpp"

// This class represents the common functionality of all Video Streams (neither VBI, not AnalogVideo)
class BtVideoStream : 
	public BtDigitizerStream,		// We are a Video capture Stream
	public VideoChanIface			// And we are also a DataBuffer provider and consumer
{
	protected:

		// Formato utilizado actualmente por el Stream
		BOOL						 bIsVideoInfo2;
		KS_VIDEOINFOHEADER_WITHMASK	 VidHeader;
		KS_VIDEOINFOHEADER2_WITHMASK VidHeader2;

		// Frame info (Stats)
		KS_FRAME_INFO				 FrameInfo;	// Frame info

		// Amount of time remaining to sample the next frame at the current FPS
		REFERENCE_TIME				 TimeToNextFrame;

#ifdef ENABLE_TIMESTAMP_FIX
		// Sampling time of the last captured sample...
		LONGLONG					 LastSamplingTime;
#endif

		// Stream Mode
		BOOL						 bFieldBased;				// If TRUE, we are capturing fields else frames
		BOOL						 bFieldTimeBetweenSamples;	// If true, time between samples is time between fields!


		HW_STREAM_OBJECT&			 VStreamObject;				// StreamObject

		MRect						 rcCropping;				// Cropping rect to use

	public:

		BtVideoStream(BtDigitizer *dev,HW_STREAM_OBJECT& so);
		virtual ~BtVideoStream();

		// Reset Statistic counters
		virtual void ResetCounters();
		virtual void IncPictureNumber();
		virtual void IncDropCount();

		// Events on channels
		virtual NTSTATUS Create() = 0;
		virtual NTSTATUS Destroy() = 0;

	protected:

		// Function called to Timestamp Buffers
		virtual void TimeStamp( PKSSTREAM_HEADER pHeader , PHW_STREAM_REQUEST_BLOCK pSrb) = 0;

	public:
		virtual NTSTATUS Open( IN PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual HW_STREAM_INFORMATION& Info( void ) = 0; 
		virtual ULONG GetVideoControlCaps( void ) ;
		virtual ULONG GetVideoControlMode( void ) ;
		virtual NTSTATUS SetVideoControlMode( ULONG mode ) ;
		virtual LONGLONG GetActualFramerateInNs( void ) ;
		virtual LONGLONG GetMaxAvailableFramerateInNs( void ) ;
		virtual NTSTATUS GetAnalogWindow( RECT& rc );
		virtual NTSTATUS SetAnalogWindow( const RECT& rc );
		virtual NTSTATUS GetDigitalWindow( RECT& rc );

		// Called from BtDigitzer to inform Analog Video Format has changed and that Cropping needs to be rescaled
		void    RescaleCropping();

	protected:
		// Called when user program supplies a buffer to fill in		
		virtual NTSTATUS SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb );

		// Functions
		virtual NTSTATUS GetConnectionProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetAllocatorProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetDroppedFramesProperty( PHW_STREAM_REQUEST_BLOCK pSrb ); // Here!

		virtual BOOL VerifyFormat( PKSDATAFORMAT pKSDataFormatToVerify );

	protected:

		// Convenience fns
		DWORD NormalizeFrame( KS_VIDEOINFOHEADER_WITHMASK &VidHdr , ColFmt &col); 
		DWORD NormalizeFrame( KS_VIDEOINFOHEADER2_WITHMASK &VidHdr , ColFmt &col); 
		bool RectEmpty(const RECT *lprc) { return (lprc->bottom == 0 && lprc->left == 0 && lprc->right == 0 && lprc->top == 0); } 

		// To get the required size to store the struct and its associated info.
		static DWORD GetBitmapFormatSize(const KS_BITMAPINFOHEADER& pHeader);
		static DWORD GetBitmapFormatSizeWithoutPalette(const KS_BITMAPINFOHEADER& pHeader);

		// To get the required size to store the struct and its associated info.
		DWORD GetVideoInfoFormatSize(const KS_VIDEOINFOHEADER& vh) { return GetBitmapFormatSize(vh.bmiHeader) + KS_SIZE_PREHEADER; }
		DWORD GetVideoInfoFormatSize(const KS_VIDEOINFOHEADER2& vh) { return GetBitmapFormatSize(vh.bmiHeader) + KS_SIZE_PREHEADER2; }
		DWORD GetVideoInfoFormatSizeWithoutPalette(const KS_VIDEOINFOHEADER& vh) { return GetBitmapFormatSizeWithoutPalette(vh.bmiHeader) + KS_SIZE_PREHEADER; }
		DWORD GetVideoInfoFormatSizeWithoutPalette(const KS_VIDEOINFOHEADER2& vh) { return GetBitmapFormatSizeWithoutPalette(vh.bmiHeader) + KS_SIZE_PREHEADER2; }

	public:

		// VideoChanIface : Xface to VideoChannel
		virtual const VideoDataBuf StreamGetNextBuffer();
		virtual void StreamBufferDone( const VideoDataBuf& pBuf , const VidField vf ); 

	protected:

		// Called whenever we could use this channel to sample for another channel
		virtual const VideoDataBuf OnSampling(bool GetSample);
};
 

#endif

