/*
	DDraw.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "Debug.hpp"
#include "DDraw.hpp"

#ifdef ENABLE_DDRAW_STUFF 

/*****************************Exported*Routine*****************************\
* BOOL DirectDraw::cbEvent
*
* Purpose:  Handles messages from DirectDraw
*
* Input:    dwEvent  - message to handle
*           pContext - our context pointer, we use our stream extension
*           dwParam1 - probably depends on the message (dwEvent)
*           dwParam2 - probably depends on the message (dwEvent)
* Output:   TRUE if success, FALSE otherwise
\**************************************************************************/

ULONG FAR PASCAL DirectDraw::cbEvent ( 
   DWORD dwEvent,
   PVOID pContext,
   DWORD dwParam1,
   DWORD dwParam2
   )
{
	TRACE(("DirectDraw::cbEvent"));
	DirectDraw* DDraw = (DirectDraw*)pContext;
	return DDraw->Event(dwEvent,dwParam1,dwParam2);
}


ULONG DirectDraw::Event ( 
   DWORD dwEvent,
   DWORD dwParam1,
   DWORD dwParam2
   )
{
	TRACE(("DirectDraw::Event"));
	
	switch( dwEvent ) {

		case DDNOTIFY_PRERESCHANGE: {
				DUMP(( "DDNOTIFY_PRERESCHANGE\n" ));
				// Release all surfaces and close DirectDraw.
				ReleaseAllSurfaces();
			}
			break;
		case DDNOTIFY_POSTRESCHANGE: {
				DUMP(( "DDNOTIFY_POSTRESCHANGE\n" ));
				DUMP(( "before Attempted Renegotiation due to DDNOTIFY_POSTRESCHANGE\n" ));
				//AttemptRenegotiation(); // Will automatically try to do it.
				DUMP(( "after Attempted Renegotiation due to DDNOTIFY_POSTRESCHANGE\n" ));
			}
			break;
		case DDNOTIFY_PREDOSBOX: {
				DUMP(( "DDNOTIFY_POSTRESCHANGE\n" ));
				// Release all surfaces and close DirectDraw.
				ReleaseAllSurfaces();
			}
			break;
		case DDNOTIFY_POSTDOSBOX: {
				DUMP(( "DDNOTIFY_POSTDOSBOX\n" ));
				DUMP(( "before Attempted Renegotiation due to DDNOTIFY_POSTDOSBOX\n" ));
				//AttemptRenegotiation(); // Will automatically retry it.
				DUMP(( "after Attempted Renegotiation due to DDNOTIFY_POSTDOSBOX\n" ));
			}
			break;
		case DDNOTIFY_CLOSEDIRECTDRAW: {
				DUMP(( "DDNOTIFY_CLOSEDIRECTDRAW\n" ));
				// Release all surfaces and close DirectDraw.
				ReleaseAllSurfaces();
			}
			break;
		case DDNOTIFY_CLOSESURFACE: {
				DUMP(( "DDNOTIFY_CLOSESURFACE\n" ));
				// Remove this surface from the Surface pool
				RemoveSurfaceFromPool((HANDLE)((DWORD_PTR)dwParam1));
			}
			break;
		default:
			ERROR(( "unknown/unhandled ddraw event\n" ));
			break;
	}
	return 0;
}

/******************************Public*Routine******************************\
* BOOL RegisterForEvents
*
* Purpose:  Set up our callbacks for kernel mode direct draw
*
* Input:    pSrb  - data SRB representing an overlay surface
* Output:   TRUE if success, FALSE otherwise
\**************************************************************************/

BOOL DirectDraw::RegisterForEvents ( void )
{
	TRACE(("DirectDraw::RegisterForEvents"));

	if (bKernelDirectDrawRegistered) {
		return TRUE;
	}

	DDREGISTERCALLBACK   ddRegisterCallback;
	DWORD                ddOut;

	// =============== DDEVENT_PRERESCHANGE ===============
	RtlZeroMemory( &ddRegisterCallback, sizeof(ddRegisterCallback) );
	RtlZeroMemory( &ddOut, sizeof(ddOut) );

	ddRegisterCallback.hDirectDraw	= hKernelDirectDrawHandle;
	ddRegisterCallback.dwEvents		= DDEVENT_PRERESCHANGE;
	ddRegisterCallback.pfnCallback	= cbEvent;
	ddRegisterCallback.pContext		= this;

	DxApi( DD_DXAPI_REGISTER_CALLBACK, (void*) &ddRegisterCallback, sizeof(ddRegisterCallback), (void*)&ddOut, sizeof(ddOut) );

	if( ddOut != DD_OK ) {
		ERROR(( "DD_DXAPI_REGISTER_CALLBACK failed.\n" ));
		return FALSE;
	}

	// =============== DDEVENT_POSTRESCHANGE ==============
	RtlZeroMemory( &ddRegisterCallback, sizeof(ddRegisterCallback) );
	RtlZeroMemory( &ddOut, sizeof(ddOut) );

	ddRegisterCallback.hDirectDraw	= hKernelDirectDrawHandle;
	ddRegisterCallback.dwEvents		= DDEVENT_POSTRESCHANGE;
	ddRegisterCallback.pfnCallback	= cbEvent;
	ddRegisterCallback.pContext		= this;

	DxApi(DD_DXAPI_REGISTER_CALLBACK, (void*) &ddRegisterCallback, sizeof(ddRegisterCallback), (void*)&ddOut, sizeof(ddOut) );

	if( ddOut != DD_OK ) {
		ERROR(( "DD_DXAPI_REGISTER_CALLBACK failed.\n" ));
		return FALSE;
	}

	// =============== DDEVENT_PREDOSBOX =================
	RtlZeroMemory( &ddRegisterCallback, sizeof(ddRegisterCallback) );
	RtlZeroMemory( &ddOut, sizeof(ddOut) );

	ddRegisterCallback.hDirectDraw	= hKernelDirectDrawHandle;
	ddRegisterCallback.dwEvents		= DDEVENT_PREDOSBOX;
	ddRegisterCallback.pfnCallback	= cbEvent;
	ddRegisterCallback.pContext		= this;

	DxApi( DD_DXAPI_REGISTER_CALLBACK, (void*) &ddRegisterCallback, sizeof(ddRegisterCallback), (void*)&ddOut, sizeof(ddOut) );

	if( ddOut != DD_OK ) {
		ERROR(( "DD_DXAPI_REGISTER_CALLBACK failed.\n" ));
		return FALSE;
	}

	// =============== DDEVENT_POSTDOSBOX ================
	RtlZeroMemory( &ddRegisterCallback, sizeof(ddRegisterCallback) );
	RtlZeroMemory( &ddOut, sizeof(ddOut) );

	ddRegisterCallback.hDirectDraw	= hKernelDirectDrawHandle;
	ddRegisterCallback.dwEvents		= DDEVENT_POSTDOSBOX;
	ddRegisterCallback.pfnCallback	= cbEvent;
	ddRegisterCallback.pContext		= this;

	DxApi( DD_DXAPI_REGISTER_CALLBACK, (void*) &ddRegisterCallback, sizeof(ddRegisterCallback), (void*)&ddOut, sizeof(ddOut) );

	if( ddOut != DD_OK ) {
		ERROR(( "DD_DXAPI_REGISTER_CALLBACK failed.\n" ));
		return FALSE;
	}

	bKernelDirectDrawRegistered = TRUE;

	return TRUE;
}

/******************************Public*Routine******************************\
* BOOL UnregisterForDirectDrawEvents
*
* Purpose:  Extract our callbacks from kernel mode direct draw
*
* Input:    pSrb  - data SRB representing an overlay surface
* Output:   TRUE if success, FALSE otherwise
\**************************************************************************/

BOOL DirectDraw::UnregisterForEvents ( void )
{
	TRACE(("DirectDraw::UnregisterForEvents"));

	if (!bKernelDirectDrawRegistered) {
		return TRUE;
	}

	DDREGISTERCALLBACK   ddRegisterCallback;
	DWORD                ddOut;

	// =============== DDEVENT_PRERESCHANGE ===============
	RtlZeroMemory( &ddRegisterCallback, sizeof(ddRegisterCallback) );
	RtlZeroMemory( &ddOut, sizeof(ddOut) );

	ddRegisterCallback.hDirectDraw	= hKernelDirectDrawHandle;
	ddRegisterCallback.dwEvents		= DDEVENT_PRERESCHANGE;
	ddRegisterCallback.pfnCallback	= cbEvent;
	ddRegisterCallback.pContext		= this;

	DxApi( DD_DXAPI_UNREGISTER_CALLBACK, (void*) &ddRegisterCallback, sizeof(ddRegisterCallback), (void*)&ddOut, sizeof(ddOut));

	if( ddOut != DD_OK ) {
		ERROR(( "DD_DXAPI_UNREGISTER_CALLBACK failed.\n" ));
		return FALSE;
	}

	// =============== DDEVENT_POSTRESCHANGE ==============
	RtlZeroMemory( &ddRegisterCallback, sizeof(ddRegisterCallback) );
	RtlZeroMemory( &ddOut, sizeof(ddOut) );

	ddRegisterCallback.hDirectDraw	= hKernelDirectDrawHandle;
	ddRegisterCallback.dwEvents		= DDEVENT_POSTRESCHANGE;
	ddRegisterCallback.pfnCallback	= cbEvent;
	ddRegisterCallback.pContext		= this;

	DxApi( DD_DXAPI_UNREGISTER_CALLBACK, (void*) &ddRegisterCallback, sizeof(ddRegisterCallback), (void*)&ddOut, sizeof(ddOut) );

	if( ddOut != DD_OK ) {
		ERROR(( "DD_DXAPI_UNREGISTER_CALLBACK failed.\n" ));
		return FALSE;
	}

	// =============== DDEVENT_PREDOSBOX ==================
	RtlZeroMemory( &ddRegisterCallback, sizeof(ddRegisterCallback) );
	RtlZeroMemory( &ddOut, sizeof(ddOut) );

	ddRegisterCallback.hDirectDraw	= hKernelDirectDrawHandle;
	ddRegisterCallback.dwEvents		= DDEVENT_PREDOSBOX;
	ddRegisterCallback.pfnCallback	= cbEvent;
	ddRegisterCallback.pContext		= this;

	DxApi( DD_DXAPI_UNREGISTER_CALLBACK, (void*) &ddRegisterCallback, sizeof(ddRegisterCallback), (void*)&ddOut, sizeof(ddOut) );

	if( ddOut != DD_OK ) {
		ERROR(( "DD_DXAPI_UNREGISTER_CALLBACK failed.\n" ));
		return FALSE;
	}

	// =============== DDEVENT_POSTDOSBOX =================
	RtlZeroMemory( &ddRegisterCallback, sizeof(ddRegisterCallback) );
	RtlZeroMemory( &ddOut, sizeof(ddOut) );

	ddRegisterCallback.hDirectDraw	= hKernelDirectDrawHandle;
	ddRegisterCallback.dwEvents		= DDEVENT_POSTDOSBOX;
	ddRegisterCallback.pfnCallback	= cbEvent;
	ddRegisterCallback.pContext		= this;

	DxApi( DD_DXAPI_UNREGISTER_CALLBACK, (void*) &ddRegisterCallback, sizeof(ddRegisterCallback), (void*)&ddOut, sizeof(ddOut) );

	if( ddOut != DD_OK ) {                                                      \
		ERROR(( "DD_DXAPI_UNREGISTER_CALLBACK failed.\n" ));
		return FALSE;
	}

	bKernelDirectDrawRegistered = FALSE;

	return TRUE;
}

/******************************Public*Routine******************************\
* BOOL OpenKernel
*
* Purpose:  Initialize kernel mode direct draw
*
* Input:    hUserDirectDrawHandle = Handle to user DirectDraw
* Output:   TRUE if success, FALSE otherwise
\**************************************************************************/

BOOL DirectDraw::OpenKernel ( HANDLE hUserDirectDrawHandle )
{
	TRACE(("DirectDraw::OpenKernel"));

	// If DirectDraw Unavailable, don't lose time trying to open it...
	// We can save tons of time with this check!
	if (bUnavailable) {
		return FALSE;
	}

	// Only open if not already open!
	if(hKernelDirectDrawHandle == 0 && hUserDirectDrawHandle != 0 ) {

		DDOPENDIRECTDRAWIN  ddOpenIn;
		DDOPENDIRECTDRAWOUT ddOpenOut;

		ENSURE( hKernelDirectDrawHandle == 0 );

		RtlZeroMemory( &ddOpenIn,	sizeof(ddOpenIn)	);
		RtlZeroMemory( &ddOpenOut,	sizeof(ddOpenOut)	);

		DUMP(( "hUserDirectDrawHandle: 0x%08X\n", hUserDirectDrawHandle ));

		ddOpenIn.dwDirectDrawHandle	= (ULONG_PTR) hUserDirectDrawHandle;
		ddOpenIn.pfnDirectDrawClose = cbEvent;
		ddOpenIn.pContext			= this;

		DxApi( DD_DXAPI_OPENDIRECTDRAW, &ddOpenIn , sizeof(ddOpenIn), &ddOpenOut, sizeof(ddOpenOut) );

		if( ddOpenOut.ddRVal != DD_OK ) {

			// Keep in mind if DirectDraw is unavailable...
			if (ddOpenOut.ddRVal == DXERR_UNSUPPORTED) {
				bUnavailable = TRUE;
			}
			
			ERROR(( "DD_DXAPI_OPENDIRECTDRAW failed:  0x%08x\n", ddOpenOut.ddRVal ));

		} else {

			hKernelDirectDrawHandle = ddOpenOut.hDirectDraw;
			bKernelDirectDrawRegistered = FALSE;

			// We had success, now register for events!
			return RegisterForEvents();

		}
	}
	return FALSE;
}
    
/******************************Public*Routine******************************\
* BOOL CloseKernelDirectDraw
*
* Purpose:  Shut down kernel mode direct draw
*
* Input:    
* Output:   TRUE if success, FALSE otherwise
\**************************************************************************/

BOOL DirectDraw::CloseKernel ( void )
{
	TRACE(("DirectDraw::CloseKernel"));   

	// Only close if already open!
	if( hKernelDirectDrawHandle != 0 ) {

		// Unregister all events
		UnregisterForEvents();

		DWORD          ddOut;
		DDCLOSEHANDLE  ddClose;
		ddClose.hHandle = hKernelDirectDrawHandle;

		DxApi( DD_DXAPI_CLOSEHANDLE, &ddClose, sizeof(ddClose), &ddOut, sizeof(ddOut) );

		hKernelDirectDrawHandle = 0;

		if( ddOut != DD_OK ) {

			ERROR(( "CloseKernelDirectDraw FAILED.\n" ));
			return FALSE;

		} else {

			return TRUE;
		}
	}
	return TRUE;
}

/******************************Public*Routine******************************\
* BOOL IsKernelLockAndFlipAvailable
*
* Purpose:  Determine whether or not video card supports page flipping
*
* Input:    
* Output:   TRUE if it's available, FALSE otherwise
\**************************************************************************/

BOOL DirectDraw::IsKernelLockAndFlipAvailable ( void )
{
	TRACE(("DirectDraw::IsKernelLockAndFlipAvailable"));

	if( hKernelDirectDrawHandle != 0 ) {

		DDGETKERNELCAPSOUT   ddGetKernelCapsOut;

		RtlZeroMemory( &ddGetKernelCapsOut, sizeof(ddGetKernelCapsOut) );

		DxApi( 
            DD_DXAPI_GETKERNELCAPS, 
            (void*) &hKernelDirectDrawHandle, 
            sizeof(hKernelDirectDrawHandle), 
            (void*) &ddGetKernelCapsOut, 
            sizeof(ddGetKernelCapsOut)
            );

		if( ddGetKernelCapsOut.ddRVal != DD_OK ) {

			ERROR(( "DDGETKERNELCAPSOUT failed.\n" ));

		} else {

			DUMP(( "KernelCaps = %08X\n", ddGetKernelCapsOut.dwCaps ));
			if( 
				( ddGetKernelCapsOut.dwCaps & DDKERNELCAPS_LOCK ) && 
				( ddGetKernelCapsOut.dwCaps & DDKERNELCAPS_FLIPOVERLAY ) )
			{
				return TRUE;
			}
			else
			{
	            return FALSE;
			}
		}
	}
	return FALSE;
}

/******************************Public*Routine******************************\
* BOOL OpenKernelDDrawSurfaceHandle
*
* Purpose:  Get a kernel DirectDraw surface handle from a data SRB
*
* Input:    pSrb  - data SRB representing an overlay surface
* Output:   HANDLE if success, NULL otherwise
\**************************************************************************/

HANDLE DirectDraw::OpenSurface ( HANDLE hUserSurfaceHandle )
{    
	TRACE(("DirectDraw::OpenSurface"));

	if( hKernelDirectDrawHandle != 0 && hUserSurfaceHandle != 0 ) {

		DDOPENSURFACEIN   ddOpenSurfaceIn;
		DDOPENSURFACEOUT  ddOpenSurfaceOut;

		RtlZeroMemory( &ddOpenSurfaceIn, sizeof(ddOpenSurfaceIn) );
		RtlZeroMemory( &ddOpenSurfaceOut, sizeof(ddOpenSurfaceOut) );

		ddOpenSurfaceIn.hDirectDraw		= hKernelDirectDrawHandle;
		ddOpenSurfaceIn.pfnSurfaceClose = cbEvent;
		ddOpenSurfaceIn.pContext		= this;
		ddOpenSurfaceIn.dwSurfaceHandle = (DWORD_PTR)hUserSurfaceHandle;

		DxApi( DD_DXAPI_OPENSURFACE, (void*)&ddOpenSurfaceIn, sizeof(ddOpenSurfaceIn), (void*)&ddOpenSurfaceOut, sizeof(ddOpenSurfaceOut) );

		if( ddOpenSurfaceOut.ddRVal != DD_OK ) {
			
			ERROR(( "DD_DXAPI_OPENSURFACE failed: 0x%08X\n", ddOpenSurfaceOut.ddRVal ));
			return NULL;

		}
		else 
		{

			DUMP(( "DD_DXAPI_OPENSURFACE: 0x%08X\n", ddOpenSurfaceOut.hSurface ));

			return HANDLE( ddOpenSurfaceOut.hSurface );

		}
	}
	return NULL;
}


/******************************Public*Routine******************************\
* BOOL CloseKernelDDrawSurfaceHandle
*
* Purpose:  Unmap a kernel DirectDraw surface handle 
*
* Input:    pSrb  - data SRB representing an overlay surface
* Output:   TRUE if success, FALSE otherwise
\**************************************************************************/

BOOL DirectDraw::CloseSurface ( HANDLE hKernelSurfaceHandle )
{
	TRACE(("DirectDraw::CloseSurface"));
	
	if( hKernelDirectDrawHandle != 0 && hKernelSurfaceHandle != 0 ) 
	{
		DWORD          ddOut;
		DDCLOSEHANDLE  ddClose;

		DUMP(( "DD_DXAPI_CLOSEHANDLE: 0x%08X\n", hKernelSurfaceHandle ));

		ddClose.hHandle = hKernelSurfaceHandle;

		DxApi( DD_DXAPI_CLOSEHANDLE, (void*)&ddClose, sizeof(ddClose), (void*) &ddOut, sizeof(ddOut) );

		if( ddOut != DD_OK ) 
		{
			ERROR(( "ReleaseKernelDDrawSurfaceHandle() FAILED.\n" ));
			return FALSE;
		}
		else 
		{
			return TRUE;
		}
	}	
	return FALSE;
}

/******************************Public*Routine******************************\
* BOOL FlipOverlay
*
* Purpose:  Do a page flip between two overlay surfaces
*
* Input:    hFromSurface   - handle of currently visible surface
*           hTotSurface    - handle of surface to make visible
* Output:   TRUE if success, FALSE otherwise
\**************************************************************************/

BOOL DirectDraw::FlipOverlay (
	HANDLE hToSurface,
	HANDLE hFromSurface
   )
{
	if (hKernelDirectDrawHandle != 0 ) {
		DDFLIPOVERLAY   ddFlipOverlay;
		DWORD			ddOut;

#if DBG
		LARGE_INTEGER     liSysTime;
		TIME_FIELDS       tf;

		KeQuerySystemTime( &liSysTime );
		RtlTimeToTimeFields( &liSysTime, &tf );

		DUMP(( "'FlipOverlay(): 0x%08X => 0x%08X   %02d:%02d:%02d:%02d\n", 
										hFromSurface, 
										hToSurface,
										tf.Hour,
										tf.Minute,
										tf.Second,
										tf.Milliseconds
                                    ));
#endif


		RtlZeroMemory( &ddFlipOverlay, sizeof(ddFlipOverlay) );
		ddFlipOverlay.hDirectDraw = hKernelDirectDrawHandle;
		ddFlipOverlay.hCurrentSurface = hFromSurface;
		ddFlipOverlay.hTargetSurface = hToSurface;
		ddFlipOverlay.dwFlags = 0;

		DxApi( DD_DXAPI_FLIP_OVERLAY, (void*)&ddFlipOverlay, sizeof(ddFlipOverlay), (void*)&ddOut, sizeof(ddOut) );

		if( ddOut != DD_OK ) 
		{
			DUMP(( "FlipOverlay() FAILED: 0x%08X.\n", ddOut ));
			DUMP(( "ddFlipOverlay.hDirectDraw = 0x%08X.\n", ddFlipOverlay.hDirectDraw ));
			DUMP(( "ddFlipOverlay.hCurrentSurface = 0x%08X.\n", ddFlipOverlay.hCurrentSurface ));
			DUMP(( "ddFlipOverlay.hTargetSurface = 0x%08X.\n", ddFlipOverlay.hTargetSurface ));
			DUMP(( "ddFlipOverlay.dwFlags = 0x%08X.\n", ddFlipOverlay.dwFlags ));
			return FALSE;
		}
		else 
		{
			DUMP(( "FlipOverlay() SUCCEEDED: 0x%08X.\n", ddOut ));
			return TRUE;
		}
	}
	return FALSE;
}




// This adds the given pSrb directdraw surface to the DirectDraw surface pool...
// Assumes DirectDraw is already open... Can be called ONLY at PASSIVE_LEVEL

BOOL DirectDraw::AddSurfaceToPool( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	// If Kernel DirectDraw is unusable, don't try to use it...
	if (bUnusable) {
		return FALSE;
	}

	// If reached maximum number of surfaces, don't accept anymore.
	if (ulSurfaceCount >= MaxSurfaces) {
		return FALSE;
	}

	// Try to get DirectDraw data from extended information...
	PKS_FRAME_INFO pFrameInfo = (PKS_FRAME_INFO)( pSrb->CommandData.DataBufferArray + 1 );
			
	// If no extended information, go out (no DirectDraw info)
	if ( pFrameInfo->ExtendedHeaderSize < sizeof(KS_FRAME_INFO) ) {
		return FALSE;
	}

	DUMP(( "pFrameInfo->dwFrameFlags = 0x%08X\n", pFrameInfo->dwFrameFlags ));
	DUMP(( "pFrameInfo->hDirectDraw = 0x%08X\n", pFrameInfo->hDirectDraw ));
	DUMP(( "pFrameInfo->hSurfaceHandle = 0x%08X\n", pFrameInfo->hSurfaceHandle ));
	DUMP(( "pFrameInfo->DirectDrawRect = { %d,%d,%d,%d }\n", pFrameInfo->DirectDrawRect.left, pFrameInfo->DirectDrawRect.right, pFrameInfo->DirectDrawRect.top, pFrameInfo->DirectDrawRect.bottom ));
	DUMP(( "pFrameInfo->Reserved1 = 0x%08X\n", pFrameInfo->Reserved1 ));
	DUMP(( "pFrameInfo->Reserved2 = 0x%08X\n", pFrameInfo->Reserved2 ));
	DUMP(( "pFrameInfo->Reserved3 = 0x%08X\n", pFrameInfo->Reserved3 ));
	DUMP(( "pFrameInfo->Reserved4 = 0x%08X\n", pFrameInfo->Reserved4 ));

	// If no User DirectDraw handle provided, go out.
	if (!pFrameInfo->hDirectDraw) {
		return FALSE;
	}
	
	// Try to get the Kernel Surface Handle
	if (!pFrameInfo->hSurfaceHandle) {
		return FALSE;
	}

	// If DirectDraw is not open, try to open it.
	if (!IsKernelOpen()) {

		// A DirectDraw Handle has been provided, Try to open Kernel DirectDraw
		if (OpenKernel(pFrameInfo->hDirectDraw)) {

			// No current surface!
			hCurrentSurface = NULL;

			// Check if Lock and Flip overlay is supported
			if (!IsKernelLockAndFlipAvailable()) {

				// Remember DirectDraw can't be used by us so we don't try this anymore.
				bUnusable = TRUE;

				// Not supported, close DirectDraw
				CloseKernel();

				// Return failure (As we can't use Kernel DirectDraw)
				return FALSE;
			}
		} else {

			// Can't open DirectDraw... Return failure (As we can't use Kernel DirectDraw)
			return FALSE;

		}
	}

	
	// Try to get the Kernel Surface Handle
	HANDLE hKernelSurface = OpenSurface(pFrameInfo->hSurfaceHandle);
	if (!hKernelSurface) {
		return FALSE;
	}

	// Lock Surface queue
	SurfaceLock.Acquire();

	// Add the mapping entry...
	sdSurface[ulSurfaceCount  ].hSurface = hKernelSurface;
	sdSurface[ulSurfaceCount++].pSrb = pSrb;

	// Unlock surface Queue
	SurfaceLock.Release();

	// Ok!
	return TRUE;
}

// This is called to remove a surface from the surface pool. Can be called ONLY at PASSIVE_LEVEL.
BOOL DirectDraw::RemoveSurfaceFromPool( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	// If DirectDraw is not open, just go out!
	if (!IsKernelOpen()) {
		return FALSE;
	}

	// Lock Surface queue
	SurfaceLock.Acquire();

	// Locate the surface descriptor...
	HANDLE hKernelSurface = NULL;
	for (ULONG Idx = 0; Idx < ulSurfaceCount; Idx++) {
		if (sdSurface[Idx].pSrb == pSrb) {
			// Remember the Surface Handle...
			hKernelSurface = sdSurface[Idx].hSurface;
			// And remove the entry...
			memmove(sdSurface + Idx,sdSurface + Idx + 1,sizeof(SURF_DESC) * (ulSurfaceCount - Idx - 1));
			// We have one less entry available...
			ulSurfaceCount--;
			break;
		}
	}

	// Unlock surface Queue
	SurfaceLock.Release();

	// Close surface if we are done.
	if (hKernelSurface) {

		// Close surface
		CloseSurface(hKernelSurface);

		// Remember there's no current surface if we were asked to close current one.
		if (hCurrentSurface == hKernelSurface) {
			hCurrentSurface = NULL;
		}
	}

	return TRUE;
}

// This is called to remove a surface from the surface pool. Can be called ONLY at PASSIVE_LEVEL.
BOOL DirectDraw::RemoveSurfaceFromPool( HANDLE hSurface )
{
	// If DirectDraw is not open, just go out!
	if (!IsKernelOpen()) {
		return FALSE;
	}

	// Lock Surface queue
	SurfaceLock.Acquire();

	// Locate the surface descriptor...
	HANDLE hKernelSurface = NULL;
	for (ULONG Idx = 0; Idx < ulSurfaceCount; Idx++) {
		if (sdSurface[Idx].hSurface == hSurface) {
			// Remember the Surface Handle...
			hKernelSurface = sdSurface[Idx].hSurface;
			// And remove the entry...
			memmove(sdSurface + Idx,sdSurface + Idx + 1,sizeof(SURF_DESC) * (ulSurfaceCount - Idx - 1));
			// We have one less entry available...
			ulSurfaceCount--;
			break;
		}
	}

	// Unlock surface Queue
	SurfaceLock.Release();

	// Close surface if we are done.
	if (hKernelSurface) {

		// Close surface
		CloseSurface(hKernelSurface);

		// Remember there's no current surface if we were asked to close current one.
		if (hCurrentSurface == hKernelSurface) {
			hCurrentSurface = NULL;
		}
	}

	return TRUE;
}

// This will release all opened surfaces and close DirectDraw: Can be called ony at PASSIVE_LEVEL
void DirectDraw::ReleaseAllSurfaces()
{
	// If DirectDraw is not open, just go out!
	if (!IsKernelOpen()) {
		return;
	}

	// Lock Surface queue
	SurfaceLock.Acquire();

	// We have the Kernel surface handle, remove all surfaces from queue and close them!
	for (ULONG Idx = 0; Idx < ulSurfaceCount; Idx++) {
		CloseSurface(sdSurface[Idx].hSurface);
	};
	// We have no surfaces now...
	ulSurfaceCount = 0;

	// Unlock surface Queue
	SurfaceLock.Release();

	// Remember there's no current surface
	hCurrentSurface = NULL;

	// Close Kernel DirectDraw.
	CloseKernel();
}

// This swaps the previous surface with the new one...
BOOL DirectDraw::ShowSurface(PHW_STREAM_REQUEST_BLOCK pSrb)
{
	// If DirectDraw is not open, just go out!
	if (!IsKernelOpen()) {
		return FALSE;
	}

	// Lock Surface queue
	SurfaceLock.Acquire();

	// Locate the surface descriptor...
	HANDLE hKernelSurface = NULL;
	for (ULONG Idx = 0; Idx < ulSurfaceCount; Idx++) {
		if (sdSurface[Idx].pSrb == pSrb) {
			// Remember the Surface Handle...
			hKernelSurface = sdSurface[Idx].hSurface;
			break;
		}
	}

	// Unlock surface Queue
	SurfaceLock.Release();

	// If no translation found, exit now.
	if (!hKernelSurface) return FALSE;

	// If we had a previous surface, flip the surfaces.
	if (hCurrentSurface && hKernelSurface) {
		// Flip overlay: This can be called at raised IRQL
		FlipOverlay(hKernelSurface,hCurrentSurface);
	}

	// And remember this as the new current surface
	hCurrentSurface = hKernelSurface;

	return TRUE;
}

#endif