/*
	DDraw.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef DDRAW_H_
#define DDRAW_H_

#include "Debug.hpp"
#include "WDMHeaders.hpp"
#include "Spinlock.hpp"

#define ENABLE_DDRAW_STUFF /* Kernel DirectDraw access... */

#ifdef ENABLE_DDRAW_STUFF 

#define DD_OK 0
#define DXERR_UNSUPPORTED                       0x80004001

// The following should be defined in ddkmapi.h, but for some reason are not!

#ifdef booboo // DDKERNELCAPS_SKIPFIELDS
/*
 * Indicates that the device supports field skipping.
 */
#define DDKERNELCAPS_SKIPFIELDS			0x00000001l

/*
 * Indicates that the device can support software autoflipping.
 */
#define DDKERNELCAPS_AUTOFLIP			0x00000002l

/*
 * Indicates that the device can switch between bob and weave.
 */
#define DDKERNELCAPS_SETSTATE			0x00000004l

/*
 * Indicates that a client can gain direct access to the frame buffer.
 */
#define DDKERNELCAPS_LOCK			0x00000008l

/*
 * Indicates that a client can manually flip the video port.
 */
#define DDKERNELCAPS_FLIPVIDEOPORT		0x00000010l

/*
 * Indicates that a client can manually flip the overlay.
 */
#define DDKERNELCAPS_FLIPOVERLAY		0x00000020l

/*
 * Indicates that the device supports a fast, asynchronous transfer
 * mechanism to system memory.
 */
#define DDKERNELCAPS_TRANSFER_SYSMEM		0x00000040l

/*
 * Indicates that the device supports a fast, asynchronous transfer
 * mechanism via AGP.
 */
#define DDKERNELCAPS_TRANSFER_AGP		0x00000080l

/*
 * Indicates that the device can report the polarity (even/odd) of
 * the curent video field.
 */
#define DDKERNELCAPS_FIELDPOLARITY		0x00000100l

/****************************************************************************
 *
 * DDKERNELCAPS IRQ CAPS
 *
 ****************************************************************************/

/*
 * The device can generate display VSYNC IRQs
 */
#define DDIRQ_DISPLAY_VSYNC			0x00000001l

/*
 * Reserved
 */
#define DDIRQ_RESERVED1				0x00000002l

/*
 * The device can generate video ports VSYNC IRQs using video port 0
 */
#define DDIRQ_VPORT0_VSYNC			0x00000004l

/*
 * The device can generate video ports line IRQs using video port 0
 */
#define DDIRQ_VPORT0_LINE			0x00000008l

/*
 * The device can generate video ports VSYNC IRQs using video port 1
 */
#define DDIRQ_VPORT1_VSYNC			0x00000010l

/*
 * The device can generate video ports line IRQs using video port 1
 */
#define DDIRQ_VPORT1_LINE			0x00000020l

/*
 * The device can generate video ports VSYNC IRQs using video port 2
 */
#define DDIRQ_VPORT2_VSYNC			0x00000040l

/*
 * The device can generate video ports line IRQs using video port 2
 */
#define DDIRQ_VPORT2_LINE			0x00000080l

/*
 * The device can generate video ports VSYNC IRQs using video port 3
 */
#define DDIRQ_VPORT3_VSYNC			0x00000100l

/*
 * The device can generate video ports line IRQs using video port 3
 */
#define DDIRQ_VPORT3_LINE			0x00000200l

/*
 * The device can generate video ports VSYNC IRQs using video port 4
 */
#define DDIRQ_VPORT4_VSYNC			0x00000400l

/*
 * The device can generate video ports line IRQs using video port 4
 */
#define DDIRQ_VPORT4_LINE			0x00000800l

/*
 * The device can generate video ports VSYNC IRQs using video port 5
 */
#define DDIRQ_VPORT5_VSYNC			0x00001000l

/*
 * The device can generate video ports line IRQs using video port 5
 */
#define DDIRQ_VPORT5_LINE			0x00002000l

/*
 * The device can generate video ports VSYNC IRQs using video port 6
 */
#define DDIRQ_VPORT6_VSYNC			0x00004000l

/*
 * The device can generate video ports line IRQs using video port 6
 */
#define DDIRQ_VPORT6_LINE			0x00008000l

/*
 * The device can generate video ports VSYNC IRQs using video port 7
 */
#define DDIRQ_VPORT7_VSYNC			0x00010000l

/*
 * The device can generate video ports line IRQs using video port 7
 */
#define DDIRQ_VPORT7_LINE			0x00020000l

/*
 * The device can generate video ports VSYNC IRQs using video port 8
 */
#define DDIRQ_VPORT8_VSYNC			0x00040000l

/*
 * The device can generate video ports line IRQs using video port 8
 */
#define DDIRQ_VPORT8_LINE			0x00080000l

/*
 * The device can generate video ports VSYNC IRQs using video port 9
 */
#define DDIRQ_VPORT9_VSYNC			0x00010000l

/*
 * The device can generate video ports line IRQs using video port 9
 */
#define DDIRQ_VPORT9_LINE			0x00020000l

#endif


/*
	Kernel DirectDraw Handling Routines
*/

#define MaxSurfaces 256

class DirectDraw {
	public:
		typedef struct {
			HANDLE hSurface;				// Surface handle
			PHW_STREAM_REQUEST_BLOCK pSrb;	// Associated Srb
		} SURF_DESC;

	protected:

		SpinLock			SurfaceLock;			// Spinlock to protect the data buffer
		SURF_DESC			sdSurface[MaxSurfaces];	// Surface mappings...
		ULONG				ulSurfaceCount;			// Count of Surfaces.

		BOOL				bUnusable;				// If DirecvtDraw is unusable
		BOOL				bUnavailable;			// If DirectDraw functions are unavailable
		HANDLE				hCurrentSurface;		// Currently displayed surface		

		HANDLE				hKernelDirectDrawHandle;	// Handle to the KernelDirectDraw
		BOOL				bKernelDirectDrawRegistered;

	public:

		DirectDraw() : 
			ulSurfaceCount(0),
			bUnusable(false) ,
			bUnavailable(false),
			hCurrentSurface(NULL),
			hKernelDirectDrawHandle(NULL) ,
			bKernelDirectDrawRegistered(FALSE)

		{}

		~DirectDraw() { CloseKernel(); }

		// Public interfaces:

		// Callable only at Passive level
		BOOL	AddSurfaceToPool( PHW_STREAM_REQUEST_BLOCK pSrb );
		BOOL	RemoveSurfaceFromPool( PHW_STREAM_REQUEST_BLOCK pSrb );
		BOOL	RemoveSurfaceFromPool( HANDLE hSurface );
		void	ReleaseAllSurfaces();

		// Callable at raised IRQL
		BOOL	ShowSurface( PHW_STREAM_REQUEST_BLOCK pSrb );

	private:

		// To open and close the DirectDraw provider
		BOOL	OpenKernel ( HANDLE hUserDirectDrawHandle );
		BOOL	CloseKernel ( void );
		BOOL	IsKernelOpen() const { return hKernelDirectDrawHandle != NULL; }	

		// To open/close handles to the surfaces associated to this.
		HANDLE	OpenSurface ( HANDLE hUserSurfaceHandle );
		BOOL	CloseSurface ( HANDLE hKernelSurfaceHandle );

		// Flip surfaces.
		BOOL	FlipOverlay ( HANDLE hToKernelSurfaceHandle , HANDLE hFromKernelSurfaceHandle );
		BOOL	IsKernelLockAndFlipAvailable ( void );

		// Functions to register for events
		BOOL RegisterForEvents ( void );
		BOOL UnregisterForEvents ( void );
	
		// Callback
		static ULONG FAR PASCAL cbEvent ( DWORD dwEvent, PVOID pContext, DWORD dwParam1, DWORD dwParam2 );
		ULONG  Event ( DWORD dwEvent, DWORD dwParam1, DWORD dwParam2 );

};

#endif

#endif