 /*
	MSP3400.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  ChangeLog:
	When: 08/19/2002 - 09/12/2002
	Who:  ejtagle & Jan Nikitenko <niki@penguin.cz>
	What: MSP34xx support was much/much/much improved
		  LangC support / Speedups / Enhaced autodetetion
		  - Jan worked very hard on this... to implement Lang C (=A+B dual lang) directshow,
		  Lots of fixes to the m_simple autodetection method and also lost of fixes to the compatibility
		  mode (MSP34xxD or less) support.
		  - I (ejtagle) sped up the autodetection process and added compatibility to the ATV2000 program
		    (and most DirectShow based programs). Also Added Dolby(tm) support for those MSP34xx that support it.
*/

/*
			NICAM	Simple	BTSC	A2	Eq	 STATUS
3410B		Yes		No		No		Yes No	 No
3400C		No		No		No		Yes Yes  No    
3400D		No		Yes 	No		Yes Yes  No 	 [1]
3405D		No		Yes		No		Yes	No	 No		 [1]
3410D		Yes		No		No		Yes No	 No
3415D		Yes		Yes		No		Yes	No	 No		 [1]
3401G		No		Yes 	No		Yes Yes  Yes	 
3405G		No		Yes		No		Yes	No	 Yes
3415G		Yes		Yes		No		Yes	No	 Yes
3425G		No		Yes		Yes		Yes	No	 Yes
3445G		No		Yes		Yes		Yes	No	 Yes
3455G		Yes		Yes		Yes		Yes	No	 Yes
3465G		Yes		Yes		Yes		Yes	No	 Yes
3411G		Yes 	Yes 	No		Yes Yes  Yes
3421G		No		Yes 	Yes 	Yes Yes  Yes
3441G		No		Yes 	Yes 	Yes Yes  Yes
3451G		Yes 	Yes 	Yes 	Yes Yes  Yes
3461G		Yes 	Yes 	No		Yes Yes  Yes

  [1] = Simple autodetection is available, but complex to implement, so it is not supported
		as it requires nearly the same overhead to keep checking reception conditions as the
		by-hand way. It also requires lots of checks to make sure detected std is the broadcasted
		one... It's easier to autodetect it manually.
*/

/* TV Audio Norms
		Norm	Base	Subc
	M	A2		4.5FM	4.72
	M	BTSC	4.5FM
	M	(jap)	4.5FM	
	B/G NICAM	5.5FM	5.85
	B/G A2		5.5FM	5.74
	I	NICAM	6.0FM	6.552
	L	NICAM	6.5AM	5.85 
	D/K NICAM	6.5FM	5.85
	D/K A2		6.5FM	6.25 / 6.74 / 5.74

 */

#include "MSP3400.hpp"
#include "..\HwCapture\Pisces.hpp"

/* ------------------------------------------------------------------------ */

/* ----------------------------------------------------------------------- */
/* functions for talking to the MSP3400C Sound processor				   */

bool MSP34xx::Reset()
{
	static BYTE reset_off[3] = {0x00, 0x80, 0x00 };
	static BYTE reset_on[3]  = {0x00, 0x00, 0x00 };

	i2c.WR(i2caddr,reset_off,3,NULL,0,true);  /* XXX ignore errors here */
	if (!i2c.WR(i2caddr,reset_on, 3,NULL,0)) {
		ERROR(("msp3400: chip Reset failed\n"));
		return false;
	}
	return true;
}

// Read address of the MSP3400
// dev = subaddr
WORD MSP34xx::Read(BYTE subaddr, WORD addr)
{
	BYTE writebf[3];
	BYTE readbf[2];

	writebf[0] = subaddr + 1;
	writebf[1] = addr >> 8;
	writebf[2] = addr & 0xff;

	int err;
	for (err = 0; err < 3;) {
		if (i2c.WR(i2caddr,writebf,3,readbf,2)) 
			break;
		
		ERROR(("msp34xx: I/O error #%d (Read 0x%02x/0x%02x)\n", err, subaddr, addr));
		err ++;
	}
	if (3 == err) {
		ERROR(("msp34xx: giving up, reseting chip. Sound will go off,\n"));
		Reset();
		return 0;
	}
	return ( WORD(readbf[0]) << 8) | readbf[1];
}

// Write to address of the msp
bool MSP34xx::Write(BYTE subaddr, WORD addr, WORD val)
{
	BYTE buffer[5];

	buffer[0] = subaddr;
	buffer[1] = addr >> 8;
	buffer[2] = addr &	0xff;
	buffer[3] = val  >> 8;
	buffer[4] = val  &	0xff;

	int err;
	for (err = 0; err < 3;) {
		if (i2c.WR(i2caddr,buffer, 5,NULL,0))
			break;
		err++;
		ERROR(("msp34xx: I/O error #%d (Write 0x%02x/0x%02x)\n",err, subaddr, addr));
	}
	if (3 == err) {
		ERROR(("msp34xx: giving up, reseting chip. Sound will go off,\n"));
		Reset();
		return false;
	}
	return true;
}

/* ----------------------------------------------------------------------- */

#define SCART_MASK	  0
#define SCART_IN1	  1
#define SCART_IN2	  2
#define SCART_IN1_DA  3
#define SCART_IN2_DA  4
#define SCART_IN3	  5
#define SCART_IN4	  6
#define SCART_MONO	  7
#define SCART_MUTE	  8

#define SC_NN	0xffff /* None */

void MSP34xx::SetScart(int in, int out)
{
	static WORD scarts[3][9] = {
		/* MASK	 IN1	 IN2	 IN1_DA  IN2_DA  IN3	 IN4	 MONO	 MUTE	*/
		{  0x0320, 0x0000, 0x0200, SC_NN,   SC_NN, 0x0300, 0x0020, 0x0100, 0x0320 },
		{  0x0c40, 0x0440, 0x0400, 0x0c00, 0x0040, 0x0000, 0x0840, 0x0800, 0x0c40 },
		{  0x3080, 0x1000, 0x1080, 0x0000, 0x0080, 0x2080, 0x3080, 0x2000, 0x3000 },
	};

#if DBG
	static char *scart_names[] = {
		"mask", "in1", "in2", "in1 da", "in2 da", "in3", "in4", "mono", "mute"
	};
#endif

	if (SC_NN == scarts[out][in])
		return;

	DUMP(("msp34xx: scart switch: %s => %d\n",scart_names[in],out));

	m_acb &= ~scarts[out][SCART_MASK];
	m_acb |=  scarts[out][in];

	Write(I2C_MSP3400C_DFP, 0x0013, m_acb);
}

void MSP34xx::SetCarrier3400(const DWORD cdo1,const DWORD cdo2)
{
	ENSURE(( m_simple == false ));
	Write(I2C_MSP3400C_DEM, 0x0093, WORD(cdo1 & 0xfff));
	Write(I2C_MSP3400C_DEM, 0x009b, WORD(cdo1 >> 12));
	Write(I2C_MSP3400C_DEM, 0x00a3, WORD(cdo2 & 0xfff));
	Write(I2C_MSP3400C_DEM, 0x00ab, WORD(cdo2 >> 12));
	Write(I2C_MSP3400C_DEM, 0x0056, 0); /*LOAD_REG_1/2*/

	// If nicam available ...
	if (m_nicam) {

		/* nicam prescale */
		Write(I2C_MSP3400C_DFP, 0x0010, 0x5a00); /* was: 0x3000 */

		/* and start NICAM process 3410B only */
		if (m_nicamkick) {
			Write(I2C_MSP3400C_DEM, 0x0078, 0); /*SEARCH_NICAM*/
		}
	}

	if (m_autofmam) {
		/* and set AUTOFM/AM (3400D 3410D 3415D only) */
		Write(I2C_MSP3400C_DEM, 0x0021, 1); 
	}
}

void MSP34xx::_SetVolume(const int left,const int right) // max = 65535 = 0db
{
	int vol,val,balance;

	vol 	= (left > right) ? left : right;
	val 	= (vol * 0x73 / 65535) << 8; // Maximum 0 db. 
	balance = 0;
	if (vol > 0)
		balance = ((right-left) * 127) / vol;

	DUMP(("msp34xx: SetVolume: %d:%d 0x%02x 0x%02x\n",left,right,val>>8,balance));

	Write(I2C_MSP3400C_DFP, 0x0000, WORD(val)); /* loudspeaker */
	Write(I2C_MSP3400C_DFP, 0x0006, WORD(val)); /* headphones  */

	/* scart - on/off only */
	Write(I2C_MSP3400C_DFP, 0x0007, (val != 0) ? 0x4000 : 0);
	Write(I2C_MSP3400C_DFP, 0x0001, WORD(balance << 8));
}

void MSP34xx::_SetBass(const int bass) // max = 65535 
{
	int val = ((bass-32768) * 0x60 / 65535) << 8;

	if (!m_equalizer) {
		DUMP(("msp34xx: _SetBass: %d 0x%02x\n",bass, val>>8));
		Write(I2C_MSP3400C_DFP, 0x0002, WORD(val) ); /* loudspeaker */
	}
}

void MSP34xx::_SetTreble(const int treble) // max = 65535 
{
	int val = ((treble-32768) * 0x60 / 65535) << 8;

	if (!m_equalizer) {
		DUMP(("msp34xx: _SetTreble: %d 0x%02x\n",treble, val>>8));
		Write(I2C_MSP3400C_DFP, 0x0003, WORD(val) ); /* loudspeaker */
	}
}

void MSP34xx::_SetEQ(const int* bands) // Center value is 32768, max = 65535
{
	if (m_equalizer) {
		Write(I2C_MSP3400C_DFP, 0x0002, 0x0000 ); /* loudspeaker: Enable equalizer */
		Write(I2C_MSP3400C_DFP, 0x0003, 0x0000 ); /* loudspeaker: Enable equalizer */
		Write(I2C_MSP3400C_DFP, 0x0020, 0xFF00 ); /* loudspeaker: Enable equalizer */
		WORD Idx;
		for (Idx=0;Idx < 5;Idx++) {
			WORD val = ((((bands[Idx] - 32768) * 0x60) / 32767) << 8);
			Write(I2C_MSP3400C_DFP, 0x0021 + Idx, val ); // Set the bands!
		}
	}
}

void MSP34xx::_SetDolby(void)
{
	// If using an external Dolby processor, disable the internal effects...
	if (m_extdolby) {

		// Disable Spatial Effects...
		Write(I2C_MSP3400C_DFP,0x0005,0x0000); 

		// And disable internal Dolby if the MSP supports it...
		if (m_mspwithdolby) {
			Write(I2C_MSP3400C_DFP,0x0048,0x0000); 
			Write(I2C_MSP3400C_DFP,0x0049,0x0000); 
			Write(I2C_MSP3400C_DFP,0x004A,0x0000); 
			Write(I2C_MSP3400C_DFP,0x004B,0x0000); 
			Write(I2C_MSP3400C_DFP,0x004D,0x0000); 
		}

		return;
	}

	// If we don't have a Dolby capable processor...
	if (!m_mspwithdolby) {

		// If the MSP34xx does not have an integrated Dolby decoder, 
		// the only available thing is "Spatial effect"...

		// Enable Spatial Effects ...
		Write(I2C_MSP3400C_DFP,0x0005,WORD(
			(m_dolbyplmode == DPLMPl_Normal || m_dolbymode == DPL_Through) 
				? 0x0000 // Disables Spatial Effect...
				: (
					(((int(m_dolbyspatial&0x7F) << 1) - 0x80) << 8) |	// Spatial effects...
					((m_dolbyplmode == DPLMPl_Wide) ? 0x00 : 0x20) |// SBE And pseudoStereo or SBE
					0x8 )
					)
		); 

	} else {
		// We have a Dolby capable MSP34xx... just set Dolby params...

		// Disable Spatial Effects... (Can't run at the same time as Panorama/3DPanorama)
		Write(I2C_MSP3400C_DFP,0x0005,0x0000); 

		// Based upon mode, set it
		switch(m_dolbymode) {
			default:
			case DPL_Through:		
				Write(I2C_MSP3400C_DFP,0x0048,0x0000); // Virtual Surround processing off
				Write(I2C_MSP3400C_DFP,0x004D,0x0000); // Disable noise generator.
				break;

			case DPL_Prologic:		/* Dolby prologic mode */
				Write(I2C_MSP3400C_DFP,0x0048,0x0100); // Virtual surround processing on
				Write(I2C_MSP3400C_DFP,0x004D,0x0000); // Disable noise generator.
				break;

			case DPL_Noise:			/* Noise mode */
				Write(I2C_MSP3400C_DFP,0x0048,0x0000); // Virtual Surround processing off
				Write(I2C_MSP3400C_DFP,0x004D,WORD(0x8000 | (m_dolbynoise & 0xF0))); // Enable noise and set the noise type.
				break;
		}

		// Set Virtual surround Spatial effects
		Write(I2C_MSP3400C_DFP,0x0049, WORD(int(m_dolbyspatial) << 8) );

		// Set Panorama effect...
		Write(I2C_MSP3400C_DFP,0x004A, WORD(int(m_dolbypan)     << 8) );

		// Based on requested mode, set it.
		Write(I2C_MSP3400C_DFP,0x004B, 
			((m_dolbyplmode == DPLMPl_Panorama) ? 0x50 :
			((m_dolbyplmode == DPLMPl_3DPanorama) ? 0x60 : 0)));
			
	}
}

/* This macro is allowed for *constants* only, cc must calculate it
   at compile time.  Remember -- no floats in kernel mode */
#define CARRIER(freq) ((DWORD)( float(freq/18.432)*float(1<<24) ))

// To avoid Warnings!
#define _W(x) WORD(x)


// Set MSP mode... ONLY used when m_simple == 0
void MSP34xx::SetMode3400(MSPMode type)
{
	int i;

	DUMP(("msp3400: setmode: %d\n",type));
	ENSURE(( m_simple == false ));

	// Initialization info for each mode 
	static INIT_DATA_DEM init_data[] = {

		/* AM (for carrier detect / msp3400 */
		{ { 75, 19, 36, 35, 39, 40 }, { 75, 19, 36, 35, 39, 40 },
		CARRIER(5.5), CARRIER(5.5),
		0x00d0, 0x0500,   0x0020, 0x3000},

		/* AM (for carrier detect / msp34x1G) */
		{ { _W(-1), _W(-1), _W(-8), 2, 59, 126 }, { _W(-1), _W(-1), _W(-8), 2, 59, 126 },
		CARRIER(5.5), CARRIER(5.5),
		0x00d0, 0x0100,   0x0020, 0x3000},

		/* FM Radio */
		{ { _W(-8), _W(-8), 4, 6, 78, 107 }, { _W(-8), _W(-8), 4, 6, 78, 107 },
		CARRIER(10.7), CARRIER(10.7),
		0x00d0, 0x0480, 0x0020, 0x3000 },

		/* Terrestial FM-mono + FM-stereo */
		{ {  3, 18, 27, 48, 66, 72 }, {  3, 18, 27, 48, 66, 72 },
		CARRIER(5.5), CARRIER(5.5),
		0x00d0, 0x0480,   0x0030, 0x3000},

		/* Sat FM-mono */
		{ {  1,  9, 14, 24, 33, 37 }, {  3, 18, 27, 48, 66, 72 },
		CARRIER(6.5), CARRIER(6.5),
		0x00c6, 0x0480,   0x0000, 0x3000},

		/* NICAM/FM -- B/G (5.5/5.85), D/K (6.5/5.85) */
		{ { _W(-2), _W(-8), _W(-10), 10, 50, 86 }, {  3, 18, 27, 48, 66, 72 },
		CARRIER(5.5), CARRIER(5.5),
		0x00d0, 0x0040,   0x0120, 0x3000},

		/* NICAM/FM -- I (6.0/6.552) */
		{ {  2, 4, _W(-6), _W(-4), 40, 94 }, {	3, 18, 27, 48, 66, 72 },
		CARRIER(6.0), CARRIER(6.0),
		0x00d0, 0x0040,   0x0120, 0x3000},

		/* NICAM/AM -- L (6.5/5.85) */
		{ {  _W(-2), _W(-8), _W(-10), 10, 50, 86 }, {  _W(-4), _W(-12), _W(-9), 23, 79, 126 },
		CARRIER(6.5), CARRIER(6.5),
		0x00c6, 0x0140,   0x0120, 0x7c03},
	};

	// Store current mode
	m_mode		   = type;

	Write(I2C_MSP3400C_DEM, 0x00bb, 		 /* ad_cv */
			init_data[type].ad_cv);
	
	for (i = 5; i >= 0; i--)				 /* fir 1 */
		Write(I2C_MSP3400C_DEM, 0x0001,
			init_data[type].fir1[i]);
	
	Write(I2C_MSP3400C_DEM, 0x0005, 0x0004); /* fir 2 */
	Write(I2C_MSP3400C_DEM, 0x0005, 0x0040);
	Write(I2C_MSP3400C_DEM, 0x0005, 0x0000);

	for (i = 5; i >= 0; i--)
		Write(I2C_MSP3400C_DEM, 0x0005,
			init_data[type].fir2[i]);
	
	Write(I2C_MSP3400C_DEM, 0x0083, 	/* MODE_REG */
			init_data[type].mode_reg);
	
	SetCarrier3400(init_data[type].cdo1,
				init_data[type].cdo2);  /* Also starts NICAM processing if required... */
	
	Write(I2C_MSP3400C_DEM, 0x0056, 0); /*LOAD_REG_1/2*/

	if (m_extdolby) {
		Write(I2C_MSP3400C_DFP, 0x0008,
			   0x0520); /* I2S1 */
		Write(I2C_MSP3400C_DFP, 0x0009,
			   0x0620); /* I2S2 */
	} else {
		Write(I2C_MSP3400C_DFP, 0x0008,
				init_data[type].dfp_src);
		Write(I2C_MSP3400C_DFP, 0x0009,
				init_data[type].dfp_src);
	}
	Write(I2C_MSP3400C_DFP, 0x000a,
			init_data[type].dfp_src);
	Write(I2C_MSP3400C_DFP, 0x000b,
			init_data[type].dfp_src);
	Write(I2C_MSP3400C_DFP, 0x000e,
			init_data[type].dfp_matrix);

}

/* turn on/off nicam + stereo */
void MSP34xx::SetStereoMode3400(AudioMode mode)
{
	WORD src  =0;

	// Extern and FM Radio mode uses SCART...
	if (m_currconn == AudioConLineIn ||
		(m_currconn == AudioConRadio && m_extfmradio) ) {

		// Extern mode uses SCART...
		DUMP(("msp3400: extern/radio setstereo: %d\n",mode));
		src = 0x0220;	// Transparent Stereo from External SCART connector...

	} else {

		WORD nicam = 0; /* channel source: FM/AM or nicam */

		/* switch demodulator */
		switch (m_mode) {
			case FM_TERRA:
				DUMP(("msp3400: FM SetStereoMode: %d\n",mode));
				if (mode == Am_Lang1 || mode == Am_Lang2) {
					Write(I2C_MSP3400C_DFP, 0x000e, 0x3000);
				} else 
				if (mode == Am_Stereo) {
					Write(I2C_MSP3400C_DFP, 0x000e, 0x3001);
				} else {
					Write(I2C_MSP3400C_DFP, 0x000e, 0x3000);
				}
				break;
			case FM_SAT:
				DUMP(("msp3400: SAT SetStereoMode: %d\n",mode));
				if (mode == Am_Lang1 || mode == Am_Lang2) {
					SetCarrier3400(CARRIER(7.38), CARRIER(7.02));			
				} else 
				if (mode == Am_Stereo) {
					SetCarrier3400(CARRIER(7.2), CARRIER(7.02));			
				} else {
					SetCarrier3400(CARRIER(6.5), CARRIER(6.5));
				}
				break;
			case FM_NICAM1:
			case FM_NICAM2:
			case AM_NICAM:
				DUMP(("msp3400: NICAM SetStereoMode: %d\n",mode));
				if (m_nicam_on) {
					nicam = 0x0100;
				}
				break;
			case BTSC:
				DUMP(("msp3400: BTSC SetStereoMode: %d\n",mode));
				nicam = 0x0300;
				break;
			case FM_RADIO:
				DUMP(("msp3400: FM-Radio setstereo: %d\n",mode));
				break;
		}

		/* switch audio */

		/* special values: documented by Jan Nikitenko <niki@penguin.cz>
			----VALUES:                                ----MY COMMENT:
			SUM/DIFF        0100 0000       40 hex ... A+B mixed to one signal
															into left and right
			AB_XCHANGE      0101 0000       50 hex ... A to right, B to left
			PHASE_CHANGE_B  0110 0000       60 hex ... A to left, B to right
			PHASE_CHANGE_A  0111 0000       70 hex ... A to left, B to right
			A_ONLY          1000 0000       80 hex ... A to left, silence to right
			B_ONLY          1001 0000       90 hex ... silence to left, B to right
			------------------------------------------------------------------------- */


		if (mode == Am_Lang1) {
			// LangB
			src = 0x0010 | nicam;
		} else 
		if (mode == Am_Lang2) {
			// directshow LangC, dualA->left+dualB->right
			src = 0x0050 | nicam;
		} else 
		if (mode == Am_Stereo) {
			src = 0x0020 | nicam;

		} else { // Mono
			// LangA
			src = 0x0000 | nicam;

			if (m_mode == AM_NICAM) {
				DUMP(("msp3400: switching to AM mono\n"));
	
				/* AM mono decoding is handled by tuner, not MSP chip */
				/* SCART switching control register */
				SetScart(SCART_MONO,0);
				src = 0x0220;
			}
		}
	}
	
	// If using Dolby
	if (m_extdolby) {
		Write(I2C_MSP3400C_DFP, 0x0008,0x0520);
		Write(I2C_MSP3400C_DFP, 0x0009,0x0620);
	} else {
		Write(I2C_MSP3400C_DFP, 0x0008,src);
		Write(I2C_MSP3400C_DFP, 0x0009,src);
	}

	Write(I2C_MSP3400C_DFP, 0x000a,src);
	Write(I2C_MSP3400C_DFP, 0x000b,src);
}

// Note we MUST NOT Access the Carrier registers if we are operating in Simple mode...
void MSP34xx::SetStereoMode34x1G(AudioMode mode)
{
	WORD src  =0;
	
	// Extern and FM Radio mode uses SCART...
	if (m_currconn == AudioConLineIn ||
		(m_currconn == AudioConRadio && m_extfmradio) ) {

		// Extern mode uses SCART...
		DUMP(("msp3400: extern/radio setstereo: %d\n",mode));
		if (mode == Am_Mono) {
			src = 0x0200;	// Downmixed mono from External SCART connector...
		} else {
			src = 0x0220;	// Transparent Stereo from External SCART connector...
		}

	} else {

		/* switch audio */
		if (mode == Am_Lang1) {

			//"Stereo or B" channel: Analog or digital mono
			// sound, stereo if available. In case of bilingual broadcast,
			// it contains language B (on left and right).
			src = 0x0420; // Stereo or lang B	 -DirectShow = LANG B
		} else 
		if (mode == Am_Lang2) {
	
			//"Stereo or A/B" channel: Analog or digital mono
			// sound, stereo if available. In case of bilingual broadcast,
			// it contains both languages A (left) and B (right).
			src = 0x0120; // Stereo or lang A/B  -DirectShow = LANG C
		} else 
		if (mode == Am_Stereo) {	// Lang A in stereo... (Default Stereo)
	
			//"Stereo or A" channel: Analog or digital mono
			// sound, stereo if available. In case of bilingual broadcast,
			// it contains language A (on left and right).
			src = 0x0320;			// Stereo or lang A	 -DirectShow = LANG A, Or DirectShow STEREO
	
		} else { // Mono... Could mean:
				// Lang A in mono (Dual channel reception)
				// Downmix stereo to mono (Stereo transmission)
				// Mono transmission
	
			// Assume mono mode....
	
			//"Stereo or A" channel downmixed to mono: Analog or digital mono
			// sound, stereo downmixed to mono if available. In case of bilingual
			// broadcast, it contains language A (on left and right).
	
			src = 0x0330;			// Downmixed Mono mode... - DirectShow = Mono
									// Means Stereo or LangA downmixed to Mono...
	
			if (m_mode == AM_NICAM) {
				DUMP(("msp3400: switching to AM mono\n"));
		
				/* AM mono decoding is handled by tuner, not MSP chip */
				/* SCART switching control register */
				SetScart(SCART_MONO,0);
				src = 0x0220;
			}
		}
	}

	// If using Dolby
	if (m_extdolby) {
		Write(I2C_MSP3400C_DFP, 0x0008,0x0520);
		Write(I2C_MSP3400C_DFP, 0x0009,0x0620);
	} else {
		Write(I2C_MSP3400C_DFP, 0x0008,src);
		Write(I2C_MSP3400C_DFP, 0x0009,src);
	}

	Write(I2C_MSP3400C_DFP, 0x000a,src);
	Write(I2C_MSP3400C_DFP, 0x000b,src);

}

void MSP34xx::SetStereoMode(AudioMode mode)
{
	if (m_simple) {
		SetStereoMode34x1G(mode);
	} else {
		SetStereoMode3400(mode);
	}
}

bool MSP34xx::AutodetectStereoModes3400()
{
	int val,count;

	// Radio and LineIn won't use autodetection...
	if ((m_currconn == AudioConRadio && m_extfmradio) ||
		m_currconn == AudioConLineIn) {
		return false;
	}

	// Get Old detected values ...
	AudioModes newaudiomodesAv	= m_audiomodesAv;
	bool newnicam				= m_nicam_on;
	bool update 				= false;

	switch (m_mode) {
		case FM_RADIO:

			// Read autodetected value
			val = Read(I2C_MSP3400C_DFP, 0x18);
			if (val > 32768) val -= 65536;

			// If we have found something, just evaluate things...
			if (val > 4096 || val < -4096) {

				DUMP(("msp34xx: stereo detect register: %d\n",val));
		
				if (val > 4096 || val < -4096) {
					newaudiomodesAv = Am_Mono | Am_Stereo;
				} else {
					newaudiomodesAv = Am_Mono;
				}
				newnicam = false;
			}
			break;
			
		case FM_TERRA:

			// Read autodetected value
			val = Read(I2C_MSP3400C_DFP, 0x18);
			if (val > 32768) val -= 65536;

			// If we have found something, just evaluate things...
			if (val > 4096 || val < -4096) {

				DUMP(("msp34xx: stereo detect register: %d\n",val));
		
				if (val > 4096) {
					newaudiomodesAv = Am_Mono | Am_Stereo;
				} else if (val < -4096) {
					newaudiomodesAv = Am_Mono | Am_Lang1 | Am_Lang2;
				} else {
					newaudiomodesAv = Am_Mono;
				}
				newnicam = false;
			}
			break;

		case FM_NICAM1:
		case FM_NICAM2:
		case AM_NICAM:
			val = Read(I2C_MSP3400C_DEM, 0x23);
			DUMP(("msp34xx: nicam sync=%d, mode=%d\n",val & 1, (val & 0x1e) >> 1));

			if (val & 1) {
				/* nicam synced */
				switch ((val & 0x1e) >> 1)	{
					case 0:
					case 8:
						newaudiomodesAv = Am_Stereo;
						break;
					case 1:
					case 9:
						newaudiomodesAv = Am_Mono | Am_Lang1;
						break;
					case 2:
					case 10:
						newaudiomodesAv = Am_Mono | Am_Lang1 | Am_Lang2;
						break;
					default:
						newaudiomodesAv = Am_Mono;
						break;
				}
				newnicam = true;
			} else {
				newnicam = false;
				newaudiomodesAv = Am_Mono;
			}
			break;

		case BTSC: // 
			val = Read(I2C_MSP3400C_DEM, 0x200);
			DUMP(("msp34x1G: status=0x%x (pri=%s, sec=%s, %s%s%s)\n",
				val,
				(val & 0x0002) ? "no"	  : "yes",
				(val & 0x0004) ? "no"	  : "yes",
				(val & 0x0040) ? "stereo" : "mono",
				(val & 0x0080) ? ", nicam 2nd mono" : "",
				(val & 0x0100) ? ", bilingual/SAP"	: ""));

			newaudiomodesAv = Am_Mono |
								((val & 0x0040) ? Am_Stereo : 0) |
								((val & 0x0180) ? (Am_Lang1 | Am_Lang2) : 0) ;
			break;
	}

	if (newaudiomodesAv != m_audiomodesAv) {

		update = true;
			//DUMP(("msp34xx: watch: stereo %d => %d\n",
			//	m_stereo,newstereo));

		m_audiomodesAv = newaudiomodesAv;
	}

	if (newnicam != m_nicam_on) {
		update = true;
		DUMP(("msp3400: watch: nicam %d => %d\n",
			m_nicam_on,newnicam));
		m_nicam_on = newnicam;
	}
	return update;
}

bool MSP34xx::AutodetectStereoModes34x1G()
{
	int val,count;

	// Radio and LineIn won't use autodetection...
	if ((m_currconn == AudioConRadio && m_extfmradio) ||
		m_currconn == AudioConLineIn) {
		return false;
	}

	// Get Old detected values ...
	AudioModes newaudiomodesAv = m_audiomodesAv;	
	bool update    = false;

	// All the parts that use the m_simple access method should try the STATUS register...
	val = Read(I2C_MSP3400C_DEM, 0x200);
	DUMP(("msp34x1G: status=0x%x (pri=%s, sec=%s, %s%s%s)\n",
			val,
			(val & 0x0002) ? "no"	  : "yes",
			(val & 0x0004) ? "no"	  : "yes",
			(val & 0x0040) ? "stereo" : "mono",
			(val & 0x0080) ? ", nicam 2nd mono" : "",
			(val & 0x0100) ? ", bilingual/SAP"	: ""));

	newaudiomodesAv = Am_Mono |
						((val & 0x0040) ? Am_Stereo : 0) |
						((val & 0x0180) ? (Am_Lang1 | Am_Lang2) : 0) ;

	if (newaudiomodesAv != m_audiomodesAv ) {

		update = true;
			//DUMP(("msp34xx: watch: stereo %d => %d\n",
			//	m_stereo,newstereo));

		m_audiomodesAv = newaudiomodesAv;
	}
	return update;
}

bool MSP34xx::AutodetectStereoModes()
{
	if (m_simple) {
		return AutodetectStereoModes34x1G();
	} else {
		return AutodetectStereoModes3400();
	}
}

/* stereo/multilang monitoring */
void MSP34xx::WatchStereo(bool ForceSet)
{
	// Autodetect stereo modes.., If the autodetected modes changed, set the available mode...
	if (AutodetectStereoModes() || ForceSet) {

		// Now check the desired Audio mode. If unavailable, revert to one available.
		AudioMode curr = desiredam;
		if (curr == Am_Stereo && (m_audiomodesAv & Am_Stereo) == 0) curr = Am_Lang1;
		if (curr == Am_Lang1  && (m_audiomodesAv & Am_Lang1)  == 0) curr = Am_Lang2;
		if (curr == Am_Lang2  && (m_audiomodesAv & Am_Lang2)  == 0) curr = Am_Mono;

		// Set the supported mode (Optional for the m_simple method, but useful. Required for the other method
		SetStereoMode(curr);
	}
}


// Mode autodetection for MSP3400
void MSP34xx::AutodetectMode3400()
{

#if DBG
	#define _DT(x) , x
#else
	#define _DT(x)
#endif

#define SUBCARRIER_COUNT(x) (sizeof(x)/sizeof(SUBCARRIER_DETECT))

	static SUBCARRIER_DETECT subcarrier_45[] = {
		{ CARRIER(4.724212)	,FM_TERRA	_DT("4.724 NTSC-M FM-stereo (Korea)") }
	};

	static SUBCARRIER_DETECT subcarrier_55[] = {
		{ CARRIER(5.7421875),FM_TERRA	_DT("5.742 PAL B/G FM-stereo")	 }, 
		{ CARRIER(5.85)		,FM_NICAM1	_DT("5.85  PAL B/G NICAM")		 }
	};

	static SUBCARRIER_DETECT subcarrier_60[] = {
		{ CARRIER(6.552)	,FM_NICAM2	_DT("6.552 PAL I NICAM (UK)") }
	};

	static SUBCARRIER_DETECT subcarrier_65[] = {
		{ CARRIER(5.85)		,AM_NICAM	_DT("5.85  L NICAM or AM MONO")}, /* ONLY when SECAM */
		{ CARRIER(5.85)		,FM_NICAM1	_DT("5.85  PAL D/K + NICAM")},
		{ CARRIER(6.2578125),FM_TERRA	_DT("6.25  PAL D/K1 FM-stereo")		},
		{ CARRIER(6.7421875),FM_TERRA	_DT("6.74  PAL D/K2 FM-stereo")		},
		{ CARRIER(7.02)		,FM_SAT		_DT("7.02  PAL SAT  FM-stereo stereo/bilingual")	}, /* the following 3 means:*/
		{ CARRIER(7.20)		,FM_SAT		_DT("7.20  PAL SAT  FM-stereo stereo")	},
		{ CARRIER(7.38)		,FM_SAT		_DT("7.38  PAL SAT  FM-stereo bilingual")	}
	};

	static MAINCARRIER_DETECT carrier_main[] = {
		/* Sorted from highest to lowest carrier...
			Note that we will probe for all main sound carriers, except 4.5 Mhz that will be
			only detected if the current std is NOT PAL. (Because PAL colour carrier is ~4.5Mhz)
		*/
		{ TN_CCIR_DKL   | TN_CCIR_I|TN_CCIR_BGHN  ,	CARRIER(6.5),subcarrier_65, SUBCARRIER_COUNT(subcarrier_65)	_DT("6.5 PAL D/K + SAT + SECAM")},
		{ TN_CCIR_I	    | TN_CCIR_DKL|TN_CCIR_BGHN,	CARRIER(6.0),subcarrier_60, SUBCARRIER_COUNT(subcarrier_60)	_DT("6.0 PAL I")				},
		{ TN_CCIR_BGHN  | TN_CCIR_I|TN_CCIR_DKL ,	CARRIER(5.5),subcarrier_55, SUBCARRIER_COUNT(subcarrier_55)	_DT("5.5 PAL B/G")				}, 
		{ TN_CCIR_MC                            ,	CARRIER(4.5),subcarrier_45, SUBCARRIER_COUNT(subcarrier_45)	_DT("4.5 NTSC") 				} 
	};

	int max1,max2,val1,val2,val;
	DWORD idx;
	
	// If Radio mode, set that (don't need to autodetect anything!)
	if (m_currconn == AudioConRadio) {

		/* set msp3400 to FM radio mode (unneeded, as tuner handles FM radio) */
		SetMode3400(FM_RADIO);

		if (m_extfmradio) {		
			// Route FM radio using SCART
			SetScart(SCART_IN2,0);
			Write(I2C_MSP3400C_DFP,0x000d,0x1900);
		}
		
		// Set the available modes...
		m_audiomodesAv = Am_Stereo;
		SetStereoMode3400(Am_Stereo);

		// And return.
		return;
	}

	// Nothing to do in extern
	if (m_currconn == AudioConLineIn) {

		// Just set scart 
		SetScart(SCART_IN1,0);
		Write(I2C_MSP3400C_DFP,0x000d,0x1900);

		// Only Stereo available (we are using scart)
		m_audiomodesAv = Am_Stereo;
		SetStereoMode3400(Am_Stereo);
		return;
	}

	// Mute
	_SetVolume(0, 0);
	SetMode3400(AM_DETECT /* +1 */ ); // Init with AM demodulator coeffs...

	DUMP(("Video Standard: 0x%08x\n", m_norm));
					
	/* carrier detect pass #1 -- main carrier */
	val1 = 0;
	max1 = 0;
	for (idx = 0; 
		 idx < sizeof(carrier_main) / sizeof(MAINCARRIER_DETECT); 
		 idx++) {

		// Only if supported by the current videonorm...
		if ((m_norm & carrier_main[idx].norms) != 0) {

			// Set Carrier...
			SetCarrier3400(	carrier_main[idx].cdo,
							carrier_main[idx].cdo);
	
			// Wait 200 ms in 10 ms chunks to let things stabilize
			for (int times = 0; times < 20 ; times++) {
				// Early cancel if not autodetecting or a pending command (autodetection or suspension)
				if (t_action != Autodetect ||
					(t_pendingcmds) > 0) {
					return;
				}
				mdelay(10);
			}

			val = Read(I2C_MSP3400C_DFP, 0x1b);
			if (val > 32768) val -= 65536;
			if (val1 < val) {
				val1 = val;
				max1 = idx;
			}
		}
	}

	DUMP(("msp3400: carrier1 val:%5d / %s\n",val1, carrier_main[max1].name));
	
	/* carrier detect pass #2 -- second (stereo) carrier */
	val2 = 0;
	max2 = 0;
	for (idx = 0; 
		 idx < carrier_main[max1].count; 
		 idx++) {

		// Set Carrier...
		SetCarrier3400(	carrier_main[max1].subcarrier[idx].cdo,
						carrier_main[max1].subcarrier[idx].cdo);
	
		// Wait 200 ms in 10 ms chunks to let things stabilize
		for (int times = 0; times < 20 ; times++) {
			// Early cancel if not autodetecting or a pending command (autodetection or suspension)
			if (t_action != Autodetect ||
				(t_pendingcmds) > 0) {
				return;
			}
			mdelay(10);
		}

		val = Read(I2C_MSP3400C_DFP, 0x1b);
		if (val > 32768) val -= 65536;
		if (val2 < val) {
			val2 = val;
			max2 = idx;
		}
	}

	// program the msp3400 according to the results
	DWORD   m_main	 = carrier_main[max1].cdo;

	// Default to mono sound...
	DWORD   m_second = m_main; 
	MSPMode mode	 = FM_TERRA;

	// If an stereo carrier was detected... 
	if (val2) {

		// Just get the carrier and mode.
		m_second = carrier_main[max1].subcarrier[max2].cdo;
		mode	 = carrier_main[max1].subcarrier[max2].mode;

		DUMP(("msp3400: carrier2 val: %5d / %s\n", val2,carrier_main[max1].subcarrier[max2].name));
	}

	// If we are dealing with a SECAM format, we should convert...
	if (mode == AM_NICAM || mode == FM_NICAM1) {
		// If we are dealing with SECAM...
		if (m_norm & KS_AnalogVideo_SECAM_Mask) {
			// Translate it to AM_NICAM
			mode = AM_NICAM;
			// And prescale SCART...
			Write(I2C_MSP3400C_DFP,0x000d,0x1900);
		} else {
			// Translate it to FM_NICAM1
			mode = FM_NICAM1;
		}
	}

	// If we are dealing with a SAT reception, based on case, we must convert...
	if (mode == FM_SAT) {
		// Based on detected carriers, determine available modes...
		if (m_second == CARRIER(7.38)) {
			m_audiomodesAv = Am_Mono | Am_Lang1 | Am_Lang2;
		} else 
		if (m_second == CARRIER(7.2)) {
			m_audiomodesAv = Am_Mono | Am_Stereo;
		} else 
		if (m_second == CARRIER(7.02)) {
			m_audiomodesAv = Am_Mono | Am_Stereo | Am_Lang1 | Am_Lang2;
		} else {
			m_audiomodesAv = Am_Mono;
		}

		// Both carriers will be the same to start this in Mono
		m_main = m_second = CARRIER(6.5); // (mono sound)

	} else {

		// All the others assume Mono by default
		m_audiomodesAv = Am_Mono;
	}

	// If we are dealing with a NICAM signal, but we don't support it, convert it to mono
	if ((mode == AM_NICAM || mode == FM_NICAM1 || mode == FM_NICAM2) &&
		(m_nicam == false)) {

		// Convert it to Mono audio...
		mode     = FM_TERRA;
		m_second = m_main;

		// Mono available...
		m_audiomodesAv = Am_Mono;
	}

	// Set the MSP mode...
	SetMode3400(mode);

	// Remember if we are dealing with NICAM...
	m_nicam_on = (mode == AM_NICAM || mode == FM_NICAM1 || mode == FM_NICAM2) ? true : false;

	// And set audio carriers...
	SetCarrier3400(m_second, m_main);

	// Start with mono sound...
	SetStereoMode3400(Am_Mono);

	return;
}

/* ----------------------------------------------------------------------- */
/* this one uses the automatic sound standard detection of newer		   */
/* msp34xx chip versions												   */

void MSP34xx::AutodetectMode34x1G()
{

#if DBG
	static MODES modelist[] = {
		{ 0x0000, 0, 0, "ERROR" },
		{ 0x0001, 0, 0, "autodetect start" },
		{ 0x0002, CARRIER(4.5) , CARRIER(4.72),		"4.5/4.72  M Dual FM-Stereo" },
		{ 0x0003, CARRIER(5.5) , CARRIER(5.7421875),"5.5/5.74  B/G Dual FM-Stereo" },
		{ 0x0004, CARRIER(6.5) , CARRIER(6.2578125),"6.5/6.25  D/K1 Dual FM-Stereo" },
		{ 0x0005, CARRIER(6.5) , CARRIER(6.7421875),"6.5/6.74  D/K2 Dual FM-Stereo" },
		{ 0x0006, CARRIER(6.5) , CARRIER(6.5),		"6.5  D/K FM-Mono (HDEV3)" },
		{ 0x0007, CARRIER(6.5) , CARRIER(5.7421875),"6.5/5.74  D/K3 Dual FM-Stereo" },
		{ 0x0008, CARRIER(5.5) , CARRIER(5.85),		"5.5/5.85  B/G NICAM FM" },
		{ 0x0009, CARRIER(6.5) , CARRIER(5.85),		"6.5/5.85  L NICAM AM" },
		{ 0x000a, CARRIER(6.0) , CARRIER(6.55),		"6.0/6.55  I NICAM FM" },
		{ 0x000b, CARRIER(6.5) , CARRIER(5.85),		"6.5/5.85  D/K NICAM FM" },
		{ 0x000c, CARRIER(6.5) , CARRIER(5.85),		"6.5/5.85  D/K NICAM FM (HDEV2)" },
		{ 0x000d, CARRIER(6.5) , CARRIER(5.85),		"6.5/5.85  D/K NICAM FM (HDEV3)" },
		{ 0x0020, CARRIER(4.5) , CARRIER(4.5),		"4.5  M BTSC-Stereo" },
		{ 0x0021, CARRIER(4.5) , CARRIER(4.5),		"4.5  M BTSC-Mono + SAP" },
		{ 0x0030, CARRIER(4.5) , CARRIER(4.5),		"4.5  M EIA-J Japan Stereo" },
		{ 0x0040, CARRIER(10.7), CARRIER(10.7),		"10.7  FM-Stereo Radio" },
		{ 0x0050, CARRIER(6.5) , CARRIER(6.5),		"6.5  SAT-Mono" },
		{ 0x0051, CARRIER(7.02), CARRIER(7.20),		"7.02/7.20  SAT-Stereo" },
		{ 0x0060, CARRIER(7.2) , CARRIER(7.2),		"7.2  SAT ADR" },
		{	   0, 0, 0, NULL } , /* EOF */
	};
#endif

	WORD mode,std = 1,val;
	int i;
	
	// Just route audio in extern mode...
	if (m_currconn == AudioConLineIn) {

		SetScart(SCART_IN1,0);
		Write(I2C_MSP3400C_DFP,0x000d,0x1900);
		m_audiomodesAv = Am_Stereo;
		SetStereoMode(Am_Stereo);

		return;
	}

	/* put into sane state (and mute) */
	Reset();

	/* start autodetect */
	if (m_currconn == AudioConRadio) { // Radio MODE
		mode = 0x0003;	// (Automatic Sound Select = true)
		std  = 0x0040;	// Easy!
	} else if (m_norm & ( KS_AnalogVideo_SECAM_L | KS_AnalogVideo_SECAM_L1 ) ) { // Audio carrier at 6.5Mhz
		// Norm L 
		mode = 0x6003;	// standard L (Secam) (Automatic Sound Select = true) (ignore 4.5Mhz carrier)
		std  = 1;		// Autodetect...
	} else if (m_norm & (KS_AnalogVideo_PAL_M | KS_AnalogVideo_PAL_NC | KS_AnalogVideo_NTSC_50)) {
		// All those PAL stds use a 4.5 MHz audio carrier... so we can't ignore it.
		mode = 0x3003;	// Standard D/K or BTSC (Automatic Sound Select = true)
		std  = 1;		// Autodetect...
	} else if (m_norm & KS_AnalogVideo_PAL_Mask ) {
		// All the other PAL stds don't have a 4.5MHz audio carrier, but a chroma carrier at 4.42Mhz (close enough to 4.5Mhz) so we must ignore it...
		mode = 0x7003;	// Ignore 4.5 Nhz carrier; Std DK (Automatic Sound Select = true)
		std  = 1;		// Autodetect...
	} else if (m_norm & KS_AnalogVideo_NTSC_M_J ) {
		// NTSC Japan...
		mode = 0x5003;	// Standard NTSC japan... (or D/K) (Automatic Sound Select = true)
		std  = 1;
	} else if (m_norm & KS_AnalogVideo_NTSC_Mask ) {
		mode = 0x3003;	// BTSC (Automatic Sound Select = true)
		std  = 1;		// Autodetect...
	} else if (m_norm & KS_AnalogVideo_SECAM_Mask ) { // SECAM always transmits using a 6.5MHz audio carrier
		mode = 0x7003;	// Std DK and 4.5 MHz carrier is ignored. (Automatic Sound Select = true)
		std  = 1;
	} else {
		mode = 0x3003;	// (Automatic Sound Select = true)
		std  = 1;
	}

	Write(I2C_MSP3400C_DEM, 0x30, mode);
	Write(I2C_MSP3400C_DEM, 0x20, std);


#if DBG
	{
		int i;
		for (i = 0; modelist[i].name != NULL; i++)
			if (modelist[i].retval == std)
				break;
		DUMP(("msp34x1G: setting mode: %s (0x%04x)\n",
			modelist[i].name ? modelist[i].name : "unknown",std));
	}
#endif

	if (std != 1) {
		/* programmed some specific mode */
		val = std;
	} else {
		/* triggered autodetect */
		for (;;) {

			/* check results */
			val = Read(I2C_MSP3400C_DEM, 0x7e);
			if (val <= 0x07ff)
				break;
			DUMP(("msp34x1G: detection still in progress\n"));

			// Early cancel autodetection if asked to!
			if (t_action != Autodetect) {
				return;
			}

			// Wait 200 ms in 10 ms chunks to get autodetection
			for (int times = 0; times < 20 ; times++) {
				// Early cancel if not autodetecting or a pending command (autodetection or suspension)
				if (t_action != Autodetect ||
					(t_pendingcmds) > 0) {
					return;
				}
				mdelay(10);
			}
		}
	}

	// We MUST differenciate between all the modes...
#if DBG
	{
		for (i = 0; modelist[i].name != NULL; i++)
			if (modelist[i].retval == val)
				break;

		DUMP(("msp34x1G: current mode: %s (0x%04x)\n",
			modelist[i].name ? modelist[i].name : "unknown",
			val));
	}
#endif

	/* set various prescales */
	Write(I2C_MSP3400C_DFP, 0x0d, 0x1900); /* scart */
	Write(I2C_MSP3400C_DFP, 0x0e, (val == 0x00009 ) ? 0x7c03 : 0x2403); /* FM/AM prescale */
	Write(I2C_MSP3400C_DFP, 0x10, 0x5a00); /* nicam */

	/* set stereo */
	/*
	0000 = Can't find std

	B/G Dual FM Stereo	  0003 = B/G FM
	B/G NICAM FM		  0008 = B/G NICAM
	I NICAM FM			  000A = I
	0040 = FM Radio

	M Dual FM Stereo	  0002 | M Korea (modus[14,13] = 00  |
	BTSC Stereo 		  0020 | M Japan (modus[14,13] = 01  | 4.5MHz carrier
	M EIAJ Japan Stereo   0030 | BTSC	   (modus[14,13] = 10  |

	L NICAM AM			  0009 | L AM				 modus[12] = 0	| 6.5 MHz carrier
	D/K1 Dual FM Stereo   0004 | D/K1 , D/K2 , D/K3  modus[12] = 1	|

	L NICAM AM			  0009 | L NICAM			 modus[12] = 0	| 6.5MHz carrier
	D/K NICAM FM		  000B | D/K NICAM			 modus[12] = 1	|

	D/K2 Dual FM Stereo   0005 
	D/K  FM Mono (Hi Dev) 0006
	D/K3 Dual FM Stereo   0007
	D/K  NICAM FM (HiDev) 000c
	D/K  NICAM FM (HiDv3) 000d
	BTSC MONO + SAP 	  0021
	FM Stereo Radio 	  0040
	Sat Mono			  0050
	Sat Stereo			  0051
	Sat Adr 			  0060


	If automatic Sound Select = true
		0003 == 0008
		0004 == 0005 == 0007 == 000B


	Must Only choose between:
		0003, 0008
			B/G Dual FM Stereo
			B/G NICAM FM
		000A
			I NICAM FM
		0002
			M Dual FM Stereo
		0020 
			BTSC Stereo
		0030
			M EIAJ Japan tereo
		0009
			L NICAM AM
		0004, 0005, 0007, 000b
			D/K1 Dual FM Stereo
			D/K2 Dual FM Stereo
			D/K3 Dual FM Stereo
			D/K NICAM FM
		0040
			FM Stereo Radio
	*/
	switch (val) {
		case 0x0008: /* B/G NICAM */
		case 0x000b: /* D/K NICAM */
		case 0x000c: /* D/K NICAM (HD1) */
		case 0x000d: /* D/K NICAM (HD1) */
		case 0x000a: /* I	NICAM */
			if (val == 0x0008 || val == 0x000b || val == 0x000c || val == 0x000d)
				m_mode = FM_NICAM1;
			else
				m_mode = FM_NICAM2;

			/* just turn on stereo */
			m_audiomodesAv = Am_Stereo;
			m_nicam_on	 = true;
			break;

		case 0x0009:			
			m_mode = AM_NICAM;

			m_audiomodesAv = Am_Mono;			
			m_nicam_on = true;
			break;
	
		case 0x0020: /* BTSC */

			/* just turn on stereo */
			m_mode	 = BTSC;

			m_audiomodesAv = Am_Stereo;
			m_nicam_on = false;
			break;

		case 0x0021: /* BTSC + SAP */

			/* just turn on stereo */
			m_mode	 = BTSC;
			m_audiomodesAv = Am_Stereo | Am_Lang1 | Am_Lang2;
			m_nicam_on = false;
			break;

		case 0x0040: /* FM radio */

			m_mode	   = FM_RADIO;
			m_nicam_on = false;
			
			if (m_extfmradio) {
				/* scart routing */
				SetScart(SCART_IN2,0);
				Write(I2C_MSP3400C_DFP,0x000d,0x1900);
			}

			m_audiomodesAv = Am_Stereo | Am_Mono;
			break;

		case 0x0003: /* B/G  Dual FM */
		case 0x0004: /* D/K1 Dual FM */
		case 0x0005: /* D/K2 Dual FM */
		case 0x0006: /* D/K FM Mono */
		case 0x0007: /* D/K3 Dual FM */
		case 0x0002:
		case 0x0030:

			m_mode		   = FM_TERRA;
			m_audiomodesAv = Am_Mono;
			m_nicam_on	   = false;
	}

	// Startup with mono sound...
	SetStereoMode34x1G(Am_Mono);
}

// Autodetection of available modes
void MSP34xx::AutodetectMode()
{
	// Make sure noone will interfere us while we do this...
	InterlockedIncrement((LONG*)&t_inautodetection);

	if (m_simple) {
		// Use new autodetect method
		AutodetectMode34x1G();
	} else {
		// Use the old one
		AutodetectMode3400();
	}

	// Release lock; race conditions are meaningless from here... (stereo detection can be used concurrently)
	InterlockedDecrement((LONG*)&t_inautodetection);

}

void MSP34xx::AutodetectProc()
{
	while(1) {

		// Do we have some pending cmds ?
		while (InterlockedDecrement((LONG*)&t_pendingcmds) >= 0) {

			// We got a command, check requested action.
			switch (t_action) {
				case Exit:
					return; // End thread
				case SuspendAutodetect:
					t_suspended.Set();
					break;
				default:
				case Autodetect:
		
					// Autodetect current standard
					AutodetectMode();

					// Only when autodetection was not breaked - Jan Nikitenko 
					//	Autodetect Stereo and set the detected mode if possible!
					if (t_action != Autodetect ||
						(t_pendingcmds) > 0) {
						break;
					}

					// Autodetect Stereo and set the detected mode if possible!
					WatchStereo(true);

					// Set all the parameters (maybe they changed while we were autodetecting stds)
					_SetBass(m_bass);
					_SetTreble(m_treble);
					_SetEQ(m_eqband);
					_SetDolby();
					if (m_currconn == AudioConMute) {
						SetScart(SCART_MUTE,0); // Mute
						_SetVolume(0,0);
					} else {
						_SetVolume(m_left,m_right);
					}

					// Wait 1s in 10 ms chunks for autodetection to complete...
					for (int times = 0; times < 100 ; times++) {
						// Early cancel if not autodetecting or a pending command (autodetection or suspension)
						if (t_action != Autodetect ||
							(t_pendingcmds) > 0) {
							break;
						}
						mdelay(10);
					}

					// After that 1sec delay, Autodetect Stereo and set the detected mode if possible (all modes available should be already detected)
					WatchStereo(true);
				
					break;
			}
		};

		// No more available commands, undo the last decrement...
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// And wait for a command
		t_exec.WaitFor();
	};
}

// Out thread proc
VOID MSP34xx::ThreadProc(IN PVOID ctx)
{
	// Execute our procedure...
	MSP34xx* ptr = (MSP34xx*) ctx;
	ptr->AutodetectProc();

	// Signal we are ended (we do it from an static fn so no hidden this references are used) ...
	ptr->t_ended.Set();

	// And end the Thread!
	ExitThread();
}

// To support Hibernate mode
void MSP34xx::Restart( void )
{
	if ( t_action == SuspendAutodetect ) { // Only if autodetect thread is suspending/suspended ...
		// Revision code
		WORD rev1,rev2;

		if (!Reset()) {
			ERROR(("msp3400: no chip found\n"));
		}
	
		// Get Revision Code
		rev1 = Read(I2C_MSP3400C_DFP, 0x1e);
		rev2 = Read(I2C_MSP3400C_DFP, 0x1f);
		if (0 == rev1 && 0 == rev2) {
			ERROR(("msp3400: error while reading chip version\n"));
		}

#if 0
		/* this will turn on a 1kHz beep - might be useful for debugging... */
		Write(I2C_MSP3400C_DFP, 0x0014, 0x1040);
#endif

		// Find-out if the chip supports NICAM...
		m_nicam = (
					(rev2 >> 8) == 00 || 
					(rev2 >> 8) == 01 ||
					(rev2 >> 8) == 05 || 
					(rev2 >> 8) == 21 || 
					(rev2 >> 8) == 41 || 
					(rev2 >> 8) == 25 || 
					(rev2 >> 8) == 45 
					) ? false : true;

		// Check if we need a NICAM kick to start this up (3410B only)
		m_nicamkick = ( ((rev2 >> 8) == 10) && ((rev1&0xff)+'@') == 'B' ) ? true : false;

		// Check if we have an autofm/am feature (MSP34xxD)
		m_autofmam  = ( ((rev1&0xff)+'@') == 'D' ) ? true : false;

		// Check if we can use the simple autodetect method... It must be SIMPLE, not as the MSP34xxD, that is not easy to use...
		m_simple = ((rev1&0xff)+'@' >= 'G');

		// Check if the chip supports a graphic Equalizer. 
		m_equalizer = ((rev1&0xff)+'@' >= 'C' && 
						((rev2>>8) % 10) != 5 &&
						(rev2>>8) != 10	);

		// Check if the chip has an integrated Dolby processor... Must be
		// MSP34x1G
		m_mspwithdolby = ((rev1&0xff)+'@' >= 'G' && 
						 ((rev2>>8) % 10) == 1);

		// All Ok! - Go!
		DUMP(("Found MSP34%02d%c-%c%d NICAM:%c, SIMPLE:%c, EQ:%c, DOLBY:%c\n",
			(rev2>>8)&0xff, (rev1&0xff)+'@', ((rev1>>8)&0xff)+'@', rev2&0x1f
			, (m_nicam)			?'Y':'N' 
			, (m_simple)		?'Y':'N' 
			, (m_equalizer)		?'Y':'N' 
			, (m_mspwithdolby)	?'Y':'N'
		));

		// 

		// Reinit autodetection
		t_action = Autodetect;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();
	}
}

// Wait until autodetection is complete!
void MSP34xx::Suspend()
{
	if ( t_action == Autodetect ) { // Only if thread is autodetecting...
		// Clear the suspended notification...
		t_suspended.Reset();

		// Post our suspend command.
		t_action = SuspendAutodetect;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();

		// And wait until the suspension command is completed.
		t_suspended.WaitFor();
	}
}
		
// Initialization!
MSP34xx::MSP34xx(I2C& bus,const BYTE Addr,const bool DolbyThroughDPL3518, const bool ExternalFMRadioTuner) :
	i2c(bus) , i2caddr(Addr) ,
	m_extdolby(DolbyThroughDPL3518)  , // Dolby isnt supported by all boards. (needs DPL 351xA)
	m_extfmradio(ExternalFMRadioTuner),						// Most cards use the TV tuner as an FM radio tuner
	m_nicam(false)	,	// Start assuming no NICAM
	m_equalizer(false), // Start assuming no graphic equalizer available
	m_mode(FM_TERRA) ,
	m_nicamkick(false),
	m_autofmam(false) ,
	m_nicam_on(false) ,
	m_audiomodesAv(0) ,
	m_norm(KS_AnalogVideo_NTSC_M) ,
	m_left(65535),
	m_right(65535),
	m_bass(32768),
	m_treble(32768),
	m_currconn(AudioConTuner),		// TV tuner by default
	m_mspwithdolby(false),			// By default MSP34xx does NOT have an integrated Dolby processor
	m_dolbymode(DPL_Through),		// Dolby processor mode (if the MSP34xx has it...)
	m_dolbyplmode(DPLMPl_Normal),	// No special Dolby mode by default...
	m_dolbypan(0),					// Current Panorama
	m_dolbyspatial(0),				// Current spatial
	m_dolbynoise(0xb0),				// Current noise mode
	t_action(SuspendAutodetect),
	t_pendingcmds(0L),				// Pending commands.
	t_inautodetection(0L),			// Autodetection lock initialized to No lock...
	desiredam(Am_Stereo)
{
	// ensure all bands start at neutral
	for (int i = 0;i < 5; i++) {
		m_eqband[i] = 32768;
	}

	// Create the autodetection Thread
	bool ret = CreateThread(ThreadProc,this);
	if (!ret) { // if failure, simulate thread termination
		t_action = Exit;
		t_ended.Set();
	} 

	// And reinit the chip
	Restart();
}

MSP34xx::~MSP34xx()
{
	// Make sure to end Thread now!
	if (t_action != Exit) {
		// Reset ended event
		t_ended.Reset();

		// Post our exit command
		t_action = Exit;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();

		// And wait until the exit command is completed
		t_ended.WaitFor();
	}
}


// Interfase to the Digitizer
// XFace fn:
void MSP34xx::SetAudioConnector(AudioConnector conn) 
{
	// Only if changing...
	if( m_currconn != conn) {

		// Remember the current connector 
		m_currconn = conn;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();
	}
}

/* -32767 , 0 = 0Db, 32767 */
// XFace fn:
void MSP34xx::SetVolume(const int Left,const int Right) 
{
	int xm_left = (Left+32768) * 2; // As internal representation requires 65535 = 0db
	int xm_right = (Right+32768) * 2;
	if (xm_left > 65535) xm_left = 65535;
	if (xm_right > 65535) xm_right = 65535;

	// Store left and right volume...
	m_left = xm_left;
	m_right = xm_right;

	// Do we have access to the MSP registers (autodetection is NOT in progress) ?
	if (InterlockedIncrement((LONG*)&t_inautodetection) == 1) {

		// Yes, noone is using it; Set Volume!
		if (m_currconn == AudioConMute) {
			SetScart(SCART_MUTE,0); // Mute
			_SetVolume(0,0);
		} else {
			_SetVolume(m_left,m_right);
		}

	} // If no, autodetection will set values for us.

	// Release register use.
	InterlockedDecrement((LONG*)&t_inautodetection);
}

/* -32767 , 0 = 0Db, 32767 */
// Xface fn:
void MSP34xx::SetBassTreble(const int Bass,const int Treble) 
{
	int xbass = Bass + 32768;
	int xtreble = Treble + 32768;
	if (xbass > 65535) xbass = 65535;
	if (xbass < 0) xbass = 0;
	if (xtreble > 65535) xtreble = 65535;
	if (xtreble < 0) xtreble = 0;

	// Store bass and treble...
	m_bass = xbass;
	m_treble = xtreble; // Internal representation uses 32768 as 0Db

	// Do we have access to the MSP registers (autodetection is NOT in progress) ?
	if (InterlockedIncrement((LONG*)&t_inautodetection) == 1) {

		// Yes, noone is using it; Set Bass and Treble
		_SetBass(m_bass);
		_SetTreble(m_treble);

	}// If no, autodetection will set values for us.

	// Release register use.
	InterlockedDecrement((LONG*)&t_inautodetection);
}

// Xface fn:
void MSP34xx::SetAudioMode(AudioMode audio) 
{
	// Remember the new desired mode
	desiredam = audio;

	// Do we have access to the MSP registers (autodetection is NOT in progress) ?
	if (InterlockedIncrement((LONG*)&t_inautodetection) == 1) {

		// Autodetect stereo/bilingual/SAP modes... Will also set the desired mode...
		WatchStereo(true);

	}// If no, autodetection will set values for us.

	// Release register use.
	InterlockedDecrement((LONG*)&t_inautodetection);
}

// XFace fn:
void MSP34xx::VideoStandardChanged(KS_AnalogVideoStandard vf)
{
	// Remember the new standard (this requires a new autodetection)
	m_norm = vf;

	// Increment number of pending cmds
	InterlockedIncrement((LONG*)&t_pendingcmds);

	// Schedule command execution
	t_exec.Signal();
}

// Called when videoChannel has changed
// XFace fn:
void MSP34xx::ChannelChanged() 
{
	// Trigger a new autodetection (this requires a new autodetection)

	// Increment number of pending cmds
	InterlockedIncrement((LONG*)&t_pendingcmds);

	// Schedule command execution
	t_exec.Signal();
}

// XFace fn:
AudioModes MSP34xx::AudioModesDetected()
{
	// Do we have access to the MSP registers (autodetection is NOT in progress) ?
	if (InterlockedIncrement((LONG*)&t_inautodetection) == 1) {

		// Autodetect stereo/bilingual/SAP modes... This will update available options
		WatchStereo();

	}// If no, autodetection will set values for us.

	// Release register use.
	InterlockedDecrement((LONG*)&t_inautodetection);

	// And return Detected modes.
	return m_audiomodesAv;

}

// Xface fn:
void MSP34xx::SetEqualizer(LONG* Bands) // (minimum = -32768, maximum = 32767
{
	for (int i=0;i< 5;i++) {
		int val = Bands[i] + 32768;
		if (val > 65535) val = 65535;
		if (val < 0) val = 0;
		m_eqband[i] = val;
	}

	// Do we have access to the MSP registers (autodetection is NOT in progress) ?
	if (InterlockedIncrement((LONG*)&t_inautodetection) == 1) {

		// Set equalizer bands...
		_SetEQ(m_eqband);

	}// If no, autodetection will set values for us.

	// Release register use.
	InterlockedDecrement((LONG*)&t_inautodetection);

}

DWORD MSP34xx::EqualizerBandsAvailable(DWORD* freqs)
{
	freqs[0] = 120; // Hz
	freqs[1] = 500;
	freqs[2] = 1500;
	freqs[3] = 5000;
	freqs[4] = 10000;
	return 5;
}

void MSP34xx::SetDolby(DOLBY_PARAMS& params)
{
	// Just store Dolby Settings...
	m_dolbymode = DPLMode(params.mode);
	m_dolbynoise = params.noise;
	m_dolbypan = params.pan;
	m_dolbyplmode = DPLPlMode(params.plmode);
	m_dolbyspatial = params.spatial;

	// Do we have access to the MSP registers (autodetection is NOT in progress) ?
	if (InterlockedIncrement((LONG*)&t_inautodetection) == 1) {

		// And set the chip according to it!
		_SetDolby();

	}// If no, autodetection will set values for us.

	// Release register use.
	InterlockedDecrement((LONG*)&t_inautodetection);
}
