/*
	MSP3400.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 * programming the msp34* sound processor family
 */

/* B version does NOT contain an Equalizer, all the newer ones have*/

#include "HwAudio.hpp"
#include "SyncEvent.hpp"
#include "NotifyEvent.hpp"

#define I2C_MSP3400C_0					0x80
#define I2C_MSP3400C_1_UNUSEDBYANY		0x84
#define I2C_MSP3400C_2_UNUSEDBYANY		0x88

#define I2C_MSP3400C_DEM   0x10
#define I2C_MSP3400C_DFP   0x12

class I2C;

class MSP34xx : public HwAudio {

	protected:

		// Commands the thread understands
		typedef enum {
			Autodetect,			// Do autodetection
			SuspendAutodetect,	// Suspend execution
			Exit				// Exit thread
		} Action;

	public:

		typedef enum {
			AM_DETECT   =0,
			FM_RADIO    =2,
			FM_TERRA    =3,
			FM_SAT      =4,
			FM_NICAM1   =5,
			FM_NICAM2   =6,
			AM_NICAM    =7,
			BTSC        =8	// Only supported on newer parts...
		} MSPMode;

		// Initialization info for each mode 
		typedef struct {
			WORD fir1[6];
			WORD fir2[6];
			DWORD cdo1;
			DWORD cdo2;
			WORD ad_cv;
			WORD mode_reg;
			WORD dfp_src;
			WORD dfp_matrix;
		} INIT_DATA_DEM;

		typedef struct {
			DWORD	cdo;	// Carrier frequency
			MSPMode mode;	// Decoding mode
#if DBG		// Only if debugging...
			char *name;
#endif
		} SUBCARRIER_DETECT;

		typedef struct {
			DWORD				norms;		// Supported norms...
			DWORD				cdo;		// Main audio carrier...
			SUBCARRIER_DETECT*	subcarrier;	// Pointer to the possible subcarrier freqs
			DWORD				count;		// Count of subcarriers
#if DBG		// Only if debugging...
			char *				name;
#endif
		} MAINCARRIER_DETECT;

#if DBG		
		typedef struct {
			WORD  retval;
			DWORD main, second;
			char *name;
		} MODES;
#endif

	private: 

		// Operational modes
		bool					m_extdolby;			// Using external Dolby Processos
		bool					m_extfmradio;		// Using external FM radio...
		bool					m_nicam;			// Nicam available
		bool					m_equalizer;		// Equalizer available
		MSPMode					m_mode;				// current MSP mode
		bool					m_nicamkick;		// If the MSP version requires a NICAN start command.
		bool					m_autofmam;			// If it has an AUTOFM/AM feature
		bool					m_nicam_on;			// Nicam is active
		bool					m_simple;			// Using simple autodetection/mode setting
		AudioModes				m_audiomodesAv;		// Available Audiomodes...
		volatile KS_AnalogVideoStandard	m_norm;		// Current TV norm (Must be actualized!=
		int						m_left;
		int						m_right;
		int						m_bass;
		int						m_treble;
		int						m_eqband[5];		// The audio equalizer bands
		WORD					m_acb;				// SCART
		volatile AudioConnector	m_currconn;			// Current audio connector.
		bool					m_watch_stereo;		// If stereo must be watched!

		// Dolby Parameters
		bool					m_mspwithdolby;		// MSP34xx has an integrated Dolby processor...
		DPLMode					m_dolbymode;		// Dolby processor mode (if the MSP34xx has it...)
		DPLPlMode				m_dolbyplmode;		// Current Prologic mode
		BYTE					m_dolbypan;			// Current Panorama
		BYTE					m_dolbyspatial;		// Current spatial
		BYTE					m_dolbynoise;		// Current noise mode

		// I2C Bus and Address
		I2C&					i2c;				// Bus
		const BYTE				i2caddr;			// Direccin I2C

		// For thread communication
		SyncEvent				t_exec;				// Used to signal thread that must do something
		volatile LONG			t_pendingcmds;		// Pending commands...
		volatile LONG			t_inautodetection;	// We use a lock counter to make sure we don't interfere with Autodetection procedures...
		volatile Action			t_action;			// Action thread must do
		NotifyEvent				t_suspended;		// Used to signal thread is suspended
		NotifyEvent				t_ended;			// Used to signal thread is ended
		AudioMode				desiredam;			// The desired audio mode

	private:

		// Private functions
		bool Reset();
		WORD Read(BYTE subaddr, WORD addr);
		bool Write(BYTE subaddr, WORD addr, WORD val);
		void SetScart(int in, int out);

		void _SetVolume(const int left, const int right);
		void _SetBass(const int bass);
		void _SetTreble(const int treble);
		void _SetEQ(const int* bands);
		void _SetDolby(void);

		void SetCarrier3400(const DWORD cdo1, const DWORD cdo2);
		void SetMode3400(MSPMode type);
		void SetStereoMode3400(AudioMode mode);
		void AutodetectMode3400();
		bool AutodetectStereoModes3400();

		void SetStereoMode34x1G(AudioMode mode);
		void AutodetectMode34x1G();
		bool AutodetectStereoModes34x1G();

		// Usables
		void AutodetectMode();
		void SetStereoMode(AudioMode mode);
		void WatchStereo(bool ForceSet = false);
		bool AutodetectStereoModes();

	protected:
		static VOID ThreadProc(IN PVOID ctx);
		void AutodetectProc();

	public:

		// Initialization!
		MSP34xx(I2C& bus,const BYTE Addr,const bool DolbyThroughDPL3518 = false, const bool ExternalFMRadiotuner = true);
		virtual ~MSP34xx();

		// Called when VideoNorm changed!
		virtual void VideoStandardChanged(KS_AnalogVideoStandard vf);

		// Called when videoChannel has changed
		virtual void ChannelChanged();

		// Called when connector needs to be changed
		virtual void SetAudioConnector(AudioConnector conn);

		/* -32767 , 0 = 0Db, 32767 */
		virtual void SetVolume(const int Left,const int Right);

		/* -32767 , 0 = 0Db, 32767 */
		virtual void SetBassTreble(const int Bass,const int Treble);

		// Called to change AudioMode
		virtual void SetAudioMode(AudioMode audio);

		virtual bool IsVolumeAvailable()	{ return true; }
		virtual bool IsBassTrebleAvailable(){ return !m_equalizer; }
		virtual bool IsLoudnessAvailable()	{ return false; }

		// Audiomodes
		virtual AudioModes AudioModesAvailable() { return (Am_Mono | Am_Stereo | Am_Lang1 | Am_Lang2); }
		virtual AudioModes AudioModesDetected();

		// To support Hibernate mode
		virtual void Restart( void );
		virtual void Suspend( void );

		// To set the equalizer
		virtual void SetEqualizer(LONG* Bands);
		virtual DWORD EqualizerBandsAvailable(DWORD* freqs);

		// Yes! - We are a (pseudo)Dolby chip (unless an external Dolby decoder is used)
		virtual bool IsDolbyAvailable()		{ return (m_extdolby == false); }

		// And the fn we need to set those params!
		virtual void SetDolby(DOLBY_PARAMS& params);


};

