/*
	TDA7432.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* The TDA7432 is made by STS-Thompson
 * http://www.st.com
 * http://us.st.com/stonline/books/pdf/docs/4056.pdf
 *
 * TDA7432: I2C-bus controlled basic audio processor
 *
 * The TDA7432 controls basic audio functions like volume, balance,
 * and tone control (including loudness).  It also has four channel
 * output (for front and rear).  Since most vidcap cards probably
 * don't have 4 channel output, this driver will set front & rear
 * together (no independent control).
 */ 


#include "TDA7432.hpp"
#include "..\HwCapture\Pisces.hpp"


	/* Subaddresses for TDA7432 */

#define TDA7432_IN	0x00 /* Input select                 */
#define TDA7432_VL	0x01 /* Volume                       */
#define TDA7432_TN	0x02 /* Bass, Treble (Tone)          */
#define TDA7432_LF	0x03 /* Attenuation LF (Left Front)  */
#define TDA7432_LR	0x04 /* Attenuation LR (Left Rear)   */
#define TDA7432_RF	0x05 /* Attenuation RF (Right Front) */
#define TDA7432_RR	0x06 /* Attenuation RR (Right Rear)  */
#define TDA7432_LD	0x07 /* Loudness                     */


		/* Masks for bits in TDA7432 subaddresses */

/* Many of these not used - just for documentation */

/* Subaddress 0x00 - Input selection and bass control */

/* Bits 0,1,2 control input:
 * 0x00 - Stereo input
 * 0x02 - Mono input
 * 0x03 - Mute
 * Mono probably isn't used - I'm guessing only the stereo
 * input is connected on most cards, so we'll set it to stereo.
 * 
 * Bit 3 controls bass cut: 0/1 is non-symmetric/symmetric bass cut
 * Bit 4 controls bass range: 0/1 is extended/standard bass range
 * 
 * Highest 3 bits not used
 */
 
#define TDA7432_STEREO_IN	0
#define TDA7432_MONO_IN		2	/* Probably won't be used */
#define TDA7432_MUTE		3	/* Probably won't be used */
#define TDA7432_BASS_SYM	1 << 3
#define TDA7432_BASS_NORM	1 << 4

/* Subaddress 0x01 - Volume */

/* Lower 7 bits control volume from -79dB to +32dB in 1dB steps
 * Recommended maximum is +20 dB
 *
 * +32dB: 0x00
 * +20dB: 0x0c
 *   0dB: 0x20
 * -79dB: 0x6f
 *
 * MSB (bit 7) controls loudness: 1/0 is loudness on/off
 */

#define	TDA7432_VOL_0DB		0x20
#define TDA7432_LD_ON		1 << 7


/* Subaddress 0x02 - Tone control */ 

/* Bits 0,1,2 control absolute treble gain from 0dB to 14dB
 * 0x0 is 14dB, 0x7 is 0dB
 *
 * Bit 3 controls treble attenuation/gain (sign)
 * 1 = gain (+)
 * 0 = attenuation (-)
 *
 * Bits 4,5,6 control absolute bass gain from 0dB to 14dB
 * (This is only true for normal base range, set in 0x00)
 * 0x0 << 4 is 14dB, 0x7 is 0dB
 * 
 * Bit 7 controls bass attenuation/gain (sign)
 * 1 << 7 = gain (+)
 * 0 << 7 = attenuation (-) 
 *
 * Example:
 * 1 1 0 1 0 1 0 1 is +4dB bass, -4dB treble
 */

#define TDA7432_TREBLE_0DB		0xf 
#define TDA7432_TREBLE			7
#define TDA7432_TREBLE_GAIN		1 << 3
#define TDA7432_BASS_0DB		0xf << 4
#define TDA7432_BASS			7 << 4
#define TDA7432_BASS_GAIN		1 << 7

 
/* Subaddress 0x03 - Left  Front attenuation */
/* Subaddress 0x04 - Left  Rear  attenuation */
/* Subaddress 0x05 - Right Front attenuation */
/* Subaddress 0x06 - Right Rear  attenuation */

/* Bits 0,1,2,3,4 control attenuation from 0dB to -37.5dB
 * in 1.5dB steps.
 *
 * 0x00 is     0dB
 * 0x1f is -37.5dB
 *
 * Bit 5 mutes that channel when set (1 = mute, 0 = unmute)
 * We'll use the mute on the input, though (above) 
 * Bits 6,7 unused
 */

#define TDA7432_ATTEN_0DB	0x00


/* Subaddress 0x07 - Loudness Control */

/* Bits 0,1,2,3 control loudness from 0dB to -15dB in 1dB steps
 * when bit 4 is NOT set
 *
 * 0x0 is   0dB
 * 0xf is -15dB
 *
 * If bit 4 is set, then there is a flat attenuation according to
 * the lower 4 bits, as above.
 *
 * Bits 5,6,7 unused
 */

/*
 * TDA7432 attenuation: min (0dB) is 0, max (-37.5dB) is 0x1f
 */

void TDA7432::Restart()
{
	i2c.Write(i2caddr, TDA7432_IN , 
		BYTE(
		(( Muted ) ? TDA7432_MUTE : TDA7432_STEREO_IN)	|	/* Main (stereo) input or muted  */
		TDA7432_BASS_SYM  |									/* Symmetric bass cut    */
		TDA7432_BASS_NORM									/* Normal bass range     */ 
		) ); 

	i2c.Write(i2caddr, TDA7432_VL ,
		TDA7432_VOL_0DB 	 /* 0dB Volume            */
		| TDA7432_LD_ON   ); /* Turn loudness on?     */

	i2c.Write(i2caddr, TDA7432_TN ,
		TDA7432_TREBLE_0DB | /* 0dB Treble            */
		TDA7432_BASS_0DB); 	 /* 0dB Bass              */

	i2c.Write(i2caddr, TDA7432_LF ,
		TDA7432_ATTEN_0DB);	 /* 0dB attenuation       */

	i2c.Write(i2caddr, TDA7432_LR ,
		TDA7432_ATTEN_0DB);	 /* 0dB attenuation       */

	i2c.Write(i2caddr, TDA7432_RF,
		TDA7432_ATTEN_0DB);	 /* 0dB attenuation       */

	i2c.Write(i2caddr, TDA7432_RR,
		TDA7432_ATTEN_0DB);	 /* 0dB attenuation       */

	i2c.Write(i2caddr, TDA7432_LD,
		1);					/* insmod parameter      */
}

TDA7432::TDA7432(I2C& bus,const BYTE Addr) 
	: MainVol(0) , Loudness(0) , i2c(bus), i2caddr(Addr) , Muted(false)
{
	TDA7432::Restart();
}

// {0x0 -0x20- 0x6f
void TDA7432::SetVolume(const int Left,const int Right) 
{

	/* Lower 7 bits control volume from -79dB to +32dB in 1dB steps
	 * Recommended maximum is +20 dB
	 *
	 * +32dB: 0x00
	 * +20dB: 0x0c
	 *   0dB: 0x20
	 * -79dB: 0x6f
	 *
	 * MSB (bit 7) controls loudness: 1/0 is loudness on/off
	 */

	MainVol = max(Left,Right); // Get maximum volume!
	// Limit range!
	if (MainVol > 13280) MainVol = 13280;
	// Calculate master volume
	const BYTE mv = (0x20 - (MainVol / 415)) |
		((Loudness) ? TDA7432_LD_ON : 0)	;

	i2c.Write(i2caddr, TDA7432_VL , mv);

	// Calculate difference for each channel
	int DifLeft = (Left - MainVol) * 3 / 2;
	int DifRight = (Right - MainVol) * 3 / 2;

	/* Bits 0,1,2,3,4 control attenuation from 0dB to -37.5dB
	 * in 1.5dB steps.
	 *
	 * 0x00 is     0dB
	 * 0x1f is -37.5dB
	 *
	 * Bit 5 mutes that channel when set (1 = mute, 0 = unmute)
	 */

	if (DifLeft  < -12865) DifLeft  = -12865;
	if (DifRight < -12865) DifRight = -12865;
	if (DifLeft  > 0) DifLeft  = 0;
	if (DifRight > 0) DifRight = 0;
	
	const BYTE lv = (- (DifLeft /415)) | ((DifLeft  == -12865) ? 0x20 : 0);
	const BYTE rv = (- (DifRight/415)) | ((DifRight == -12865) ? 0x20 : 0);

	i2c.Write(i2caddr, TDA7432_LF , lv);
	i2c.Write(i2caddr, TDA7432_LR , lv);
	i2c.Write(i2caddr, TDA7432_RF , rv);
	i2c.Write(i2caddr, TDA7432_RR , rv);
} 

void TDA7432::SetBassTreble(const int Bass,const int Treble) 
{
	/* Subaddress 0x02 - Tone control */ 

	/* Bits 0,1,2 control absolute treble gain from 0dB to 14dB
	 * 0x0 is 14dB, 0x7 is 0dB
	 *
	 * Bit 3 controls treble attenuation/gain (sign)
	 * 1 = gain (+)
	 * 0 = attenuation (-)
	 *
	 * Bits 4,5,6 control absolute bass gain from 0dB to 14dB
	 * (This is only true for normal base range, set in 0x00)
	 * 0x0 << 4 is 14dB, 0x7 is 0dB
	 * 
	 * Bit 7 controls bass attenuation/gain (sign)
	 * 1 << 7 = gain (+)
	 * 0 << 7 = attenuation (-) 
	 *
	 * Example:
	 * 1 1 0 1 0 1 0 1 is +4dB bass, -4dB treble
	 */
	const BYTE tone = 
		(( Bass >= 0 )   ? 0x80 : 0x0) | ((7 - (abs(Bass)   / 4681) ) << 4) |
		(( Treble >= 0 ) ? 0x08 : 0x0) | ((7 - (abs(Treble) / 4681) )     ) ;

	i2c.Write(i2caddr, TDA7432_TN , tone);
}

/* -32767 , 0 = 0Db ,32767 */
void TDA7432::SetLoudness(const int Loudness_) 
{
	/* Subaddress 0x07 - Loudness Control */

	/* Bits 0,1,2,3 control loudness from 0dB to -15dB in 1dB steps
	 * when bit 4 is NOT set
	 *
	 * 0x0 is   0dB
	 * 0xf is -15dB
	 *
	 * If bit 4 is set, then there is a flat attenuation according to
	 * the lower 4 bits, as above.
	 *
	 * Bits 5,6,7 unused
	 */

	Loudness = Loudness_;
	if (Loudness > 0) Loudness = 0;
	const BYTE v = -Loudness / 2048;
	i2c.Write(i2caddr, TDA7432_LD,v);	

	const BYTE mv = (0x20 - (MainVol / 415)) |
		((Loudness) ? TDA7432_LD_ON : 0)	;

	i2c.Write(i2caddr, TDA7432_VL , mv);
}

// Selection of Audio Connections
void TDA7432::SetAudioConnector(AudioConnector conn)
{
	switch(conn) {
		case AudioConTuner:
		case AudioConRadio:
		case AudioConLineIn:
			Muted = false;
			break;
		case AudioConMute:
			Muted = true;
			break;
	}

	// Mute or unmute the audio!
	i2c.Write(i2caddr, TDA7432_IN ,
		BYTE(
		(( Muted ) ? TDA7432_MUTE : TDA7432_STEREO_IN)	|	/* Main (stereo) input or muted  */
		TDA7432_BASS_SYM  |									/* Symmetric bass cut    */
		TDA7432_BASS_NORM									/* Normal bass range     */ 
		) ); 
}
