/*
	TDA9850.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TDA9850.hpp"
#include "..\HwCapture\Pisces.hpp"

/* For the TDA9850
 * (The TDA9855 is used on the Diamond DTV2000 and the TDA9850 is used 
 * on STB cards.  Other cards probably use these chips as well.)
 */
 
		/* subaddresses for TDA9850 */
#define TDA9850_C4	0x04 /* Control 1 for TDA9850 */
#define TDA9850_C5	0x05 /* Control 2 for TDA9850 */
#define TDA9850_C6	0x06 /* Control 3 for TDA9850 */
#define TDA9850_C7	0x07 /* Control 4 for TDA9850 */
#define TDA9850_A1	0x08 /* Alignment 1 for both chips */
#define TDA9850_A2	0x09 /* Alignment 2 for both chips */
#define TDA9850_A3	0x0a /* Alignment 3 for both chips */


/* Masks for bits in subaddresses */

/* 0x04 - Control 1 in TDA9850 */
/* Unique to TDA9850: */
/* lower 4 bits control stereo noise threshold, over which stereo turns off
 * set to values of 0x00 through 0x0f for Ster1 through Ster16 */


/* 0x05 - Control 2 in TDA9850*/
/* lower 4 bits contol SAP noise threshold, over which SAP turns off
 * set to values of 0x00 through 0x0f for SAP1 through SAP16 */


/* 0x06 - Control 3 in TDA9850 */
#define TDA9850_SAP		3<<6 /* Selects SAP output, mute if not received */
#define TDA9850_STEREO	1<<6 /* Selects Stereo output, mono if not received */
#define TDA9850_MONO	0    /* Forces Mono output */
#define TDA9850_LMU		1<<3 /* Mute (OUTL/OUTR for 9850) */

/* 0x07 - Control 4 in TDA9850 */
/* lower 4 bits control input gain from -3.5dB (0x0) to 4dB (0xF)
 * in .5dB steps -  0dB is 0x7 */


/* 0x08, 0x09 - A1 and A2 (read/write) */
/* lower 5 bits are wideband and spectral expander alignment
 * from 0x00 to 0x1f - nominal at 0x0f and 0x10 (read/write) */
#define TDA9850_STP		1<<5 /* Stereo Pilot/detect (read-only) */
#define TDA9850_SAPP	1<<6 /* SAP Pilot/detect (read-only) */
#define TDA9850_STS		1<<7 /* Stereo trigger 1= <35mV 0= <30mV (write-only)*/


/* 0x0a - A3 */
/* lower 3 bits control timing current for alignment: -30% (0x0), -20% (0x1),
 * -10% (0x2), nominal (0x3), +10% (0x6), +20% (0x5), +30% (0x4) */
#define TDA9850_ADJ	1<<7 /* Stereo adjust on/off (wideband and spectral */

void TDA9850::Restart()
{
	i2c.Write(i2caddr, TDA9850_C4, 0x08);		/* Set stereo noise thresh to nominal */
	i2c.Write(i2caddr, TDA9850_C5, 0x08);		/* Set SAP noise threshold to nominal */
	i2c.Write(i2caddr, TDA9850_C6, TDA9850_STEREO);	/* Select Stereo mode for decoder */
	i2c.Write(i2caddr, TDA9850_C7, 0x07);		/* 0dB input gain */
	i2c.Write(i2caddr, TDA9850_A1, 0x10);		/* Select nominal wideband expander */
	i2c.Write(i2caddr, TDA9850_A2, 0x10);		/* Select nominal spectral expander and 30mV trigger */
	i2c.Write(i2caddr, TDA9850_A3, 0x3);		/* Set: nominal timing current, 420ohm AVL attack */
}

TDA9850::TDA9850(I2C& bus,const BYTE Addr)
	: i2c(bus) , i2caddr(Addr)
{
	TDA9850::Restart();
}

void TDA9850::SetAudioMode(AudioMode audio) 
{
	if (audio == Am_Lang1) {
		i2c.Write(i2caddr, TDA9850_C6, TDA9850_SAP);
	} else 
	if (audio == Am_Stereo ) {
		i2c.Write(i2caddr, TDA9850_C6, TDA9850_STEREO);
	} else {
		i2c.Write(i2caddr, TDA9850_C6, TDA9850_MONO);
	}
}

AudioModes TDA9850::AudioModesDetected()
{
	BYTE val = 0;
	i2c.Read(i2caddr,&val);

	return (Am_Mono |
			(((val & TDA9850_STP ) != 0) ? Am_Stereo : 0) |
			(((val & TDA9850_SAPP) != 0) ? Am_Lang1  : 0)
			);

}


