/*
	TDA9855.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TDA9855.hpp"
#include "..\HwCapture\Pisces.hpp"

/*
 * TDA9855
 */

		/* subaddresses for TDA9855 */
#define TDA9855_VR	0x00 /* Volume, right */
#define TDA9855_VL	0x01 /* Volume, left */
#define TDA9855_BA	0x02 /* Bass */
#define TDA9855_TR	0x03 /* Treble */
#define TDA9855_SW	0x04 /* Subwoofer - not connected on DTV2000 */

		/* subaddesses for both chips */
#define TDA9855_C5	0x05 /* Control 1 for TDA9855 */
#define TDA9855_C6	0x06 /* Control 2 for TDA9855 */
#define TDA9855_C7	0x07 /* Control 3 for TDA9855 */
#define TDA9855_A1	0x08 /* Alignment 1 for both chips */
#define TDA9855_A2	0x09 /* Alignment 2 for both chips */
#define TDA9855_A3	0x0a /* Alignment 3 for both chips */

		/* Masks for bits in TDA9855 subaddresses */
/* 0x00 - VR in TDA9855 */
/* 0x01 - VL in TDA9855 */
/* lower 7 bits control gain from -71dB (0x28) to 16dB (0x7f)
 * in 1dB steps - mute is 0x27 */


/* 0x02 - BA in TDA9855 */ 
/* lower 5 bits control bass gain from -12dB (0x06) to 16.5dB (0x19)
 * in .5dB steps - 0 is 0x0E */


/* 0x03 - TR in TDA9855 */
/* 4 bits << 1 control treble gain from -12dB (0x3) to 12dB (0xb)
 * in 3dB steps - 0 is 0x7 */

		/* Masks for bits in both chips' subaddresses */
/* 0x04 - SW in TDA9855, */
/* Unique to TDA9855: */
/* 4 bits << 2 control subwoofer/surround gain from -14db (0x1) to 14db (0xf)
 * in 3dB steps - mute is 0x0 */
 

/* 0x05 - C5 - Control 1 in TDA9855 */
/* Unique to TDA9855: */
#define TDA9855_MUTE	1<<7 /* GMU, Mute at outputs */
#define TDA9855_AVL	1<<6 /* AVL, Automatic Volume Level */
#define TDA9855_LOUD	1<<5 /* Loudness, 1==off */
#define TDA9855_SUR	1<<3 /* Surround / Subwoofer 1==.5(L-R) 0==.5(L+R) */
			     /* Bits 0 to 3 select various combinations
                              * of line in and line out, only the 
                              * interesting ones are defined */
#define TDA9855_EXT	1<<2 /* Selects inputs LIR and LIL.  Pins 41 & 12 */
#define TDA9855_INT	0    /* Selects inputs LOR and LOL.  (internal) */

/* 0x06 - C6 - Control 2 in TDA9855, Control 3 in TDA9850 */
#define TDA9855_SAP	3<<6 /* Selects SAP output, mute if not received */
#define TDA9855_STEREO	1<<6 /* Selects Stereo ouput, mono if not received */
#define TDA9855_MONO	0    /* Forces Mono output */
#define TDA9855_LMU	1<<3 /* Mute (LOR/LOL for 9855, OUTL/OUTR for 9850) */

/* Unique to TDA9855: */
#define TDA9855_TZCM	1<<5 /* If set, don't mute till zero crossing */
#define TDA9855_VZCM	1<<4 /* If set, don't change volume till zero crossing*/
#define TDA9855_LINEAR	0    /* Linear Stereo */
#define TDA9855_PSEUDO	1    /* Pseudo Stereo */
#define TDA9855_SPAT_30	2    /* Spatial Stereo, 30% anti-phase crosstalk */
#define TDA9855_SPAT_50	3    /* Spatial Stereo, 52% anti-phase crosstalk */
#define TDA9855_E_MONO	7    /* Forced mono - mono select elseware, so useless*/


/* 0x07 - C7 - Control 3 in TDA9855 */
/* lower 4 bits control input gain from -3.5dB (0x0) to 4dB (0xF)
 * in .5dB steps -  0dB is 0x7 */


/* 0x08, 0x09 - A1 and A2 (read/write) */
/* lower 5 bites are wideband and spectral expander alignment
 * from 0x00 to 0x1f - nominal at 0x0f and 0x10 (read/write) */
#define TDA9855_STP	1<<5 /* Stereo Pilot/detect (read-only) */
#define TDA9855_SAPP	1<<6 /* SAP Pilot/detect (read-only) */
#define TDA9855_STS	1<<7 /* Stereo trigger 1= <35mV 0= <30mV (write-only)*/


/* 0x0a - A3 */
/* lower 3 bits control timing current for alignment: -30% (0x0), -20% (0x1),
 * -10% (0x2), nominal (0x3), +10% (0x6), +20% (0x5), +30% (0x4) */
#define TDA9855_ADJ	1<<7 /* Stereo adjust on/off (wideband and spectral */

/* 2 bits << 5 control AVL attack time: 420ohm (0x0), 730ohm (0x2), 
 * 1200ohm (0x1), 2100ohm (0x3) */

void TDA9855::Restart()
{
	i2c.Write(i2caddr, TDA9855_VR, 0x6f);		/* 0dB */
	i2c.Write(i2caddr, TDA9855_VL, 0x6f);		/* 0dB */
	i2c.Write(i2caddr, TDA9855_BA, 0x0e);		/* 0dB */
	i2c.Write(i2caddr, TDA9855_TR, (0x07 << 1));	/* 0dB */
	i2c.Write(i2caddr, TDA9855_SW, 0x8 << 2);	/* 0dB */
	i2c.Write(i2caddr, TDA9855_C5, 
		TDA9855_MUTE | TDA9855_AVL | 
		TDA9855_LOUD | TDA9855_INT );

	/* Set Mute, AVL, Loudness off, Internal sound */
	i2c.Write(i2caddr, TDA9855_C6,
		TDA9855_STEREO | TDA9855_LINEAR |
		TDA9855_TZCM | TDA9855_VZCM );

	/* Stereo linear mode, also wait til zero crossings  */
	i2c.Write(i2caddr, TDA9855_C7,0x07);		/* 0dB input gain */
	
	i2c.Write(i2caddr, TDA9855_A1, 0x10);		/* Select nominal wideband expander */
	i2c.Write(i2caddr, TDA9855_A2, 0x10);		/* Select nominal spectral expander and 30mV trigger */
	i2c.Write(i2caddr, TDA9855_A3, 0x3);		/* Set: nominal timing current, 420ohm AVL attack */
}

TDA9855::TDA9855(I2C& bus,const BYTE Addr) :
	i2c(bus), i2caddr(Addr)
{
	TDA9855::Restart();
}

void TDA9855::SetAudioMode(AudioMode audio) 
{
	if (audio == Am_Lang1) {
		i2c.Write(i2caddr, TDA9855_C6, TDA9855_SAP |
			TDA9855_LINEAR |
			TDA9855_TZCM | TDA9855_VZCM	);
	} else 
	if (audio == Am_Stereo) {
		i2c.Write(i2caddr, TDA9855_C6,
			TDA9855_STEREO | TDA9855_LINEAR |
			TDA9855_TZCM | TDA9855_VZCM	);
	} else {
		i2c.Write(i2caddr, TDA9855_C6, TDA9855_MONO |
			TDA9855_LINEAR |
			TDA9855_TZCM | TDA9855_VZCM	);
	}
}

void TDA9855::SetVolume(const int Left,const int Right) /* 0 = Gain 1 */
{
	// 0x6F = 0db
	int AdjLeft  = 0x6F + Left  / 455;
	int AdjRight = 0x6F + Right / 455;
	AdjLeft  = (AdjLeft  > 0x7F ) ? 0x7F : AdjLeft ;
	AdjRight = (AdjRight > 0x7F ) ? 0x7F : AdjRight;

	i2c.Write(i2caddr, TDA9855_VL, BYTE(AdjLeft));
	i2c.Write(i2caddr, TDA9855_VR, BYTE(AdjRight));
} 

void TDA9855::SetBassTreble(const int Bass,const int Treble) 
{
	int AdjBass   =  0xE + Bass   / 2979;
	int AdjTreble = (0xE + Treble / 4096) & (~1);
	AdjBass  = (AdjBass  > 0x19 ) ? 0x19 : AdjBass ;
	AdjBass  = (AdjBass  < 0x06 ) ? 0x06 : AdjBass ;
	AdjTreble = (AdjTreble < 0x06 ) ? 0x06 : AdjTreble;
	AdjTreble = (AdjTreble > 0x16 ) ? 0x16 : AdjTreble;

	i2c.Write(i2caddr, TDA9855_BA, BYTE(AdjBass)); 
	i2c.Write(i2caddr, TDA9855_TR, BYTE(AdjTreble));
}

AudioModes TDA9855::AudioModesDetected()
{
	BYTE val = 0;
	i2c.Read(i2caddr,&val);

	return (Am_Mono |
			(((val & TDA9855_STP ) != 0) ? Am_Stereo : 0) |
			(((val & TDA9855_SAPP) != 0) ? Am_Lang1  : 0)
			);

}

