/*
	TDA9873.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

ChangeLog:

  01/21/2002  Sven Grothklags		Fixed Typhoon TView RDS card entry and TDA9873/74 code!

*/

#include "TDA9873.hpp"
#include "..\HwCapture\Pisces.hpp"

/* ---------------------------------------------------------------------- */
/* audio chip descriptions - defines+functions for tda9873h               */

/* Subaddresses for TDA9873H */

#define TDA9873_SW	0x00 /* Switching                    */
#define TDA9873_AD	0x01 /* Adjust                       */
#define TDA9873_PT	0x02 /* Port                         */

/* Subaddress 0x00: Switching Data 
 * B7..B0:
 *
 * B1, B0: Input source selection
 *  0,  0  internal
 *  1,  0  external stereo
 *  0,  1  external mono
 */
#define TDA9873_INTERNAL    0
#define TDA9873_EXT_STEREO  2
#define TDA9873_EXT_MONO    1

/*  B3, B2: output signal select
 *  B4    : transmission mode 
 *  0, 0, 1   Mono
 *  1, 0, 0   Stereo
 *  1, 1, 1   Stereo (reversed channel)    
 *  0, 0, 0   Dual AB
 *  0, 0, 1   Dual AA
 *  0, 1, 0   Dual BB
 *  0, 1, 1   Dual BA
 */

#define TDA9873_TR_MONO     4
#define TDA9873_TR_STEREO   1 << 4
#define TDA9873_TR_REVERSE  (1 << 3) & (1 << 2)
#define TDA9873_TR_DUALA    1 << 2
#define TDA9873_TR_DUALB    1 << 3

/* output level controls
 * B5:  output level switch (0 = reduced gain, 1 = normal gain)
 * B6:  mute                (1 = muted)
 * B7:  auto-mute           (1 = auto-mute enabled)
 */

#define TDA9873_GAIN_NORMAL 1 << 5
#define TDA9873_MUTE        1 << 6
#define TDA9873_AUTOMUTE    1 << 7

/* Subaddress 0x01:  Adjust/standard */

/* Lower 4 bits (C3..C0) control stereo adjustment on R channel (-0.6 - +0.7 dB)
 * Recommended value is +0 dB
 */

#define	TDA9873_STEREO_ADJ	0x06 /* 0dB gain */

/* Bits C6..C4 control FM stantard  
 * C6, C5, C4
 *  0,  0,  0   B/G (PAL FM)
 *  0,  0,  1   M
 *  0,  1,  0   D/K(1)
 *  0,  1,  1   D/K(2)
 *  1,  0,  0   D/K(3)
 *  1,  0,  1   I
 */
#define TDA9873_BG		0
#define TDA9873_M       1
#define TDA9873_DK1     2
#define TDA9873_DK2     3
#define TDA9873_DK3     4
#define TDA9873_I       5

/* C7 controls identification response time (1=fast/0=normal)
 */
#define TDA9873_IDR_NORM 0
#define TDA9873_IDR_FAST 1 << 7


/* Subaddress 0x02: Port data */ 

/* E1, E0   free programmable ports P1/P2
    0,  0   both ports low
    0,  1   P1 high
    1,  0   P2 high
    1,  1   both ports high
*/

#define TDA9873_PORTS    3

/* E2: test port */
#define TDA9873_TST_PORT 1 << 2

/* E5..E3 control mono output channel (together with transmission mode bit B4)
 *
 * E5 E4 E3 B4     OUTM
 *  0  0  0  0     mono
 *  0  0  1  0     DUAL B
 *  0  1  0  1     mono (from stereo decoder)
 */
#define TDA9873_MOUT_MONO   0
#define TDA9873_MOUT_FMONO  0
#define TDA9873_MOUT_DUALA  0 
#define TDA9873_MOUT_DUALB  1 << 3 
#define TDA9873_MOUT_ST     1 << 4 
#define TDA9873_MOUT_EXTM   (1 << 4 ) & (1 << 3)
#define TDA9873_MOUT_EXTL   1 << 5 
#define TDA9873_MOUT_EXTR   (1 << 5 ) & (1 << 3)
#define TDA9873_MOUT_EXTLR  (1 << 5 ) & (1 << 4)
#define TDA9873_MOUT_MUTE   (1 << 5 ) & (1 << 4) & (1 << 3)

/* Status bits: (chip read) */
#define TDA9873_PONR        0 /* Power-on reset detected if = 1 */
#define TDA9873_STEREO      2 /* Stereo sound is identified     */
#define TDA9873_DUAL        4 /* Dual sound is identified       */

TDA9873::ANInfo TDA9873::Info[] = 
{
	{ TN_CCIR_BGHN, TDA9873_STEREO_ADJ | (TDA9873_BG  << 4) },
	{ TN_CCIR_MC  , TDA9873_STEREO_ADJ | (TDA9873_M   << 4) },
	{ TN_CCIR_DKL , TDA9873_STEREO_ADJ | (TDA9873_DK1 << 4) },
	{ TN_CCIR_DKL , TDA9873_STEREO_ADJ | (TDA9873_DK2 << 4) },
	{ TN_CCIR_DKL , TDA9873_STEREO_ADJ | (TDA9873_DK3 << 4) },
	{ TN_CCIR_I   , TDA9873_STEREO_ADJ | (TDA9873_I   << 4) },
};
#define NUM_AUDIONORMS (sizeof(TDA9873::Info) / sizeof(TDA9873::Info[0]))

void TDA9873::Restart()
{
	if ( t_action == SuspendAutodetect ) { // Only if autodetect thread is suspending/suspended ...
		shadow[0] = 0xa4;
		shadow[1] = 0x06;

		// Init!
		i2c.Write(i2caddr, TDA9873_SW, shadow[0]);
		i2c.Write(i2caddr, TDA9873_AD, shadow[1]);
		i2c.Write(i2caddr, TDA9873_PT, 0x03);

		// Reinit autodetection
		t_action = Autodetect;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();
	}
}

// Wait until autodetection is complete!
void TDA9873::Suspend()
{
	if ( t_action == Autodetect ) { // Only if thread is autodetecting...
		// Clear the suspended notification...
		t_suspended.Reset();

		// Post our suspend command.
		t_action = SuspendAutodetect;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();

		// And wait until the suspension command is completed.
		t_suspended.WaitFor();
	}
}


TDA9873::TDA9873(I2C& bus,const BYTE Addr) :
	i2c(bus) , i2caddr(Addr) , 
	t_action(SuspendAutodetect),
	t_pendingcmds(0L),		// No pending cmds...
	std(0) , currVideoStandard(KS_AnalogVideo_None),
	desiredam(Am_Stereo)

{
	// Create the autodetection Thread
	bool ret = CreateThread(ThreadProc,this);
	if (!ret) { // if failure, simulate thread termination
		t_action = Exit;
		t_ended.Set();
	} 

	// And reinit the chip
	Restart();
}


TDA9873::~TDA9873()
{
	// Make sure to end Thread now!
	if (t_action != Exit) {
		// Reset ended event
		t_ended.Reset();

		// Post our exit command
		t_action = Exit;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();

		// And wait until the exit command is completed
		t_ended.WaitFor();
	}
}

// Out thread proc
VOID TDA9873::ThreadProc(IN PVOID ctx)
{
	TDA9873* ptr = (TDA9873*) ctx;
	ptr->AutodetectProc();

	// Signal we are ended (we do it from an static fn so no hidden this references are used) ...
	ptr->t_ended.Set();

	// And end the Thread!
	ExitThread();

}

void TDA9873::SetChipMode(AudioMode audio)
{
	BYTE tshadow = shadow[0] & 0xe3;
	if (audio == Am_Lang2) {
		tshadow |= TDA9873_TR_DUALB;
	} else 
	if (audio == Am_Lang1) {
		tshadow |= TDA9873_TR_DUALA;
	} else 
	if (audio == Am_Stereo) {
		tshadow |= TDA9873_TR_STEREO;
	} else {
		tshadow |= TDA9873_TR_MONO;
	}

	if(tshadow != shadow[0]) {
		shadow[0] = tshadow;
		i2c.Write(i2caddr, TDA9873_SW, shadow[0]);
	}
}

void TDA9873::AutodetectProc()
{
	while(1) {

		// Do we have some pending cmds ?
		while (InterlockedDecrement((LONG*)&t_pendingcmds) >= 0) {

			// We got it, check requested action.
			switch (t_action) {
				case Exit:
					return;
				case SuspendAutodetect:
					t_suspended.Set();
					break;
				case Autodetect:
				{
					// We are requested to start an autodetection, do it!
				
					// First try the current std if it applies...
					if (Info[std].Norms & currVideoStandard) {

						// Set the standard with fast detect
						shadow[1] = 0x80 | Info[std].Val;
						i2c.Write(i2caddr, TDA9873_AD, shadow[1]);

						// Wait 0.5 seconds before checking in 10 ms chunks
						int times;
						for (times = 0; times < 50 ; times++) {
							// Early cancel 
							if (t_action != Autodetect || 
								t_pendingcmds > 0) {
								break;
							}
							mdelay(10);
						}
						// Early cancel 
						if (times < 50) {
							break;
						}

						// If Stereo or Dual detected, assume standard was found!
						BYTE val = 0; i2c.Read(i2caddr,&val);
						if ((val & (TDA9873_DUAL | TDA9873_STEREO)) != 0) {
						
							// Set the autodetected standard
							shadow[1] = Info[std].Val;
							i2c.Write(i2caddr, TDA9873_AD, shadow[1]);
	
							// Wait 2 seconds before checking
							for (times = 0; times < 200 ; times++) {
								// Early cancel 
								if (t_action != Autodetect ||
									t_pendingcmds > 0) {
									break;
								}
								mdelay(10);
							}
							// Early cancel 
							if (times < 200) {
								break;
							}
	
							// Now check the desired Audio mode. If unavailable, revert to one available.
							AudioMode curr = desiredam;
							AudioModes detected = AudioModesDetected();
							if (curr == Am_Stereo && (detected & Am_Stereo) == 0) curr = Am_Lang1;
							if (curr == Am_Lang1 && (detected & Am_Lang1) == 0)   curr = Am_Lang2;
							if (curr == Am_Lang2 && (detected & Am_Lang2) == 0)   curr = Am_Mono;
	
							// Set the supported mode
							SetChipMode(curr);
							break;
						}
					}

					// Current standard didn't work... We will try all available to us.
					BYTE stdtry;
					for (stdtry = 0; 
						stdtry < NUM_AUDIONORMS && t_action == Autodetect; 
						stdtry++) {
	
						// If norm applies to this
						if (Info[stdtry].Norms & currVideoStandard) {
	
							// Assume this std will be as it applies to the current video norm
							std = stdtry;
	
							// Set the standard with fast detect
							shadow[1] = 0x80 | Info[stdtry].Val;
							i2c.Write(i2caddr, TDA9873_AD, shadow[1]);
	
							// Wait 0.5 seconds before checking
							int times;
							for (times = 0; times < 50 ; times++) {
								// Early cancel 
								if (t_action != Autodetect ||
									t_pendingcmds > 0) {
									break;
								}
								mdelay(10);
							}
							// Early cancel 
							if (times < 50) {
								break;
							}
	
							// If Stereo or Dual detected, assume standard was found!
							BYTE val = 0; i2c.Read(i2caddr,&val);
							if ((val & (TDA9873_DUAL | TDA9873_STEREO)) != 0) {
								break;
							}
						}
					}

					// Cancelout if not autodetecting
					if (t_action != Autodetect ||
						t_pendingcmds > 0) {
						break;
					}

					// Set the autodetected standard (or the one that applies)
					shadow[1] = Info[std].Val;
					i2c.Write(i2caddr, TDA9873_AD, shadow[1]);
	
					// Wait 2 seconds before checking
					int times;
					for (times = 0; times < 200 ; times++) {
						// Early cancel 
						if (t_action != Autodetect ||
							t_pendingcmds > 0) {
							break;
						}
						mdelay(10);
					}
					// Early cancel 
					if (times < 200) {
						break;
					}

					// Now check the desired Audio mode. If unavailable, revert to one available.
					AudioMode curr = desiredam;
					AudioModes detected = AudioModesDetected();
					if (curr == Am_Stereo && (detected & Am_Stereo) == 0) curr = Am_Lang1;
					if (curr == Am_Lang1 && (detected & Am_Lang1) == 0)   curr = Am_Lang2;
					if (curr == Am_Lang2 && (detected & Am_Lang2) == 0)   curr = Am_Mono;

					// Set the supported mode
					SetChipMode(curr);
					break;
				}
				default:
					break;
			}
		};

		// No more available commands, undo the last decrement...
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// And wait for a command
		t_exec.WaitFor();
	};
}

// Called when videoChannel has changed
void TDA9873::ChannelChanged()
{
	// Trigger a new autodetection

	// Increment number of pending cmds
	InterlockedIncrement((LONG*)&t_pendingcmds);

	// Schedule command execution
	t_exec.Signal();
}

// Called when VideoNorm changed!
void TDA9873::VideoStandardChanged(KS_AnalogVideoStandard vf)
{
	// Remember the new norm 
	currVideoStandard = vf;

	// Trigger a new autodetection

	// Increment number of pending cmds
	InterlockedIncrement((LONG*)&t_pendingcmds);

	// Schedule command execution
	t_exec.Signal();
}

AudioModes TDA9873::AudioModesDetected()
{
	BYTE val = 0;
	i2c.Read(i2caddr,&val);

	return (Am_Mono |
			(((val & TDA9873_STEREO ) != 0) ? Am_Stereo             : 0) |
			(((val & TDA9873_DUAL   ) != 0) ? (Am_Lang1 | Am_Lang2) : 0)
			);

}

void TDA9873::SetAudioMode(AudioMode audio) 
{
	// Remember the desired audio mode
	desiredam = audio;

	// Trigger a new autodetection to change the mode

	// Increment number of pending cmds
	InterlockedIncrement((LONG*)&t_pendingcmds);

	// Schedule command execution
	t_exec.Signal();
}

void TDA9873::SetAudioConnector(AudioConnector conn)
{
	// inputmap:   {0xa0, 0xa2, 0xa0, 0xa0, 0xc0}

	BYTE tshadow = shadow[0] & ~0xe3;
	switch(conn) {
		case AudioConLineIn:
		case AudioConTuner:
			tshadow |= 0xa0;
			break;
		case AudioConRadio:
			tshadow |= 0xa2;
			break;
		case AudioConMute:
			tshadow |= 0xc0;
			break;
	}

	if(tshadow != shadow[0]) {
		shadow[0] = tshadow;
		i2c.Write(i2caddr, TDA9873_SW, shadow[0]);
	}
}

