/*
	TDA9874A.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

ChangeLog:

  01/21/2002  Sven Grothklags		Fixed exit condition on autodetect code (oops!)

*/

#include "TDA9874.hpp"
#include "..\HwCapture\Pisces.hpp"


// The TDA8974A is the next generation of the TDA9874H-
//  so, if we want to work with both of them, consider them to be
//  the TDA9874H and don't use the extended features of the TDA9875A


/* ---------------------------------------------------------------------- */
/* audio chip description - defines+functions for tda9874a                */
/* Dariusz Kowalewski <darekk@automex.pl>                                 */

/* Subaddresses for TDA9874 (slave rx) */
#define TDA9874_AGCGR		0x00	/* AGC gain *///
#define TDA9874_GCONR		0x01	/* general config *///
#define TDA9874_MSR			0x02	/* monitor select *///
#define TDA9874_C1FRA		0x03	/* carrier 1 freq. */
#define TDA9874_C1FRB		0x04	/* carrier 1 freq. */
#define TDA9874_C1FRC		0x05	/* carrier 1 freq. */
#define TDA9874_C2FRA		0x06	/* carrier 2 freq. */
#define TDA9874_C2FRB		0x07	/* carrier 2 freq. */
#define TDA9874_C2FRC		0x08	/* carrier 2 freq. */
#define TDA9874_DCR			0x09	/* demodulator config */
#define TDA9874_FMER		0x0a	/* FM de-emphasis */
#define TDA9874_FMMR		0x0b	/* FM dematrix *///
#define TDA9874_C1OLAR		0x0c	/* ch.1 output level adj. *///
#define TDA9874_C2OLAR		0x0d	/* ch.2 output level adj. *///
#define TDA9874_NCONR		0x0e	/* NICAM config *///
#define TDA9874_NOLAR		0x0f	/* NICAM output level adj. *///
#define TDA9874_NLELR		0x10	/* NICAM lower error limit */
#define TDA9874_NUELR		0x11	/* NICAM upper error limit */
#define TDA9874_AMCONR		0x12	/* audio mute control *///
#define TDA9874_SDACOSR		0x13	/* stereo DAC output select *///
#define TDA9874_AOSR		0x14	/* analog output select *///
#define TDA9874_DAICONR		0x15	/* digital audio interface config */
#define TDA9874_I2SOSR		0x16	/* I2S-bus output select */
#define TDA9874_I2SOLAR		0x17	/* I2S-bus output level adj. */

/* Subaddresses for TDA9874 (slave tx) */
#define TDA9874_DSR			0x00	/* device status */
#define TDA9874_NSR			0x01	/* NICAM status */
#define TDA9874_NECR		0x02	/* NICAM error count */
#define TDA9874_DR1			0x03	/* add. data LSB */
#define TDA9874_DR2			0x04	/* add. data MSB */
#define TDA9874_LLRA		0x05	/* monitor level read-out LSB */
#define TDA9874_LLRB		0x06	/* monitor level read-out MSB */
#define TDA9874_SIFLR		0x07	/* SIF level */
#define TDA9874_TR2			252	/* test reg. 2 */
#define TDA9874_TR1			253	/* test reg. 1 */
#define TDA9874_DIC			254	/* device id. code */
#define TDA9874_SIC			255	/* software id. code */

/*
0 A2, B/G
1 A2, M (Korea)
2 A2, D/K (1)
3 A2, D/K (2)
4 A2, D/K (3)
5 NICAM, I
6 NICAM, B/G
7 NICAM, D/K
8 NICAM, L
*/

/*
Analog Systems
M	mono	4.5			15 25 50 mono			-			15/75
M	A2+		4.5/4.724	15 25 50 1/2 (L + R)	1/2 (L - R)	15/75 (Korea)
B/G	A2		5.5/5.742	27 50 80 1/2 (L + R)	R			15/50
I	mono	6.0			27 50 80 mono			-			15/50
D/K	A2(2)	6.5/6.742	27 50 80 1/2 (L + R)	R			15/50
D/K	A2*(1)	6.5/6.258	27 50 80 1/2 (L + R)	R			15/50
D/K (3) A2	6.5/5.742	27 50 80 1 2(L + R) R 15/50

NICAM
B/G	5.5 FM	-	-	27	50	5.85	J17	40
I	6.0 FM	-	-	27	50	6.552	J17	100
D/K	6.5 FM	-	-	27	50	5.85	J17	40
L	6.5 AM	54	100 -	-	5.85	J17	40
Note:Should add +2Khz to C2 carrier for nicam
*/

#define CFREQ(x) DWORD(((x) * 16777216.0)/12.288)


TDA9874::ANInfo TDA9874::Info[] = {
	// First NICAM, because autodetection works better
	// NICAM, I
	{	TN_CCIR_I,		CFREQ(6.0),CFREQ(6.552),0x08,0x00,0x00,TRUE ,FALSE}, // Dematrix is Mono so NICAM reverts to Mono if BER is high
	// NICAM, BG
	{ 	TN_CCIR_BGHN,	CFREQ(5.5),CFREQ(5.85 ),0x08,0x00,0x00,TRUE ,FALSE},
	// NICAM, DK
	{	TN_CCIR_DKL,	CFREQ(6.5),CFREQ(5.85 ),0x08,0x00,0x00,TRUE ,FALSE},
	// Now, all the A2 systems
	// A2, M (Korea)
	{ 	TN_CCIR_MC,		CFREQ(4.5),CFREQ(4.724),0x20,0x22,0x05,FALSE,FALSE},
	// A2, BG 
	{	TN_CCIR_BGHN,	CFREQ(5.5),CFREQ(5.742),0x00,0x00,0x04,FALSE,FALSE},
	// A2, DK (1)
	{	TN_CCIR_DKL,	CFREQ(6.5),CFREQ(6.258),0x00,0x00,0x04,FALSE,FALSE},
	// A2, DK (2)
	{	TN_CCIR_DKL,	CFREQ(6.5),CFREQ(6.742),0x00,0x00,0x04,FALSE,FALSE},
	// A2, DK (3)
	{	TN_CCIR_DKL,	CFREQ(6.5),CFREQ(5.742),0x00,0x00,0x04,FALSE,FALSE},
	// And, the last is NICAM AM
	// NICAM, L
	{	TN_CCIR_DKL,	CFREQ(6.5),CFREQ(5.85 ),0x09,0x00,0x00,TRUE ,TRUE },
};
#define NUM_AUDIONORMS (sizeof(TDA9874::Info) / sizeof(TDA9874::Info[0]))


void TDA9874::Restart()
{
	if ( t_action == SuspendAutodetect ) { // Only if autodetect thread is suspending/suspended ...
		i2c.Write(i2caddr, TDA9874_AGCGR, 0x00);		// 0 dB 

		// Muting
		i2c.Write(i2caddr, TDA9874_AMCONR, 0xFF); /* mute enable ALL outs */

		// Channel Gain settings
		i2c.Write(i2caddr, TDA9874_C1OLAR, 0x00); /* 0 dB */
		i2c.Write(i2caddr, TDA9874_C2OLAR, 0x00); /* 0 dB */

		// NICAM settings
		i2c.Write(i2caddr, TDA9874_NCONR, 0x00); // Enable NICAM automute on BER (switchs to mono), and NICAM deemphasis. Should be selected by user based on needs
		i2c.Write(i2caddr, TDA9874_NOLAR, 0x00); /* 0 dB */
		i2c.Write(i2caddr, TDA9874_NLELR, 0x14); /* NICAM lower error limit */
		i2c.Write(i2caddr, TDA9874_NUELR, 0x50); /* NICAM upper error limit */

		// Start with something!
		SetChipStandard(false);
		SetChipMode(Am_Mono);

		// Reinit autodetection
		t_action = Autodetect;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();

	}
}

// Wait until autodetection is complete!
void TDA9874::Suspend()
{
	if ( t_action == Autodetect ) { // Only if thread is autodetecting...
		// Clear the suspended notification...
		t_suspended.Reset();

		// Post our suspend command.
		t_action = SuspendAutodetect;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();

		// And wait until the suspension command is completed.
		t_suspended.WaitFor();
	}
}


TDA9874::TDA9874(I2C& bus,const BYTE Addr) :
	i2c(bus) , i2caddr(Addr) ,
	m_SIF(0),		/* SIF = 0 (sound IF input 1 (Terrestrial TV)*/
	std(0),			/* M standard */ 
	currVideoStandard(KS_AnalogVideo_None),
	t_action(SuspendAutodetect),
	t_pendingcmds(0L),		// No pending cmds...
	desiredam(Am_Stereo)
{
	// Create the autodetection Thread
	bool ret = CreateThread(ThreadProc,this);
	if (!ret) { // if failure, simulate thread termination
		t_action = Exit;
		t_ended.Set();
	} 

	Restart();
}

TDA9874::~TDA9874()
{
	// Make sure to end Thread now!
	if (t_action != Exit) {
		// Reset ended event
		t_ended.Reset();

		// Post our exit command
		t_action = Exit;

		// Increment number of pending cmds
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// Schedule command execution
		t_exec.Signal();

		// And wait until the exit command is completed
		t_ended.WaitFor();
	}
}

// Out thread proc
VOID TDA9874::ThreadProc(IN PVOID ctx)
{
	TDA9874* ptr = (TDA9874*) ctx;
	ptr->AutodetectProc();

	// Signal we are ended (we do it from an static fn so no hidden this references are used) ...
	ptr->t_ended.Set();

	// And end the Thread!
	ExitThread();
}

// Function to change audio mode
void TDA9874::SetChipMode(AudioMode audio)
{
	// FM settings
	i2c.Write(i2caddr, TDA9874_FMMR, 
		(  audio == Am_Mono ) ? 0x00 : 
		(( audio == Am_Lang1 || audio == Am_Lang2) ? 0x02 : 
		Info[std].FMMR) ); 

	// Select the proper outs.
	i2c.Write(i2caddr, TDA9874_AOSR, 
		( audio == Am_Mono ) ? 0x40 :
		(( audio == Am_Stereo ) ? 0x00 :
		(( audio == Am_Lang1 ) ? 0x10 :
		(( audio == Am_Lang2 ) ? 0x20 : 0x00 ))) ); 
}

// Function to set Chip Standard
void TDA9874::SetChipStandard(bool FastCheck)
{
	BYTE Buffer[4];
	
	i2c.Write(i2caddr, TDA9874_GCONR, 
		0xc0 | m_SIF | ((Info[std].AGCOff) ? 0x02 : 0x00) ); // Select the proper SIF and turn AGC on/off if needed to
	i2c.Write(i2caddr, TDA9874_MSR, 
		(Info[std].Nicam) ? 0x03 : 0x02 
		); // We monitor channel (1+2)/2 (not used by now)

	// Demodulator settings

	// Now program the needed Carrier frequencies...
	Buffer[0] = TDA9874_C1FRA;
	Buffer[1] = BYTE((Info[std].c1 >> 16) & 0xFF);
	Buffer[2] = BYTE((Info[std].c1 >> 8 ) & 0xFF);
	Buffer[3] = BYTE((Info[std].c1      ) & 0xFF);
	i2c.WR(i2caddr,Buffer,4,NULL,0);
	Buffer[0] = TDA9874_C2FRA;
	Buffer[1] = BYTE((Info[std].c2 >> 16) & 0xFF);
	Buffer[2] = BYTE((Info[std].c2 >> 8 ) & 0xFF);
	Buffer[3] = BYTE((Info[std].c2      ) & 0xFF);
	i2c.WR(i2caddr,Buffer,4,NULL,0);
	i2c.Write(i2caddr, TDA9874_DCR ,
		Info[std].DCR | ((FastCheck) ? 0x80 : 0x00) ); // Fast Detect if asked to

	// FM Settings
	i2c.Write(i2caddr, TDA9874_FMER, Info[std].FMER);

	// Set the signal source... 
	i2c.Write(i2caddr, TDA9874_SDACOSR, 
		(Info[std].Nicam) ? 0x01:0x00); /* Signal source */

}

void TDA9874::AutodetectProc()
{
	int dsr,nsr;
	BYTE readbf[1];
	BYTE writebf[1];
	while(1) {

		// Do we have some pending cmds ?
		while (InterlockedDecrement((LONG*)&t_pendingcmds) >= 0) {

			// We got it, check requested action.
			switch (t_action) {
				case Exit:
					return;
				case SuspendAutodetect:
					t_suspended.Set();
					break;
				default:
				case Autodetect:
				{

					// We are requested to start an autodetection, do it!
				
					// First try the current std if it applies...
					if (Info[std].Norms & currVideoStandard) {

						// set this standard to see if we can autodetect this
						SetChipStandard(true);

						// Wait 1 second before checking
						int times;
						for (times = 0; times < 100 ; times++) {
							// Early cancel 
							if (t_action != Autodetect ||
								t_pendingcmds > 0) {
								break;
							}
							mdelay(10);
						}
						// Early cancel 
						if (times < 100) {
							break;
						}

						// If Stereo or Dual detected, assume standard was found!
						writebf[0] = TDA9874_DSR;
						i2c.WR(i2caddr,writebf,1,readbf,1);
						dsr = readbf[0];

						// Read NICAM status Register
						writebf[0] = TDA9874_NSR;
						i2c.WR(i2caddr,writebf,1,readbf,1);
						nsr = readbf[0];

						if(
							( ( Info[std].Nicam) && ( (nsr & 0x03) || (dsr & 0x8) ) ) ||	// Stereo or dual NICAM identified or NICAM identified
							( (!Info[std].Nicam) && ( dsr & 0x6 ) )							// FM stereo or dual identified
							) {

							// set the autodetected standard
							SetChipStandard(false);

							// Wait 4 seconds before checking
							int times;
							for (times = 0; times < 400 ; times++) {
								mdelay(10);
								// Early cancel 
								if (t_action != Autodetect ||
									t_pendingcmds > 0) {
									break;
								}
							}
							// Early cancel 
							if (times < 400) {
								break;
							}

							// If NICAM, check if we can use AUTOMUTE!
							if (Info[std].Nicam) {

								// Check if we are allowed to use automute!
								writebf[0] = TDA9874_DSR;
								i2c.WR(i2caddr,writebf,1,readbf,1);
								dsr = readbf[0];

								// And set it!
								i2c.Write(i2caddr, TDA9874_NCONR, ((dsr & 32) == 0) ? 0x01 : 0x00);
							}

							// Now check the desired Audio mode. If unavailable, revert to one available.
							AudioMode curr = desiredam;
							AudioModes detected = AudioModesDetected();
							if (curr == Am_Stereo && (detected & Am_Stereo) == 0) curr = Am_Lang1;
							if (curr == Am_Lang1 && (detected & Am_Lang1) == 0)   curr = Am_Lang2;
							if (curr == Am_Lang2 && (detected & Am_Lang2) == 0)   curr = Am_Mono;
	
							// Set the supported mode
							SetChipMode(curr);

							break;
						}
					}

					// Didn't work... Try autodetection
					// We are requested to start an autodetection, do it!
					BYTE stdtry;
					for (stdtry = 0; 
						stdtry < NUM_AUDIONORMS && t_action == Autodetect; 
						stdtry ++) {

						// If audio norm applies to the video norm...
						if (Info[stdtry].Norms & currVideoStandard) {

							// Assume this std will be as it applies to the current video norm
							std = stdtry;

							// set this standard to see if we can autodetect this
							SetChipStandard(true);

							// Wait 1 second before checking
							int times;
							for (times = 0; times < 100 ; times++) {
								mdelay(10);
								// Early cancel 
								if (t_action != Autodetect ||
									t_pendingcmds > 0) {
									break;
								}
							}
							// Early cancel 
							if (times < 100) {
								break;
							}

							// If Stereo or Dual detected, assume standard was found!
							writebf[0] = TDA9874_DSR;
							i2c.WR(i2caddr,writebf,1,readbf,1);
							dsr = readbf[0];

							// Read NICAM status Register
							writebf[0] = TDA9874_NSR;
							i2c.WR(i2caddr,writebf,1,readbf,1);
							nsr = readbf[0];
							if(Info[std].Nicam) {
								if(nsr & 0x03)	// Stereo or dual NICAM identified
									break;
								if(dsr & 0x8)	// NICAM identified
									break;
							} else {
								if(dsr & 0x06)	// FM stereo or dual identified
									break;
							}
						}
					}

					// set the autodetected standard
					SetChipStandard(false);


					// Wait 4 seconds before checking in 10 ms chunks
					int times;
					for (times = 0; times < 400; times++) {
						// Early cancel 
						if (t_action != Autodetect ||
							t_pendingcmds > 0) {
							break;
						}
						mdelay(10);
					}
					// Cancel out if not autodetecting
					if (times < 400) {
						break;
					}

					// If NICAM, check if we can use AUTOMUTE!
					if (Info[std].Nicam) {

						// Check if we are allowed to use automute!
						writebf[0] = TDA9874_DSR;
						i2c.WR(i2caddr,writebf,1,readbf,1);
						dsr = readbf[0];

						// And set it!
						i2c.Write(i2caddr, TDA9874_NCONR, ((dsr & 32) == 0) ? 0x01 : 0x00);
					}

					// Now check the desired Audio mode. If unavailable, revert to one available.
					AudioMode curr = desiredam;
					AudioModes detected = AudioModesDetected();
					if (curr == Am_Stereo && (detected & Am_Stereo) == 0) curr = Am_Lang1;
					if (curr == Am_Lang1 && (detected & Am_Lang1) == 0)   curr = Am_Lang2;
					if (curr == Am_Lang2 && (detected & Am_Lang2) == 0)   curr = Am_Mono;
	
					// Set the supported mode
					SetChipMode(curr);
				}
				break;
			}
		};

		// No more available commands, undo the last decrement...
		InterlockedIncrement((LONG*)&t_pendingcmds);

		// And wait for a command
		t_exec.WaitFor();
	};
}

AudioModes TDA9874::AudioModesDetected()
{
	BYTE readbf[1];
	BYTE writebf[1];

	if (Info[std].Nicam)  {
		// NICAM : Read NICAM status Register
		writebf[0] = TDA9874_NSR;
		i2c.WR(i2caddr,writebf,1,readbf,1);

		return (Am_Mono |
				(((readbf[0] & 0x02) != 0) ? Am_Stereo : 0) |
				(((readbf[0] & 0x01) != 0) ? (Am_Lang1 | Am_Lang2)  : 0)  /* NSR.D/SB */ 
				);

	} else {
		// Dual... : Read Status Register
		writebf[0] = TDA9874_DSR;
		i2c.WR(i2caddr,writebf,1,readbf,1);

		return (Am_Mono |
				(((readbf[0] & 0x02) != 0) ? Am_Stereo : 0) |
				(((readbf[0] & 0x04) != 0) ? (Am_Lang1 | Am_Lang2) : 0)  /* DSR.IDDUA */
				);

	}
	return 0;
}

void TDA9874::SetAudioConnector(AudioConnector conn) 
{
	switch(conn) {
		case AudioConTuner: // Changed channel 
		case AudioConRadio:
		case AudioConLineIn:
			i2c.Write(i2caddr, TDA9874_AMCONR, 0x00); /* unmute ALL outs */
			break;
		case AudioConMute:
			i2c.Write(i2caddr, TDA9874_AMCONR, 0xFF); /* mute ALL outs */
			break;
	}
}

void TDA9874::SetAudioMode(AudioMode audio) 
{
	// Remember the desired audio mode
	desiredam = audio;

	// And trigger a new autodetection to set it

	// Increment number of pending cmds
	InterlockedIncrement((LONG*)&t_pendingcmds);

	// Schedule command execution
	t_exec.Signal();
}

// Called when videoChannel has changed
void TDA9874::ChannelChanged()
{
	// Trigger a new autodetection

	// Increment number of pending cmds
	InterlockedIncrement((LONG*)&t_pendingcmds);

	// Schedule command execution
	t_exec.Signal();

}

// Called when VideoNorm changed!
void TDA9874::VideoStandardChanged(KS_AnalogVideoStandard vf)
{
	// Remember the new norm 
	currVideoStandard = vf;

	// Trigger a new autodetection

	// Increment number of pending cmds
	InterlockedIncrement((LONG*)&t_pendingcmds);

	// Schedule command execution
	t_exec.Signal();

}


