/*
	TDA9875.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TDA9875.hpp"
#include "..\HwCapture\Pisces.hpp"

/*
 * For the TDA9875 chip
 * (The TDA9875 is used on the Diamond DTV2000 french version 
 * Other cards probably use these chips as well.)
 */

		/* subaddresses for TDA9875 */
#define TDA9875_MUT         0x12  /*General mute  (value --> 0b11001100*/
#define TDA9875_CFG         0x01  /* Config register (value --> 0b00000000 */
#define TDA9875_DACOS       0x13  /*DAC i/o select (ADC) 0b0000100*/
#define TDA9875_LOSR        0x16  /*Line output select regirter 0b0100 0001*/

#define TDA9875_CH1V        0x0c  /*Chanel 1 volume (mute)*/
#define TDA9875_CH2V        0x0d  /*Chanel 2 volume (mute)*/
#define TDA9875_SC1         0x14  /*SCART 1 in (mono)*/
#define TDA9875_SC2         0x15  /*SCART 2 in (mono)*/

#define TDA9875_ADCIS       0x17  /*ADC input select (mono) 0b0110 000*/
#define TDA9875_AER         0x19  /*Audio effect (AVL+Pseudo) 0b0000 0110*/
#define TDA9875_MCS         0x18  /*Main channel select (DAC) 0b0000100*/
#define TDA9875_MVL         0x1a  /* Main volume gauche */
#define TDA9875_MVR         0x1b  /* Main volume droite */
#define TDA9875_MBA         0x1d  /* Main Basse */
#define TDA9875_MTR         0x1e  /* Main treble */
#define TDA9875_ACS         0x1f  /* Auxilary channel select (FM) 0b0000000*/
#define TDA9875_AVL         0x20  /* Auxilary volume gauche */
#define TDA9875_AVR         0x21  /* Auxilary volume droite */
#define TDA9875_ABA         0x22  /* Auxilary Basse */
#define TDA9875_ATR         0x23  /* Auxilary treble */

#define TDA9875_MSR         0x02  /* Monitor select register */
#define TDA9875_C1MSB       0x03  /* Carrier 1 (FM) frequency register MSB */
#define TDA9875_C1MIB       0x04  /* Carrier 1 (FM) frequency register (16-8]b */
#define TDA9875_C1LSB       0x05  /* Carrier 1 (FM) frequency register LSB */
#define TDA9875_C2MSB       0x06  /* Carrier 2 (nicam) frequency register MSB */
#define TDA9875_C2MIB       0x07  /* Carrier 2 (nicam) frequency register (16-8]b */
#define TDA9875_C2LSB       0x08  /* Carrier 2 (nicam) frequency register LSB */
#define TDA9875_DCR         0x09  /* Demodulateur configuration regirter*/
#define TDA9875_DEEM        0x0a  /* FM de-emphasis regirter*/
#define TDA9875_FMAT        0x0b  /* FM Matrix regirter*/

/* values */
#define TDA9875_MUTE_ON	    0xff /* general mute */
#define TDA9875_MUTE_OFF    0xcc /* general no mute */

void TDA9875::Restart()
{
	i2c.Write(i2caddr, TDA9875_CFG, 0xd0 ); /*reg de config 0 (reset)*/
	i2c.Write(i2caddr, TDA9875_MSR, 0x03 );    /* Monitor 0b00000XXX*/
	i2c.Write(i2caddr, TDA9875_C1MSB, 0x00 );  /*Car1(FM) MSB XMHz*/
	i2c.Write(i2caddr, TDA9875_C1MIB, 0x00 );  /*Car1(FM) MIB XMHz*/
	i2c.Write(i2caddr, TDA9875_C1LSB, 0x00 );  /*Car1(FM) LSB XMHz*/
	i2c.Write(i2caddr, TDA9875_C2MSB, 0x00 );  /*Car2(NICAM) MSB XMHz*/
	i2c.Write(i2caddr, TDA9875_C2MIB, 0x00 );  /*Car2(NICAM) MIB XMHz*/
	i2c.Write(i2caddr, TDA9875_C2LSB, 0x00 );  /*Car2(NICAM) LSB XMHz*/
	i2c.Write(i2caddr, TDA9875_DCR, 0x00 );    /*Demod config 0x00*/
	i2c.Write(i2caddr, TDA9875_DEEM, 0x44 );   /*DE-Emph 0b0100 0100*/
	i2c.Write(i2caddr, TDA9875_FMAT, 0x00 );   /*FM Matrix reg 0x00*/
	i2c.Write(i2caddr, TDA9875_SC1, 0x00 );    /* SCART 1 (SC1)*/
	i2c.Write(i2caddr, TDA9875_SC2, 0x01 );    /* SCART 2 (sc2)*/
        
	i2c.Write(i2caddr, TDA9875_CH1V, 0x10 );  /* Chanel volume 1 mute*/
	i2c.Write(i2caddr, TDA9875_CH2V, 0x10 );  /* Chanel volume 2 mute */
	i2c.Write(i2caddr, TDA9875_DACOS, 0x02 ); /* sig DAC i/o(in:nicam)*/
	i2c.Write(i2caddr, TDA9875_ADCIS, 0x6f ); /* sig ADC input(in:mono)*/
	i2c.Write(i2caddr, TDA9875_LOSR, 0x00 );  /* line out (in:mono)*/
 	i2c.Write(i2caddr, TDA9875_AER, 0x00 );   /*06 Effect (AVL+PSEUDO) */
	i2c.Write(i2caddr, TDA9875_MCS, 0x44 );   /* Main ch select (DAC) */
	i2c.Write(i2caddr, TDA9875_MVL, 0x03 );   /* Vol Main left 10dB */
	i2c.Write(i2caddr, TDA9875_MVR, 0x03 );   /* Vol Main right 10dB*/
	i2c.Write(i2caddr, TDA9875_MBA, 0x00 );   /* Main Bass Main 0dB*/
	i2c.Write(i2caddr, TDA9875_MTR, 0x00 );   /* Main Treble Main 0dB*/
	i2c.Write(i2caddr, TDA9875_ACS, 0x44 );   /* Aux chan select (dac)*/	
	i2c.Write(i2caddr, TDA9875_AVL, 0x00 );   /* Vol Aux left 0dB*/
	i2c.Write(i2caddr, TDA9875_AVR, 0x00 );   /* Vol Aux right 0dB*/
	i2c.Write(i2caddr, TDA9875_ABA, 0x00 );   /* Aux Bass Main 0dB*/
	i2c.Write(i2caddr, TDA9875_ATR, 0x00 );   /* Aux Aigus Main 0dB*/
	
	i2c.Write(i2caddr, TDA9875_MUT, 0xcc );   /* General mute  */
}

TDA9875::TDA9875(I2C& bus,const BYTE Addr) :
	i2c(bus) , i2caddr(Addr)
{
	TDA9875::Restart();
}

void TDA9875::SetVolume(const int Left,const int Right) /* 0 = Gain 1 */
{
	int AdjLeft  = Left  / 390;
	AdjLeft  = (AdjLeft  > 24) ? 24 : AdjLeft;
	int AdjRight = Right / 390;
	AdjRight = (AdjRight > 24) ? 24 : AdjRight;
	i2c.Write(i2caddr, TDA9875_MVL, AdjLeft  & 0xFF );
	i2c.Write(i2caddr, TDA9875_MVR, AdjRight & 0xFF );
}

void TDA9875::SetBassTreble(const int Bass,const int Treble) 
{
	int AdjBass   = Bass   / 2048;
	AdjBass  = (AdjBass  < -12) ? -12 : AdjBass;
	int AdjTreble = Treble / 2731;
	i2c.Write(i2caddr, TDA9875_MBA, AdjBass & 0x1F);
	i2c.Write(i2caddr, TDA9875_MTR, AdjTreble & 0x1F);
}

AudioModes TDA9875::AudioModesDetected()
{
	BYTE readbf[1];
	BYTE writebf[1];

	// Read Status Register
	readbf[0] = 0;
	writebf[0] = 0;
	i2c.WR(i2caddr,writebf,1,readbf,1);

	return (Am_Mono |
			(((readbf[0] & 0x2) != 0) ? Am_Stereo : 0) |
			(((readbf[0] & 0xC) != 0) ? Am_Lang1  : 0)  // NICAM or DUAL
			);
}

