/*
	TEA6320.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Log:
  04/19/2002	Kevin Moll <kevinmoll@hotmail.com> : Fixed input selection values
*/

#include "TEA6320.hpp"
#include "..\HwCapture\Pisces.hpp"

/*
 * for the TEA6320 chip 
 */

#define TEA6320_V			0x00	/* volume : 43 =0db*/
#define TEA6320_FFR			0x01	/* Front fader right */
#define TEA6320_FFL			0x02	/* Front fader left */
#define TEA6320_FRR			0x03	/* Rear fader right */
#define TEA6320_FRL			0x04	/* Rear fader left */
#define TEA6320_BA			0x05	/* Bass */
#define TEA6320_TR			0x06	/* Treble */
#define TEA6320_S			0x07	/* Switch */

/* values for those registers: */
#define TEA6320_S_SA       0x07  /* stereo A input */
#define TEA6320_S_SB       0x06  /* stereo B */
#define TEA6320_S_SC       0x05  /* stereo C */
#define TEA6320_S_SD       0x04  /* stereo D */
#define TEA6320_S_GMU      0x80  /* general mute */

void TEA6320::Restart()
{
	i2c.Write(i2caddr, TEA6320_S , TEA6320_S_GMU); /* mute */
}

TEA6320::TEA6320(I2C& bus,const BYTE Addr) :
	i2c(bus) , i2caddr(Addr)
{
	TEA6320::Restart();
}

void TEA6320::SetAudioConnector(AudioConnector conn)
{
	switch(conn) {
		case AudioConTuner:
			i2c.Write(i2caddr, TEA6320_S, TEA6320_S_SA);
			break;
		case AudioConRadio:
			i2c.Write(i2caddr, TEA6320_S, TEA6320_S_SB);
			break;
		case AudioConLineIn:
			i2c.Write(i2caddr, TEA6320_S, TEA6320_S_SC);
			break;
		case AudioConMute:
			i2c.Write(i2caddr, TEA6320_S, TEA6320_S_GMU);
			break;
	}
}
		
void TEA6320::SetVolume(const int Left,const int Right) /* 0 = Gain 1 */
{
	// Volume 
	// 12 = -31Db, 43 = 0db, 63 = 20Db
	int Max = max(Left,Right);		// Estimate maximum volume (G)
	int LeftDif = Left - Max;		// Estimate attenuation for left
	int RightDif = Right - Max;		// Estimate attenuation for right

	// Now map volume.
	int Vol = (Max/1639) + 43;
	if (Vol < 12) Vol = 12;
	else if (Vol > 63) Vol = 63;

	// Now map differences 63 = 0Db, 0 = 63Db
	int LeftAtt = (LeftDif/1639) + 63;
	int RightAtt = (RightDif/1639) + 63;
	if (LeftAtt > 63) LeftAtt = 63;
	else if (LeftAtt < 0) LeftAtt = 0;
	if (RightAtt > 63) RightAtt = 63;
	else if (RightAtt < 0) RightAtt = 0;

	// Set volume
	i2c.Write(i2caddr, TEA6320_V, BYTE(0x80 | Vol)); // No loudness support by now!
	i2c.Write(i2caddr, TEA6320_FFR, BYTE(RightAtt));
	i2c.Write(i2caddr, TEA6320_FFL, BYTE(LeftAtt ));
	i2c.Write(i2caddr, TEA6320_FRR, BYTE(RightAtt));
	i2c.Write(i2caddr, TEA6320_FRL, BYTE(LeftAtt ));

}

void TEA6320::SetBassTreble(const int Bass,const int Treble) 
{
	// Bass: 27 = 15 Db, 17 = 0Db, 8 =-15Db
	// Treble: 25 = 12Db, 17 = 0Db, 8=-12

	int BassMapped = (Bass / 3277) + 17;
	if (BassMapped > 27) BassMapped = 27;
	else if (BassMapped < 8) BassMapped = 8;

	int TrebleMapped = (Treble / 4096) + 17;
	if (TrebleMapped > 25) TrebleMapped = 25;
	else if (TrebleMapped < 8) TrebleMapped = 8;

	i2c.Write(i2caddr, TEA6320_BA, BYTE(BassMapped)) ;
	i2c.Write(i2caddr, TEA6320_TR, BYTE(TrebleMapped));
}

// Note : No Reading available!!!!
