/*
	ColSpace.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#define INITGUID
extern "C" {
#include "guiddef.h"
#include "uuids.h"

}

// We need the I420 definition!

// 30323449-0000-0010-8000-00AA00389B71  'I420' == MEDIASUBTYPE_I420
DEFINE_GUID(MEDIASUBTYPE_I420,
0x30323449, 0x0000, 0x0010, 0x80, 0x00, 0x00, 0xaa, 0x00, 0x38, 0x9b, 0x71);


#include "debug.hpp"
#include "mytypes.hpp"
#include "colspace.hpp"

BYTE const ColorSpace::BitCount_ [] =
{
// RGB32 RGB24 RGB16 RGB15 YUY2 BTYUV Y8 RGB8 PL_422 PL_411 YUV9 YUV12 VBI UYVY RAW I420
   32,   24,   16,   16,   16,  12,   8, 8,   16,    12,    9,   12,    8, 16,  8,  12
};

BYTE const ColorSpace::YPlaneBitCount_ [] = // bits per pixel for the first color plane
{
// RGB32 RGB24 RGB16 RGB15 YUY2 BTYUV Y8 RGB8 PL_422 PL_411 YUV9 YUV12 VBI UYVY RAW I420
   32,   24,   16,   16,   16,  12,   8, 8,   8,     8,     8,   8,     8, 16,  8,  8
};

// Note that all formats align lines to DWORDs... So we make sure here we are not cutting a group of pixels...
// -If we are dealing with a non-planar format, then the stride of each
//	scanline is simply the next multiple of four of the required amount of
//	bytes required to store it. Windows always aligns scanlines to DWORDs, so this
//	maps nicely to the Bt8x8 hardware
// -If we are dealing with a planar format, then we also need that the
//  Y plane scanlines to be dword aligned... and the chroma planes scanlines
//  will have half that stride. As the Bt8x8 is unable to handle scanlines
//  not aligned to DWORD boundaries, this will force the Y plane stride also.

BYTE const ColorSpace::XRestriction_ [] =
{
// RGB32 RGB24 RGB16 RGB15 YUY2 BTYUV Y8 RGB8 PL_422 PL_411 YUV9 YUV12 VBI UYVY RAW I420
   1,    1,    1,    1,    2,   8,    1, 1,   8,     16,    16,  8,    1,  2,   1,  2
};

BYTE const ColorSpace::YRestriction_ [] =
{
// RGB32 RGB24 RGB16 RGB15 YUY2 BTYUV Y8 RGB8 PL_422 PL_411 YUV9 YUV12 VBI UYVY RAW I420
   1,    1,    1,    1,   1,    1,    1, 1,   1,     1,     4,    2,   1,  1,   1,  2
};

FOURCC const ColorSpace::FourccArr_ [] =
{
// 32      24      16      15
   BI_RGB, BI_RGB, BI_RGB, BI_RGB, FCC_YUY2, FCC_Y41P, FCC_Y8, BI_RGB,
   FCC_422, FCC_411, FCC_YVU9, FCC_YV12, FCC_VBI, FCC_UYVY, FCC_RAW, FCC_I420
};

const GUID *ColorSpace::VideoGUIDs [] =
{
   &MEDIASUBTYPE_RGB32, &MEDIASUBTYPE_RGB24, &MEDIASUBTYPE_RGB565,   &MEDIASUBTYPE_RGB555,
   &MEDIASUBTYPE_YUY2,  &MEDIASUBTYPE_Y41P,  &MEDIASUBTYPE_RGB8,     &MEDIASUBTYPE_RGB8,
   NULL,                NULL,                &MEDIASUBTYPE_YVU9,     &MEDIASUBTYPE_YV12,
   NULL,                &MEDIASUBTYPE_UYVY,  NULL,                   &MEDIASUBTYPE_I420
};


/* Function: IsDivisible
 * Purpose: This function checks that the first value passed in is
 *   evenly divisible by the second
 * Input: ToCheck: int - value to be checked
 *        Divisor: int
 * Output: bool
 * Note: The function assumes Divisor is a power of 2
 */
inline bool IsDivisible(const int ToCheck,const int Divisor )
{
   return bool( !( ToCheck & ( Divisor - 1 ) ) );
}

/* Constructor: ColorSpace::ColorSpace
 * Input: fcc: FOURCC
 */
ColorSpace::ColorSpace(const FOURCC fcc,const int bitCount , const DWORD mask0 , const DWORD mask1 , const DWORD mask2 ) : CurColor_( CF_BelowRange )
{
	DUMP(("ColorSpace(%x, %d, ('%c%c%c%c'))\n", 
		fcc, 
		bitCount,
		fcc & 0xff,
		(fcc >> 8) & 0xff,
		(fcc >> 16) & 0xff,
		(fcc >> 24) & 0xff
	));

	switch( fcc ) {
		case KS_BI_RGB:
			switch ( bitCount ) {
				default: 
				case 8:  CurColor_ = CF_RGB8;    break;
				case 16: CurColor_ = CF_RGB15;   break; // This is Per ActiveMovie Implementation!
				case 24: CurColor_ = CF_RGB24;   break;
				case 32: CurColor_ = CF_RGB32;   break;
			}
			break;

		//case KS_BI_RLE8:
		//case KS_BI_RLE4:

		case KS_BI_BITFIELDS:
			switch ( bitCount ) {
				default: 
				case 16: {
						if (mask0 == 0xF800 && mask1 == 0x07E0 && mask2 == 0x001F) {
							CurColor_ = CF_RGB16;   
						} else
						if (mask0 == 0x7C00 && mask1 == 0x03E0 && mask2 == 0x001F) {
							CurColor_ = CF_RGB15;   
						}
					}
					break;
				case 32: CurColor_ = CF_RGB32;   break;
			}
			break;
 		case 0xe436eb7b:// ??? RGB16 ???
		case FCC_YUY2:
		case FCC_Y41P:
		case FCC_Y8:
		case FCC_422:
		case FCC_411:
		case FCC_YVU9:
		case FCC_YV12:
		case FCC_VBI:
		case FCC_UYVY:
		case FCC_RAW:
		case FCC_I420:
		default:
			for ( int fccArrIdx = CF_RGB32; fccArrIdx < CF_AboveRange; fccArrIdx++ )
				if ( fcc == FourccArr_ [fccArrIdx] ) {
					CurColor_ = (ColFmt)fccArrIdx;
					break;
				}
			break;
	}
}

/* Constructor: ColorSpace::ColorSpace
 * Input: guid: const GUID &
 */
ColorSpace::ColorSpace( const GUID &guid ) : CurColor_( CF_BelowRange )
{
   for ( int idx = CF_RGB32; idx < CF_AboveRange; idx++ ) {
      if ( VideoGUIDs [idx] ) {
         if ( guid == *VideoGUIDs [idx] ) {
            CurColor_ = (ColFmt)idx;
            break;
         }
      }
   }
}

/* Method: ColorSpace::CheckDimentions
 * Purpose: This functions checks that the size of a buffer corresponds to the
 *   restrictions imposed by a color format
 * Input: size: reference to SIZE structure
 * Output: bool: True or False
 */
bool ColorSpace::CheckDimentions( const SIZE &size ) const
{
	bool res = bool( CurColor_ > CF_BelowRange && CurColor_ < CF_AboveRange &&
                 IsDivisible( size.cx, XRestriction_ [CurColor_] ) &&
                 IsDivisible( size.cy, YRestriction_ [CurColor_] )  );
#if DBG
	if (!res) {
		if (CurColor_ > CF_BelowRange && CurColor_ < CF_AboveRange) {
			ERROR((	"ColorSpace::CheckDimentions failed:\n"
					"Colorspace(%x, %d, ('%c%c%c%c'))\n"
					"SIZE to check: (cx=%d,cy=%d)\n"
					"Restriction  : (x =%d,y =%d)\n"
					,FourccArr_[CurColor_]
					,BitCount_[CurColor_]
					,(FourccArr_[CurColor_] & 0xff)
					,(FourccArr_[CurColor_] >> 8) & 0xff
					,(FourccArr_[CurColor_] >> 16) & 0xff
					,(FourccArr_[CurColor_] >> 24) & 0xff
					,size.cx, size.cy
					,XRestriction_ [CurColor_], YRestriction_ [CurColor_] ));
		} else {
			ERROR((	"ColorSpace::CheckDimentions failed: Invalid Colorspace\n"));
		}
	}
#endif
	return res;
}

#if DBG
bool ColorSpace::CheckLeftTop( const MPoint &lt ) const 
{ 
	bool res = bool( !( lt.x & 3 ) ); /* Force alignment of the scanlines to DWORDs */ 
	if (!res) {
		if (CurColor_ > CF_BelowRange && CurColor_ < CF_AboveRange) {
			ERROR((	"ColorSpace::CheckLeftTop failed (alignment to DWORD):\n"
					"Colorspace(%x, %d, ('%c%c%c%c'))\n"
					"point to check: (x=%d,y=%d)\n"
					,FourccArr_[CurColor_]
					,BitCount_[CurColor_]
					,(FourccArr_[CurColor_] & 0xff)
					,(FourccArr_[CurColor_] >> 8) & 0xff
					,(FourccArr_[CurColor_] >> 16) & 0xff
					,(FourccArr_[CurColor_] >> 24) & 0xff
					,lt.x, lt.y ));
		} else {
			ERROR((	"ColorSpace::CheckLeftTop failed: Invalid Colorspace\n"));
		}
	}
	return res;
}
#endif