/*
	ColSpace.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __COLSPACE_H
#define __COLSPACE_H

#include "mytypes.hpp"

typedef DWORD           FOURCC;          /*a four character code */
   /* constants for the biCompression field from windows.h*/
   #define BI_RGB        0L
   #define BI_RLE8       1L
   #define BI_RLE4       2L
   #define BI_BITFIELDS  3L


/* Type: ColorFormat
 * Purpose: Enumerates all possible color formats that BtPisces can produce
 */

#ifdef DOCUMENTATION
   Color Format Register Settings

   0000 = RGB32
   0001 = RGB24
   0010 = RGB16
   0011 = RGB15
   0100 = YUY2 4:2:2   (YUY2)
   0101 = BtYUV 4:1:1  (Y41P or Y411)
   0110 = Y8
   0111 = RGB8 (Dithered)
   1000 = YCrCb 4:2:2 Planar 
   1001 = YCrCb 4:1:1 Planar
   1010 = Reserved
   1011 = Reserved
   1100 = Reserved
   1101 = Reserved
   1110 = Raw 8X Data
   1111 = Reserved
#endif

typedef enum
{
   CF_BelowRange = -1, CF_RGB32, CF_RGB24, CF_RGB16, CF_RGB15, CF_YUY2,
   CF_BTYUV, CF_Y8, CF_RGB8, CF_PL_422, CF_PL_411, CF_YUV9, CF_YUV12, CF_VBI,
   CF_UYVY, CF_RAW = 0x0E, CF_I420, CF_AboveRange
} ColFmt;

// #define IS_PLANAR(a)  ((a) > CF_RGB8 && (a) < CF_VBI)

/* Class: ColorSpace:
 * Purpose: This class provides the functionality of the BtPisces color
 *   space converter
 * Attributes: CurColor_: ColFmt - current color format
 * Operations:
      SetColorFormat( ColFmt aColor ): - this method sets the color format
      ColFmt GetColorFormat(): - this method returns the current color format
      BYTE GetBitCount(): - this method returns number of bpp for the current
         color format.
 */
class ColorSpace
{
   private:
      ColFmt CurColor_;
      static const BYTE BitCount_ [];
      static const BYTE XRestriction_ [];
      static const BYTE YRestriction_ [];
      static const BYTE YPlaneBitCount_ [];
      static const FOURCC FourccArr_ [];
      static const GUID *VideoGUIDs [];
      
   public:
      void   SetColorFormat(const ColFmt aColor ) { CurColor_ = aColor; }
      DWORD  GetBitCount() const { return BitCount_ [CurColor_]; }
      DWORD  GetPitchBpp() const { return YPlaneBitCount_ [CurColor_]; }
      FOURCC GetFourcc() const { return FourccArr_ [CurColor_]; }
      bool   CheckDimentions( const SIZE &aSize ) const;
#if DBG
      bool   CheckLeftTop( const MPoint &lt ) const;
#else
      bool   CheckLeftTop( const MPoint &lt ) const { return bool( !( lt.x & 3 ) ); } /* Force alignment of the scanlines to DWORDs */
#endif
      ColorSpace( const ColFmt aColForm ) : CurColor_( aColForm ) {}
      ColorSpace( const FOURCC fcc, const int bitCount , const DWORD mask0 = 0, const DWORD mask1 = 0, const DWORD mask2 = 0 );
      ColorSpace( const GUID &guid );

      ColFmt GetColorFormat() const { return CurColor_; }

      bool IsValid() const { return bool( CurColor_ > CF_BelowRange && CurColor_ < CF_AboveRange ); }
};

#endif __COLSPACE_H
