/*
	Command.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __COMMAND_H
#define __COMMAND_H

#include "debug.hpp" // Debugging purposes
#include "CppRt.hpp"
#include "MyTypes.hpp"


/* Type: Instruction
 * Purpose: enumerates all RISC commands
 */
typedef enum
{
   WRIT    = 0x01, SKIP = 0x02, WRITEC   = 0x05,
   JUMP    = 0x07, SYNC = 0x08, WRITE123 = 0x09,
   SKIP123 = 0x0A, WRITE1S23 = 0x0B, Reserved = 0xFF
} Instruction;

/* Type: SyncCode
 * Purpose: enumerates all sync codes coming out of decoder
*/
typedef enum
{
   SC_FM1 = 0x6, SC_FM3  = 0xE, SC_VRE = 0x4,
   SC_VRO = 0xC, SC_RESYNC = 1, SC_Reserved = 0xFF
} SyncCode;

// Macros to generate the RISC instructions as fast as possible!
#ifdef FORCE_RANGE_RISCS

#define COUNT_MSK & 0xFFF
#define BEN_MSK & 0xF
#define RSET_MSK & 0xFF
#define OP_MSK & 0xF
#define STAT_MSK & 0xF

#else

#define COUNT_MSK
#define BEN_MSK
#define RSET_MSK
#define OP_MSK
#define STAT_MSK

#endif

// RISC updating primitive
#if 0

// When we write directly to memory...
#define USES_RISCCREATOR
#define UPD(ptr,size_r) ptr += size_r

// RISC writing primitive
#if 1
#define WDW(addr,off,val) ((volatile DWORD*)(addr))[(off)] = (DWORD)(val)
#else
#define WDW(addr,off,val) WRITE_REGISTER_ULONG( ((PULONG)(addr))+(off),(val) )
#endif

#else

#if 0
// When we buffer writes
#define USES_RISCCREATOR DWORD __r_buff[5]
#define UPD(ptr,size_r) memcpy(ptr,__r_buff,size_r * sizeof(DWORD)); ptr+= size_r
#define WDW(addr,off,val) __r_buff[(off)] = (DWORD)(val)

#else

// The following sequence forces the compiler to create the RISC instruction and write it at once as the last step. 
// Compiler is not allowed to write operands as soon they are calculated: It calculates all the required bytes
// and then writes as a single operation all the instructions
#define USES_RISCCREATOR register DWORD __r_buff[5]
#define UPD(ptr,size_r) if (size_r == 1) { \
							((volatile DWORD*)ptr)[0] = __r_buff[0]; \
						} else if (size_r == 2) { \
							((volatile DWORD*)ptr)[0] = __r_buff[0]; \
							((volatile DWORD*)ptr)[1] = __r_buff[1]; \
						} else if (size_r == 3) { \
							((volatile DWORD*)ptr)[0] = __r_buff[0]; \
							((volatile DWORD*)ptr)[1] = __r_buff[1]; \
							((volatile DWORD*)ptr)[2] = __r_buff[2]; \
						} else if (size_r == 5) { \
							((volatile DWORD*)ptr)[0] = __r_buff[0]; \
							((volatile DWORD*)ptr)[1] = __r_buff[1]; \
							((volatile DWORD*)ptr)[2] = __r_buff[2]; \
							((volatile DWORD*)ptr)[3] = __r_buff[3]; \
							((volatile DWORD*)ptr)[4] = __r_buff[4]; \
						} \
						ptr += size_r

#define WDW(addr,off,val) __r_buff[(off)] = (DWORD)(val)

#endif

#endif

// RISC instruction core
#define RISC_CORE(count,byteenables,rset,irq,eol,sol,opcode) \
					(count COUNT_MSK) | \
					((byteenables BEN_MSK) << 12) | \
					((rset RSET_MSK) << 16) | \
					((irq) ? 0x1000000 : 0) | \
					((eol) ? 0x4000000 : 0) | \
					((sol) ? 0x8000000 : 0) | \
					((opcode OP_MSK) << 28 )
								
// Useful macros
#define RISC_SIZE_WRITE 2
#define RISC_WRITE(instrptr,physaddr,count,byteenables,rset,irq,eol,sol) \
		WDW(instrptr,0,RISC_CORE(count,byteenables,rset,irq,eol,sol,0x1)); \
		WDW(instrptr,1,physaddr); \
		UPD(instrptr,RISC_SIZE_WRITE)

#define RISC_SIZE_WRITE123 5			
#define RISC_WRITE123(instrptr,physaddr1,count1,physaddr2,count2,physaddr3,count3,byteenables,rset,irq,eol,sol) \
		WDW(instrptr,0,RISC_CORE(count1,byteenables,rset,irq,eol,sol,0x9)); \
		WDW(instrptr,1,(count2 COUNT_MSK) | ((count3 COUNT_MSK) << 16)); \
		WDW(instrptr,2,physaddr1); \
		WDW(instrptr,3,physaddr2); \
		WDW(instrptr,4,physaddr3); \
		UPD(instrptr,RISC_SIZE_WRITE123)

#define RISC_SIZE_WRITE1S23 3
#define RISC_WRITE1S23(instrptr,physaddr1,count1,count2,count3,byteenables,rset,irq,eol,sol) \
		WDW(instrptr,0,RISC_CORE(count1,byteenables,rset,irq,eol,sol,0xB)); \
		WDW(instrptr,1,(count2 COUNT_MSK) | ((count3 COUNT_MSK) << 16)); \
		WDW(instrptr,2,physaddr1); \
		UPD(instrptr,RISC_SIZE_WRITE1S23)


#define RISC_SIZE_WRITEC 1
#define RISC_WRITEC(instrptr,physaddr,count,byteenables,rset,irq,eol) \
		WDW(instrptr,0,RISC_CORE(count,byteenables,rset,irq,eol,0,0x5)); \
		UPD(instrptr,RISC_SIZE_WRITEC)

#define RISC_SIZE_SKIP 1
#define RISC_SKIP(instrptr,count,rset,irq,eol,sol) \
		WDW(instrptr,0,RISC_CORE(count,0,rset,irq,eol,sol,0x2)); \
		UPD(instrptr,RISC_SIZE_SKIP)


#define RISC_SIZE_SKIP123 2
#define RISC_SKIP123(instrptr,count1,count2,count3,rset,irq,eol,sol) \
		WDW(instrptr,0,RISC_CORE(count1,0,rset,irq,eol,sol,0xA)); \
		WDW(instrptr,1,(count2 COUNT_MSK) | ((count3 COUNT_MSK) << 16)); \
		UPD(instrptr,RISC_SIZE_SKIP123)

#define RISC_SIZE_JUMP 2
#define RISC_JUMP(instrptr,physaddr,rset,irq) \
		WDW(instrptr,0,RISC_CORE(0,0,rset,irq,0,0,0x7)); \
		WDW(instrptr,1,physaddr); \
		UPD(instrptr,RISC_SIZE_JUMP)

#define RISC_SIZE_SYNC 2
#define RISC_SYNC(instrptr,status,resync,rset,irq) \
		WDW(instrptr,0,RISC_CORE(status STAT_MSK,((resync) ? 0x8 : 0),rset,irq,0,0,0x8)); \
		WDW(instrptr,1,0); \
		UPD(instrptr,RISC_SIZE_SYNC)


/*
 * Some interesting things about Set and Reset:
 *	-if Reset = 0xf and Set = 0x0, Status is set to 0!
 *  -if Reset = 0xf and Set = 0xf, Status is INCREMENTED!!! (As a 4 bit counter!)
 *
 * Note: Set bits are the 4 lsBits
 *		 Reset bits are the 4 msBits
 */

 
#define RST_RISCS 0xF0
#define INC_RISCS 0xFF

#endif __COMMAND_H

