/*
	Decoder.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "mytypes.hpp"
#include "decoder.hpp"
#include "Registry.hpp"
#include "Pisces.hpp"
#include "..\BtCard.hpp"


//===========================================================================
// Bt848 Decoder Class Implementation
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////
Decoder::Decoder(BtPisces* p,const DWORD_PTR dwBase	) :

	pisces(p),

	// init register min, max, default
	m_regHueparam( -9000, 8930, 0 ),				//
	m_regBrightnessparam( -4625 , 4589 , 750 ),		// 
	m_regContrastparam( 0 , 237 , 100 ),			// 
	m_regSaturationparam( 0, 202, 100 ),			// 
	m_regWhiteBalanceparam( -100 , 100, 0),			// 

	m_regHue( -128, 127, 0 ),						// Hue register range that matches the Hue param
	m_regBrightness( -128 , 127 , 0) ,				// Brightness register range that matches the Brightness param
	m_regContrast( 0 , 511 , 216) ,					// Contrast register range that matches the Contrast param for analog signals
	m_regContrastDigital( 0 , 303 , 128) ,			// Contrast register range that matches the Contrast param for digital signals
	m_regSaturationNTSCPAL_U( 0, 511, 0xFE ),		// Saturation register range that matches the Saturation param for analog signals
	m_regSaturationNTSCPAL_V( 0, 364, 0xB4 ),		// Saturation register range that matches the Saturation param for analog signals
	m_regSaturationSECAM_U( 0, 273, 0x87 ),			// Saturation register range that matches the Saturation param for analog signals
	m_regSaturationSECAM_V( 0, 269, 0x85 ),			// Saturation register range that matches the Saturation param for analog signals
	m_regSaturationDigital_U( 0, 259, 0x80 ),		// Saturation register range that matches the Saturation param for digital ins
	m_regSaturationDigital_V( 0, 259, 0x80 ),		// Saturation register range that matches the Saturation param for digital ins

	decRegSTATUS ( ( (0x00 + 0) * 4) + dwBase, RW ) ,
	decFieldPRES( decRegSTATUS, 7, 1, RW) ,
	decFieldHLOC( decRegSTATUS, 6, 1, RW) ,
	decFieldEVENFIELD( decRegSTATUS, 5, 1, RW) ,
	decFieldNUML( decRegSTATUS, 4, 1, RW) ,
	decFieldCSEL( decRegSTATUS, 3, 1, RW) ,
	decFieldLOF( decRegSTATUS, 1, 1, RW) ,
	decFieldCOF( decRegSTATUS, 0, 1, RW) ,

	decRegIFORM ( ( (0x01 + 0) * 4) + dwBase, RW ) ,
	decFieldHACTIVE( decRegIFORM, 7, 1, RW) ,
	decFieldMUXSEL( decRegIFORM, 5, 2, RW) ,
	decFieldXTSEL( decRegIFORM, 3, 2, RW) ,
	decFieldFORMAT( decRegIFORM, 0, 3, RW) ,

	decRegTDEC ( ( (0x02 + 0) * 4) + dwBase, RW ) ,
	decFieldDEC_FIELD( decRegTDEC, 7, 1, RW) ,
	decFieldDEC_FIELDALIGN( decRegTDEC, 6, 1, RW) ,
	decFieldDEC_RAT( decRegTDEC, 0, 6, RW) ,

	decRegBRIGHT ( ( (0x0A + 0) * 4) + dwBase, RW ) ,

	decRegMISCCONTROL ( ( (0x0B + 0) * 4) + dwBase, RW ) ,
	decFieldLNOTCH( decRegMISCCONTROL, 7, 1, RW) ,
	decFieldCOMP( decRegMISCCONTROL, 6, 1, RW) ,
	decFieldLDEC( decRegMISCCONTROL, 5, 1, RW) ,
	decFieldCBSENSE( decRegMISCCONTROL, 4, 1, RW) ,
	decFieldMISCCONTROL_RES( decRegMISCCONTROL, 3, 1, RW) ,
	decFieldCON_MSB( decRegMISCCONTROL, 2, 1, RW) ,
	decFieldSAT_U_MSB( decRegMISCCONTROL, 1, 1, RW) ,
	decFieldSAT_V_MSB( decRegMISCCONTROL, 0, 1, RW) ,

	decRegCONTRAST_LO ( ( (0x0C + 0) * 4) + dwBase, RW ) ,

	decRegSAT_U_LO ( ( (0x0D + 0) * 4) + dwBase, RW ) ,

	decRegSAT_V_LO ( ( (0x0E + 0) * 4) + dwBase, RW ) ,

	decRegHUE ( ( (0x0F + 0) * 4) + dwBase, RW ) ,

	decRegSCLOOP ( ( (0x10 + 0) * 4) + dwBase, RW ) ,
	decFieldCAGC( decRegSCLOOP, 6, 1, RW) ,
	decFieldCKILL( decRegSCLOOP, 5, 1, RW) ,
	decFieldHFILT( decRegSCLOOP, 3, 2, RW) ,

	decRegWC_UP( ( (0x11 + 0) * 4) + dwBase, RW ) ,

	decRegOFORM ( ( (0x12 + 0) * 4) + dwBase, RW ) ,
	decFieldRANGE( decRegOFORM, 7, 1, RW) ,
	decFieldCORE( decRegOFORM, 5, 2, RW) ,

	decRegVSCALE_HI ( ( (0x13 + 0) * 4) + dwBase, RW ) ,
	decFieldYCOMB( decRegVSCALE_HI, 7, 1, RW) ,
	decFieldCOMB( decRegVSCALE_HI, 6, 1, RW) ,
	decFieldINT( decRegVSCALE_HI, 5, 1, RW) ,

	decRegTEST ( ( (0x15 + 0) * 4) + dwBase, RW ) ,

	decRegADELAY ( ( (0x18 + 0) * 4) + dwBase, RW ) ,

	decRegBDELAY ( ( (0x19 + 0) * 4) + dwBase, RW ) ,

	decRegADC ( ( (0x1A + 0) * 4) + dwBase, RW ) ,
	decFieldSYNC_T( decRegADC, 5, 1, RW) ,
	decFieldAGC_EN( decRegADC, 4, 1, RW) ,
	decFieldCLK_SLEEP( decRegADC, 3, 1, RW) ,
	decFieldY_SLEEP( decRegADC, 2, 1, RW) ,
	decFieldC_SLEEP( decRegADC, 1, 1, RW) ,
	decFieldCRUSH( decRegADC, 0, 1, RW), 

	decRegVTC ( ( (0x1B + 0) * 4) + dwBase, RW ) ,
	decFieldHSFMT( decRegVTC, 7, 1, RW) ,

	decRegWC_DN( ( (0x1E + 0) * 4) + dwBase, RW ) ,

	decRegSRESET ( ( (0x1F + 0) * 4) + dwBase, RW ) ,

	decRegODD_MISCCONTROL ( ( (0x0B + -0x03) * 4) + 0x8C + dwBase, RW ) ,
	decFieldODD_LNOTCH( decRegODD_MISCCONTROL, 7, 1, RW) ,
	decFieldODD_COMP( decRegODD_MISCCONTROL, 6, 1, RW) ,
	decFieldODD_LDEC( decRegODD_MISCCONTROL, 5, 1, RW) ,
	decFieldODD_CBSENSE( decRegODD_MISCCONTROL, 4, 1, RW) ,
	decFieldODD_MISCCONTROL_RES( decRegODD_MISCCONTROL, 3, 1, RW) ,
	decFieldODD_CON_MSB( decRegODD_MISCCONTROL, 2, 1, RW) ,
	decFieldODD_SAT_U_MSB( decRegODD_MISCCONTROL, 1, 1, RW) ,
	decFieldODD_SAT_V_MSB( decRegODD_MISCCONTROL, 0, 1, RW) ,

	decRegODD_SCLOOP ( ( (0x10 + -0x03) * 4) + 0x8C + dwBase,  RW ) ,
	decFieldODD_CAGC( decRegODD_SCLOOP, 6, 1, RW) ,
	decFieldODD_CKILL( decRegODD_SCLOOP, 5, 1, RW) ,
	decFieldODD_HFILT( decRegODD_SCLOOP, 3, 2, RW) ,

	decRegODD_VSCALE_HI ( ( (0x13 + -0x03) * 4) + 0x8C + dwBase, RW ) ,
	decFieldODD_YCOMB( decRegODD_VSCALE_HI, 7, 1, RW) ,
	decFieldODD_COMB( decRegODD_VSCALE_HI, 6, 1, RW) ,
	decFieldODD_INT( decRegODD_VSCALE_HI, 5, 1, RW) ,

	decRegODD_VTC ( ( (0x1B + -0x03) * 4) + 0x8C + dwBase, RW ) ,
	decFieldODD_HSFMT( decRegODD_VTC, 7, 1, RW),
	decFieldPLOCK( decRegSTATUS, 2, 1, RW),
	decFieldPEAK( decRegSCLOOP, 7, 1, RW),
	decFieldVertEn( decRegWC_DN, 7, 1, RW),
	decFieldODD_PEAK( decRegODD_SCLOOP, 7, 1, RW),

	decRegDVSIF ( ( (0x3F + 0) * 4) + dwBase, RW) ,
	decFieldGPX_EN( decRegDVSIF, 7, 1, RW) ,
	decFieldVSIF_BCF( decRegDVSIF, 6, 1, RW) ,
	decFieldVSIF_ESO( decRegDVSIF, 5, 1, RW) ,
	decFieldSVREF( decRegDVSIF, 3, 2, RW),
	decFieldVSFMT( decRegDVSIF, 0, 3, RW),

	decRegVTOTAL_LO ( 0xB0 + dwBase, RW) ,
	decRegVTOTAL_HI ( 0xB4 + dwBase, RW) ,
	decFieldVTOTAL_HI( decRegVTOTAL_HI, 0, 2, RW),

	decRegVTOTAL( decRegVTOTAL_LO, 8, decFieldVTOTAL_HI, RW ),
	decFieldHSFMT2( decRegVTC, 6, 2, RW) ,
	decFieldODD_HSFMT2( decRegODD_VTC, 6, 2, RW),
	decFieldVSFLDALIGN( decRegVSCALE_HI, 7, 1, RW) ,
	decFieldODD_VSFLDALIGN( decRegODD_VSCALE_HI, 7, 1, RW),

	decRegColorControl( 0xD8 + dwBase , RW), 
	decFieldGAMMA( decRegColorControl, 4, 1, RW)

{
	// ENSURE the called fns here DO NOT use the BtPisces pointer, as
	// the object is not completely initialized!

	// Get Defaults
	m_BrightnessUnadj = 100;
	
	// This will force the settings when we are asked to!
	m_parColorEnable = true;
	m_parBacklightCompensation = -1;
	m_parContrast = -1;
	m_parBrightness = -1;
	m_parHue = -1;
	m_parSaturation = -1;
	m_parWhiteBalance = -1;
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
Decoder::~Decoder()
{
}

// This sets my preferred defaults
void Decoder::SetOptimumDefaults()
{
	// need to set this to 0x4F
	decRegWC_UP = 0x4F;

	// and this one to 0x7F to make sure CRUSH bit works. Also enable new VPRES circuitry (bit7)
	decRegWC_DN = 0x7F | 0x80;

	// HACTIVE should always be 0
	decFieldHACTIVE = 0;

	// HSFMT (odd and even) should always be 0
	decFieldHSFMT = decFieldODD_HSFMT = 0;
}

//===== Device Status register ==============================================

/////////////////////////////////////////////////////////////////////////////
// Method:  BYTE Decoder::GetDeviceStatusReg( void )
// Purpose: Obtain device status register value
// Input:   None
// Output:  None
// Return:  value of status register in BYTE
/////////////////////////////////////////////////////////////////////////////
BYTE Decoder::GetDeviceStatusReg( void )
{
	BYTE status = (BYTE)decRegSTATUS;
	decRegSTATUS = 0x00;
	return status;
}



//===== Input Format register ===============================================

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetVideoInput( Connector source )
// Purpose: Select which connector as input
// Input:   Connector source - SVideo, Tuner, Composite
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetVideoInput( ULONG source ,bool IsSVideo )
{
	// Set source
	decFieldMUXSEL = source;

	// Turn on Luma ADC
	SetLumaADC(true);

	// Turn off Chroma ADC if not used.
	SetChromaADC(( IsSVideo ) ? true : false);

	// set to composite or Y/C component video depends on video source
	SetCompositeVideo(( IsSVideo ) ? false : true);

	return Success;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetCrystal( Crystal crystalNo )
// Purpose: Select which crystal as input
// Input:   Crystal crystalNo:
//            XT0         - Crystal_XT0
//            XT1         - Crystal_XT1
//            Auto select - Crystal_AutoSelect
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetCrystal( Crystal crystalNo )
{
	if ( ( crystalNo < Crystal_XT0 ) || ( crystalNo >  Crystal_AutoSelect ) )
		return Fail;

	decFieldXTSEL = crystalNo;
	return Success;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetVideoFormat( VideoFormat format )
// Purpose: Set video format
// Input:   Video format -
//            Auto format:          BtVFormat_AutoDetect
//            NTSC (M):             BtVFormat_NTSC
//            PAL (B, D, G, H, I):  BtVFormat_PAL_BDGHI
//            PAL (M):              BtVFormat_PAL_M
//            PAL(N):               BtVFormat_PAL_N
//            SECAM:                BtVFormat_SECAM
//            PAL (NC):             BtVFormat_PAL_NC
// Output:  None
// Return:  Fail if error in parameter, else Success
// Notes:   Available video formats are: NTSC, PAL(B, D, G, H, I), PAL(M),
//                                       PAL(N), SECAM
//          This function also sets the AGCDelay (ADELAY) and BrustDelay
//          (BDELAY) registers
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetVideoFormat( BtVideoFormat format )
{
	DUMP(( "Decoder::SetVideoFormat( %d )\n", (DWORD)format ));

	if ( (format < BtVFormat_AutoDetect)  ||
		(format > BtVFormat_PAL_N_COMB) )
		return Fail;

	if (format == BtVFormat_AutoDetect) {			// auto format detect by examining the number of lines
		if ( Decoder::Is525LinesVideo() ) // lines == 525 -> NTSC
			decFieldFORMAT	= BtVFormat_NTSC;
		else  // lines == 625 -> PAL/SECAM
			decFieldFORMAT	= BtVFormat_PAL_BDGHI;    // PAL_BDGHI covers most areas 
	} else {
		decFieldFORMAT	= format;
	}

	RecalcSaturation();

	return Success;
}


//===== Temporal Decimation register ========================================

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetRate( bool fields, VidField even, int rate )
// Purpose: Set frames or fields rate
// Input:   bool fields   - true for fields, false for frames
//          VidField even - true to start decimation with even field, false odd
//          int  rate     - decimation rate: frames (1-50/60); fields(1-25/30)
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetRate( bool fields, VidField vf, int rate )
{
	int nMax;
	if ( Is525LinesVideo() == true )
	    nMax = 30;  // NTSC
	else
	    nMax = 25;  // PAL/SECAM

	// if setting frame rate, double the max value
	if ( fields == false )
	    nMax *= 2;

	if ( rate < 0 || rate > nMax )
	    return Fail;

	decFieldDEC_FIELD = (fields == false) ? Off : On;
	decFieldDEC_FIELDALIGN = (vf == VF_Even) ? On : Off;
	int nDrop = (BYTE) nMax - rate;
	decFieldDEC_RAT = (BYTE) (fields == false) ? nDrop : nDrop * 2;

	return Success;
}


//===== Brightness Control register =========================================

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetBrightness( int param )
// Purpose: Set video brightness
// Input:   int param - parameter value (-11059 [750] 12467)
// Output:  None
// Return:  Fail if error in parameter, else Success
// Note:    See IsAdjustContrast() for detailed description of the contrast
//          adjustment calculation
/////////////////////////////////////////////////////////////////////////////

/*
	For Bt848[A] ONLY:
		-The BRIGHTNESS function does precede the CONTRAST function in the
		 decoder signal processing sequence, therefore any brightness applied
		 is also gained by the CONTRAST setting.
		 To make the Brightness independant of Contrast, set it to:
			set brightness to B/(C/C0)
				  where B is value of brightness before adjustment
						C is contrast value
					    C0 is nominal contrast value (0xD8 or 0x80 if digital video)

	For the Bt878+:
		-CONTRAST function does precede BRIGHTNESS function in the decoder
		 signal processing sequence, so they can be assumed to be independant.
 */
ErrorCode Decoder::SetBrightness( int param )
{
	// If we already have that, dont do anything!
	if (m_parBrightness == param) 
		return Success;

	if( m_regBrightnessparam.OutOfRange( param ) )
		return Fail;

	// Remember value
	m_parBrightness = param;

	// Perform mapping to our range: If unable, quit without error (Not user's fault).
	int mapped;
	if ( Mapping( param, m_regBrightnessparam, &mapped, m_regBrightness ) == Fail )
		return Fail;

	m_BrightnessUnadj = mapped; // Remember unadjusted Brightness!

	// Adjust contrast to make it independant of brightness (only needed for
	// the Bt848[A] chipset)

	int	m_Brightness = 0;
	if (pisces->IsGeminiFamily()) {

		// Bt878 handles this properly
		m_Brightness = m_BrightnessUnadj;

	} else {

		// Bt848[A] must compensate!
		int	m_Contrast = decRegCONTRAST_LO | (decFieldCON_MSB << 8); // Always >= 0

		// Now, adjust Brightness, based on contrast to make brightness independant
		// of contrast
		if (!m_Contrast) { // Avoid division by 0!
			m_Brightness =  (!m_Brightness) ? 0 : 
							((m_Brightness>0) ? 127 : -128);
		} else {
			m_Brightness = mapped 
							* (pisces->IsDigitalSignal() ? m_regContrastDigital.Default() : m_regContrast.Default())
							/ m_Contrast;
		}

		// Be careful, brightness must be checked to know if it is out of bounds!
		if (m_Brightness < -128) 
			m_Brightness = -128;
		else if (m_Brightness > 127) 
			m_Brightness = 127;
	}

	// Set the value!
	decRegBRIGHT = (BYTE) m_Brightness & 0xff;

	return Success;
}


//===== Luma Gain register (CON_MSB, CONTRAST_LO) ===========================

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetContrast( int param )
// Purpose: Set video contrast
// Input:   int param - parameter value (gain factor x 100) -> 0 - 236, def = 100
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetContrast( int param )
{
	// If we already have that, dont do anything!
	if (m_parContrast == param ) 
		return Success;

	if( m_regContrastparam.OutOfRange( param ) )
		return Fail;

	// Remember value
	m_parContrast = param;

	// perform mapping to our range (if error, return success, not user fault)
	int mapped = 0;
	if (pisces->IsDigitalSignal()) {
		if ( Mapping( param, m_regContrastparam, &mapped, m_regContrastDigital ) == Fail )
			return Success;
	} else {
		if ( Mapping( param, m_regContrastparam, &mapped, m_regContrast ) == Fail )
			return Success;
	}

	// contrast is set by a 9 bit value; set LSB first
	decRegCONTRAST_LO = mapped;

	// now set the Miscellaneous Control Register CON_V_MSB to the 9th bit value
	decFieldCON_MSB = decFieldODD_CON_MSB = ( (mapped & 0x0100) ? On : Off );

	// Bt848[A] Applies contrast after brightness, so brightness register must be
	// compensated to keep brightness constant.
	if (!pisces->IsGeminiFamily()) {

		// Now, adjust Brightness, based on contrast
		int m_Brightness = 0;
		if (!mapped) { // Avoid Division by 0!
			m_Brightness =  (!m_BrightnessUnadj) ? 0 : 
							((m_BrightnessUnadj > 0) ? 127 : -128);
		} else {
			m_Brightness = m_BrightnessUnadj 
							* (pisces->IsDigitalSignal() ? m_regContrastDigital.Default() : m_regContrast.Default())
							/ mapped;
		}


		// Be careful, brightness must be checked to know if it is out of bounds!
		if (m_Brightness < -128) 
			m_Brightness = -128;
		else if (m_Brightness > 127) 
			m_Brightness = 127;

		decRegBRIGHT = (BYTE) m_Brightness & 0xff;
	}

	return Success;
}

//===== Hue Control register (HUE) ==========================================

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetHue( int param )
// Purpose: Set video hue
// Input:   int param - parameter value (-8930 to 9000); default 0)
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetHue( int param )
{
	// If we have already set it , dont modify it!
	if (param == m_parHue)
		return Success;

	if( m_regHueparam.OutOfRange( param ) )
		return Fail;

	// Remember value
	m_parHue = param;

	// perform mapping to our range (if error, return success, not user fault)
	int m_Hue;
	if ( Mapping( param, m_regHueparam, &m_Hue, m_regHue ) == Fail )
		return Success;


	decRegHUE = m_Hue & 0xff;

	return Success;
}



//===== Chroma Gain register (SAT_U_MSB, SAT_V_MSB, SAT_U_LO, SAT_V_LO) =====

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetSaturation( int param )
// Purpose: Set color saturation by modifying U and V values
// Input:   int param - parameter value (0 - 384; default 100)
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetSaturation( int param )
{
	// If already did it, don't do it again!
	if (param == m_parSaturation) 
		return Success;

	// If out of bounds, error
	if( m_regSaturationparam.OutOfRange( param ) ) {
		return Fail;
	}

	// Remember this!
	m_parSaturation = param; 

	// color saturation is controlled by two nine bit values:
	// ChromaU & ChromaV
	// To maintain normal color balance, the ratio between the 2 register
	// values should be kept at the default ratio for the current videonorm
	
	// Also Use WhiteBalance to UNBALANCE this:
	// WhileBalance [-100 a 100]

	int UValue = 0;
	int VValue = 0;
	if (pisces->IsDigitalSignal()) {

		// Digital video has it's own values!
		if ( Mapping( param, m_regSaturationparam, &UValue, m_regSaturationDigital_U ) == Fail )
			return Fail;
		if ( Mapping( param, m_regSaturationparam, &VValue, m_regSaturationDigital_V ) == Fail )
			return Fail;

		UValue += m_parWhiteBalance * m_regSaturationDigital_U.Default() / 100;
		VValue -= m_parWhiteBalance * m_regSaturationDigital_V.Default() / 100;

	} else {
		// Analog Video

		if ( GetVideoFormat() == BtVFormat_SECAM ) {

			// SECAM
			if ( Mapping( param, m_regSaturationparam, &UValue, m_regSaturationSECAM_U ) == Fail )
				return Fail;
			if ( Mapping( param, m_regSaturationparam, &VValue, m_regSaturationSECAM_V ) == Fail )
				return Fail;

			UValue += m_parWhiteBalance * m_regSaturationSECAM_U.Default() / 100;
			VValue -= m_parWhiteBalance * m_regSaturationSECAM_V.Default() / 100;

		} else {

			// PAL and NTSC
			if ( Mapping( param, m_regSaturationparam, &UValue, m_regSaturationNTSCPAL_U ) == Fail )
				return Fail;
			if ( Mapping( param, m_regSaturationparam, &VValue, m_regSaturationNTSCPAL_V ) == Fail )
				return Fail;

			UValue += m_parWhiteBalance * m_regSaturationNTSCPAL_U.Default() / 100;
			VValue -= m_parWhiteBalance * m_regSaturationNTSCPAL_V.Default() / 100;

		}
	}

	// Limit Values!
	if (UValue > 511) UValue = 511;
	else if (UValue < 0) UValue = 0;
	if (VValue > 511) VValue = 511;
	else if (VValue < 0) VValue = 0;
	
	// Turn off color if requested to do so!
	if (!m_parColorEnable) {
		UValue = VValue = 0;
	}

	// set U
	decRegSAT_U_LO = UValue;

	// now set the Miscellaneous Control Register SAT_U_MSB to the 9th bit value
	decFieldSAT_U_MSB = decFieldODD_SAT_U_MSB = ( (UValue & 0x0100) ? On : Off );

	// set V
	decRegSAT_V_LO = VValue;

	// now set the Miscellaneous Control Register SAT_V_MSB to the 9th bit value
	decFieldSAT_V_MSB = decFieldODD_SAT_V_MSB = ( (VValue & 0x0100) ? On : Off );

	return Success;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  Decoder::SetWhiteBalance( void )
// Purpose: Set Whitebalance value
// Input:   None
// Output:  None
// Return:  Saturation parameter (-100 a 100)
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetWhiteBalance(LONG Data)
{
	// If already have that, accept it!
	if (Data == m_parWhiteBalance) 
		return Success;

	// If out of bounds, error
	if( m_regWhiteBalanceparam.OutOfRange( Data ) ) {
		return Fail;
	}

	// Remember this!
	m_parWhiteBalance = Data;

	// Adjust croma!
	RecalcSaturation();
	return Success;
}

ErrorCode Decoder::SetColorEnable(bool Data)
{
	if (Data == m_parColorEnable) 
		return Success;

	m_parColorEnable = (Data) ? true : false;
	// Adjust chroma!
	RecalcSaturation();
	return Success;
}

void  Decoder::RecalcSaturation()
{
	LONG old = m_parSaturation;
	m_parSaturation = -1;
	SetSaturation(old);
}


//===== Miscellaneous Control register (E_CONTROL, O_CONTROL) ===============



/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetCompositeVideo( bool mode )
// Purpose: Select composite or Y/C component video
// Input:   bool mode - true = Composite; false = Y/C Component
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void Decoder::SetCompositeVideo( bool mode )
{
  if ( mode == true )
  {
    // composite video
    decFieldCOMP = decFieldODD_COMP = Off;
    SetChromaADC( false );			// disable chroma ADC
    SetLumaNotchFilter( true );		// enable luma notch filter
  }
  else
  {
    // Y/C Component video
    decFieldCOMP = decFieldODD_COMP = On;
    SetChromaADC( true );			// enable chroma ADC
    SetLumaNotchFilter( false );	// disable luma notch filter
  }
}




/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetHorizontalFilter( HorizFilter hFilter )
// Purpose: Control the configuration of the optional 6-tap Horizontal Low-Pass filter
// Input:   HoriFilter hFilter:
//            Auto Format - HFilter_AutoFormat
//            CIF         - HFilter_CIF
//            QCIF        - HFilter_QCIF
//            ICON        - HFilter_ICON
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetHorizontalFilter( HorizFilter hFilter )
{
  if ( (hFilter < HFilter_AutoFormat) ||
       (hFilter > HFilter_ICON) )
    return Fail;

  decFieldHFILT = decFieldODD_HFILT = hFilter;
  return Success;
}



/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetLumaCoring( CoringLevel cLevel )
// Purpose: Set luminance level such that luminance signal is truncated to zero
//          if below this level
// Input:   CoringLevel cLevel -
//            Coring_None: no coring
//            Coring_8:    8
//            Coring_16:   16
//            Coring_32:   32
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetLumaCoring( CoringLevel cLevel )
{
  if ( ( cLevel < Coring_None) || ( cLevel > Coring_32 ) )
    return Fail;

  decFieldCORE = cLevel;
  return Success;
}

 

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetVScaleMode( int VSMode )
// Purpose: Select vertical scaling mode
// Input:   int VSMode - 0 = non-interlace
//                       1 = interlace
//                       2 = field aligned (Gemini family only)
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void Decoder::SetVScaleMode( int VSMode )
{
   if ( !pisces->IsGeminiFamily() )
      decFieldINT = decFieldODD_INT = ( VSMode == 0 ) ? 0 : 1;
   else
   {
      decFieldVSFLDALIGN = decFieldODD_VSFLDALIGN = ( VSMode == 2 ) ? 1 : 0;
      decFieldINT        = decFieldODD_INT        = ( VSMode == 1 ) ? 1 : 0;
   }
}
 

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetVScaleMode( void )
// Purpose: Get vertical scaling mode
// Input:   None
// Output:  None
// Return:  int VSMode - 0 = non-interlace
//                       1 = interlace
//                       2 = field aligned (Gemini family only)
/////////////////////////////////////////////////////////////////////////////
int Decoder::GetVScaleMode( void )
{
   if ( !pisces->IsGeminiFamily() )
      return( decFieldINT );
   else
      return ( decFieldINT ? 1 : ( decFieldVSFLDALIGN << 1 ) );
}

//===== Video Signal Interface Format (VSIF) ================================

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetVideoSignalFormat( VideoSignalFormat vsf )
// Purpose: Set video signal format
// Input:   Video signal formats:
//            Signal_Analog         - Analog
//            Signal_CCIR656        - CCIR 656
//            Signal_ByteStream     - ByteStream
//            Signal_ExtHSYNC_VSYNC - External HSYNC, VSYNC
//            Signal_ExtHSYNC_FIELD - External HSYNC, FIELD
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetVideoSignalFormat( SignalFormat format )
{
	if (	( format <  Signal_Analog )         ||
			( format >  Signal_ExtHSYNC_FIELD ) ||
			( format == Signal_Reserved3 ) )
		return Fail;

	switch (format) {
		case Signal_CCIR656:
		case Signal_ByteStream:
		case Signal_ExtHSYNC_VSYNC:
		case Signal_ExtHSYNC_FIELD:

			// GPX_EN has been deleted in Gemini family
			if ( !pisces->IsGeminiFamily() )
				RemapGPDATA( true );
			break;

		case Signal_Analog:
			
			// GPX_EN has been deleted in Gemini family
			if ( !pisces->IsGeminiFamily() )
				RemapGPDATA( true );
			break;

		default:
			return Fail;
	}

	decFieldVSFMT = format;
	return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetSyncVideoRef( SyncVideoRef ref )
// Purpose: Set sync video reference
// Input:   Sync video reference:
//            VRef_alignedCb - HS/VS aligned with Cb
//            VRef_alignedY0 - HS/VS aligned with Y0
//            VRef_alignedCr - HS/VS aligned with Cr
//            VRef_alignedY1 - HS/VS aligned with Y1
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetSyncVideoRef( SyncVideoRef ref )
{
	if ( ( ref < VRef_alignedCb ) ||
        ( ref > VRef_alignedY1 ) )
		return Fail;

	decFieldSVREF = ref;
	return Success;
}


//===== Video Timing Control (VTC) ==========================================

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetHReset( int clk )
// Purpose: Set HRESET length (HSFMT)
// Input:   int clk - 64, 48, 32 or 16 pixels (default is 64)
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void Decoder::SetHReset( int clk )
{
	if ( pisces->IsGeminiFamilyRevD() )  // Gemini and at least Rev D
	{
		switch ( clk )
		{
			case 48: decFieldHSFMT2 = decFieldODD_HSFMT2 = 1; break;
			case 32: decFieldHSFMT2 = decFieldODD_HSFMT2 = 2; break;
			case 16: decFieldHSFMT2 = decFieldODD_HSFMT2 = 3; break;
			default: decFieldHSFMT2 = decFieldODD_HSFMT2 = 0; break;
		}
	}
	else
		decFieldHSFMT = decFieldODD_HSFMT = ( clk <= 32 );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetHReset( void )
// Purpose: Get HRESET length (HSFMT)
// Input:   None
// Output:  None
// Return:  HRESET length - 64, 48, 32 or 16 pixels 
/////////////////////////////////////////////////////////////////////////////
int Decoder::GetHReset( void )
{
	if ( pisces->IsGeminiFamilyRevD() )  // Gemini and at least Rev D
		return ( (( decFieldHSFMT2 == 1 ) ? 48 :
				(( decFieldHSFMT2 == 2 ) ? 32 :
				(( decFieldHSFMT2 == 3 ) ? 16 : 64))) );
	else
		return ( ( decFieldHSFMT == 0 ) ? 64 : 32 );
}



//===== Expected # of horizontal lines per frame (VTOTAL) - new for Gemini ==

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::SetLineCount( int lines )
// Purpose: Set expected number of horizontal video lines per frame
// Input:   None
// Output:  None
// Return:  Success, Fail
/////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::SetLineCount( int lines )
{
   if ( ( lines < 0 ) || ( lines > 0x3FF ) )    // max 10 bits
      return ( Fail );

   decRegVTOTAL = lines;
   return ( Success );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Decoder::Mapping( int fromValue, CRegInfo fromRange,
//                                           int * toValue, CRegInfo toRange )
// Purpose: Map a value in certain range to a value in another range
// Input:   int fromValue - value to be mapped from
//          CRegInfo fromRange - range of value mapping from
//          CRegInfo toRange   - range of value mapping to
// Output:  int * toValue - mapped value
// Return:  Fail if error in parameter, else Success
// Comment: No range checking is performed here. Assume parameters are in
//          valid ranges.
//          The mapping function does not assume default is always the mid
//          point of the whole range. It only assumes default values of the
//          two ranges correspond to each other.
//
////////////////////////////////////////////////////////////////////////////
ErrorCode Decoder::Mapping( int fromValue, const CRegInfo& fromRange,
   int * toValue, const CRegInfo& toRange )
{
   // calculate intermediate values
   int ToLowRange    = toRange.Default() - toRange.Low();
   int ToHighRange   = toRange.High() - toRange.Default();

   int FromLowRange  = fromRange.Default() - fromRange.Low();
   int FromHighRange = fromRange.High() - fromRange.Default();

   // prevent divide by zero
   if ( !FromLowRange || !FromHighRange )
      return ( Fail );

   // perform mapping
   if (FromLowRange > 0) { // Low < Def < High

		if ( fromValue <= fromRange.Default() )
			*toValue = ((fromValue - fromRange.Low()) * ToLowRange / FromLowRange) + toRange.Low();
		else
			*toValue = ((fromValue - fromRange.Default()) * ToHighRange / FromHighRange) + toRange.Default();

   } else { // Low > Def > High

		if ( fromValue >= fromRange.Default() )
			*toValue = ((fromValue - fromRange.Low()) * ToLowRange / FromLowRange) + toRange.Low();
		else
			*toValue = ((fromValue - fromRange.Default()) * ToHighRange / FromHighRange) + toRange.Default();

   }

   return ( Success );
}

ErrorCode Decoder::SetBacklightCompensation(LONG Data)
{
	if (Data <0 || Data > 1) {
		return Fail;
	}

	if (m_parBacklightCompensation == Data)
		return Success;

	// Remember this
	m_parBacklightCompensation = Data;

	switch (Data) {
		case 1:
			return SetLumaCoring(Coring_8);
		case 2:
			return SetLumaCoring(Coring_16);
		case 3:
			return SetLumaCoring(Coring_32);
	}
	return SetLumaCoring(Coring_None);
}

LONG Decoder::GetBacklightCompensation()
{
	switch (GetLumaCoring()) {
		case Coring_None:
			return (m_parBacklightCompensation=0);
		case Coring_8:
			return (m_parBacklightCompensation=1);
		case Coring_16:
			return (m_parBacklightCompensation=2);
		case Coring_32:
			return (m_parBacklightCompensation=3);
	}
	return (m_parBacklightCompensation=0);
}

