/*
	Decoder.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __DECODER_H
#define __DECODER_H

#include "debug.hpp"
#include "regField.hpp"
#include "CompReg.hpp"
#include "mytypes.hpp"
#include "myvstds.hpp"
#include "..\Enums.hpp"

/* Type: State
 * Purpose: used to define on-off operations
 */
typedef enum { Off, On } State;

/* Type: Field
 * Purpose: defines fields
 */
typedef enum { VF_Even, VF_Odd } VidField;

/* Type: PsVideoFormat
 * Purpose: Used to define video format
 */
typedef enum {  BtVFormat_AutoDetect,
                BtVFormat_NTSC,
                BtVFormat_NTSC_J,
                BtVFormat_PAL_BDGHI,
                BtVFormat_PAL_M,
                BtVFormat_PAL_N,
                BtVFormat_SECAM,
                BtVFormat_PAL_N_COMB,		// this mode was new to 848A
} BtVideoFormat;

/* Type: LumaRange
 * Purpose: Used to define Luma Output Range
 */
typedef enum { LumaNormal, LumaFull } LumaRange;

/* Type: OutputRounding
 * Purpose: Controls the number of bits output
 */
typedef enum { RND_Normal, RND_6Luma4Chroma, RND_7Luma5Chroma } OutputRounding;

/* Type: ClampLevel
 * Purpose: Defines the clamp levels
 */
typedef enum { ClampLow, ClampMiddle, ClampNormal, ClampHi } ClampLevel;


/*
 * Type: Crystal
 * Purpose: Defines which crystal to use
 */
typedef enum { Crystal_XT0 = 1, Crystal_XT1, Crystal_AutoSelect } Crystal;


/*
 * Type: HoriFilter
 * Purpose: Defines horizontal low-pass filter
 */
typedef enum { HFilter_AutoFormat,
               HFilter_CIF,
               HFilter_QCIF,
               HFilter_ICON } HorizFilter;

/*
 * Type: CoringLevel
 * Purpose: Defines Luma coring level
 */
typedef enum { Coring_None,
               Coring_8,
               Coring_16,
               Coring_32 } CoringLevel;

/*
 * Type: ThreeState
 * Purpose: Defines output three-states for the OE pin
 */
typedef enum { TS_Timing_Data,
               TS_Data,
               TS_Timing_Data_Clock,
               TS_Clock_Data } ThreeState;

/*
 * Type: SCLoopGain
 * Purpose: Defines subcarrier loop gain
 */
typedef enum { SC_Normal, SC_DivBy8, SC_DivBy16, SC_DivBy32 } SCLoopGain;

/*
 * Type: ComparePt
 * Purpose: Defines the majority comparison point for the White Crush Up function
 */
typedef enum { CompPt_3Q, CompPt_2Q, CompPt_1Q, CompPt_Auto } ComparePt;

/*
 * Type: PeakFilter
 * Purpose: Defines peaking filters
 */
typedef enum { PeakFilter_Min = 1,
               PeakFilter_Med,
               PeakFilter_Max } PeakFilter;


/* Type: SignalFormat
 * Purpose: Defines video signal format
 */
typedef enum {  Signal_Analog = 0,
                Signal_CCIR656,
                Signal_ByteStream,
                Signal_Reserved3,
                Signal_ExtHSYNC_VSYNC,
                Signal_ExtHSYNC_FIELD } SignalFormat;

/* Type: SyncVideoRef
 * Purpose: Defines synchrous video reference
 */
typedef enum {  VRef_alignedCb = 0,
                VRef_alignedY0,
                VRef_alignedCr,
                VRef_alignedY1 } SyncVideoRef;

/*
 *
 */
typedef enum {
	TEST_None				 = 0x01,	// Means normal operation
	TEST_DisableLockToSignal = 0xEF     // Means disable lock to video signal
} TESTValues;

/////////////////////////////////////////////////////////////////////////////
// CLASS CRegInfo
//
// Description:
//    Provides Low, High, and default values for a register. To use this class,
//    user will declare an object of this class and provide Low, High and default
//    values of the register.
//
// Attributes:
//    int intLow - low range value
//    int intHigh - high range value
//    int intDefault - default value
//
// Methods:
//    Low() : return minimum value of the register
//    High() : return maximum value of the register
//    Default(): return default value of the register
//    OutOfRange() : check if an value is out of range
//
/////////////////////////////////////////////////////////////////////////////
class CRegInfo
{
	const int intLow;       // low limit value
	const int intHigh;      // high limit value
	const int intDefault;   // default value

public:
	
	CRegInfo(const int low,const int high,const int def ) :
	  intLow(low) , intHigh(high) , intDefault(def) {}

	// return min, max and default value of a register
	inline int Low( void ) const { return intLow; }
	inline int High( void ) const { return intHigh; }
	inline int Default( void ) const { return intDefault; }

	// check if an value is out of range of a register
	inline bool OutOfRange( int x ) const
	{
		if (intLow < intHigh) {
			if( (x < intLow) || (x > intHigh) )
				return true;
		} else {
			if( (x < intHigh) || (x > intLow) )
				return true;
		}
		return false;
	}
};

// Foward declaration
class BtPisces;

/////////////////////////////////////////////////////////////////////////////
// CLASS Decoder
//
// Description:
//    This class encapsulates the register fields in the decoder portion of
//    the Bt848.
//    A complete set of functions are developed to manipulate all the
//    register fields in the decoder for the Bt848.
//    For Read-Write register field, "Set..." function is provided to modify
//    the content of the reigster field. And either "Get..." (for more
//    than 1 bit) or "Is..." (for 1 bit) function is provided to obtain the
//    value of the register field.
//    For Read-Only register field, only "Get..." (for more than 1 bit) or
//    "Is..." (for 1 bit) function is provided to obtain the content of the
//    register field.
//    When there are odd-field complements to the even-field register field,
//    same value is set to both odd and even register fields.
//    Several direct register content modifying/retrieval functions are
//    implemented for direct access to the register contents. They were
//    originally developed for testing purpose only. They are retained in the
//    class for convenience only and usage of these functions must be very cautious.
//
// Methods:
//    See below
//
// Note: 1) Scaling registers are not implemented.
//       2) Odd-fields are set to the same value as the even-field registers
/////////////////////////////////////////////////////////////////////////////

class Decoder
{
	private:

		RegisterB decRegSTATUS;
		RegField  decFieldPRES;
		RegField  decFieldHLOC;
		RegField  decFieldEVENFIELD;
		RegField  decFieldNUML;
		RegField  decFieldCSEL;
		RegField  decFieldPLOCK;  // replaces decFieldSTATUS_RES in Status bit2
		RegField  decFieldLOF;
		RegField  decFieldCOF;
		RegisterB decRegIFORM;
		RegField  decFieldHACTIVE;
		RegField  decFieldMUXSEL;
		RegField  decFieldXTSEL;
		RegField  decFieldFORMAT;
		RegisterB decRegTDEC;
		RegField  decFieldDEC_FIELD;
		RegField  decFieldDEC_FIELDALIGN;
		RegField  decFieldDEC_RAT;
		RegisterB decRegBRIGHT;
		RegisterB decRegMISCCONTROL;
		RegField  decFieldLNOTCH;
		RegField  decFieldCOMP;
		RegField  decFieldLDEC;
		RegField  decFieldCBSENSE;
		RegField  decFieldMISCCONTROL_RES;
		RegField  decFieldCON_MSB;
		RegField  decFieldSAT_U_MSB;
		RegField  decFieldSAT_V_MSB;
		RegisterB decRegCONTRAST_LO;
		RegisterB decRegSAT_U_LO;
		RegisterB decRegSAT_V_LO;
		RegisterB decRegHUE;
		RegisterB decRegSCLOOP;
		RegField  decFieldPEAK;
		RegField  decFieldCAGC;
		RegField  decFieldCKILL;
		RegField  decFieldHFILT;
		RegisterB decRegWC_UP;
		RegisterB decRegOFORM;
		RegField  decFieldRANGE;
		RegField  decFieldCORE;
		RegisterB decRegVSCALE_HI;
		RegField  decFieldYCOMB;
		RegField  decFieldVSFLDALIGN;
		RegField  decFieldCOMB;
		RegField  decFieldINT;
		RegisterB decRegTEST;
		RegisterB decRegADELAY;
		RegisterB decRegBDELAY;
		RegisterB decRegADC;
		RegField  decFieldSYNC_T;
		RegField  decFieldAGC_EN;
		RegField  decFieldCLK_SLEEP;
		RegField  decFieldY_SLEEP;
		RegField  decFieldC_SLEEP;
		RegField  decFieldCRUSH;
		RegisterB decRegVTC;
		RegField  decFieldHSFMT;
		RegisterB decRegWC_DN;
		RegField  decFieldVertEn;
		RegisterB decRegSRESET;
		RegisterB decRegODD_MISCCONTROL;
		RegField  decFieldODD_LNOTCH;
		RegField  decFieldODD_COMP;
		RegField  decFieldODD_LDEC;
		RegField  decFieldODD_CBSENSE;
		RegField  decFieldODD_MISCCONTROL_RES;
		RegField  decFieldODD_CON_MSB;
		RegField  decFieldODD_SAT_U_MSB;
		RegField  decFieldODD_SAT_V_MSB;
		RegisterB decRegODD_SCLOOP;
		RegField  decFieldODD_PEAK;
		RegField  decFieldODD_CAGC;
		RegField  decFieldODD_CKILL;
		RegField  decFieldODD_HFILT;
		RegisterB decRegODD_VSCALE_HI;
		RegField  decFieldODD_YCOMB;
		RegField  decFieldODD_VSFLDALIGN;
		RegField  decFieldODD_COMB;
		RegField  decFieldODD_INT;
		RegisterB decRegODD_VTC;
		RegField  decFieldODD_HSFMT;
		RegisterB decRegDVSIF;
		RegField  decFieldGPX_EN;
		RegField  decFieldVSIF_BCF;
		RegField  decFieldVSIF_ESO;
		RegField  decFieldSVREF;
		RegField  decFieldVSFMT;
		RegisterB decRegVTOTAL_LO;       // new for Gemini
		RegisterB decRegVTOTAL_HI;
		RegField  decFieldVTOTAL_HI;
		CompositeReg decRegVTOTAL;
		RegField  decFieldHSFMT2;        // Gemini Rev D: expanded HSFMT to 2 bits - VTC[7:6]
		RegField  decFieldODD_HSFMT2;    // Gemini Rev D: expanded HSFMT to 2 bits - VTC[7:6]

		RegisterDW decRegColorControl; 
		RegField decFieldGAMMA; 
		
		// Range of each parameter
		const CRegInfo m_regHueparam;
		const CRegInfo m_regBrightnessparam;
		const CRegInfo m_regContrastparam;
		const CRegInfo m_regSaturationparam;
		const CRegInfo m_regWhiteBalanceparam;

		// Register range that matches the parameter range
		const CRegInfo m_regHue;
		const CRegInfo m_regBrightness;
		const CRegInfo m_regContrast;
		const CRegInfo m_regContrastDigital;
		const CRegInfo m_regSaturationNTSCPAL_U;
		const CRegInfo m_regSaturationNTSCPAL_V;
		const CRegInfo m_regSaturationSECAM_U;
		const CRegInfo m_regSaturationSECAM_V;
		const CRegInfo m_regSaturationDigital_U;
		const CRegInfo m_regSaturationDigital_V;

		int			m_BrightnessUnadj; // Needed to adjust brightness based on contrast on Bt848[A]

		bool		m_parColorEnable;
		LONG		m_parBacklightCompensation;
		int			m_parBrightness;
		int			m_parContrast;
		int			m_parHue;
		LONG		m_parSaturation;
		LONG		m_parWhiteBalance;
		
		BtPisces *	pisces;

	public:
		// constructor and destructor
		Decoder(BtPisces* p, const DWORD_PTR dwBase);
		void SetOptimumDefaults(); // To set the default settings!
		~Decoder();

		bool      IsVideoPresent( void );
		bool      IsDeviceInHLock( void );
		bool      Is525LinesVideo( void );

		// PLL
		bool      IsPLLOutOfLock( void );
		void      ResetPLLOutOfLock( void );

		// Input Format register (IFORM)
		ErrorCode SetVideoInput( ULONG ,bool IsSVideo);
		ULONG     GetVideoInput( void );
		ErrorCode SetCrystal( Crystal );
		int       GetCrystal( void );

		BtVideoFormat GetVideoFormat( void );
		ErrorCode SetVideoFormat( BtVideoFormat );
		
		// Temporal Decimation register (TDEC)
		ErrorCode SetRate( bool, VidField, int );

		// Brightness Control register (BRIGHT)
		ErrorCode SetBrightness( int );
		int       GetBrightness( void );

		// Luma Gain register (CON_MSB, CONTRAST_LO)
		ErrorCode SetContrast( int );
		int       GetContrast( void );

		// Chroma Gain register (SAT_U_MSB, SAT_V_MSB, SAT_U_LO, SAT_V_LO)
		ErrorCode SetSaturation( int );
		int       GetSaturation( void );

		// Hue Control register (HUE)
		ErrorCode SetHue( int );
		int       GetHue( void );

		ErrorCode SetWhiteBalance(LONG Data);
		LONG	  GetWhiteBalance();
		ErrorCode SetBacklightCompensation(LONG Data);
		LONG	  GetBacklightCompensation();
		ErrorCode SetColorEnable(bool Data);
		bool	  GetColorEnable();

		bool	  GetAutoSaturation();
		ErrorCode SetAutoSaturation(bool Auto);
		bool	  GetAutoGamma();
		ErrorCode SetAutoGamma(bool Auto);

		// Device Status register (DSTATUS)
		BYTE      GetDeviceStatusReg( void );
		bool      IsEvenField( void );
		bool      IsCrystal0Selected( void );
		bool      IsLumaOverflow( void );
		void      ResetLumaOverflow( void );
		bool      IsChromaOverflow( void );
		void      ResetChromaOverflow( void );

		// Miscellaneous Control register (E_CONTROL, O_CONTROL)
		void      SetLumaNotchFilter( bool );
		bool      IsLumaNotchFilter( void );
		void      SetCompositeVideo( bool );
		bool      IsCompositeVideo( void );
		void      SetLumaDecimation( bool );
		bool      IsLumaDecimation( void );
		void      SetCbFirst( bool );
		bool      IsCbFirst( void );

		// SC Loop Control register (E_SCLOOP, O_SCLOOP)
		void      SetChromaAGC( bool );
		bool      IsChromaAGC( void );
		void      SetLowColorAutoRemoval( bool );
		bool      IsLowColorAutoRemoval( void );
		ErrorCode SetHorizontalFilter( HorizFilter );
		int       GetHorizontalFilter( void );
		void      SetLumaLowPassFilter( bool mode );
		bool      IsLumaLowPassFilter( void );
		void      SetPeakFilter( PeakFilter peak );
		int       GetPeakFilter( void );

		// Output Format register (OFORM)
		void      SetFullOutputRange( bool );
		bool      IsFullOutputRange( void );
		ErrorCode SetLumaCoring( CoringLevel );
		int       GetLumaCoring( void );

		// Vertical Scaling register (E_VSCALE_HI, O_VSCALE_HI)
		void      SetChromaComb( bool );
		bool      IsChromaComb( void );
		void      SetVScaleMode( int VSMode );
		int       GetVScaleMode( void );

		// AGC Delay register (ADELAY)
		void      SetAGCDelay( BYTE );
		int       GetAGCDelay( void );

		// Burst Delay register (BDELAY)
		void      SetBurstDelay( BYTE );
		int       GetBurstDelay( void );

		// ADC Interface register (ADC)
		void      SetAnalogThresholdLow( bool );
		bool      IsAnalogThresholdLow( void );
		void      SetAGCFunction( bool );
		bool      IsAGCFunction( void );
		void      PowerDown( bool );
		bool      IsPowerDown( void );
		void      SetLumaADC( bool );
		bool      IsLumaADC( void );
		void      SetChromaADC( bool );
		bool      IsChromaADC( void );
		void      SetAdaptiveAGC( bool );
		bool      IsAdaptiveAGC( void );
		void	  SetVerticalDetect( bool mode );
		bool	  GetVerticalDetect( void );

		// Software Reset register (SRESET)
		void      SoftwareReset( void );

		// The TEST register!
		void	  SetTEST( TESTValues value );

		// Video Input format (VSIF)
		ErrorCode SetVideoSignalFormat( SignalFormat format );
		int       GetVideoSignalFormat( void );
		ErrorCode SetSyncVideoRef( SyncVideoRef ref );
		int       GetSyncVideoRef( void );
		void      EnableSyncOutput( bool mode );
		bool      IsSyncOutputEnabled( void );
		void      EnableChromaFilterBypass( bool mode );
		bool      IsChromaFilterBypassEnabled( void );
		void      RemapGPDATA( bool mode );
		bool      IsGPDATARemapped( void );

		// Video Timing Control (VTC)
		void      SetHReset( int clk );
		int       GetHReset( void );

		// VTOTAL
		ErrorCode SetLineCount( int lines );
		int		  GetLineCount( void );

	protected:
		// mapping function
		static ErrorCode Mapping( int,const CRegInfo&, int *,const CRegInfo& );
		void	  RecalcSaturation();
};


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsVideoPresent( void )
// Purpose: Detect if video is present
// Input:   None
// Output:  None
// Return:  true if video present; else false
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsVideoPresent( void )
{
	return (bool) (decFieldPRES == 1);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsDeviceInHLock( void )
// Purpose: Detect if device is in H-lock
// Input:   None
// Output:  None
// Return:  true if device in H-lock; else false
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsDeviceInHLock( void )
{
	return (bool) (decFieldHLOC == 1);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsEvenField( void )
// Purpose: Reflect whether an even or odd field is being decoded
// Input:   None
// Output:  None
// Return:  true if even field; else false
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsEvenField( void )
{
	return (bool) (decFieldEVENFIELD == 1);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::Is525LinesVideo( void )
// Purpose: Check to see if we are dealing with 525 lines video signal
// Input:   None
// Output:  None
// Return:  true if 525 lines detected; else false (assume 625 lines)
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::Is525LinesVideo( void )
{
	return (bool) (decFieldNUML == 0);  //525
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsCrystal0Selected( void )
// Purpose: Reflect whether XTAL0 or XTAL1 is selected
// Input:   None
// Output:  None
// Return:  true if XTAL0 selected; else false (XTAL1 selected)
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsCrystal0Selected( void )
{
	return (bool) (decFieldCSEL == 0);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsPLLOutOfLock( void )
// Purpose: Indicates if PLL is out of lock
// Input:   None
// Output:  None
// Return:  true if PLL is out of lock; else false
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsPLLOutOfLock( void )
{
	return (bool) (decFieldPLOCK == 1);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::ResetPLLOutOfLock( void )
// Purpose: Reset PLL out of lock bit
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::ResetPLLOutOfLock( void )
{
	decFieldPLOCK = 0;  // write to it will reset the bit
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsLumaOverflow( void )
// Purpose: Indicates if luma ADC overflow
// Input:   None
// Output:  None
// Return:  true if luma ADC overflow; else false
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsLumaOverflow( void )
{
	return (bool) (decFieldLOF == 1);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::ResetLumaOverflow( void )
// Purpose: Reset luma ADC overflow bit
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::ResetLumaOverflow( void )
{
	decFieldLOF = 0;  // write to it will reset the bit
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsChromaOverflow( void )
// Purpose: Indicates if chroma ADC overflow
// Input:   None
// Output:  None
// Return:  true if chroma ADC overflow; else false
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsChromaOverflow( void )
{
	return (bool) (decFieldCOF == 1);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::ResetChromaOverflow( void )
// Purpose: Reset chroma ADC overflow bit
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::ResetChromaOverflow( void )
{
	decFieldCOF = 0;  // write to it will reset the bit
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetVideoInput( void )
// Purpose: Get which connector is input
// Input:   None
// Output:  None
// Return:  Video source - SVideo, Tuner, Composite
/////////////////////////////////////////////////////////////////////////////
inline ULONG Decoder::GetVideoInput( void )
{
	return ((int)decFieldMUXSEL);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetCrystal( void )
// Purpose: Get which crystal is input
// Input:   None
// Output:  None
// Return:   Crystal Number:
//            XT0         - Crystal_XT0
//            XT1         - Crystal_XT1
//            Auto select - Crystal_AutoSelect
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetCrystal( void )
{
	return ((int)decFieldXTSEL);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetBrightness( void )
// Purpose: Obtain brightness value
// Input:   None
// Output:  None
// Return:  Brightness parameter (-11059 [750] 12467)
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetBrightness( void )
{
	return m_parBrightness;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetContrast( void )
// Purpose: Obtain contrast value
// Input:   None
// Output:  None
// Return:  Contrast parameter 0 - 236, def = 100
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetContrast( void )
{
	return m_parContrast;
}



/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetHue( void )
// Purpose: Obtain hue value
// Input:   None
// Output:  None
// Return:  Hue parameter (-8930 to 9000)
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetHue( void )
{
	return m_parHue;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetSaturation( void )
// Purpose: Obtain saturation value
// Input:   None
// Output:  None
// Return:  Saturation parameter (0-384)
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetSaturation( void )
{
  return m_parSaturation;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetWhiteBalance( void )
// Purpose: Obtain Whitebalance value
// Input:   None
// Output:  None
// Return:  Saturation parameter (-100 a 100)
/////////////////////////////////////////////////////////////////////////////
inline LONG Decoder::GetWhiteBalance()
{
	return m_parWhiteBalance;
}

inline bool Decoder::GetColorEnable()
{
	return m_parColorEnable;
}

inline bool Decoder::GetAutoSaturation() // Means AGC Color
{
	return (IsChromaAGC()) ? true : false;
}

inline ErrorCode Decoder::SetAutoSaturation(bool Auto)
{
	SetChromaAGC((Auto) ? true : false);
	return Success;
}

inline bool Decoder::GetAutoGamma()
{
	return (decFieldGAMMA == 1);
}

inline ErrorCode Decoder::SetAutoGamma(bool Auto)
{
	decFieldGAMMA = (Auto) ? 1 : 0;
	return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetLumaNotchFilter( bool mode )
// Purpose: Enable/Disable luma notch filter
// Input:   bool mode - true = Enable; false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetLumaNotchFilter( bool mode )
{
  decFieldLNOTCH = decFieldODD_LNOTCH = (mode == false) ? On : Off;  // reverse
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsLumaNotchFilter( void )
// Purpose: Check if luma notch filter is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable; false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsLumaNotchFilter( void )
{
  return (decFieldLNOTCH == Off);  // reverse
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsCompositeVideo( void )
// Purpose: Check if selected composite or Y/C component video
// Input:   None
// Output:  None
// Return:  true = Composite; false = Y/C Component
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsCompositeVideo( void )
{
  return (decFieldCOMP == Off);  // reverse
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetLumaDecimation( bool mode )
// Purpose: Enable/Disable luma decimation filter
// Input:   bool mode - true = Enable; false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetLumaDecimation( bool mode )
{
   // value of 0 turns the decimation on
   decFieldLDEC = decFieldODD_LDEC = (mode == true) ? 0 : 1;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsLumaDecimation( void )
// Purpose: Check if luma decimation filter is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable; false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsLumaDecimation( void )
{
  return (decFieldLDEC == Off);  // reverse
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetCbFirst( bool mode )
// Purpose: Control whether the first pixel of a line is a Cb or Cr pixel
// Input:   bool mode - true = Normal Cb, Cr order, false = Invert Cb, Cr order
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetCbFirst( bool mode )
{
  decFieldCBSENSE = decFieldODD_CBSENSE = (mode == false) ? On : Off;  // reverse
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsCbFirst( void )
// Purpose: Check if the first pixel of a line is a Cb or Cr pixel
// Input:   None
// Output:  None
// Return:  true = Normal Cb, Cr order, false = Invert Cb, Cr order
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsCbFirst( void )
{
  return (decFieldCBSENSE == Off);  // reverse
}

//===== SC Loop Control register (E_SCLOOP, O_SCLOOP) =======================

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetChromaAGC( bool mode )
// Purpose: Enable/Disable Chroma AGC compensation
// Input:   bool mode - true = Enable, false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetChromaAGC( bool mode )
{
  decFieldCAGC = decFieldODD_CAGC = (mode == false) ? Off : On;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsChromaAGC( void )
// Purpose: Check if Chroma AGC compensation is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable, false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsChromaAGC( void )
{
  return (decFieldCAGC == On);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetLowColorAutoRemoval( bool mode )
// Purpose: Enable/Disable low color detection and removal
// Input:   bool mode - true = Enable, false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetLowColorAutoRemoval( bool mode )
{
  decFieldCKILL = decFieldODD_CKILL = (mode == false) ? Off : On;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsLowColorAutoRemoval( void )
// Purpose: Check if low color detection and removal is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable, false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsLowColorAutoRemoval( void )
{
  return (decFieldCKILL == On);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetHorizontalFilter( void )
// Purpose: Get the configuration of the optional 6-tap Horizontal Low-Pass filter
// Input:   None
// Output:  None
// Return:  Which filter is using:
//            Auto Format - HFilter_AutoFormat
//            CIF         - HFilter_CIF
//            QCIF        - HFilter_QCIF
//            ICON        - HFilter_ICON
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetHorizontalFilter( void )
{
  return ((int)decFieldHFILT);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetLumaLowPassFilter( bool mode )
// Purpose: Select using normal luma low pass or luma peaking filter
// Input:   bool mode - true = peaking filter, false = normal low pass filter
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetLumaLowPassFilter( bool mode )
{
  decFieldPEAK = decFieldODD_PEAK = (mode == false) ? Off : On;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsLumaLowPassFilter( void )
// Purpose: Check if peaking filters is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable, false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsLumaLowPassFilter( void )
{
  return (decFieldPEAK == On);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetPeakFilter( PeakFilter peak )
// Purpose: Select peaking filter
// Input:   PeakFilter peak:
//             PeakFilter_Min - minimum peaking
//             PeakFilter_Med - medium peaking
//             PeakFilter_Max - maximum peaking
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetPeakFilter( PeakFilter peak )
{
      decFieldHFILT = decFieldODD_HFILT = peak;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetPeakFilter( void )
// Purpose: Return which peaking filter is selected
// Input:   None
// Output:  None
// Return:  PeakFilter peak:
//             PeakFilter_Min - minimum peaking
//             PeakFilter_Med - medium peaking
//             PeakFilter_Max - maximum peaking
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetPeakFilter( void )
{
  return ((int)decFieldHFILT);
}


//===== Output Format register (OFORM) ======================================

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetFullOutputRange( bool mode )
// Purpose: Enable/Disable full output range
// Input:   bool mode - true = Enable, false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetFullOutputRange( bool mode )
{
  decFieldRANGE = (mode == false) ? Off : On;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsFullOutputRange( void )
// Purpose: Check if full output range is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable, false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsFullOutputRange( void )
{
  return (decFieldRANGE == On);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetLumaCoring( void )
// Purpose: Get luminance level such that luminance signal is truncated to zero
//          if below this level
// Input:   None
// Output:  None
// Return:  Luma coring level -
//            Coring_None: no coring
//            Coring_8:    8
//            Coring_16:   16
//            Coring_32:   32
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetLumaCoring( void )
{
  return ((int)decFieldCORE);
}


//===== Vertical Scaling register (E_VSCALE_HI, O_VSCALE_HI) ================

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetChromaComb( bool mode )
// Purpose: Enable/Disable chroma comb
// Input:   bool mode - true = Enable, false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetChromaComb( bool mode )
{
  decFieldCOMB = (mode == false) ? Off : On;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsChromaComb( void )
// Purpose: Check if chroma comb is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable, false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsChromaComb( void )
{
  return (decFieldCOMB == On);
}
 
//===== AGC Delay register (ADELAY) =========================================

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetAGCDelay( BYTE value )
// Purpose: Set AGC Delay register
// Input:   Value to be set to
// Output:  None
// Return:  None
// NOTE:    This function set the AGC Delay register to the specified value.
//          No calculation is involved.
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetAGCDelay( BYTE value )
{
  decRegADELAY = value;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetAGCDelay( void )
// Purpose: Get AGC Delay register
// Input:   None
// Output:  None
// Return:  Register value
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetAGCDelay( void )
{
  return ((int)decRegADELAY);
}


//===== Burst Delay register (BDELAY) =========================================

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetBurstDelay( BYTE value )
// Purpose: Set Burst Delay register
// Input:   Value to be set to
// Output:  None
// Return:  None
// NOTE:    This function set the Burst Delay register to the specified value.
//          No calculation is involved.
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetBurstDelay( BYTE value )
{
  decRegBDELAY = value;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetBurstDelay( void )
// Purpose: Get Burst Delay register
// Input:   None
// Output:  None
// Return:  Register value
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetBurstDelay( void )
{
  return ((int)decRegBDELAY);
}


//===== ADC Interface register (ADC) =========================================

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetAnalogThresholdLow( bool mode )
// Purpose: Define high/low threshold level below which SYNC signal can be detected
// Input:   bool mode - true = low threshold (~75mV), false = high threshold (~125mV)
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetAnalogThresholdLow( bool mode )
{
  decFieldSYNC_T = (mode == false) ? Off : On;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsAnalogThresholdLow( void )
// Purpose: Check if high or low threshold level below which SYNC signal can be detected
// Input:   None
// Output:  None
// Return:  true = low threshold (~75mV), false = high threshold (~125mV)
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsAnalogThresholdLow( void )
{
  return (decFieldSYNC_T == On);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetAGCFunction( bool mode )
// Purpose: Enable/Disable AGC function
// Input:   bool mode - true = Enable, false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetAGCFunction( bool mode )
{
  decFieldAGC_EN = (mode == false) ? On : Off; // reverse
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsAGCFunction( void )
// Purpose: Check if AGC function is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable, false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsAGCFunction( void )
{
  return (decFieldAGC_EN == Off);   // reverse
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::PowerDown( bool mode )
// Purpose: Select normal or shut down clock operation
// Input:   bool mode - true = shut down, false = normal operation
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::PowerDown( bool mode )
{
  decFieldCLK_SLEEP = (mode == false) ? Off : On;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsPowerDown( void )
// Purpose: Check if clock operation has been shut down
// Input:   None
// Output:  None
// Return:  true = shut down, false = normal operation
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsPowerDown( void )
{
  return (decFieldCLK_SLEEP == On);
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetLumaADC( bool mode )
// Purpose: Select normal or sleep Y ADC operation
// Input:   bool mode - true = normal, false = sleep
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetLumaADC( bool mode )
{
  decFieldY_SLEEP = (mode == false) ? On : Off; // reverse
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsLumaADC( void )
// Purpose: Check if Y ADC operation is in normal operation or sleeping
// Input:   None
// Output:  None
// Return:  true = normal, false = sleep
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsLumaADC( void )
{
  return (decFieldY_SLEEP == Off);  // reverse
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetChromaADC( bool mode )
// Purpose: Select normal or sleep C ADC operation
// Input:   bool mode - true = normal, false = sleep
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetChromaADC( bool mode )
{
  decFieldC_SLEEP = (mode == false) ? On : Off; // reverse
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsChromaADC( void )
// Purpose: Check if C ADC operation is in normal operation or sleeping
// Input:   None
// Output:  None
// Return:  true = normal, false = sleep
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsChromaADC( void )
{
  return (decFieldC_SLEEP == Off); // reverse
}


/*^^////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SetAdaptiveAGC( bool mode )
// Purpose: Set adaptive or non-adaptive AGC operation
// Input:   bool mode - true = Adaptive, false = Non-adaptive
// Output:  None
// Return:  None
*////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetAdaptiveAGC( bool mode )
{
   decFieldCRUSH = (mode == false) ? Off : On;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsAdaptiveAGC( void )
// Purpose: Check if adaptive or non-adaptive AGC operation is selected
// Input:   None
// Output:  None
// Return:  true = Adaptive, false = Non-adaptive
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsAdaptiveAGC( void )
{
  return (decFieldCRUSH == On);
}

//===== White Crush Down Count (WC_DN) ======================================

/////////////////////////////////////////////////////////////////////////////
// Method:  Decoder::SetVerticalDetect( bool mode )
// Purpose: Select additional video detect algorithm
// Input:   bool mode - true = ???, false = ???
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SetVerticalDetect( bool mode )
{
   decFieldVertEn = (mode == false) ? Off : On;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::GetVerticalDetect( void )
// Purpose: Check which video detect algorithm is used
// Input:   None
// Output:  None
// Return:  true = ???, false = ???
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::GetVerticalDetect( void )
{
  return (decFieldVertEn == On);
}


//===== Software Reset register (SRESET) ====================================

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::SoftwareReset( void )
// Purpose: Perform software reset; all registers set to default values
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::SoftwareReset( void )
{
	decRegSRESET = 0x00;  // write any value will do
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetVideoSignalFormat( void )
// Purpose: Obtain video signal format
// Input:   None
// Output:  None
// Return:  Video signal format:
//            Signal_Analog         - Analog
//            Signal_CCIR656        - CCIR 656
//            Signal_ByteStream     - ByteStream
//            Signal_ExtHSYNC_VSYNC - External HSYNC, VSYNC
//            Signal_ExtHSYNC_FIELD - External HSYNC, FIELD
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetVideoSignalFormat( void )
{
	return decFieldVSFMT;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetSyncVideoRef( void )
// Purpose: Obtain sync video reference
// Input:   None
// Output:  None
// Return:  Sync video reference:
//            VRef_alignedCb - HS/VS aligned with Cb
//            VRef_alignedY0 - HS/VS aligned with Y0
//            VRef_alignedCr - HS/VS aligned with Cr
//            VRef_alignedY1 - HS/VS aligned with Y1
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetSyncVideoRef( void )
{
	return decFieldSVREF;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::EnableSyncOutput( bool mode )
// Purpose: Enable/Disable syncs output for synchronizing video input
// Input:   bool mode - true = Enable; false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::EnableSyncOutput( bool mode )
{
	decFieldVSIF_ESO = (mode == false) ? 0 : 1;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsSyncOutputEnabled( void )
// Purpose: Check if syncs output for synchronizing video input is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable; false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsSyncOutputEnabled( void )
{
	return (decFieldVSIF_ESO == On);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::EnableChromaFilterBypass( bool mode )
// Purpose: Enable/Disable bypass of chroma filters
// Input:   bool mode - true = Enable; false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::EnableChromaFilterBypass( bool mode )
{
	decFieldVSIF_BCF = (mode == false) ? 0 : 1;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsChromaFilterBypassEnabled( void )
// Purpose: Check if bypass of chroma filters is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable; false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsChromaFilterBypassEnabled( void )
{
	return (decFieldVSIF_BCF == On);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Decoder::RemapGPDATA( bool mode )
// Purpose: Enable/Disable remapping of GPDATA[5:0] inputs from GPIO[5:0] to GPx[5:0]
// Input:   bool mode - true = Enable; false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void Decoder::RemapGPDATA( bool mode )
{
	decFieldGPX_EN = (mode == false) ? 0 : 1;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool Decoder::IsGPDATARemapped( void )
// Purpose: Check if remapping of GPDATA[5:0] inputs from GPIO[5:0] to GPx[5:0]
//          is enable or disable
// Input:   None
// Output:  None
// Return:  true = Enable; false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool Decoder::IsGPDATARemapped( void )
{
  return (decFieldGPX_EN == On);
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetLineCount( void )
// Purpose: Get expected number of horizontal video lines per frame
// Input:   None
// Output:  None
// Return:  Expected number of horizontal video lines per frame
/////////////////////////////////////////////////////////////////////////////
inline int Decoder::GetLineCount( void )
{
   return ( decRegVTOTAL );
}

// The TEST register!
// Seems that 2 values can be useful: 
//	0x01 means normal operation mode
//  0xEF means disable Ultralock and don't lock to video
inline void Decoder::SetTEST( TESTValues value )
{
	decRegTEST = value;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int Decoder::GetVideoFormat( void )
// Purpose: Obtain video format
// Input:   None
// Output:  None
// Return:  Video format
//            Auto format:          BtVFormat_AutoDetect
//            NTSC (M):             BtVFormat_NTSC
//            PAL (B, D, G, H, I):  BtVFormat_PAL_BDGHI
//            PAL (M):              BtVFormat_PAL_M
//            PAL(N):               BtVFormat_PAL_N
//            SECAM:                BtVFormat_SECAM
/////////////////////////////////////////////////////////////////////////////
inline BtVideoFormat Decoder::GetVideoFormat( void )
{
	return BtVideoFormat( DWORD(decFieldFORMAT) );
}


#endif // __DECODER_H

