/*
	Field.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "Field.hpp"
#include "Pisces.hpp"

/*
 * Field::Field
 */
Field::Field( BtPisces* p, const VidField vf) :
	pisces(p),			FieldID_(vf) ,
	bufferxface(NULL) , bIsOpen(false) ,
	dwBufsInProgress(0)
{
}

Field::~Field()
{
	TRACE(("Field::~Field"));
	// If field is still open...
	ENSURE(bIsOpen == false);
}

bool Field::Open(FieldBufferXface* iface)
{
	// If already open, dont open it again!
	if (bIsOpen) {
		return false;
	}

	// Make sure it is closed
	Close();

	// Set DataProvider!
	bufferxface = iface; 

	// Remember we are open
	bIsOpen = true;

	// Enable capture
	Enable();

	// Return success
	return true;
}

// Post the close operation. No new buffers will be filled from now on.
void Field::Close()
{
	bIsOpen = false;

	// If last close operation is pending, we must wait until
	// it completes before we change dataprovider.
	// Wait until all buffers are gone!
	while (dwBufsInProgress) {
		DUMP(("Waiting to Drain buffers on Close...\n"));
		mdelay(40); // 40ms (this is maximum time between fields)
	};

	// Disable field capture
	Disable();
}

/* Note: this is called at IRQL = DISPATCH_LEVEL */
void Field::BufferDone( const FieldDataBuf& pBuf ) 
{ 
	TRACE(("Field::BufferDone"));

	if ( pBuf.RealBuffer() && bufferxface ) {

		ENSURE(dwBufsInProgress);

		// Signal that buffer was complete
		bufferxface->VideoChannelBufferDone( pBuf , GetFieldID() ); 

		// Keep a count on buffers remaining to be completed
		dwBufsInProgress --;

	}
}

// Request a new buffer
/* Note: this is called at IRQL = DISPATCH_LEVEL */
const FieldDataBuf Field::GetNextBuffer( ) 
{ 
	TRACE(("Field::GetNextBuffer"));

	FieldDataBuf vBuf;
	if ( bufferxface && bIsOpen	) { /* don't get buffers if we are waiting to close */

		// Get a new buffer
		vBuf = bufferxface->VideoChannelGetNextBuffer( GetFieldID() ); 

		// If a buffer was given, keep a count on buffers to complete
		if (vBuf.RealBuffer()) {

			// Remember we have at least one buffer to do.
			dwBufsInProgress ++;

		}
	}
	return vBuf; 
}


VideoField::VideoField(BtPisces* p,const VidField field, const DWORD_PTR dwBase,
					   RegBase *ColReg, RegBase *WordSwap, RegBase *ByteSwap ) : 
	Field( p, field ),
	LocalScaler_( p, field , dwBase ), 	
	LocalColSpace_( CF_RGB32, *ColReg, *WordSwap, *ByteSwap )
{
}

const MSize VideoField::GetImageSize() const  // No set, is autocalculated!
{ 
	return GetDigitalWindow().Size(); 
} 

VBIField::VBIField(BtPisces *p, const VidField field , const DWORD_PTR dwBase ) : 
	Field( p, field ),
	VBIPacketSize( 0x00E0 + dwBase, RW), 
	VBI_PKT_LO( VBIPacketSize, 0, 8, RW),
	VBIDelay( 0x00E4 + dwBase, RW), 
	VBI_PKT_HI( VBIDelay, 0, 1, RW), 
	EXT_RAW( VBIDelay, 1, 1, RW), 
	VBI_HDELAY( VBIDelay, 2, 6, RW) ,
	AnalogWin_(0,0,100,10) , // Set to something that makes sense
	DigitalWin_(0,0,100,10)
{
}

const MSize	VBIField::GetImageSize() const 
{
	ULONG f_req = pisces->GetCurrFSCX4() / 1000 ;

	// Get Digital window
	
	// With VBI, digital window denotes something different from expected:
	// Width is given in 10ns (nanoseconds) units from the leading edge of the HSync pulse
	// Height is given in lines, and BOTH are INCLUSIVE


	// Note we will get left = us * 100, right = us * 100
	// top = first line -- inclusive
	// bottom = last line -- inclusive
	MSize size;

	// Height is easy: Just remember that, with VBI lines are inclusive!
	size.cy = DigitalWin_.Height() + 1;

	// width is tricky
	size.cx = DigitalWin_.Width() * f_req / (1000 * 100); // Calculate number of pixels to digitize.

	// Return size;
	return size;
}

ErrorCode VBIField::SetColorFormat(const ColFmt color ) 
{ 
	if ( color == CF_VBI ) return Success; 
	return Fail;
}

ErrorCode VBIField::SetAnalogWindow(const MRect &r ) 
{ 
	AnalogWin_ = r; 
	return Success; 
}

// NTSC : Divide width by 286
//        Divide start by 286
//		  Total time for 400 DWORDS = 55.87us
// PAL  : Divide width by 354
//		  Divide start by 354
//		  Total time for 500 DWORDS = 56.38us
//
// Teletext = 51.89us (data area)

ErrorCode VBIField::SetDigitalWindow(const MRect &r )
{
	DigitalWin_ = r;

	DUMP(( "setting digital window to: %d,%d,%d,%d\n", r.left, r.top, r.right, r.bottom ));

	ULONG f_req = pisces->GetCurrFSCX4() / 1000 ;

	// Get number of DWORDs
	DWORD dwNoOfDWORDs = (r.Width() * f_req / (4000 * 100)); 
	VBI_PKT_LO = (BYTE)dwNoOfDWORDs;
	VBI_PKT_HI = (dwNoOfDWORDs > 0xff) ? 1 : 0; // set the 9th bit

	// Calculate the original hdelay from the trailing HRESET signal
	// The trailing edge of HRESET is assumed to happen 32, 64 or 1 Clkx1 cycles after the 
	// falling edge of the Sync signal in the analog video.
	VBI_HDELAY = r.left * f_req / (1000 * 100);

	DUMP(("Number of DWORDs = %08x, Delay = %d clkx1\n", dwNoOfDWORDs , r.left * f_req / 1000 ));

	return Success;
}

void VBIField::VideoFormatChanged( )
{ 
	// And reset the digital window
	SetDigitalWindow( DigitalWin_ );
}

// Called to enable field capture
void Field::Enable() 
{ 
	pisces->SetFieldEnableState(GetFieldID(),IsVBI(),true); 
}

// Called to disable field capture
void Field::Disable() 
{ 
	pisces->SetFieldEnableState(GetFieldID(),IsVBI(),false); 
}
