/*
	Field.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __FIELD_HPP
#define __FIELD_HPP

#include "debug.hpp"
#include "mytypes.hpp"
#include "Decoder.hpp"
#include "PsColSpc.hpp"
#include "FieldIfc.hpp"
#include "Scaler.hpp"


/* Class: Field
 * Purpose: Encapsulates the operation of a single field. This could be a Video or a VBI field
 */

// Forward declarations
class BtPisces;

class Field
{
	private:

		// Field Identificator (Even or Odd)
		const VidField			FieldID_;			

		// Count of Buffers in process of being filled
		volatile DWORD			dwBufsInProgress;

		// Data buffer provider and receptor (videoChannel)
		FieldBufferXface*		bufferxface;	// Pointer to the video buffer provider and receptor for this instance

		// If Field is in use or not - Does NOT disable field - Is it is true, field is
		// open, if it is false, either there is a pending close or it is closed.
		bool					bIsOpen;

	protected:

		// The pointer to the BtPisces object that can enable/disable fields
		BtPisces*				pisces;

	public:

		Field( BtPisces* p,const VidField );
		virtual ~Field();

		// Functions to ensure smooth sharing of fields
		bool					Open(FieldBufferXface* iface );
		bool					IsOpen() const { return ( bIsOpen); }
		void					Close();
		
		// Field ID (Even or Odd)
	   	VidField				GetFieldID() const { return FieldID_; }
		virtual bool			IsVBI() const = 0;
		bool					IsVideo() const { return !IsVBI(); }

		// Set/Get Field attributes: Be careful: This shouldn't be changed while we are streaming!
		virtual const MSize		GetImageSize() const = 0;
		virtual ErrorCode		SetColorFormat( const ColFmt aColor ) = 0;
		virtual ColFmt			GetColorFormat() const = 0;
		virtual DWORD			GetBitCount() const = 0;

		virtual ErrorCode		SetAnalogWindow( const MRect &r ) = 0;
		virtual const MRect&	GetAnalogWindow( ) const = 0;

		virtual ErrorCode		SetDigitalWindow( const MRect &r ) = 0;
		virtual const MRect&	GetDigitalWindow( ) const = 0;

		virtual void			VideoFormatChanged( ) = 0;

		// Send notification of buffer completion to the VideoChannel
		void					BufferDone(const FieldDataBuf& pBuf);

		// Request a new buffer from our VideoChannel
		const FieldDataBuf		GetNextBuffer();

		// Called to enable field capture
		void					Enable();

		// Called to disable field capture
		void					Disable();

};


/* Class: VideoField
 * Purpose: A Video Field , with scaling capability.
 * Attributes:
 * Operations:
 */
class VideoField : public Field
{
	private:

		// Private colorspace for this field
		PsColorSpace		LocalColSpace_;	// Colorspace for this field

		// Local Scaler
   		Scaler				LocalScaler_;	// Local Scaler

	public:

		VideoField( BtPisces* p, const VidField field, const DWORD_PTR dwBase,
			RegBase *ColReg, RegBase *WordSwap, RegBase *ByteSwap );
		virtual bool		IsVBI() const {return false;}

		// Set/Get Field attributes: Be careful: This must not be changed while we are streaming!
		virtual const MSize	GetImageSize() const ;
		virtual ErrorCode	SetColorFormat( const ColFmt aColor ) { LocalColSpace_.SetColorFormat( aColor ); return Success; }
		virtual ColFmt		GetColorFormat() const { return LocalColSpace_.GetColorFormat(); }
		virtual DWORD		GetBitCount() const { return LocalColSpace_.GetBitCount(); }

		virtual ErrorCode	 SetAnalogWindow( const MRect &r ) { return LocalScaler_.SetAnalogWin( r ); }
		virtual const MRect& GetAnalogWindow( ) const { return LocalScaler_.GetAnalogWin(); }

		virtual ErrorCode	 SetDigitalWindow( const MRect &r ) { return LocalScaler_.SetDigitalWin( r ); }
		virtual const MRect& GetDigitalWindow( ) const { return LocalScaler_.GetDigitalWin(); }

		virtual void		VideoFormatChanged( ) {	LocalScaler_.VideoFormatChanged( ); }
		
		// Only a VideoField can have this!
		void				TurnVFilter( const State s ) { LocalScaler_.TurnVFilter( s ); }
		ErrorCode			SetSharpness(const ULONG Val) { return LocalScaler_.SetSharpness(Val); }
		ULONG				GetSharpness(void) const { return LocalScaler_.GetSharpness(); }
		ErrorCode			SetAutoSharpness(const bool Auto) {  return LocalScaler_.SetAutoSharpness(Auto); }
		bool				GetAutoSharpness() const { return LocalScaler_.GetAutoSharpness(); }

};

/* Class: VBIField
 * Purpose: A VBI Field
 * Attributes:
 * Operations:
 */

class VBIField : public Field
{
	private:

		RegisterDW	VBIPacketSize;
		RegField	VBI_PKT_LO;
		RegisterDW	VBIDelay;
		RegField	VBI_PKT_HI;
		RegField	EXT_RAW;
		RegField	VBI_HDELAY;

		// We must remember those windows!
		MRect AnalogWin_;
		MRect DigitalWin_;

	public:
		VBIField(BtPisces* p,const VidField field ,const DWORD_PTR dwBase) ;

		virtual bool		IsVBI() const {return true;}

		// Set/Get Field attributes: Be careful: This must not be changed while we are streaming!
		virtual const MSize	GetImageSize() const ;
		virtual ErrorCode	SetColorFormat(const  ColFmt );
		virtual ColFmt		GetColorFormat() const { return CF_VBI; };
		virtual DWORD		GetBitCount() const { return ColorSpace( CF_VBI ).GetBitCount(); }

		virtual ErrorCode	 SetAnalogWindow(const MRect &r );
		virtual const MRect& GetAnalogWindow() const { return AnalogWin_; }

		virtual ErrorCode	 SetDigitalWindow(const MRect &r );
		virtual const MRect& GetDigitalWindow() const { return DigitalWin_; }

		virtual void		 VideoFormatChanged( );

		~VBIField() {}
};


#endif
