/*
	GPIO.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Debug.hpp"
#include "gpio.hpp"
#include "Pisces.hpp"
#include "Register.hpp"
#include "RegField.hpp"
#include "..\BtCard.hpp"

/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////
GPIO::GPIO( 
	BtPisces* p , const DWORD_PTR dwAddress ) :

	// construct all GPIO related register and register fields
	decRegGPIO ( dwAddress + 0x10C, RW ),               // GPIO register
	decFieldGPCLKMODE( decRegGPIO, 10, 1, RW ),
	decFieldGPIOMODE( decRegGPIO, 11, 2, RW ),
	decFieldGPWEC( decRegGPIO, 13, 1, RW ),
	decFieldGPINTI( decRegGPIO, 14, 1, RW ),
	decFieldGPINTC( decRegGPIO, 15, 1, RW ),
	decRegGPOE( dwAddress + 0x118, RW ),    // GPOE register
	decRegGPIE( dwAddress + 0x11C, RW ),    // GPIE register
	decRegGPDATA( dwAddress + 0x200, RW ),  // GPDATA register
	MAX_GPDATA_SIZE(64),					// maximum number of DWORDs in GPDATA
	MAX_GPIO_BIT(23),						// maximum bit number for GPIO registers
	pisces(p) ,								// Digitizer object
	dwGPDATAAddress( dwAddress + 0x200 ) ,	// Address of GPDATA
	icbCallback(NULL)						// No callback by default
{
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
GPIO::~GPIO()
{
}

// This fn will register a GPIO int callback (or will unregister it)
void GPIO::RegisterIntCallback(IntCallback* cb)
{
	// Disable int before unsetting a callback
	if (!cb) {
		pisces->DisableUserInt(GPINT_I);
	}
	// Set the callback
	icbCallback = cb;
	// And enable it if setting a new callback
	if (cb) {
		pisces->EnableUserInt(GPINT_I);
	}
}

// This is called from Pisces when a GPIO related int was received
// WARNING:  this is called at IRQL = DISPATCH_LEVEL */
void GPIO::IntReceived()
{
	// If someone registered the int, call it!
	if (icbCallback) {
		icbCallback->cbInt();
	}
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPOE( int bit, State s )
// Purpose: Set or clear a bit in GPOE
// Input:   int bit - bit to be set or clear
//          State s - On to set; Off to clear
// Output:  None
// Return:  true if Success
/////////////////////////////////////////////////////////////////////////////
bool GPIO::SetGPOE( int bit, State s )
{
	// range checking
	if ( ( bit < 0 ) || ( bit > MAX_GPIO_BIT ) )
		return ( false );

	RegField decFieldTemp( decRegGPOE, (BYTE)bit, 1, RW );   // create a reg field
	decFieldTemp = s;

	return ( true );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPOE( int bit )
// Purpose: Get value of a bit in GPOE
// Input:   int bit - bit to get value from
// Output:  None
// Return:  int - On (1), Off (0), or -1 for parameter error
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPOE( int bit )
{
	// range checking
	if ( ( bit < 0 ) || ( bit > MAX_GPIO_BIT ) )
		return ( -1 );

	RegField decFieldTemp( decRegGPOE, (BYTE)bit, 1, RW );   // create a reg field
	return( decFieldTemp );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPIE( int bit, State s )
// Purpose: Set or clear a bit in GPIE
// Input:   int bit - bit to be set or clear
//          State s - On to set; Off to clear
// Output:  None
// Return:  true if Success ,else Fail
/////////////////////////////////////////////////////////////////////////////
bool GPIO::SetGPIE( int bit, State s )
{
	// GPIE is deleted in Bt878 family
	if ( pisces->IsBt878Family() )
      return ( false );

	// range checking
	if ( ( bit < 0 ) || ( bit > MAX_GPIO_BIT ) )
		return ( false );

	RegField decFieldTemp( decRegGPIE, (BYTE)bit, 1, RW );   // create a reg field
	decFieldTemp = s;

	return ( true );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPIE( DWORD value )
// Purpose: Set GPIE value
// Input:   DWORD value - value to be set to
// Output:  None
// Return:  true if success
/////////////////////////////////////////////////////////////////////////////
bool GPIO::SetGPIE( DWORD value )
{
	// GPIE is deleted in Bt878 family
	if ( pisces->IsBt878Family() )
		return ( false );

	decRegGPIE = ( value & 0x00FFFFFFL );     // bits [23:0]
	return true;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPIE( int bit )
// Purpose: Get value of a bit in GPIE
// Input:   int bit - bit to get value from
// Output:  None
// Return:  int - On (1), Off (0), or -1 for parameter error
/////////////////////////////////////////////////////////////////////////////
int GPIO::GetGPIE( int bit )
{
	// GPIE is deleted in Bt878 family
	if ( pisces->IsBt878Family() )
		return ( -1 );

	// range checking
	if ( ( bit < 0 ) || ( bit > MAX_GPIO_BIT ) )
		return ( -1 );

	RegField decFieldTemp( decRegGPIE, (BYTE)bit, 1, RW );   // create a reg field
	return( decFieldTemp );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  DWORD GPIO::GetGPIE( void )
// Purpose: Get value of GPIE
// Input:   None
// Output:  None
// Return:  Content of GPIE (DWORD)
/////////////////////////////////////////////////////////////////////////////
DWORD GPIO::GetGPIE( void )
{
	// GPIE is deleted in Bt878 family
	if ( pisces->IsBt878Family() )
		return ( ~0 );

	return ( decRegGPIE & 0x00FFFFFFL );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPDATA( GPIOReg * data, int size, int offset )
// Purpose: Set GPDATA registers contents
// Input:   GPIOReg * data - ptr to data to be copied from
//          int size       - number of registers to be copied; max is 64
//          int offset     - how many registers to skip (default = 0)
// Output:  None
// Return:  true if Success , else Fail
/////////////////////////////////////////////////////////////////////////////
bool GPIO::SetGPDATA( ULONG * data, int size, int offset )
{
	// check size and offset for out of bound
	if ( ( offset < 0 ) || ( size < 0 ) || ( size + offset > MAX_GPDATA_SIZE ) )
		return ( false );

	// Pass data to GPDATA
	for (int idx = 0; idx < size; idx++ ) {
		WRITE_REGISTER_ULONG( PULONG(dwGPDATAAddress) + (offset + idx) , data[idx] );
	}

	return ( true );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::GetGPDATA( GPIOReg * data, int size, int offset )
// Purpose: Get GPDATA registers contents
// Input:   GPIOReg * data - ptr to data to be copied to
//          int size       - number of registers to be copied; max is 64
//          int offset     - how many registers to skip (default = 0)
// Output:  None
// Return:  true if Success or Fail
/////////////////////////////////////////////////////////////////////////////
bool GPIO::GetGPDATA( ULONG * data, int size, int offset )
{
	// check size and offset for out of bound
	if ( ( offset < 0 ) || ( size < 0 ) || ( size + offset > MAX_GPDATA_SIZE ) )
		return ( false );

	// Read data from GPDATA
	for (int idx = 0; idx < size; idx++ ) {
		data[idx] = READ_REGISTER_ULONG( PULONG(dwGPDATAAddress) + (offset + idx) );
	}
	return ( true );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::SetGPDATA( int fromBit, int toBit,
//                                     DWORD value, int offset )
// Purpose: Set GPDATA contents with specified range of bits
// Input:   int fromBit - starting bit
//          int toBit   - ending bit
//          DWORD value - value to set to
//          int offset  - how many registers to skip (default = 0)
// Output:  None
// Return:  true if Success or Fail
// Comment: By specifying a range of bits to be set, it allows a portion of
//          the GDATA register to be modified. For example,
//             SetGPDATA( 8, 10, 5, 0 )
//          would set the first GPDATA register's [10:8] to value 0x101
/////////////////////////////////////////////////////////////////////////////
bool GPIO::SetGPDATA( int fromBit, int toBit, DWORD value, int offset )
{
	// check size and offset for out of bound
	if ( ( fromBit < 0 ) || ( fromBit > MAX_GPIO_BIT ) ||
		( toBit   < 0 ) || ( toBit   > MAX_GPIO_BIT ) || ( fromBit > toBit ) ||
		( offset  < 0 ) || ( offset  > MAX_GPDATA_SIZE ) )
      return ( false );

	// make sure value can "fit" into range of bits specified
	if ( value >= (DWORD) ( 0x00000001L << ( toBit - fromBit + 1 ) ) )
		return ( false );

	RegisterDW reg( dwGPDATAAddress + offset * sizeof( ULONG ), RW );
	RegField field( reg, (BYTE)fromBit, (BYTE)(toBit - fromBit + 1), RW );
	field = value;

	return ( true );
}


/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode GPIO::GetGPDATA( int fromBit, int toBit,
//                                     DWORD * value, int offset )
// Purpose: Get GPDATA contents with specified range of bits
// Input:   int fromBit - starting bit
//          int toBit   - ending bit
//          int offset  - how many registers to skip (default = 0)
// Output:  DWORD * value - value retrieved
// Return:  true if Success or Fail
// Comment: By specifying a range of bits to be set, it allows a portion of
//          the GDATA register to be retrieved. For example,
//             GetGPDATA( 8, 10, &data, 0 )
//          would set *data to the first GPDATA register's [10:8] content
/////////////////////////////////////////////////////////////////////////////
bool GPIO::GetGPDATA( int fromBit, int toBit, DWORD * value, int offset )
{
	// check size and offset for out of bound
	if ( ( fromBit < 0 ) || ( fromBit > MAX_GPIO_BIT ) ||
		( toBit   < 0 ) || ( toBit   > MAX_GPIO_BIT ) || ( fromBit > toBit ) ||
        ( offset  < 0 ) || ( offset  > MAX_GPDATA_SIZE ) )
		return ( false );

	RegisterDW reg( dwGPDATAAddress + offset * sizeof( ULONG ), RW );
	RegField field( reg, (BYTE)fromBit, (BYTE)(toBit - fromBit + 1), RW );
	*value = field;
	return ( true );
}


