/*
	GPIO.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef __GPIO_HPP
#define __GPIO_HPP

#include "regField.hpp"
#include "Decoder.hpp"
#include "IntCallback.hpp"

// Forward decls
class BtPisces;

/////////////////////////////////////////////////////////////////////////////
// CLASS GPIO
//
// Description:
//    This class encapsulates the register fields in the GPIO register of the
//    Bt848. A complete set of functions are developed to manipulate all the
//    register fields in the GPIO for the Bt848.
//
/////////////////////////////////////////////////////////////////////////////

class GPIO {

public:

	//===========================================================================
	// GPIO modes
	//===========================================================================
	typedef enum { GPIO_NORMAL,
               GPIO_SPI_OUTPUT,
               GPIO_SPI_INPUT,
               GPIO_DEBUG_TEST } GPIOMode;

private:

	//===========================================================================
	// Constants
	//===========================================================================
	const int MAX_GPDATA_SIZE /*= 64*/;		// maximum number of DWORDs in GPDATA
	const int MAX_GPIO_BIT    /*= 23*/;		// maximum bit number for GPIO registers


	BtPisces *		pisces;					// Pointer to the digitizer object
	DWORD_PTR		dwGPDATAAddress;		// Address of DATA register in Bt848
	IntCallback*	icbCallback;			// Pointer to the object calback to call when an int arrives

public:
	//  Constructor
	GPIO(BtPisces* p, const DWORD_PTR dwAddress );
	~GPIO();

	// Member functions
	void      SetGPCLKMODE( State );
	int       GetGPCLKMODE( void );
	void      SetGPIOMODE( GPIOMode );
	int       GetGPIOMODE( void );
	void      SetGPWEC( State );
	int       GetGPWEC( void );
	void      SetGPINTI( State );
	int       GetGPINTI( void );
	void      SetGPINTC( State );
	int       GetGPINTC( void );
	bool	  SetGPOE( int, State );
	void      SetGPOE( DWORD );
	int       GetGPOE( int );
	DWORD     GetGPOE( void );
	bool      SetGPIE( int, State );
	bool      SetGPIE( DWORD );
	int       GetGPIE( int );
	DWORD     GetGPIE( void );
	bool      SetGPDATA( ULONG *, int, int offset = 0 );
	bool      GetGPDATA( ULONG *, int, int offset = 0 );
	bool      SetGPDATA( int, int, DWORD, int offset = 0 );
	bool      GetGPDATA( int, int, DWORD *, int offset = 0 );

	// This fn will register a GPIO int callback (or will unregister it)
	// MUST be called when video is not streaming (initialization time)
	void	  RegisterIntCallback(IntCallback* cb = NULL);

	// Callback to signal from ISR GPIO int was received.
	/* Note: this is called at IRQL = DISPATCH_LEVEL */
	void	  IntReceived();

protected:

	RegisterW decRegGPIO;
	RegField decFieldGPCLKMODE;
	RegField decFieldGPIOMODE;
	RegField decFieldGPWEC;
	RegField decFieldGPINTI;
	RegField decFieldGPINTC;
	RegisterDW decRegGPOE;
	RegisterDW decRegGPIE;
	RegisterDW decRegGPDATA;
	 
};

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPCLKMODE( State s )
// Purpose: Set or clear GPCLKMODE
// Input:   State s - On to set; Off to clear
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void GPIO::SetGPCLKMODE( State s )
{
	decFieldGPCLKMODE = s;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPCLKMODE( void )
// Purpose: Get value of GPCLKMODE
// Input:   None
// Output:  None
// Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
inline int GPIO::GetGPCLKMODE( void )
{
	return ( decFieldGPCLKMODE );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPIOMODE( GPIOMode mode )
// Purpose: Set GPIO mode
// Input:   GPIOMode mode - GPIO_NORMAL, GPIO_SPI_OUTPUT, GPIO_SPI_INPUT,
//                          GPIO_DEBUG_TEST
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void GPIO::SetGPIOMODE( GPIOMode mode )
{
	decFieldGPIOMODE = mode;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPIOMODE( void )
// Purpose: Get GPIO mode
// Input:   None
// Output:  None
// Input:   int - GPIO_NORMAL, GPIO_SPI_OUTPUT, GPIO_SPI_INPUT, GPIO_DEBUG_TEST
/////////////////////////////////////////////////////////////////////////////
inline int GPIO::GetGPIOMODE( void )
{
	return( decFieldGPIOMODE );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPWEC( State s )
// Purpose: Set or clear GPWEC
// Input:   State s - On to set; Off to clear
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void GPIO::SetGPWEC( State s )
{
	decFieldGPWEC = s;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPWEC( void )
// Purpose: Get value of GPWEC
// Input:   None
// Output:  None
// Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
inline int GPIO::GetGPWEC( void )
{
	return ( decFieldGPWEC );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPINTI( State s )
// Purpose: Set or clear GPINTI
// Input:   State s - On to set; Off to clear
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void GPIO::SetGPINTI( State s )
{
	decFieldGPINTI = s;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPINTI( void )
// Purpose: Get value of GPINTI
// Input:   None
// Output:  None
// Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
inline int GPIO::GetGPINTI( void )
{
	return ( decFieldGPINTI );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPINTC( State s )
// Purpose: Set or clear GPINTC
// Input:   State s - On to set; Off to clear
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void GPIO::SetGPINTC( State s )
{
	decFieldGPINTC = s;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int GPIO::GetGPINTC( void )
// Purpose: Get value of GPINTC
// Input:   None
// Output:  None
// Return:  int - On (1) or Off (0)
/////////////////////////////////////////////////////////////////////////////
inline int GPIO::GetGPINTC( void )
{
	return ( decFieldGPINTC );
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void GPIO::SetGPOE( DWORD value )
// Purpose: Set GPOE value
// Input:   DWORD value - value to be set to
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void GPIO::SetGPOE( DWORD value )
{
	decRegGPOE = ( value & 0x00FFFFFFL );     // bits [23:0]
}

/////////////////////////////////////////////////////////////////////////////
// Method:  DWORD GPIO::GetGPOE( void )
// Purpose: Get value of GPOE
// Input:   None
// Output:  None
// Return:  Content of GPOE (DWORD)
/////////////////////////////////////////////////////////////////////////////
inline DWORD GPIO::GetGPOE( void )
{
	return ( decRegGPOE & 0x00FFFFFFL );
}

#endif  // __GPIO_H

