/*
	I2C.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
	Note:
		I2C Hardware multiread/write for the Bt878 is disabled. Seems NOT to be working
		Any ideas ???
 */

/////////////////////////////////////////////////////////////////////////////
//
//  Module:
//
//    I2C.cpp
//
//  Abstract:
//
//    Implementation of I2C register
//
//  Note:
//    Due to certain behavior and operation requirements of the I2C
//    (listed below), an I2C register structure is created and acts as a temp
//    storage for I2C.
//
//    Behavior of I2C:
//       1) When modifying more than 1 field of I2C, must set the content of the
//          whole register.
//       2) Be very careful of the register type; RO, RW & RR.
//       3) Since changing channel requires writing to I2C and read back the
//          status from I2C, care must be taken not to modify any other bits
//          except the ones supposed to be altered.
//
//    ***** IMPORTANT NOTE for software control of I2C: *****
//       1) There is a bug somewhere in Register class and a problem exists
//          when try to read from the SDA line. The problem is after sending
//          the tuner address for read operation, software set SDA to high
//          so that slave can pull it low for ACK signal. Even using a logic
//          analyser, we never see SDA goes high due to the fact that slave
//          probably have pull it low already. That's ok. Next step we do is
//          a read SDA to make sure slave did ACK (by checking SDA=0). Then
//          we generate a clock pulse to let slave to release pulling the SDA
//          line low. As soon as slave releases SDA, we should see SDA goes
//          back to high (since we wrote 1 to the register field SDA before).
//          We never see SDA goes high thus we are unable to read valid bits
//          from the slave.
//          The fix for it for right now is to use pointer to I2C register and
//          set SDA and SCL directly to 1 or 0 without going through register
//          class. In the future, register class should be used again after
//          finding out what the problem is.
//       2) Everytime we releases the SCL line (i.e. let it go high), we need
//          to check and make sure SCL really goes high before we continue
//          because the slave could pull it low for whatever reason
//          (for example, slave is not ready) and releases it later. What to
//          do is to have a timeout loop and check if SCL really goes high.
//
/////////////////////////////////////////////////////////////////////////////

//===========================================================================
// I2C DATA/CONTROL REGISTER API
//===========================================================================
			
#include "debug.hpp"
#include "I2C.hpp"	  
#include "Pisces.hpp"
#include "..\BtCard.hpp"

//**************************************************************************
//	DEFINES
//**************************************************************************
#define I2C_WRITE         0         /* write operation */
#define I2C_READ          1         /* read operation */

#define PCI_FREQ          33000000L /* frequency for PCI in hz */
#define I2CDIV_MAX        15        /* I2C maximum divider */

#define I2C_TIMEOUT		  500		/* Timeout in ms to wait for slave bus release ( wait state insert ) */

//===========================================================================
// Bt848 I2C Class Implementation
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////
I2C::I2C(BtPisces * p, const DWORD_PTR dwAddress ) :
	// construct I2C register and register fields
	decRegINT_STAT ( dwAddress + 0x100, RW ),                // Interrupt Status register
	decFieldI2CDONE( decRegINT_STAT, 8, 1, RR ),
	decFieldRACK( decRegINT_STAT, 25, 1, RO ),
	decRegI2C ( dwAddress + 0x110, RW ),                     // I2C Data/Control register
	decFieldI2CDB0( decRegI2C, 24, 8, RW ),
	decFieldI2CDB1( decRegI2C, 16, 8, RW ),
	decFieldI2CDB2( decRegI2C,  8, 8, RW ),
	decFieldI2CDIV( decRegI2C,  4, 4, RW),
	decFieldSYNC( decRegI2C, 3, 1, RW),
	decFieldW3B( decRegI2C, 2, 1, RW),
	decFieldSCL( decRegI2C, 1, 1, RW),
	decFieldSDA( decRegI2C, 0, 1, RW),
	decFieldI2CMODE( decRegI2C, 7, 1, RW),   // new for Gemini
	decFieldI2CRATE( decRegI2C, 6, 1, RW),
	decFieldI2CNOSTOP( decRegI2C, 5, 1, RW),
	decFieldI2CNOS1B( decRegI2C, 4, 1, RW),
	decFieldI2CW3BRA( decRegI2C, 2, 1, RW) ,
	GotI2CInt(false),
	swbitdelayus(1L),
	hwtimeoutms(I2C_TIMEOUT),
	currFreq(100000), // Default to 100Khz
	errNum(ERR_OK),
	mode(Mode_None),
	pisces(p)
{
	// Shouldn't use pointer here or fns that use the pointer:
	// ** BtPisces not completely initialized! ***
	
	// Gemini: points to same location as in old I2C
	pGeminiShadow = (_Gemini_reg *)&Shadow;

	// initialize I2C register shadow
	ResetShadow();
	
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
I2C::~I2C()
{
}


/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::InitHWMode( long freq )
//  Purpose: Initialize I2C for hardware control of SCL and SDA
//  Input:   long freq - frequency (hz) to run SCL at
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
bool I2C::InitHWMode( long freq )
{
	// If already in the requested mode, exit!
	if ( mode == Mode_HW) {
		return true;
	}

	// If no freq. given use default freq.
	if (freq == 0) {
		freq = 100000;
	} 

	// initialize I2C register shadow
	ResetShadow();

	decFieldSCL = Shadow.scl = 1;   // must be 1 for hardware mode
	decFieldSDA = Shadow.sda = 1;   // must be 1 for hardware mode

	SetHWFreq( freq );   // set frequency for hardware control

	// I2C is running hardware mode
	mode = Mode_HW;

	// Assume no int received right now...
	GotI2CInt = false;

	return true;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  void I2C::SetHWFreq( long freq )
//  Purpose: Set frequency for hardware SCL
//  Input:   long freq - frequency (hz) to run SCL at. (137.5khz to 2.0625Mhz)
//             PCI frequency 33Mhz: SCL = (412.50Khz to 33.81Khz)
//                           25Mhz: SCL = (312.50Khz to 25.61Khz)
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
void I2C::SetHWFreq( long freq )
{
	
	if (freq > 1) {
		if ( pisces->IsBt878Family() ) {
			pGeminiShadow->mode = 1;     // hardware mode
			pGeminiShadow->rate = ( freq == 400000 ) ? 1 : 0;
			pGeminiShadow->nos1b = pGeminiShadow->nostop = 0;
		} else {
			unsigned int i2cdiv;
			i2cdiv = (unsigned int) (PCI_FREQ / (64 * freq));
			if ( i2cdiv > I2CDIV_MAX )
				i2cdiv = I2CDIV_MAX;
			Shadow.div = i2cdiv;
		}

		// Calculate the hardware timeout to allow in ms
		hwtimeoutms = 1000 * 10 * 3 * 3 / freq; // assumes 10 bits per byte, 3 bytes to allow. 3 times to transmit

		// At least, allow 1 ms
		if (!hwtimeoutms) hwtimeoutms = 1;

		// If we allow slave to insert wait states
		if (Shadow.sync) {
			// Pick the longest one.
			if (hwtimeoutms < I2C_TIMEOUT) {
				// To allow slave to insert wait states...
				hwtimeoutms = I2C_TIMEOUT;
			}
		}

	} else {
		Shadow.div = 0; // This forces software mode
	}

	decFieldI2CDIV = Shadow.div;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  void I2C::ResetShadow( void )
//  Purpose: Reset register shadow
//  Input:   int maxWait - maximum waiting time in milliseconds
//  Output:  None
//  Return:  Success if done; Fail if timeout
/////////////////////////////////////////////////////////////////////////////
void I2C::ResetShadow( void )
{
	// initialize I2C register shadow
	Shadow.addr_rw = 0;
	Shadow.byte1   = 0;
	Shadow.byte2   = 0;
	Shadow.div     = 0;
	Shadow.sync    = 1; // Allow insertion of waits
	Shadow.w3b     = 0;
	Shadow.scl     = 1; // both must be released
	Shadow.sda     = 1; // by default
}
   
/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::HWWaitUntilDone( )
//  Purpose: Wait for hardware I2C to finish
//  Input:   None
//  Output:  None
//  Return:  Success if done; Fail if timeout
/////////////////////////////////////////////////////////////////////////////
bool I2C::HWWaitUntilDone( void )
{
	TRACE(("I2C::HWWaitUntilDone"));	

	DWORD startTime = GetTickCountInMs();

	// loop until either I2CDONE is set or timeout
	do {

		// Wait for I2C to send the byte using interrupts - But use a timeout 
		// so we won't lock if int does not arrive
		eI2CDone.WaitFor( hwtimeoutms );

		if ( GotI2CInt ) {

			// Prepare for next call...
			GotI2CInt = false;

			return true;
		}

		// timeout?
	} while ( GetTickCountInMs() - startTime < (DWORD)hwtimeoutms );

	// Prepare for next call...
	GotI2CInt = false;

	errNum = ERR_TIMEOUT;
	return false;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::HWIsDone( void )
//  Purpose: Determine if I2C has finished a read or write operation by
//           checking the I2CDONE bit in the interrupt status register
//  Input:   None
//  Output:  None
//  Return:  true if done; else false
/////////////////////////////////////////////////////////////////////////////
bool I2C::HWIsDone( void )
{
	if ( decFieldI2CDONE != 0 ) {
		// Need to clear the bit when it is set; don't want to alter any other bits
		// Writing a 1 to the bit clears it.
		decFieldI2CDONE = 1;
		return true;
	}
	return false;
}


/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::InitSWMode( long freq )
//  Purpose: Initialize I2C for software control of SCL and SDA
//  Input:   long freq - frequency (hz) to run SCL at
//  Output:  None
//  Return:  None
//  Note:    After calling I2CIsInitOK(), application should call this
//           function and check for return value is Success before starting
//           software communication.
/////////////////////////////////////////////////////////////////////////////
bool I2C::InitSWMode( long freq )
{
	// If already in the requested mode, exit!
	if ( mode == Mode_SW) {
		return true;
	}

	// initialize I2C register shadow
	ResetShadow();

	decFieldSCL = Shadow.scl = 1;   // must be 1 for soft mode
	decFieldSDA = Shadow.sda = 1;   // must be 1 for soft mode

	SetHWFreq( 0 );					// set frequency to 0 for software control

	// Calculate the required time
	if (freq) {
		swbitdelayus = 1000000 / (freq * 2);
	}
	else {
		swbitdelayus = 1000000 / (100000 * 2);
	}
      
	// I2C is running software mode
	mode = Mode_SW;

	// Reinit bus
	SWStart();
	SWStop();

	// Prepare for next call...
	GotI2CInt = false;

	return true;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::SWStart( void )
//  Purpose: Generate START condition using software control
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWStart( void )
{
	// SendStart - send an I2c start
	// i.e. SDA 1 -> 0 with SCL = 1

	// Both lines should be already HIGH (=Bus not BUSY)
	if ( !SWSetSDA( LevelHi )  ) { return false; }
	if ( !SWSetSCL( LevelHi )  ) { return false; }
	if ( !SWSetSDA( LevelLow ) ) { return false; }
	if ( !SWSetSCL( LevelLow ) ) { return false; }
	// Both lines should be now LOW

	return true;
}                  

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::SWStop( void )
//  Purpose: Generate STOP condition using software control
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWStop( void )
{
	// SendStop - sends an I2C stop, releasing the bus.
	// i.e. SDA 0 -> 1 with SCL = 1

	// SCL should be now LOW
	if ( !SWSetSCL( LevelLow ) ) { return false; }
	if ( !SWSetSDA( LevelLow ) ) { return false; }
	if ( !SWSetSCL( LevelHi )  ) { return false; }
	if ( !SWSetSDA( LevelHi )  ) { return false; }
	// Both lines should be now HIGH (=Bus not BUSY)

	return true;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::SWRead( BYTE * value )
//  Purpose: Read a byte from the slave
//  Input:   None
//  Output:  BYTE * value - byte read from slave
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWRead( BYTE * value )
{
	*value = 0x00;   

	// SCL line should be now LOW

	// read 8 bits from I2c into Accumulator
	for( BYTE mask = 0x80; mask > 0; mask = (BYTE)( mask >> 1 ) ) {
		//--> Here data is output by the slave 
		if ( !SWSetSCL( LevelLow ) ) { return false; }
		if ( !SWSetSCL( LevelHi )  ) { return false; } 
		//--> Data from slave should be stable here
		if ( SWReadSDA() == 1)
			*value = (BYTE)( *value | mask ); // set the bit
	}
	// SCL line is now LOW
	return true;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::SWWrite( BYTE value )
//  Purpose: Write a byte to the slave
//  Input:   BYTE value - byte to be written to slave
//  Output:  None
//  Return:  Success or Fail (maybe NOACK)
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWWrite( BYTE value )
{
	// Both lines should be now LOW

	// generate bit patterns by setting SCL and SDA lines
	for ( BYTE mask = 0x80; mask > 0; mask = (BYTE)(mask >> 1) ) {                                          
		// Send one data bit.
		if ( !SWSetSCL( LevelLow ) ) { return false; }
		//Put data bit on pin.
		if ( !SWSetSDA( ( value & mask ) ? LevelHi : LevelLow ) ) { return false; }
     
		if ( !SWSetSCL( LevelHi ) ) { return false; }
	}

	return SWWaitForACK(); // Can fail with NOACK
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::SWSendACK( void )
//  Purpose: Send ACK to slave
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWSendACK( void )
{
  
   // Generate ACK signal
   // i.e. SDA = 0 with SCL = 1

   if ( !SWSetSCL( LevelLow ) ) { return false; }
   if ( !SWSetSDA( LevelLow ) ) { return false; }
   if ( !SWSetSCL( LevelHi )  ) { return false; }
   if ( !SWSetSCL( LevelLow ) ) { return false; }
   if ( !SWSetSDA( LevelHi )  ) { return false; }
   return true;
}                  

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::SWSendNACK( void )
//  Purpose: Send NACK to slave
//  Input:   None
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWSendNACK( void )
{
   // Generate NACK signal
   // i.e. SDA = 1 with SCL = 1

   if ( !SWSetSCL( LevelLow ) ) { return false; }
   if ( !SWSetSDA( LevelHi )  ) { return false; }
   if ( !SWSetSCL( LevelHi )  ) { return false; }
   if ( !SWSetSCL( LevelLow ) ) { return false; }
   if ( !SWSetSDA( LevelLow ) ) { return false; }
   return true;
}                  

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::SWSetSCL( Level scl )
//  Purpose: Set software SCL value
//  Input:   Level scl - Hi releases the SCL output; Low forces the SCL output low
//  Output:  None
//  Return:  Success or Fail (ERR_SCL)
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWSetSCL( Level scl )
{
	Shadow.scl = scl;
	decRegI2C = *(DWORD *)&Shadow;

	// loop until SCL really changes or timeout (so we will allow slave to insert waits)
	DWORD maxWait = I2C_TIMEOUT; // Timeout in mSec
	DWORD startTime = GetTickCountInMs();

	while (1) {
		// has SCL changed yet?
		if ( SWReadSCL() == scl )
			break;

		// timeout?
		DWORD elapsed = GetTickCountInMs() - startTime;
		if ( elapsed > (DWORD)maxWait ) {
			errNum = ERR_SCL;
			return false;
		}

		// If more than 10 ms of busywait, give windows a chance
		if (elapsed > 10) {
			mdelay(1);
		}
	}

	SWBitDelay();
	return true;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::SWSetSDA( Level sda )
//  Purpose: Set software SDA value
//  Input:   Level sda - Hi releases the SDA output; Low forces the SDA output low
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWSetSDA( Level sda )
{
	Shadow.sda = sda;
	decRegI2C = *(DWORD *)&Shadow;

	SWBitDelay();
	return true;
}


/////////////////////////////////////////////////////////////////////////////
//  Method:  ErrorCode I2C::SetSync( State sync )
//  Purpose: Set I2C sync value
//  Input:   sync: On  - allow slave to insert wait states
//                 Off - slave cannot insert wait states
//  Output:  None
//  Return:  Success or Fail
/////////////////////////////////////////////////////////////////////////////
bool I2C::SetSync( State sync )
{
   // check if correct mode is selected
	if ( ( mode != Mode_HW ) && ( mode != Mode_SW ) )
	{
		errNum = ERR_MODE;
		return false;
	}

	Shadow.sync = sync;
	decRegI2C = *(DWORD *)&Shadow;
	return true;
}


/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::SWWaitForACK( void )
//  Purpose: Determine if ACK is received in software mode
//  Input:   None
//  Output:  None
//  Return:  Success if ACK received; Fail if timeout (no ack)
/////////////////////////////////////////////////////////////////////////////
bool I2C::SWWaitForACK( void )
{
	TRACE(("I2C::SWWaitForACK"));

	if ( !SWSetSCL( LevelLow ) ) { return false; }
	if ( !SWSetSDA( LevelHi )  ) { return false; }

	// loop until either ACK or timeout
	DWORD maxWait = (swbitdelayus * 3 / 1000);
	if (!maxWait) maxWait = 1;

	DWORD startTime = GetTickCountInMs();

	while (1) {

		// SDA pin == 0 means the slave ACK'd
		if ( SWReadSDA() == 0 ) {
			if ( !SWSetSCL( LevelHi )  ) { return false; }
			if ( !SWSetSCL( LevelLow ) ) { return false; }
			return true;
		}

		// timeout?
		DWORD elapsed = GetTickCountInMs() - startTime;
		if ( elapsed  > (DWORD)maxWait ) {
			ERROR(("No ACK from slave\n"));
			SWStop();
			errNum = ERR_NOACK;
			return false;
		}

		// If more than 10 ms of busywait, give windows a chance
		if (elapsed > 10) {
			mdelay(1);
		}
   } // while
}

// I2C interfase
// General interfase to read/write bytes in Hardware mode
bool I2C::HWWR(const BYTE address,const BYTE* pOut,const UINT OutLen , BYTE* pIn,const UINT InLen,const bool DontWaitACK)
{
	TRACE(("I2C::HWWR"));

	// Start assuming no errors
	errNum = ERR_OK;
	UINT i = 0;

	// check if correct mode is selected
	if ( mode != Mode_HW ) {
		errNum = ERR_MODE;
		return false;
	}

	// This checks that caller did something with sense
	if (InLen < 1 && OutLen < 1) {
		errNum = ERR_PARAMETER;
		return false;
	}

	// Try to handle this request in compatibility mode if possible
	if (InLen == 0 && OutLen < 3 && OutLen > 0) { // Seems to be a pure write!

		_reg Temp = Shadow;   // obtain previous settings that didn't change
		Temp.addr_rw = address | I2C_WRITE;
		Temp.byte1   = pOut[0];

		if (OutLen == 1) {
			Temp.byte2   = 0;
			Temp.w3b     = 0;    // Cheated for Gemini: w3b is same bit as w3bra
		} else { // must be 2 bytes
			Temp.byte2   = pOut[1];
			Temp.w3b     = 1;		// Cheated for Gemini: w3b is same bit as w3bra
		}

		decRegI2C = *(DWORD *)&Temp;

		if ( !HWWaitUntilDone() ) {
			ERROR(("I2C timeout - not done\n"));
			return false;
		}

		if (DontWaitACK || HWReceivedACK() == true ) {
			return true;
		}

		ERROR(("Slave did not ACK\n"));

		// No ack, flag it!
		errNum = ERR_NOACK;
		return false;
	}

	if (OutLen == 0 && InLen == 1) { // Seems to be a pure read!

		_reg Temp = Shadow;   // obtain previous settings that didn't change
		Temp.addr_rw = address | I2C_READ;
		Temp.byte1   = 0;
		Temp.byte2   = 0;
		Temp.w3b     = 0;    // Cheated for Gemini: w3b is same bit as w3bra

		decRegI2C = *(DWORD *)&Temp;

		if ( !HWWaitUntilDone() ) {
			ERROR(("I2C timeout - not done\n"));
			return false;
		}

		pIn[0] = BYTE(decFieldI2CDB2);   // returned value is in 3rd byte

		if (DontWaitACK || HWReceivedACK() == true ) {
			return true;
		}

		ERROR(("Slave did not ACK\n"));

		// No ack, flag it!
		errNum = ERR_NOACK;
		return false;
	}

	// check if hardware supports multi-byte read/write
	if ( !pisces->IsBt878Family() ) {
		errNum = ERR_WRONGHARDWARE;
		return false;
	}

	// first send the chip address and the first data as a write
	_Gemini_reg Temp = *pGeminiShadow;   // obtain previous settings that didn't change

	if (OutLen) {

		Temp.addr_rw = address | I2C_WRITE;
		Temp.byte1   = pOut[0];
		Temp.byte2   = 0;
		Temp.nos1b	 = 0; // Trasmits START and 2 bytes
		Temp.w3bra	 = 0; // Transactions of 2 bytes
		Temp.nostop  = (InLen == 0 && OutLen == 1) ? 0 : 1; // (does not transmit stop at the end of transaction unless no reading and we are supposed to send just one byte
	
		decRegI2C = *(DWORD *)&Temp;

		if ( !HWWaitUntilDone() ) {
			return false;
		}

		if (!DontWaitACK && HWReceivedACK() == false ) {
			errNum = ERR_NOACK;
			return false;
		}

		// write 1 byte at a time to the device until all bytes transmitted
		for (i = 1; i < OutLen; i++ ) {

			Temp.addr_rw = pOut[i];
			Temp.byte1   = 0;
   			Temp.byte2   = 0;
			Temp.nos1b   = 1; // Do NOT transmit START again and write only 1 byte
			Temp.w3bra	 = 0; // Transactions of 1 byte
			Temp.nostop  = (InLen == 0 && (i == OutLen - 1)) ? 0 : 1; // (does not transmit stop at the end of transaction unless no bytes to read and we are going to send the last byte)
		
			decRegI2C = *(DWORD *)&Temp;

	   		if ( !HWWaitUntilDone() ) {
				ERROR(("I2C timeout - not done\n"));
				return false;
			}

			if (!DontWaitACK && HWReceivedACK() == false ) {
				ERROR(("Slave did not ACK\n"));
				errNum = ERR_NOACK;
				return false;
			}
		}
	}

	if (InLen) {
		// continuously read 1 byte at a time until all done (if we have to)!
		for (i = 0; i < InLen; i++ ) {

			Temp.addr_rw = address | I2C_READ;
			Temp.byte1   = 0;
   			Temp.byte2   = 0;
			Temp.nos1b   = ( !i ) ? 0 : 1;	  // Only the first BYTE, set to 0 (0=Transmits START and the address)
			Temp.w3bra   = ( i == InLen - 1 ) ? 0 : 1;  // last byte, set to 0 (0=master NACKs to end reads from slave) 
			Temp.nostop  = ( i == InLen - 1 ) ? 0 : 1;  // last byte, set to 0 (0=transmits STOP at end of transaction)

			decRegI2C = *(DWORD *)&Temp;

			if ( !HWWaitUntilDone() ) {
				return false;
			}

			pIn[i] = BYTE(decFieldI2CDB2);   // returned value is in 3rd byte
		}
	}
	return true;
}

bool I2C::SWWR(const BYTE address,const BYTE* pOut,const UINT OutLen , BYTE* pIn,const UINT InLen,const bool DontWaitACK)
{
	TRACE(("I2C::SWWR"));

	// Start assuming no errors
	errNum = ERR_OK;
	UINT l = 0;

	// check if correct mode is selected
	if ( mode != Mode_SW )
	{
		errNum = ERR_MODE;
		return false;
	}

	// This checks that caller did something with sense
	if (InLen < 1 && OutLen < 1) {
		errNum = ERR_PARAMETER;
		return false;
	}

	if (OutLen) {
		// Send Start
		if (!SWStart() ) return false;

		// Send I2C Write Address
		if ( !SWWrite(address | I2C_WRITE)) {
			if (!DontWaitACK && GetLastError() == ERR_NOACK) return false;
		}
		

		// Send Command
		for (l = 0; l<OutLen; l++) {
			if ( !SWWrite(pOut[l])) {
				if (!DontWaitACK && GetLastError() == ERR_NOACK) return false;
			}
		}
	}

	if (InLen) {
		// Send the (Repeated if WR) Start condition to start reception
		if (!SWStart() )	return false;

		// Send I2C Read Address
		if ( !SWWrite(address | I2C_READ)) {
			if (!DontWaitACK && GetLastError() == ERR_NOACK) return false;
		}

		// Receive response
		for (l = 0; l < InLen ; l++) {
			if ( !SWRead(pIn + l) ) return false;
			// ACK if it is NOT the end of data, else don't ack.
			if (l == InLen - 1) {
				// It's the last data, send NACK
				if ( !SWSendNACK() ) return false;
			} else {
				// It's not the last data, send ACK
				if ( !SWSendACK() ) return false;
			}
		}
	}

	if (InLen || OutLen) {
		// Send Stop!
		return SWStop();
	}

	return true;
}

// I2C HAL
// Our HAL for I2C ...	
bool I2C::SetFreq(const DWORD freq )
{
	// Get protection mutex
	mutex.Acquire();
	// Set Frequency
	if (currFreq != freq) {
		// Remember new frequency
		currFreq = freq;
		// And force setting the new frequency!
		mode = Mode_None; 
	}
	// Release protection mutex
	mutex.Release();
	return true;
}

// The universal RW primitive
bool I2C::WR(const BYTE address,const BYTE* pOut,const UINT OutLen , BYTE* pIn,const UINT InLen,const bool DontWaitACK )
{
	TRACE(("I2C::WR"));

	bool ret = false;

	// This is something that should ALWAYS be true, or something is pretty bad!
	ENSURE( pisces->PoweredDown() == false );

	// Avoid locks... We prefer to fail the fns
	if (pisces->PoweredDown()) {
		return false;
	}

	//If already 
	// Get protection mutex
	mutex.Acquire();

	// If we can do it in Hardware mode...
	if (
		( currFreq == 100000 || currFreq == 400000 ) &&
		( 
			/*pisces->IsBt878Family() || */ // <- Disabled the multiwrite, as it seems not to be working. I don't know why
			(InLen == 0 && OutLen < 3 && OutLen > 0) || (OutLen == 0 && InLen == 1) )
	   ) 
	{
		// Go to HW mode
		InitHWMode(currFreq);
		// Do the read/write in Hardware!
		ret = HWWR(address,pOut,OutLen,pIn,InLen,DontWaitACK);

	} else {
		// Go to Software mode
		InitSWMode(currFreq);
		// Do the read/write in Software!
		ret = SWWR(address,pOut,OutLen,pIn,InLen,DontWaitACK);
	}

	// Release protection mutex
	mutex.Release();

	// return status
	return ret;
}

