/*
	I2C.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __I2C_HPP
#define __I2C_HPP

#include "CppRt.hpp"
#include "MyTypes.hpp"
#include "Decoder.hpp"
#include "regField.hpp"
#include "Mutex.hpp"
#include "SyncEvent.hpp"

// Forward decls
class BtPisces;

/////////////////////////////////////////////////////////////////////////////
// CLASS I2C
//
// Description:
//    This class encapsulates the register fields in the I2C register of the
//    Bt848. A complete set of functions are developed to manipulate all the
//    register fields in the I2C for the Bt848.
//
/////////////////////////////////////////////////////////////////////////////

class I2C
{
public:
	/* Type: Level
	 * Purpose: used to define a pin state
	 */
	typedef enum { 
		LevelLow = 0, LevelHi 
	} Level;

	typedef enum {
		ERR_OK    = 0,		// no error
		ERR_INIT     ,		// error in initialization
		ERR_MODE     ,		// invalid I2C mode (must be either HW or SW)
		ERR_NOACK    ,		// no ACK received from slave
		ERR_TIMEOUT  ,		// timeout error
		ERR_SCL      ,		// unable to change SCL line
		ERR_SDA      ,		// unable to change SDA line
		ERR_PARAMETER,		// parameter error
		ERR_WRONGHARDWARE	// wrong hardware
	} Status;


private:
	// define which mode the I2C is selected
	enum Mode { Mode_None, Mode_HW, Mode_SW };
   
	bool			initOK;			// initialization is successful?
	DWORD			swbitdelayus;	// bit delay for software control
	DWORD			hwtimeoutms;	// maximum time to wait for hardware completion
	Status			errNum;			// error number
	Mode			mode;			// which mode the I2C is running in
	Mutex			mutex;			// Mutex used to ensure only one user of I2C at a time
	SyncEvent		eI2CDone;		// Condition Var to signal I2C op. is done.
	volatile bool	GotI2CInt;		// This will become true when we get a I2C int
	DWORD			currFreq;		// Current I2C frequency

	//**************************************************************************
	//	Structures
	//**************************************************************************
	struct _reg    // I2C register structure9
	{
		unsigned int sda:1;
		unsigned int scl:1;
		unsigned int w3b:1;
		unsigned int sync:1;
		unsigned int div:4;
		unsigned int byte2:8;
		unsigned int byte1:8;
		unsigned int addr_rw:8;
	} Shadow;

	struct _Gemini_reg           // I2C register structure for Gemini
	{	
		unsigned int sda:1;
		unsigned int scl:1;
  		unsigned int w3bra:1;
		unsigned int sync:1;
		unsigned int nos1b:1;
		unsigned int nostop:1;
		unsigned int rate:1;
		unsigned int mode:1;
		unsigned int byte2:8;
		unsigned int byte1:8;
		unsigned int addr_rw:8;
	} * pGeminiShadow;

protected:
	RegisterDW decRegINT_STAT;
	RegField decFieldI2CDONE;
	RegField decFieldRACK;
	RegisterDW decRegI2C;
	RegField decFieldI2CDB0;
	RegField decFieldI2CDB1;
	RegField decFieldI2CDB2;
	RegField decFieldI2CDIV;
	RegField decFieldSYNC;
	RegField decFieldW3B;
	RegField decFieldSCL;
	RegField decFieldSDA;

	// new for Gemini
	RegField decFieldI2CMODE;
	RegField decFieldI2CRATE;
	RegField decFieldI2CNOSTOP;
	RegField decFieldI2CNOS1B;
	RegField decFieldI2CW3BRA;

	// Pointer to the Card Information
	BtPisces*	pisces;

public:
	// constructor and destructor
	I2C(BtPisces* p, const DWORD_PTR dwAddress );
	~I2C();

	// Basic I2C access fns.
	bool	SetFreq( const DWORD freq );
	DWORD	GetFreq() const { return currFreq; }	// Only hardware mode
	bool	WR(const BYTE address,const BYTE* pOut,const UINT OutLen , BYTE* pIn,const UINT InLen,const bool DontWaitACK = false );
	Status	GetLastError( void );

	// Convenience fns.
	bool	Read(const BYTE address, BYTE *value );
	bool	Write(const BYTE address,const BYTE value1 );
	bool	Write(const BYTE address,const BYTE value1,const BYTE value2 );
	
	// Callback to signal from ISR I2C Op is done.
	/* Note: this is called at IRQL = DISPATCH_LEVEL */
	void	IntReceived() {GotI2CInt = true; eI2CDone.Signal(); }

private:

	// I2C Primitives
	bool	InitHWMode( long freq = 0);				// uses current freq. if 0. HW mode supports only 100KHz and 400 KHz
	bool	InitSWMode( long freq = 0);				// uses current freq. if 0
	
	// Hw Mode
	void	SetHWFreq( long freq );					// Only Hardware mode
	int		ReadDiv( void );

	// Sw mode
    bool	SWStart( void );
	bool	SWStop( void );
	bool	SWRead( BYTE * value );
	bool	SWWrite( BYTE value );
	bool	SWSendACK( void );
	bool	SWSendNACK( void );
	bool	SWSetSCL( Level );
	int		SWReadSCL( void );
	bool	SWSetSDA( Level );
	int		SWReadSDA( void );
	bool	SetSync( State );
	int		ReadSync( void );
	void	ResetShadow( void );      // reset register shadow
	bool	HWWaitUntilDone( void );  // wait until  completes operation
	bool	HWIsDone( void );         // check interrupt bit for operation done
	bool	HWReceivedACK( void );    // check interrupt bit for received ACK
	void	SWBitDelay( void );       // insert delay to simulate frequency
	bool	SWWaitForACK( void );     // wait for ACK from receiver using software

	// Our universal RW primitives
	bool	HWWR(const BYTE address,const BYTE* pOut,const UINT OutLen , BYTE* pIn,const UINT InLen,const bool DontWaitACK = false);
	bool	SWWR(const BYTE address,const BYTE* pOut,const UINT OutLen , BYTE* pIn,const UINT InLen,const bool DontWaitACK = false);

};

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::ReadDiv( void )
//  Purpose: Obtain value of programmable divider
//  Input:   None
//  Output:  None
//  Return:  Value of programmable divider
/////////////////////////////////////////////////////////////////////////////
inline int I2C::ReadDiv( void )
{
	return decFieldI2CDIV;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  Status I2C::GetLastError( void )
//  Purpose: Obtain last I2C error number
//  Input:   None
//  Output:  None
//  Return:  Last I2C error number
/////////////////////////////////////////////////////////////////////////////
inline I2C::Status I2C::GetLastError( void )
{
   return errNum;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  bool I2C::HWReceivedACK( void )
//  Purpose: Determine if ACK is receieved
//  Input:   None
//  Output:  None
//  Return:  true if ACK received; else false
/////////////////////////////////////////////////////////////////////////////
inline bool I2C::HWReceivedACK( void )
{
   return ( ( decFieldRACK != 0 ) ? true : false );
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::SWReadSCL( void )
//  Purpose: Read software SCL value
//  Input:   None
//  Output:  None
//  Return:  SCL value
/////////////////////////////////////////////////////////////////////////////
inline int I2C::SWReadSCL( void )
{
	return decFieldSCL;
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::SWReadSDA( void )
//  Purpose: Read software SDA value
//  Input:   None
//  Output:  None
//  Return:  SDA value
/////////////////////////////////////////////////////////////////////////////
inline int I2C::SWReadSDA( void )
{
	return decFieldSDA;
}


/////////////////////////////////////////////////////////////////////////////
//  Method:  void I2C::SWBitDelay( void )
//  Purpose: Insures minimum high and low clock times on I2C bus.
//  Input:   None
//  Output:  None
//  Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void I2C::SWBitDelay( void )
{
	// Wait the bit time
	busywaitus(swbitdelayus); 
}

/////////////////////////////////////////////////////////////////////////////
//  Method:  int I2C::ReadSync( void )
//  Purpose: Read I2C sync value
//  Input:   None
//  Output:  None
//  Return:  Sync value
/////////////////////////////////////////////////////////////////////////////
inline int I2C::ReadSync( void )
{
	return decFieldSYNC;
}

inline bool I2C::Read(const BYTE address, BYTE *value )
{
	return WR(address,NULL,0,value,1);
}

inline bool I2C::Write(const BYTE address,const BYTE value1 )
{
	return WR(address,&value1,1,NULL,0);
}

inline bool I2C::Write(const BYTE address,const BYTE value1,const BYTE value2 )
{
	BYTE Out[] = { value1, value2 };
	return WR(address,Out,2,NULL,0);
}

#endif // __I2C_H
