/*
	Pisces.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __PISCES_H
#define __PISCES_H

#include "CppRt.hpp"	// To get new and delete ops
#include "RiscEng.hpp"
#include "Field.hpp"
#include "Decoder.hpp"
#include "Pll.hpp"
#include "TimGen.hpp"
#include "Vidch.hpp"
#include "Defaults.hpp"

#define BIT(x) ((DWORD)1 << (x))

const DWORD VSYNC_I     = BIT(1);         // 0x00000001
const DWORD HSYNC_I     = BIT(2);         // 0x00000002
const DWORD I2CDONE_I   = BIT(8);         // 0x00000100
const DWORD GPINT_I     = BIT(9);         // 0x00000200
const DWORD RISC_I      = BIT(11);        // 0x00000800
const DWORD FBUS_I      = BIT(12);        // 0x00001000
const DWORD FTRGT_I     = BIT(13);        // 0x00002000
const DWORD FDSR_I      = BIT(14);        // 0x00004000
const DWORD PPERR_I     = BIT(15);        // 0x00008000
const DWORD RIPERR_I    = BIT(16);        // 0x00010000
const DWORD PABORT_I    = BIT(17);        // 0x00020000
const DWORD OCERR_I     = BIT(18);        // 0x00040000
const DWORD SCERR_I     = BIT(19);        // 0x00080000
const DWORD DMAERR_I    = BIT(20);        // 0x00100000

const DWORD PARITY_I    = (BIT(15)|BIT(16)); // 0x00018000

const DWORD EN_TRITON1_BUG_FIX   = BIT(23);  // 0x00800000
	
#define  MAX_REGISTER   0x200 // Maximum register

#include "DPC.hpp"
#include "GPIO.hpp"
#include "I2C.hpp"
#include "Queue.hpp"

#if DBG
#define PISCES_SIGNATURE 0xaabb5533
#endif

/* Type: VideoFormat
 * Purpose: Used to define video format
 */
typedef enum {  VFormat_NTSC = 0,
                VFormat_NTSC_J,
                VFormat_PAL_BDGHI,
                VFormat_PAL_M,
                VFormat_PAL_N,
                VFormat_SECAM,
                VFormat_PAL_N_COMB,		// this mode was new to 848A
				VFormat_PAL_60,			// this mode can be created in the Bt878+
				VFormat_NTSC_50,		// this mode can be created in the Bt878+
				VFormat_NTSC_Digital,	// NTSC 720x480 digital
				VFormat_PAL_Digital,	// PAL 720x576 digital
				VFormat_SVI_Digital,	// Silicon Vision Digital camera
				VFormat_LVM_Digital,	// Logitech Videoman Digital camera
				VFormat_RQS_Digital		// Rockwell QuartzSight Digital camera
} VideoFormat;

// Forward decls
class BtCard;

/* Class: BtPisces
 * Purpose: Controls the BtPisces video capture chip
 * Attributes:
 * Operations:
 * Note: Every time any of the Set functions is called, operation must stop.
 *   After all changes are done execution can resume. This means that all RISC
 *   programs are destroyed ( if they exist ), changes made, new RISC programs
 *   are created ( if needed )
*/
class BtPisces
{
#if DBG
	public:
		DWORD Signature;
#endif
	protected:

		RegisterDW ColorControl; 
		RegField BSWAP_EVEN; 
		RegField BSWAP_ODD; 
		RegField WSWAP_EVEN; 
		RegField WSWAP_ODD; 

		RegisterDW InterruptStatus; 
		RegField FMTCHG; 
		RegField VSYNC; 
		RegField HSYNC; 
		RegField OFLOW; 
		RegField HLOCK; 
		RegField VPRES; 
		RegField I2CDONE; 
		RegField GPINT; 
		RegField RISCI; 
		RegField FBUS; 
		RegField FTRGT; 
		RegField FDSR; 
		RegField PPERR; 
		RegField RIPERR; 
		RegField PABORT; 
		RegField OCERR; 
		RegField SCERR; 
		RegField FIELD; 
		RegField RACK; 
		RegField V5IO; 
		RegField RISC_EN; 
		RegField RISCS;

		RegisterDW InterruptMask; 
		RegField IMASK_FMTCHG; 
		RegField IMASK_VSYNC; 
		RegField IMASK_HSYNC; 
		RegField IMASK_OFLOW; 
		RegField IMASK_HLOCK; 
		RegField IMASK_VPRES; 
		RegField IMASK_I2CDONE; 
		RegField IMASK_GPINT; 
		RegField IMASK_RISCI; 
		RegField IMASK_FBUS; 
		RegField IMASK_FTRGT; 
		RegField IMASK_FDSR; 
		RegField IMASK_PPERW; 
		RegField IMASK_RIPERW; 
		RegField IMASK_PABORT; 
		RegField IMASK_OCERW; 
		RegField IMASK_SCERW; 
	
		RegisterDW ColorFormat; 
		RegField COLOR_EVEN; 
		RegField COLOR_ODD;

		RegisterDW CaptureControl; 
		RegField CAPTURE_EVEN; 
		RegField CAPTURE_ODD; 
		RegField CAPTURE_VBI_EVEN; 
		RegField CAPTURE_VBI_ODD; 
		RegField DITH_FRAME; 

		// Field capture counter (only bt848a or better)
		RegisterB FCNTR;

	public:

		// Predefined timing generator tables
		typedef enum { 
			TGB_NTSC624 = 0 , 
			TGB_NTSC780, 
			TGB_NTSC858, 
			TGB_NTSC392, 
			TGB_PAL858,

			TGB_None = 0xFF	// No timing generator table required...
			} TimeGenByte;

		// structure contains video information
		typedef struct {
			WORD			StdX;                //
			WORD			Clkx1_HACTIVE;       //
			WORD			Clkx1_HDELAY;        //
			WORD			Min_Pixels;          //
			WORD			Max_Pixels;          //
			WORD			Min_Lines;           //
			WORD			Max_Lines;			 //
			WORD			Max_VFilter1_Pixels; //
			WORD			Max_VFilter2_Pixels; //
			WORD			Max_VFilter3_Pixels; //
			WORD			Max_VFilter1_Lines;  //
			WORD			Max_VFilter2_Lines;  //
			WORD			Max_VFilter3_Lines;  //
			WORD			VDelay;				 //
			ULONG			FscX4;
			BtVideoFormat	VF;					 // Real VideoFormat to use
			DWORD			VTotal;				 // VTotal (if used)
			BYTE			ADelay;				 // ADelay
			BYTE			BDelay;				 // BDelay
			bool			ChromaComb;			 // Chroma comb filter
			LONGLONG		TimeBetweenFields;	 //  Time Between Fields
			VidField		TopField;			 // Top Field
			ULONG			FieldsPerSecondX100; // Fields per second * 100
			bool			DigitalSignal;		 // If video source is a digital signal
			TimeGenByte		TGB_Table;			 // Timing generator table to be used
			BYTE			HReset;				 // HReset pulse to use
			bool			AutoHFilter;		 // If we can use automatic HFilter
		} VideoInfoStruct;

	protected:

		// Video information for each mode
		static VideoInfoStruct VideoInfo[];

		// And the required timing generator tables
		static BYTE SRAMTable[][ 52 ];

	private:

		// The decoder part of the Bt8xx
		Decoder				Decoder_;

		// all possible fields (All supported by the Bt848)
		VideoField			Even_;
		VideoField			Odd_;
		VBIField			VBIE_;
		VBIField			VBIO_;

		// The RISC engine we use to create all programs
		RISCEng				Engine_	;				

		// PLL of the Bt848A+
		PLL					PLL_;

		// Timing Generator of the Bt848A+
		TimGen				TimGen_;

		// The other parts of a Bt8xx
		I2C					i2c_;
		GPIO				gpio_;
		
		// All RISC programs we could require
		RiscPrgHandle		RISCProg[12];	

		// And the array of enabled programs
		bool				RISCProgEnabled[12];

		// Number of the last executed group
		DWORD				dwLastExecutedGroup;

		// Gamma value
		DWORD				dwGamma;

		// Interrupt variables
		volatile LONG		i_lPendingEntries;	// Pending ISR entries to be processed...
		DWORD				i_MergedInts;		// Merged interrupt sources (so no int loss)
		DPC					i_Dpc;				// Delayed Procedure Call.
		Queue<DWORD,9>		i_Queue;			// Queue of ints (512 pending ints)
		
		// PowerDown / Up Variables to remember State
		bool				bPoweredDown;		// If Chip is powered Down
		char				achSavedRegs[MAX_REGISTER+4];
		BYTE				SavedSRAM[64];
#ifdef SAVE_PCI_CONFIG
		DWORD				dwSavedPCIConfig[16];	// The PCI configuration space
#endif

		// Base address of Chip
		DWORD_PTR				dwBaseAddress;		

		// Var to count fields (only bt848)
		DWORD				dwFields;

		// Pointer to the device object (To Access PCI)
		const PDEVICE_OBJECT PDO;

		// Pointer to the Card Descriptor
		BtCard*				Card;

		// Mutex to protect RISC programs 
		LONG				RISC_Lock;

		// And the Extra ints we must enable 
		DWORD				dwEnabledInts;		// This is the mask of ints that should be enabled as requested by extern objects

		// Current video information
		VideoInfoStruct*	viCurrent;			
		
	public:		

		// decoder 'set' group
		ErrorCode	SetGamma	  ( DWORD value );
		ErrorCode	SetBrightness( DWORD value ) { return Decoder_.SetBrightness( value ); }
		ErrorCode	SetSaturation( DWORD value ) { return Decoder_.SetSaturation( value ); }
		ErrorCode	SetConnector ( DWORD value , bool IsSVideo) { return Decoder_.SetVideoInput( value , IsSVideo ); }
		ErrorCode	SetContrast  ( DWORD value ) { return Decoder_.SetContrast( value ); }
		ErrorCode	SetHue       ( DWORD value ) { return Decoder_.SetHue( value ); }
		ErrorCode	SetWhiteBalance( DWORD Data ) { return Decoder_.SetWhiteBalance( Data ); }
		ErrorCode	SetBacklightCompensation( DWORD Data ) { return Decoder_.SetBacklightCompensation( Data ); }
		ErrorCode	SetColorEnable( bool Data ) { return Decoder_.SetColorEnable( Data ); }
		ErrorCode	SetSharpness( DWORD Data );
		ErrorCode	SetAutoSaturation(bool Auto) { return Decoder_.SetAutoSaturation(Auto);  }
		ErrorCode	SetAutoGamma(bool Auto) { return Decoder_.SetAutoGamma(Auto); }
		ErrorCode	SetAutoSharpness(bool Auto);
		ErrorCode	SetVideoFormat(const DWORD xtal0freq, const DWORD xtal1freq,const VideoFormat vf);
		ErrorCode	SetCrystal( Crystal cr) { return Decoder_.SetCrystal(cr) ; }
		ErrorCode	SetVideoSignalFormat( SignalFormat format ) { return Decoder_.SetVideoSignalFormat( format ); }
		void		SetFullOutputRange( bool mode ) { Decoder_.SetFullOutputRange( mode ); }
		void		SetHReset( int clk ) { Decoder_.SetHReset( clk ); }
		ErrorCode	SetSyncVideoRef( SyncVideoRef ref ) { Decoder_.SetSyncVideoRef( ref ); }
		void		EnableSyncOutput( bool mode ) { Decoder_.EnableSyncOutput( mode ); }

		// decoder 'get' group
		DWORD		GetGamma() { return dwGamma; }
		DWORD		GetBrightness() { return Decoder_.GetBrightness(); }
		DWORD		GetSaturation() { return Decoder_.GetSaturation(); }
		DWORD		GetConnector () { return Decoder_.GetVideoInput(); }
		DWORD		GetContrast  () { return Decoder_.GetContrast(); }
		DWORD		GetHue       () { return Decoder_.GetHue(); }
		DWORD		GetWhiteBalance() { return Decoder_.GetWhiteBalance(); }
		DWORD		GetBacklightCompensation() { return Decoder_.GetBacklightCompensation(); }
		DWORD		GetColorEnable() { return Decoder_.GetColorEnable(); }
		DWORD		GetSharpness() { return Even_.GetSharpness(); }
		bool		GetAutoSaturation() { return Decoder_.GetAutoSaturation();  }
		bool		GetAutoGamma() { return Decoder_.GetAutoGamma(); }
		bool		GetAutoSharpness() { return Even_.GetAutoSharpness();  }

		// misc decoder functions
		bool		IsDeviceInHLock() { return Decoder_.IsDeviceInHLock(); }
		bool		Is525LinesVideo() { return Decoder_.Is525LinesVideo(); }
		void		EnableChromaFilterBypass( bool mode ) { Decoder_.EnableChromaFilterBypass( mode ); }
		ULONG		TimeDefaultField( Crystal cr ); // Get time in ns

		// Info on the current videoformat
		DWORD		MaxLinesPerField( );
		LONGLONG	TimeBetweenFields( );	// Time between fields in 100 ns units
		ULONG		FieldsPerSecondX100( );	// Amount of Fields per second for the current videostandard * 100
		VidField	GetTopField( );			// Get top field for current videoformat
		void		GetMaxFrame(RECT& r);
		VideoInfoStruct* GetVideoInfo() { return viCurrent; }
		ULONG		GetCurrFSCX4() { return viCurrent->FscX4; }
		void		TurnVFilter( State s );

		// Info on the current chipset
		bool		IsDigitalSignal() const;
		bool		IsGeminiFamily() const;
		bool		IsGeminiFamilyRevD() const;
		bool		IsBt878Family() const;
		bool		HasPEAK() const;

		// GPIO Functions
		GPIO&		GetGPIO() { return gpio_; }
		void		SetGPCLKMODE( State s) { gpio_.SetGPCLKMODE(s); }
		int			GetGPCLKMODE( void ) { return gpio_.GetGPCLKMODE(); }
		void		SetGPIOMODE( GPIO::GPIOMode m) { gpio_.SetGPIOMODE(m); }
		int			GetGPIOMODE( void ) { return gpio_.GetGPIOMODE(); }
		void		SetGPWEC( State s) { gpio_.SetGPWEC(s); }
		int			GetGPWEC( void ) { return gpio_.GetGPWEC(); }
		void		SetGPINTI( State s) { gpio_.SetGPINTI(s); }
		int			GetGPINTI( void ) { return gpio_.GetGPINTI(); }
		void		SetGPINTC( State s) { gpio_.SetGPINTC(s); }
		int			GetGPINTC( void ) { return gpio_.GetGPINTC(); }
		bool		SetGPOE( int x, State s) { return gpio_.SetGPOE(x,s); }
		void		SetGPOE( DWORD d) { gpio_.SetGPOE(d); }
		int			GetGPOE( int i) { return gpio_.GetGPOE(i); }
		DWORD		GetGPOE( void ) { return gpio_.GetGPOE(); }
		bool		SetGPIE( int x, State s) { return gpio_.SetGPIE(x,s); }
		bool		SetGPIE( DWORD d) { return gpio_.SetGPIE(d); }
		int			GetGPIE( int x) { return gpio_.GetGPIE(x); }
		DWORD		GetGPIE( void ) { return gpio_.GetGPIE(); }
		bool		SetGPDATA( DWORD * d, int x , int offset = 0 ) { return gpio_.SetGPDATA(d,x,offset); }
		bool		GetGPDATA( DWORD * d, int x , int offset = 0 ) { return gpio_.GetGPDATA(d,x,offset); }
		bool		SetGPDATA( int x, int y, DWORD z, int offset = 0 ) { return gpio_.SetGPDATA(x,y,z,offset); }
		bool		GetGPDATA( int x, int y, DWORD *z, int offset = 0 ) { return gpio_.GetGPDATA(x,y,z,offset); }

		// I2C Functions
		I2C&		GetI2C() { return i2c_; }
		bool		I2CSetFreq( const DWORD freq ) { return i2c_.SetFreq(freq); }
		DWORD		I2CGetFreq() const { return i2c_.GetFreq(); }	
		bool		I2CWR(const BYTE address,const BYTE* pOut,const UINT OutLen , BYTE* pIn,const UINT InLen,const bool DontWaitACK = false ) { return i2c_.WR(address,pOut,OutLen,pIn,InLen,DontWaitACK); }
		bool		I2CRead(const BYTE address, BYTE *value ) { return i2c_.Read( address, value ); }
		bool		I2CWrite(const BYTE address,const BYTE value1 ) { return i2c_.Write( address, value1 ); }
		bool		I2CWrite(const BYTE address,const BYTE value1,const BYTE value2 ) { return i2c_.Write( address, value1, value2 ); }
		int			I2CGetLastError( void ) { return i2c_.GetLastError(); }

		// User interrupts access: MUST BE CALLED WHEN VIDEO IS NOT STREAMING (AT INITIALIZATION TIME)
		void		EnableUserInt(const DWORD nbr);
		void		DisableUserInt(const DWORD nbr);

		// PLL access functions (if fin == fout, disables pll)
		void		SetPll(unsigned int fin, unsigned int fout);
		void		SetPllToSleep();

		// Timing Generator access fns
		void		TimGenLoadSRAM(PBYTE p,int Bytes) { TimGen_.LoadSRAM(p,Bytes); }
		void		TimGenSetDecInputClk( DecInputClk clk) { TimGen_.SetDecoderInputClock(clk); }

		// Chip identification
		DWORD_PTR		GetBase( void ) const { return dwBaseAddress; }

		// Misc fns
		BYTE		GetFieldCount();

		// Disable all ints generated by the Bt8xx hardware
		ErrorCode	DisableInts();

		// Power fns
		ErrorCode	PowerDown( void );
		ErrorCode	PowerUp( void );
		bool		PoweredDown( void ) const { return bPoweredDown; }

		// Video Channel functions
		VideoChannel* OpenVideoChannel(
			const VCMode mode,			// Mode of opening
			VideoChanIface * xface,		// Data buffer provider - consumer
			const MRect& analogwin,		// AnalogWindow
			const MRect& digitalwin,	// DigitalWindow
			const ColFmt ColorFormat	// Color format
			);
	
		void		CloseVideoChannel( VideoChannel * chan );

		// This fns are called by the Field class to enable/disable field capture
		void SetFieldEnableState(const VidField evenodd, bool isVBI, bool Enable);

		// Constructors and destructors!
		BtPisces(
			BtCard *cd,
			const PDEVICE_OBJECT pPDO,
			const PVOID HwDeviceExtension_,
			const DWORD_PTR dwBase );
		~BtPisces();

		// Called when a Hardware interrupt has occurred
		BOOLEAN		HwInterrupt();

	private:

		// The following 2 must be called only from Interrupt() and can't use the register class
		void		Interrupt(DWORD status);
		void		int_RISCInt(void);
		void		Int_Restart(void);

		// Power functions
		void		SaveRegister( DWORD_PTR dwReg ) { *((PDWORD)( achSavedRegs + dwReg )) = READ_REGISTER_ULONG ( PDWORD(dwReg + dwBaseAddress) ); }
		void		RestoreRegister( DWORD_PTR dwReg ) { WRITE_REGISTER_ULONG( PDWORD(dwReg + dwBaseAddress) , *((PDWORD)( achSavedRegs + dwReg )) ); }
		void		SaveRegisterState( void );
		void		RestoreRegisterState( void );
		ErrorCode	SetBt878PowerState(DWORD dwNewPowerState );

		// Misc Fns
		static VOID cbDpcFunc (IN PKDPC Dpc , IN PVOID DeferredContext, IN PVOID SystemArgument1, IN PVOID SystemArgument2 );
		// Function called on Interrupt but as a delayed Call
		void		DpcFunc(); // 

};


#endif
