/*
	RegBase.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __REGBASE_H
#define __REGBASE_H

#include "debug.hpp"
#include "CppRt.hpp"
#include "MyTypes.hpp"

/* Type: AllFs
 * Purpose: To be used as an error return value from register accessing
 *   functions. All bits are set to 1.
 */
const DWORD AllFs = (DWORD) ~0L;

/* Function: ReturnAllFs
 * Purpose: This function is used in the register access methods to indicate
 *   that some sort of error has occured. Used for easing the debugging, as
 *   it contains a macro to print the error if DEBUG is #defined
 */
inline DWORD ReturnAllFs()
{
	ERROR(("Illegal Read"));
    return  AllFs;
}

/*
 * Type: RegisterType
 * Purpose: A type to differentiate between diferent kinds of registers.
 *   Depending on the type register may not peforms certain operations
 *   RW - read/write, RO - read-only
*/
typedef enum { RW, RO, RR } RegisterType;

/* Class: RegBase
 * Purpose:
 *   Defines the interface and encapsulates the register access.
 * Attributes:
 *   type_: RegisterType - defines the access permission for the register.
 *   dwShadow_: DWORD - a local copy of the register. Used for returning a value
 *   of write-only registers
 * Operations:
 *   operator DWORD(): data access method. Pure virtual
 *   DWORD operator=( DWORD ): assignment operator. Pure virtual. This assignment
 *      operator does not return a reference to the class because of the performance reasons
 *   RegisterType GetRegisterType()
 *   void SetShadow( DWORD ): assigns a value of a register to a shadow
 *   DWORD GetShadow( void ): retrieves a value from a shadow
 */
class RegBase
{

#if DBG
	private:
		const RegisterType type_;
#endif

	public:
		RegBase( const RegisterType aType ) 
#if DBG
		: type_( aType )
#endif
		{}

#if DBG
		RegisterType GetRegisterType() const { return type_; }
#endif
		virtual operator DWORD() = 0;
		virtual DWORD operator=( DWORD dwValue ) = 0;
		virtual ~RegBase() {}
};

#endif __REGBASE_H
