/*
	RegField.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __REGFIELD_H
#define __REGFIELD_H

#include "Register.hpp"

// maximum size of a register in bits
static const BYTE MaxWidth = 32;

/* Function: MakeAMask
 * Purpose: Creates a bit mask to be used in different register classes
 * Input:
 *   bWidth: BYTE - width of a mask in bits
 * Output:
 *   DWORD
 * Note: This function is inline
 */
static inline DWORD MakeAMask( BYTE bWidth )
{
   return ( ( (bWidth >= 32) ? 0 : ((DWORD)1 << bWidth) ) - 1);
}


/* Class: RegField
 * Purpose: This class encapsulates the behaviour of a register which is a set
 *   of bits withing a larger register
 * Attributes:
 *   Owner_: Register & - reference to the register that contains this field.
 *   It is a reference to Register class because actual register can be either one of
 *   byte, word or dword registers.
 *   StartBit_: BYTE - starting position of this field
 *   FieldWidth_: BYTE - width of this field in bits
 * Operations:
 *   operator DWORD(): data access method. Returns a value of the register
 *   DWORD operator=( DWORD ): assignment operator. Used to set the register
 *   These operations assume that a parent register has RW attribute set, though
 *   not all register fields of it are read-write. If RW is not used for the parent
 *   this class may be in error.
 * Note: the error handling provided by the class is minimal. It is a responibility
 *   of the user to pass correct parameters to the constructor. The class has
 *   no way of knowing if the correct owning registe passed in is correct,
 *   for example. If starting bit or width is beyond the maximum field width
 *   the mask used to isolate the field will be 0xFFFFFFFF
 */
class RegField : public RegBase
{
	private:
		Register &Owner_;
		const BYTE      StartBit_;
		const DWORD		Mask_;
#if DBG 
#else
		// Keep track of the register type ourselves
		const RegisterType rType;
		RegisterType GetRegisterType() const { return rType; }
#endif		

   public:
      virtual operator DWORD();
      virtual DWORD operator=( DWORD dwValue );
      RegField( Register &AnOwner,const BYTE nStart,const BYTE nWidth,const RegisterType aType ) :
		RegBase( aType ), Owner_( AnOwner ), StartBit_( nStart ),
		Mask_(MakeAMask( nWidth ) << nStart ) 
#if DBG
#else
		// Keep track of the register type ourselves
		, rType(aType)
#endif
         {}
};


#endif __REGFIELD_H
