/*
	Register.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __REGISTER_H
#define __REGISTER_H

#include "regbase.hpp"

/* Type: Register
 * Purpose: An intermediate class between the RegBase and actual usable classes.
 *   Actual classes are RegisterB, RegisterW, RegisterDW
 * Attributes:
 *   uAddr_: unsigned int - an address of the register
 * Operations:
  *   operator DWORD(): data access method.
 *   DWORD operator=( DWORD ): assignment operator.
 * Note:
 *   The reason to have operators in this class is for the register field class
 *   to have a member of type 'reference to Register'. Otherwise RegField is not
 *   able to use access methods.
 */
class Register : public RegBase
{
	private:
		const DWORD_PTR uAddr_;

    public:
		DWORD_PTR GetAddr() const { return uAddr_; }

		virtual operator DWORD() = 0;
		virtual DWORD operator=( DWORD dwValue ) = 0;

        Register(const DWORD_PTR uAddr,const RegisterType aType ) :
            RegBase( aType ), uAddr_( uAddr ) {}
};

/* Type: RegisterB
 * Purpose: A register that performs the BYTE I/O
 * Note:
 *   This class has no additional data members, it just overloads operators
 */
class RegisterB : public Register
{
	public:
		virtual operator DWORD();
		virtual DWORD operator=( DWORD dwValue );
		RegisterB(const DWORD_PTR uAddr,const RegisterType aType ) :
            Register( uAddr, aType ) {};
};

/* Type: RegisterW
 * Purpose: A register that performs the WORD I/O
 * Note:
 *   This class has no additional data members, it just overloads operators
 */
class RegisterW : public Register
{
	public:
		virtual operator DWORD();
		virtual DWORD operator=( DWORD dwValue );
		RegisterW(const DWORD_PTR uAddr,const RegisterType aType ) :
			Register( uAddr, aType ) {};

};


/* Type: RegisterDW
 * Purpose: A register that performs the DWORD I/O
 * Note:
 *   This class has no additional data members, it just overloads operators
 */
class RegisterDW : public Register
{
	public:
		virtual operator DWORD();
		virtual DWORD    operator=( DWORD dwValue );

		RegisterDW(const DWORD_PTR uAddr,const RegisterType aType ) :
			Register( uAddr, aType ) {};
};

#endif __REGISTER_H
