/*
	RiscEng.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "Risceng.hpp"
#include "CppRt.hpp"	// To get new and delete ops
#include "Defaults.hpp"

RISCEng::RISCEng(
	const PVOID HwDeviceExtension_,
	const DWORD_PTR dwBase ) :

	// Save all things needed
	HwDeviceExtension(HwDeviceExtension_) ,
	pRISCMem( (PBYTE) StreamClassGetDmaBuffer( HwDeviceExtension_ ) ) ,
	// Construct registers
	RISC_IPC( 0x0114 + dwBase, RW) , 
	Control( 0x010C + dwBase, RW), 
	FIFO_ENABLE( Control, 0, 1, RW), 
	RISC_ENABLE( Control, 1, 1, RW), 
	PKTP( Control, 2, 2, RW), 
	PLTP1( Control, 4, 2, RW), 
	PLTP23( Control, 6, 2, RW), 
	GPCLKMODE( Control, 10, 1, RW), 
	GPIOMODE( Control, 11, 2, RW), 
	GPWEC( Control, 13, 1, RW), 
	GPINTI( Control, 14, 1, RW), 
	GPINTC( Control, 15, 1, RW) ,

	bMustBeRunning ( FALSE )
{
	ENSURE( dwBase );
	ENSURE( HwDeviceExtension_);
	ENSURE( pRISCMem ) ;

	// Init the memory allocator flags
	memset(bVBIBlkIsAlloc,0,sizeof(bVBIBlkIsAlloc));
	memset(bFieldBlkIsAlloc,0,sizeof(bFieldBlkIsAlloc));

}

RISCEng::~RISCEng()
{
}

void RISCEng::Start( RiscPrgHandle ToStart )
{ 
	DUMP(("Starting RISC program (%x)\n", &ToStart));
	ENSURE( ToStart );

	// Trigger point = 1 !
	PKTP = 3;	// 32 DWORDS for PACKED
	PLTP1 = 2;	// 16 DWORDS for PLANAR
	PLTP23 = 2;	// 16 DWORDS for PLANAR

	// Note: Brooktree is using 8 DWORDS for packed, 4 DWORDS for PLANAR.
	//		 Linux is using 32 DWORD for packed, 16 DWORDS for PLANAR.

	// Start program!
	DWORD Phys = ToStart->GetPhysProgAddr() & ~0x3;
	DUMP(("Physical Address: %x, Addr IPC = %x\n",Phys,RISC_IPC.GetAddr() ));

	RISC_ENABLE = 0; 
	FIFO_ENABLE = 0; 
	RISC_IPC = Phys;
	FIFO_ENABLE = 1; 
	RISC_ENABLE = 1;  

	bMustBeRunning = TRUE;
}

void RISCEng::Stop( void )
{
	RISC_ENABLE = 0; 
	FIFO_ENABLE = 0; 

	bMustBeRunning = FALSE;
}


/* Method: RISCEng::CreateProgram
 * Purpose: Creates a RISC program for a video/VBI field
 * Input:
 * Output: RiscPrgHandle
 */

RiscPrgHandle RISCEng::CreateProgram(
		Field*			OurField,		// The field associated to this RISC program
		const RISCProgram::SMode RStat,	// Action to perform on RISCS (RISC Status)
		const bool		FireInt)		// If we must fire an int at the end of this program

{
	// We must have a field associated
	ENSURE(OurField);

	// Allocate memory for this program
	LPDWORD dwLinearAddress = AllocSpace( !(OurField->IsVBI()) ); /* We use the VBI format to create Skippers and VBI fields */

	// create yet another risc program object
	RISCProgram *YAProgram = 
		new RISCProgram(
				HwDeviceExtension, dwLinearAddress,
				OurField ,
				RStat,
				FireInt);
	
	// and make it create the program itself
	if (YAProgram) YAProgram->Create(FieldDataBuf());

	// Return the newly created program
	return YAProgram;
}



/* Method: RISCEng::CreateProgram
 * Purpose: Creates a RISC program for a SYNC AND JUMP program
 * Input:
 * Output: RiscPrgHandle
 */

RiscPrgHandle RISCEng::CreateProgram(
		const SyncCode SyncBits ,
		const RISCProgram::SMode RStat,	// Action to perform on RISCS (RISC Status)
		const bool		FireInt)		// If we must fire an int at the end of this program

{
	// Allocate memory for this program
	LPDWORD dwLinearAddress = AllocSpace( false ); 

	// create yet another risc program object
	RISCProgram *YAProgram = new RISCProgram(HwDeviceExtension, dwLinearAddress , SyncBits , RStat , FireInt );

	// and make it create the program itself
	if (YAProgram) YAProgram->Create( ) ;

	return YAProgram;
}

/* Method: RISCEng::DestroyProgram
 * Purpose: Removes a program from a chain and destroy it
 * Input: ToDie: RiscPrgHandle - pointer to a program to destroy
 * Output: None
 * Botes: RISC engine must be off!
 */
void RISCEng::DestroyProgram( RiscPrgHandle ToDie )
{
	LPDWORD Addr = ToDie->GetProgAddress();

	// Delete the program
	delete ToDie;

	// Release memory allocated to that program
	FreeSpace(Addr);
}

/* Functions to alloc and release memory for RISC preograms */
LPDWORD RISCEng::AllocSpace(const bool IsVideoProg )
{
	TRACE(("RISCEng::AllocSpace"));

	DWORD_PTR dwLinAddr = 0;

	if (IsVideoProg) {
		// Video Program ...

		// Search for a free block
		for (
			UINT i = 0;
			i < sizeof(bFieldBlkIsAlloc) / sizeof(bFieldBlkIsAlloc[0]) ;
			i ++ ) {

			// Found a free block ?
			if (!bFieldBlkIsAlloc[i]) {
				// Yes! 

				// Remember we have allocated this block
				bFieldBlkIsAlloc[i] = true;

				// Calculate the begining address of this block
				dwLinAddr = MaxVidSize * i + DWORD_PTR(pRISCMem) + VideoOffset;
				
				DUMP(("Got Field RISC Program Buffer at %x\n",dwLinAddr));

				// return 
				break;
			}
		}

	} else {
		// VBI or Skipper RISC Program ... 

		// Search for a free block
		for (
			UINT i = 0;
			i < sizeof(bVBIBlkIsAlloc) / sizeof(bVBIBlkIsAlloc[0]) ;
			i ++ ) {

			// Found a free block ?
			if (!bVBIBlkIsAlloc[i]) {
				// Yes! 

				// Remember we have allocated this block
				bVBIBlkIsAlloc[i] = true;

				// Calculate the begining address of this block
				dwLinAddr = MaxVBISize * i + DWORD_PTR(pRISCMem);
				
				DUMP(("Got VBI RISC Program Buffer at %x\n",dwLinAddr));

				// return 
				break;
			}
		}

	}

	ENSURE(dwLinAddr);

	return LPDWORD(dwLinAddr);
}

void RISCEng::FreeSpace( const LPDWORD Addr )
{
	TRACE(("RISCEng::FreeSpace"));
	DUMP(("Address:  %x\n",Addr));

	// Ensure pointer is valid!
	ENSURE ( DWORD_PTR(Addr) >= DWORD_PTR(pRISCMem) && DWORD_PTR(Addr) <= DWORD_PTR(pRISCMem) + RISCProgramsSize );

	// First, translate pointer to a relative way
	DWORD_PTR dwOff = DWORD_PTR(Addr) - DWORD_PTR(pRISCMem);

	if (dwOff >= VideoOffset ) {

		// Must be a video program
		ENSURE( (dwOff - VideoOffset) % MaxVidSize == 0 );

		// Must be a Video Program ...
		UINT_PTR Idx = (dwOff - VideoOffset) / MaxVidSize;

		ENSURE ( bFieldBlkIsAlloc[Idx] == true );

		// Free that block!
		bFieldBlkIsAlloc[Idx] = false;

	} else {
		// Must be a VBI program ...
		ENSURE( (dwOff) % MaxVBISize == 0 );

		// Calc index
		UINT_PTR Idx = (dwOff) / MaxVBISize;

		ENSURE ( bVBIBlkIsAlloc[Idx] == true );

		// Free that block!
		bVBIBlkIsAlloc[Idx] = false;

	}

}
