/*
	RiscEng.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __RISCENG_HPP
#define __RISCENG_HPP

#include "RiscProg.hpp"
#include "CompReg.hpp"

typedef RISCProgram *RiscPrgHandle;

/* Class: RISCEng
 * Purpose: This class provides control over the BtPisces' RISC engine.
 * It also creates and destroys RISC programs
 * Attributes:
 * Operations:
 */                
class RISCEng
{
	protected:

		// Needed to be able to Get address of DMA buffer for programs
		const PVOID HwDeviceExtension;

		// Pointer to address of Buffer of RISC programs
		const PBYTE pRISCMem;

		// Array of indices of blocks allocated,
		bool bVBIBlkIsAlloc  [8];				// Total number of RISC program blocks for VBI and Sync
		bool bFieldBlkIsAlloc[4];				// Total number of RISC program blocks for Video Fields

		// Registers
		RegisterDW RISC_IPC;

		RegisterDW Control; 
		RegField FIFO_ENABLE; 
		RegField RISC_ENABLE; 
		RegField PKTP; 
		RegField PLTP1; 
		RegField PLTP23; 
		RegField GPCLKMODE; 
		RegField GPIOMODE; 
		RegField GPWEC; 
		RegField GPINTI; 
		RegField GPINTC;

		// To know if Risc engine should be running
		bool bMustBeRunning;

	public:
		// To create a new Video/VBI field program
		RiscPrgHandle CreateProgram(
				Field*						OurField,			// The field associated to this RISC program
				const RISCProgram::SMode	RStat = RISCProgram::Sm_None,	// Action to perform on RISCS (RISC Status)
				const bool		FireInt = false								// If we must fire an int at the end of this program
				);


		// To create a new SYNC program
		RiscPrgHandle CreateProgram(
				const SyncCode SyncBits ,
				const RISCProgram::SMode	RStat = RISCProgram::Sm_None,	// Action to perform on RISCS (RISC Status)
				const bool		FireInt = false								// If we must fire an int at the end of this program
				);
				
		// To Change data Buffers for Field programs!
		void SetFieldDataBuf( RiscPrgHandle prg, const FieldDataBuf& pBuf );
		const FieldDataBuf& GetFieldDataBuf(RiscPrgHandle prg ) { return prg->GetFieldDataBuf(); }

		// Destroys programs and all associated resources	
		void DestroyProgram( RiscPrgHandle ToDie );

		// Start with the execution of this program
		void Start( RiscPrgHandle ToStart );

		// Stop execution of programs!
		void Stop( ); 

		// Check if RISC engine is running actually
		bool IsRunning();

		// Check if RISC engine SHOULD be running
		bool MustBeRunning() const { return bMustBeRunning; }

		// Chain a program to another
		void Chain( RiscPrgHandle hPrev, RiscPrgHandle hNext );

		// Unlink the program from chain
		void Unlink( RiscPrgHandle hprg );

		RISCEng( 
			const PVOID HwDeviceExtension_,
			const DWORD_PTR dwBase ) ;

		~RISCEng();

	private:

		// Functions to alloc and release memory for RISC preograms
		LPDWORD AllocSpace( const bool IsFieldProg );
		void FreeSpace( const LPDWORD Addr );

};

inline bool RISCEng::IsRunning()
{
	return (RISC_ENABLE && FIFO_ENABLE);
}

/*
 * To change data buffer
 */
inline void RISCEng::SetFieldDataBuf( 
		RiscPrgHandle prg,					  
		const FieldDataBuf& pBuf		// Data Buffer descriptor
		) 
{
#if DBG

	// Ensure we have a video field
	ENSURE(prg);
	if (prg) {
		ENSURE(prg->GetField());
	}
#endif

	// Recreate the program with the associated buffer
	prg->Create( pBuf );
}


/* Method: RISCEng::Chain
 * Purpose: Chains two RISC programs together
 * Output: None
 */
inline void RISCEng::Chain( RiscPrgHandle hPrev, RiscPrgHandle hNext )
{
	DUMP(("*** Linked hPrev(%x) to hNext(%x)\n", hPrev, hNext));
	hPrev->ChainTo( hNext );
}

/* Method: RISCEng::Unlink
 * Purpose: Unlinks this programs
 * Output: None
 */
inline void RISCEng::Unlink( RiscPrgHandle hprg )
{
	DUMP(("*** Unlinked hProg(%x)\n",hprg));
	hprg->Unlink();
}


#endif
