/*
	RiscProg.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "riscprog.hpp"

// #define BREAK_IN_PAGES /* Just a test... unneeded */

inline DWORD RISCProgram::GetDMAPhysAddr(const PVOID HwDeviceExtension_,const LPDWORD dwLinearAddress_)
{
	DWORD Len;
	return StreamClassGetPhysicalAddress( HwDeviceExtension_, NULL, dwLinearAddress_ , DmaBuffer, &Len ).LowPart;
}

// To generate Data Programs to capture Video or VBI info - This programs do generate ints!
RISCProgram::RISCProgram( 
	// HdExt (Needed to get Physical address
	const PVOID		HwDevExt,
	// Linear address of program
	const LPDWORD	dwLinAddr,
	// Field associated to this program
	Field*			OurField,
	// Action to perform on RISC status
	const SMode		RStat,
	// If we must fire an int at the end of this program
	const bool		FireInt
	) :
	RISCSMode_(RStat),
	bInterrupt_(FireInt),
	HwDeviceExtension_(HwDevExt) ,
	dwLinearAddress_(dwLinAddr) ,
	dwPhysicalAddress_( GetDMAPhysAddr( HwDevExt, dwLinAddr ) ) ,
	dwSize_(0),
	pChainAddress_( dwLinAddr ), 
	SyncBits_ ( SyncCode(0) ),
	pDataBuf_( ),
	OurField_( OurField ),
	pNext_( NULL )
{
	TRACE(("RISCProgram::RISCProgram"));
	DUMP(("Video/VBI: this = %x, Linear Address: %x , Physical Address: %x\n",this, dwLinearAddress_,dwPhysicalAddress_));

	ENSURE(dwLinearAddress_);
	ENSURE(dwPhysicalAddress_);
	ENSURE(OurField_);
}                                      

// For SYNC programs
// Note that we MUST NOT generate a SYNC program for a given field if we are not capturing data on that field!
RISCProgram::RISCProgram(
	// HdExt (Needed to get Physical address
	const PVOID		HwDevExt,
	// Linear address of program
	const LPDWORD	dwLinAddr,
	// The Sync codes we will generate! 
	const SyncCode	SyncBits,
	// Action to perform on RISC status
	const SMode		RStat,
	// If we must fire an int at the end of this program
	const bool		FireInt

 ) :
	RISCSMode_(RStat),
	bInterrupt_(FireInt),
	HwDeviceExtension_(HwDevExt) ,
	dwLinearAddress_(dwLinAddr) ,
	dwPhysicalAddress_( GetDMAPhysAddr( HwDevExt, dwLinAddr ) ) ,
	dwSize_(0),
	pChainAddress_( dwLinAddr ),
	SyncBits_ ( SyncBits ),	
	pDataBuf_( ),
	OurField_( NULL ),
	pNext_( NULL )
	
{
	TRACE(("RISCProgram::RISCProgram(2)"));
	DUMP(("SYNC this = %x, Linear Address: %x , Physical Address: %x\n",this, dwLinearAddress_,dwPhysicalAddress_));
	
	ENSURE(dwLinearAddress_);
	ENSURE(dwPhysicalAddress_);
}

RISCProgram::~RISCProgram()
{
	TRACE(("RISCProgram::~RISCProgram"));
}

//
// Dump goes to the debugger
//
#if DBG
void RISCProgram::Dump( )
{
	DUMP(("RiscProgram(%x) ProgAddr(%x) PhysProgAddr(%x)\n",
                 this,
                 GetProgAddress( ),
                 GetPhysProgAddr( )));

	DWORD* pProgLoc = (DWORD*) GetProgAddress( );
	DWORD dwTmpSize_ = dwSize_;

	DUMP(("size = %d\n", dwSize_ ));
	DUMP(("phys addr = %x\n", GetPhysProgAddr( )));

	PULONG pulProg = (PULONG) (GetProgAddress());
	while( dwTmpSize_ >= 4 * 4) {
		DUMP(("%08x %08x %08x %08x\n",
                pulProg[0],
                pulProg[1],
                pulProg[2],
                pulProg[3]));
		pulProg += 4;
		dwTmpSize_ -= 4 * 4; // In bytes!!!
	}
	switch( dwTmpSize_ ) {
		case 3 * 4:
			DUMP(("%08x %08x %08x\n",
                   pulProg[0],
                   pulProg[1],
                   pulProg[2]
                   ));
			break;
		case 2 * 4:
			DUMP(("%08x %08x\n",
                   pulProg[0],
                   pulProg[1]
                   ));
			break;
		case 1 * 4:
			DUMP(("%08x\n",
                   pulProg[0]
                   ));
			break;
	}
}   

#endif

/* Method: RISCProgram::Create
 * Purpose: Creates a simple SYNC and JUMP program. Useful to syncronize to
 * start of even or odd field
 */

void RISCProgram::Create( )
{
	TRACE(("RISCProgram::Create(SyncCode)"));

	USES_RISCCREATOR; // We will create RISC instructions here...

	// that's were the instructions are going
	LPDWORD pProgLoc = GetProgAddress();

	// put one of the VRx codes here:
	// We will always allow reSync to the start of an Even/Odd Field
	RISC_SYNC(pProgLoc,SyncBits_,1,0,0); 

	// Remember the address of the last instruction
	pChainAddress_ = pProgLoc; 
	
	// Reestablish chaining
	Rechain();
}

/* Method: RISCProgram::Create
 * Purpose: Creates a RISC program for a Video Field
 * Input: buf = Buffer to use
 * Output: None
 * Note: It is likely this function is used to simply change dst addresses of
 *   an already existing program. It does not seem to make much sense to write
 *   basically the same function ( or the one that has to parse existing program)
 *   to change addresses.
 */

/* Note: this is called at IRQL = DISPATCH_LEVEL */
void RISCProgram::Create( const FieldDataBuf& pBuf ) // Srb with the buffer to use and other goodies
{
	TRACE(("RISCProgram::Create"));
	USES_RISCCREATOR; // We will create RISC instructions here...

	// We MUST have associated a field (Be a VIDEO/VBI program)
	ENSURE(OurField_);

	// Remember the DataBuf where it is our data buffer 
	// (We will need it on the interrupt). Keep a local copy of the info!
	// This buffer can be NULL if we don't have a buffer right now!
	pDataBuf_ = pBuf;

	// that's were the instructions are going
	LPDWORD pProgLoc = (LPDWORD) GetProgAddress();

	// Avoid GPF - ing!
	// Only if our field is enabled, generate the SYNC instruction and the program!
	// (else the program is a NO-OP)
	if ( OurField_ ) {

		// By default, do not create an skipper
		bool CreateSkipper = false;

		// If no buffer, Create Skipper !
		if ( !pBuf.RealBuffer() ) {
			DUMP(("No real buffer, creating Skipper\n"));
			CreateSkipper = true;
		} else {
			DUMP(("Buffer Address = %x\n", DWORD( pBuf.GetBufAddr() ) ));
			ULONG Len;
			// If we can't get the physical addr, fail!
			if ( !pBuf.GetBufPhysAddr( 0 , Len) ) {
				ERROR(("No Physical Address, creating Skipper\n"));
				CreateSkipper = true;
			}
		}

		
		// Used to interleave/subsample for each image data format
		// By default, do not flip image, and assume we will generate a planar image
		BYTE SizeDivider = 0;
		BYTE PitchDivider  = 0;
		bool Flip   = false;
		bool Planar = true;
		BYTE AlternateSwitch = 0;
		BYTE Offset = 0;

		// prepare all the ugly things
		switch ( OurField_->GetColorFormat() ) {
			case CF_RGB32:	case CF_RGB24:	case CF_RGB16: 
			case CF_RGB15:	case CF_BTYUV:	case CF_RGB8:
				Flip = true; // all this formats are flipped (upside-down) by definition
			case CF_Y8:		case CF_YUY2:
			case CF_UYVY:	case CF_RAW:
			case CF_VBI:

				// packed data to follow
				Planar = false;
				break;

			case CF_PL_422:
				SizeDivider = 1;
				PitchDivider  = 1;
				break;

			case CF_PL_411:
				SizeDivider = 2;
				PitchDivider  = 2;
				break;

			case CF_YUV9:
				Offset = 2; // Sample Chroma at the 3rd line in YUV9
				AlternateSwitch = 3;
				SizeDivider = 4; // 2^4 = 16
				PitchDivider  = 2;
				break;

			case CF_I420:
			case CF_YUV12:
				AlternateSwitch = 1;
				SizeDivider = 2;
				PitchDivider  = 1;
	
		} /*endswitch*/

		// Get image dimensions
		const DWORD dwImgSizeX = DWORD(OurField_->GetImageSize().cx);
		const DWORD dwImgSizeY = DWORD(OurField_->GetImageSize().cy);
	
		DUMP(("Offset    = %d\n",pBuf.GetBufOffset() ));
		DUMP(("Pitch     = %d\n",pBuf.GetBufPitch() ));
		DUMP(("ColorFmt  = %d\n",OurField_->GetColorFormat() ));
		DUMP(("BPP		 = %d\n",OurField_->GetBitCount() ));
		DUMP(("ImageSize = (%d,%d)\n",dwImgSizeX,dwImgSizeY ));

		// Calculate pitchs and sizes and starting offsets to use
		LONG lYPlanePitch  = pBuf.GetBufPitch();
		LONG lYPlaneSize   = dwImgSizeY * lYPlanePitch;
		LONG lYPlaneOffset = (LONG)pBuf.GetBufOffset(); // Normally, can be 0 or dwPitch / 2 for interleaved

		DUMP(("pProgLoc = %x\n",pProgLoc));
	
		if ( Planar ) {

			// put the Sync code to start the Planar image data program
			RISC_SYNC(pProgLoc,SC_FM3,0,0,0); // Dont allow Resync

			// Ensure an image is to be created!
			if (dwImgSizeX) {
	
				// Planar modes ... Call the appropiate helper to create the right program
				if (CreateSkipper) {

					// Planar Skipper

					pProgLoc = CreatePlanarSkipper(pProgLoc,
						dwImgSizeX,dwImgSizeY,
						PitchDivider);

				} else {

					// Planar NonSkipper

					// Calculate starting points for each plane
					LONG lChromaPitch  = lYPlanePitch >> PitchDivider;
					LONG lChromaSize   = lYPlaneSize >> SizeDivider;
					LONG lUPlaneOffset = lYPlaneSize + (lYPlaneOffset >> PitchDivider);
					LONG lVPlaneOffset = lUPlaneOffset + lChromaSize;

					// U goes first for this color format
					if ( OurField_->GetColorFormat() == CF_I420 ) {
						LONG lTmp = lUPlaneOffset;
						lUPlaneOffset = lVPlaneOffset;
						lVPlaneOffset = lTmp;
					}

					pProgLoc = CreatePlanarNonSkipper(pProgLoc,pBuf,
						dwImgSizeX,dwImgSizeY,
						PitchDivider,AlternateSwitch,
						lYPlaneOffset,lVPlaneOffset,lUPlaneOffset,
						lYPlanePitch,lChromaPitch,Offset);
				}
			}

		} else {
			// Non - Planar modes

			// put the Sync code to start the Non-Planar image data program
			RISC_SYNC(pProgLoc,SC_FM1,0,0,0); // Dont allow Resync

			// Ensure an image is to be created!
			if (dwImgSizeX) {

				if (CreateSkipper) {

					// NonPlanar Skipper
					pProgLoc = CreateNonPlanarSkipper(pProgLoc,
						DWORD(dwImgSizeX * OurField_->GetBitCount() / 8 ),
						dwImgSizeY); 

				} else {

					// NonPlanar NonSkipper

					// if flipped, adjust the pointers (Only for RGB!)
					if ( Flip ) {
						lYPlaneOffset += lYPlaneSize - lYPlanePitch;
						lYPlanePitch = -lYPlanePitch;
					}

					pProgLoc = CreateNonPlanarNonSkipper(pProgLoc,pBuf,
						DWORD(dwImgSizeX * OurField_->GetBitCount() / 8 ),
						dwImgSizeY,lYPlaneOffset,lYPlanePitch);
				}
			} /* endif */
		}
	}
	
	// Remember address where we must put a JUMP to chain this program to another
	pChainAddress_ = pProgLoc;

	// Reestablish chaining 
	Rechain();

}


DWORD* RISCProgram::CreatePlanarNonSkipper(
	DWORD* pProgLoc,const FieldDataBuf& pBuf,
	DWORD dwImgSizeX,DWORD dwImgSizeY,
	BYTE PitchDivider,BYTE AlternateSwitch,
	LONG lYOffset,LONG lVOffset,LONG lUOffset,
	LONG lYPitch,LONG lChromaPitch,
	BYTE Offset)
{
	TRACE(("RISCProgram::CreatePlanarNonSkipper"));
	USES_RISCCREATOR; // We will create RISC instructions here...

	// Planar (No Skipper)

	// now go into a loop (up to the hight of the image) and create
	// a command for every line. Commands depend on the data format
	for (DWORD i = 0; i < dwImgSizeY ; i++ ) {

		// DUMP(("Cycle %d\n",i)); // Too much information for me !!

		// temps for the loop
		LONG lYPlane = lYOffset; // [0]
		ULONG WidthToDo = dwImgSizeX;

		// now take care of vertically sub-sampled planar modes (Only if not creating an Skipper)
		if ( (BYTE(i & 0xFF) & AlternateSwitch) == Offset ) {

			LONG lVPlane = lVOffset; // [1] Cb
			LONG lUPlane = lUOffset; // [2] Cr

			while(1) {
	
				// Get physical address and count of bytes remaining for each plane

				ULONG blY = WidthToDo;
				DWORD_PTR dwAddresses0 = pBuf.GetBufPhysAddr(lYPlane,blY);

#ifdef BREAK_IN_PAGES
				// Get the remaining length to the end of page.
				DWORD dwRemaining0 = 4096 - (DWORD(pBuf.GetBufAddr(lYPlane)) & 4095);
				// Make sure we dont cross a page without splitting.
				blY = (blY > dwRemaining0) ? dwRemaining0 : blY;
#endif

				ULONG blV;
				DWORD_PTR dwAddresses1 = pBuf.GetBufPhysAddr(lVPlane,blV);

#ifdef BREAK_IN_PAGES
				// Get the remaining length to the end of page.
				DWORD dwRemaining1 = 4096 - (DWORD(pBuf.GetBufAddr(lVPlane)) & 4095);
				// Make sure we dont cross a page without splitting.
				blV = (blV > dwRemaining1) ? dwRemaining1 : blV;
#endif
				
				// Make all planes measure in the same units and get the smallest count!	
				blV <<= PitchDivider;
				blY = (blV < blY)  ? blV : blY;

				ULONG blU;
				DWORD_PTR dwAddresses2 = pBuf.GetBufPhysAddr(lUPlane,blU);

#ifdef BREAK_IN_PAGES
				// Get the remaining length to the end of page.
				DWORD dwRemaining2 = 4096 - (DWORD(pBuf.GetBufAddr(lUPlane)) & 4095);
				// Make sure we dont cross a page without splitting.
				blU = (blU > dwRemaining2) ? dwRemaining2 : blU;
#endif

				// Make all planes measure in the same units and get the smallest count!
				blU <<= PitchDivider;
				blY = (blU < blY)  ? blU : blY;

				// Ensure less than bytes to do!
				blY = (blY > WidthToDo) ? WidthToDo : blY;
	
				// Avoid infinite loop and GPFing when Physical addr. not available or Offset out of buffer!
				if (! (blY >> PitchDivider) ) return pProgLoc;

				// Create the instruction!
				RISC_WRITE123(
					pProgLoc,
					dwAddresses0,blY, 
					dwAddresses1,(blY >> PitchDivider),
					dwAddresses2,(blY >> PitchDivider),
					0,0,0,
					(WidthToDo == blY),(WidthToDo == dwImgSizeX));

				// Calculate bytes remaining
				WidthToDo -= blY;
				
				// If ended, finish!
				if (!WidthToDo) break;

				// Increment the address of those planes
				lYPlane += blY;
				lVPlane += (blY >> PitchDivider);
				lUPlane += (blY >> PitchDivider);
			};
				
			//  Advance the address offset to the next scan line
			lVOffset += lChromaPitch;
			lUOffset += lChromaPitch;

		} else {

			while(1) {
	
				// Get physical address and count of bytes remaining for each plane
				ULONG blY;
				DWORD_PTR dwAddress = pBuf.GetBufPhysAddr(lYPlane,blY);

#ifdef BREAK_IN_PAGES
				// Get the remaining length to the end of page.
				DWORD dwRemaining = 4096 - (DWORD(pBuf.GetBufAddr(lYPlane)) & 4095);
				// Make sure we dont cross a page without splitting.
				blY = (blY > dwRemaining) ? dwRemaining : blY;
#endif

				// Ensure less than bytes to do!
				blY = (blY > WidthToDo) ? WidthToDo : blY;
	
				// Avoid infinite loop and GPFing when Physical addr. not available or Offset out of buffer!
				if (! blY ) return pProgLoc;

				// Create the instruction!
				RISC_WRITE1S23(
					pProgLoc,
					dwAddress,
					blY,blY >> PitchDivider,blY >> PitchDivider,
					0,0,0,
					(WidthToDo == blY),(WidthToDo == dwImgSizeX));
				
				// blY now containts the longest row that can be written
				WidthToDo -= blY;

				// If no more to do, finish!
				if (!WidthToDo) break;

				// Increment the address of those planes
				lYPlane += blY;
			};
				
		}

		lYOffset += lYPitch;

	} /* endfor */
	return pProgLoc;
}

DWORD* RISCProgram::CreatePlanarSkipper(DWORD* pProgLoc,
	DWORD dwImgSizeX,DWORD dwImgSizeY,
	BYTE PitchDivider)
{
	TRACE(("RISCProgram::CreatePlanarSkipper"));
	USES_RISCCREATOR; // We will create RISC instructions here...

	// Planar (Skipper)

	// Cache the count
	DWORD Chroma = dwImgSizeX >> PitchDivider;

	// now go into a loop (up to the hight of the image) and create
	// a command for every line. Commands depend on the data format
	for (DWORD i = 0; i < dwImgSizeY ; i++ ) {
		// DUMP(("Cycle %d\n",i)); // Too much information for me !!
		// Create the instruction!
		RISC_SKIP123(
			pProgLoc,
			dwImgSizeX,Chroma,Chroma,
			0,0,1,1);
					
	} /* endfor */
	return pProgLoc;
}

DWORD* RISCProgram::CreateNonPlanarNonSkipper(DWORD* pProgLoc,const FieldDataBuf& pBuf,
	ULONG Start,DWORD dwImgSizeY,
	LONG lPlaneOffset,LONG lPlanePitch)
{
	TRACE(("RISCProgram::CreateNonPlanarNonSkipper"));
	USES_RISCCREATOR; // We will create RISC instructions here...

	// Non-Planar, Non-Skipper

	// ULONG Start = (WORD)(wImgSizeX * OurField_->GetBitCount() / 8 );
	
	// now go into a loop (up to the height of the image) and create
	// a command for every line. Commands depend on the data format

	for (DWORD i = 0; i < dwImgSizeY ; i++ ) {

		// DUMP(("Cycle %d\n",i)); // Too much information for me !!

		// Split in all the required instructions!
		ULONG WidthToDo = Start;
		LONG lPlane = lPlaneOffset;
		
		while (1) {

			// Get physical address and count of bytes remaining for plane
			ULONG bl;
			DWORD_PTR dwAddress = pBuf.GetBufPhysAddr(lPlane,bl);

#ifdef BREAK_IN_PAGES
			// Get the remaining length to the end of page.
			DWORD dwRemaining = 4096 - (DWORD(pBuf.GetBufAddr(lPlane)) & 4095);
			// Make sure we dont cross a page without splitting.
			bl = (bl > dwRemaining) ? dwRemaining : bl;
#endif

			// Make sure we don't place too much data
			bl = (bl > WidthToDo) ? WidthToDo : bl;

			// Avoid infinite loop and GPFing when Physical addr. not available or Offset out of buffer!
			if (!bl) return pProgLoc;

			// Create the instruction
			RISC_WRITE(
				pProgLoc,
				dwAddress,bl,
				0,0,0,
				(WidthToDo == bl),(WidthToDo == Start));

			// bl now containts the longest row that can be written
			WidthToDo -= bl;

			// If no more to do, finish
			if (!WidthToDo) break;

			// Increment address of plane
			lPlane += bl;
		}

		lPlaneOffset += lPlanePitch;
	
	} /* endfor */
	return pProgLoc;
}

DWORD* RISCProgram::CreateNonPlanarSkipper(
		DWORD* pProgLoc,ULONG Start,DWORD dwImgSizeY)
{
	TRACE(("RISCProgram::CreateNonPlanarSkipper"));
	USES_RISCCREATOR; // We will create RISC instructions here...

	// Non-Planar, Skipper

	// ULONG Start = (WORD)(wImgSizeX * OurField_->GetBitCount() / 8 );

	// now go into a loop (up to the hight of the image) and create
	// a command for every line. Commands depend on the data format
	for (DWORD i = 0; i < dwImgSizeY ; i++ ) {

		// DUMP(("Cycle %d\n",i)); // Too much information for me !!

		// Create the instruction
		RISC_SKIP(
			pProgLoc,
			Start,
			0,0,1,1);

	} /* endfor */
	return pProgLoc;
}


/* Method: RISCProgram::ChainTo
 * Purpose: Chains this program to the next one
 * Output: None
 */
void RISCProgram::ChainTo( RISCProgram *to )
{
	TRACE(("RISCProgram::ChainTo"));

	ENSURE ( to );

	// now we know where we are chaining to
	pNext_ = to;
	
	// create the chaining
	Rechain();
}

void RISCProgram::Unlink( void )
{
	TRACE(("RISCProgram::Unlink"));
	pNext_ = NULL;

	// create the chaining
	Rechain();
}

/* Method: RISCProgram::Rechain()
 * Purpose: Creates the last jump of a program that chains to the next
 *			The rechaining is from us to another one.
 */
inline void RISCProgram::Rechain()
{
	USES_RISCCREATOR; // We will create RISC instructions here...

	// Get Addr where to put the chaining sequence
	LPDWORD pProgLoc = pChainAddress_;

	// Get the start address!
	LPDWORD pProgStart = GetProgAddress();
	
	// Get the program we will jump to...
	RISCProgram* pDest = (pNext_) ? pNext_ : this;

	// Reestablish chaining to the next program
	// Use the jump to fire an INT if the current program
	// needs to fire it, and set the RISCStatus to the one of the next
	// program. All of this by using the JUMP...

	RISC_JUMP(pProgLoc, DWORD(pDest->GetPhysProgAddr()) , RISCSMode_, bInterrupt_);

	// Remember size of this program
    dwSize_ = (DWORD)(DWORD_PTR(pProgLoc) - DWORD_PTR(pProgStart));
}

/*
/* Method: RISCProgram::Interrupt()
 * Purpose: Prepare this program for the next execution
 */

/* Note: this is called at IRQL = DISPATCH_LEVEL */
void RISCProgram::Interrupt()
{
	TRACE(("RISCProgram::Interrupt"));

	if (!IsSyncPrg())  {

		DUMP(("VIDEO/VBI Program\n"));

		// We MUST have associated a field (Be a VIDEO/VBI program)
		ENSURE(OurField_);

		// Get a copy of the buffer descriptor to be able to return it
		FieldDataBuf dDone = pDataBuf_;

		// Now, get a new Buffer if Possible, creating a new program to digitize to it
		// Doing this in this way, we will be sure this program won't modify the (already filled) buffer
		Create(OurField_->GetNextBuffer());

		// If we had a buffer to fill ...
		if (dDone.RealBuffer() ) {
			// Signal the associated field the already filled buffer is processed
			OurField_->BufferDone(dDone);
		}

	}
#if DBG	
	else {
		DUMP(("SYNC Program\n"));
	}
#endif

}

/* Recreates programs from scratch (To restart them - Shouldnt be necesary, though)
 */
void RISCProgram::Recreate()
{
	TRACE(("RISCProgram::Recreate"));

	// If we are a VIDEO/VBI program, dispatch buffers!
	if (IsSyncPrg())  {

		DUMP(("SYNC Program\n"));

		// Recreate Sync based on Field enabling
		Create();

	} else {

		DUMP(("VIDEO/VBI Program\n"));

		// Now, recreate the program
		Create(pDataBuf_);
	}

}
