/*
	RiscProg.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __RISCPROG_H
#define __RISCPROG_H

#include "debug.hpp"
#include "CppRt.hpp"
#include "MyTypes.hpp"
#include "colspace.hpp"
#include "command.hpp"
#include "Field.hpp"

/* Class: RISCProgram
 * Purpose: Facilitates creation and maintenance of a RISC programs
 * Attributes: ImageSize_: SIZE - structure containing dimentions of the image
 */

class RISCProgram
{
	public:

		// Type of action to perform on RISCS
		typedef enum {
			Sm_None   = 0x00,	// No action
			Sm_SetTo0 = 0xF0,	// Set it to 0
			Sm_SetTo1 = 0xE1,	// Set it to 1
			Sm_SetTo2 = 0xD2,	// Set it to 2
			Sm_SetTo3 = 0xC3,	// Set it to 3
			Sm_SetTo4 = 0xB4,	// Set it to 4
			Sm_SetTo5 = 0xA5,	// Set it to 5
			Sm_SetTo6 = 0x96,	// Set it to 6
			Sm_SetTo7 = 0x87,	// Set it to 7
			Sm_SetTo8 = 0x78,	// Set it to 8
			Sm_SetTo9 = 0x69,	// Set it to 9
			Sm_SetToA = 0x5A,	// Set it to A
			Sm_SetToB = 0x4B,	// Set it to B
			Sm_SetToC = 0x3C,	// Set it to C
			Sm_SetToD = 0x2D,	// Set it to D
			Sm_SetToE = 0x1E,	// Set it to E
			Sm_SetToF = 0x0F,	// Set it to F
			Sm_Inc	  = 0xFF	// Increment counter
		} SMode;

	private:
		// Address (Linear and Physical) of this program
		const LPDWORD			dwLinearAddress_;
		const DWORD				dwPhysicalAddress_;

		// Size of the program
		DWORD					dwSize_;		

		// Hardware device Extension: Is needed to get the physical Address of the program
		const PVOID				HwDeviceExtension_;

		// Address (linear) of the instruction in this program that must be modified to make
		// this program jump to another program (to chain the other program to this)
		PDWORD					pChainAddress_; 

		// Object describing the data Buffer and other associated info
		FieldDataBuf			pDataBuf_; 

		// Pointer to the field associated to this program (if it is)
		Field*					OurField_;

		// Sync code we should generate (For Sync programs)
		const SyncCode			SyncBits_;

		// Type of action to perform on RISCS (For Data programs)
		const SMode				RISCSMode_;

		// Vars to maintain the linked list
		RISCProgram *			pNext_;

		// This one is to make the program interrupt.
		bool					bInterrupt_;

	private:

		// Create a new skipper program : Destroys the old one
		void					Create( );

		// Create a videofield program : Destroys the old one
		void					Create(	const FieldDataBuf& buf );

   		// To get the data buffer where we will put our digitized field/data
		const FieldDataBuf&		GetFieldDataBuf( void ) const { return pDataBuf_; }
		
		// Program attributes

	public:
		bool		 IsSyncPrg() const { return OurField_ == NULL; }

		// Get Program size
		ULONG        GetProgSize() const { return dwSize_; }

		// Get program linear address
		LPDWORD      GetProgAddress() const { return dwLinearAddress_; };

		// Get program physical address
		DWORD		 GetPhysProgAddr() const { return dwPhysicalAddress_; }

		// chaining/unchaining group :

		// Next is the program that will be executed after us. (we jump to it)
		RISCProgram *GetNext() const { return pNext_; }

	private:

		void         ChainTo( RISCProgram *To );
		void		 Unlink( void );

	public:

		// To Know the field associated to Video/Data programs
		const Field* GetField( void ) const { return OurField_; }

		// Called when this Program has interrupted us!
		void		 Interrupt();

#ifdef DBG
		void         Dump();
#endif

	protected:

		// For RISC programs for videofields: Must have associated a field
		RISCProgram( 
			// HdExt (Needed to get Physical address
			const PVOID		HwDevExt,
			// Linear address of program
			const LPDWORD	dwLinAddr,
			// Field associated to this program
			Field*			OurField,
			// Action to perform on RISCS (RISC Status)
			const SMode		RStat = Sm_None,
			// If we must fire an int at the end of this program
			const bool		FireInt = false
			);

		// For SYNC programs
		RISCProgram( 
			// HdExt (Needed to get Physical address
			const PVOID		HwDeviceExtension_,
			// Linear address of program
			const LPDWORD	dwLinearAddress_,
			// The Sync codes we will generate! 
			const SyncCode	SyncBits,
			// Action to perform on RISCS (RISC Status)
			const SMode		RStat = Sm_None,
			// If we must fire an int at the end of this program
			const bool		FireInt = false
			);

		~RISCProgram( );

		friend class RISCEng;			// Only this can build us

	public:

		// To recreate programs for Restart!
		void Recreate();

	private:

		// Helpers
		void Rechain();
		
		// Convenience funtion
		static DWORD GetDMAPhysAddr(const PVOID HwDeviceExtension_,const LPDWORD dwLinearAddress_);

		// Creation Helpers
		static DWORD* CreatePlanarNonSkipper(
			DWORD* pProgLoc,const FieldDataBuf& pBuf,
			DWORD dwImgSizeX,DWORD dwImgSizeY,
			BYTE PitchDivider,BYTE AlternateSwitch,
			LONG lYOffset,LONG lVOffset,LONG lUOffset,
			LONG lYPitch,LONG lChromaPitch,
			BYTE Offset);

		static DWORD* CreatePlanarSkipper(DWORD* pProgLoc,
			DWORD dwImgSizeX,DWORD dwImgSizeY,
			BYTE PitchDivider);

		static DWORD* CreateNonPlanarNonSkipper(DWORD* pProgLoc,const FieldDataBuf& pBuf,
			ULONG Start,DWORD dwImgSizeY,
			LONG lPlaneOffset,LONG lPlanePitch);

		static DWORD* CreateNonPlanarSkipper(
			DWORD* pProgLoc,ULONG Start,DWORD dwImgSizeY);

};

#endif
