/*
	Scaler.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "Scaler.hpp"
#include "Pisces.hpp"

//===========================================================================
// Bt848 Scaler Class Implementation
//===========================================================================

/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////

Scaler::Scaler(BtPisces* p,const VidField field ,const DWORD_PTR dwBase):
	pisces(p),

	regCROP ( (0x03 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,
	fieldVACTIVE_MSB( regCROP, 4, 2, RW) ,
	fieldHDELAY_MSB( regCROP, 2, 2, RW) ,
	fieldHACTIVE_MSB( regCROP, 0, 2, RW) ,

	regVACTIVE_LO ( (0x05 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,

	regHDELAY_LO ( (0x06 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,

	regHACTIVE_LO ( (0x07 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,

	regHSCALE_HI ( (0x08 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,
	fieldHSCALE_MSB( regHSCALE_HI, 0, 8, RW) ,

	regHSCALE_LO ( (0x09 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,
	regSCLOOP ( (0x10 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,
	fieldHFILT( regSCLOOP, 3, 2, RW) ,
	fieldPEAK( regSCLOOP, 7 , 1, RW) , 

	regVSCALE_HI ( (0x13 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,
	fieldVSCALE_MSB( regVSCALE_HI, 0, 5, RW) ,
	fieldCOMB( regVSCALE_HI, 6, 1, RW) ,

	regVSCALE_LO ( (0x14 * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,
	regVActive( regVACTIVE_LO, 8, fieldVACTIVE_MSB, RW ),
	regVScale( regVSCALE_LO, 8, fieldVSCALE_MSB, RW ),
	regHDelay( regHDELAY_LO, 8, fieldHDELAY_MSB, RW ),
	regHActive( regHACTIVE_LO, 8, fieldHACTIVE_MSB, RW ),
	regHScale( regHSCALE_LO, 8, fieldHSCALE_MSB, RW ),

	regVTC ( (0x1B * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW ) ,
	fieldVFILT( regVTC, 0, 2, RW), 

	regCONTROL( (0x0B * 4) + ( ( ( field == VF_Even ) ? 0 : 0x80 ) + dwBase ), RW), 
	fieldLNOTCH( regCONTROL, 7 , 1, RW) , 
	fieldLDEC( regCONTROL, 5 , 1, RW) ,
	fieldCBSENSE( regCONTROL, 4, 1, RW) ,

	// Since VDelay in hardware is reversed; i.e. odd reg is really even field
	// and vice versa, construct the opposite here
	regReverse_CROP ( (0x03 * 4) + ( ( field == VF_Even ) ? 0x80 : 0 ) + dwBase , RW ),
	fieldVDELAY_MSB( regReverse_CROP, 6, 2, RW),
	regVDELAY_LO ( (0x04 * 4) + ( ( field == VF_Even ) ? 0x80 : 0 ) + dwBase , RW ),
	regVDelay( regVDELAY_LO, 8, fieldVDELAY_MSB, RW ),

	VFilterFlag_( On ),
	
	// Set to something that makes sense
	AnalogWin_(0,0,100,100) ,
	DigitalWin_(0,0,100,100) ,
	m_sharpness(100) ,
	m_sharpauto(true)
{
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Scaler::SetSharpness( ULONG Sharpness )
// Purpose: Set Sharpness for videofield
// Input:   Sharpness (0-255)
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
ErrorCode Scaler::SetSharpness(const ULONG Sharpness_)
{
	ULONG Sharpness = Sharpness_;
	if (Sharpness < 18) Sharpness = 18;
	else if (Sharpness > 81) Sharpness = 81;

	if (m_sharpness == Sharpness)
		return Success;

	m_sharpness = Sharpness;

	// every invocation of SetSharpness potentially changes the Sharpness
	Scale();

	return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Scaler::SetAutoSharpness( BOOL Auto)
// Purpose: Set Sharpness for videofield
// Input:   Sharpness (0-255)
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
ErrorCode Scaler::SetAutoSharpness(const bool Auto)
{
	if (m_sharpauto == Auto) 
		return Success;
	
	m_sharpauto = Auto;

	// every invocation of SetSharpness potentially changes the Sharpness
	Scale();

	return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
Scaler::~Scaler()
{
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void Scaler::VideoFormatChanged( void )
// Purpose: Set which video format is using
// Input:   -
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void Scaler::VideoFormatChanged( void )
{
	// every invocation of VideoFormatChanged potentially changes the start
	Scale();

}

#define bound(max_val,val,min_val) max( int(min_val) , min( int(max_val) , int(val) ) )

/////////////////////////////////////////////////////////////////////////////
// Method:  void Scaler::Scale( MRect & clientScr )
// Purpose: Perform scaling
// Input:   MRect & clientScr - rectangle to scale to
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void Scaler::Scale()
{
	TRACE(("Scaler::Scale"));

	// Get the current video info parameters...
	BtPisces::VideoInfoStruct* m_ptrVideoIn = pisces->GetVideoInfo();

	// Avoid using NULL pointers!
	if (!m_ptrVideoIn) {
		return;
	}

	DUMP(("DigitalWin=%d,%d,%d,%d), AnalogWin:(%d,%d,%d,%d)\n", AnalogWin_.left, AnalogWin_.top, AnalogWin_.right, AnalogWin_.bottom, DigitalWin_.left, DigitalWin_.top, DigitalWin_.right, DigitalWin_.bottom ));

	// Calc all the required values

	// Ensure we dont try to do something we cant do (too much zoom)
	DWORD rdWidth = DWORD(bound( m_ptrVideoIn->StdX, WORD(DigitalWin_.Width()), m_ptrVideoIn->Min_Pixels ));
	// Make sure analog window is less or equal than allowed!
	DWORD raWidth = DWORD(bound( m_ptrVideoIn->StdX, WORD(AnalogWin_.Width()), rdWidth ));
	// Make sure Left border start is bounded.
	DWORD rLeft = DWORD(bound( m_ptrVideoIn->StdX - raWidth, WORD(AnalogWin_.left) , 0 ));
	// Translate raWidth to the analog coordinates used by the Bt Chip (Always will grow raWidth)
	raWidth = raWidth * m_ptrVideoIn->Clkx1_HACTIVE / m_ptrVideoIn->StdX;
	// Translate left origin of analog video window to the analog coordinates used by the Bt Chip (Always will grow raWidth)
	DWORD raLeft = rLeft * m_ptrVideoIn->Clkx1_HACTIVE / m_ptrVideoIn->StdX;
	// hActive is simply the actual number of requested pixels per video line!
	DWORD hActive = rdWidth;
	// Calculate the hScale bounding it
	DWORD hScale = ( (raWidth * 4096) / rdWidth ); 
	hScale = DWORD(bound( (65535-4096) , hScale, 4096 ));
	// hDelay is the time to wait before active video starts, relative to start of Sync, and respect to the scaled freq. clock
	DWORD hDelay = ( raLeft + m_ptrVideoIn->Clkx1_HDELAY ) * 4096 / hScale;
	// Don't forget to substract the offset!
	hScale -= 4096;

	DUMP(("rdWidth:%d, raWidth:%d,  rLeft:%d, raLeft:%d\n", rdWidth,raWidth,rLeft,raLeft));
	DUMP(("hActive:%d, hScale:%04x, hDelay:%d\n", hActive, hScale, hDelay ));
	

	// Place all those calculated values into their corresponding registers!
	
	// need to adjust CBSENSE when allowing both odd and even value for HDelay
	fieldCBSENSE = BOOL( hDelay & 1 );
	regHDelay = hDelay;
	regHActive = hActive;
	regHScale = hScale;

	// If we are working with digital video, and HSCALE == 0, enable the bypass of chroma filters!
	if (pisces->IsDigitalSignal() && hScale == 0) {
		pisces->EnableChromaFilterBypass( true );
	} else {
		pisces->EnableChromaFilterBypass( false );
	}

	// Adjust filters!
	if (m_sharpauto) {
		if ( m_ptrVideoIn->AutoHFilter )
		  fieldHFILT = HFilter_AutoFormat;
		else  // SECAM
			if ( hActive < DWORD(m_ptrVideoIn->Clkx1_HACTIVE) / 7 )
				fieldHFILT = HFilter_ICON;
			else
				fieldHFILT = HFilter_QCIF;
		fieldPEAK = 0;	 // Peaking (1 = On)
		fieldLNOTCH = 0; // This can be disabled to get full Black/White bandwidth (0 = On)
		fieldLDEC = 1;	 // Reduces bandwidth of luma. (0 = On)
		fieldCOMB = 1;	 // Averages color to reduce artifacts  (1 = On)

	} else {
		
/*		LNOTCH -> Color subcarrier elimination
		LDEC -> use or not filter LP or PEAK
		PEAK -> Select between peak and filtering 
		COMB = CromaComb: Use this to average chroma components of next lines

			PEAK/HFILT		LDEC	PEAK	COMB
			ICON 11			ON(0)	OFF(0)	ON(1)
			QCIF 10			ON		OFF		ON
			CIF	 01			ON		OFF		ON
			-				OFF(1)	OFF		ON
			11				ON		ON(1)	ON
			10				ON		ON		ON
			01				ON		ON		ON
			00				ON		ON		ON
			ICON 11			ON0		OFF0	OFF
			QCIF 10			ON		OFF		OFF
			CIF	 01			ON		OFF		OFF
			-				OFF		OFF		OFF
			11				ON		ON		OFF
			10				ON		ON		OFF
			01				ON		ON		OFF
			00				ON		ON		OFF */

		ULONG val = m_sharpness - 18;			// Get value of horizontal sharpness
		BYTE PeakHFilt	= BYTE((~(val >> 4)) & 3);		// Bits 54
		BYTE Peak		= 0;							// Peak = Off
		// Allow PEAKing only in Full Size in Bt848A or better.
		if (hActive > DWORD(m_ptrVideoIn->Max_Pixels) / 2 && pisces->HasPEAK() ) {
			Peak		= BYTE((val >> 3) & 1);			// Bit 3	
		}
		BYTE LDec		= (PeakHFilt || Peak) ? 0 : 1;		// 
		BYTE Comb		= BYTE((~(val >> 2)) & 1);			// Bit 2

		fieldHFILT = PeakHFilt;
		fieldPEAK = Peak;
		fieldLDEC = LDec;
		fieldCOMB  = Comb;
		fieldLNOTCH = 0; // This is a MUST to avoid color interference!
	}

	// Ensure we dont try to do something we cant do (too much zoom) Height is in lines per field!
	DWORD rdHeight = DWORD(bound( m_ptrVideoIn->Max_Lines, WORD(DigitalWin_.Height()), m_ptrVideoIn->Min_Lines ));
	// Make sure analog window is less or equal than allowed!
	DWORD raHeight = DWORD(bound( m_ptrVideoIn->Max_Lines, WORD(AnalogWin_.Height()), rdHeight ));
	// Make sure line start is bounded to the possible range...
	DWORD rTop = DWORD(bound( m_ptrVideoIn->Max_Lines - raHeight , WORD(AnalogWin_.top) , 0));
	// vActive is simply the number of source lines to use!
	DWORD vActive = (raHeight << 1);
	// vDelay is simply the number of lines to skip before using + the number of inactive lines of video
	DWORD vDelay = (rTop << 1) + m_ptrVideoIn->VDelay;
	// vScale is the right scaling factor
	DWORD vScale = ( 0x10000 - ( (  (raHeight * 512) / rdHeight ) - 512 ) ) & 0x1FFF;

	DUMP(("rdHeight:%d, raHeight:%d, rTop:%d, raTop:%d\n", rdHeight,raHeight,rTop));
	DUMP(("vActive:%d, vScale:%04x, vDelay:%d\n", vActive, vScale, vDelay ));

	WORD m_VFilter = 0;

	// Calculate Vertical Filter
	// this is to remove junk lines at the top of video. flag set to off
	// when image hight is above CIF
	if ( VFilterFlag_ == Off ) {
		m_VFilter  = 0;
	} else {
		if ( ( hActive <= m_ptrVideoIn->Max_VFilter3_Pixels ) &&
	        ( rdHeight   <= m_ptrVideoIn->Max_VFilter3_Lines ) )
			m_VFilter = 3;
		else if ( ( hActive <= m_ptrVideoIn->Max_VFilter2_Pixels ) &&
					( rdHeight   <= m_ptrVideoIn->Max_VFilter2_Lines ) )
			m_VFilter = 2;
		else if ( ( hActive <= m_ptrVideoIn->Max_VFilter1_Pixels ) &&
					( rdHeight   <= m_ptrVideoIn->Max_VFilter1_Lines ) )
			m_VFilter = 1;
		else
			m_VFilter = 0;
	}
	fieldVFILT = m_VFilter;

	// Place results for vertical scale/crop region
	WORD moreDelay;

	// increase VDelay will eliminate garbage lines at top of image
	switch ( m_VFilter ) {
		case 3:
			moreDelay = 4;
			break;

		case 2:
			moreDelay = 2;
			break;
             
		case 1:
		case 0:
		default:
			moreDelay = 0;
			break;
	}

	vDelay += moreDelay;

	// According to the data sheet, VDelay must be even to avoid
	// appearant field reversal.
	regVDelay = vDelay & (~1);
	regVActive = vActive; 
	regVScale = vScale;

}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Scaler::SetDigitalWin( const MRect &DigWin )
// Purpose: Sets the size and location of the digital window
// Input:   const MRect &DigWin - window size to set to
// Output:  None
// Return:  Success or Fail if passed rect is bigger then analog window
// Note:    This function can affect the scaling, so Scale() is called
/////////////////////////////////////////////////////////////////////////////
ErrorCode Scaler::SetDigitalWin( const MRect &DigWin )
{
	DigitalWin_ = DigWin;

	// every invocation of SetDigitalWin potentially changes the scaling
	Scale();

	return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode Scaler::SetAnalogWin( const MRect &AWin )
// Purpose: Sets the size and location of the analog window
// Input:   const MRect &AWin - window size to set to
// Output:  None
// Return:  
/////////////////////////////////////////////////////////////////////////////
ErrorCode Scaler::SetAnalogWin( const MRect &AWin )
{
	AnalogWin_ = AWin;

	// every invocation of SetAnalogWin potentially changes the start
	Scale();

	return Success;
}

