/*
	TimGen.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TimGen.hpp"

//===========================================================================
// Bt848A/Bt849 TimGen Class Implementation
//===========================================================================


/////////////////////////////////////////////////////////////////////////////
// Constructor
/////////////////////////////////////////////////////////////////////////////

TimGen::TimGen( const DWORD_PTR dwBase ):
	timRegTGLB( dwBase + ( 0x20  * 4) , RW ) ,
	timRegTGCTRL( dwBase + ( 0x21 * 4) , RW ) ,
	timFieldTGCKO( timRegTGCTRL, 5, 2, RW) ,
	timFieldTGCKI( timRegTGCTRL, 3, 2, RW) ,
	timFieldTGC_AI( timRegTGCTRL, 2, 1, RW) ,
	timFieldTGC_AR( timRegTGCTRL, 1, 1, RW) ,
	timFieldTGC_VM( timRegTGCTRL, 0, 1, RW)
{
}


/////////////////////////////////////////////////////////////////////////////
// Destructor
/////////////////////////////////////////////////////////////////////////////
TimGen::~TimGen()
{
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode TimGen::LoadSRAM( PBYTE p, int nBytes )
// Purpose: Load time generator SRAM
// Input:   PBYTE p    - points to value to load from
//          int nBytes - number of bytes to load
// Output:  None
// Return:  Fail if error loading SRAM, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode TimGen::LoadSRAM( PBYTE p, int nBytes )
{
   EnableTimGenVideoMode( false );  // select read/write mode
   TimGenAddrReset();               // address reset

   // Load data table in SRAM
   PBYTE q = p;
   for ( int i = 0; i < nBytes; i++ ) {
      timRegTGLB = *q++;
   }

   EnableTimGenVideoMode( true );   // select video mode
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode TimGen::ReadSRAM( PBYTE p, int nBytes )
// Purpose: Load time generator SRAM
// Input:   int nBytes - number of bytes to copy
// Output:  PBYTE p    - points to value to be copied to
// Return:  Fail if error reading SRAM, else Success
// Note:    Assume *p is large enough to hold nBytes of data
/////////////////////////////////////////////////////////////////////////////
ErrorCode TimGen::ReadSRAM( PBYTE p, int nBytes )
{
   EnableTimGenVideoMode( false );  // select read/write mode
   TimGenAddrReset();               // address reset

   // Read bytes from sram
   PBYTE q = p;
   for ( int i = 0; i < nBytes; i++ ) {
      *q++ = BYTE(timRegTGLB);
      IncrTimGenReadAddr();
   }
   EnableTimGenVideoMode( true );   // select video mode
   return Success;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  void TimGen::TimGenAddrReset( void )
// Purpose: Reset timing generator address
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void TimGen::TimGenAddrReset( void )
{
  timFieldTGC_AR = 0;
  timFieldTGC_AR = 1;
  timFieldTGC_AR = 0;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  void TimGen::IncrTimGenReadAddr( void )
// Purpose: Increment timing generator read address
// Input:   None
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
void TimGen::IncrTimGenReadAddr( void )
{
  timFieldTGC_AI = 0;
  timFieldTGC_AI = 1;
  timFieldTGC_AI = 0;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode TimGen::SetDecoderInputClock( DecInputClk clk )
// Purpose: Select decoder input clock
// Input:   Decoder input clock:
//            DecClk_Normal   - Normal Xtal0/Xtal1 mode
//            DecClk_PLL      - PLL
//            DecClk_GPCLK    - GPCLK
//            DecClk_invGPCLK - GPCLK inverted
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode TimGen::SetDecoderInputClock( DecInputClk clk )
{
   if ( ( clk < DecClk_XTAL     ) ||
        ( clk > DecClk_invGPCLK ) )
      return Fail;

   timFieldTGCKI = clk;
   return Success;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  ErrorCode TimGen::SetGPCLKOutputClock( GPCLKOutputClk clk )
// Purpose: Select GPCLK output clock
// Input:   GPCLK output clock:
//            GPCLK_CLKx1  - CLKx1
//            GPCLK_Xtal0  - Xtal0
//            GPCLK_PLL    - PLL
//            GPCLK_invPLL - PLL inverted
// Output:  None
// Return:  Fail if error in parameter, else Success
/////////////////////////////////////////////////////////////////////////////
ErrorCode TimGen::SetGPCLKOutputClock( GPCLKOutputClk clk )
{
   if ( ( clk < GPCLK_CLKx1  ) ||
        ( clk > GPCLK_invPLL ) )
      return Fail;

   timFieldTGCKO = clk;
   return Success;
}


