/*
	TimGen.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __TIMGEN_HPP
#define __TIMGEN_HPP

#include "debug.hpp"
#include "regField.hpp"
#include "mytypes.hpp"

/* Type: DecoderInputClock
 * Purpose: Defines decoder input clock select
 */
typedef enum {  DecClk_XTAL,
                DecClk_PLL,
                DecClk_GPCLK,
                DecClk_invGPCLK } DecInputClk;

/* Type: GPCLKOutputClock
 * Purpose: Defines GPCLK output clock select
 */
typedef enum {  GPCLK_CLKx1,
                GPCLK_Xtal0,
                GPCLK_PLL,
                GPCLK_invPLL } GPCLKOutputClk;


/////////////////////////////////////////////////////////////////////////////
// CLASS TimGen
//
// Description:
//    This class encapsulates the Timing Generator Control register in the
//    Bt848A/Bt849.
//
// Methods:
//    See below
//
// Note:
//
/////////////////////////////////////////////////////////////////////////////

class TimGen
{
	private:

		RegisterB timRegTGLB;
		RegisterB timRegTGCTRL;
		RegField  timFieldTGCKO;
		RegField  timFieldTGCKI;
		RegField  timFieldTGC_AI;
		RegField  timFieldTGC_AR;
		RegField  timFieldTGC_VM;

	public:

		TimGen( const DWORD_PTR dwBase );
		~TimGen();

		ErrorCode LoadSRAM( PBYTE fp, int nBytes );
		ErrorCode ReadSRAM( PBYTE p, int nBytes );
		void	  EnableTimGenVideoMode( bool mode );
		bool	  IsTimGenVideoModeEnabled( void );
		void	  TimGenAddrReset( void );
		void	  IncrTimGenReadAddr( void );
		ErrorCode SetDecoderInputClock( DecInputClk clk );
		int		  GetDecoderInputClock( void );
		ErrorCode SetGPCLKOutputClock( GPCLKOutputClk clk );
		int		  GetGPCLKOutputClock( void );

};

/////////////////////////////////////////////////////////////////////////////
// Method:  void TimGen::EnableTimGenVideoMode( bool mode )
// Purpose: Enable/Disable Timing Generator Video Mode (0-read/write mode)
// Input:   bool mode - true = Enable; false = Disable
// Output:  None
// Return:  None
/////////////////////////////////////////////////////////////////////////////
inline void TimGen::EnableTimGenVideoMode( bool mode )
{
  timFieldTGC_VM = (mode == false) ? 0 : 1;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  bool TimGen::IsTimGenVideoModeEnabled( void )
// Purpose: Check if timing generator video mode is enabled or disable
// Input:   None
// Output:  None
// Return:  true = Enable; false = Disable
/////////////////////////////////////////////////////////////////////////////
inline bool TimGen::IsTimGenVideoModeEnabled( void )
{
  return (timFieldTGC_VM == 1) ? true : false;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int TimGen::GetDecoderInputClock( void )
// Purpose: Obtain which decoder input clock is selected
// Input:   None
// Output:  None
// Return:  int - Decoder input clock:
//            DecClk_Normal   - Normal Xtal0/Xtal1 mode
//            DecClk_PLL      - PLL
//            DecClk_GPCLK    - GPCLK
//            DecClk_invGPCLK - GPCLK inverted
/////////////////////////////////////////////////////////////////////////////
inline int TimGen::GetDecoderInputClock( void )
{
   return timFieldTGCKI;
}

/////////////////////////////////////////////////////////////////////////////
// Method:  int TimGen::GetGPCLKOutputClock( void )
// Purpose: Obtain which GPCLK output clock is selected
// Input:   None
// Output:  None
// Return:  int - GPCLK output clock:
//            GPCLK_CLKx1  - CLKx1
//            GPCLK_Xtal0  - Xtal0
//            GPCLK_PLL    - PLL
//            GPCLK_invPLL - PLL inverted
/////////////////////////////////////////////////////////////////////////////
inline int TimGen::GetGPCLKOutputClock( void )
{
   return timFieldTGCKO;
}


#endif // __TIMGEN_HPP










