/*
	VidCh.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "Pisces.hpp"
#include "vidch.hpp"

// VideoChannel Functions
VideoChannel::VideoChannel( VideoChanIface* xface , const VCMode cm ) :
	bufferxface(xface) , State_(Stopped) , mode(cm) , dwBufsInProgress(0)
{
	ENSURE(xface);
}

// Signal to the stream Buffer is done
/* Note: this is called at IRQL = DISPATCH_LEVEL */
void VideoChannel::BufferDone( const VideoDataBuf& pBuf , const VidField vf ) 
{ 
	TRACE(("VideoChannel::BufferDone"));

	// If it is a real Buffer, decrement count of pending buffers.
	if (pBuf.RealBuffer()) {
		// Remember we have one less buffer to complete.
		dwBufsInProgress --;
	}

	// If the buffer has an owner ...
	if (pBuf.GetOwner()) {
		pBuf.GetOwner()->StreamBufferDone(pBuf , vf); // Tell the Stream Class the field that completed this buffer (sometimes, needed)
	}
}

// ONLY If we are running, get a new buffer from the stream
/* Note: this is called at IRQL = DISPATCH_LEVEL */
const VideoDataBuf VideoChannel::GetNextBuffer() 
{ 
	TRACE(("VideoChannel::GetNextBuffer"));

	VideoDataBuf ret; // Start with a NULL buffer
	if (State_ == Started) { // Only if started!!!

		// Get a buffer from the buffer provider
		ret = bufferxface->StreamGetNextBuffer(); 

		// If it is a real Buffer, count it as pending.
		if (ret.RealBuffer()) {

			// Remember we have at least one buffer to do.
			dwBufsInProgress ++;
		}
	} else {
		DUMP(("Channel not running, returning NULL\n"));
	}

	return ret;
}

// Stop a VideoChannel: Must Stop channel and wait until all pending 
// databuffers are processed!
ErrorCode VideoChannel::Stop()
{
	TRACE(("VideoChannel::Stop"));

	// Remember channel will be Stopped (don't get any databuffers from the provider)
	if (State_ != Stopped) {

		// If we were not Stopped, stop Stream to avoid getting new buffers!
		State_ = Stopped;

		// Now wait 'til we complete all the remaining buffers letting the other tasks run
		while (dwBufsInProgress) {

			DUMP(("VideoChannel::Stop -> Waiting to Complete all Pending Buffers...\n"));

			// While we wait, let other tasks do their job.
			mdelay(40);
		};
	}

	// We Stopped!
	return Success;
}


// SingleVideoChannel Functions
SingleVideoChannel::SingleVideoChannel( VideoChanIface* xface, const VCMode cm , Field& field) :
	VideoChannel(xface,cm) , OurField_(NULL)
{
	// Try to open a new instance of this field
	if (field.Open(this)) {
		OurField_ = &field;
	}
}

SingleVideoChannel::~SingleVideoChannel()
{
	// If we were able to open the field ...
	if (OurField_) {

		// Stop channel first
		Stop();

		// Close our field 
		OurField_->Close();
	}
}

const FieldDataBuf SingleVideoChannel::VideoChannelGetNextBuffer( const VidField vf ) 
{ 
	TRACE(("SingleVideoChannel::VideoChannelGetNextBuffer"));

	// Get next buffer
	VideoDataBuf pBuf = GetNextBuffer();

	// Prepare our FieldDataBuf
	return FieldDataBuf(pBuf,0,pBuf.GetPitch()); 
}

void SingleVideoChannel::VideoChannelBufferDone( const FieldDataBuf& pBuf , const VidField vf ) 
{ 
	TRACE(("SingleVideoChannel::VideoChannelBufferDone"));

	BufferDone(pBuf.GetBuf(), vf); 
}

ErrorCode SingleVideoChannel::SetAnalogWindow( const MRect &r )
{
	// Calculate the analog window. Remember, we are subsampling, probably from
	// full frame to half vertical res. That's why we use a single field, but ,
	// for the stream, we seem to be using the full frame
	MRect rc(	r.left,r.top/2,
				r.right,r.bottom/2 );

	// Set the analog window of our field
	return OurField_->SetAnalogWindow(rc);
}

const MRect SingleVideoChannel::GetAnalogWindow( ) const 
{
	// Get the analog window of our field
	MRect rc = OurField_->GetAnalogWindow();
	// And calculate the original window
	return MRect(rc.left ,rc.top * 2, rc.right , rc.bottom * 2);
}

// PairedVideoChannel functions
PairedVideoChannel::PairedVideoChannel( VideoChanIface* xface, const VCMode cm , BtPisces* digitizer_) :
	VideoChannel(xface,cm) , OurOddField_(NULL) , OurEvenField_(NULL) , digitizer(digitizer_)
{
}

PairedVideoChannel::~PairedVideoChannel()
{
	// If we were able to open the fields ...
	if (OurOddField_ && OurEvenField_) {

		// Stop channel first
		Stop();

		// Close fields!
		OurOddField_->Close();
		OurEvenField_->Close();
	}
}

ErrorCode PairedVideoChannel::SetColorFormat( const ColFmt aColor )
{
	// Tell our fields of the new color format. Will be applied to the next buffer
	ErrorCode ret = Success;
	ColFmt oldCol = OurOddField_->GetColorFormat();
	if (OurOddField_->SetColorFormat(aColor) == Fail) ret = Fail;
	if (OurEvenField_->SetColorFormat(aColor) == Fail) {
		// Restore the odd field color format!
		OurOddField_->SetColorFormat(oldCol);
		ret = Fail;
	}
	return ret;
}

ColFmt PairedVideoChannel::GetColorFormat() const 
{
	// Get the colorformat from the odd field!
	ENSURE(OurOddField_->GetColorFormat() == OurEvenField_->GetColorFormat()); 
	return OurOddField_->GetColorFormat();
}
		
// InterlacedVideoChannel functions
InterlacedVideoChannel::InterlacedVideoChannel( VideoChanIface* xface, const VCMode cm , Field& odd , Field& even , BtPisces* digitizer_) :
	PairedVideoChannel(xface,cm,digitizer_) , pCurrentBuf() // NULL buffer
{
	// Try to open the needed fields - Code is here because I need to pass the "this"
	// pointer to this class and NOT to the base class
	// Note that the pairing operation will work despite the fact that
	// the RISC programs associated to each field are created one by one
	// and one of them can be executed while the other is not created yet
	// because the Channel is Stopped and does not supply buffers to the first one.
	if (odd.Open(this)) {
		OurOddField_ = &odd;
		if (even.Open(this)) {
			OurEvenField_ = &even;

		} else {

			odd.Close();
			OurOddField_ = NULL;
		}
	}
}

InterlacedVideoChannel::~InterlacedVideoChannel()
{
}

// FieldBufferXface
const FieldDataBuf InterlacedVideoChannel::VideoChannelGetNextBuffer(const VidField vf )
{
	TRACE(("InterlacedVideoChannel::VideoChannelGetNextBuffer"));

	// Create a VideoDataBuf (the one that we will return)
	VideoDataBuf ret;

	// If we are asked by the Odd field, we will return a new databuffer,
	// If we are asked by the even field, we will return the databuffer of the odd field

	// If it is the master field ...
	DWORD dwOffset = 0;
	if (vf == VF_Odd) {
		DUMP(("Gave New Buffer to Odd field\n"));
	
		// Get a new videoBuffer to return
		ret =  GetNextBuffer();

		// And remember it as we need it for the paired field
		pCurrentBuf = ret;

		// And adjust buffer offset to achieve interleaving: This is depending on TopField and if Videomode is flipped or not
		switch (GetColorFormat()) {
			case CF_RGB32:	case CF_RGB24:	case CF_RGB16: 
			case CF_RGB15:	case CF_BTYUV:	case CF_RGB8: // All these modes are flipped!
				dwOffset = ( (digitizer->GetTopField() == VF_Even) ? 0 : ret.GetPitch() );
				break;
			default:
				dwOffset = ( (digitizer->GetTopField() == VF_Even) ? ret.GetPitch() : 0 );
				break;
		}

	} else {
		DUMP(("Gave same buffer of Odd field\n"));

		// Get the Previous buffer to return it to the paired Field
		ret = pCurrentBuf;

		// And remember this buffer won't be valid anymore
		pCurrentBuf = VideoDataBuf();

		// And adjust buffer offset to achieve interleaving: This is depending on TopField and if Videomode is flipped or not
		switch (GetColorFormat()) {
			case CF_RGB32:	case CF_RGB24:	case CF_RGB16: 
			case CF_RGB15:	case CF_BTYUV:	case CF_RGB8: // All these modes are flipped!
				dwOffset = ( (digitizer->GetTopField() == VF_Even) ? ret.GetPitch() : 0 );
				break;
			default:
				dwOffset = ( (digitizer->GetTopField() == VF_Even) ? 0 : ret.GetPitch() );
				break;
		}

	}	

	// Return the needed buffer
	return FieldDataBuf(ret,dwOffset,ret.GetPitch() * 2);
}

void InterlacedVideoChannel::VideoChannelBufferDone(const FieldDataBuf& pBuf ,const VidField vf ) 
{
	TRACE(("InterlacedVideoChannel::VideoChannelBufferDone"));

	// If we are asked to notify the stream class by the Slave field, ignore it, (not complete yet!)
	if (vf == VF_Even) {
		DUMP(("Even field: Signal user the frame is done\n"));

		// When finished with the Slave field , buffer is completely captured!
		BufferDone(pBuf.GetBuf() , VF_Even);
	}
}
	
ErrorCode InterlacedVideoChannel::SetAnalogWindow( const MRect &r )
{
	// Calculate the even and odd analog windows
	MRect odd(	r.left,r.top/2,
				r.right,r.bottom/2 + (r.Height() & 1) );
	MRect even(	r.left,r.top/2,
				r.right,r.bottom/2);

	// Set the analog window of our fields
	ErrorCode ret = Success;
	MRect oldrc = OurOddField_->GetAnalogWindow();
	if (OurOddField_->SetAnalogWindow(odd) == Fail) ret = Fail;
	if (OurEvenField_->SetAnalogWindow(even) == Fail) {
		// Restore old Rect
		OurOddField_->SetAnalogWindow(oldrc);
		ret = Fail;
	}

	// If both were ok, return success
	return ret;
}

const MRect InterlacedVideoChannel::GetAnalogWindow( ) const 
{
	// Get the analog window of both fields
	MRect odd = OurOddField_->GetAnalogWindow();
	MRect even = OurEvenField_->GetAnalogWindow();
	
	// And calculate the original window
	return MRect(odd.left,odd.top+even.top,odd.right,odd.bottom+even.bottom);
}

ErrorCode InterlacedVideoChannel::SetDigitalWindow( const MRect &r ) 
{
	// Calculate the even and odd digital windows
	MRect odd(	r.left,r.top/2,
				r.right,r.bottom/2 + (r.Height() & 1) );
	MRect even(	r.left,r.top/2,
				r.right,r.bottom/2);

	// Set the analog window of our fields
	ErrorCode ret = Success;
	MRect oldrc = OurOddField_->GetDigitalWindow();
	if (OurOddField_->SetDigitalWindow(odd) == Fail) ret = Fail;
	if (OurEvenField_->SetDigitalWindow(even) == Fail) {
		OurOddField_->SetDigitalWindow(oldrc);
		ret = Fail;
	}

	// If both were ok, return success
	return ret;
}

const MRect InterlacedVideoChannel::GetDigitalWindow( ) const 
{
	// Get the digital window of both fields
	MRect odd = OurOddField_->GetDigitalWindow();
	MRect even = OurEvenField_->GetDigitalWindow();
	
	// And calculate the original window
	return MRect(odd.left,odd.top+even.top,odd.right,odd.bottom+even.bottom);
}

// AlternatingVideoChannel functions
AlternatingVideoChannel::AlternatingVideoChannel( VideoChanIface* xface, const VCMode cm , Field& odd , Field& even , BtPisces* digitizer_) :
	PairedVideoChannel(xface,cm,digitizer_)
{
	// Try to open the needed fields - Code is here because I need to pass the "this"
	// pointer to this class and NOT to the base class
	// Note that the pairing operation will work despite the fact that
	// the RISC programs associated to each field are created one by one
	// and one of them can be executed while the other is not created yet
	// because the Channel is Stopped and does not supply buffers to the first one.
	if (odd.Open(this)) {
		OurOddField_ = &odd;
		if (even.Open(this)) {
			OurEvenField_ = &even;

		} else {

			odd.Close();
			OurOddField_ = NULL;
		}
	}
}

AlternatingVideoChannel::~AlternatingVideoChannel()
{
}

// FieldBufferXface
const FieldDataBuf AlternatingVideoChannel::VideoChannelGetNextBuffer( const VidField vf )
{
	TRACE(("AlternatingVideoChannel::VideoChannelGetNextBuffer"));

	// Get a new buffer
	VideoDataBuf pBuf = GetNextBuffer();

	// And prepare the field buffer!
	return FieldDataBuf(pBuf,0,pBuf.GetPitch());
}

void AlternatingVideoChannel::VideoChannelBufferDone( const FieldDataBuf& pBuf , const VidField vf ) 
{
	TRACE(("AlternatingVideoChannel::VideoChannelBufferDone"));

	// Each field finishes its own buffer ... 
	BufferDone(pBuf.GetBuf() , vf);
}
	
ErrorCode AlternatingVideoChannel::SetAnalogWindow( const MRect &r )
{
	// Set the analog window of our fields
	ErrorCode ret = Success;
	MRect oldrc = OurOddField_->GetAnalogWindow();
	if (OurOddField_->SetAnalogWindow(r) == Fail) ret = Fail;
	if (OurEvenField_->SetAnalogWindow(r) == Fail) {
		OurOddField_->SetAnalogWindow(oldrc);
		ret = Fail;
	}

	// If both were ok, return success
	return ret;
}

const MRect AlternatingVideoChannel::GetAnalogWindow( ) const 
{
	// Get the analog window of the odd field (both are the same)
	return OurOddField_->GetAnalogWindow();
}

ErrorCode AlternatingVideoChannel::SetDigitalWindow( const MRect &r ) 
{
	// Set the analog window of our fields
	ErrorCode ret = Success;
	MRect oldrc = OurOddField_->GetDigitalWindow();
	if (OurOddField_->SetDigitalWindow(r) == Fail) ret = Fail;
	if (OurEvenField_->SetDigitalWindow(r) == Fail) {
		OurOddField_->SetDigitalWindow(oldrc);
		ret = Fail;
	}

	// If both were ok, return success
	return ret;
}

const MRect AlternatingVideoChannel::GetDigitalWindow( ) const 
{
	// Get the digital window of odd field (both are the same)
	return OurOddField_->GetDigitalWindow();
}

const FieldDataBuf VirtualChannel::VideoChannelGetNextBuffer( const VidField vf ) 
{ 
	TRACE(("VirtualChannel::VideoChannelGetNextBuffer"));
	VideoDataBuf pBuf = GetNextBuffer();
	return FieldDataBuf(pBuf,0,pBuf.GetPitch()); 
}

void VirtualChannel::VideoChannelBufferDone( const FieldDataBuf& pBuf , const VidField vf ) 
{ 
	TRACE(("VirtualChannel::VideoChannelBufferDone"));
	BufferDone(pBuf.GetBuf(), vf); 
}
