/*
	VidCh.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef __VIDCH_HPP
#define __VIDCH_HPP

#include "mytypes.hpp"
#include "field.hpp"
#include "vidchifc.hpp"
#include "fieldifc.hpp" // We will be a databuffer provider!

class BtPisces;	// Forward declaration

/* enum StreamState: State at which a stream can be */
typedef enum { 
	Stopped, 
	Started 
} StreamState;

/* Mode of channel */
typedef enum { 
	VCM_AutoOdd,		// If can be opened with one Field, prefer Odd, else interlace
	VCM_AutoEven,		// If can be opened with one Field, prefer Even, else interlace
	VCM_Odd,			// Open the Odd field!
	VCM_Even,			// Open the Even field!
	VCM_Interlaced,		// Open always interlaced!
	VCM_Alternating,	// Open always as alternating fields
	VCM_VBI,			// Open as VBI
	VCM_Virtual			// Open a non-existant channel that seems to be a real one. Very useful to give priorities (suspend service to one, give to other)
} VCMode;

/*
	Notes:
		There exists 4 kinds of usable channels:
			- SingleVideoChannel:		
				Used to stream only one field as a sample
			- AlternatingVideoChannel:	
				Used to stream both fields, but as separate sample , alternationg between both fields
			- InterleavedVideoChannel:	
				Used to stream both fields interleaved, as an only sample, to sample an entire frame.
			- VirtualChannel:			
				Used as a low priority substitute for the above 3, to get preview if possible. If we are NOT using
				the total framerate available for one of the above kinds of channels, and the colorformat and the
				image size match, we can use the remaining non-used frames to get preview images.
 */


/* Class: VideoChannel
 * Purpose: An (abstract) Video Stream
 * Attributes:
 * Methods
 */
class VideoChannel {

	protected:

		// State of Stream
		volatile StreamState	State_;

		// Count of Buffers in process of being filled
		volatile DWORD			dwBufsInProgress;

		// Stream Data buffer provider
		VideoChanIface*			bufferxface;		// Pointer to the video buffer provider

		// Mode of VideoChannel
		const VCMode			mode;

	public:

		VideoChannel( VideoChanIface* , const VCMode cm );
		virtual ~VideoChannel( void ) {}

		// Initialization OK
		virtual bool WasOpen() const = 0;

		// Stream Attributes
				StreamState GetState( void ) const { return State_; }

		virtual ErrorCode	SetColorFormat( const ColFmt aColor ) = 0;
		virtual ColFmt		GetColorFormat( void ) const = 0;
		virtual ErrorCode	SetAnalogWindow( const MRect &r ) = 0;
		virtual const MRect	GetAnalogWindow( ) const = 0;
		virtual ErrorCode	SetDigitalWindow( const MRect &r ) = 0;
		virtual const MRect GetDigitalWindow( ) const = 0;

		// Actions over channel
		ErrorCode			Start( void ) { State_ = Started; return Success;}
		ErrorCode			Stop( void );

		// Mode of videochannel
		const VCMode		GetMode( void ) const { return mode; }

	protected:

		// Buffer Get / Set from the stream
		void BufferDone( const VideoDataBuf& pBuf ,const VidField vf ); 
		const VideoDataBuf GetNextBuffer( void );

};


/* Class: SingleVideoChannel
 * Purpose: An One Field based Video Stream
 * Attributes:
 * Methods
 */
class SingleVideoChannel :
	public VideoChannel ,	// Implementators of this interfase
	public FieldBufferXface // We are a buffer provider for fields!
{

	protected:

		Field*	OurField_;		// The field we use to stream data

	protected: 

		SingleVideoChannel( VideoChanIface* , const VCMode cm , Field& );
		virtual ~SingleVideoChannel( void );

		// Initialization OK
		virtual bool WasOpen() const { return OurField_ != NULL; };

	public:

		friend class BtPisces; // The only one that can create and destroy a videoChannel

		// FieldBufferXface
		virtual const FieldDataBuf VideoChannelGetNextBuffer( const VidField vf );
		virtual void VideoChannelBufferDone( const FieldDataBuf& pBuf , const VidField vf );
	
		// Stream Attributes
		virtual ErrorCode	SetColorFormat( const ColFmt aColor ) { return OurField_->SetColorFormat(aColor); }
		virtual ColFmt		GetColorFormat( void ) const { return OurField_->GetColorFormat(); }
		virtual ErrorCode	SetAnalogWindow( const MRect &r ) ;
		virtual const MRect	GetAnalogWindow( ) const ;
		virtual ErrorCode	SetDigitalWindow( const MRect &r ) { return OurField_->SetDigitalWindow(r); }
		virtual const MRect GetDigitalWindow( ) const { return OurField_->GetDigitalWindow(); }

};

/* Class: PairedVideoChannel
 * Purpose: Implements basic functionality of a two Field based Video Stream
 * Attributes:
 * Methods
 */
class PairedVideoChannel :
	public VideoChannel 	// Implementators of this interfase
{

	protected:

		// The fields to be paired (grouped) to form this videoChannel
		Field*		OurOddField_;		// The Odd field instance we use to stream data 
		Field*		OurEvenField_;		// The even field instance we use to stream data.

		// Pointer to the BtPisces object
		BtPisces*	digitizer;

	public: 

		PairedVideoChannel( VideoChanIface* , const VCMode cm , BtPisces* digitizer);
		virtual ~PairedVideoChannel( void );

		// Initialization OK
		virtual bool WasOpen() const { return ((OurEvenField_ != NULL) && (OurOddField_ != NULL)); }

		// Stream Attributes
		virtual ErrorCode	SetColorFormat( const ColFmt aColor ) ;
		virtual ColFmt		GetColorFormat( void ) const ;
		virtual ErrorCode	SetAnalogWindow( const MRect &r ) = 0;
		virtual const MRect	GetAnalogWindow( ) const = 0;
		virtual ErrorCode	SetDigitalWindow( const MRect &r ) = 0;
		virtual const MRect	GetDigitalWindow( ) const = 0;

};

/* Class: InterlacedVideoChannel
 * Purpose: An interlaced (with 2 fields) Video Stream
 * Attributes:
 * Methods
 */
class InterlacedVideoChannel :
	public PairedVideoChannel ,	// Implementators of a Paired interfase
	public FieldBufferXface // We are a buffer provider for fields!
{

	protected:

		// Temporary hold variable so both fields get the same Buffer
		VideoDataBuf pCurrentBuf; 
		
	protected: 

		InterlacedVideoChannel( VideoChanIface* , 
			const VCMode cm , Field& odd, Field& even ,
			BtPisces* digitizer); 

		virtual ~InterlacedVideoChannel( void );

	public:

		friend class BtPisces; // The only one that can create and destroy a videoChannel

		// FieldBufferXface
		virtual const FieldDataBuf VideoChannelGetNextBuffer( const VidField vf );
		virtual void VideoChannelBufferDone( const FieldDataBuf& pBuf , const VidField vf ) ; 
	
		// Stream Attributes
		virtual ErrorCode	SetAnalogWindow( const MRect &r ) ;
		virtual const MRect	GetAnalogWindow( ) const ;
		virtual ErrorCode	SetDigitalWindow( const MRect &r ) ; 
		virtual const MRect	GetDigitalWindow( ) const ;

};

/* Class: AlternatingVideoChannel
 * Purpose: An alternating (with 2 fields) Video Stream
 * Attributes:
 * Methods
 */
class AlternatingVideoChannel :
	public PairedVideoChannel ,	// Implementators of a Paired interfase
	public FieldBufferXface // We are a buffer provider for fields!
{

	protected: 

		AlternatingVideoChannel( VideoChanIface* , const VCMode cm , 
			Field& odd, Field& even ,
			BtPisces* digitizer);

		virtual ~AlternatingVideoChannel( void );

	public:

		friend class BtPisces; // The only one that can create and destroy a videoChannel

		// FieldBufferXface
		virtual const FieldDataBuf VideoChannelGetNextBuffer( const VidField vf );
		virtual void VideoChannelBufferDone( const FieldDataBuf& pBuf , const VidField vf ) ; 
	
		// Stream Attributes
		virtual ErrorCode	SetAnalogWindow( const MRect &r ) ;
		virtual const MRect	GetAnalogWindow( ) const ;
		virtual ErrorCode	SetDigitalWindow( const MRect &r ) ;
		virtual const MRect	GetDigitalWindow( ) const ;

};


// Our virtual channel!
class VirtualChannel :
	public VideoChannel 	// Implementators of this interfase
{
	protected:

		ColFmt	ColorFormat;
		MRect   Analog;
		MRect	Digital;

	protected: 

		VirtualChannel( VideoChanIface* i, const VCMode cm ) : VideoChannel(i,cm) {}
		virtual ~VirtualChannel( void ) {}

		// Initialization OK
		virtual bool WasOpen() const { return true; };

	public:

		friend class BtPisces; // The only one that can create and destroy a videoChannel

		// FieldBufferXface
		virtual const FieldDataBuf VideoChannelGetNextBuffer( const VidField vf );
		virtual void VideoChannelBufferDone( const FieldDataBuf& pBuf , const VidField vf );
	
		// Stream Attributes
		virtual ErrorCode	SetColorFormat( const ColFmt aColor ) { ColorFormat = aColor; return Success; }
		virtual ColFmt		GetColorFormat( void ) const { return ColorFormat; }
		virtual ErrorCode	SetAnalogWindow( const MRect &r ) { Analog = r; return Success;}
		virtual const MRect	GetAnalogWindow( ) const { return Analog; }
		virtual ErrorCode	SetDigitalWindow( const MRect &r ) { Digital = r; return Success;}
		virtual const MRect	GetDigitalWindow( ) const { return Digital; }

};

#endif // __VIDCH_HPP

	
