/*
	TDA9887.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#include "TDA9887.hpp"
#include "..\HwCapture\Pisces.hpp"

 /*
Norm:		Audio Carrier	Video Bandwidth

SECAM_DKK1L	6.5MHz			6.0Mhz
PAL_D		6.5MHz			6.0MHz 

PAL_I		6.0Mhz			5.5MHz

PAL_BGH		5.5Mhz			5.0MHz
PAL_N		5.5MHz			5.0MHz
SECAM_BGH	5.5MHz			5.0MHz

NTSC_M		4.5Mhz			4.2MHz
PAL_M		4.5Mhz			4.2MHz
PAL_NC		4.5Mhz			4.2MHz

 */

/// TDA9885 (PAL, NTSC)
/// TDA9886 (PAL, SECAM, NTSC)
/// TDA9887 (PAL, SECAM, NTSC, FM Radio)


void TDA9887::SetMode(const bool EnableTuner)
{
	// Original subaddress 0 value: 0x16 ... 
	// Seems Pinnacle uses 0x54 | 0x80 = 0xD4 as the subaddress 0 contents... -> That enables the MT2032 tuner...
	// Then, it disables the tuner access using 0x54... Arghhh!

#if 0
	// NTSC use 45.75Mhz IF
	BYTE buf_ntsc[]		 = { 0x00, 0x54, 0x70, 0x44 };

	// PAL use 38.9Mhz IF
	BYTE buf_pal_bghn[]  = { 0x00, 0x54, 0x70, 0x49 };
	BYTE buf_pal_i []    = { 0x00, 0x54, 0x70, 0x4a };
	BYTE buf_pal_dk[]    = { 0x00, 0x54, 0x70, 0x4b };
	BYTE buf_pal_l[]     = { 0x00, 0x46, 0x50, 0x4b };

	// Fm stereo
	static BYTE buf_fm_stereo[] = { 0x00, 0xce, 0x0d, 0x77 };
#endif


	BYTE buf_pal_bghn[]  = { 0x00, 0x96, 0x70, 0x49 };
	BYTE buf_pal_i[]     = { 0x00, 0x96, 0x70, 0x4a };
	BYTE buf_pal_dk[]    = { 0x00, 0x96, 0x70, 0x4b };
	BYTE buf_pal_l[]     = { 0x00, 0x86, 0x50, 0x4b };
	BYTE buf_fm_stereo[] = { 0x00, 0x8e, 0x0d, 0x77 };
	BYTE buf_ntsc[]	     = { 0x00, 0x96, 0x70, 0x44 };
	BYTE buf_ntsc_jp[]   = { 0x00, 0x96, 0x70, 0x40 };


	BYTE *buf = NULL;

	if (m_tm == HwTuner::Tm_Radio_FM || m_tm == HwTuner::Tm_Radio_AM) {
		// Radio mode...
		buf = buf_fm_stereo;
	} else
	if (m_vf & TN_CCIR_DKL) {
		buf = buf_pal_dk;
		if (m_vf & KS_AnalogVideo_SECAM_L) {
			buf = buf_pal_l;
		}
	} else
	if (m_vf & TN_CCIR_I) {
		buf = buf_pal_i;
	} else 
	if (m_vf & TN_CCIR_BGHN) {
		buf = buf_pal_bghn;
	} else 
	if (m_vf & TN_CCIR_MC) {
		if (m_vf & KS_AnalogVideo_NTSC_M_J) {
			buf = buf_ntsc_jp;
		} else {
			// NTSC / PAL-M / PAL-NC
			buf = buf_ntsc;
		}
	} else {
		// Radio mode...
		buf = buf_fm_stereo;
	}

	// Enable/Disable Tuner access ...
	if (EnableTuner) {
		buf[1] |= 0x80;
	} else {
		buf[1] &= 0x7F;
	}

	DUMP(("tda9887: 0x%02x 0x%02x 0x%02x 0x%02x\n", buf[0],buf[1],buf[2],buf[3]));

	// Init TDA... 
	i2c.WR(i2caddr,buf,4,NULL,0); 
}

// We provide AFC status report
bool TDA9887::AFCProvider() 
{
	return true;
}

// To get the AFC information...
int TDA9887::GetAFC()
{
	// Read current status
	BYTE ret = 0;

	// Try to read AFC
	if (!i2c.Read(i2caddr,&ret)) return 0; // No AFC on failure...

	// Convert to an offset
	int val = ((ret >> 1) & 0xF); if (val & 8) val |= (-8);

	// Return the frequency delta...
	return -1 * ((val * 25000) + 12500);
}

void TDA9887::OnPreTunerSet(KS_AnalogVideoStandard vf,HwTuner::TunerMode tm)
{
	m_vf = vf;
	m_tm = tm;
	SetMode(true); // Leave Tuner enabled...
	
}

void TDA9887::OnPostTunerSet()
{
	SetMode(false); // Disable Tuner access ...
}

TDA9887::TDA9887(I2C& bus, const BYTE Addr) :
	i2c(bus) , i2caddr(Addr) 
	, m_vf(KS_AnalogVideoStandard(0)) , m_tm(HwTuner::Tm_TV)
{
	SetMode(true); // Leave Tuner enabled...
}

	
