/*
	PolledIR.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/



#include "PolledIR.hpp"

void PolledIR::StartPolling()
{
	// Get the current code
	LastCode = PeekCurrCode();

	// if a code was available, post it!
	if (LastCode) {
		Codes.Insert(LastCode);
	}

	// Create the autodetection Thread
	bool ret = CreateThread(ThreadProc,this);
	if (!ret) { // if failure, simulate thread termination
		t_action = Exit;
		t_ended.Set();
	} 

	Restart();
}

void PolledIR::EndPolling()
{
	// Make sure to end Thread now!
	if (t_action != Exit) {

		// Reset ended event
		t_ended.Reset();
		// Post our exit command
		t_action = Exit;
		t_exec.Signal();
		// And wait until the exit command is completed
		t_ended.WaitFor();
	}
}

void PolledIR::Restart()
{
	// We don't restart polling, as it is done On-Demand /when the first GetCode command
	// is received)
	bSuspended = false;
}

DWORD PolledIR::GetCode()
{
	// Only if not already suspended
	if (!bSuspended) {
		// Restart the code capturing process if we need to.
		if ( t_action == SuspendChecking ) { // Only if autodetect thread is suspending/suspended ...
			// Reinit autodetection
			t_action = Checking;
			t_exec.Signal();
		}
	}

	// And return the last received code.
	return HwIR::GetCode();
}

// Wait until autodetection is complete!
void PolledIR::Suspend()
{
	if ( t_action == Checking ) { // Only if thread is autodetecting...

		// Clear the suspended notification...
		t_suspended.Reset();

		// Post our suspend command.
		t_action = SuspendChecking;
		t_exec.Signal();

		// And wait until the suspension command is completed.
		t_suspended.WaitFor();
	}
	bSuspended = true;
}

// Out thread proc
VOID PolledIR::ThreadProc(IN PVOID ctx)
{
	PolledIR* ptr = (PolledIR*) ctx;
	ptr->PollProc();

	// Signal we are ended (we do it from an static fn so no hidden this references are used) ...
	ptr->t_ended.Set();

	// And end the Thread!
	ExitThread();
}

void PolledIR::PollProc()
{
	while(1) {

		// Wait for a command
		t_exec.WaitFor();

		// We got it, check requested action.
		switch (t_action) {
			case Exit:
				return;

			case SuspendChecking:
				t_suspended.Set();
				break;

			default:
			case Checking: 

				// We must be checking received IR codes
				while(t_action == Checking) {

					// Get current code
					DWORD CurrCode = PeekCurrCode();

					// If it is different from the last one, post it
					// (unless no code or key release)
					if (CurrCode != LastCode) {
						LastCode = CurrCode;

						// Is it a 'press' keycode ?
						if ( CurrCode ) {

							// Yes, place it into the Queue (if it becomes full, code will be discarded)
							Codes.Insert(CurrCode);
						}
					}

					// Wait 100 ms before trying this again
					mdelay(100);
				}
				break;
		}
	};
}

