/*
	HwTuner.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 * Interfase to hardware tuner. Some functions are required cause the tuner handles
 * FM Stereo reception. SetFrequency assumes we do the fine Tuning ourselves!
 */

#ifndef _HWTUNER_HPP_
#define _HWTUNER_HPP_

#include "Debug.hpp"
#include "CppRt.hpp"
#include "MyTypes.hpp"
#include "..\Enums.hpp"

class HwTuner {

	public:

		// Tuner Modes
		typedef enum {
			Tm_TV,			// Mode TV
			Tm_Radio_AM,	// Mode Radio AM
			Tm_Radio_FM		// Mode Radio FM
		} TunerMode;

		// Tuning method
		typedef enum {
			Tm_Exact,		// Tune directly to the Freq, without fine tuning
			Tm_Fine,		// Specifies that the tuning operation should perform a comprehensive search for the best tuning
			Tm_Coarse		// Specifies that the tuning operation should perform a fast search, attempting only to determine if a valid signal is present
		} TuningMethod;

		// Supported tuners
		typedef enum {
			/* I2C */
			TC_TEMIC_PAL		, /* 0  */
			TC_PHILIPS_PAL_I,	  /* 1  */
			TC_PHILIPS_NTSC,	  /* 2  */
			TC_PHILIPS_SECAM,	  /* 3  */
			TC_PHILIPS_PAL,		  /* 4  */
			TC_TEMIC_NTSC,		  /* 5  */
			TC_TEMIC_PAL_I,		  /* 6  */
			TC_TEMIC_4036FY5_NTSC,/* 7  */
			TC_ALPS_TSBH1_NTSC,	  /* 8  */
			TC_ALPS_TSBE1_PAL,	  /* 9  */
			TC_ALPS_TSBB5_PAL_I,  /* 10 */
			TC_ALPS_TSBE5_PAL,	  /* 11 */
			TC_ALPS_TSBC5_PAL,	  /* 12 */
			TC_TEMIC_4006FH5_PAL, /* 13 */
			TC_ALPS_TSHC6_NTSC,	  /* 14 */
			TC_TEMIC_PAL_DK,	  /* 15 */
			TC_TEMIC_NTSC_M,	  /* 16 */
			TC_TEMIC_4066FY5_PAL_I,     /* 17 */
			TC_TEMIC_4006FN5_MULTI_PAL,  /* 18 */  /* B/G, I and D/K autodetected */
			TC_TEMIC_4009FR5_PAL,      /* 19 */  /* incl. FM radio */
			TC_TEMIC_4039FR5_NTSC,  /* 20 */  /* incl. FM radio */
			TC_TEMIC_4046FM5,       /* 21 */ /* you must actively select B/G, D/K, I, L, L` !  */
			TC_PHILIPS_PAL_DK,		/* 22 */
			TC_PHILIPS_FQ1216ME,	/* 23 */ /* you must actively select B/G/D/K, I, L, L` */

			TC_LG_PAL_I_FM,			/* 24 */
			TC_LG_PAL_I,			/* 25 */
			TC_LG_NTSC_FM,			/* 26 */
			TC_LG_PAL_FM,			/* 27 */
			TC_LG_PAL,				/* 28 */

			TC_TEMIC_4009FN5_MULTI_PAL_FM, /* 29 */
			TC_SHARP_2U5JF5540_NTSC,		/* 30 */
			TC_Samsung_PAL_TCPM9091PD27,	/* 31 */
			TC_TEMIC_4106FH5, 				/* 32 */	/* 4106 FH5 (3X 7808, 7865)*/
			TC_TEMIC_4012FY5,				/* 33 */	/* 4012 FY5 (3X 0971, 1099)*/
			TC_TEMIC_4136FY5,				/* 34 */	/* 4136 FY5 (3X 7708, 7746)*/
			TC_LG_PAL_NEW_TAPC,				/* 35 */


			TC_PHILIPS_1236D_NTSC_INPUT1, /* 36 */
			TC_PHILIPS_1236D_NTSC_INPUT2, /* 37 */

			TC_MT2032,						/* 38 */
			TC_MT2050,						/* 39 */

			/* Non I2C */
			TC_TEA5757=0x100,	  /* 40 */

			/* Absent */
			TC_Absent = 0xfe,
			/* Unknown */
			TC_Unknown = 0xff
		} Tuner;

	public:

		// constructors and destructors... Need to be declared here as we want the 'virtual' keyword for destructor
		HwTuner() {}
		virtual ~HwTuner() {}

		// Called to Set Frequency: Returns 0 if carrier not detected.
		virtual DWORD SetFrequency(
			KS_AnalogVideoStandard vf,	// VideoFormat of Signal (For TV mode)
			TunerMode		mode,		// Tuner Mode (TV/FM/AM)
			DWORD			Frecuency,	// Frequency of channel
			TuningMethod	method) = 0;// Tuning method

		// Called to set audio mode (Some tuners need this cause they handle FM Stereo Decoding)
		virtual bool SetAudioMode(
			AudioMode audio
			)											{ return true; }

		// To get capacities of Tuner
		virtual bool CanTuneTV()						{ return false; }
		virtual bool TVTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			)											{ return false; }
		virtual bool TVStereoAvailable()				{ return false; }
		virtual bool TVStereoDetected()					{ return false; }
		virtual ULONG TVTunerVideoStandardsSupported()	{ return 0; }


		virtual bool CanTuneAM()						{ return false; }
		virtual bool AMTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			)											{ return false; }
		virtual bool AMStereoAvailable()				{ return false; }
		virtual bool AMStereoDetected()					{ return false; }

		virtual bool CanTuneFM()						{ return false; }
		virtual bool FMTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			)											{ return false; }
		virtual bool FMStereoAvailable()				{ return false; }
		virtual bool FMStereoDetected()					{ return false; }

		// Input selection
		virtual DWORD NumberOfInputs()					{ return 1; }
		virtual bool SetInput(DWORD input)				{ return true; }

		// To support Hibernate mode
		virtual void Restart( void )					{}
		virtual void Suspend( void )					{}

};




#endif