/*
	I2CTuner.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
   LOG:

 01/01/03 Fred Hoogduin <fred@hoogduin.net>  
	-New tuning algorithm... (faster and more reliable TV channel tuning and Scanning)
	 (thanks a lot!)
*/

#include "I2CTuner.hpp"

/* tv standard selection for Temic 4046 FM5
   this value takes the low bits of control byte 2
   from datasheet Rev.01, Feb.00 
     standard     BG      I       L       L2      D
     picture IF   38.9    38.9    38.9    33.95   38.9
     sound 1      33.4    32.9    32.4    40.45   32.4
     sound 2      33.16   
     NICAM        33.05   32.348  33.05           33.05
 */
#define TEMIC_SET_PAL_I         0x05
#define TEMIC_SET_PAL_DK        0x09
#define TEMIC_SET_PAL_L         0x0a // SECAM ?
#define TEMIC_SET_PAL_L2        0x0b // change IF !
#define TEMIC_SET_PAL_BG        0x0c

/* tv tuner system standard selection for Philips FQ1216ME
   this value takes the low bits of control byte 2
   from datasheet "1999 Nov 16" (supersedes "1999 Mar 23")
     standard 		BG	DK	I	L	L`
     picture carrier	38.90	38.90	38.90	38.90	33.95
     colour		34.47	34.47	34.47	34.47	38.38
     sound 1		33.40	32.40	32.90	32.40	40.45
     sound 2		33.16	-	-	-	-
     NICAM		33.05	33.05	32.35	33.05	39.80
 */
#define PHILIPS_SET_PAL_I		0x01 /* Bit 2 always zero !*/
#define PHILIPS_SET_PAL_BGDK	0x09
#define PHILIPS_SET_PAL_L2		0x0a
#define PHILIPS_SET_PAL_L		0x0b	

/* system switching for Philips FI1216MF MK2
   from datasheet "1996 Jul 09",
 */
#define PHILIPS_MF_SET_BG		0x01 /* Bit 2 must be zero, Bit 3 is system output */
#define PHILIPS_MF_SET_PAL_L	0x03
#define PHILIPS_MF_SET_PAL_L2	0x02

/* ---------------------------------------------------------------------- */

/*
 *	The floats in the tuner struct are computed at compile time
 *	by cc and cast back to integers. Thus we don't violate the
 *	"no float in kernel" rule.
 */

#define MHZ(x) (WORD(x * 4))

I2CTuner::Type I2CTuner::Info [] = {

	{/* "Temic 4002 FH5 [NORM B/G]",*/ I2CTuner::TEMIC, TN_CCIR_BGHN, false, 
		MHZ(48.25) , MHZ(144.00),MHZ(467.00), MHZ(855.25), MHZ(38.90),
		0x02,0x04,0x01}, //ok!

	{/* "Philips PAL_I ",*/ I2CTuner::Philips, TN_CCIR_I, true,
		MHZ(45.75) , MHZ(140.25),MHZ(463.25), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

	{/* "Philips NTSC",*/ I2CTuner::Philips, TN_CCIR_MC, true,
		MHZ(55.25) , MHZ(157.25),MHZ(451.25), MHZ(801.25), MHZ(45.75),
		0xA0,0x90,0x30},

	{/* "Philips SECAM",*/ I2CTuner::Philips, TN_CCIR_I | TN_CCIR_BGHN | TN_CCIR_DKL, true, // PAL BDGHIL
		MHZ(45.75) , MHZ(168.25),MHZ(447.25), MHZ(855.25), MHZ(38.90),
		0xA7,0x97,0x37},

	{/* "Philips PAL",*/ I2CTuner::Philips, TN_CCIR_BGHN, true,// Fi1216MK2 CCIR B/G
		MHZ(45.75) , MHZ(168.25),MHZ(447.25), MHZ(855.25), MHZ(38.90),
		0xA0,0x90,0x30},

	{/* "Temic 4032 FY5 [NORM M/NC]",*/ I2CTuner::TEMIC, TN_CCIR_MC, false,
		MHZ(54.00) , MHZ(158.00),MHZ(464.00), MHZ(801.25), MHZ(45.75),
		0x02,0x04,0x01}, //ok!

	{/* "Temic 4062 FY5 [NORM I]",*/ I2CTuner::TEMIC, TN_CCIR_I, false,
		MHZ(45.75) , MHZ(144.00),MHZ(467.00), MHZ(855.25), MHZ(38.90),
		0x02,0x04,0x01}, //ok!

 	{/* "Temic 4036 FY5 [NORM M/NC]",*/ I2CTuner::TEMIC, TN_CCIR_MC, false,
		MHZ(54.00) , MHZ(158.00),MHZ(453.00), MHZ(801.25), MHZ(45.75),
		0xa0,0x90,0x30}, //ok!

	{/* "Alps HSBH1",*/ I2CTuner::TEMIC, TN_CCIR_MC, true,
		MHZ(55.25)  , MHZ(137.25) ,MHZ(385.25) , MHZ(801.25) , MHZ(45.75) ,
		0x01,0x02,0x08},

	{/* "Alps TSBE1",*/ I2CTuner::TEMIC, TN_CCIR_BGHN, true,
		MHZ(45.75)  , MHZ(137.25) ,MHZ(385.25) , MHZ(855.25) , MHZ(45.75) ,
		0x01,0x02,0x08},

	{/* "Alps TSBB5",*/ I2CTuner::Alps, TN_CCIR_I, true, /* tested (UK UHF) with Modtec MM205 */
		MHZ(45.75)  , MHZ(133.25) ,MHZ(351.25) , MHZ(855.25) , MHZ(39.50) ,
		0x01,0x02,0x08},

	{/* "Alps TSBE5",*/ I2CTuner::Alps, TN_CCIR_BGHN, true,/* untested - data sheet guess. Only IF differs. */
		MHZ(45.75)  , MHZ(133.25) ,MHZ(351.25) , MHZ(855.25) , MHZ(38.90) ,
		0x01,0x02,0x08},

	{/* "Alps TSBC5",*/ I2CTuner::Alps, TN_CCIR_BGHN, true,/* untested - data sheet guess. Only IF differs. */
		MHZ(45.75)  , MHZ(133.25) ,MHZ(351.25) , MHZ(855.25) , MHZ(38.00) ,
		0x01,0x02,0x08},

	{/* "Temic 4006 FH5 [NORM B/G]",*/ I2CTuner::TEMIC,TN_CCIR_BGHN, false,
		MHZ(48.25) , MHZ(169.00),MHZ(454.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30}, //ok!

  	{/* "Alps TSCH6",*/ I2CTuner::Alps, TN_CCIR_MC, true,
  		MHZ(55.25)  , MHZ(137.25) ,MHZ(385.25) , MHZ(801.25) , MHZ(45.75) ,
		0x14,0x12,0x11},

  	{/* "Temic 4016 FY5 [NORM D/K/L]",*/ I2CTuner::TEMIC, TN_CCIR_DKL , false,
		MHZ(49.75) , MHZ(169.00),MHZ(454.00), MHZ(863.25), MHZ(38.90),
		0xa0,0x90,0x30}, //ok!

  	{/* "Philips NTSC_M (MK2)",*/ I2CTuner::Philips, TN_CCIR_MC, true,
  		MHZ(54.00) , MHZ(160.00),MHZ(454.00), MHZ(801.25), MHZ(45.75),
		0xa0,0x90,0x30},

	{/* "Temic 4066 FY5 [NORM I]",*/ I2CTuner::TEMIC, TN_CCIR_I,false,
		MHZ(45.75) , MHZ(169.00),MHZ(454.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30}, //ok!

	{/* "Temic 4006 FN5 [NORM* auto]",*/ I2CTuner::TEMIC, TN_CCIR_BGHN | TN_CCIR_I | TN_CCIR_DKL, false,
		MHZ(45.75), MHZ(169.00), MHZ(454.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

	{/* "Temic 4009 FR5 [NORM B/G] + FM",*/ I2CTuner::TEMIC, TN_CCIR_BGHN , true,
		MHZ(45.75), MHZ(141.00), MHZ(464.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

	{/* "Temic 4039 FR5 [NORM M/NC] + FM",*/ I2CTuner::TEMIC, TN_CCIR_MC, true,
		MHZ(54.00) ,MHZ(158.00), MHZ(453.00), MHZ(801.25), MHZ(45.75),
		0xa0,0x90,0x30},

	{/* "Temic 4046 FM5 [NORM multi]",*/ I2CTuner::TEMIC, TN_CCIR_I | TN_CCIR_DKL | TN_CCIR_BGHN, false, // PAL_IDKLBG
		MHZ(45.75), MHZ(169.00), MHZ(454.00), MHZ(855.25),  MHZ(38.90),
		0xa0,0x90,0x30},

	{/* "Philips PAL_DK",*/ I2CTuner::Philips, TN_CCIR_DKL, true,
		MHZ(45.75), MHZ(170.00),MHZ(450.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

	{/* "Philips PAL/SECAM multi (FQ1216ME)",*/ I2CTuner::Philips, TN_CCIR_I | TN_CCIR_DKL | TN_CCIR_BGHN, true, // PAL_IBGDKL
		MHZ(45.75), MHZ(170.00),MHZ(450.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

    {/* "LG TAPC-I001D [NORM I] + FM",*/ I2CTuner::LGINNOTEK, TN_CCIR_I, true,
        MHZ(45.75), MHZ(170.00),MHZ(450.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

    {/* "LG TAPC-I701D [NORM I]",*/ I2CTuner::LGINNOTEK, TN_CCIR_I, false,
        MHZ(45.75), MHZ(170.00),MHZ(450.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30}, 

    {/* "LG TPI8NSR01F [NORM M/NC] + FM",*/ I2CTuner::LGINNOTEK, TN_CCIR_MC, true,
        MHZ(54.00) ,MHZ(210.00),MHZ(497.00), MHZ(801.25), MHZ(45.75),
		0xa0,0x90,0x30},

    {/* "LG TPI8PSB01D [NORM B/G] + FM",*/ I2CTuner::LGINNOTEK, TN_CCIR_BGHN, true,
        MHZ(45.75), MHZ(170.00),MHZ(450.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

    {/* "LG TPI8PSB11D [NORM B/G]",*/ I2CTuner::LGINNOTEK, TN_CCIR_BGHN, false,
        MHZ(45.75), MHZ(170.00),MHZ(450.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

	{/* "Temic 4009 [NORM* Auto] + FM",*/ I2CTuner::TEMIC, TN_CCIR_I | TN_CCIR_DKL | TN_CCIR_BGHN, true,
		MHZ(45.75), MHZ(141.00), MHZ(464.00), MHZ(855.25), MHZ(38.90),
		0xa0,0x90,0x30},

	{/* "SHARP NTSC_JP (2U5JF5540)",*/ I2CTuner::Sharp, TN_CCIR_MC, true,/* 940=16*58.75 NTSC@Japan */
		MHZ(54.00),MHZ(137.25),MHZ(317.25), MHZ(801.25), MHZ(58.75),
		0x01,0x02,0x08},

	{/* "Samsung TCPM9091PD27 [NORM B/G/I/D/K]",*/ I2CTuner::Samsung, TN_CCIR_BGHN | TN_CCIR_DKL | TN_CCIR_I , false, /* from sourceforge v3tv */
		MHZ(45.75),MHZ(169),MHZ(464),MHZ(855.25), MHZ(38.90), 
		0xA0,0x90,0x30},

	{/* "Temic 4106 FH5 [NORM B/G]",*/ I2CTuner::TEMIC, TN_CCIR_BGHN, false,
		MHZ(45.75), MHZ(141.00),MHZ(464.00),MHZ(855.25), MHZ(38.90), 
		0xa0,0x90,0x30},

	{/* "Temic 4012 FY5 [NORM D/K/L]",*/ I2CTuner::TEMIC, TN_CCIR_DKL, false,
		MHZ(49.75), MHZ(140.25),MHZ(460.00),MHZ(863.25), MHZ(38.00), 
		0x02,0x04,0x01},

	{/* "Temic 4136 FY5 [NORM M/NC]",*/ I2CTuner::TEMIC, TN_CCIR_MC,false,
		MHZ(54.00) ,MHZ(158.00),MHZ(453.00),MHZ(801.25), MHZ(45.75), 
		0xa0,0x90,0x30},

	{/* "LG PAL (newer TAPC series)",*/ I2CTuner::LGINNOTEK, TN_CCIR_BGHN,true,
        MHZ(45.75),MHZ(170.00),MHZ(450.00), MHZ(855.25), MHZ(38.90), 
		0x01,0x02,0x08},

	{/* "Philips 1236D ATSC/NTSC Input 1",*/ I2CTuner::Philips, TN_CCIR_MC, true,
		MHZ(54.00) , MHZ(157.25),MHZ(451.25), MHZ(801.25), MHZ(45.75), 
		0xA3,0x93,0x33},

	{/* "Philips 1236D ATSC/NTSC Input 2",*/ I2CTuner::Philips, TN_CCIR_MC, true,
		MHZ(54.00) , MHZ(157.25),MHZ(451.25), MHZ(801.25), MHZ(45.75), 
		0xA2,0x92,0x32},

};


/* ---------------------------------------------------------------------- */
BYTE I2CTuner::GetStatus()
{
	BYTE stat;
	if (!i2c.Read(i2caddr,&stat)) 
		stat = 0;
	return stat;
}

// Returns 0 on correct frequency
int I2CTuner::GetAFCStatus()
{
	int Stat = (GetStatus() & I2CTUNER_AFC) - 2;
	// Alps tuner has a slight problem: values 0 AND 1 are correct tuning, so fix that
	if (Info[m_type].Vendor == Alps && Stat > 0) {
		Stat --;
	}
	return Stat;
}

void I2CTuner::SetTvFreq(DWORD dwfreq, KS_AnalogVideoStandard vf /* VideoFormat */, bool FastTune)
{
	BYTE config = 0;
	
	WORD unit = WORD(250000 / m_step);
	WORD freq = WORD(dwfreq / m_step);
	if (freq < Info[m_type].thresh1 * unit) 
		config = Info[m_type].VHF_L;
	else if (freq < Info[m_type].thresh2 * unit) 
		config = Info[m_type].VHF_H;
	else
		config = Info[m_type].UHF;

	/* tv norm specific stuff for multi-norm tuners */
	switch (m_type) {
	case TC_PHILIPS_SECAM:
		/* 0x01 -> ??? no change ??? */
		/* 0x02 -> PAL BDGHI / SECAM L */
		/* 0x04 -> ??? PAL others / SECAM others ??? */
		config &= ~0x02;
		if (vf &
			(KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_D |
			 KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H |
			 KS_AnalogVideo_PAL_I | KS_AnalogVideo_SECAM_L) ) {
			config |= 0x02;
		}
		break;

	case TC_TEMIC_4046FM5:
		config &= ~0x0f;
		if (vf & KS_AnalogVideo_PAL_I) {
			config |= TEMIC_SET_PAL_I;
		} else 
		if (vf & KS_AnalogVideo_PAL_D) {
			config |= TEMIC_SET_PAL_DK;
		} else 
		if (vf & KS_AnalogVideo_SECAM_L) {
			config |= TEMIC_SET_PAL_L;
		} else 
		if (vf & 
			(KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_G) ) {
			config |= TEMIC_SET_PAL_BG;
		}
		break;
	case TC_PHILIPS_FQ1216ME:
		config &= ~0x0f;
		if (vf & KS_AnalogVideo_PAL_I) {
			config |= PHILIPS_SET_PAL_I;
		} else if (vf & KS_AnalogVideo_SECAM_L) {
			config |= PHILIPS_SET_PAL_L;
		} else if ( vf & 
			(KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_G |
			KS_AnalogVideo_PAL_D | KS_AnalogVideo_SECAM_K) ) {
			config |= PHILIPS_SET_PAL_BGDK;
		}
		break;
	}

	// If fasttuning, flag it!
	if (FastTune) {
		config |= m_fast;
	}

	WORD div = freq + WORD(Info[m_type].IF * unit);
	
    /*
     * Philips FI1216MK2 remark from specification :
     * for channel selection involving band switching, and to ensure
     * smooth tuning to the desired channel without causing
     * unnecessary charge pump action, it is recommended to consider
     * the difference between wanted channel frequency and the
     * current channel frequency.  Unnecessary charge pump action
     * will result in very low tuning voltage which may drive the
     * oscillator to extreme conditions.
     */
    /*
     * Progfou: specification says to send config data before
     * frequency in case (wanted frequency < current frequency).
     */

	if (m_type == TC_PHILIPS_SECAM && dwfreq < m_freq) {
		i2c.Write(i2caddr,m_config,config);
		i2c.Write(i2caddr,(div >> 8) & 0x7f,div & 0xff);
	} else {
		i2c.Write(i2caddr,(div >> 8) & 0x7f,div & 0xff);
		i2c.Write(i2caddr,m_config,config);
	}
	m_freq = dwfreq;
}

/*
Norm:		Audio Carrier	Video Bandwidth

SECAM_DKK1L	6.5MHz			6.0Mhz
PAL_D		6.5MHz			6.0MHz

PAL_I		6.0Mhz			5.5MHz

PAL_BGH		5.5Mhz			5.0MHz
PAL_N		5.5MHz			5.0MHz
SECAM_BG	5.5MHz			5.0MHz

NTSC_M		4.5Mhz			4.2MHz
PAL_M		4.5Mhz			4.2MHz
PAL_NC		4.5Mhz			4.2MHz

 */

void I2CTuner::SetRadioFreq(DWORD dwfreq)
{

	if (m_bUseTVAudioforFM) {
		// Find out type of tuner...
		if (Info[m_type].Norms & KS_AnalogVideo_NTSC_M) {

			// Tuner supports NTSC, adjust frecuency appropiately and tune the TV Channel
			SetTvFreq(dwfreq - 4500000 ,KS_AnalogVideo_NTSC_M,false);

		} else 
		if (Info[m_type].Norms & KS_AnalogVideo_PAL_B) {

			// Tuner supports PAL, adjust frecuency appropiately and tune the TV Channel
			SetTvFreq(dwfreq - 5500000 ,KS_AnalogVideo_PAL_B,false);

		}else 
		if (Info[m_type].Norms & KS_AnalogVideo_PAL_D) {

			// Tuner supports PAL, adjust frecuency appropiately and tune the TV Channel
			SetTvFreq(dwfreq - 6500000 ,KS_AnalogVideo_PAL_D,false);

		}else 
		if (Info[m_type].Norms & KS_AnalogVideo_PAL_I) {

			// Tuner supports PAL, adjust frecuency appropiately and tune the TV Channel
			SetTvFreq(dwfreq - 6000000 ,KS_AnalogVideo_PAL_I,false);

		}
	} else {
		
		BYTE config = 0xa5 | ((m_bForceMono) ? 2 : 0);
		WORD div = WORD((dwfreq + 10700000) / 50000);
  		div &= 0x7fff;
		i2c.Write(i2caddr,(div >> 8) & 0x7f,div & 0xff);
		i2c.Write(i2caddr,0x88,config);
	}
	m_freq = dwfreq;
}

// So we can power-down the tuner...
void I2CTuner::Suspend( void )
{
	WORD TV_POWERDOWN    = 0x8EA1;  // Philips TV power down 

	// Just power-down the chip...
	i2c.Write(i2caddr,(TV_POWERDOWN >> 8) & 0xff,TV_POWERDOWN & 0xff);
}



/* ---------------------------------------------------------------------- */

I2CTuner::I2CTuner(	I2C& bus,const BYTE Addr,const Tuner ttype,
					const bool DoesFMNatively, const bool UseTVAudioforFM,
   					const bool FastTune,			// Use fastuning
					const bool HalfStep			// Use halfstep

				   )
	: i2c(bus) , i2caddr(Addr) 
	, m_fast((FastTune) ? 0x40 : 0)
	, m_config((HalfStep) ? 0x8a : 0x8e)
	, m_step((HalfStep) ? 31250 : 62500)
	, m_type(ttype) , m_freq(100000000) 
	, m_mode(Tm_TV) , m_bForceMono(false) 
	, m_bDoesFMNatively(DoesFMNatively)
	, m_bUseTVAudioforFM(UseTVAudioforFM)
{
}

// Called to Set Frecuency: Sets frecuency and Does Autotuning - Returns null if signal was not detected!
DWORD I2CTuner::SetFrequency(
		KS_AnalogVideoStandard	vf,			// VideoFormat of Signal (For TV mode)
		TunerMode				tm,			// Tuner Mode (TV/FM/AM)
		DWORD					Frequency,	// Frecuency of channel
		TuningMethod			method)		// Tuning method
{
	// Store Tuner Mode
	m_mode = tm;
	if (m_mode == Tm_TV) {
		if ( Frequency < ULONG(Info[m_type].min) * 250000UL ||
			Frequency > ULONG(Info[m_type].max) * 250000UL ) {
			return 0; // Out of bounds, return signal not present
		}
		return SetTVFreqWithAFC(Frequency,vf,method);
	}
	else {
		if ( Frequency < 68000000 ||
			Frequency > 108000000 ) {
			return 0; // Out of bounds, return signal not present
		}
		return SetRadioFreqWithAFC(Frequency);
	}
}

bool I2CTuner::SetAudioMode(
			AudioMode audio
			)
{
	m_bForceMono = (audio == Am_Mono);
	if (m_mode == Tm_Radio_FM) {
		if ( m_freq < 68000000 ||
			m_freq > 108000000 ) {
			return true; // Out of bounds, return OK.
		} 
		SetRadioFreqWithAFC(m_freq);
	}
	return true;	
}

// To get capacities of Tuner
bool I2CTuner::CanTuneTV()
{
	return true;
}

// the routine to autotune radio: Returns 0 on signal not found
DWORD I2CTuner::SetRadioFreqWithAFC(DWORD freq)
{
	SetRadioFreq(freq);
	return freq;
}


/////////////////////////////////////////////////////////////////////////////
// Method:  DWORD I2CTuner::SetTVFreqWithAFC(DWORD Frec, KS_AnalogVideoStandard vf)
// Purpose: Set channel with Automatic Fine Turning
// Note:    This function was provided by Intel Corp. and has been modified
//          for our need.
//			Now, we modified this fn to speed up tuning. If yolu want the old method,
//			just define OLD_TUNING_WAY and recompile the driver
/////////////////////////////////////////////////////////////////////////////
DWORD I2CTuner::SetTVFreqWithAFC(DWORD dwFreq, KS_AnalogVideoStandard vf,TuningMethod method)
{
	int		n;					// iteration count
	int		m=0;				// how many times was PLL not locked?
    bool	centered = false;	// "tuned in" flag
    DWORD	SearchFreq;			// our temporary copy of frequency bits

#ifdef OLD_TUNING_WAY
	// ## Old method...

	//--------------------------
	// Array of frequency offset values for AFT search
	// This zig-zags -- -1/4 MHz, +1/4MHz, -1/2MHz, etc., to +2-1/2 MHz
	//--------------------------

    static int  freqdelta[] = {
			- 250000,  250000,  
			- 500000,  500000, 
			- 750000,  750000,
			-1000000, 1000000,
			-1250000, 1250000,
			-1500000, 1500000,
			-1750000, 1750000,
			-2000000, 2000000,
			-2250000, 2250000,
			-2500000, 2500000 };

	// # iterations in AFT loop & in search table
    const int ITERATIONS = sizeof( freqdelta ) / sizeof( int );    

	// Number of PLL locking Attempts
	const int PLL_ATTEMPTS		= 50;	// 50 attempts
	const int PLL_SETTLE_DELAY	= 10;	// 10 ms

#else

	// ## New method...
	const int ITERATIONS = 20;	// # iterations in AFC loop,

	// Max search range in Hz
	const int MAXRANGE = 250000;

	// Number of PLL locking Attempts
	const int PLL_ATTEMPTS		= 25;	// 25 attempts
	const int PLL_SETTLE_DELAY	= 15;	// 15 ms
#endif

	//--------------------------
	// Set frecuency
	//--------------------------
	SearchFreq = dwFreq;
	SetTvFreq(dwFreq,vf,true);

	// If Method == Exact, end Here. 
	if (method == Tm_Exact) {
		// We must try to find out if signal was detected here

		do { // Repeat until PLL locked or timeout.
			// we have to wait approx. PLL_SETTLE_DELAY ms for tuner to lock
			mdelay(PLL_SETTLE_DELAY);
			// Assume PLL failure
			m++;
		} while (!IsLocked() && m < PLL_ATTEMPTS);

		// If PLL didnt lock, or signal was not detected, return 0 (No carrier found)
		if (m >= PLL_ATTEMPTS || GetAFCStatus() >= 2) {
			DUMP(( "SetTVFreqWithAFC not on %d, AFC %d\n", SearchFreq, GetAFCStatus() ));
			return 0; // Carrier not found!
		}

		// Else, Carrier found, return frequency.
		DUMP(( "SetTVFreqWithAFC exact on %d, AFC %d\n", SearchFreq, GetAFCStatus() ));
		SetTvFreq(SearchFreq,vf,false); // Normal tuning mode
		return SearchFreq;
	}

	//----------------------------------------------------
	// Loop to settle in on final frequency
	// First, delay to allow tuner PLL to settle
	//   Tuner specs claim 150 msec as settling time
	//   We are delaying a shorter time, then checking for lock.
	//----------------------------------------------------

	DUMP(( "SetTVFreqWithAFC searching around %d, method %d\n", dwFreq, method ));
    for (n=0; (n < ITERATIONS) && (m < PLL_ATTEMPTS); ) {

		//--------------------------
		// Now take a look at status, see if we are centered on a channel.
		//--------------------------

		// we have to wait approx. PLL_SETTLE_DELAY ms for tuner to lock
		mdelay(PLL_SETTLE_DELAY);

		//--------------------------
		// Test Phase lock condition
		//  (if phase lock is not achieved, we will just repeat the loop,
		//   without changing frequency, giving PLL more time to settle out).
		//--------------------------
        if ( IsLocked() ) {

#ifdef OLD_TUNING_WAY

			//--------------------------
			// PLL Locked, test AFC condition
			//--------------------------
			switch ( GetAFCStatus() )
            {
				case -2:     // carrier sensed, need to lower frequency
				case -1:
					SearchFreq -= m_step;
					SetTvFreq(SearchFreq,vf, (method != Tm_Coarse) ); // Fast tuning if no coarse mode
					// If coarse method, end Here
					if (method == Tm_Coarse) {
						DUMP(( "SetTVFreqWithAFC coarse on %d, AFC %d\n", SearchFreq, GetAFCStatus() ));
						return SearchFreq; // Signal was detected
					}
					break;
				case 0:		// carrier sensed, we are on correct frequency
					centered = true;
					// If coarse method, end Here
					if (method == Tm_Coarse) {
						DUMP(( "SetTVFreqWithAFC coarse on %d, AFC %d\n", SearchFreq, GetAFCStatus() ));
						SetTvFreq(SearchFreq,vf, false ); // Normal tuning
						return SearchFreq; // Signal was detected
					}
					break;
				case 1:		// carrier sensed, need to raise frequency
					SearchFreq += m_step;
					SetTvFreq(SearchFreq,vf, (method != Tm_Coarse) ); // Fast tuning if no coarse mode);
					// If coarse method, end Here
					if (method == Tm_Coarse) {
						DUMP(( "SetTVFreqWithAFC coarse on %d, AFC %d\n", SearchFreq, GetAFCStatus() ));
						return SearchFreq; // Signal was detected
					}
					break;
				default:
					// carrier not sensed, search using offset table
					SearchFreq = dwFreq + freqdelta[n];
					SetTvFreq(SearchFreq,vf,true);
					break;
            } // switch

#else

			//--------------------------
			// PLL Locked, test AFC condition
			//--------------------------
			int iAFC = GetAFCStatus();
			DUMP(( "  AFC %d at %d Hz\n", iAFC, SearchFreq ));

			// Check if channel is properly tuned...
			if ( iAFC == 0 ) {

				// Centered, we are on correct frequency
				centered = true;

			} else {

				// Adjust frequency based on AFC info, and tune again
				SearchFreq += (iAFC * (int) m_step);
				if ( abs( (int)SearchFreq - (int)dwFreq) <= MAXRANGE ) {

					SetTvFreq(SearchFreq,vf, true );

				} else {

					// Do not try to tune out of freq. range
					break;

				}
			}
#endif

			//------------
			// Increment n to control loop and access next frequency search offset
			//------------
            n++;
            m = 0;         // Reset PLL lock attempt counter

        } // if phase locked

		else {

			//--------------------------
			// Else PLL not locked.
			// Don't increment n.  Simply loop again until PLL is locked.
			//--------------------------
			m ++;
        } // else not phase locked

		//--------------------------
		// If we detected centered frequency,  break out of loop (we are done).
		// else repeat loop.
		// If phase lock had not been detected, we did not change tuner settings,
		//   we will just repeat the time delay, and check again for phase lock.
		//--------------------------
        if (centered) {

            break; // out of for loop

        } // if centered

	} // for ITERATIONS iterations or for max PLL_ATTEMPTS to lock

	//----------------------------------------------------
	// Tuning loop is done, let's see how we did
	//----------------------------------------------------
	if ( centered ) {
		DUMP(( "SetTVFreqWithAFC centered on %d, AFC %d\n", SearchFreq, GetAFCStatus() ));
		SetTvFreq(SearchFreq,vf,false);
		return SearchFreq; // Signal was detected
	}

	//--------------------------
	// Else no carrier, return
	//--------------------------
	DUMP(( "SetTVFreqWithAFC not on %d, AFC %d\n", dwFreq, GetAFCStatus() ));
	SetTvFreq(dwFreq,vf,false);
	return 0; // No Carrier detected, so return 0!
}

bool I2CTuner::TVTunerCaps(
	ULONG& MinFrecuency,
	ULONG& MaxFrecuency,
	ULONG& TuningGranularity,
	ULONG& SettlingTime
	)
{
	MinFrecuency		=  ULONG(Info[m_type].min) * 250000UL;
	MaxFrecuency		=  ULONG(Info[m_type].max) * 250000UL;
	TuningGranularity	=  m_step;
	SettlingTime		=  1; // ms (does not matter, we will tune ourselves) (i do not like 0 values)
	return true;
}

bool I2CTuner::TVStereoAvailable()
{
	return false;
}

bool I2CTuner::TVStereoDetected()
{
	return false;
}

ULONG I2CTuner::TVTunerVideoStandardsSupported()
{
	return Info[m_type].Norms;
}

bool I2CTuner::CanTuneFM()
{
	return m_bDoesFMNatively ||	m_bUseTVAudioforFM;
}

bool I2CTuner::FMTunerCaps(
	ULONG& MinFrecuency,
	ULONG& MaxFrecuency,
	ULONG& TuningGranularity,
	ULONG& SettlingTime
)
{
	MinFrecuency		=  68000000;
	MaxFrecuency		= 108000000;
	TuningGranularity	=	  50000;
	SettlingTime		=		300; // ms
	return true;
}

bool I2CTuner::FMStereoAvailable()
{
	return m_bDoesFMNatively;	// TV audio used for FM reception wont be stereo.
}

bool I2CTuner::FMStereoDetected()
{
	return m_bDoesFMNatively;
}


