/*
	I2CTuner.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef _I2CTUNER_HPP_
#define _I2CTUNER_HPP_

#include "HwTuner.hpp"
#include "..\HwCapture\Pisces.hpp"

// I2C Addresses of Tuners
#define I2C_TUNER_0 0xC2
#define I2C_TUNER_1 0xC4
#define I2C_TUNER_2 0xC6
#define I2C_TUNER_3 0xC0 /* Floating pin... Bad idea !!!*/
#define I2C_TUNER_4 0xC8
#define I2C_TUNER_5 0xCA
#define I2C_TUNER_6 0xCC
#define I2C_TUNER_7 0xCE 

#define I2CTUNER_POR       0x80
#define I2CTUNER_FL        0x40
#define I2CTUNER_MODE      0x38
#define I2CTUNER_AFC       0x07

#define I2CTUNER_STEREO    0x10 /* radio mode */
#define I2CTUNER_SIGNAL    0x07 /* radio mode */


// This class represents an I2C Tuner
class I2CTuner : public HwTuner {

	public:

		// Vendors
		typedef enum {
			TEMIC,
			Philips,
			Sony,
			Alps,
			LGINNOTEK,
			Sharp,
			Samsung,
			Microtune,
			NoTuner
		} Vendor;

		// Type of tuners
		typedef struct {
			/*char * name; */
			
			Vendor	Vendor;
			ULONG	Norms;

			// If it supoorts FM tuning
			BOOL	fm;

			// Minimum frequency
			WORD	min;

			// Thresholds for band changing
			WORD	thresh1; /* VHF-L -> VHF-H */   
			WORD	thresh2; /* VHF-H -> UHF */

			// Maximum frequency
			WORD	max;

			// Intermediate frecuency
			WORD	IF;
			
			// Code for Bands
			BYTE	VHF_L;
			BYTE	VHF_H;
			BYTE	UHF;

			BYTE	config;		// Code to send configuration to tuner

		} Type;

	public:

		static Type Info[];	// Info for each tuner

	private:
		
		const BYTE	m_fast;				// Bit to use to get fast tuning
		const BYTE	m_config;			// Config byte to use to tune TV
		const DWORD m_step;				// Step size
		const Tuner	m_type;				// chip type 
		DWORD		m_freq;				// Current frecuency
		TunerMode	m_mode;				// Active mode for tuner
		bool		m_bForceMono;		// Force mono output
		const bool	m_bDoesFMNatively;	// If tuner supports FM natively
		const bool	m_bUseTVAudioforFM;	// Tries to get FM mono radio with the TV tuner audio circuit.

		// Interfase
		I2C&		i2c;
		const BYTE	i2caddr;

	private:

		// Private Fns
		BYTE GetStatus();
		bool IsLocked() { return (GetStatus() & I2CTUNER_FL) != 0; }
		int  GetAFCStatus();
		int  GetSignal() { return (GetStatus() & I2CTUNER_SIGNAL)<<13; }

		void SetTvFreq(DWORD freq, KS_AnalogVideoStandard vf, bool FastTune = false);
		void SetRadioFreq(DWORD freq);

		// AFC versions
		DWORD SetTVFreqWithAFC(DWORD Frec,KS_AnalogVideoStandard vf,TuningMethod method);
		DWORD SetRadioFreqWithAFC(DWORD freq);

	public:
		
		I2CTuner(I2C& bus,
				 const BYTE Addr,
				 const Tuner ttype,
				 const bool DoesFM,
				 const bool UseTVAudioforFM,	// Try to use TV audio circuit for FM.
				 const bool FastTune,			// Use fastuning
				 const bool HalfStep			// Use halfstep
				 );

		// Called to Set Frecuency: Returns the Real Frecuency tuned
		virtual DWORD SetFrequency(
			KS_AnalogVideoStandard	vf,			// VideoFormat of Signal (For TV mode)
			TunerMode				tm,			// Tuner Mode (TV/FM/AM)
			DWORD					Frequency,	// Frequency of channel
			TuningMethod			method);	

		// Called to set audio mode (Some tuners need this cause they handle FM Stereo Decoding)
		virtual bool SetAudioMode(
			AudioMode audio
			);

		// To get capacities of Tuner
		virtual bool CanTuneTV();
		virtual bool TVTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			);
		virtual bool TVStereoAvailable();
		virtual bool TVStereoDetected();
		virtual ULONG TVTunerVideoStandardsSupported();

		virtual bool CanTuneFM();
		virtual bool FMTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			);
		virtual bool FMStereoAvailable();
		virtual bool FMStereoDetected();

		// So we can power-down the tuner...
		virtual void Suspend( void );
};

#endif