/*
	MT2032.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "MT2032.hpp"

// IsSpurInBand()?
bool MT2032::SpurCheck(int f1, int f2, int spectrum_from,int spectrum_to) // All in Hz
{
	int n1=1,n2,f;

	f1 = f1 / 1000; //scale to kHz to avoid 32bit overflows
	f2 = f2 / 1000;
	spectrum_from /= 1000;
	spectrum_to /= 1000;

	DUMP(("SpurCheck f1=%d f2=%d  from=%d to=%d\n",f1,f2,spectrum_from,spectrum_to));

    do {
        n2 = -n1;
        f = n1 * (f1 - f2);
        do {
            n2--;
            f = f - f2;
            if ((f > spectrum_from) && (f < spectrum_to)) {
                return true;
            }
        } while ((f > (f2 - spectrum_to)) || (n2 > -5));
        n1++;
    } while (n1 < 5);

    return false;
}

bool MT2032::SetLORegs(int rfin, int if1, int if2, int spectrum_from,	int spectrum_to) //all in Hz
{
    int fref = 5250 * 1000; /* 5.25MHz */

    /* per spec 2.3.1 */
    int desired_lo1 = rfin + if1;
    int lo1 = (2 * (desired_lo1 / 1000) + (fref / 1000)) / (2 * fref / 1000);
    int lo1freq = lo1 * fref;
    int desired_lo2 = lo1freq - rfin - if2;

    /* per spec 2.3.2 */
    for (int nLO1adjust = 1; nLO1adjust < 3; nLO1adjust++) {
        if (!SpurCheck(lo1freq, desired_lo2, spectrum_from, spectrum_to)) {
            break;
        }

        if (lo1freq < desired_lo1) {
            lo1 += nLO1adjust;
        } else {
            lo1 -= nLO1adjust;
        }

        lo1freq = lo1 * fref;
        desired_lo2 = lo1freq - fref - if2;
    }

    /* per spec 2.3.3 */
    int s = lo1freq / 1000 / 1000;

	BYTE sel = 0;
    if (m_optimize_vco) {
        if (s > 1890) {
            sel = 0;
        } else if (s > 1720) {
            sel = 1;
        } else if (s > 1530) {
            sel = 2;
        } else if (s > 1370) {
            sel = 3;
        } else {
            sel = 4;    /* >1090 */
        }
    } else {
        if (s > 1790) {
            sel = 0;    /* <1958 */
        } else if (s > 1617) {
            sel = 1;
        } else if (s > 1449) {
            sel = 2;
        } else if (s > 1291) {
            sel = 3;
        } else {
            sel = 4;    /* >1090 */
        }
    }

    /* per spec 2.3.4 */
    int lo1n = lo1 / 8;
    int lo1a = lo1 - (lo1n * 8);
    int lo2  = desired_lo2 / fref;
    int lo2n = lo2 / 8;
    int lo2a = lo2 - (lo2n * 8);
    int lo2num = ((desired_lo2 / 1000) % (fref / 1000)) * 3780 / (fref / 1000); /* scale to fit in 32bit arith */
    int lo2freq = (lo2a + 8 * lo2n) * fref + lo2num * (fref / 1000) / 3780 * 1000;

	// Check parameters are bounded and valid...
    if (lo1a < 0 ||  
        lo1a > 7 ||  
        lo1n < 17 ||  
        lo1n > 48 ||  
        lo2a < 0 ||  
        lo2a > 7 ||  
        lo2n < 17 ||  
        lo2n > 30)
    {
        return false;
    }

	BYTE buf[4];

	// send only the relevant registers per Rev. 1.2
	buf[0] = 0;	// Registers 0-2
    buf[1] = BYTE(lo1n - 1);
    buf[2] = BYTE(lo1a | (sel << 4));
    buf[3] = 0x86;                  /* LOGC */
	i2c.WR(i2caddr,buf,4,NULL,0);	// send registers 0-2 

	buf[0] = 5;	// send registers 5-7
    buf[1] = BYTE((lo2n - 1) | (lo2a << 5));
    buf[2] = (rfin < 400 * 1000 * 1000) ? 0xe4 : 0xf4;   /* set PKEN per rev 1.2 ; also enable ADs */
    buf[3] = BYTE(8 + m_xogc);
	i2c.WR(i2caddr,buf,4,NULL,0);	// send registers 5-7

	buf[0] = 11;	// send 0xb - 0xc
    buf[1] = BYTE(lo2num & 0xff);
    buf[2] = BYTE((lo2num >> 8) | 0x80); /* Lo2RST */
	i2c.WR(i2caddr,buf,3,NULL,0);	// send 0xb - 0xc

    return true;
}

bool MT2032::CheckLOLock()
{
	BYTE buf[2];
	for(int mtry=0;mtry<10;mtry++) {

		buf[0] = 0x0e; // STATUS
		i2c.WR(i2caddr,buf,1,buf,1);

		DUMP(("mt2032 Reg.E=0x%02x\n",buf[0]));

		// If PLLs are locked, we are done.
		if ((buf[0] & 0x06)==6) {
			return true;
		}
		
		DUMP(("mt2032: pll wait 1ms for lock (0x%2x)\n",buf[0]));

		// Delay 1 ms to allow locking...
		mdelay(1);
	}
	// We failed...
	return false;
}

bool MT2032::OptimizeVCO()
{
	BYTE buf[2];

	buf[0]=0x01; // LO1 byte 2
	i2c.WR(i2caddr,buf,1,buf,1);			// Read LO1 byte 2
	BYTE Sel = BYTE((buf[0] >> 4) & 0x7);	// Get SEL
	BYTE LO1b2 = buf[0];
	
	buf[0]=0x0F; // TUNEAD
	i2c.WR(i2caddr,buf,1,buf,1);

	DUMP(("mt2032 Reg.F=0x%02x\n",buf[0]));

	// Based on TAD1 readings...
	switch (buf[0] & 0x07) {
		case 0:
		case 1:
			// If optimal VCO has been chosen, keep it
			return true;
		case 2:
			// A lower numbered VCO should be used...
			if(Sel>0) Sel--;
			else return true;
			break;
		case 5:
		case 6:
			// A Higher numbered VCO should be used...
			if(Sel<4) Sel++;
			else return true;
			break;
		default:
			// (should not happen)
			return true;
	}

	DUMP(("mt2032 OptimizeVCO: sel=%d\n",Sel));
	buf[0] = 0x01;	// LO1 byte 2
	buf[1] = (LO1b2 & 0x8F) | BYTE(Sel << 4); // Modify VCO sel field.
	i2c.WR(i2caddr,buf,2,NULL,0);	// And select the proper VCO

	// Get locking status again...
	return CheckLOLock();
}


void MT2032::SetIfFreq(int rfin, int if1, int if2, int from, int to) // All in Hz
{
	BYTE buf[2];
	
	DUMP(("MT2032::SetIfFreq rfin=%d if1=%d if2=%d from=%d to=%d\n",rfin,if1,if2,from,to));

	// Set LO registers to tune to the proper frequency...
	if (!SetLORegs(rfin,if1,if2,from,to))
		return;

	// Wait for PLLs to lock (per manual), retry LINT if not.
	bool locked = false;
	for(int lint_try=0; lint_try<2; lint_try++) {

		// Check if we locked PLLs
		locked = CheckLOLock();
		
		// If we locked and we have to optimize, do that
		if(locked && m_optimize_vco) {
			locked = OptimizeVCO();
		}

		// If PLLs locked, we are done.
		if (locked) break;
		
		// We failed; try LINT
		DUMP(("mt2032: re-init PLLs by LINT\n")); 
		buf[0] =0x7;  // Control Byte 2
		buf[1] =BYTE(0x80 | 8 | m_xogc); // set LINT to re-init PLLs
		i2c.WR(i2caddr,buf,2,NULL,0);
		mdelay(10);
		buf[0] =0x7;  // Control Byte 2
		buf[1] =BYTE( 8 | m_xogc) ;
		i2c.WR(i2caddr,buf,2,NULL,0);
	}

	if (!locked) {
		DUMP(("MT2032 Fatal Error: PLLs didn't lock.\n"));
	}

	buf[0] = 2;		// LO Gc Ctrl
	buf[1] = 0x20;	// LOGC for optimal phase noise
	i2c.WR(i2caddr,buf,2,NULL,0);
}


DWORD MT2032::SetTvFreq(DWORD freq,  KS_AnalogVideoStandard vf /* VideoFormat */) // In Hz
{
	int if2,from,to;

	// First enable tuner access...
	if (m_ifdem) m_ifdem->OnPreTunerSet(m_std,m_mode);

	// signal bandwidth and picture carrier
	if((ULONG(vf) & (TN_CCIR_MC)) != 0) {
		from = 40750*1000;
		to   = 46750*1000;
		if2  = 45750*1000; 
	}
	else {  // Pal 
		from = 32900*1000;
		to   = 39900*1000;
		if2  = 38900*1000;
	}

#ifdef AUTOTUNING
	// Perform autotuning...
	int iters = 10;
	while (iters--) {
#endif
		// Set frequency...
		SetIfFreq(freq, 1090*1000*1000, if2, from, to);

#ifdef AUTOTUNING
		// Use the AFC provider if available, else, use the internal one...
		int afc = 0;
		if (m_ifdem && m_ifdem->AFCProvider()) {
			afc = m_ifdem->GetAFC();
		} else {
			BYTE ret = 0;
			if (!i2c.Read(BYTE(0x0E),&ret)) break; //(status)
			afc = 100000 * (((ret >> 4) & 7) - 2); // AFC > 2 means increase freq. <2 means decrease freq.
		}

		// Adjust freq appropiately...
		if (afc >= 62500) freq += 62500;
		else if (afc < -62500) freq -= 62500;
		else break; // If we are tuned, break now
	};
#endif

	// And we are done - Disable tuner access
	if (m_ifdem) m_ifdem->OnPostTunerSet();

#ifdef AUTOTUNING
	// If not found, return
	if (!iters) return 0;
#endif

	// Return the tuned frequency...
	return freq;
}

void MT2032::SetRadioFreq(DWORD freq) // In Hz
{               
	int if2 = 10700 * 1000; //  10.7MHz FM intermediate frequency

	// First enable tuner access...
	if (m_ifdem) m_ifdem->OnPreTunerSet(m_std,m_mode);

	// Set radio frequency...
	SetIfFreq(freq, 1085*1000*1000, if2,if2,if2);

	// And we are done - Disable tuner access
	if (m_ifdem) m_ifdem->OnPostTunerSet();
}

/* ---------------------------------------------------------------------- */

void MT2032::Restart()
{
	// First enable tuner access...
	if (m_ifdem) m_ifdem->OnPreTunerSet(m_std,m_mode);

	// Initialization as described in "MT203x Programming Procedures", Rev 1.2, Feb.2001
	BYTE buf[21];
	
	buf[0]=0;
	i2c.WR(i2caddr,buf,1,buf,21);

	DUMP(("MT2032: Companycode=%02x%02x Part=%02x Revision=%02x\n",
                buf[0x11],buf[0x12],buf[0x13],buf[0x14]));
	   
	// Initialization Sequence follows...
	static BYTE init_0[] = {2  ,  0xFF,0x0F,0x1F};
	static BYTE init_1[] = {6  ,  0xE4,0x8F,0xC3,0x4E,0xEC};
	static BYTE init_2[] = {0xD,  0x32};

	// Write all registers per Spec...
	i2c.WR(i2caddr,init_0,4,NULL,0);
	i2c.WR(i2caddr,init_1,6,NULL,0);
	i2c.WR(i2caddr,init_2,2,NULL,0);

	// Adjust XOGC (register 7), wait for XOK
	BYTE xogc = 7; 
	bool xok = false;

	while (1) {

		// Allow 10 ms for tuner voltages to settle....
		mdelay(10);	

		buf[0] = 0x0E;							// STATUS reg
		i2c.WR(i2caddr,buf,1,buf,1);
		xok = (buf[0] & 0x01) ? true : false;	// XOK
		DUMP(( "MT2032: xok = %c\n" , (xok) ? 'Y' : 'N' ));
		if (xok) break;

		xogc--;
		if (xogc < 4) {
			xogc = 4; // min. 4 per spec
			break;
		}

		buf[0] = 0x07;							// CONTROL2 reg
		buf[1] = 0x88 | xogc;					// LINT | LDON | xogc
		i2c.WR(i2caddr,buf,2,NULL,0);
	};

	DUMP(( "MT2032: xogc = 0x%02x\n",xogc & 0x07 ));

	// Remember the optimized XOGC value
	m_xogc = xogc;

	// And we are done - Disable tuner access
	if (m_ifdem) m_ifdem->OnPostTunerSet();
}

MT2032::MT2032(I2C& ibus, const BYTE Addr, HwIFDem* dem)
	: i2c(ibus) , i2caddr(Addr)
	, m_mode(Tm_TV) , m_std(KS_AnalogVideoStandard(0))
	, m_xogc(4), m_optimize_vco(true) , m_ifdem(dem)
{
	MT2032::Restart();
}

// Called to Set Frecuency: Sets frecuency and Does Autotuning - Returns null if signal was not detected!
DWORD MT2032::SetFrequency(
		KS_AnalogVideoStandard	vf,			// VideoFormat of Signal (For TV mode)
		TunerMode				tm,			// Tuner Mode (TV/FM/AM)
		DWORD					Frequency,	// Frecuency of channel
		TuningMethod			method)		// Tuning method
{
	// Store Tuner Mode and std
	m_mode = tm;
	m_std  = vf;
	if (m_mode == Tm_TV) {
		if (Frequency <  45750000 ||
			Frequency > 855250000 ) {
			return 0;
		}
		// Set freq and detect the returned one...
		return SetTvFreq(Frequency,vf);
	}
	else {
		if (Frequency <  76000000 ||
			Frequency > 108000000 ) {
			return 0;
		}
		SetRadioFreq(Frequency);
	}
	return Frequency;
}

// To get capacities of Tuner
bool MT2032::CanTuneTV()
{
	return true;
}


bool MT2032::TVTunerCaps(
	ULONG& MinFrecuency,
	ULONG& MaxFrecuency,
	ULONG& TuningGranularity,
	ULONG& SettlingTime
	)
{
	MinFrecuency		=   45750000;
	MaxFrecuency		=  855250000;
	TuningGranularity	=  62500;
	SettlingTime		=  50; // ms (does not matter, we will tune ourselves)
	return true;
}

bool MT2032::TVStereoAvailable()
{
	return false;
}

bool MT2032::TVStereoDetected()
{
	return false;
}

ULONG MT2032::TVTunerVideoStandardsSupported()
{
	return TN_CCIR_MC | TN_CCIR_I | TN_CCIR_DKL | TN_CCIR_BGHN;
}

bool MT2032::CanTuneFM()
{
	return true;
}

bool MT2032::FMTunerCaps(
	ULONG& MinFrecuency,
	ULONG& MaxFrecuency,
	ULONG& TuningGranularity,
	ULONG& SettlingTime
)
{
	MinFrecuency		=  76000000;
	MaxFrecuency		= 108000000;
	TuningGranularity	=	  50000;
	SettlingTime		=		300; // ms
	return true;
}

bool MT2032::FMStereoAvailable()
{
	return true;	
}

bool MT2032::FMStereoDetected()
{
	return true;
}

