/*
	MT2050.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "MT2050.hpp"

int MT2050::SpurCheck(int flos1, int flos2, int fifbw, int fout)
{
    int n1 = 1, n2, f, nmax = 11;
	long Band;

    flos1 /= 1000;     /* scale to kHz to avoid 32bit overflows */
    flos2 /= 1000;
    fifbw /= 1000;
    fout  /= 1000;

	Band = fout + fifbw / 2;

	do {
		n2 = -n1;
		f = n1 * (flos1 - flos2);
		do {
			n2--;
			f = f - flos2;
			if (abs((abs(f) - fout)) < (fifbw >> 1))
			{
				return 1;
			}
		} while ((f > (flos2 - fout - (fifbw >> 1))) && (n2 > -nmax));
		n1++;
	} while (n1 < nmax);

    return 0;
}

void MT2050::SetIFFreq(int rfin, int if1, int if2, int fifbw)
{
    BYTE buf[6];

	//3.1 Calculate LO frequencies
	long flo1 = rfin + if1;
	flo1 /= 1000000;
	flo1 *= 1000000;
	long flo2 = flo1 - rfin - if2;

	//3.2 Avoid spurs
	int n = 0;
	long ftest;
	char SpurInBand;

	long flos1 = flo1;
	long flos2 = flo2;
	long fif1_bw = 16000000;

	do {
		if ((n & 1) == 0) {
			flos1 = flos1 - 1000000 * n;
			flos2 = flos2 - 1000000 * n;
		} else {
			flos1 = flos1 + 1000000 * n;
			flos2 = flos2 + 1000000 * n;
		}

		//check we are still in bandwidth
		ftest = abs(flos1 - rfin - if1 + (fifbw >> 1));
		if (ftest > (fif1_bw >> 1)) {
			flos1 = flo1;
			flos2 = flo2;
			DUMP(( "No spur"));
			break;
		}
		n++;
		SpurInBand = (char)SpurCheck(flos1, flos2, fifbw, if2);
	} while(SpurInBand != 0);

	flo1 = flos1;
	flo2 = flos2;

	//3.3 Calculate LO registers
	long LO1I, LO2I, flo1step, flo2step;
	long flo1rem, flo2rem, flo1tune, flo2tune;
	int num1, num2, Denom1, Denom2;
	int div1a, div1b, div2a, div2b;

	flo1step = 1000000;
	flo2step = 50000;
	LO1I = flo1 / 4000000;
	LO2I = flo2 / 4000000;
	flo1rem = flo1 % 4000000;
	flo2rem = flo2 % 4000000;
	flo1tune = flo1step * ((flo1rem + flo1step / 2) / flo1step);
	flo2tune = flo2step * ((flo2rem + flo2step / 2) / flo2step);
	Denom1 = 4;
	Denom2 = 4095;
	num1 = ( (flo1tune+(2000000 / Denom1)) / (4000000 / Denom1) );
	num2 = ( (flo2tune+(2000000 / Denom2)) / (4000000 / Denom2) );
	if (num1 >= Denom1) {
		num1 = 0;
		LO1I++;
	}
	if (num2 >= Denom2) {
		num2 = 0;
		LO2I++;
	}
	div1a = (LO1I / 12) - 1;
	div1b = LO1I % 12;
	div2a = (LO2I / 8) - 1;
	div2b = LO2I % 8;

	//3.4 Writing registers
	if (rfin < 277000000) {
		buf[1] = 128 + 4 * div1b + num1;
	} else {
		buf[1] = 4 * div1b + num1;
	}
	buf[2] = (BYTE)div1a;
	buf[3] = 32 * div2b + (num2 / 256);
	buf[4] = num2 % 256;
	if (num2 == 0) {
		buf[5] = (BYTE)div2a;
	} else {
		buf[5] = 64 + div2a;
	}

	buf[0] = 1;
	i2c.WR(i2caddr,buf,6,NULL,0);

	//3.5 Allow LO to lock
	bool m_Locked = false;
	mdelay(50);

	int nlock = 0, Status;
	do {
		buf[0] = 7;
		i2c.WR(i2caddr,buf,1,buf,1);
		buf[0] &= 0x88;
		if (buf[0] == 0x88) {
			m_Locked = true;
			break;
		}
		mdelay(2);
		nlock++;
	} while(nlock < 100);
}

DWORD MT2050::SetTvFreq(DWORD freq,  KS_AnalogVideoStandard vf /* VideoFormat */) // In Hz
{
	int if2,fifbw;

	// signal bandwidth and picture carrier
	if((ULONG(vf) & (TN_CCIR_MC)) != 0) {
		if2 = 45750 * 1000;
		fifbw = 6750000;
	}
	else {  // Pal 
		if2 = 38900 * 1000;
		fifbw = 8750000;
	}

	// First enable tuner access...
	if (m_ifdem) m_ifdem->OnPreTunerSet(m_std,m_mode);

	// Set frequency...
    SetIFFreq(freq, 1220 * 1000 * 1000, if2, fifbw);

	// And we are done - Disable tuner access
	if (m_ifdem) m_ifdem->OnPostTunerSet();

	// Return the tuned frequency...
	return freq;
}

void MT2050::SetRadioFreq(DWORD freq) // In Hz
{               
	int if2 = 10700 * 1000; //  10.7MHz FM intermediate frequency

	// First enable tuner access...
	if (m_ifdem) m_ifdem->OnPreTunerSet(m_std,m_mode);

	// Set radio frequency...
    SetIFFreq(freq, 1220 * 1000 * 1000, if2, 300000);

	// And we are done - Disable tuner access
	if (m_ifdem) m_ifdem->OnPostTunerSet();
}

/* ---------------------------------------------------------------------- */

void MT2050::Restart()
{
	// First enable tuner access...
	if (m_ifdem) m_ifdem->OnPreTunerSet(m_std,m_mode);

    // Get chip info
    BYTE rdbuf[22];
    BYTE wrbuf[]={0};    
    
    if (i2c.WR(i2caddr,wrbuf,2,rdbuf,21)) {
		DUMP(("MT2050: Companycode=%02x%02x Part=%02x Revision=%02x",
                    rdbuf[0x11],rdbuf[0x12],rdbuf[0x13],rdbuf[0x14]));
    }

	// Initialize Registers per spec.
	static BYTE init[] = {1, 0x2F,0x25,0xC1,0x00,0x63,0x10,0x85,0x28,0x0F,0x24};
	i2c.WR(i2caddr,init,11,NULL,0);

	// Get SRO
	BYTE buf[2];
	buf[0] = 0xD;
	i2c.WR(i2caddr,buf,1,buf,1);
	if ((buf[0] & 0x40) != 0) {
		DUMP(("MT2050: SRO Crystal problem - tuner will not function!"));
	}

	// And we are done - Disable tuner access
	if (m_ifdem) m_ifdem->OnPostTunerSet();
}

MT2050::MT2050(I2C& ibus, const BYTE Addr, HwIFDem* dem)
	: i2c(ibus) , i2caddr(Addr)
	, m_mode(Tm_TV) , m_std(KS_AnalogVideoStandard(0))
	, m_ifdem(dem)
{
	MT2050::Restart();
}

// Called to Set Frecuency: Sets frecuency and Does Autotuning - Returns null if signal was not detected!
DWORD MT2050::SetFrequency(
		KS_AnalogVideoStandard	vf,			// VideoFormat of Signal (For TV mode)
		TunerMode				tm,			// Tuner Mode (TV/FM/AM)
		DWORD					Frequency,	// Frecuency of channel
		TuningMethod			method)		// Tuning method
{
	// Store Tuner Mode and std
	m_mode = tm;
	m_std  = vf;
	if (m_mode == Tm_TV) {
		if (Frequency <  45750000 ||
			Frequency > 855250000 ) {
			return 0;
		}
		// Set freq and detect the returned one...
		return SetTvFreq(Frequency,vf);
	}
	else {
		if (Frequency <  76000000 ||
			Frequency > 108000000 ) {
			return 0;
		}
		SetRadioFreq(Frequency);
	}
	return Frequency;
}

// To get capacities of Tuner
bool MT2050::CanTuneTV()
{
	return true;
}


bool MT2050::TVTunerCaps(
	ULONG& MinFrecuency,
	ULONG& MaxFrecuency,
	ULONG& TuningGranularity,
	ULONG& SettlingTime
	)
{
	MinFrecuency		=   45750000;
	MaxFrecuency		=  855250000;
	TuningGranularity	=  62500;
	SettlingTime		=  50; // ms (does not matter, we will tune ourselves)
	return true;
}

bool MT2050::TVStereoAvailable()
{
	return false;
}

bool MT2050::TVStereoDetected()
{
	return false;
}

ULONG MT2050::TVTunerVideoStandardsSupported()
{
	return TN_CCIR_MC | TN_CCIR_I | TN_CCIR_DKL | TN_CCIR_BGHN;
}

bool MT2050::CanTuneFM()
{
	return true;
}

bool MT2050::FMTunerCaps(
	ULONG& MinFrecuency,
	ULONG& MaxFrecuency,
	ULONG& TuningGranularity,
	ULONG& SettlingTime
)
{
	MinFrecuency		=  76000000;
	MaxFrecuency		= 108000000;
	TuningGranularity	=	  50000;
	SettlingTime		=		300; // ms
	return true;
}

bool MT2050::FMStereoAvailable()
{
	return true;	
}

bool MT2050::FMStereoDetected()
{
	return true;
}

