/*
	TEA5757.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "TEA5757.hpp"

#define TEA5757_FM_STEREO	(0x000000)
#define TEA5757_FM_MONO		(0x400000)
#define TEA5757_AM_STEREO	(0x200000)
#define TEA5757_AM_MONO		(0x600000)

ULONG TEA5757::BUS_IN(Line l)
{
	const BYTE bit = m_bus[l];
	DWORD Data = 0;
	gpio.GetGPDATA(bit,bit,&Data);
	return Data;
}

ULONG TEA5757::Read()
{
	ULONG value = 0;
	ULONG timeout;
	int i;
	
	// better safe than sorry 
	BUS_SETTOOUT(CLK);
	BUS_SETTOOUT(WE);
	BUS_SETTOIN(MOST);
	BUS_SETTOIN(DATA);

	// Start Reading it
	BUS_LOW(WE);
	BUS_LOW(CLK);
	
	udelay(10);

	for(timeout = 0;
	    BUS_IN(DATA) == 1 && timeout < 10;
		timeout++
		) mdelay(1);	/* 10 ms */

	if (BUS_IN(DATA) == 1) {
		return 0xffffffff; /* Timeout */
	}

	for(i = 0; i < 24; i++)	{

		udelay(10);
		BUS_HIGH(CLK);
		udelay(10);

		//DUMP(("%c\n", (BUS_IN(MOST) == 0)?'T':'-'));

		BUS_LOW(CLK);
		value <<= 1;					
		value |= (BUS_IN(DATA) == 0)?0:1;	/* MSB first */
		//DUMP(("%c\n", (BUS_IN(MOST) == 0)?'S':'M'));

		// Get Mono/Stereo indicator
		m_bStereoDetected = (BUS_IN(MOST) == 0) ;
	}
	DUMP(("tea5757: read 0x%X\n", value));
	return value;
}

void TEA5757::Write(const ULONG value)
{
	int i;
	ULONG reg = value;
	
	// better safe than sorry 
	BUS_SETTOOUT(CLK);
	BUS_SETTOOUT(WE);
	BUS_SETTOIN(MOST);
	BUS_SETTOOUT(DATA);
	
	// Start Writing!
	BUS_LOW(CLK);
	BUS_HIGH(WE);

	for(i = 0; i < 25; i++)	{

		if (reg & 0x1000000)
			BUS_HIGH(DATA);
		else
			BUS_LOW(DATA);

		reg <<= 1;

		BUS_HIGH(CLK);
		udelay(10);

		BUS_LOW(CLK);
		udelay(10);

	}

	BUS_LOW(WE);	/* unmute !!! */
	return;
}

void TEA5757::Restart()
{
	// better safe than sorry 
	BUS_SETTOOUT(CLK);
	BUS_SETTOOUT(WE);
	BUS_SETTOIN(MOST);
	BUS_SETTOIN(DATA);
	
	// just to be on the safe side...
	BUS_LOW(CLK);
	BUS_LOW(WE); 
}

TEA5757::TEA5757(	GPIO& gp,
					const BYTE bWE,
					const BYTE bCLK,
					const BYTE bDATA,
					const BYTE bMOST) : 
	gpio(gp) ,
	m_bStereoDetected(false) ,
	m_bForceMono(false) ,
	m_Tm(Tm_Radio_FM) ,
	m_dwFrequency(88000000)
{
	// Prepare Buffer array
	m_bus[WE] = bWE;
	m_bus[CLK] = bCLK;
	m_bus[DATA] = bDATA;
	m_bus[MOST] = bMOST;

	TEA5757::Restart();
}

// Called to Set Frecuency
DWORD TEA5757::SetFrequency(
			KS_AnalogVideoStandard vf,	// VideoFormat of Signal (For TV mode)
			TunerMode	tm,				// Tuner Mode (TV/FM/AM)
			DWORD dwFrequency,			// Frecuency of channel
			TuningMethod method)		// TEA does its own fine tuning!
{
	switch (tm) {
		case Tm_Radio_AM:
			{
				DWORD config = (dwFrequency / 1000) + 455;
				config &= 0x7fff;
				config |= TEA5757_AM_MONO;
				// Should switch bands here .... How?
				Write(config);

				// Remember settings!
				m_dwFrequency = dwFrequency;
				m_Tm = tm;

				// Wait 100 ms // Should be 500ms!
				mdelay(100);

				// Read back results
				if ((config = Read()) != 0xffffffff) {
					DUMP(("TEATuner: AMStatus= %x\n",config));

					config &= 0x7fff;
					if (config && config != 0x7fff ) {
						DWORD ulRdCurrentFrequency = (config - 455) * 1000;
						if (abs(int(ulRdCurrentFrequency - dwFrequency)) < 10000 ) {
							m_dwFrequency = ulRdCurrentFrequency;
						}
					}
				}
			}
			return m_dwFrequency;
		case Tm_Radio_FM:
			{
				// Set frecuency ...
				DWORD config = (dwFrequency / 12500) + 856;
				config &= 0x7fff;
				config |= (m_bForceMono) ? TEA5757_FM_MONO : TEA5757_FM_STEREO;
				Write(config);

				// Remember settings!
				m_dwFrequency = dwFrequency;
				m_Tm = tm;

				// Wait 100 ms
				mdelay(100);

				// Read back results
				if ((config = Read()) != 0xffffffff) {
					DUMP(("TEATuner: FMStatus= %x\n",config));

					config &= 0x7fff;
					if (config && config != 0x7fff ) {
						DWORD ulRdCurrentFrequency = (config - 856) * 12500;
						if (abs(int(ulRdCurrentFrequency - dwFrequency)) < 100000 ) {
							m_dwFrequency = ulRdCurrentFrequency;
						}
					}
				}
			}
			return m_dwFrequency;
	}
	return 0;
}

// Called to set audio mode (Some tuners need this cause they handle FM Stereo Decoding)
bool TEA5757::SetAudioMode(
			AudioMode audio
			) 
{
	m_bForceMono = (audio == Am_Mono);
	TEA5757::SetFrequency(
			KS_AnalogVideo_NTSC_M /* Non Important! */,
			m_Tm,
			m_dwFrequency,
			Tm_Fine);
	return true;
}

// To get capacities of Tuner
bool TEA5757::CanTuneAM()
{
	return true;
}

bool TEA5757::AMTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			)
{
	MinFrecuency			= 144000;
	MaxFrecuency			= 30000000;
	TuningGranularity		= 1000; // Granularity
	SettlingTime			= 500;	// in ms
	return true;
}

bool TEA5757::AMStereoAvailable()
{
	return false;
}

bool TEA5757::AMStereoDetected()
{
	return false;
}

bool TEA5757::CanTuneFM()
{
	return true;
}

bool TEA5757::FMTunerCaps(
			ULONG& MinFrecuency,
			ULONG& MaxFrecuency,
			ULONG& TuningGranularity,
			ULONG& SettlingTime
			)
{
	MinFrecuency			=  50000000;
	MaxFrecuency			= 150000000;
	TuningGranularity		=     12500; // Granularity
	SettlingTime			=       100; // in ms
	return true;
}


bool TEA5757::FMStereoAvailable()
{
	return true;
}

bool TEA5757::FMStereoDetected()
{
	// Read Status
	Read();
	return m_bStereoDetected 
			| m_bForceMono // To let Pick Stereo when in Mono Mode
			;
}

