/*
	GRANDTEC16.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	This file was written by John Newbigin <jn@it.swin.edu.au> based on code
	written by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
Technical Details of card:
  
The card has had the names if the extra IC's removed so determining what is on there took
a bit effort.  The card has 16 camera inputs which are arranged into groups of 4.
Each group of 4 is connected to a 4:1 multiplexer (external to the BT878).  This multiplexer
appears to be a FSAV331 or equivelent which is a dual 4:1 multiplexer.

Each multiplexer is then connected to one of the 4 inputs on the BT878.

The multiplexer input select is connected to the GPIO.  The following table shows the
states for selecting the 16 cameras

Camera	MUX	GPOE		GPDATA
1		2	0x0000ff	0xffff00
2		2	0x0000ff	0xffffa0
3		2	0x0000ff	0xffff50
4		2	0x0000ff	0xfffff0
5		1	0x0000ff	0xffff00
6		1	0x0000ff	0xffffa0
7		1	0x0000ff	0xffff50
8		1	0x0000ff	0xfffff0
9		3	0x0000ff	0xffff00
10		3	0x0000ff	0xffffa0
11		3	0x0000ff	0xffff50
12		3	0x0000ff	0xfffff0
13		0	0x0000ff	0xffff00
14		0	0x0000ff	0xffffa0
15		0	0x0000ff	0xffff50
16		0	0x0000ff	0xfffff0

*/


#include "GRANDTEC16.hpp"
#include "..\HwCapture\Pisces.hpp"

GRANDTECx16::GRANDTECx16(GPIO& g) :
	gpio(g) 
{
	last_mux = -1;
	last_mux2 = -1;

	Restart();
}

void GRANDTECx16::Restart( void )
{
	DWORD Val = 0xFFFF00;
	gpio.SetGPDATA(&Val,1,0);

	gpio.SetGPOE( /*pisces.GetGPOE() |*/ 0x0000FF );
	udelay(8);	// without this we would see the 0x1800 mask ?? do we need this?

	gpio.SetGPDATA(&Val,1,0);
}

int	GRANDTECx16::CompositeCount( void ) const
{ 
	return 16;
}

	
condesc_t GRANDTECx16::SetConnector(VideoConnector conn)
{
	DWORD Val = 0;
	int mux; // the mux on the bt878
	int mux2; // the mux connected to the gpio

	DUMP(("selecting %d\n", conn));

	switch (conn) {
///////////////////////////////////////////
//	mux = 2
///////////////////////////////////////////

		case VideoConComposite0:
			mux = 2;
			mux2 = 0;
			break;

		case VideoConComposite1:
			mux = 2;
			mux2 = 1;
			break;

		case VideoConComposite2:
			mux = 2;
			mux2 = 2;
			break;

		case VideoConComposite3:
			mux = 2;
			mux2 = 3;
			break;
///////////////////////////////////////////
//	mux = 1
///////////////////////////////////////////
		case VideoConComposite4:
			mux = 1;
			mux2 = 0;
			break;

		case VideoConComposite5:
			mux = 1;
			mux2 = 1;
			break;

		case VideoConComposite6:
			mux = 1;
			mux2 = 2;
			break;

		case VideoConComposite7:
			mux = 1;
			mux2 = 3;
			break;
///////////////////////////////////////////
//	mux = 3
///////////////////////////////////////////
		case VideoConComposite8:
			mux = 3;
			mux2 = 0;
			break;

		case VideoConComposite9:
			mux = 3;
			mux2 = 1;
			break;

		case VideoConComposite10:
			mux = 3;
			mux2 = 2;
			break;

		case VideoConComposite11:
			mux = 3;
			mux2 = 3;
			break;
///////////////////////////////////////////
//	mux = 0
///////////////////////////////////////////
		case VideoConComposite12:
			mux = 0;
			mux2 = 0;
			break;

		case VideoConComposite13:
			mux = 0;
			mux2 = 1;
			break;

		case VideoConComposite14:
			mux = 0;
			mux2 = 2;
			break;

		case VideoConComposite15:
			mux = 0;
			mux2 = 3;
			break;
///////////////////////////////////////////

		default:
			return -1;

	}

	if(mux != -1)
	{
		if(mux2 == 0)
		{
			mux2 = 0xffff00;
		}
		else if(mux2 == 1)
		{
			mux2 = 0xffffa0;
		}
		else if(mux2 == 2)
		{
			mux2 = 0xffff50;
		}
		else if(mux2 == 3)
		{
			mux2 = 0xfffff0;
		}

		DUMP(("Setting MUX = %d  GPDATA = 0x%02X\n", mux, mux2));
	
		if(mux2 != last_mux2)
		{
//			pisces.GetGPDATA(&Val,1,0);
//			DUMP(("Current Value = 0x%06X\n", Val));
//			Val &= 0xFFFF00;
//			Val |= mux2;

			Val = mux2;
			DUMP(("New Value = 0x%06X\n", Val));
			gpio.SetGPDATA(&Val,1,0);

			last_mux2 = mux2;
		}
		last_mux = mux;
	}
	return CONDESC(mux,0,0); // mux, svideo, isdigital
}

