/*
	TDA8540.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/* 4x4 video switch matrix */
// -----------------------------------------------------------------------
// TDA8540 Control Code
// Philips composite/s-video 4x4 switch IC
// 19 Jul 2001 Dan Schmelzer
//
// See datasheet at:
// http://www.semiconductors.philips.com/acrobat/datasheets/TDA8540_3.pdf
//
// Slave address byte (SLV) possibilities for the TDA8540 chip
// See page 5 of 2/6/95 datasheet
// 0x90, 0x92, 0x94, 0x96, 0x98, 0x9A, 0x9C
//
// Subcommand byte (SUB) possibilities choosing function
// See page 6 of 2/6/95 datasheet
// 0x00 = Access to switch control (SW1)
// 0x01 = Access to gain/clamp/auxilliary pins control (GCO)
// 0x02 = Access to output enable control (OEN)
// If commands are given in succession, then SUB is automatically incremented
// and the next register is written to
//
// Switch Control Register (SW1)
// See page 6 of 2/6/95 datasheet
// Four output bit pairs P1-4 make up control byte; input chosen for each output
// 00 for Input 0; 01 for Input 1; 10 for Input 2; 11 for input 3
//
// Gain & Clamp Control Register (GCO)
// See page 6 of 2/6/95 datasheet
// MS 4 bits control gain on outputs[3:0] (low is 1x, high is 2x)
// 2 bits control clamp action or mean value on inputs [1:0] ; LS 2 bits
// control value of auxilliary outputs D1, D0
//
// Output Enable Control Register (OEN)
// See page 7 of 2/6/95 datasheet
// MS 4 bits reserved; LS 4 bits controls which output(s) from 3 to 0 are
// active (high)
//
// Upon reset, the outputs are set to active and connected to IN0; the gains
// are set at 2x and inputs IN0 and IN1 are clamped.


#include "TDA8540.hpp"
#include "..\HwCapture\Pisces.hpp"

void TDA8540::Restart()
{
	// Initialize and set the Philips TDA8540 4x4 switch matrix
	// 0xD2 SW1 choose OUT3=IN3; OUT2=IN1; OUT1=IN0; OUT0=IN2
	// 0x07 GCO choose (0000) gain; (01) clamp; (11) aux    
	// 0x03 OEN choose OUT0 and OUT1 high (i.e., s-video)
	static BYTE buf[] = {0x00,0xD2, 0x07, 0x03};
	i2c.WR(i2caddr,buf,4,NULL,0); // Init TDA... 
}

TDA8540::TDA8540(I2C& bus, const BYTE Addr,const bool Svideo_jumper) :
	i2c(bus) , i2caddr(Addr) , svj(Svideo_jumper)
{
	TDA8540::Restart();
}

int TDA8540::SVideoCount( void ) const
{
	return ((svj)?1:0);
}

int TDA8540::CompositeCount( void ) const
{
	return 3 + ((svj)?0:1);
}

condesc_t TDA8540::SetConnector(VideoConnector conn)
{
	switch (conn) {
		case VideoConSVideo0: {
			if (!svj) return -1;

			// Initialize and set the Philips TDA8540 4x4 switch matrix for s-video
			// 0xD2 SW1 choose OUT3=IN3; OUT2=IN1; OUT1=IN0; OUT0=IN2
			// 0x07 GCO choose (0000) gain; (01) clamp; (11) aux
			// 0x03 OEN choose OUT0 and OUT1 high (i.e., s-video)
			static BYTE buf[] = {0x00, 0xD2, 0x07, 0x03};
			i2c.WR(i2caddr,buf,4,NULL,0); // Init TDA... 
			}
			return CONDESC(2,1,0);
		case VideoConComposite0: {
			// Set the Philips TDA8540 4x4 switch matrix for composite switch 1 input
			// 0x00 SW1 choose OUT0=IN0
			// 0x87 GCO choose (1000) gain; (01) clamp; (11) aux    
			// 0x01 OEN choose OUT0 high
			static BYTE buf[] = {0x00, 0x00, 0x87, 0x01};
			i2c.WR(i2caddr,buf,4,NULL,0); // Init TDA... 
			}
			return CONDESC(2,0,0);
		case VideoConComposite1: {
			// Set the Philips TDA8540 4x4 switch matrix for composite switch 2 input
			// 0x01 SW1 choose OUT0=IN1
			// 0x87 GCO choose (1000) gain; (01) clamp; (11) aux    
			// 0x01 OEN choose OUT0 high
			static BYTE buf[] = {0x00, 0x01, 0x87, 0x01};
			i2c.WR(i2caddr,buf,4,NULL,0); // Init TDA... 
			}
			return CONDESC(2,0,0);
		case VideoConComposite2: {
			if (svj) { //SVideo jumper...
				// Set the Philips TDA8540 4x4 switch matrix for composite switch 3 input
				// 0x01 SW1 choose OUT0=IN3
				// 0x87 GCO choose (1000) gain; (01) clamp; (11) aux    
				// 0x01 OEN choose OUT0 high
				static BYTE buf[] = {0x00, 0x03, 0x87, 0x01};
				i2c.WR(i2caddr,buf,4,NULL,0); // Init TDA... 
			} else {
				// Set the Philips TDA8540 4x4 switch matrix for composite switch 3 input
				// 0x01 SW1 choose OUT0=IN2
				// 0x87 GCO choose (1000) gain; (01) clamp; (11) aux    
				// 0x01 OEN choose OUT0 high
				static BYTE buf[] = {0x00, 0x02, 0x87, 0x01};
				i2c.WR(i2caddr,buf,4,NULL,0); // Init TDA... 
			}
			}
			return CONDESC(2,0,0);
		case VideoConComposite3: {
				if (svj) return -1;

				// Set the Philips TDA8540 4x4 switch matrix for composite switch 3 input
				// 0x01 SW1 choose OUT0=IN3
				// 0x87 GCO choose (1000) gain; (01) clamp; (11) aux    
				// 0x01 OEN choose OUT0 high
				static BYTE buf[] = {0x00, 0x03, 0x87, 0x01};
				i2c.WR(i2caddr,buf,4,NULL,0); // Init TDA... 
			}
			return CONDESC(2,0,0);
	}
	return -1;
}

/*
SASEM_S 	S 1 2 3 
SDI100_C	S 1 2 3 4
SDI100_S	S 1 2 3
*/