/*
	VideoFormats.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


/*
 * File with helper macros to declare all formats supported.
 */

#ifndef _VIDEOFORMATS_HPP_
#define _VIDEOFORMATS_HPP_

#include <staticuids.hpp>

// ----------------------------------------------------

typedef struct tagKS_VIDEOINFOHEADER_WITHMASK {

    RECT                rcSource;          // The bit we really want to use
    RECT                rcTarget;          // Where the video should go
    DWORD               dwBitRate;         // Approximate bit data rate
    DWORD               dwBitErrorRate;    // Bit error rate for this stream
    REFERENCE_TIME      AvgTimePerFrame;   // Average time per frame (100ns units)

    KS_BITMAPINFOHEADER bmiHeader;

	DWORD				dwBitMasks[3];     // True colour masks

} KS_VIDEOINFOHEADER_WITHMASK;

typedef struct tagKS_VIDEOINFOHEADER2_WITHMASK {
    RECT				rcSource;
    RECT				rcTarget;
    DWORD				dwBitRate;
    DWORD				dwBitErrorRate;
    REFERENCE_TIME		AvgTimePerFrame;
    DWORD				dwInterlaceFlags;   // use AMINTERLACE_* defines. Reject connection if undefined bits are not 0
    DWORD				dwCopyProtectFlags; // use AMCOPYPROTECT_* defines. Reject connection if undefined bits are not 0
    DWORD				dwPictAspectRatioX; // X dimension of picture aspect ratio, e.g. 16 for 16x9 display
    DWORD				dwPictAspectRatioY; // Y dimension of picture aspect ratio, e.g.  9 for 16x9 display
    DWORD				dwReserved1;        // must be 0; reject connection otherwise
    DWORD				dwReserved2;        // must be 0; reject connection otherwise

    KS_BITMAPINFOHEADER bmiHeader;

	DWORD				dwBitMasks[3];		// True colour masks

} KS_VIDEOINFOHEADER2_WITHMASK;

//===========================================================================
// Video DATARANGEs with MASKs
//===========================================================================

typedef struct tagKS_DATARANGE_VIDEO_WITHMASK {
   KSDATARANGE                  DataRange;
   BOOL                         bFixedSizeSamples;      // all samples same size?
   BOOL                         bTemporalCompression;   // all I frames?
   DWORD                        StreamDescriptionFlags; // KS_VIDEO_DESC_*
   DWORD                        MemoryAllocationFlags;  // KS_VIDEO_ALLOC_*
   KS_VIDEO_STREAM_CONFIG_CAPS  ConfigCaps;
   KS_VIDEOINFOHEADER_WITHMASK  VideoInfoHeader;        // default format
} KS_DATARANGE_VIDEO_WITHMASK, *PKS_DATARANGE_VIDEO_WITHMASK;

typedef struct tagKS_DATARANGE_VIDEO2_WITHMASK {
   KSDATARANGE                  DataRange;
   BOOL                         bFixedSizeSamples;      // all samples same size?
   BOOL                         bTemporalCompression;   // all I frames?
   DWORD                        StreamDescriptionFlags; // KS_VIDEO_DESC_*
   DWORD                        MemoryAllocationFlags;  // KS_VIDEO_ALLOC_*
   KS_VIDEO_STREAM_CONFIG_CAPS  ConfigCaps;
   KS_VIDEOINFOHEADER2_WITHMASK VideoInfoHeader;        // default format
} KS_DATARANGE_VIDEO2_WITHMASK, *PKS_DATARANGE_VIDEO2_WITHMASK;


// ----------------------------------------------------
// The following macro estimates the size of the bitmap in bytes...
// -If we are dealing with a non-planar format, then the stride of each
//	scanline is simply the next multiple of four of the required amount of
//	bytes required to store it. Windows always aligns scanlines to DWORDS
// -If we are dealing with a planar format, then we also need that the
//  Y plane scanlines to be dword aligned... and the chroma planes scanlines
//  will have half that stride. As the Bt8x8 is unable to handle scanlines
//  not aligned to DWORD boundaries, this will force the Y plane stride also,
//  so this macro also holds true for all planar formats

// All scanlines of bitmaps are always aligned to DWORD.
// For planar bitmaps...
#define BMP_SIZE(p_x,p_y,p_bpp) \
	(((((p_bpp * p_x) + 31L) & (~(31L))) / 8L) * p_y)

// ----------------------------------------------------

/* 
	PAL-BDGHI	= 768x576x50
	SECAM-L		= 768x576x50
	PAL-N       = 768x576x50
	PAL_60		= 768x480x60
	PAL-NC		= 744x576x50
	NTSC-50		= 756x576x50
	PAL-M		= 756x480x60
	NTSC-J		= 756x480x60
	NTSC		= 756x480x60
*/

// Helper macro

// Default is maximum frame VIDEOINFO (Autointerlace)
// Using KS_DATARANGE_VIDEO
#define DECL_STRMFMTFORNORM(tp, minx, miny, defx , defy , maxx, maxy , bpp , sbpp, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb , norms , cropx,cropy, acropx,acropy, granx , grany , maxfps , fcc ) \
{ \
	/* KSDATARANGE */ \
	{ \
		{ \
			sizeof( KS_DATARANGE_VIDEO ), 0, \
			BMP_SIZE(maxx,maxy,sbpp), 0, /* SampleSize, Reserved */  \
			{ STATIC_KSDATAFORMAT_TYPE_VIDEO }, \
			{ v1,v2,v3,{v4,v5,v6,v7,v8,v9,va,vb} }, /* MEDIASUBTYPE_... */ \
			{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO }	\
		} \
	}, \
	true			, /* FixedSizeSamples */ \
	true			, /* no TemporalCompression */ \
	tp				, /* flags, now reserved */ \
	0 , \
	/* _KS_VIDEO_STREAM_CONFIG_CAPS */ \
	{ \
		{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO }, /* GUID */ \
		norms,   /* AnalogVideoStandard */ \
		{ maxx,maxy } , { minx,miny } , { maxx , maxy }, /* MaxInsize[X,Y] , MinCrop[X,Y] , MaxCrop[X,Y] */ \
		cropx , cropy , acropx , acropy ,	/* CropGranularity[X,Y] , CropAlign[X,Y] */ \
		{ minx,miny } , { maxx,maxy } ,		/* MinOutSize[X,Y] , MaxOutSize[X,Y] */ \
		granx , grany	,										/* Outgranularity [X,Y] */ \
		0,0,2,2,													/* StretchShrinkQual... */ \
		(__int64)(10000000i64 / maxfps) ,100000000i64,	/* MinFrameInterval, MaxFrameInterval (100ns units) */ \
		((BMP_SIZE(minx,miny,bpp) * 8) / 10),((BMP_SIZE(maxx,maxy,bpp) * 8) * maxfps)	/* MinBitsPerSecond,MaxBitsPerSecond */ \
	}, \
	/* KS_VIDEOINFOHEADER (default format) */ \
	{ \
		{ 0,0, 0,0 },{ 0,0,0,0 },					/* rcSource, rcTarget */ \
		(BMP_SIZE(maxx,maxy,bpp) * maxfps) * 8, 0 ,		/* Bitrate, Error */ \
		(__int64)(10000000i64 / maxfps),			/* AvgTimePerFrame (100ns units) */ \
		{ \
			sizeof( KS_BITMAPINFOHEADER ), \
			maxx,maxy,								/* defWidth, defHeight */ \
			1,										/*    WORD       biPlanes; */ \
			bpp,									/*    WORD       biBitCount; */ \
			fcc,									/*    DWORD      biCompression; */ \
			BMP_SIZE(maxx,maxy,bpp),				/*    DWORD      biSizeImage; */ \
			0,0,0,0									/*    LONG       biXPelsPerMeter,biYPelsPerMeter,biClrUsed,biClrImportant */ \
		} \
	} \
}


// Default is DEFAULT frame VIDEOINFO (Autointerlace)
// Using KS_DATARANGE_VIDEO
#define DECL_STRMFMTFORNORM_DEF(tp, minx, miny, defx , defy , maxx, maxy , bpp , sbpp, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb , norms , cropx,cropy, acropx,acropy, granx , grany , maxfps , fcc ) \
{ \
	/* KSDATARANGE */ \
	{ \
		{ \
			sizeof( KS_DATARANGE_VIDEO ), 0, \
			BMP_SIZE(defx,defy,sbpp), 0, /* SampleSize, Reserved */  \
			{ STATIC_KSDATAFORMAT_TYPE_VIDEO }, \
			{ v1,v2,v3,{v4,v5,v6,v7,v8,v9,va,vb} }, /* MEDIASUBTYPE_... */ \
			{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO }	\
		} \
	}, \
	true			, /* FixedSizeSamples */ \
	true  /*true*/	, /* TemporalCompression */ \
	tp /*tp */		, /* flags, now reserved */ \
	0 , \
	/* _KS_VIDEO_STREAM_CONFIG_CAPS */ \
	{ \
		{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO }, /* GUID */ \
		norms,   /* AnalogVideoStandard */ \
		{ maxx,maxy } , { minx,miny } , { maxx , maxy }, /* MaxInsize[X,Y] , MinCrop[X,Y] , MaxCrop[X,Y] */ \
		cropx , cropy , acropx , acropy ,	/* CropGranularity[X,Y] , CropAlign[X,Y] */ \
		{ minx,miny } , { maxx,maxy } ,		/* MinOutSize[X,Y] , MaxOutSize[X,Y] */ \
		granx , grany	,										/* Outgranularity [X,Y] */ \
		0,0,2,2,													/* StretchShrinkQual... */ \
		(__int64)(10000000i64 / maxfps) ,100000000i64,		/* MinFrameInterval, MaxFrameInterval (100ns units) */ \
		((BMP_SIZE(minx,miny,bpp) * 8) / 10),((BMP_SIZE(maxx,maxy,bpp) * 8) * maxfps)	/* MinBitsPerSecond,MaxBitsPerSecond */ \
	}, \
	/* KS_VIDEOINFOHEADER (default format) */ \
	{ \
		{ 0,0, 0,0 },{ 0,0,0,0 },					/* rcSource, rcTarget */ \
		(BMP_SIZE(defx,defy,bpp) * maxfps) * 8, 0 ,		/* Bitrate, Error */ \
		(__int64)(10000000i64 / maxfps),			/* AvgTimePerFrame (100ns units) */ \
		{ \
			sizeof( KS_BITMAPINFOHEADER ), \
			defx,defy,								/* defWidth, defHeight */ \
			1,										/*    WORD       biPlanes; */ \
			bpp,									/*    WORD       biBitCount; */ \
			fcc,									/*    DWORD      biCompression; */ \
			BMP_SIZE(defx,defy,bpp),				/*    DWORD      biSizeImage; */ \
			0,0,0,0									/*    LONG       biXPelsPerMeter,biYPelsPerMeter,biClrUsed,biClrImportant */ \
		} \
	} \
}

///----
// Default is maximum frame VIDEOINFO2 (Autointerlace)
// Using KS_DATARANGE_VIDEO2
#define DECL_STRMFMTFORNORM2(tp, minx, miny, defx , defy , maxx, maxy , bpp , sbpp, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb , norms , cropx,cropy, acropx,acropy, granx , grany , maxfps , fcc , mflags) \
{ \
	/* KSDATARANGE2 */ \
	{ \
		{ \
			sizeof( KS_DATARANGE_VIDEO2 ), 0, \
			BMP_SIZE(maxx,maxy,sbpp), 0, /* SampleSize, Reserved */  \
			{ STATIC_KSDATAFORMAT_TYPE_VIDEO }, \
			{ v1,v2,v3,{v4,v5,v6,v7,v8,v9,va,vb} }, /* MEDIASUBTYPE_... */ \
			{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO2 }	\
		} \
	}, \
	true			, /* FixedSizeSamples */ \
	true  /*true*/	, /* TemporalCompression */ \
	tp /*tp */		, /* flags, now reserved */ \
	0 , \
	/* _KS_VIDEO_STREAM_CONFIG_CAPS */ \
	{ \
		{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO2 }, /* GUID */ \
		norms,   /* AnalogVideoStandard */ \
		{ maxx,maxy } , { minx,miny } , { maxx , maxy }, /* MaxInsize[X,Y] , MinCrop[X,Y] , MaxCrop[X,Y] */ \
		cropx , cropy , acropx , acropy ,	/* CropGranularity[X,Y] , CropAlign[X,Y] */ \
		{ minx,miny } , { maxx,maxy } ,		/* MinOutSize[X,Y] , MaxOutSize[X,Y] */ \
		granx , grany	,										/* Outgranularity [X,Y] */ \
		0,0,2,2,													/* StretchShrinkQual... */ \
		(__int64)(10000000i64 / maxfps) ,100000000i64,		/* MinFrameInterval, MaxFrameInterval (100ns units) */ \
		((BMP_SIZE(minx,miny,bpp) * 8) / 10),((BMP_SIZE(maxx,maxy,bpp) * 8) * maxfps)	/* MinBitsPerSecond,MaxBitsPerSecond */ \
	}, \
	/* KS_VIDEOINFOHEADER2 (default format) */ \
	{ \
		{ 0,0, 0,0 },{ 0,0,0,0 },					/* rcSource, rcTarget */ \
		(BMP_SIZE(maxx,maxy,bpp) * maxfps) * 8, 0 ,		/* Bitrate, Error */ \
		(__int64)(10000000i64 / maxfps),	/* AvgTimePerFrame (100ns units) */ \
		    mflags,									/* dwInterlaceFlags */ \
			0,										/* dwCopyProtectFlags */ \
			4,3,									/* dwPictAspectRatio */ \
			0,0,									/* Reserved */ \
    	{ /* BITMAPINFOHEADER */ \
			sizeof( KS_BITMAPINFOHEADER ), \
			maxx,maxy,								/* defWidth, defHeight */ \
			1,										/*    WORD       biPlanes; */ \
			bpp,									/*    WORD       biBitCount; */ \
			fcc,									/*    DWORD      biCompression; */ \
			BMP_SIZE(maxx,maxy,bpp),				/*    DWORD      biSizeImage; */ \
			0,0,0,0									/*    LONG       biXPelsPerMeter,biYPelsPerMeter,biClrUsed,biClrImportant */ \
		} \
	} \
}

// Helper macros to use masks
// Helper macro

// Default is maximum frame VIDEOINFO (Autointerlace)
// Using KS_DATARANGE_VIDEO_WITHMASK
#define DECL_STRMFMTFORNORM_WITHMASK(tp, minx, miny, defx , defy , maxx, maxy , bpp , sbpp, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb , norms , cropx,cropy, acropx,acropy, granx , grany , maxfps , fcc, mask0, mask1, mask2 ) \
{ \
	/* KSDATARANGE_VIDEO_WITHMASK */ \
	{ \
		{ \
			sizeof( KS_DATARANGE_VIDEO_WITHMASK ), 0, \
			BMP_SIZE(maxx,maxy,sbpp), 0, /* SampleSize, Reserved */  \
			{ STATIC_KSDATAFORMAT_TYPE_VIDEO }, \
			{ v1,v2,v3,{v4,v5,v6,v7,v8,v9,va,vb} }, /* MEDIASUBTYPE_... */ \
			{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO }	\
		} \
	}, \
	true			, /* FixedSizeSamples */ \
	true  /*true*/	, /* TemporalCompression */ \
	tp /*tp */		, /* flags, now reserved */ \
	0 , \
	/* _KS_VIDEO_STREAM_CONFIG_CAPS */ \
	{ \
		{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO }, /* GUID */ \
		norms,   /* AnalogVideoStandard */ \
		{ maxx,maxy } , { minx,miny } , { maxx , maxy }, /* MaxInsize[X,Y] , MinCrop[X,Y] , MaxCrop[X,Y] */ \
		cropx , cropy , acropx , acropy ,	/* CropGranularity[X,Y] , CropAlign[X,Y] */ \
		{ minx,miny } , { maxx,maxy } ,		/* MinOutSize[X,Y] , MaxOutSize[X,Y] */ \
		granx , grany	,										/* Outgranularity [X,Y] */ \
		0,0,2,2,													/* StretchShrinkQual... */ \
		(__int64)(10000000i64 / maxfps) ,100000000i64,		/* MinFrameInterval, MaxFrameInterval (100ns units) */ \
		((BMP_SIZE(minx,miny,bpp) * 8) / 10),((BMP_SIZE(maxx,maxy,bpp) * 8) * maxfps)	/* MinBitsPerSecond,MaxBitsPerSecond */ \
	}, \
	/* KS_VIDEOINFOHEADER_WITHMASK (default format) */ \
	{ \
		{ 0,0, 0,0 },{ 0,0,0,0 },					/* rcSource, rcTarget */ \
		(BMP_SIZE(maxx,maxy,bpp) * maxfps) * 8, 0 ,		/* Bitrate, Error */ \
		(__int64)(10000000i64 / maxfps),	/* AvgTimePerFrame (100ns units) */ \
		{ \
			sizeof( KS_BITMAPINFOHEADER ) , \
			maxx,maxy,								/* defWidth, defHeight */ \
			1,										/*    WORD       biPlanes; */ \
			bpp,									/*    WORD       biBitCount; */ \
			fcc,									/*    DWORD      biCompression; */ \
			BMP_SIZE(maxx,maxy,bpp),				/*    DWORD      biSizeImage; */ \
			0,0,0,0									/*    LONG       biXPelsPerMeter,biYPelsPerMeter,biClrUsed,biClrImportant */ \
		}, \
		{ mask0, mask1 , mask2 } \
	} \
}


// Default is DEFAULT frame VIDEOINFO (Autointerlace)
// Using KS_DATARANGE_VIDEO_WITHMASK
#define DECL_STRMFMTFORNORM_DEF_WITHMASK(tp, minx, miny, defx , defy , maxx, maxy , bpp , sbpp, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb , norms , cropx,cropy, acropx,acropy, granx , grany , maxfps , fcc, mask0, mask1, mask2 ) \
{ \
	/* KSDATARANGE_VIDEO_WITHMASK */ \
	{ \
		{ \
			sizeof( KS_DATARANGE_VIDEO_WITHMASK ), 0, \
			BMP_SIZE(defx,defy,sbpp), 0, /* SampleSize, Reserved */  \
			{ STATIC_KSDATAFORMAT_TYPE_VIDEO }, \
			{ v1,v2,v3,{v4,v5,v6,v7,v8,v9,va,vb} }, /* MEDIASUBTYPE_... */ \
			{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO }	\
		} \
	}, \
	true			, /* FixedSizeSamples */ \
	true  /*true*/	, /* TemporalCompression */ \
	tp /*tp */		, /* flags, now reserved */ \
	0 , \
	/* _KS_VIDEO_STREAM_CONFIG_CAPS */ \
	{ \
		{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO }, /* GUID */ \
		norms,   /* AnalogVideoStandard */ \
		{ maxx,maxy } , { minx,miny } , { maxx , maxy }, /* MaxInsize[X,Y] , MinCrop[X,Y] , MaxCrop[X,Y] */ \
		cropx , cropy , acropx , acropy ,	/* CropGranularity[X,Y] , CropAlign[X,Y] */ \
		{ minx,miny } , { maxx,maxy } ,		/* MinOutSize[X,Y] , MaxOutSize[X,Y] */ \
		granx , grany	,										/* Outgranularity [X,Y] */ \
		0,0,2,2,													/* StretchShrinkQual... */ \
		(__int64)(10000000i64 / maxfps) ,100000000i64,		/* MinFrameInterval, MaxFrameInterval (100ns units) */ \
		((BMP_SIZE(minx,miny,bpp) * 8) / 10),((BMP_SIZE(maxx,maxy,bpp) * 8) * maxfps)	/* MinBitsPerSecond,MaxBitsPerSecond */ \
	}, \
	/* KS_VIDEOINFOHEADER_WITHMASK (default format) */ \
	{ \
		{ 0,0, 0,0 },{ 0,0,0,0 },					/* rcSource, rcTarget */ \
		(BMP_SIZE(defx,defy,bpp) * maxfps) * 8, 0 ,		/* Bitrate, Error */ \
		(__int64)(10000000i64 / maxfps),	/* AvgTimePerFrame (100ns units) */ \
		{ \
			sizeof( KS_BITMAPINFOHEADER ), \
			defx,defy,								/* defWidth, defHeight */ \
			1,										/*    WORD       biPlanes; */ \
			bpp,									/*    WORD       biBitCount; */ \
			fcc,									/*    DWORD      biCompression; */ \
			BMP_SIZE(defx,defy,bpp),				/*    DWORD      biSizeImage; */ \
			0,0,0,0									/*    LONG       biXPelsPerMeter,biYPelsPerMeter,biClrUsed,biClrImportant */ \
		}, \
		{ mask0, mask1 , mask2 } \
	} \
}

///----
// Default is maximum frame VIDEOINFO2 (Autointerlace)
// Using KS_DATARANGE_VIDEO2_WITHMASK
#define DECL_STRMFMTFORNORM2_WITHMASK(tp, minx, miny, defx , defy , maxx, maxy , bpp , sbpp, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb , norms , cropx,cropy, acropx,acropy, granx , grany , maxfps , fcc, mask0, mask1, mask2 , mflags) \
{ \
	/* KSDATARANGE_VIDEO2_WITHMASK */ \
	{ \
		{ \
			sizeof( KS_DATARANGE_VIDEO2_WITHMASK ), 0, \
			BMP_SIZE(maxx,maxy,sbpp), 0, /* SampleSize, Reserved */  \
			{ STATIC_KSDATAFORMAT_TYPE_VIDEO }, \
			{ v1,v2,v3,{v4,v5,v6,v7,v8,v9,va,vb} }, /* MEDIASUBTYPE_... */ \
			{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO2 }	\
		} \
	}, \
	true			, /* FixedSizeSamples */ \
	true  /*true*/	, /* TemporalCompression */ \
	tp /*tp */		, /* flags, now reserved */ \
	0 , \
	/* _KS_VIDEO_STREAM_CONFIG_CAPS */ \
	{ \
		{ STATIC_KSDATAFORMAT_SPECIFIER_VIDEOINFO2 }, /* GUID */ \
		norms,   /* AnalogVideoStandard */ \
		{ maxx,maxy } , { minx,miny } , { maxx , maxy }, /* MaxInsize[X,Y] , MinCrop[X,Y] , MaxCrop[X,Y] */ \
		cropx , cropy , acropx , acropy ,	/* CropGranularity[X,Y] , CropAlign[X,Y] */ \
		{ minx,miny } , { maxx,maxy } ,		/* MinOutSize[X,Y] , MaxOutSize[X,Y] */ \
		granx , grany	,										/* Outgranularity [X,Y] */ \
		0,0,2,2,													/* StretchShrinkQual... */ \
		(__int64)(10000000i64 / maxfps) ,100000000i64,		/* MinFrameInterval, MaxFrameInterval (100ns units) */ \
		((BMP_SIZE(minx,miny,bpp) * 8) / 10),((BMP_SIZE(maxx,maxy,bpp) * 8) * maxfps)	/* MinBitsPerSecond,MaxBitsPerSecond */ \
	}, \
	/* KS_VIDEOINFOHEADER2_WITHMASK (default format) */ \
	{ \
		{ 0,0, 0,0 },{ 0,0,0,0 },					/* rcSource, rcTarget */ \
		(BMP_SIZE(maxx,maxy,bpp) * maxfps)*8, 0 ,		/* Bitrate, Error */ \
		(__int64)(10000000i64 / maxfps),	/* AvgTimePerFrame (100ns units) */ \
		    mflags,									/* dwInterlaceFlags */ \
			0,										/* dwCopyProtectFlags */ \
			4,3,									/* dwPictAspectRatio */ \
			0,0,									/* Reserved */ \
    	{ /* BITMAPINFOHEADER */ \
			sizeof( KS_BITMAPINFOHEADER ), \
			maxx,maxy,								/* defWidth, defHeight */ \
			1,										/*    WORD       biPlanes; */ \
			bpp,									/*    WORD       biBitCount; */ \
			fcc,									/*    DWORD      biCompression; */ \
			BMP_SIZE(maxx,maxy,bpp),				/*    DWORD      biSizeImage; */ \
			0,0,0,0									/*    LONG       biXPelsPerMeter,biYPelsPerMeter,biClrUsed,biClrImportant */ \
		}, \
		{ mask0, mask1 , mask2 } \
	} \
}



///----

/* 
	PAL-BDGHI	= 768x576x50
	SECAM-L		= 768x576x50
	PAL-N       = 768x576x50
	PAL_60		= 768x480x60
	PAL-NC		= 745x576x50
	NTSC-50		= 756x576x50
	PAL-M		= 756x480x60
	NTSC-J		= 756x480x60
	NTSC		= 756x480x60

  -Note that to comply with the required alignment for the YVU9 ... X size should be a multiple of 16
*/

// Macro to declare for all norms for all the VideoFormats. Uses KS_DATARANGE_VIDEO
#define DECL_STRFMT( type, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc) \
	DECL_STRMFMTFORNORM_DEF( /* Fools stupid programs that get the fisrt format */\
		type ,										/* Type */	\
		48,32,										/* min */	\
		320,240,									/* default */	\
		752,480,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_M | KS_AnalogVideo_NTSC_M_J | KS_AnalogVideo_NTSC_M ,/* Norms */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		30,											/* maxfps */	\
		fcc ) ,										/* FOURCC */	\
	DECL_STRMFMTFORNORM_DEF( /* This one is to fool NetMeeting!! (emulate digital video */ \
		type ,										/* Type */ \
		64,40,										/* min */ \
		320,240,									/* default */ \
		320,240,									/* max */ \
		bpp1,bpp2,									/* bpp */ \
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */ \
		KS_AnalogVideo_None, 						/* Norm : To emulate a digital Camera */ \
		1,1,1,1,									/* crop, acrop */ \
		granx,grany,								/* gran */ \
		30,											/* maxfps (maximum value!) */ \
		fcc ) ,										/* FOURCC */ \
	DECL_STRMFMTFORNORM(	\
		type ,										/* Type */	\
		64,40,										/* min */	\
		384,288,									/* default */	\
		768,576,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_N | KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_D |	\
		KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H |	\
		KS_AnalogVideo_PAL_I | KS_AnalogVideo_SECAM_L | /* Norms */	\
		KS_AnalogVideo_SECAM_B | KS_AnalogVideo_SECAM_D |	\
		KS_AnalogVideo_SECAM_G | KS_AnalogVideo_SECAM_H |	\
		KS_AnalogVideo_SECAM_K | KS_AnalogVideo_SECAM_K1,	/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		25,											/* maxfps */	\
		fcc ) ,										/* FOURCC */	\
	DECL_STRMFMTFORNORM(	\
		type ,										/* Type */	\
		48,40,										/* min */	\
		320,288,									/* default */	\
		752,576,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_NTSC_50,						/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		25,											/* maxfps */	\
		fcc ) ,										/* FOURCC */	\
	DECL_STRMFMTFORNORM(	\
		type ,										/* Type */	\
		48,40,										/* min */	\
		320,288,									/* default */	\
		736,576,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_NC,						/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		25,											/* maxfps */	\
		fcc ) ,										/* FOURCC */	\
	DECL_STRMFMTFORNORM(	\
		type ,										/* Type */	\
		64,32,										/* min */	\
		384,240,									/* default */	\
		768,480,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_60, 						/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		30,											/* maxfps */	\
		fcc )										/* FOURCC */	

// Macro to declare for all norms for all the VideoFormats. Uses KS_DATARANGE_VIDEO_WITHMASK
#define DECL_STRFMT_WITHMASK( type, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2) \
	DECL_STRMFMTFORNORM_DEF_WITHMASK( /* Fools stupid programs that get the fisrt format */\
		type ,										/* Type */	\
		48,32,										/* min */	\
		320,240,									/* default */	\
		752,480,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_M | KS_AnalogVideo_NTSC_M_J | KS_AnalogVideo_NTSC_M ,/* Norms */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		30,											/* maxfps */	\
		fcc, mask0, mask1, mask2 ) ,				/* FOURCC and MASKs */	\
	DECL_STRMFMTFORNORM_DEF_WITHMASK( /* This one is to fool NetMeeting!! (emulate digital video */ \
		type ,										/* Type */ \
		64,40,										/* min */ \
		320,240,									/* default */ \
		320,240,									/* max */ \
		bpp1,bpp2,									/* bpp */ \
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */ \
		KS_AnalogVideo_None, 						/* Norm : To emulate a digital Camera */ \
		1,1,1,1,									/* crop, acrop */ \
		granx,grany,								/* gran */ \
		30,											/* maxfps (maximum value!) */ \
		fcc, mask0, mask1, mask2 ) ,				/* FOURCC and MASKS */ \
	DECL_STRMFMTFORNORM_WITHMASK(	\
		type ,										/* Type */	\
		64,40,										/* min */	\
		384,288,									/* default */	\
		768,576,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_N | KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_D |	\
		KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H |	\
		KS_AnalogVideo_PAL_I | KS_AnalogVideo_SECAM_L | /* Norms */	\
		KS_AnalogVideo_SECAM_B | KS_AnalogVideo_SECAM_D |	\
		KS_AnalogVideo_SECAM_G | KS_AnalogVideo_SECAM_H |	\
		KS_AnalogVideo_SECAM_K | KS_AnalogVideo_SECAM_K1,	/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		25,											/* maxfps */	\
		fcc, mask0, mask1, mask2 ) ,				/* FOURCC and MASKS */	\
	DECL_STRMFMTFORNORM_WITHMASK(	\
		type ,										/* Type */	\
		48,40,										/* min */	\
		320,288,									/* default */	\
		752,576,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_NTSC_50,	/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		25,											/* maxfps */	\
		fcc, mask0, mask1, mask2 ) ,				/* FOURCC and MASKs */	\
	DECL_STRMFMTFORNORM_WITHMASK(	\
		type ,										/* Type */	\
		48,40,										/* min */	\
		320,288,									/* default */	\
		736,576,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_NC,	/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		25,											/* maxfps */	\
		fcc, mask0, mask1, mask2 ) ,				/* FOURCC and MASKs */	\
	DECL_STRMFMTFORNORM_WITHMASK(	\
		type ,										/* Type */	\
		64,32,										/* min */	\
		384,240,									/* default */	\
		768,480,									/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_60, 						/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		30,											/* maxfps */	\
		fcc, mask0, mask1, mask2 )					/* FOURCC and MASKs */	


// Macro to declare for all norms all the VideoFormats for capture
// Uses KS_DATARANGE_VIDEO
#define DECL_CAPTURESTRFMT( bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc) \
	DECL_STRFMT( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc) 

// Macro to declare for all norms all the VideoFormats for capture
// Uses KS_DATARANGE_VIDEO
#define DECL_PREVIEWSTRFMT( bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc) \
	DECL_STRFMT( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc) 

// Macro to declare for all norms all the VideoFormats for capture
// Uses KS_DATARANGE_VIDEO_WITHMASK
#define DECL_CAPTURESTRFMT_WITHMASK( bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 ) \
	DECL_STRFMT_WITHMASK( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 ) 

// Macro to declare for all norms all the VideoFormats for capture
// Uses KS_DATARANGE_VIDEO_WITHMASK
#define DECL_PREVIEWSTRFMT_WITHMASK( bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 ) \
	DECL_STRFMT_WITHMASK( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 ) 

// Macro to declare for all norms for all the VideoFormats. 
// Uses KS_DATARANGE_VIDEO2
#define DECL_STRFMT2( type, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mflags, vertdiv) \
	DECL_STRMFMTFORNORM2( \
		type ,										/* Type */	\
		48,(32 / vertdiv),							/* min */	\
		320,(240 / vertdiv),						/* default */	\
		752,(480 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_M | KS_AnalogVideo_NTSC_M_J | KS_AnalogVideo_NTSC_M , 		/* Norms */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(30 * vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mflags ) ,									/* flags */	\
	DECL_STRMFMTFORNORM2(	\
		type ,										/* Type */	\
		64,(40 / vertdiv),							/* min */	\
		384,(288 / vertdiv),						/* default */	\
		768,(576 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_N | KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_D |	\
		KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H |	\
		KS_AnalogVideo_PAL_I | KS_AnalogVideo_SECAM_L | /* Norms */	\
		KS_AnalogVideo_SECAM_B | KS_AnalogVideo_SECAM_D |	\
		KS_AnalogVideo_SECAM_G | KS_AnalogVideo_SECAM_H |	\
		KS_AnalogVideo_SECAM_K | KS_AnalogVideo_SECAM_K1,	/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(25 * vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mflags ) ,									/* flags */	\
	DECL_STRMFMTFORNORM2(	\
		type ,										/* Type */	\
		48,(40 / vertdiv),							/* min */	\
		320,(288 / vertdiv),						/* default */	\
		752,(576 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_NTSC_50,						/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(25 * vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mflags ) ,									/* flags */	\
	DECL_STRMFMTFORNORM2(	\
		type ,										/* Type */	\
		48,(40 / vertdiv),							/* min */	\
		320,(288 / vertdiv),						/* default */	\
		736,(576 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_NC,						/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(25 * vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mflags ) ,									/* flags */	\
	DECL_STRMFMTFORNORM2(	\
		type ,										/* Type */	\
		64,(32 / vertdiv),							/* min */	\
		384,(240 / vertdiv),						/* default */	\
		768,(480 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_60, 						/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(30 * vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mflags ) 									/* flags */	

// Macro to declare for all norms for all the VideoFormats. 
// Uses KS_DATARANGE_VIDEO2_WITHMASK
#define DECL_STRFMT2_WITHMASK( type, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , mflags, vertdiv) \
	DECL_STRMFMTFORNORM2_WITHMASK( \
		type ,										/* Type */	\
		48,(32 / vertdiv),							/* min */	\
		320,(240 / vertdiv),						/* default */	\
		752,(480 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_M | KS_AnalogVideo_NTSC_M_J | KS_AnalogVideo_NTSC_M , 		/* Norms */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(30*vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mask0, mask1, mask2 ,						/* masks */ \
		mflags ) ,									/* flags */	\
	DECL_STRMFMTFORNORM2_WITHMASK(	\
		type ,										/* Type */	\
		64,(40 / vertdiv),							/* min */	\
		384,(288 / vertdiv),						/* default */	\
		768,(576 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_N | KS_AnalogVideo_PAL_B | KS_AnalogVideo_PAL_D |	\
		KS_AnalogVideo_PAL_G | KS_AnalogVideo_PAL_H |	\
		KS_AnalogVideo_PAL_I | KS_AnalogVideo_SECAM_L | /* Norms */	\
		KS_AnalogVideo_SECAM_B | KS_AnalogVideo_SECAM_D |	\
		KS_AnalogVideo_SECAM_G | KS_AnalogVideo_SECAM_H |	\
		KS_AnalogVideo_SECAM_K | KS_AnalogVideo_SECAM_K1,	/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(25*vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mask0, mask1, mask2 ,						/* masks */ \
		mflags ) ,									/* flags */	\
	DECL_STRMFMTFORNORM2_WITHMASK(	\
		type ,										/* Type */	\
		48,(40 / vertdiv),							/* min */	\
		320,(288 / vertdiv),						/* default */	\
		752,(576 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_NTSC_50,	/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(25*vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mask0, mask1, mask2 ,						/* masks */ \
		mflags ) ,									/* flags */	\
	DECL_STRMFMTFORNORM2_WITHMASK(	\
		type ,										/* Type */	\
		48,(40 / vertdiv),							/* min */	\
		320,(288 / vertdiv),						/* default */	\
		736,(576 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_NC,	/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(25*vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mask0, mask1, mask2 ,						/* masks */ \
		mflags ) ,									/* flags */	\
	DECL_STRMFMTFORNORM2_WITHMASK(	\
		type ,										/* Type */	\
		64,(32 / vertdiv),							/* min */	\
		384,(240 / vertdiv),						/* default */	\
		768,(480 / vertdiv),						/* max */	\
		bpp1,bpp2,									/* bpp */	\
		v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb,			/* MEDIASUBTYPE */	\
		KS_AnalogVideo_PAL_60, 						/* Norm */	\
		1,1,1,1,									/* crop, acrop */	\
		granx,grany,								/* gran */	\
		(30*vertdiv),								/* maxfps */	\
		fcc ,										/* FOURCC */ \
		mask0, mask1, mask2 ,						/* masks */ \
		mflags ) 									/* flags */	

// Macro to declare for all norms all the VideoFormats for capture
// Uses KS_DATARANGE_VIDEO2

#define DECL_CAPTURESTRFMT2( bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc ) \
	DECL_STRFMT2( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOnly | KS_INTERLACE_FieldPatField1Only) , 2 ), \
	DECL_STRFMT2( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOnly | KS_INTERLACE_FieldPatField2Only) , 2 ), \
	DECL_STRFMT2( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOrWeave | KS_INTERLACE_FieldPatBothRegular) , 2), \
	DECL_STRFMT2( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_FieldPatBothRegular | KS_INTERLACE_DisplayModeBobOrWeave) , 1)

// Macro to declare for all norms all the VideoFormats for capture
// Uses KS_DATARANGE_VIDEO2
#define DECL_PREVIEWSTRFMT2( bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc) \
	DECL_STRFMT2( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOnly | KS_INTERLACE_FieldPatField1Only) , 2), \
	DECL_STRFMT2( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOnly | KS_INTERLACE_FieldPatField2Only) , 2), \
	DECL_STRFMT2( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOrWeave | KS_INTERLACE_FieldPatBothRegular) , 2), \
	DECL_STRFMT2( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_FieldPatBothRegular | KS_INTERLACE_DisplayModeBobOrWeave), 1)


// Macro to declare for all norms all the VideoFormats for capture
// Uses KS_DATARANGE_VIDEO2_WITHMASK

#define DECL_CAPTURESTRFMT2_WITHMASK( bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 ) \
	DECL_STRFMT2_WITHMASK( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_FieldPatBothRegular | KS_INTERLACE_DisplayModeBobOrWeave) , 1), \
	DECL_STRFMT2_WITHMASK( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOnly | KS_INTERLACE_FieldPatField1Only) , 2 ), \
	DECL_STRFMT2_WITHMASK( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOnly | KS_INTERLACE_FieldPatField2Only) , 2 ), \
	DECL_STRFMT2_WITHMASK( KS_VIDEOSTREAM_CAPTURE, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOrWeave | KS_INTERLACE_FieldPatBothRegular) , 2)

// Macro to declare for all norms all the VideoFormats for capture
// Uses KS_DATARANGE_VIDEO2_WITHMASK
#define DECL_PREVIEWSTRFMT2_WITHMASK( bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 ) \
	DECL_STRFMT2_WITHMASK( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_FieldPatBothRegular | KS_INTERLACE_DisplayModeBobOrWeave), 1), \
	DECL_STRFMT2_WITHMASK( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOnly | KS_INTERLACE_FieldPatField1Only) , 2), \
	DECL_STRFMT2_WITHMASK( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOnly | KS_INTERLACE_FieldPatField2Only) , 2), \
	DECL_STRFMT2_WITHMASK( KS_VIDEOSTREAM_PREVIEW, bpp1, bpp2, v1,v2,v3,v4,v5,v6,v7,v8,v9,va,vb, granx, grany, fcc, mask0, mask1, mask2 , \
		(KS_INTERLACE_IsInterlaced | KS_INTERLACE_1FieldPerSample | KS_INTERLACE_DisplayModeBobOrWeave | KS_INTERLACE_FieldPatBothRegular) , 2)

/* Non-Interleaved Bob (field 1 only)
KS_INTERLACE_IsInterlaced | 
KS_INTERLACE_1FieldPerSample | 
KS_INTERLACE_FieldPatField1Only |
KS_INTERLACE_DisplayModeBobOnly */

/* Non-Interleaved Bob (field 2 only)
KS_INTERLACE_IsInterlaced | 
KS_INTERLACE_1FieldPerSample | 
KS_INTERLACE_FieldPatField2Only |
KS_INTERLACE_DisplayModeBobOnly */

/* Alternating fields
KS_INTERLACE_IsInterlaced |
KS_INTERLACE_1FieldPerSample |
KS_INTERLACE_FieldPatBothRegular |
KS_INTERLACE_DisplayModeBobOrWeave  */

/* Interlaced fields
KS_INTERLACE_IsInterlaced |
KS_INTERLACE_FieldPatBothRegular |
KS_INTERLACE_DisplayModeBobOrWeave  */



// Some useful macros.
#define GET_LPDATAFORMATS(x) \
	(PKSDATAFORMAT) &x[0],	(PKSDATAFORMAT) &x[1],	(PKSDATAFORMAT) &x[2],	(PKSDATAFORMAT) &x[3],	(PKSDATAFORMAT) &x[4],	(PKSDATAFORMAT) &x[5]
	 
#define GET_LPDATAFORMATS2(x) \
	(PKSDATAFORMAT) &x[0],	(PKSDATAFORMAT) &x[1],	(PKSDATAFORMAT) &x[2],	(PKSDATAFORMAT) &x[3],	(PKSDATAFORMAT) &x[4], \
	(PKSDATAFORMAT) &x[5],	(PKSDATAFORMAT) &x[6],	(PKSDATAFORMAT) &x[7],	(PKSDATAFORMAT) &x[8],	(PKSDATAFORMAT) &x[9], \
	(PKSDATAFORMAT) &x[10],	(PKSDATAFORMAT) &x[11],	(PKSDATAFORMAT) &x[12],	(PKSDATAFORMAT) &x[13],	(PKSDATAFORMAT) &x[14], \
	(PKSDATAFORMAT) &x[15],	(PKSDATAFORMAT) &x[16],	(PKSDATAFORMAT) &x[17],	(PKSDATAFORMAT) &x[18]


// And now some colorspace defs

// copied from mmsystem.h
#define mmioFOURCC( ch0, ch1, ch2, ch3 )                                \
                ( (DWORD)(BYTE)(ch0) | ( (DWORD)(BYTE)(ch1) << 8 ) |    \
                ( (DWORD)(BYTE)(ch2) << 16 ) | ( (DWORD)(BYTE)(ch3) << 24 ) )


#define KS_BI_YUY2    mmioFOURCC( 'Y', 'U', 'Y', '2' ) /* 0x32595559 */
#define KS_BI_Y41P    mmioFOURCC( 'Y', '4', '1', 'P' ) /* 0x50313459 */
#define KS_BI_Y8      mmioFOURCC( 'Y', '8', ' ', ' ' )
#define KS_BI_422     mmioFOURCC( '4', '2', '2', ' ' )
#define KS_BI_411     mmioFOURCC( '4', '1', '1', ' ' )
#define KS_BI_YVU9    mmioFOURCC( 'Y', 'V', 'U', '9' ) /* 0x39555659 */
#define KS_BI_YV12    mmioFOURCC( 'Y', 'V', '1', '2' ) /* 0x32315659 */
#define KS_BI_VBI     0xf72a76e0L  //mmioFOURCC( 'V', 'B', 'I', ' ' )
#define KS_BI_UYVY    mmioFOURCC( 'U', 'Y', 'V', 'Y' ) /* 0x59565955 */
#define KS_BI_RAW     mmioFOURCC( 'R', 'A', 'W', ' ' )
#define KS_BI_I420    mmioFOURCC( 'I', '4', '2', '0' ) /* 0x30323449 */

#endif