/*
	Adapter.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#ifndef ADAPTER_HPP_
#define ADAPTER_HPP_

#include "Debug.hpp"
#include "WDMHeaders.hpp"
#include "SrbQueue.hpp"
#include "SpinLock.hpp"

// Adapter signature, to track down 'this' problems!
#if DBG
#define ADAPTER_SIGNATURE 0xCAFEAA55
#endif

// Class declaration
class Adapter {

	public:

		// Hardware Extension to be able to store a pointer to our instance
		typedef struct tagDATA_EXTENSION {
			class Adapter * adapter;			// Pointer to our instance	
		} DATA_EXTENSION, *PDATA_EXTENSION;

		// Power States
		typedef enum {
			Pwr_DeviceOn,	// Device Is ON
			Pwr_DeviceIdle,	// Device is idle, but can be used anytime
			Pwr_DeviceOff	// Device is OFF
		} PwrState;

#if DBG
		DWORD Signature;	// Signature to track down problems!
#endif

	protected:

		// Data of this instance
		PDEVICE_OBJECT			 pPDO;
		SrbQueue				 srbQueue;	// SRB Queue
		SpinLock				 lock;
		volatile BOOL			 srbInProcess;
		volatile PHW_STREAM_REQUEST_BLOCK mustQuitSrb;
		PwrState				 pwrState;	// The state of the device
		LONG					 ISRLock;	// Lock count for the ISR

	public:

		Adapter(PDEVICE_OBJECT pPDO_);
		virtual ~Adapter();
		virtual void Uninit( void );

		PHW_STREAM_REQUEST_BLOCK MustQuit(void) const {return mustQuitSrb;}
		static void CompleteCtrlPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		static void CompleteDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb ) { CompleteCtrlPacket( pSrb ); }

	public:

		// All this fns must be called by user!
		void ReceiveCtrlPacket( PHW_STREAM_REQUEST_BLOCK pSrb ); 
		static void STREAMAPI cbReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		static void STREAMAPI cbCancelPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		static void STREAMAPI cbTimeoutPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		static BOOLEAN STREAMAPI cbInterrupt( IN PVOID pHwDevExt );


	private:

		void CancelPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		void TimeoutPacket( PHW_STREAM_REQUEST_BLOCK pSrb );

	protected:

		// Device related fns.
		virtual BOOLEAN HwInterrupt( void );

		// functions to process each command
		virtual NTSTATUS CompleteInitialization( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		static  NTSTATUS UnInitialize( PHW_STREAM_REQUEST_BLOCK pSrb ); // Static to let know "this" will probably disappear!
		virtual NTSTATUS PowerUp( void );
		virtual NTSTATUS PowerDown( void );
		virtual NTSTATUS GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS OpenInstance( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS CloseInstance( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS UnknownCommand( PHW_STREAM_REQUEST_BLOCK pSrb );

		// Stream related command fns.
		virtual NTSTATUS GetStreamInfo( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS StreamFormatFromRange( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamOpen( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamClose( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamGetState( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamSetState( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamSetFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamGetFormat( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamGetProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamSetProperty( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual NTSTATUS StreamIndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb );

		// Stream packet interfase
		virtual void	 StreamReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual void	 StreamCancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual void	 StreamTimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );

	protected:
		void AcquireISR( void );
		void ReleaseISR( void );
};

#endif

