/*
	Queue.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/


#ifndef __CQUEUE_HPP_
#define __CQUEUE_HPP_

#include "Debug.hpp"

// -This Queue MUST not become Full because entry loss would happen.
// -The maximum capacity allowed is MaxElements - 1.
// -Can be used on a multithread context and no locking of Queue is 
//  required if the only fns called are Insert() by the push thread
//	and Remove() by the pop thread. All other kinds of uses must protect
//  the Queue with a Mutex


#define MAX_CPUS 16	/* Maximum allowable number of CPUs (who has a bord with more than this ?) */



template<class E,int log2MaxElements> class Queue {

	private:

		// List of pointers to the queue
		E m_eList[(1 << log2MaxElements)];
	
		// Index to the first element to Get
		LONG m_lFirstToGet;

		// Index to the first free element
		LONG m_lFirstFree;

		// Count of elements in Queue.
		LONG m_lCount;

	public:

		// Constructor
		Queue() : m_lFirstToGet (0L) , m_lFirstFree(0L) , m_lCount(0) 
		{
			TRACE(("Queue::Queue"));
			ENSURE(((1 << log2MaxElements) > MAX_CPUS));
		}

#if DBG
		// Debugging Purposes (not SMP safe)
		void Dump() {
			DUMP(("Dump: FirstFree = %08x, FirstToGet = %08x\n", m_lFirstFree, m_lFirstToGet ));
			LONG Idx    = m_lFirstToGet & ((1 << log2MaxElements)-1);	// Atomic read (Hope!)
			LONG EndIdx = m_lFirstFree  & ((1 << log2MaxElements)-1);	// Atomic read (Hope!)
			while (Idx != EndIdx) {

				DUMP(("Element #%d , Value %08x\n",Idx,m_eList[Idx]));
				Idx = (Idx + 1)         & ((1 << log2MaxElements)-1);
			};
		}
#endif

		// To insert an element ... (SMP safe)
		bool Insert(E el) {

			TRACE(("Queue::Insert"));
			DUMP(("El = %08x / Before Insert, FirstFree = %08x, FirstToGet = %08x\n", el , m_lFirstFree, m_lFirstToGet ));

			// Reserve an element...
			LONG Count = InterlockedIncrement(&m_lCount);

			// If Queue was full, fail insertion...
			if (Count >= ((1 << log2MaxElements) - MAX_CPUS) ) {

				// Free the test element...
				InterlockedDecrement(&m_lCount);

				// Failure...
				ERROR(("Queue is full\n"));
				return false;
			}

			// Make room for the new element and get the place where the element should be put
			LONG Pos = (InterlockedIncrement(&m_lFirstFree) - 1) & ((1 << log2MaxElements)-1);

			// Insert element.
			m_eList[Pos] = el;

			// Success!
			return true;
		}

		// To remove an element ... (SMP Safe)
		bool Remove(E& el) {

			TRACE(("Queue::Remove"));

			// Check if we have at least an element...
			LONG Count = InterlockedDecrement(&m_lCount);

			// If Queue was empty, fail remove...
			if (Count < 0) {

				// Free the element...
				InterlockedIncrement(&m_lCount);

				// Failure...
				DUMP(("Queue is empty\n"));
				return false;
			}

			// Get the First element to get idx and increment index
			LONG Pos = (InterlockedIncrement(&m_lFirstToGet) - 1) & ((1 << log2MaxElements)-1);

			DUMP(("Removed %08x, After Remove, FirstFree = %08x, FirstToGet = %08x\n",el , m_lFirstFree, m_lFirstToGet ));

			// Now return the first element to get
			el = m_eList[Pos];

			// Success
			return true;
		}

		// To peek contents of the last inserted element (SMP safe, But nooen can be sure of Queue contents between Peek and Poke)
		bool Peek(E& el)
		{
			TRACE(("Queue::Peek"));

			// Check that queue has at least one element!
			if (m_lCount < 1) { // Atomic read

				DUMP(("Queue is empty\n"));
				// Failure...
				return false;
			}

			// Calculate index of last inserted element
			LONG LastPushed = m_lFirstFree & ((1 << log2MaxElements)-1); // Assumes atomic reads
			LastPushed =    ((LastPushed) ? LastPushed : (1 << log2MaxElements) ) - 1;

			// Now return the last element pushed without removing it!
			el = m_eList[LastPushed];

			// Success...
			return true;
		}

		// To poke (change) the first element to be removed from Queue (Not SMP safe)
		bool Poke( E el )
		{
			TRACE(("Queue::Poke"));

			// Check that queue has at least one element - Else, simply push it.
			if (m_lCount < 1) {
				return Insert(el);
			}

			// Calculate index of last inserted element
			LONG LastPushed = m_lFirstFree & ((1 << log2MaxElements)-1); // Assumes atomic reads
			LastPushed = ((LastPushed) ? LastPushed : (1 << log2MaxElements) ) - 1;

			// Now set the last element pushed without removing it!
			m_eList[LastPushed] = el;

			return true;
		}

		// To remove a selected element if found (Not SMP safe)
		bool RemoveThis(E el) 
		{
			TRACE(("Queue::RemoveThis(el)"));
			DUMP(("El = %08x / Before Lookup, FirstFree = %08x, FirstToGet = %08x\n", el , m_lFirstFree, m_lFirstToGet ));
			
#if DBG
			Dump();
#endif

			LONG Idx    = m_lFirstToGet & ((1 << log2MaxElements)-1); // Assume Atomic Read
			LONG EndIdx = m_lFirstFree  & ((1 << log2MaxElements)-1); // Assume Atomic Read
			while (Idx != EndIdx) {

				if (m_eList[Idx] == el ) {

					DUMP(("Found it at %d, Removing item\n",Idx));

					// Got the element, we must take that out ...
					// Move from Idx to FirstFree 1 element backwards
					LONG Next = ( Idx + 1 ) & ((1 << log2MaxElements)-1);
					while (Next != EndIdx) {
						// Pass element!
						m_eList[Idx] = m_eList[Next];
						// Advance pointers!
						Idx  = Next;
						Next = ( Idx + 1 )  & ((1 << log2MaxElements)-1);
					};

					// And remember we have one less element,.
					m_lFirstFree = ((EndIdx == 0) ? (1 << log2MaxElements) : EndIdx ) - 1; // Assumes Atomic Writes

					// Keep in mind we have less elements into Queue
					InterlockedDecrement(&m_lCount);

					// Got It!
					DUMP(("Removed. Now, FirstFree = %08x, FirstToGet = %08x\n", m_lFirstFree, m_lFirstToGet ));

#if DBG
					Dump();
#endif

					return true;
				}

				Idx = (Idx + 1) & ((1 << log2MaxElements)-1);
			};
			return false;
		}
};

#endif