/*
	Stream.hpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

/*
 * Classes for Streams
 *
 */

#ifndef STREAM_H_
#define STREAM_H_

#include "WDMHeaders.hpp"

// Base Class
class BaseStream {

	protected:

		BaseStream() {}
		virtual ~BaseStream() {}

	public:

		virtual HW_STREAM_INFORMATION& Info( void ) = 0;
		virtual ULONG GetVideoControlCaps( void ) = 0;
		virtual ULONG GetVideoControlMode( void ) = 0;
		virtual NTSTATUS SetVideoControlMode( ULONG mode ) = 0;
		virtual LONGLONG GetActualFramerateInNs( void ) = 0;
		virtual LONGLONG GetMaxAvailableFramerateInNs( void ) = 0;
		virtual NTSTATUS GetAnalogWindow( RECT& rc ) = 0;
		virtual NTSTATUS SetAnalogWindow( const RECT& rc ) = 0;
		virtual NTSTATUS GetDigitalWindow( RECT& rc ) = 0;
		virtual BOOL IsOpen( void ) = 0;

		// One function for each possible Packet!
		virtual NTSTATUS FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS Open( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS Close( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetState( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetState( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS ProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS IndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;

		virtual void ReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual void TimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual void CancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;

	protected:

		// Utility Functions
		static void CompleteDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );

		static bool CompareGUIDsAndFormatSize(
			IN const PKSDATARANGE DataRange1,
			IN const PKSDATARANGE DataRange2,
			IN BOOL bCheckSize);

};

// Output Streams (Data generated by Driver)
class CaptureStream : public BaseStream {

	public:

		CaptureStream();

		virtual HW_STREAM_INFORMATION& Info( void ) = 0;
		virtual ULONG GetVideoControlCaps( void ) = 0;
		virtual ULONG GetVideoControlMode( void ) = 0;
		virtual NTSTATUS SetVideoControlMode( ULONG mode ) = 0;
		virtual LONGLONG GetActualFramerateInNs( void ) = 0;
		virtual LONGLONG GetMaxAvailableFramerateInNs( void ) = 0;
		virtual NTSTATUS GetAnalogWindow( RECT& rc ) = 0;
		virtual NTSTATUS SetAnalogWindow( const RECT& rc ) = 0;
		virtual NTSTATUS GetDigitalWindow( RECT& rc ) = 0;
		virtual BOOL IsOpen( void ) = 0;

		// One function for each possible Packet!
		virtual NTSTATUS FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS Open( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS Close( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetState( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetState( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS ProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS IndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;

		virtual void ReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual void TimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual void CancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;

	protected:

		// Called when a Data packet is received!
		virtual void OnReadData( PHW_STREAM_REQUEST_BLOCK pSrb) = 0;

};

class AnalogStream : public BaseStream {

	public:

		virtual HW_STREAM_INFORMATION& Info( void ) = 0;
		virtual ULONG GetVideoControlCaps( void ) ;
		virtual ULONG GetVideoControlMode( void ) ;
		virtual NTSTATUS SetVideoControlMode( ULONG mode ) ;
		virtual LONGLONG GetActualFramerateInNs( void ) ;
		virtual LONGLONG GetMaxAvailableFramerateInNs( void ) ;
		virtual NTSTATUS GetAnalogWindow( RECT& rc );
		virtual NTSTATUS SetAnalogWindow( const RECT& rc );
		virtual NTSTATUS GetDigitalWindow( RECT& rc );
		virtual BOOL IsOpen( void ) = 0;

		// One function for each possible Packet!
		virtual NTSTATUS Open( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS Close( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS FormatFromRange( IN PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetState( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetState( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS ProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetFormat( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS GetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS SetProperty( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;
		virtual NTSTATUS IndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;

		virtual void ReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual void TimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );
		virtual void CancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb );

	protected:

		virtual NTSTATUS ChangeNotifyChannels( PHW_STREAM_REQUEST_BLOCK pSrb ) = 0;

};

       
#endif

