/*
	Adapter.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Adapter.hpp"
#include "debug.hpp"
#include "CppRt.hpp"

Adapter::Adapter(PDEVICE_OBJECT pPDO_) : 
	pPDO(pPDO_) , 
	srbInProcess(FALSE) ,
#if DBG
	Signature(ADAPTER_SIGNATURE) ,
#endif
	mustQuitSrb(NULL)  ,
	pwrState(Pwr_DeviceOn) ,
	ISRLock(0)
{
}

Adapter::~Adapter() 
{
	TRACE(("Adapter::~Adapter"));
}

void Adapter::Uninit() 
{
	TRACE(("Adapter::Uninit"));

	// Get mutex and cancel all pending SRBs SHOULD NOT be neccesary! .... 
	lock.Acquire();	
	
	PHW_STREAM_REQUEST_BLOCK pSrb;

	while (srbQueue.Remove(pSrb)) {
		pSrb->Status = STATUS_SUCCESS;
		CompleteCtrlPacket( pSrb );
	};

	lock.Release();
}

void Adapter::CompleteCtrlPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::CompleteCtrlPacket"));

	DUMP(("Completing a %s Control SRB %08x\n",(pSrb->Flags & SRB_HW_FLAGS_STREAM_REQUEST) ? "Stream" : "Device" , pSrb));

	if (pSrb->Flags & SRB_HW_FLAGS_STREAM_REQUEST) {
		// It's an stream request, complete it!
		StreamClassStreamNotification( StreamRequestComplete, pSrb->StreamObject, pSrb );
	} else {
		// It's a device request, complete it!
		StreamClassDeviceNotification( DeviceRequestComplete, pSrb->HwDeviceExtension, pSrb );
	}
}


void Adapter::ReceiveCtrlPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::ReceiveCtrlPacket"));

	DUMP(("Receiving a %s SRB %08x\n",(pSrb->Flags & SRB_HW_FLAGS_STREAM_REQUEST) ? "Stream" : "Device" , pSrb));

	// If we must Quit, just discard all the following received packets
	if (mustQuitSrb) {
		pSrb->Status = STATUS_SUCCESS;
		CompleteCtrlPacket( pSrb );

		// Return to caller
		return;
	}

	// Mark the SRB as pending , until we process it!
	pSrb->Status = STATUS_PENDING;


	// The SRBs are processed in the order they were received.

	// Get Mutex
	lock.Acquire();		

	// Insert SRB into Queue
	if (!srbQueue.Insert( pSrb )) {
		ERROR(("SRB Queue is full; aborting command\n"));

		// If we were not able to handle it, just signal that to windows...
		pSrb->Status = STATUS_INSUFFICIENT_RESOURCES;
		CompleteCtrlPacket( pSrb );
	}

	if( srbInProcess ) { 
		DUMP(( "SRB in progress\n" ));
		lock.Release();
		return;
	}

	srbInProcess = TRUE;
	do {
	
		// First try to get a PowerOn SRB if one was received... If no SRB to process, break loop
		if (!srbQueue.Remove(pSrb)) {
			break;
		}

		// Free mutex
		lock.Release();  

		ENSURE( pSrb );
		ENSURE( pSrb->Status != STATUS_CANCELLED );

		// Make sure Adapter is ON if processing something ...
		bool Done = false;
		if (pSrb->Command != SRB_CHANGE_POWER_STATE && pSrb->Command != SRB_PAGING_OUT_DRIVER) {
			// Check if we can do it...
			if (pwrState == Pwr_DeviceIdle) {
				// If we are Idle, reactivate it...
				if (PowerUp() != STATUS_SUCCESS) {
					pSrb->Status = STATUS_INVALID_DEVICE_REQUEST;
					Done = true;
				} else {
					// We are now on, ready to go.
					pwrState = Pwr_DeviceOn;
				}
			} else 
			if (pwrState == Pwr_DeviceOff) {
				// If we are Off, We can't handle this in this way...
				pSrb->Status = STATUS_INVALID_DEVICE_REQUEST;
				Done = true;
			} else {
				// We are ready to handle it.
				Done = false;
			}
		}

		// If Command not processed yet...
		if (!Done) {
			switch( pSrb->Command )	{
				
				// --- Adapter SRBs ---------------------------------------
			case SRB_INITIALIZATION_COMPLETE:
				pSrb->Status = CompleteInitialization( pSrb );
				break;
				
			case SRB_UNINITIALIZE_DEVICE:
				mustQuitSrb = pSrb;
				break;
				
			case SRB_CHANGE_POWER_STATE:
				// We split those fns into PowerUp and PowerDown fns
				switch (pSrb->CommandData.DeviceState) {
				case PowerDeviceD3:
					DUMP(("PowerDeviceD3 - powering down\n"));
					
					// If On, turn it off
					if (pwrState == Pwr_DeviceOn) {
						pSrb->Status = PowerDown();
						if (pSrb->Status == STATUS_SUCCESS) {
							pwrState = Pwr_DeviceOff;
						}
					} else {
						pwrState = Pwr_DeviceOff;		// We promote Idle to Off
						pSrb->Status = STATUS_SUCCESS;
					}
					break;
				case PowerDeviceD0:
					DUMP(("PowerDeviceD0 - powering up\n"));
					
					// If not On, turn it!
					if (pwrState != Pwr_DeviceOn) { // Off or Idle
						pSrb->Status = PowerUp();
						if (pSrb->Status == STATUS_SUCCESS) {
							pwrState = Pwr_DeviceOn;
						}
					} else {
						pSrb->Status = STATUS_SUCCESS;
					}
					break;
				default:
					// All the others are ok,
					pSrb->Status = STATUS_SUCCESS;
					break;
				}
				break;
				
			case SRB_PAGING_OUT_DRIVER:
					
				// This is handled as a PowerDown Operation with Idle (we make sure Adapter is turned off)
				DUMP(("PagingOutDriver: - Idling Device\n"));
				// If on, turn it idle
				if (pwrState == Pwr_DeviceOn) { // Only On is promoted to Idle.
					pSrb->Status = PowerDown();
					if (pSrb->Status == STATUS_SUCCESS) {
						pwrState = Pwr_DeviceIdle;
					}
				} else {
					pSrb->Status = STATUS_SUCCESS;
				}
				break;
				
			case SRB_GET_DEVICE_PROPERTY:
				pSrb->Status = GetProperty( pSrb );
				break;        
				
			case SRB_SET_DEVICE_PROPERTY:
				pSrb->Status = SetProperty( pSrb );
				break;
				
			case SRB_OPEN_DEVICE_INSTANCE:
				pSrb->Status = OpenInstance( pSrb );
				break;
				
			case SRB_CLOSE_DEVICE_INSTANCE:
				pSrb->Status = CloseInstance( pSrb );
				break;
				
			case SRB_UNKNOWN_DEVICE_COMMAND:
				pSrb->Status = UnknownCommand( pSrb );
				break;
				
			case SRB_GET_STREAM_INFO:
				pSrb->Status = GetStreamInfo( pSrb );
				break;
				
			case SRB_GET_DATA_INTERSECTION: // Return a format, given a range
				pSrb->Status = StreamFormatFromRange( pSrb );
				break;
				
			case SRB_OPEN_STREAM:
				pSrb->Status = StreamOpen( pSrb );
				break;
				
			case SRB_CLOSE_STREAM:
				pSrb->Status = StreamClose( pSrb );
				break;
				
				// --- Stream SRBs ---------------------------------------
			case SRB_SET_STREAM_STATE:
				pSrb->Status = StreamSetState( pSrb );
				break;
				
			case SRB_GET_STREAM_STATE:
				pSrb->Status = StreamGetState( pSrb );
				break;
				
			case SRB_PROPOSE_DATA_FORMAT:
				pSrb->Status = StreamProposeFormat( pSrb );
				break;
				
			case SRB_SET_DATA_FORMAT:
				pSrb->Status = StreamSetFormat( pSrb );
				break;
				
			case SRB_GET_DATA_FORMAT:
				pSrb->Status = StreamGetFormat( pSrb );
				break;
				
			case SRB_GET_STREAM_PROPERTY:
				pSrb->Status = StreamGetProperty( pSrb );
				break;
				
			case SRB_SET_STREAM_PROPERTY:
				pSrb->Status = StreamSetProperty( pSrb );
				break;
				
			case SRB_INDICATE_MASTER_CLOCK:
				pSrb->Status = StreamIndicateMasterClock( pSrb );
				break;
				
			default:
				ERROR(( "( SRB unknown 0x%08X )\n", pSrb->Command ));
				pSrb->Status = STATUS_NOT_IMPLEMENTED;
				break;
			}
		}

		if (pSrb != mustQuitSrb) {
			CompleteCtrlPacket( pSrb );
		}

		lock.Acquire();
	} while (1);

	srbInProcess = FALSE;
	lock.Release();

}

void STREAMAPI Adapter::cbReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE_EP(( "Adapter::cbReceiveDataPacket" ));
	PDATA_EXTENSION pDataExt = (PDATA_EXTENSION) pSrb->HwDeviceExtension;
	if( pDataExt == NULL ) {

		ERROR(( "DataExtension = NULL" ));
		pSrb->Status = STATUS_CANCELLED;
		CompleteDataPacket( pSrb );

	} else if ( pDataExt->adapter == NULL ) {

		ERROR(( "adapter = NULL" ));
		pSrb->Status = STATUS_CANCELLED;
		CompleteDataPacket( pSrb );

	} else {

#if DBG
		if (pDataExt->adapter->Signature != ADAPTER_SIGNATURE) {
			ERROR(( "Invalid signature for adapter!" ));
			pSrb->Status = STATUS_CANCELLED;
			CompleteDataPacket( pSrb );

		} else
#endif

		// Get pointer to Device and call our function
		pDataExt->adapter->StreamReceiveDataPacket(pSrb);
	}
}

void STREAMAPI Adapter::cbCancelPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE_EP(( "Adapter::cbCancelPacket" ));
	PDATA_EXTENSION pDataExt = (PDATA_EXTENSION) pSrb->HwDeviceExtension;
	if( pDataExt == NULL ) {

		ERROR(( "DataExtension = NULL" ));
		pSrb->Status = STATUS_CANCELLED;

	} else if ( pDataExt->adapter == NULL ) {

		ERROR(( "adapter = NULL" ));
		pSrb->Status = STATUS_CANCELLED;

	} else {

#if DBG
		if (pDataExt->adapter->Signature != ADAPTER_SIGNATURE) {
			ERROR(( "Invalid signature for adapter!" ));
			pSrb->Status = STATUS_CANCELLED;
		} else
#endif

		// Get pointer to Device and call our function
		pDataExt->adapter->CancelPacket(pSrb);
	}
}

void STREAMAPI Adapter::cbTimeoutPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE_EP(( "Adapter::cbTimeoutPacket" ));
	PDATA_EXTENSION pDataExt = (PDATA_EXTENSION) pSrb->HwDeviceExtension;
	if( pDataExt == NULL ) {

		ERROR(( "DataExtension = NULL" ));
		pSrb->Status = STATUS_SUCCESS;

	} else if ( pDataExt->adapter == NULL ) {

		ERROR(( "adapter = NULL" ));
		pSrb->Status = STATUS_SUCCESS;

	} else {

#if DBG
		if (pDataExt->adapter->Signature != ADAPTER_SIGNATURE) {
			ERROR(( "Invalid signature for adapter!" ));
			pSrb->Status = STATUS_SUCCESS;
		} else
#endif

		// Get pointer to Device and call our function
		pDataExt->adapter->TimeoutPacket(pSrb);
	}
}

BOOLEAN STREAMAPI Adapter::cbInterrupt( IN PVOID pHwDevExt )
{
	TRACE_EP(("Adapter::cbInterrupt"));
	PDATA_EXTENSION pDataExt = (PDATA_EXTENSION)pHwDevExt;

	// If no extension , or no object ... 
	if( pDataExt == NULL ) {
		ERROR(( "DataExtension = NULL" ));
		return FALSE;
	} 

	if ( pDataExt->adapter == NULL ) {
		ERROR(( "adapter = NULL" ));
		return FALSE;
	}

#if DBG
		if (pDataExt->adapter->Signature != ADAPTER_SIGNATURE) {
			ERROR(( "Invalid signature for adapter!" ));
			return FALSE;
		} 
#endif

	// Adapter exists... Make sure we can actually call this...
	if (InterlockedIncrement(&pDataExt->adapter->ISRLock) > 1) {
		// Someone else is owning the ISR, don't call our ISR
		InterlockedDecrement(&pDataExt->adapter->ISRLock);
		// We were not causing the IRQ.
		return FALSE;
	}


	// Else, give control to user.
	BOOLEAN res = pDataExt->adapter->HwInterrupt();

	// Release ISR.
	InterlockedDecrement(&pDataExt->adapter->ISRLock);

	// Return the result.
	return res;
}


// This is called to cancel a Control/Data SRB
void Adapter::CancelPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "Adapter::CancelPacket" ));

	// Get mutex
	lock.Acquire();		

	// Assume it is a Control Srb and try to take out of Control Queue.
	bool Found		  = srbQueue.RemoveThis( pSrb );

	// Unlock mutex
	lock.Release();


	if (Found) {
		pSrb->Status = STATUS_CANCELLED;
	} else {
		// Not found, if it is a Stream Srb, Dispatch cancellation
		// assuming it is a Data Srb.
		if (pSrb->Flags & SRB_HW_FLAGS_STREAM_REQUEST) {
			StreamCancelDataPacket( pSrb );
		} 
#if DBG
		else {	
			ERROR(("Can't cancel this Adapter SRB (%08x)\n",pSrb));
		}
#endif
	}
}

// This is called to timeout a packet
void Adapter::TimeoutPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "Adapter::TimeoutPacket" ));

	// Get mutex
	lock.Acquire();		

	// Assume it is a Control Srb and try to take out of Control Queue.
	bool Found = srbQueue.RemoveThis( pSrb );

	// Unlock mutex
	lock.Release();

	if (Found) {
		pSrb->Status = STATUS_SUCCESS;
	} else {
		// Not found, if it is a Stream Srb, Dispatch timeout
		// assuming it is a Data Srb.
		if (pSrb->Flags & SRB_HW_FLAGS_STREAM_REQUEST) {
			StreamTimeoutDataPacket( pSrb );
		} 
#if DBG
		else {	
			ERROR(("Can't timeout this SRB (%08x) ... Extending life!\n",pSrb));
			pSrb->TimeoutCounter = pSrb->TimeoutOriginal;
		}
#endif
	}
}

// Be careful, called at raised IRQL!
BOOLEAN Adapter::HwInterrupt( void )
{
	TRACE(( "Adapter::HwInterrupt" ));
	return FALSE; // We don't own the int by default!
}

NTSTATUS Adapter::UnInitialize( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "Adapter::UnInitialize" ));
	PDATA_EXTENSION pDataExt = (PDATA_EXTENSION)pSrb->HwDeviceExtension;

	Adapter* adapter = pDataExt->adapter;

	// Uninit adapter
	adapter->Uninit();

	// Acquire the ISR...
	adapter->AcquireISR();

	// Noone is inside the ISR (we own it)... Avoid calling our adapter anymore. 
	// As soon as we assign it 0, no more IRQs are routed there!
	pDataExt->adapter = 0;

	// Destroy Adapter! - End of Show!
	delete adapter;

	return STATUS_SUCCESS;
}

void Adapter::AcquireISR( void )
{
	TRACE(("Adapter::AcquireISR"));

	// Get the ISR lock...
	while (InterlockedIncrement(&ISRLock) > 1) {
		InterlockedDecrement(&ISRLock);
		mdelay(1);
		DUMP(("waiting for ISR to be idle\n"));
	}
}

void Adapter::ReleaseISR( void )
{
	TRACE(("Adapter::ReleaseISR"));
	InterlockedDecrement(&ISRLock);
}

NTSTATUS Adapter::PowerDown( void )
{
	TRACE(("Adapter::PowerDown"));
	return STATUS_SUCCESS;
}

NTSTATUS Adapter::PowerUp( void )
{
	TRACE(("Adapter::PowerUp"));
	return STATUS_SUCCESS;
}

NTSTATUS Adapter::OpenInstance( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::OpenInstance"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::CloseInstance( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::CloseInstance"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::UnknownCommand( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::UnknownCommand"));
	return STATUS_NOT_IMPLEMENTED;
}


NTSTATUS Adapter::StreamFormatFromRange( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::StreamFormatFromRange"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamOpen( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::StreamOpen"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamClose( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::StreamClose"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamGetState( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamGetState")); 
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamSetState( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamSetState"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamProposeFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamProposeFormat"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamSetFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamSetFormat")); 
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamGetFormat( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamGetFormat"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamGetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamGetProperty"));
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamSetProperty( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamSetProperty")); 
	return STATUS_NOT_IMPLEMENTED;
}

NTSTATUS Adapter::StreamIndicateMasterClock( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamIndicateMasterClock"));
	return STATUS_NOT_IMPLEMENTED;
}

void Adapter::StreamReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{ 
	TRACE(("Adapter::StreamReceiveDataPacket"));
	pSrb->Status = STATUS_CANCELLED;
	CompleteDataPacket(pSrb);
}

void Adapter::StreamCancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::StreamCancelDataPacket"));
}

void Adapter::StreamTimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("Adapter::StreamTimeoutDataPacket"));
}
