/*
	Debug.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "Debug.hpp"

// Debugging Aids
#define LOG_TOFILE				1							/* Used to log ALL debug messages to a file (don't use it with a debugger) */
#define DEFAULT_LOG_FILE_NAME	L"\\??\\C:\\Bt848.log"		/* The log filename */
#define TRACE_ALL				1							/* To force tracing of all messages */

#if DBG
#include "WDMHeaders.hpp"

// We will trace from this
char gTrace0[64] = { 0 };
char gTrace1[64] = { 0 };
char gTrace2[64] = { 0 };
char gTrace3[64] = { 0 };

// Our logging fn
void LogPrint(char* szFormat, ...);

// Function to check if FunctionName matches
bool _Trace::Matchs(char * Fn,char *with)
{
	do {
		// * means match from here all!
		if (*with == '*') {
			return true;
		}

		// Match with case
		if (*Fn != *with) {
			return false;
		}

		Fn++;
		with++;
	} while (*Fn || *with) ;
	// If we get here, both strings are exact!
	return true;
}

void _Trace::PrintIndent(int stk,int i)
{
	char _Buffer[64];
	memset(_Buffer,'|',i);
	_Buffer[i] = 0;
	DbgPrint("[%02d]%s",stk,_Buffer);
	LogPrint("[%02d]%s",stk,_Buffer);
}


_Trace::_Trace(bool EntryPoint,char* Fn) : 
	// Remember the original Settings
	iOrigDebugStack(DebugStack)
{
	// If we must point to a new Debug stack, do it!
	if (EntryPoint) {
		InterlockedIncrement(PLONG(&DebugStack));
	}

	// Now, we must push the function name into the Stack
	LONG Pos = InterlockedIncrement(PLONG(&Stack[DebugStack].Top));

	// Store Name in Stack
	Stack[DebugStack].Stack[Pos-1] = Fn;

	// Remember the setting to restore it!
	bOrigMustTrace = Stack[DebugStack].MustTrace;

	// Check if we must trace from here ... 
	if (!Stack[DebugStack].MustTrace) {
#ifndef TRACE_ALL
		Stack[DebugStack].MustTrace = (
			Matchs(Fn,gTrace0) ||
			Matchs(Fn,gTrace1) ||
			Matchs(Fn,gTrace2) ||
			Matchs(Fn,gTrace3) );
#else 
		Stack[DebugStack].MustTrace = true;
#endif
	}

	// If we must trace, do it!
	if (MustTrace()) {
		// Print fn
		DbgTimeStamp();
		PrintIndent(DebugStack,Stack[DebugStack].Top - 1);
		DbgPrint("/-- %s()\n",Fn);
		LogPrint("/-- %s()\n",Fn);
	}
}

_Trace::~_Trace()
{
	// If we must trace ...
	if (MustTrace()) {
		// Print fn
		DbgTimeStamp();
		PrintIndent(DebugStack,Stack[DebugStack].Top - 1);
		DbgPrint("\\-- %s()\n",
			(Stack[DebugStack].Stack[Stack[DebugStack].Top - 1]) ? 
				Stack[DebugStack].Stack[Stack[DebugStack].Top - 1] : 
				"???" );
		LogPrint("\\-- %s()\n",
			(Stack[DebugStack].Stack[Stack[DebugStack].Top - 1]) ? 
				Stack[DebugStack].Stack[Stack[DebugStack].Top - 1] : 
				"???" );
	}

	// Restore Stack
	InterlockedDecrement(PLONG(&Stack[DebugStack].Top));
	Stack[DebugStack].MustTrace = bOrigMustTrace;
	DebugStack = iOrigDebugStack;
}

void _Trace::TraceBack( void )
{
	bool First = true;
	for (int idx = Stack[DebugStack].Top - 1 ; idx >= 0; idx--) {
		if (First) {
			DbgPrint("In %s()\n",
				(Stack[DebugStack].Stack[idx]) ? 
				Stack[DebugStack].Stack[idx] : 
				"???" );
			LogPrint("In %s()\n",
				(Stack[DebugStack].Stack[idx]) ? 
				Stack[DebugStack].Stack[idx] : 
				"???" );
			First = false;
		} else {
			DbgPrint(" Called By %s()\n",
				(Stack[DebugStack].Stack[idx]) ? 
				Stack[DebugStack].Stack[idx] : 
				"???" );
			LogPrint(" Called By %s()\n",
				(Stack[DebugStack].Stack[idx]) ? 
				Stack[DebugStack].Stack[idx] : 
				"???" );
		}
	};
}

// Debug Stack to be used
int					_Trace::DebugStack = 0;	

// Stacks
_Trace::StackEntry	_Trace::Stack[16] = {
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}},
	{false,0,{0}}
};

typedef struct {
	char buf[65536]; // Character buffer
	LONG FirstToGet; // First char to get...
	LONG FirstFree;	 // And the first free char..
} dbg_queue;

void DbgTimeStamp()
{
	// Get current system time...
	LARGE_INTEGER time;
	KeQuerySystemTime(&time);
	
	// And output the TimeStamp info
	DbgPrint("%10u-%10u|",time.HighPart,time.LowPart);
	LogPrint("%10u-%10u|",time.HighPart,time.LowPart);
}


// The prototype of the generalized printf-like proto
int dbg_printf(dbg_queue* p, char *control, int* argptr);

// This fn is our DbgPrint-like fn that logs with extra logging facilities!
void LogPrint(char* szFormat, ...)
{
#ifdef LOG_TOFILE

	// Our print buffer... 
	static dbg_queue dq = {{0},0L,0L};

	// Print the debug message in the debug Queue...
	// And then, the things we need...
	dbg_printf(&dq,szFormat,(int *)(&szFormat+1));

	// If we were called at passive level, we can actually do the log dump - Do it!
	if (KeGetCurrentIrql() == PASSIVE_LEVEL) {

		ULONG Length;

	    IO_STATUS_BLOCK  IoStatus;
		OBJECT_ATTRIBUTES objectAttributes;
		NTSTATUS status;
		HANDLE FileHandle;
	    UNICODE_STRING fileName;

		// get a handle to the log file object
		fileName.Buffer = NULL;
		fileName.Length = 0;
		fileName.MaximumLength = sizeof(DEFAULT_LOG_FILE_NAME) + sizeof(UNICODE_NULL);
		fileName.Buffer = (unsigned short*) ExAllocatePool(PagedPool, fileName.MaximumLength);
	    if (!fileName.Buffer)
	    {
		    return ;
		}

		RtlZeroMemory(fileName.Buffer, fileName.MaximumLength);
		status = RtlAppendUnicodeToString(&fileName, (PWSTR)DEFAULT_LOG_FILE_NAME);
	
		InitializeObjectAttributes (&objectAttributes,
									(PUNICODE_STRING)&fileName,
									OBJ_CASE_INSENSITIVE,
									NULL,
									NULL );

		status = ZwCreateFile(&FileHandle,
						  FILE_APPEND_DATA,
						  &objectAttributes,
						&IoStatus,
						  0, 
						  FILE_ATTRIBUTE_NORMAL,
						  FILE_SHARE_WRITE,
						  FILE_OPEN_IF,
						  FILE_SYNCHRONOUS_IO_NONALERT,
						  NULL,     
						  0 );

		if(NT_SUCCESS(status)) {
			// While there is something to write...
			while ( ((dq.FirstFree - dq.FirstToGet) & 0xFFFF) != 0) {

				LONG Pos = (InterlockedIncrement(&dq.FirstToGet) - 1) & 0xFFFF;
				CHAR c = (CHAR) dq.buf[Pos];
	
				ZwWriteFile(FileHandle,
					  NULL,
					NULL,
					NULL,
					&IoStatus,
					&c,
					1,
					NULL,
					NULL );
			};

			ZwClose(FileHandle);
		}
		if (fileName.Buffer) {
			ExFreePool (fileName.Buffer);
		}
	}
#endif
}

#ifdef LOG_TOFILE

// Here it follows an alternative sprintf-like fn but with buffer overflow protection...
// so we can redirect Debug output...

// Written by: Philip J. Erdelsky, 75746.3411@compuserve.com
// Date:          06-08-1992
// Restrictions:  Public domain; no restrictions on use 

#define BITS_PER_BYTE           8

struct parameters {
  int number_of_output_chars;
  short minimum_field_width;
  char options;
    #define MINUS_SIGN    1
    #define RIGHT_JUSTIFY 2
    #define ZERO_PAD      4
    #define CAPITAL_HEX   8
  short edited_string_length;
  short leading_zeros;
  int (*output_function)(void *, int);
  void *output_pointer;
};

static void output_and_count(struct parameters *p, int c)
{
  if (p->number_of_output_chars >= 0)
  {
    int n = (*p->output_function)(p->output_pointer, c);
    if (n>=0) p->number_of_output_chars++;
    else p->number_of_output_chars = n;
  }
}

static void output_field(struct parameters *p, char *s)
{
  short justification_length =
    p->minimum_field_width - p->leading_zeros - p->edited_string_length;
  if (p->options & MINUS_SIGN)
  {
    if (p->options & ZERO_PAD)
      output_and_count(p, '-');
    justification_length--;
  }
  if (p->options & RIGHT_JUSTIFY)
    while (--justification_length >= 0)
      output_and_count(p, p->options & ZERO_PAD ? '0' : ' ');
  if (p->options & MINUS_SIGN && !(p->options & ZERO_PAD))
    output_and_count(p, '-');
  while (--p->leading_zeros >= 0)
    output_and_count(p, '0');
  while (--p->edited_string_length >= 0)
    output_and_count(p, *s++);
  while (--justification_length >= 0)
    output_and_count(p, ' ');
}
    
int general_printf(int (*output_function)(void *, int), void *output_pointer,
  char *control_string, int *argument_pointer)
{
  struct parameters p;
  char control_char;
  p.number_of_output_chars = 0;
  p.output_function = output_function;
  p.output_pointer = output_pointer;
  control_char = *control_string++;
  while (control_char != '\0')
  {
    if (control_char == '%')
    {
      short precision = -1;
      short long_argument = 0;
      short base = 0;
      control_char = *control_string++;
      p.minimum_field_width = 0;
      p.leading_zeros = 0;
      p.options = RIGHT_JUSTIFY;
      if (control_char == '-')
      {
        p.options = 0;
        control_char = *control_string++;
      }
      if (control_char == '0')
      {
        p.options |= ZERO_PAD;
        control_char = *control_string++;
      }
      if (control_char == '*')
      {
        p.minimum_field_width = *argument_pointer++;
        control_char = *control_string++;
      }
      else
      {
        while ('0' <= control_char && control_char <= '9')
        {
          p.minimum_field_width =
            p.minimum_field_width * 10 + control_char - '0';
          control_char = *control_string++;
        }
      }
      if (control_char == '.')
      {
        control_char = *control_string++;
        if (control_char == '*')
        {
          precision = *argument_pointer++;
          control_char = *control_string++;
        }
        else
        {
          precision = 0;
          while ('0' <= control_char && control_char <= '9')
          {
            precision = precision * 10 + control_char - '0';
            control_char = *control_string++;
          }
        }
      }
      if (control_char == 'l')
      {
        long_argument = 1;
        control_char = *control_string++;
      }
      if (control_char == 'd')
        base = 10;
      else if (control_char == 'x')
        base = 16;
      else if (control_char == 'X')
      {
        base = 16;
        p.options |= CAPITAL_HEX;
      }
      else if (control_char == 'u')
        base = 10;
      else if (control_char == 'o')
        base = 8;
      else if (control_char == 'b')
        base = 2;
      else if (control_char == 'c')
      {
        base = -1;
        p.options &= ~ZERO_PAD;
      }
      else if (control_char == 's')
      {
        base = -2;
        p.options &= ~ZERO_PAD;
      }
      if (base == 0)  /* invalid conversion type */
      {
        if (control_char != '\0')
        {
          output_and_count(&p, control_char);
          control_char = *control_string++;
        }
      }
      else
      {
        if (base == -1)  /* conversion type c */
        {
          char c = *argument_pointer++;
          p.edited_string_length = 1;
          output_field(&p, &c);
        }
        else if (base == -2)  /* conversion type s */
        {
          char *string;
          p.edited_string_length = 0;
          string = * (char **) argument_pointer;
          argument_pointer += sizeof(char *) / sizeof(int);
          while (string[p.edited_string_length] != 0)
            p.edited_string_length++;
          if (precision >= 0 && p.edited_string_length > precision)
            p.edited_string_length = precision;
          output_field(&p, string);
        }
        else  /* conversion type d, b, o or x */
        {
          unsigned long x;
          char buffer[BITS_PER_BYTE * sizeof(unsigned long) + 1];
          p.edited_string_length = 0;
          if (long_argument) 
          {
            x = * (unsigned long *) argument_pointer;
            argument_pointer += sizeof(unsigned long) / sizeof(int);
          }
          else if (control_char == 'd')
            x = (long) *argument_pointer++;
          else
            x = (unsigned) *argument_pointer++;
          if (control_char == 'd' && (long) x < 0)
          {
            p.options |= MINUS_SIGN;
            x = - (long) x;
          }
          do 
          {
            int c;
            c = x % base + '0';
            if (c > '9')
            {
              if (p.options & CAPITAL_HEX)
                c += 'A'-'9'-1;
              else
                c += 'a'-'9'-1;
            }
            buffer[sizeof(buffer) - 1 - p.edited_string_length++] = c;
          }
          while ((x/=base) != 0);
          if (precision >= 0 && precision > p.edited_string_length)
            p.leading_zeros = precision - p.edited_string_length;
          output_field(&p, buffer + sizeof(buffer) - p.edited_string_length);
        }
        control_char = *control_string++;
      }
    }
    else
    {
      output_and_count(&p, control_char);
      control_char = *control_string++;
    }
  }
  return p.number_of_output_chars;
} 

static int fill_string(void *p, int c)
{
	dbg_queue* dq = (dbg_queue*) p;

	// If not full, insert char into queue
	if (((dq->FirstFree - dq->FirstToGet +1) & 0xFFFF) != 0) {
		LONG Pos = (InterlockedIncrement(&dq->FirstFree) - 1) & 0xFFFF;
		dq->buf[Pos] = (char)c;
	}
	return 0;
}

int dbg_printf(dbg_queue* p, char *control, int* argptr)
{
	int n = general_printf(fill_string, p, control,argptr);
	return n;
} 

#endif

#endif