/*
	PCI.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "pci.hpp"

NTSTATUS PCI::SynchCompletionRoutine(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp, IN PVOID Event)
{
	TRACE(("PCI::SynchCompletionRoutine"));

	KeSetEvent( (PKEVENT)Event, IO_NO_INCREMENT, FALSE );
	return STATUS_MORE_PROCESSING_REQUIRED;
}

/**************************************************************************\
* NTSTATUS ReadWriteConfig
*
* This routine is a working replacement for StreamClassReadWriteConfig.
*
* Must be called at low priority
*
* History:
*  27-Oct-1998 -by- Tom Zakrajsek [tomz]
*   Wrote it.
*
\**************************************************************************/

BOOL PCI::ReadWriteConfig(
    BOOL  bRead,
    PVOID pBuf,
    ULONG ulOffset,
    ULONG ulLen
)
{
    TRACE(("PCI::ReadWriteConfig"));

	PIO_STACK_LOCATION  nextStack;
    PIRP                irp;
    NTSTATUS            ntStatus;
    KEVENT              event;
    
    ENSURE( KeGetCurrentIrql() < DISPATCH_LEVEL );

    if( bRead ) {
       RtlZeroMemory( pBuf, ulLen );
    }

    irp = IoAllocateIrp( pDeviceObject->StackSize, FALSE );

    if( !irp ) {
       ERROR(("Failed to allocate irp\n" ));
       return FALSE;
    }

	// Initialize IRP status to NOT_SUPPORTED (Driver Verifier got that)
	irp->IoStatus.Status = STATUS_NOT_SUPPORTED;

    KeInitializeEvent( &event, NotificationEvent, FALSE );

    IoSetCompletionRoutine(
       irp,
       SynchCompletionRoutine,
       &event,
       TRUE,
       TRUE,
       TRUE
    );

    nextStack = IoGetNextIrpStackLocation( irp );
    ENSURE( nextStack != NULL );
	nextStack->MajorFunction = IRP_MJ_PNP;
    nextStack->MinorFunction = bRead ? IRP_MN_READ_CONFIG : IRP_MN_WRITE_CONFIG;
    nextStack->Parameters.ReadWriteConfig.WhichSpace = 0;
    nextStack->Parameters.ReadWriteConfig.Buffer = pBuf;
    nextStack->Parameters.ReadWriteConfig.Offset = ulOffset;
    nextStack->Parameters.ReadWriteConfig.Length = ulLen;

    ntStatus = IoCallDriver( pDeviceObject, irp );

    if( ntStatus == STATUS_PENDING ) {
        // wait for irp to complete
        KeWaitForSingleObject( &event, Suspended, KernelMode, FALSE, NULL );
    }
	IoFreeIrp( irp );

    if( !NT_SUCCESS( ntStatus ) ) {
        ERROR(( "IoCallDriver() bad status\n" ));
		return FALSE;
    }
	return TRUE;
}

// Following functions are provided to interact with PCI directly
BOOL PCI::ReadConfigDWORD( DWORD reg, DWORD& retDWORD )
{
	TRACE(("PCI::ReadConfigDWORD"));
	return ReadWriteConfig(TRUE,&retDWORD,reg ,4);
}

BOOL PCI::WriteConfigDWORD( DWORD reg, DWORD writeDWORD )
{
	TRACE(("PCI::WriteConfigDWORD"));
	return ReadWriteConfig(FALSE,&writeDWORD,reg ,4);
}

// Vendor and Device ID Register (0x00)
WORD PCI::GetVendorID( void )
{
	TRACE(("PCI::GetVendorID"));

	DWORD Reg0;
	ReadConfigDWORD(0,Reg0);
	return (WORD) (Reg0 & 0xffff);
}

WORD PCI::GetDeviceID( void )
{
	TRACE(("PCI::GetDeviceID"));

	DWORD Reg0;
	ReadConfigDWORD(0,Reg0);
	return (WORD) (Reg0 >> 16);
}
