/*
	Registry.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "Registry.hpp"

/*++

Routine Description:

    Reads the specified registry value

Arguments:

    Handle - handle to the registry key
    KeyNameString - value to read
    KeyNameStringLength - length of string
    Data - buffer to read data into
    DataLength - length of data buffer

Return Value:

    NTSTATUS returned as appropriate

--*/
NTSTATUS Registry::GetRegistryValue(
		IN HANDLE Handle,
		IN const PUNICODE_STRING KeyName,
		IN PCHAR Data, IN ULONG DataLength )
{
	TRACE(("Registry::GetRegistryValue"));

	NTSTATUS        Status = STATUS_INSUFFICIENT_RESOURCES;
	ULONG           Length;
	PKEY_VALUE_FULL_INFORMATION FullInfo;

	Length = sizeof( KEY_VALUE_FULL_INFORMATION ) + DataLength + KeyName->MaximumLength;

	FullInfo = (struct _KEY_VALUE_FULL_INFORMATION *)ExAllocatePool(PagedPool, Length);

	if ( FullInfo ) {
		Status = ZwQueryValueKey( Handle, KeyName, KeyValueFullInformation,
			FullInfo, Length, &Length );

		if ( NT_SUCCESS( Status ) ) {

			if ( DataLength >= FullInfo->DataLength ) {
				RtlCopyMemory( Data, ((PUCHAR) FullInfo) + FullInfo->DataOffset,
					FullInfo->DataLength );

			} else {
				Status = STATUS_BUFFER_TOO_SMALL;
			}                   // buffer right length
		}                       // if success
		ExFreePool( FullInfo );
	}                           // if fullinfo
	return Status;
}

NTSTATUS Registry::SetRegistryValue(
		HANDLE Handle,                 // handle to the registry key
		const PUNICODE_STRING KeyName, // keyvalue to write
		ULONG Type,                    // Type of key, should be REG_XXX
		PCHAR Data,                    // buffer containing value to be written
		ULONG DataLength               // length of data buffer
		)
{
	TRACE(("Registry::SetRegistryValue"));

	NTSTATUS Status;

	// Write the data
	Status = ZwSetValueKey( 
							Handle, 
							KeyName, 
							0,
							Type, 
							Data, 
							DataLength 
						  );
	return Status;
}


/* Function: OpenDriverKey
 * Purpose: Opens the DriverData key off the main driver key
 * Input: PhysicalDeviceObject : DEVICE_OBJECT *
 * Output: Open key handle or NULL
 */
HANDLE Registry::OpenDriverKey( const PDEVICE_OBJECT PhysicalDeviceObject )
{
	TRACE(("Registry::OpenDriverKey"));

	NTSTATUS   Status;
	HANDLE     DevHandle;

	// Check we are running at PASSIVE_LEVEL, else fail to avoid problems
	if (KeGetCurrentIrql() != PASSIVE_LEVEL) {
		ERROR(("Called at IRQL != PASSIVE_LEVEL (Current= %d)", KeGetCurrentIrql() ));
		return NULL;
	}

	Status = IoOpenDeviceRegistryKey( PhysicalDeviceObject, PLUGPLAY_REGKEY_DRIVER,
		STANDARD_RIGHTS_ALL, &DevHandle );

	HANDLE KeyHandle = NULL;
	if ( NT_SUCCESS( Status ) ) {
		OBJECT_ATTRIBUTES attr;
		UNICODE_STRING UDevDataName;
		PWCHAR WDataName = L"DriverData";

		RtlInitUnicodeString( &UDevDataName, WDataName );
		InitializeObjectAttributes( &attr, &UDevDataName, OBJ_INHERIT, DevHandle,
			NULL );

		ZwOpenKey( &KeyHandle, KEY_QUERY_VALUE, &attr );
		ZwClose( DevHandle );
	}
	return KeyHandle;
}
/* Function: Registry::PrepareKeyName
 * Purpose: Creates a UNICODE name for a key
 * Parameters: UKeyName: UNICODE_STRING * - key will be created here
 *   name: PCHAR - regular "C" string
 */
void Registry::PrepareKeyName( PUNICODE_STRING UKeyName, PCHAR name )
{
	TRACE(("Registry::PrepareKeyName"));

	ANSI_STRING  AKeyName;
	RtlInitAnsiString( &AKeyName, name );
	RtlAnsiStringToUnicodeString( UKeyName, &AKeyName, TRUE );
}

Registry::Registry( const PDEVICE_OBJECT PhysicalDeviceObject )
{
	TRACE(("Registry::Registry"));
	KeyHandle = OpenDriverKey( PhysicalDeviceObject );
}

bool Registry::ReadKey(char* Key ,DWORD& val, DWORD def )
{
	TRACE(("Registry::ReadKey"));
	if ( KeyHandle ) {

		UNICODE_STRING  UKeyName;
		NTSTATUS   Status;

		PrepareKeyName( &UKeyName, Key );
		Status = GetRegistryValue( KeyHandle, &UKeyName, (PCHAR) &val, sizeof( DWORD ) );
		RtlFreeUnicodeString( &UKeyName );

		if ( NT_SUCCESS(Status ) ) {
			return TRUE;
		}
	}
	// Return the default value
	val = def;
	return FALSE;
}

bool Registry::WriteKey(char* Key ,DWORD val )
{
	TRACE(("Registry::WriteKey"));

	if ( KeyHandle ) {
		UNICODE_STRING  UKeyName;
		NTSTATUS   Status;

		PrepareKeyName( &UKeyName, Key );
		Status = SetRegistryValue( KeyHandle, &UKeyName, REG_DWORD, (PCHAR) &val, sizeof(DWORD) );
		RtlFreeUnicodeString( &UKeyName );

		if ( NT_SUCCESS(Status ) ) {
			return TRUE;
		}
	}
	return FALSE;
}

Registry::~Registry()
{
	TRACE(("Registry::~Registry"));
	if (KeyHandle) {
		ZwClose( KeyHandle );
	}
}

