/*
	Stream.cpp: Part of the WDM Video Capture Driver for Bt848 based video capture cards
	Copyright (C) 2000 - 2003 by Eduardo Jos Tagle.
 
	This library is free software; you can redistribute it and/or
	modify it under the terms of the GNU Library General Public
	License as published by the Free Software Foundation; either
	version 2 of the License, or (at your option) any later version.

	This library is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
	Library General Public License for more details.

	You should have received a copy of the GNU Library General Public
	License along with this library; if not, write to the Free
	Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#include "debug.hpp"
#include "Stream.hpp"
#include "Adapter.hpp" // For Adapter::SRB_EXTENSION

// Base Stream functions
void BaseStream::CompleteDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("BaseStream::CompleteDataPacket"));
	ENSURE( pSrb );
	ENSURE( (pSrb->Flags & SRB_HW_FLAGS_STREAM_REQUEST) );

	DUMP(("Completing Stream Data SRB %08x\n",pSrb));
	StreamClassStreamNotification( StreamRequestComplete, pSrb->StreamObject, pSrb );
}

/*
** BaseStream::CompareGUIDsAndFormatSize()
**
**   Checks for a match on the three GUIDs and FormatSize
**
** Arguments:
**
**         IN DataRange1
**         IN DataRange2
**
** Returns:
**9
**   TRUE if all elements match
**   FALSE if any are different
**
** Side Effects:  none
*/

bool BaseStream::CompareGUIDsAndFormatSize(
   IN const PKSDATARANGE DataRange1,
   IN const PKSDATARANGE DataRange2,
   IN BOOL bCheckSize
   )
{
   // TRACE(("BaseStream::CompareGUIDsAndFormatSize")); -> Too much garbage on screen!

   return (
      DataRange1->MajorFormat == DataRange2->MajorFormat &&
      DataRange1->SubFormat == DataRange2->SubFormat &&
      DataRange1->Specifier == DataRange2->Specifier &&
      (bCheckSize ? ( DataRange1->FormatSize == DataRange2->FormatSize ) : TRUE) );


};


// Video Stream functions
CaptureStream::CaptureStream() 
{
	TRACE(("CaptureStream::CaptureStream"));
}

void CaptureStream::ReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("CaptureStream::ReceiveDataPacket"));

	switch ( pSrb->Command ) {
		case SRB_READ_DATA:

			OnReadData ( pSrb ); // Completion will be posted later when buffer is filled!

			break;

		default:
			// invalid / unsupported command. Fail it as such
			ERROR(("unknown data packet command(%x)\n", pSrb->Command));

			pSrb->Status = STATUS_NOT_IMPLEMENTED;
			CompleteDataPacket( pSrb );
	}
}

void AnalogStream::ReceiveDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(("AnalogStream::ReceiveDataPacket"));
	pSrb->Status = STATUS_SUCCESS;

	switch ( pSrb->Command ) {
		case SRB_READ_DATA:
			break;
		case SRB_WRITE_DATA:
			//
			// This data packet contains the channel change information
			// passed on the AnalogVideoIn
			//
			if ( pSrb->CommandData.DataBufferArray->FrameExtent == sizeof( KS_TVTUNER_CHANGE_INFO ) )
				pSrb->Status = ChangeNotifyChannels( pSrb );
			break;
		default:
			//
			// invalid / unsupported command. Fail it as such
			//
			ERROR(("(%08x) cmd(%x)\n", pSrb, pSrb->Command));
			pSrb->Status = STATUS_NOT_IMPLEMENTED;
	}

	CompleteDataPacket( pSrb );
}
    
void AnalogStream::CancelDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "AnalogStream::CancelDataPacket" ));
	pSrb->Status = STATUS_CANCELLED; 
	CompleteDataPacket(pSrb);
}


void AnalogStream::TimeoutDataPacket( PHW_STREAM_REQUEST_BLOCK pSrb )
{
	TRACE(( "AnalogStream::TimeoutDataPacket" ));
	pSrb->Status = STATUS_SUCCESS;
}

// The videocontrol for each analog stream
ULONG AnalogStream::GetVideoControlCaps( void ) 
{
	TRACE(( "AnalogStream::GetVideoControlCaps" ));
	return 0; // No videocaps
}

ULONG AnalogStream::GetVideoControlMode( void )
{
	TRACE(( "AnalogStream::GetVideoControlMode" ));
	return 0; // No videocaps
}

NTSTATUS AnalogStream::SetVideoControlMode( ULONG mode )
{
	TRACE(( "AnalogStream::SetVideoControlMode" ));
	return STATUS_NOT_SUPPORTED;
}

LONGLONG AnalogStream::GetActualFramerateInNs( void )
{
	TRACE(( "AnalogStream::GetActualFramerateInNs" ));
	return 0i64;
}

LONGLONG AnalogStream::GetMaxAvailableFramerateInNs( void )
{
	TRACE(( "AnalogStream::GetMaxAvailableFramerateX100" ));
	return 0i64;
}

NTSTATUS AnalogStream::GetAnalogWindow( RECT& rc )
{
	TRACE(( "AnalogStream::GetAnalogWindow" ));
	rc.bottom = rc.left = rc.right = rc.top = 0;
	return STATUS_NOT_SUPPORTED;
}

NTSTATUS AnalogStream::SetAnalogWindow( const RECT& rc )
{
	TRACE(( "AnalogStream::SetAnalogWindow" ));
	return STATUS_NOT_SUPPORTED;
}

NTSTATUS AnalogStream::GetDigitalWindow( RECT& rc )
{
	TRACE(( "AnalogStream::GetDigitalWindow" ));
	rc.bottom = rc.left = rc.right = rc.top = 0;
	return STATUS_NOT_SUPPORTED;
}



